
use phf::phf_map;
use phf::Map;
#[cfg(all(direct_wasm,target_arch = "wasm32"))]
use wasm_bindgen::prelude::*;

/// # Sample code
/// ```
/// let country = rust_iso3166::from_alpha2("GB").unwrap();
/// let subdivisions = country.subdivisions();
/// assert!(subdivisions.unwrap().len() > 0);
/// let country = rust_iso3166::iso3166_2::from_code("GB-EDH");
/// assert_eq!("Edinburgh, City of", country.unwrap().name);
/// println!("{:?}", rust_iso3166::iso3166_2::SUBDIVISION_COUNTRY_MAP); 
/// println!("{:?}", rust_iso3166::iso3166_2::SUBDIVISION_MAP); 
/// ```

/// Data for each Country Code defined by ISO 3166-2
#[cfg(all(direct_wasm,target_arch = "wasm32"))]
#[wasm_bindgen]
#[derive(Debug, Copy, Ord, PartialOrd, Clone, PartialEq, Eq, Hash)]
pub struct Subdivision {
    ///Name
    name: &'static str,
    ///Subdivision Type
    subdivision_type: &'static str,
    ///Code
    code: &'static str,
    ///Country Name
    country_name: &'static str,
    ///Country Code
    country_code: &'static str,
    ///Region Code
    region_code: &'static str,
}

#[cfg(all(direct_wasm,target_arch = "wasm32"))]
#[wasm_bindgen]
impl Subdivision {
    #[wasm_bindgen(getter)]
    pub fn name(&self) -> String {
        self.name.into()
    }

    #[wasm_bindgen(getter)]
    pub fn code(&self) -> String {
        self.code.into()
    }

    #[wasm_bindgen(getter)]
    pub fn subdivision_type(&self) -> String {
        self.subdivision_type.into()
    }

    #[wasm_bindgen(getter)]
    pub fn country_name(&self) -> String {
        self.country_name.into()
    }

    #[wasm_bindgen(getter)]
    pub fn country_code(&self) -> String {
        self.country_code.into()
    }

    #[wasm_bindgen(getter)]
    pub fn region_code(&self) -> String {
        self.region_code.into()
    }
}

#[cfg(any(not(direct_wasm),not(target_arch = "wasm32")))]
#[derive(Debug, Ord, PartialOrd, Copy, Clone, PartialEq, Eq, Hash)]
pub struct Subdivision {
    ///Name
    pub name: &'static str,
    ///Subdivision Type
    pub subdivision_type: &'static str,
    ///Code
    pub code: &'static str,
    ///Country Name
    pub country_name: &'static str,
    ///Country Code
    pub country_code: &'static str,
    ///Region Code
    pub region_code: &'static str,
}

/// Returns the Subdivision with the given code, if exists.
/// #Sample
/// ```
/// let sub = rust_iso3166::iso3166_2::from_code("SE-O");
/// assert_eq!("Västra Götalands län", sub.unwrap().name);
/// ```
#[cfg_attr(all(direct_wasm,target_arch = "wasm32"), wasm_bindgen(js_name = from_code_iso_3166_2))]
pub fn from_code(code: &str) -> Option<Subdivision> {
    SUBDIVISION_MAP.get(code).cloned()
}

pub const AF_BAL: Subdivision = Subdivision {
    name: "Balkh",
    code: "AF-BAL",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-BAL",
};

pub const AF_BAM: Subdivision = Subdivision {
    name: "Bāmyān",
    code: "AF-BAM",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-BAM",
};

pub const AF_BDG: Subdivision = Subdivision {
    name: "Bādghīs",
    code: "AF-BDG",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-BDG",
};

pub const AF_BDS: Subdivision = Subdivision {
    name: "Badakhshān",
    code: "AF-BDS",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-BDS",
};

pub const AF_BGL: Subdivision = Subdivision {
    name: "Baghlān",
    code: "AF-BGL",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-BGL",
};

pub const AF_DAY: Subdivision = Subdivision {
    name: "Dāykundī",
    code: "AF-DAY",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-DAY",
};

pub const AF_FRA: Subdivision = Subdivision {
    name: "Farāh",
    code: "AF-FRA",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-FRA",
};

pub const AF_FYB: Subdivision = Subdivision {
    name: "Fāryāb",
    code: "AF-FYB",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-FYB",
};

pub const AF_GHA: Subdivision = Subdivision {
    name: "Ghaznī",
    code: "AF-GHA",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-GHA",
};

pub const AF_GHO: Subdivision = Subdivision {
    name: "Ghōr",
    code: "AF-GHO",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-GHO",
};

pub const AF_HEL: Subdivision = Subdivision {
    name: "Helmand",
    code: "AF-HEL",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-HEL",
};

pub const AF_HER: Subdivision = Subdivision {
    name: "Herāt",
    code: "AF-HER",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-HER",
};

pub const AF_JOW: Subdivision = Subdivision {
    name: "Jowzjān",
    code: "AF-JOW",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-JOW",
};

pub const AF_KAB: Subdivision = Subdivision {
    name: "Kābul",
    code: "AF-KAB",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-KAB",
};

pub const AF_KAN: Subdivision = Subdivision {
    name: "Kandahār",
    code: "AF-KAN",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-KAN",
};

pub const AF_KAP: Subdivision = Subdivision {
    name: "Kāpīsā",
    code: "AF-KAP",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-KAP",
};

pub const AF_KDZ: Subdivision = Subdivision {
    name: "Kunduz",
    code: "AF-KDZ",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-KDZ",
};

pub const AF_KHO: Subdivision = Subdivision {
    name: "Khōst",
    code: "AF-KHO",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-KHO",
};

pub const AF_KNR: Subdivision = Subdivision {
    name: "Kunar",
    code: "AF-KNR",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-KNR",
};

pub const AF_LAG: Subdivision = Subdivision {
    name: "Laghmān",
    code: "AF-LAG",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-LAG",
};

pub const AF_LOG: Subdivision = Subdivision {
    name: "Lōgar",
    code: "AF-LOG",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-LOG",
};

pub const AF_NAN: Subdivision = Subdivision {
    name: "Nangarhār",
    code: "AF-NAN",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-NAN",
};

pub const AF_NIM: Subdivision = Subdivision {
    name: "Nīmrōz",
    code: "AF-NIM",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-NIM",
};

pub const AF_NUR: Subdivision = Subdivision {
    name: "Nūristān",
    code: "AF-NUR",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-NUR",
};

pub const AF_PAN: Subdivision = Subdivision {
    name: "Panjshayr",
    code: "AF-PAN",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-PAN",
};

pub const AF_PAR: Subdivision = Subdivision {
    name: "Parwān",
    code: "AF-PAR",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-PAR",
};

pub const AF_PIA: Subdivision = Subdivision {
    name: "Paktiyā",
    code: "AF-PIA",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-PIA",
};

pub const AF_PKA: Subdivision = Subdivision {
    name: "Paktīkā",
    code: "AF-PKA",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-PKA",
};

pub const AF_SAM: Subdivision = Subdivision {
    name: "Samangān",
    code: "AF-SAM",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-SAM",
};

pub const AF_SAR: Subdivision = Subdivision {
    name: "Sar-e Pul",
    code: "AF-SAR",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-SAR",
};

pub const AF_TAK: Subdivision = Subdivision {
    name: "Takhār",
    code: "AF-TAK",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-TAK",
};

pub const AF_URU: Subdivision = Subdivision {
    name: "Uruzgān",
    code: "AF-URU",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-URU",
};

pub const AF_WAR: Subdivision = Subdivision {
    name: "Wardak",
    code: "AF-WAR",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-WAR",
};

pub const AF_ZAB: Subdivision = Subdivision {
    name: "Zābul",
    code: "AF-ZAB",
    subdivision_type: "Province",
    country_name: "Afghanistan",
    country_code: "AF",
    region_code: "AF-ZAB",
};

pub const AX_AX: Subdivision = Subdivision {
    name: "Åland Islands",
    code: "AX-AX",
    subdivision_type: "Country",
    country_name: "Åland Islands",
    country_code: "AX",
    region_code: "AX-AX",
};

pub const AL_01: Subdivision = Subdivision {
    name: "Berat",
    code: "AL-01",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-01",
};

pub const AL_02: Subdivision = Subdivision {
    name: "Durrës",
    code: "AL-02",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-02",
};

pub const AL_03: Subdivision = Subdivision {
    name: "Elbasan",
    code: "AL-03",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-03",
};

pub const AL_04: Subdivision = Subdivision {
    name: "Fier",
    code: "AL-04",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-04",
};

pub const AL_05: Subdivision = Subdivision {
    name: "Gjirokastër",
    code: "AL-05",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-05",
};

pub const AL_06: Subdivision = Subdivision {
    name: "Korçë",
    code: "AL-06",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-06",
};

pub const AL_07: Subdivision = Subdivision {
    name: "Kukës",
    code: "AL-07",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-07",
};

pub const AL_08: Subdivision = Subdivision {
    name: "Lezhë",
    code: "AL-08",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-08",
};

pub const AL_09: Subdivision = Subdivision {
    name: "Dibër",
    code: "AL-09",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-09",
};

pub const AL_10: Subdivision = Subdivision {
    name: "Shkodër",
    code: "AL-10",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-10",
};

pub const AL_11: Subdivision = Subdivision {
    name: "Tiranë",
    code: "AL-11",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-11",
};

pub const AL_12: Subdivision = Subdivision {
    name: "Vlorë",
    code: "AL-12",
    subdivision_type: "County",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-12",
};

pub const AL_BR: Subdivision = Subdivision {
    name: "Berat",
    code: "AL-BR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-BR",
};

pub const AL_BU: Subdivision = Subdivision {
    name: "Bulqizë",
    code: "AL-BU",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-BU",
};

pub const AL_DI: Subdivision = Subdivision {
    name: "Dibër",
    code: "AL-DI",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-DI",
};

pub const AL_DL: Subdivision = Subdivision {
    name: "Delvinë",
    code: "AL-DL",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-DL",
};

pub const AL_DR: Subdivision = Subdivision {
    name: "Durrës",
    code: "AL-DR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-DR",
};

pub const AL_DV: Subdivision = Subdivision {
    name: "Devoll",
    code: "AL-DV",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-DV",
};

pub const AL_EL: Subdivision = Subdivision {
    name: "Elbasan",
    code: "AL-EL",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-EL",
};

pub const AL_ER: Subdivision = Subdivision {
    name: "Kolonjë",
    code: "AL-ER",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-ER",
};

pub const AL_FR: Subdivision = Subdivision {
    name: "Fier",
    code: "AL-FR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-FR",
};

pub const AL_GJ: Subdivision = Subdivision {
    name: "Gjirokastër",
    code: "AL-GJ",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-GJ",
};

pub const AL_GR: Subdivision = Subdivision {
    name: "Gramsh",
    code: "AL-GR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-GR",
};

pub const AL_HA: Subdivision = Subdivision {
    name: "Has",
    code: "AL-HA",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-HA",
};

pub const AL_KA: Subdivision = Subdivision {
    name: "Kavajë",
    code: "AL-KA",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-KA",
};

pub const AL_KB: Subdivision = Subdivision {
    name: "Kurbin",
    code: "AL-KB",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-KB",
};

pub const AL_KC: Subdivision = Subdivision {
    name: "Kuçovë",
    code: "AL-KC",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-KC",
};

pub const AL_KO: Subdivision = Subdivision {
    name: "Korçë",
    code: "AL-KO",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-KO",
};

pub const AL_KR: Subdivision = Subdivision {
    name: "Krujë",
    code: "AL-KR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-KR",
};

pub const AL_KU: Subdivision = Subdivision {
    name: "Kukës",
    code: "AL-KU",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-KU",
};

pub const AL_LB: Subdivision = Subdivision {
    name: "Librazhd",
    code: "AL-LB",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-LB",
};

pub const AL_LE: Subdivision = Subdivision {
    name: "Lezhë",
    code: "AL-LE",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-LE",
};

pub const AL_LU: Subdivision = Subdivision {
    name: "Lushnjë",
    code: "AL-LU",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-LU",
};

pub const AL_MK: Subdivision = Subdivision {
    name: "Mallakastër",
    code: "AL-MK",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-MK",
};

pub const AL_MM: Subdivision = Subdivision {
    name: "Malësi e Madhe",
    code: "AL-MM",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-MM",
};

pub const AL_MR: Subdivision = Subdivision {
    name: "Mirditë",
    code: "AL-MR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-MR",
};

pub const AL_MT: Subdivision = Subdivision {
    name: "Mat",
    code: "AL-MT",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-MT",
};

pub const AL_PG: Subdivision = Subdivision {
    name: "Pogradec",
    code: "AL-PG",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-PG",
};

pub const AL_PQ: Subdivision = Subdivision {
    name: "Peqin",
    code: "AL-PQ",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-PQ",
};

pub const AL_PR: Subdivision = Subdivision {
    name: "Përmet",
    code: "AL-PR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-PR",
};

pub const AL_PU: Subdivision = Subdivision {
    name: "Pukë",
    code: "AL-PU",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-PU",
};

pub const AL_SH: Subdivision = Subdivision {
    name: "Shkodër",
    code: "AL-SH",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-SH",
};

pub const AL_SK: Subdivision = Subdivision {
    name: "Skrapar",
    code: "AL-SK",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-SK",
};

pub const AL_SR: Subdivision = Subdivision {
    name: "Sarandë",
    code: "AL-SR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-SR",
};

pub const AL_TE: Subdivision = Subdivision {
    name: "Tepelenë",
    code: "AL-TE",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-TE",
};

pub const AL_TP: Subdivision = Subdivision {
    name: "Tropojë",
    code: "AL-TP",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-TP",
};

pub const AL_TR: Subdivision = Subdivision {
    name: "Tiranë",
    code: "AL-TR",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-TR",
};

pub const AL_VL: Subdivision = Subdivision {
    name: "Vlorë",
    code: "AL-VL",
    subdivision_type: "District",
    country_name: "Albania",
    country_code: "AL",
    region_code: "AL-VL",
};

pub const DZ_01: Subdivision = Subdivision {
    name: "Adrar",
    code: "DZ-01",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-01",
};

pub const DZ_02: Subdivision = Subdivision {
    name: "Chlef",
    code: "DZ-02",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-02",
};

pub const DZ_03: Subdivision = Subdivision {
    name: "Laghouat",
    code: "DZ-03",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-03",
};

pub const DZ_04: Subdivision = Subdivision {
    name: "Oum el Bouaghi",
    code: "DZ-04",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-04",
};

pub const DZ_05: Subdivision = Subdivision {
    name: "Batna",
    code: "DZ-05",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-05",
};

pub const DZ_06: Subdivision = Subdivision {
    name: "Béjaïa",
    code: "DZ-06",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-06",
};

pub const DZ_07: Subdivision = Subdivision {
    name: "Biskra",
    code: "DZ-07",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-07",
};

pub const DZ_08: Subdivision = Subdivision {
    name: "Béchar",
    code: "DZ-08",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-08",
};

pub const DZ_09: Subdivision = Subdivision {
    name: "Blida",
    code: "DZ-09",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-09",
};

pub const DZ_10: Subdivision = Subdivision {
    name: "Bouira",
    code: "DZ-10",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-10",
};

pub const DZ_11: Subdivision = Subdivision {
    name: "Tamanrasset",
    code: "DZ-11",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-11",
};

pub const DZ_12: Subdivision = Subdivision {
    name: "Tébessa",
    code: "DZ-12",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-12",
};

pub const DZ_13: Subdivision = Subdivision {
    name: "Tlemcen",
    code: "DZ-13",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-13",
};

pub const DZ_14: Subdivision = Subdivision {
    name: "Tiaret",
    code: "DZ-14",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-14",
};

pub const DZ_15: Subdivision = Subdivision {
    name: "Tizi Ouzou",
    code: "DZ-15",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-15",
};

pub const DZ_16: Subdivision = Subdivision {
    name: "Alger",
    code: "DZ-16",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-16",
};

pub const DZ_17: Subdivision = Subdivision {
    name: "Djelfa",
    code: "DZ-17",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-17",
};

pub const DZ_18: Subdivision = Subdivision {
    name: "Jijel",
    code: "DZ-18",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-18",
};

pub const DZ_19: Subdivision = Subdivision {
    name: "Sétif",
    code: "DZ-19",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-19",
};

pub const DZ_20: Subdivision = Subdivision {
    name: "Saïda",
    code: "DZ-20",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-20",
};

pub const DZ_21: Subdivision = Subdivision {
    name: "Skikda",
    code: "DZ-21",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-21",
};

pub const DZ_22: Subdivision = Subdivision {
    name: "Sidi Bel Abbès",
    code: "DZ-22",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-22",
};

pub const DZ_23: Subdivision = Subdivision {
    name: "Annaba",
    code: "DZ-23",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-23",
};

pub const DZ_24: Subdivision = Subdivision {
    name: "Guelma",
    code: "DZ-24",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-24",
};

pub const DZ_25: Subdivision = Subdivision {
    name: "Constantine",
    code: "DZ-25",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-25",
};

pub const DZ_26: Subdivision = Subdivision {
    name: "Médéa",
    code: "DZ-26",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-26",
};

pub const DZ_27: Subdivision = Subdivision {
    name: "Mostaganem",
    code: "DZ-27",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-27",
};

pub const DZ_28: Subdivision = Subdivision {
    name: "Msila",
    code: "DZ-28",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-28",
};

pub const DZ_29: Subdivision = Subdivision {
    name: "Mascara",
    code: "DZ-29",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-29",
};

pub const DZ_30: Subdivision = Subdivision {
    name: "Ouargla",
    code: "DZ-30",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-30",
};

pub const DZ_31: Subdivision = Subdivision {
    name: "Oran",
    code: "DZ-31",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-31",
};

pub const DZ_32: Subdivision = Subdivision {
    name: "El Bayadh",
    code: "DZ-32",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-32",
};

pub const DZ_33: Subdivision = Subdivision {
    name: "Illizi",
    code: "DZ-33",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-33",
};

pub const DZ_34: Subdivision = Subdivision {
    name: "Bordj Bou Arréridj",
    code: "DZ-34",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-34",
};

pub const DZ_35: Subdivision = Subdivision {
    name: "Boumerdès",
    code: "DZ-35",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-35",
};

pub const DZ_36: Subdivision = Subdivision {
    name: "El Tarf",
    code: "DZ-36",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-36",
};

pub const DZ_37: Subdivision = Subdivision {
    name: "Tindouf",
    code: "DZ-37",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-37",
};

pub const DZ_38: Subdivision = Subdivision {
    name: "Tissemsilt",
    code: "DZ-38",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-38",
};

pub const DZ_39: Subdivision = Subdivision {
    name: "El Oued",
    code: "DZ-39",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-39",
};

pub const DZ_40: Subdivision = Subdivision {
    name: "Khenchela",
    code: "DZ-40",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-40",
};

pub const DZ_41: Subdivision = Subdivision {
    name: "Souk Ahras",
    code: "DZ-41",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-41",
};

pub const DZ_42: Subdivision = Subdivision {
    name: "Tipaza",
    code: "DZ-42",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-42",
};

pub const DZ_43: Subdivision = Subdivision {
    name: "Mila",
    code: "DZ-43",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-43",
};

pub const DZ_44: Subdivision = Subdivision {
    name: "Aïn Defla",
    code: "DZ-44",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-44",
};

pub const DZ_45: Subdivision = Subdivision {
    name: "Naama",
    code: "DZ-45",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-45",
};

pub const DZ_46: Subdivision = Subdivision {
    name: "Aïn Témouchent",
    code: "DZ-46",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-46",
};

pub const DZ_47: Subdivision = Subdivision {
    name: "Ghardaïa",
    code: "DZ-47",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-47",
};

pub const DZ_48: Subdivision = Subdivision {
    name: "Relizane",
    code: "DZ-48",
    subdivision_type: "Province",
    country_name: "Algeria",
    country_code: "DZ",
    region_code: "DZ-48",
};

pub const AS_AS: Subdivision = Subdivision {
    name: "American Samoa",
    code: "AS-AS",
    subdivision_type: "Country",
    country_name: "American Samoa",
    country_code: "AS",
    region_code: "AS-AS",
};

pub const AD_02: Subdivision = Subdivision {
    name: "Canillo",
    code: "AD-02",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-02",
};

pub const AD_03: Subdivision = Subdivision {
    name: "Encamp",
    code: "AD-03",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-03",
};

pub const AD_04: Subdivision = Subdivision {
    name: "La Massana",
    code: "AD-04",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-04",
};

pub const AD_05: Subdivision = Subdivision {
    name: "Ordino",
    code: "AD-05",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-05",
};

pub const AD_06: Subdivision = Subdivision {
    name: "Sant Julià de Lòria",
    code: "AD-06",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-06",
};

pub const AD_07: Subdivision = Subdivision {
    name: "Andorra la Vella",
    code: "AD-07",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-07",
};

pub const AD_08: Subdivision = Subdivision {
    name: "Escaldes-Engordany",
    code: "AD-08",
    subdivision_type: "Parish",
    country_name: "Andorra",
    country_code: "AD",
    region_code: "AD-08",
};

pub const AO_BGO: Subdivision = Subdivision {
    name: "Bengo",
    code: "AO-BGO",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-BGO",
};

pub const AO_BGU: Subdivision = Subdivision {
    name: "Benguela",
    code: "AO-BGU",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-BGU",
};

pub const AO_BIE: Subdivision = Subdivision {
    name: "Bié",
    code: "AO-BIE",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-BIE",
};

pub const AO_CAB: Subdivision = Subdivision {
    name: "Cabinda",
    code: "AO-CAB",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-CAB",
};

pub const AO_CCU: Subdivision = Subdivision {
    name: "Kuando Kubango",
    code: "AO-CCU",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-CCU",
};

pub const AO_CNN: Subdivision = Subdivision {
    name: "Cunene",
    code: "AO-CNN",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-CNN",
};

pub const AO_CNO: Subdivision = Subdivision {
    name: "Kwanza Norte",
    code: "AO-CNO",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-CNO",
};

pub const AO_CUS: Subdivision = Subdivision {
    name: "Kwanza Sul",
    code: "AO-CUS",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-CUS",
};

pub const AO_HUA: Subdivision = Subdivision {
    name: "Huambo",
    code: "AO-HUA",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-HUA",
};

pub const AO_HUI: Subdivision = Subdivision {
    name: "Huíla",
    code: "AO-HUI",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-HUI",
};

pub const AO_LNO: Subdivision = Subdivision {
    name: "Lunda Norte",
    code: "AO-LNO",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-LNO",
};

pub const AO_LSU: Subdivision = Subdivision {
    name: "Lunda Sul",
    code: "AO-LSU",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-LSU",
};

pub const AO_LUA: Subdivision = Subdivision {
    name: "Luanda",
    code: "AO-LUA",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-LUA",
};

pub const AO_MAL: Subdivision = Subdivision {
    name: "Malange",
    code: "AO-MAL",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-MAL",
};

pub const AO_MOX: Subdivision = Subdivision {
    name: "Moxico",
    code: "AO-MOX",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-MOX",
};

pub const AO_NAM: Subdivision = Subdivision {
    name: "Namibe",
    code: "AO-NAM",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-NAM",
};

pub const AO_UIG: Subdivision = Subdivision {
    name: "Uíge",
    code: "AO-UIG",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-UIG",
};

pub const AO_ZAI: Subdivision = Subdivision {
    name: "Zaire",
    code: "AO-ZAI",
    subdivision_type: "Province",
    country_name: "Angola",
    country_code: "AO",
    region_code: "AO-ZAI",
};

pub const AI_AI: Subdivision = Subdivision {
    name: "Anguilla",
    code: "AI-AI",
    subdivision_type: "Country",
    country_name: "Anguilla",
    country_code: "AI",
    region_code: "AI-AI",
};

pub const AQ_AQ: Subdivision = Subdivision {
    name: "Antarctica",
    code: "AQ-AQ",
    subdivision_type: "Country",
    country_name: "Antarctica",
    country_code: "AQ",
    region_code: "AQ-AQ",
};

pub const AG_03: Subdivision = Subdivision {
    name: "Saint George",
    code: "AG-03",
    subdivision_type: "Parish",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-03",
};

pub const AG_04: Subdivision = Subdivision {
    name: "Saint John",
    code: "AG-04",
    subdivision_type: "Parish",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-04",
};

pub const AG_05: Subdivision = Subdivision {
    name: "Saint Mary",
    code: "AG-05",
    subdivision_type: "Parish",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-05",
};

pub const AG_06: Subdivision = Subdivision {
    name: "Saint Paul",
    code: "AG-06",
    subdivision_type: "Parish",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-06",
};

pub const AG_07: Subdivision = Subdivision {
    name: "Saint Peter",
    code: "AG-07",
    subdivision_type: "Parish",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-07",
};

pub const AG_08: Subdivision = Subdivision {
    name: "Saint Philip",
    code: "AG-08",
    subdivision_type: "Parish",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-08",
};

pub const AG_10: Subdivision = Subdivision {
    name: "Barbuda",
    code: "AG-10",
    subdivision_type: "Dependency",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-10",
};

pub const AG_11: Subdivision = Subdivision {
    name: "Redonda",
    code: "AG-11",
    subdivision_type: "Dependency",
    country_name: "Antigua and Barbuda",
    country_code: "AG",
    region_code: "AG-11",
};

pub const AR_A: Subdivision = Subdivision {
    name: "Salta",
    code: "AR-A",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-A",
};

pub const AR_B: Subdivision = Subdivision {
    name: "Buenos Aires",
    code: "AR-B",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-B",
};

pub const AR_C: Subdivision = Subdivision {
    name: "Ciudad Autónoma de Buenos Aires",
    code: "AR-C",
    subdivision_type: "City",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-C",
};

pub const AR_D: Subdivision = Subdivision {
    name: "San Luis",
    code: "AR-D",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-D",
};

pub const AR_E: Subdivision = Subdivision {
    name: "Entre Ríos",
    code: "AR-E",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-E",
};

pub const AR_F: Subdivision = Subdivision {
    name: "La Rioja",
    code: "AR-F",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-F",
};

pub const AR_G: Subdivision = Subdivision {
    name: "Santiago del Estero",
    code: "AR-G",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-G",
};

pub const AR_H: Subdivision = Subdivision {
    name: "Chaco",
    code: "AR-H",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-H",
};

pub const AR_J: Subdivision = Subdivision {
    name: "San Juan",
    code: "AR-J",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-J",
};

pub const AR_K: Subdivision = Subdivision {
    name: "Catamarca",
    code: "AR-K",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-K",
};

pub const AR_L: Subdivision = Subdivision {
    name: "La Pampa",
    code: "AR-L",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-L",
};

pub const AR_M: Subdivision = Subdivision {
    name: "Mendoza",
    code: "AR-M",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-M",
};

pub const AR_N: Subdivision = Subdivision {
    name: "Misiones",
    code: "AR-N",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-N",
};

pub const AR_P: Subdivision = Subdivision {
    name: "Formosa",
    code: "AR-P",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-P",
};

pub const AR_Q: Subdivision = Subdivision {
    name: "Neuquén",
    code: "AR-Q",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-Q",
};

pub const AR_R: Subdivision = Subdivision {
    name: "Río Negro",
    code: "AR-R",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-R",
};

pub const AR_S: Subdivision = Subdivision {
    name: "Santa Fe",
    code: "AR-S",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-S",
};

pub const AR_T: Subdivision = Subdivision {
    name: "Tucumán",
    code: "AR-T",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-T",
};

pub const AR_U: Subdivision = Subdivision {
    name: "Chubut",
    code: "AR-U",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-U",
};

pub const AR_V: Subdivision = Subdivision {
    name: "Tierra del Fuego",
    code: "AR-V",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-V",
};

pub const AR_W: Subdivision = Subdivision {
    name: "Corrientes",
    code: "AR-W",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-W",
};

pub const AR_X: Subdivision = Subdivision {
    name: "Córdoba",
    code: "AR-X",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-X",
};

pub const AR_Y: Subdivision = Subdivision {
    name: "Jujuy",
    code: "AR-Y",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-Y",
};

pub const AR_Z: Subdivision = Subdivision {
    name: "Santa Cruz",
    code: "AR-Z",
    subdivision_type: "Province",
    country_name: "Argentina",
    country_code: "AR",
    region_code: "AR-Z",
};

pub const AM_AG: Subdivision = Subdivision {
    name: "Aragac̣otn",
    code: "AM-AG",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-AG",
};

pub const AM_AR: Subdivision = Subdivision {
    name: "Ararat",
    code: "AM-AR",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-AR",
};

pub const AM_AV: Subdivision = Subdivision {
    name: "Armavir",
    code: "AM-AV",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-AV",
};

pub const AM_ER: Subdivision = Subdivision {
    name: "Erevan",
    code: "AM-ER",
    subdivision_type: "City",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-ER",
};

pub const AM_GR: Subdivision = Subdivision {
    name: "Geġark'unik'",
    code: "AM-GR",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-GR",
};

pub const AM_KT: Subdivision = Subdivision {
    name: "Kotayk'",
    code: "AM-KT",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-KT",
};

pub const AM_LO: Subdivision = Subdivision {
    name: "Loṙi",
    code: "AM-LO",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-LO",
};

pub const AM_SH: Subdivision = Subdivision {
    name: "Širak",
    code: "AM-SH",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-SH",
};

pub const AM_SU: Subdivision = Subdivision {
    name: "Syunik'",
    code: "AM-SU",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-SU",
};

pub const AM_TV: Subdivision = Subdivision {
    name: "Tavuš",
    code: "AM-TV",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-TV",
};

pub const AM_VD: Subdivision = Subdivision {
    name: "Vayoć Jor",
    code: "AM-VD",
    subdivision_type: "Region",
    country_name: "Armenia",
    country_code: "AM",
    region_code: "AM-VD",
};

pub const AW_AW: Subdivision = Subdivision {
    name: "Aruba",
    code: "AW-AW",
    subdivision_type: "Country",
    country_name: "Aruba",
    country_code: "AW",
    region_code: "AW-AW",
};

pub const AU_ACT: Subdivision = Subdivision {
    name: "Australian Capital Territory",
    code: "AU-ACT",
    subdivision_type: "Territory",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-ACT",
};

pub const AU_NSW: Subdivision = Subdivision {
    name: "New South Wales",
    code: "AU-NSW",
    subdivision_type: "State",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-NSW",
};

pub const AU_NT: Subdivision = Subdivision {
    name: "Northern Territory",
    code: "AU-NT",
    subdivision_type: "Territory",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-NT",
};

pub const AU_QLD: Subdivision = Subdivision {
    name: "Queensland",
    code: "AU-QLD",
    subdivision_type: "State",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-QLD",
};

pub const AU_SA: Subdivision = Subdivision {
    name: "South Australia",
    code: "AU-SA",
    subdivision_type: "State",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-SA",
};

pub const AU_TAS: Subdivision = Subdivision {
    name: "Tasmania",
    code: "AU-TAS",
    subdivision_type: "State",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-TAS",
};

pub const AU_VIC: Subdivision = Subdivision {
    name: "Victoria",
    code: "AU-VIC",
    subdivision_type: "State",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-VIC",
};

pub const AU_WA: Subdivision = Subdivision {
    name: "Western Australia",
    code: "AU-WA",
    subdivision_type: "State",
    country_name: "Australia",
    country_code: "AU",
    region_code: "AU-WA",
};

pub const AT_1: Subdivision = Subdivision {
    name: "Burgenland",
    code: "AT-1",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-1",
};

pub const AT_2: Subdivision = Subdivision {
    name: "Kärnten",
    code: "AT-2",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-2",
};

pub const AT_3: Subdivision = Subdivision {
    name: "Niederösterreich",
    code: "AT-3",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-3",
};

pub const AT_4: Subdivision = Subdivision {
    name: "Oberösterreich",
    code: "AT-4",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-4",
};

pub const AT_5: Subdivision = Subdivision {
    name: "Salzburg",
    code: "AT-5",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-5",
};

pub const AT_6: Subdivision = Subdivision {
    name: "Steiermark",
    code: "AT-6",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-6",
};

pub const AT_7: Subdivision = Subdivision {
    name: "Tirol",
    code: "AT-7",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-7",
};

pub const AT_8: Subdivision = Subdivision {
    name: "Vorarlberg",
    code: "AT-8",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-8",
};

pub const AT_9: Subdivision = Subdivision {
    name: "Wien",
    code: "AT-9",
    subdivision_type: "Federal länder",
    country_name: "Austria",
    country_code: "AT",
    region_code: "AT-9",
};

pub const AZ_ABS: Subdivision = Subdivision {
    name: "Abşeron",
    code: "AZ-ABS",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-ABS",
};

pub const AZ_AGA: Subdivision = Subdivision {
    name: "Ağstafa",
    code: "AZ-AGA",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-AGA",
};

pub const AZ_AGC: Subdivision = Subdivision {
    name: "Ağcabәdi",
    code: "AZ-AGC",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-AGC",
};

pub const AZ_AGM: Subdivision = Subdivision {
    name: "Ağdam",
    code: "AZ-AGM",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-AGM",
};

pub const AZ_AGS: Subdivision = Subdivision {
    name: "Ağdaş",
    code: "AZ-AGS",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-AGS",
};

pub const AZ_AGU: Subdivision = Subdivision {
    name: "Ağsu",
    code: "AZ-AGU",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-AGU",
};

pub const AZ_AST: Subdivision = Subdivision {
    name: "Astara",
    code: "AZ-AST",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-AST",
};

pub const AZ_BA: Subdivision = Subdivision {
    name: "Bakı",
    code: "AZ-BA",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-BA",
};

pub const AZ_BAB: Subdivision = Subdivision {
    name: "Babək",
    code: "AZ-BAB",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-BAB",
};

pub const AZ_BAL: Subdivision = Subdivision {
    name: "Balakən",
    code: "AZ-BAL",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-BAL",
};

pub const AZ_BAR: Subdivision = Subdivision {
    name: "Bərdə",
    code: "AZ-BAR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-BAR",
};

pub const AZ_BEY: Subdivision = Subdivision {
    name: "Beyləqan",
    code: "AZ-BEY",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-BEY",
};

pub const AZ_BIL: Subdivision = Subdivision {
    name: "Biləsuvar",
    code: "AZ-BIL",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-BIL",
};

pub const AZ_CAB: Subdivision = Subdivision {
    name: "Cəbrayıl",
    code: "AZ-CAB",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-CAB",
};

pub const AZ_CAL: Subdivision = Subdivision {
    name: "Cəlilabad",
    code: "AZ-CAL",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-CAL",
};

pub const AZ_CUL: Subdivision = Subdivision {
    name: "Culfa",
    code: "AZ-CUL",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-CUL",
};

pub const AZ_DAS: Subdivision = Subdivision {
    name: "Daşkəsən",
    code: "AZ-DAS",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-DAS",
};

pub const AZ_FUZ: Subdivision = Subdivision {
    name: "Füzuli",
    code: "AZ-FUZ",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-FUZ",
};

pub const AZ_GAD: Subdivision = Subdivision {
    name: "Gədəbəy",
    code: "AZ-GAD",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-GAD",
};

pub const AZ_GA: Subdivision = Subdivision {
    name: "Gәncә",
    code: "AZ-GA",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-GA",
};

pub const AZ_GOR: Subdivision = Subdivision {
    name: "Goranboy",
    code: "AZ-GOR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-GOR",
};

pub const AZ_GOY: Subdivision = Subdivision {
    name: "Göyçay",
    code: "AZ-GOY",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-GOY",
};

pub const AZ_GYG: Subdivision = Subdivision {
    name: "Göygöl",
    code: "AZ-GYG",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-GYG",
};

pub const AZ_HAC: Subdivision = Subdivision {
    name: "Hacıqabul",
    code: "AZ-HAC",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-HAC",
};

pub const AZ_IMI: Subdivision = Subdivision {
    name: "İmişli",
    code: "AZ-IMI",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-IMI",
};

pub const AZ_ISM: Subdivision = Subdivision {
    name: "İsmayıllı",
    code: "AZ-ISM",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-ISM",
};

pub const AZ_KAL: Subdivision = Subdivision {
    name: "Kəlbəcər",
    code: "AZ-KAL",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-KAL",
};

pub const AZ_KAN: Subdivision = Subdivision {
    name: "Kǝngǝrli",
    code: "AZ-KAN",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-KAN",
};

pub const AZ_KUR: Subdivision = Subdivision {
    name: "Kürdəmir",
    code: "AZ-KUR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-KUR",
};

pub const AZ_LAC: Subdivision = Subdivision {
    name: "Laçın",
    code: "AZ-LAC",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-LAC",
};

pub const AZ_LA: Subdivision = Subdivision {
    name: "Lәnkәran",
    code: "AZ-LA",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-LA",
};

pub const AZ_LAN: Subdivision = Subdivision {
    name: "Lənkəran",
    code: "AZ-LAN",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-LAN",
};

pub const AZ_LER: Subdivision = Subdivision {
    name: "Lerik",
    code: "AZ-LER",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-LER",
};

pub const AZ_MAS: Subdivision = Subdivision {
    name: "Masallı",
    code: "AZ-MAS",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-MAS",
};

pub const AZ_MI: Subdivision = Subdivision {
    name: "Mingәçevir",
    code: "AZ-MI",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-MI",
};

pub const AZ_NA: Subdivision = Subdivision {
    name: "Naftalan",
    code: "AZ-NA",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-NA",
};

pub const AZ_NEF: Subdivision = Subdivision {
    name: "Neftçala",
    code: "AZ-NEF",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-NEF",
};

pub const AZ_NV: Subdivision = Subdivision {
    name: "Naxçıvan",
    code: "AZ-NV",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-NV",
};

pub const AZ_NX: Subdivision = Subdivision {
    name: "Naxçıvan",
    code: "AZ-NX",
    subdivision_type: "Autonomous republic",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-NX",
};

pub const AZ_OGU: Subdivision = Subdivision {
    name: "Oğuz",
    code: "AZ-OGU",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-OGU",
};

pub const AZ_ORD: Subdivision = Subdivision {
    name: "Ordubad",
    code: "AZ-ORD",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-ORD",
};

pub const AZ_QAB: Subdivision = Subdivision {
    name: "Qәbәlә",
    code: "AZ-QAB",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QAB",
};

pub const AZ_QAX: Subdivision = Subdivision {
    name: "Qax",
    code: "AZ-QAX",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QAX",
};

pub const AZ_QAZ: Subdivision = Subdivision {
    name: "Qazax",
    code: "AZ-QAZ",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QAZ",
};

pub const AZ_QBA: Subdivision = Subdivision {
    name: "Quba",
    code: "AZ-QBA",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QBA",
};

pub const AZ_QBI: Subdivision = Subdivision {
    name: "Qubadlı",
    code: "AZ-QBI",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QBI",
};

pub const AZ_QOB: Subdivision = Subdivision {
    name: "Qobustan",
    code: "AZ-QOB",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QOB",
};

pub const AZ_QUS: Subdivision = Subdivision {
    name: "Qusar",
    code: "AZ-QUS",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-QUS",
};

pub const AZ_SAB: Subdivision = Subdivision {
    name: "Sabirabad",
    code: "AZ-SAB",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAB",
};

pub const AZ_SAD: Subdivision = Subdivision {
    name: "Sәdәrәk",
    code: "AZ-SAD",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAD",
};

pub const AZ_SAH: Subdivision = Subdivision {
    name: "Şahbuz",
    code: "AZ-SAH",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAH",
};

pub const AZ_SAK: Subdivision = Subdivision {
    name: "Şәki",
    code: "AZ-SAK",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAK",
};

pub const AZ_SAL: Subdivision = Subdivision {
    name: "Salyan",
    code: "AZ-SAL",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAL",
};

pub const AZ_SAR: Subdivision = Subdivision {
    name: "Şәrur",
    code: "AZ-SAR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAR",
};

pub const AZ_SA: Subdivision = Subdivision {
    name: "Şәki",
    code: "AZ-SA",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SA",
};

pub const AZ_SAT: Subdivision = Subdivision {
    name: "Saatlı",
    code: "AZ-SAT",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SAT",
};

pub const AZ_SBN: Subdivision = Subdivision {
    name: "Şabran",
    code: "AZ-SBN",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SBN",
};

pub const AZ_SIY: Subdivision = Subdivision {
    name: "Siyәzәn",
    code: "AZ-SIY",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SIY",
};

pub const AZ_SKR: Subdivision = Subdivision {
    name: "Şәmkir",
    code: "AZ-SKR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SKR",
};

pub const AZ_SMI: Subdivision = Subdivision {
    name: "Şamaxı",
    code: "AZ-SMI",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SMI",
};

pub const AZ_SM: Subdivision = Subdivision {
    name: "Sumqayıt",
    code: "AZ-SM",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SM",
};

pub const AZ_SMX: Subdivision = Subdivision {
    name: "Samux",
    code: "AZ-SMX",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SMX",
};

pub const AZ_SR: Subdivision = Subdivision {
    name: "Şirvan",
    code: "AZ-SR",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SR",
};

pub const AZ_SUS: Subdivision = Subdivision {
    name: "Şuşa",
    code: "AZ-SUS",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-SUS",
};

pub const AZ_TAR: Subdivision = Subdivision {
    name: "Tәrtәr",
    code: "AZ-TAR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-TAR",
};

pub const AZ_TOV: Subdivision = Subdivision {
    name: "Tovuz",
    code: "AZ-TOV",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-TOV",
};

pub const AZ_UCA: Subdivision = Subdivision {
    name: "Ucar",
    code: "AZ-UCA",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-UCA",
};

pub const AZ_XAC: Subdivision = Subdivision {
    name: "Xaçmaz",
    code: "AZ-XAC",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-XAC",
};

pub const AZ_XA: Subdivision = Subdivision {
    name: "Xankәndi",
    code: "AZ-XA",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-XA",
};

pub const AZ_XCI: Subdivision = Subdivision {
    name: "Xocalı",
    code: "AZ-XCI",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-XCI",
};

pub const AZ_XIZ: Subdivision = Subdivision {
    name: "Xızı",
    code: "AZ-XIZ",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-XIZ",
};

pub const AZ_XVD: Subdivision = Subdivision {
    name: "Xocavәnd",
    code: "AZ-XVD",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-XVD",
};

pub const AZ_YAR: Subdivision = Subdivision {
    name: "Yardımlı",
    code: "AZ-YAR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-YAR",
};

pub const AZ_YEV: Subdivision = Subdivision {
    name: "Yevlax",
    code: "AZ-YEV",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-YEV",
};

pub const AZ_YE: Subdivision = Subdivision {
    name: "Yevlax",
    code: "AZ-YE",
    subdivision_type: "Municipality",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-YE",
};

pub const AZ_ZAN: Subdivision = Subdivision {
    name: "Zәngilan",
    code: "AZ-ZAN",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-ZAN",
};

pub const AZ_ZAQ: Subdivision = Subdivision {
    name: "Zaqatala",
    code: "AZ-ZAQ",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-ZAQ",
};

pub const AZ_ZAR: Subdivision = Subdivision {
    name: "Zәrdab",
    code: "AZ-ZAR",
    subdivision_type: "Rayon",
    country_name: "Azerbaijan",
    country_code: "AZ",
    region_code: "AZ-ZAR",
};

pub const BS_AK: Subdivision = Subdivision {
    name: "Acklins",
    code: "BS-AK",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-AK",
};

pub const BS_BI: Subdivision = Subdivision {
    name: "Bimini",
    code: "BS-BI",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-BI",
};

pub const BS_BP: Subdivision = Subdivision {
    name: "Black Point",
    code: "BS-BP",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-BP",
};

pub const BS_BY: Subdivision = Subdivision {
    name: "Berry Islands",
    code: "BS-BY",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-BY",
};

pub const BS_CE: Subdivision = Subdivision {
    name: "Central Eleuthera",
    code: "BS-CE",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-CE",
};

pub const BS_CI: Subdivision = Subdivision {
    name: "Cat Island",
    code: "BS-CI",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-CI",
};

pub const BS_CK: Subdivision = Subdivision {
    name: "Crooked Island and Long Cay",
    code: "BS-CK",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-CK",
};

pub const BS_CO: Subdivision = Subdivision {
    name: "Central Abaco",
    code: "BS-CO",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-CO",
};

pub const BS_CS: Subdivision = Subdivision {
    name: "Central Andros",
    code: "BS-CS",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-CS",
};

pub const BS_EG: Subdivision = Subdivision {
    name: "East Grand Bahama",
    code: "BS-EG",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-EG",
};

pub const BS_EX: Subdivision = Subdivision {
    name: "Exuma",
    code: "BS-EX",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-EX",
};

pub const BS_FP: Subdivision = Subdivision {
    name: "City of Freeport",
    code: "BS-FP",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-FP",
};

pub const BS_GC: Subdivision = Subdivision {
    name: "Grand Cay",
    code: "BS-GC",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-GC",
};

pub const BS_HI: Subdivision = Subdivision {
    name: "Harbour Island",
    code: "BS-HI",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-HI",
};

pub const BS_HT: Subdivision = Subdivision {
    name: "Hope Town",
    code: "BS-HT",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-HT",
};

pub const BS_IN: Subdivision = Subdivision {
    name: "Inagua",
    code: "BS-IN",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-IN",
};

pub const BS_LI: Subdivision = Subdivision {
    name: "Long Island",
    code: "BS-LI",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-LI",
};

pub const BS_MC: Subdivision = Subdivision {
    name: "Mangrove Cay",
    code: "BS-MC",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-MC",
};

pub const BS_MG: Subdivision = Subdivision {
    name: "Mayaguana",
    code: "BS-MG",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-MG",
};

pub const BS_MI: Subdivision = Subdivision {
    name: "Moore's Island",
    code: "BS-MI",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-MI",
};

pub const BS_NE: Subdivision = Subdivision {
    name: "North Eleuthera",
    code: "BS-NE",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-NE",
};

pub const BS_NO: Subdivision = Subdivision {
    name: "North Abaco",
    code: "BS-NO",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-NO",
};

pub const BS_NS: Subdivision = Subdivision {
    name: "North Andros",
    code: "BS-NS",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-NS",
};

pub const BS_RC: Subdivision = Subdivision {
    name: "Rum Cay",
    code: "BS-RC",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-RC",
};

pub const BS_RI: Subdivision = Subdivision {
    name: "Ragged Island",
    code: "BS-RI",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-RI",
};

pub const BS_SA: Subdivision = Subdivision {
    name: "South Andros",
    code: "BS-SA",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-SA",
};

pub const BS_SE: Subdivision = Subdivision {
    name: "South Eleuthera",
    code: "BS-SE",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-SE",
};

pub const BS_SO: Subdivision = Subdivision {
    name: "South Abaco",
    code: "BS-SO",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-SO",
};

pub const BS_SS: Subdivision = Subdivision {
    name: "San Salvador",
    code: "BS-SS",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-SS",
};

pub const BS_SW: Subdivision = Subdivision {
    name: "Spanish Wells",
    code: "BS-SW",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-SW",
};

pub const BS_WG: Subdivision = Subdivision {
    name: "West Grand Bahama",
    code: "BS-WG",
    subdivision_type: "District",
    country_name: "Bahamas",
    country_code: "BS",
    region_code: "BS-WG",
};

pub const BH_13: Subdivision = Subdivision {
    name: "Al Manāmah",
    code: "BH-13",
    subdivision_type: "Governorate",
    country_name: "Bahrain",
    country_code: "BH",
    region_code: "BH-13",
};

pub const BH_14: Subdivision = Subdivision {
    name: "Al Janūbīyah",
    code: "BH-14",
    subdivision_type: "Governorate",
    country_name: "Bahrain",
    country_code: "BH",
    region_code: "BH-14",
};

pub const BH_15: Subdivision = Subdivision {
    name: "Al Muḩarraq",
    code: "BH-15",
    subdivision_type: "Governorate",
    country_name: "Bahrain",
    country_code: "BH",
    region_code: "BH-15",
};

pub const BH_16: Subdivision = Subdivision {
    name: "Al Wusţá",
    code: "BH-16",
    subdivision_type: "Governorate",
    country_name: "Bahrain",
    country_code: "BH",
    region_code: "BH-16",
};

pub const BH_17: Subdivision = Subdivision {
    name: "Ash Shamālīyah",
    code: "BH-17",
    subdivision_type: "Governorate",
    country_name: "Bahrain",
    country_code: "BH",
    region_code: "BH-17",
};

pub const BD_01: Subdivision = Subdivision {
    name: "Bandarban",
    code: "BD-01",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-01",
};

pub const BD_02: Subdivision = Subdivision {
    name: "Barguna",
    code: "BD-02",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-02",
};

pub const BD_03: Subdivision = Subdivision {
    name: "Bogra",
    code: "BD-03",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-03",
};

pub const BD_04: Subdivision = Subdivision {
    name: "Brahmanbaria",
    code: "BD-04",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-04",
};

pub const BD_05: Subdivision = Subdivision {
    name: "Bagerhat",
    code: "BD-05",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-05",
};

pub const BD_06: Subdivision = Subdivision {
    name: "Barisal",
    code: "BD-06",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-06",
};

pub const BD_07: Subdivision = Subdivision {
    name: "Bhola",
    code: "BD-07",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-07",
};

pub const BD_08: Subdivision = Subdivision {
    name: "Comilla",
    code: "BD-08",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-08",
};

pub const BD_09: Subdivision = Subdivision {
    name: "Chandpur",
    code: "BD-09",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-09",
};

pub const BD_10: Subdivision = Subdivision {
    name: "Chittagong",
    code: "BD-10",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-10",
};

pub const BD_11: Subdivision = Subdivision {
    name: "Cox's Bazar",
    code: "BD-11",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-11",
};

pub const BD_12: Subdivision = Subdivision {
    name: "Chuadanga",
    code: "BD-12",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-12",
};

pub const BD_13: Subdivision = Subdivision {
    name: "Dhaka",
    code: "BD-13",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-13",
};

pub const BD_14: Subdivision = Subdivision {
    name: "Dinajpur",
    code: "BD-14",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-14",
};

pub const BD_15: Subdivision = Subdivision {
    name: "Faridpur",
    code: "BD-15",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-15",
};

pub const BD_16: Subdivision = Subdivision {
    name: "Feni",
    code: "BD-16",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-16",
};

pub const BD_17: Subdivision = Subdivision {
    name: "Gopalganj",
    code: "BD-17",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-17",
};

pub const BD_18: Subdivision = Subdivision {
    name: "Gazipur",
    code: "BD-18",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-18",
};

pub const BD_19: Subdivision = Subdivision {
    name: "Gaibandha",
    code: "BD-19",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-19",
};

pub const BD_20: Subdivision = Subdivision {
    name: "Habiganj",
    code: "BD-20",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-20",
};

pub const BD_21: Subdivision = Subdivision {
    name: "Jamalpur",
    code: "BD-21",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-21",
};

pub const BD_22: Subdivision = Subdivision {
    name: "Jessore",
    code: "BD-22",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-22",
};

pub const BD_23: Subdivision = Subdivision {
    name: "Jhenaidah",
    code: "BD-23",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-23",
};

pub const BD_24: Subdivision = Subdivision {
    name: "Jaipurhat",
    code: "BD-24",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-24",
};

pub const BD_25: Subdivision = Subdivision {
    name: "Jhalakati",
    code: "BD-25",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-25",
};

pub const BD_26: Subdivision = Subdivision {
    name: "Kishoreganj",
    code: "BD-26",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-26",
};

pub const BD_27: Subdivision = Subdivision {
    name: "Khulna",
    code: "BD-27",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-27",
};

pub const BD_28: Subdivision = Subdivision {
    name: "Kurigram",
    code: "BD-28",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-28",
};

pub const BD_29: Subdivision = Subdivision {
    name: "Khagrachari",
    code: "BD-29",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-29",
};

pub const BD_30: Subdivision = Subdivision {
    name: "Kushtia",
    code: "BD-30",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-30",
};

pub const BD_31: Subdivision = Subdivision {
    name: "Lakshmipur",
    code: "BD-31",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-31",
};

pub const BD_32: Subdivision = Subdivision {
    name: "Lalmonirhat",
    code: "BD-32",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-32",
};

pub const BD_33: Subdivision = Subdivision {
    name: "Manikganj",
    code: "BD-33",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-33",
};

pub const BD_34: Subdivision = Subdivision {
    name: "Mymensingh",
    code: "BD-34",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-34",
};

pub const BD_35: Subdivision = Subdivision {
    name: "Munshiganj",
    code: "BD-35",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-35",
};

pub const BD_36: Subdivision = Subdivision {
    name: "Madaripur",
    code: "BD-36",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-36",
};

pub const BD_37: Subdivision = Subdivision {
    name: "Magura",
    code: "BD-37",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-37",
};

pub const BD_38: Subdivision = Subdivision {
    name: "Moulvibazar",
    code: "BD-38",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-38",
};

pub const BD_39: Subdivision = Subdivision {
    name: "Meherpur",
    code: "BD-39",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-39",
};

pub const BD_40: Subdivision = Subdivision {
    name: "Narayanganj",
    code: "BD-40",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-40",
};

pub const BD_41: Subdivision = Subdivision {
    name: "Netrakona",
    code: "BD-41",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-41",
};

pub const BD_42: Subdivision = Subdivision {
    name: "Narsingdi",
    code: "BD-42",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-42",
};

pub const BD_43: Subdivision = Subdivision {
    name: "Narail",
    code: "BD-43",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-43",
};

pub const BD_44: Subdivision = Subdivision {
    name: "Natore",
    code: "BD-44",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-44",
};

pub const BD_45: Subdivision = Subdivision {
    name: "Nawabganj",
    code: "BD-45",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-45",
};

pub const BD_46: Subdivision = Subdivision {
    name: "Nilphamari",
    code: "BD-46",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-46",
};

pub const BD_47: Subdivision = Subdivision {
    name: "Noakhali",
    code: "BD-47",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-47",
};

pub const BD_48: Subdivision = Subdivision {
    name: "Naogaon",
    code: "BD-48",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-48",
};

pub const BD_49: Subdivision = Subdivision {
    name: "Pabna",
    code: "BD-49",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-49",
};

pub const BD_50: Subdivision = Subdivision {
    name: "Pirojpur",
    code: "BD-50",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-50",
};

pub const BD_51: Subdivision = Subdivision {
    name: "Patuakhali",
    code: "BD-51",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-51",
};

pub const BD_52: Subdivision = Subdivision {
    name: "Panchagarh",
    code: "BD-52",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-52",
};

pub const BD_53: Subdivision = Subdivision {
    name: "Rajbari",
    code: "BD-53",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-53",
};

pub const BD_54: Subdivision = Subdivision {
    name: "Rajshahi",
    code: "BD-54",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-54",
};

pub const BD_55: Subdivision = Subdivision {
    name: "Rangpur",
    code: "BD-55",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-55",
};

pub const BD_56: Subdivision = Subdivision {
    name: "Rangamati",
    code: "BD-56",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-56",
};

pub const BD_57: Subdivision = Subdivision {
    name: "Sherpur",
    code: "BD-57",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-57",
};

pub const BD_58: Subdivision = Subdivision {
    name: "Satkhira",
    code: "BD-58",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-58",
};

pub const BD_59: Subdivision = Subdivision {
    name: "Sirajganj",
    code: "BD-59",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-59",
};

pub const BD_60: Subdivision = Subdivision {
    name: "Sylhet",
    code: "BD-60",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-60",
};

pub const BD_61: Subdivision = Subdivision {
    name: "Sunamganj",
    code: "BD-61",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-61",
};

pub const BD_62: Subdivision = Subdivision {
    name: "Shariatpur",
    code: "BD-62",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-62",
};

pub const BD_63: Subdivision = Subdivision {
    name: "Tangail",
    code: "BD-63",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-63",
};

pub const BD_64: Subdivision = Subdivision {
    name: "Thakurgaon",
    code: "BD-64",
    subdivision_type: "District",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-64",
};

pub const BD_A: Subdivision = Subdivision {
    name: "Barisal",
    code: "BD-A",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-A",
};

pub const BD_B: Subdivision = Subdivision {
    name: "Chittagong",
    code: "BD-B",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-B",
};

pub const BD_C: Subdivision = Subdivision {
    name: "Dhaka",
    code: "BD-C",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-C",
};

pub const BD_D: Subdivision = Subdivision {
    name: "Khulna",
    code: "BD-D",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-D",
};

pub const BD_E: Subdivision = Subdivision {
    name: "Rajshahi",
    code: "BD-E",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-E",
};

pub const BD_F: Subdivision = Subdivision {
    name: "Rangpur",
    code: "BD-F",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-F",
};

pub const BD_G: Subdivision = Subdivision {
    name: "Sylhet",
    code: "BD-G",
    subdivision_type: "Division",
    country_name: "Bangladesh",
    country_code: "BD",
    region_code: "BD-G",
};

pub const BB_01: Subdivision = Subdivision {
    name: "Christ Church",
    code: "BB-01",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-01",
};

pub const BB_02: Subdivision = Subdivision {
    name: "Saint Andrew",
    code: "BB-02",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-02",
};

pub const BB_03: Subdivision = Subdivision {
    name: "Saint George",
    code: "BB-03",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-03",
};

pub const BB_04: Subdivision = Subdivision {
    name: "Saint James",
    code: "BB-04",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-04",
};

pub const BB_05: Subdivision = Subdivision {
    name: "Saint John",
    code: "BB-05",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-05",
};

pub const BB_06: Subdivision = Subdivision {
    name: "Saint Joseph",
    code: "BB-06",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-06",
};

pub const BB_07: Subdivision = Subdivision {
    name: "Saint Lucy",
    code: "BB-07",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-07",
};

pub const BB_08: Subdivision = Subdivision {
    name: "Saint Michael",
    code: "BB-08",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-08",
};

pub const BB_09: Subdivision = Subdivision {
    name: "Saint Peter",
    code: "BB-09",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-09",
};

pub const BB_10: Subdivision = Subdivision {
    name: "Saint Philip",
    code: "BB-10",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-10",
};

pub const BB_11: Subdivision = Subdivision {
    name: "Saint Thomas",
    code: "BB-11",
    subdivision_type: "Parish",
    country_name: "Barbados",
    country_code: "BB",
    region_code: "BB-11",
};

pub const BY_BR: Subdivision = Subdivision {
    name: "Bresckaja voblasć",
    code: "BY-BR",
    subdivision_type: "Oblast",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-BR",
};

pub const BY_HM: Subdivision = Subdivision {
    name: "Gorod Minsk",
    code: "BY-HM",
    subdivision_type: "City",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-HM",
};

pub const BY_HO: Subdivision = Subdivision {
    name: "Gomel'skaja oblast'",
    code: "BY-HO",
    subdivision_type: "Oblast",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-HO",
};

pub const BY_HR: Subdivision = Subdivision {
    name: "Grodnenskaja oblast'",
    code: "BY-HR",
    subdivision_type: "Oblast",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-HR",
};

pub const BY_MA: Subdivision = Subdivision {
    name: "Mahilioŭskaja voblasć",
    code: "BY-MA",
    subdivision_type: "Oblast",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-MA",
};

pub const BY_MI: Subdivision = Subdivision {
    name: "Minskaja oblast'",
    code: "BY-MI",
    subdivision_type: "Oblast",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-MI",
};

pub const BY_VI: Subdivision = Subdivision {
    name: "Viciebskaja voblasć",
    code: "BY-VI",
    subdivision_type: "Oblast",
    country_name: "Belarus",
    country_code: "BY",
    region_code: "BY-VI",
};

pub const BE_BRU: Subdivision = Subdivision {
    name: "Brussels Hoofdstedelijk Gewest",
    code: "BE-BRU",
    subdivision_type: "Region",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-BRU",
};

pub const BE_VAN: Subdivision = Subdivision {
    name: "Antwerpen",
    code: "BE-VAN",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-VAN",
};

pub const BE_VBR: Subdivision = Subdivision {
    name: "Vlaams-Brabant",
    code: "BE-VBR",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-VBR",
};

pub const BE_VLG: Subdivision = Subdivision {
    name: "Vlaams Gewest",
    code: "BE-VLG",
    subdivision_type: "Region",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-VLG",
};

pub const BE_VLI: Subdivision = Subdivision {
    name: "Limburg",
    code: "BE-VLI",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-VLI",
};

pub const BE_VOV: Subdivision = Subdivision {
    name: "Oost-Vlaanderen",
    code: "BE-VOV",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-VOV",
};

pub const BE_VWV: Subdivision = Subdivision {
    name: "West-Vlaanderen",
    code: "BE-VWV",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-VWV",
};

pub const BE_WAL: Subdivision = Subdivision {
    name: "wallonne, Région",
    code: "BE-WAL",
    subdivision_type: "Region",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-WAL",
};

pub const BE_WBR: Subdivision = Subdivision {
    name: "Brabant wallon",
    code: "BE-WBR",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-WBR",
};

pub const BE_WHT: Subdivision = Subdivision {
    name: "Hainaut",
    code: "BE-WHT",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-WHT",
};

pub const BE_WLG: Subdivision = Subdivision {
    name: "Liège",
    code: "BE-WLG",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-WLG",
};

pub const BE_WLX: Subdivision = Subdivision {
    name: "Luxembourg",
    code: "BE-WLX",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-WLX",
};

pub const BE_WNA: Subdivision = Subdivision {
    name: "Namur",
    code: "BE-WNA",
    subdivision_type: "Province",
    country_name: "Belgium",
    country_code: "BE",
    region_code: "BE-WNA",
};

pub const BZ_BZ: Subdivision = Subdivision {
    name: "Belize",
    code: "BZ-BZ",
    subdivision_type: "District",
    country_name: "Belize",
    country_code: "BZ",
    region_code: "BZ-BZ",
};

pub const BZ_CY: Subdivision = Subdivision {
    name: "Cayo",
    code: "BZ-CY",
    subdivision_type: "District",
    country_name: "Belize",
    country_code: "BZ",
    region_code: "BZ-CY",
};

pub const BZ_CZL: Subdivision = Subdivision {
    name: "Corozal",
    code: "BZ-CZL",
    subdivision_type: "District",
    country_name: "Belize",
    country_code: "BZ",
    region_code: "BZ-CZL",
};

pub const BZ_OW: Subdivision = Subdivision {
    name: "Orange Walk",
    code: "BZ-OW",
    subdivision_type: "District",
    country_name: "Belize",
    country_code: "BZ",
    region_code: "BZ-OW",
};

pub const BZ_SC: Subdivision = Subdivision {
    name: "Stann Creek",
    code: "BZ-SC",
    subdivision_type: "District",
    country_name: "Belize",
    country_code: "BZ",
    region_code: "BZ-SC",
};

pub const BZ_TOL: Subdivision = Subdivision {
    name: "Toledo",
    code: "BZ-TOL",
    subdivision_type: "District",
    country_name: "Belize",
    country_code: "BZ",
    region_code: "BZ-TOL",
};

pub const BJ_AK: Subdivision = Subdivision {
    name: "Atakora",
    code: "BJ-AK",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-AK",
};

pub const BJ_AL: Subdivision = Subdivision {
    name: "Alibori",
    code: "BJ-AL",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-AL",
};

pub const BJ_AQ: Subdivision = Subdivision {
    name: "Atlantique",
    code: "BJ-AQ",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-AQ",
};

pub const BJ_BO: Subdivision = Subdivision {
    name: "Borgou",
    code: "BJ-BO",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-BO",
};

pub const BJ_CO: Subdivision = Subdivision {
    name: "Collines",
    code: "BJ-CO",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-CO",
};

pub const BJ_DO: Subdivision = Subdivision {
    name: "Donga",
    code: "BJ-DO",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-DO",
};

pub const BJ_KO: Subdivision = Subdivision {
    name: "Kouffo",
    code: "BJ-KO",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-KO",
};

pub const BJ_LI: Subdivision = Subdivision {
    name: "Littoral",
    code: "BJ-LI",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-LI",
};

pub const BJ_MO: Subdivision = Subdivision {
    name: "Mono",
    code: "BJ-MO",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-MO",
};

pub const BJ_OU: Subdivision = Subdivision {
    name: "Ouémé",
    code: "BJ-OU",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-OU",
};

pub const BJ_PL: Subdivision = Subdivision {
    name: "Plateau",
    code: "BJ-PL",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-PL",
};

pub const BJ_ZO: Subdivision = Subdivision {
    name: "Zou",
    code: "BJ-ZO",
    subdivision_type: "Department",
    country_name: "Benin",
    country_code: "BJ",
    region_code: "BJ-ZO",
};

pub const BM_BM: Subdivision = Subdivision {
    name: "Bermuda",
    code: "BM-BM",
    subdivision_type: "Country",
    country_name: "Bermuda",
    country_code: "BM",
    region_code: "BM-BM",
};

pub const BT_11: Subdivision = Subdivision {
    name: "Paro",
    code: "BT-11",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-11",
};

pub const BT_12: Subdivision = Subdivision {
    name: "Chhukha",
    code: "BT-12",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-12",
};

pub const BT_13: Subdivision = Subdivision {
    name: "Ha",
    code: "BT-13",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-13",
};

pub const BT_14: Subdivision = Subdivision {
    name: "Samtse",
    code: "BT-14",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-14",
};

pub const BT_15: Subdivision = Subdivision {
    name: "Thimphu",
    code: "BT-15",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-15",
};

pub const BT_21: Subdivision = Subdivision {
    name: "Tsirang",
    code: "BT-21",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-21",
};

pub const BT_22: Subdivision = Subdivision {
    name: "Dagana",
    code: "BT-22",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-22",
};

pub const BT_23: Subdivision = Subdivision {
    name: "Punakha",
    code: "BT-23",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-23",
};

pub const BT_24: Subdivision = Subdivision {
    name: "Wangdue Phodrang",
    code: "BT-24",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-24",
};

pub const BT_31: Subdivision = Subdivision {
    name: "Sarpang",
    code: "BT-31",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-31",
};

pub const BT_32: Subdivision = Subdivision {
    name: "Trongsa",
    code: "BT-32",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-32",
};

pub const BT_33: Subdivision = Subdivision {
    name: "Bumthang",
    code: "BT-33",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-33",
};

pub const BT_34: Subdivision = Subdivision {
    name: "Zhemgang",
    code: "BT-34",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-34",
};

pub const BT_41: Subdivision = Subdivision {
    name: "Trashigang",
    code: "BT-41",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-41",
};

pub const BT_42: Subdivision = Subdivision {
    name: "Monggar",
    code: "BT-42",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-42",
};

pub const BT_43: Subdivision = Subdivision {
    name: "Pemagatshel",
    code: "BT-43",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-43",
};

pub const BT_44: Subdivision = Subdivision {
    name: "Lhuentse",
    code: "BT-44",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-44",
};

pub const BT_45: Subdivision = Subdivision {
    name: "Samdrup Jongkha",
    code: "BT-45",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-45",
};

pub const BT_GA: Subdivision = Subdivision {
    name: "Gasa",
    code: "BT-GA",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-GA",
};

pub const BT_TY: Subdivision = Subdivision {
    name: "Trashi Yangtse",
    code: "BT-TY",
    subdivision_type: "District",
    country_name: "Bhutan",
    country_code: "BT",
    region_code: "BT-TY",
};

pub const BO_B: Subdivision = Subdivision {
    name: "El Beni",
    code: "BO-B",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-B",
};

pub const BO_C: Subdivision = Subdivision {
    name: "Cochabamba",
    code: "BO-C",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-C",
};

pub const BO_H: Subdivision = Subdivision {
    name: "Chuquisaca",
    code: "BO-H",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-H",
};

pub const BO_L: Subdivision = Subdivision {
    name: "La Paz",
    code: "BO-L",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-L",
};

pub const BO_N: Subdivision = Subdivision {
    name: "Pando",
    code: "BO-N",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-N",
};

pub const BO_O: Subdivision = Subdivision {
    name: "Oruro",
    code: "BO-O",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-O",
};

pub const BO_P: Subdivision = Subdivision {
    name: "Potosí",
    code: "BO-P",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-P",
};

pub const BO_S: Subdivision = Subdivision {
    name: "Santa Cruz",
    code: "BO-S",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-S",
};

pub const BO_T: Subdivision = Subdivision {
    name: "Tarija",
    code: "BO-T",
    subdivision_type: "Department",
    country_name: "Bolivia",
    country_code: "BO",
    region_code: "BO-T",
};

pub const BQ_BO: Subdivision = Subdivision {
    name: "Bonaire",
    code: "BQ-BO",
    subdivision_type: "Special municipality",
    country_name: "Bonaire, Sint Eustatius and Saba",
    country_code: "BQ",
    region_code: "BQ-BO",
};

pub const BQ_SA: Subdivision = Subdivision {
    name: "Saba",
    code: "BQ-SA",
    subdivision_type: "Special municipality",
    country_name: "Bonaire, Sint Eustatius and Saba",
    country_code: "BQ",
    region_code: "BQ-SA",
};

pub const BQ_SE: Subdivision = Subdivision {
    name: "Sint Eustatius",
    code: "BQ-SE",
    subdivision_type: "Special municipality",
    country_name: "Bonaire, Sint Eustatius and Saba",
    country_code: "BQ",
    region_code: "BQ-SE",
};

pub const BA_01: Subdivision = Subdivision {
    name: "Unsko-sanska županija",
    code: "BA-01",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-01",
};

pub const BA_02: Subdivision = Subdivision {
    name: "Posavska županija",
    code: "BA-02",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-02",
};

pub const BA_03: Subdivision = Subdivision {
    name: "Tuzlanska županija",
    code: "BA-03",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-03",
};

pub const BA_04: Subdivision = Subdivision {
    name: "Zeničko-dobojska županija",
    code: "BA-04",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-04",
};

pub const BA_05: Subdivision = Subdivision {
    name: "Bosansko-podrinjska županija",
    code: "BA-05",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-05",
};

pub const BA_06: Subdivision = Subdivision {
    name: "Srednjobosanska županija",
    code: "BA-06",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-06",
};

pub const BA_07: Subdivision = Subdivision {
    name: "Hercegovačko-neretvanska županija",
    code: "BA-07",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-07",
};

pub const BA_08: Subdivision = Subdivision {
    name: "Zapadnohercegovačka županija",
    code: "BA-08",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-08",
};

pub const BA_09: Subdivision = Subdivision {
    name: "Kanton Sarajevo",
    code: "BA-09",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-09",
};

pub const BA_10: Subdivision = Subdivision {
    name: "Kanton br. 10",
    code: "BA-10",
    subdivision_type: "Canton",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-10",
};

pub const BA_BIH: Subdivision = Subdivision {
    name: "Federacija Bosne i Hercegovine",
    code: "BA-BIH",
    subdivision_type: "Entity",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-BIH",
};

pub const BA_BRC: Subdivision = Subdivision {
    name: "Brčko distrikt",
    code: "BA-BRC",
    subdivision_type: "District with special status",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-BRC",
};

pub const BA_SRP: Subdivision = Subdivision {
    name: "Republika Srpska",
    code: "BA-SRP",
    subdivision_type: "Entity",
    country_name: "Bosnia and Herzegovina",
    country_code: "BA",
    region_code: "BA-SRP",
};

pub const BW_CE: Subdivision = Subdivision {
    name: "Central",
    code: "BW-CE",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-CE",
};

pub const BW_CH: Subdivision = Subdivision {
    name: "Chobe",
    code: "BW-CH",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-CH",
};

pub const BW_FR: Subdivision = Subdivision {
    name: "Francistown",
    code: "BW-FR",
    subdivision_type: "City",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-FR",
};

pub const BW_GA: Subdivision = Subdivision {
    name: "Gaborone",
    code: "BW-GA",
    subdivision_type: "City",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-GA",
};

pub const BW_GH: Subdivision = Subdivision {
    name: "Ghanzi",
    code: "BW-GH",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-GH",
};

pub const BW_JW: Subdivision = Subdivision {
    name: "Jwaneng",
    code: "BW-JW",
    subdivision_type: "Town",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-JW",
};

pub const BW_KG: Subdivision = Subdivision {
    name: "Kgalagadi",
    code: "BW-KG",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-KG",
};

pub const BW_KL: Subdivision = Subdivision {
    name: "Kgatleng",
    code: "BW-KL",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-KL",
};

pub const BW_KW: Subdivision = Subdivision {
    name: "Kweneng",
    code: "BW-KW",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-KW",
};

pub const BW_LO: Subdivision = Subdivision {
    name: "Lobatse",
    code: "BW-LO",
    subdivision_type: "Town",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-LO",
};

pub const BW_NE: Subdivision = Subdivision {
    name: "North East",
    code: "BW-NE",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-NE",
};

pub const BW_NW: Subdivision = Subdivision {
    name: "North West",
    code: "BW-NW",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-NW",
};

pub const BW_SE: Subdivision = Subdivision {
    name: "South East",
    code: "BW-SE",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-SE",
};

pub const BW_SO: Subdivision = Subdivision {
    name: "Southern",
    code: "BW-SO",
    subdivision_type: "District",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-SO",
};

pub const BW_SP: Subdivision = Subdivision {
    name: "Selibe Phikwe",
    code: "BW-SP",
    subdivision_type: "Town",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-SP",
};

pub const BW_ST: Subdivision = Subdivision {
    name: "Sowa Town",
    code: "BW-ST",
    subdivision_type: "Town",
    country_name: "Botswana",
    country_code: "BW",
    region_code: "BW-ST",
};

pub const BV_BV: Subdivision = Subdivision {
    name: "Bouvet Island",
    code: "BV-BV",
    subdivision_type: "Country",
    country_name: "Bouvet Island",
    country_code: "BV",
    region_code: "BV-BV",
};

pub const BR_AC: Subdivision = Subdivision {
    name: "Acre",
    code: "BR-AC",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-AC",
};

pub const BR_AL: Subdivision = Subdivision {
    name: "Alagoas",
    code: "BR-AL",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-AL",
};

pub const BR_AM: Subdivision = Subdivision {
    name: "Amazonas",
    code: "BR-AM",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-AM",
};

pub const BR_AP: Subdivision = Subdivision {
    name: "Amapá",
    code: "BR-AP",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-AP",
};

pub const BR_BA: Subdivision = Subdivision {
    name: "Bahia",
    code: "BR-BA",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-BA",
};

pub const BR_CE: Subdivision = Subdivision {
    name: "Ceará",
    code: "BR-CE",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-CE",
};

pub const BR_DF: Subdivision = Subdivision {
    name: "Distrito Federal",
    code: "BR-DF",
    subdivision_type: "Federal district",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-DF",
};

pub const BR_ES: Subdivision = Subdivision {
    name: "Espírito Santo",
    code: "BR-ES",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-ES",
};

pub const BR_GO: Subdivision = Subdivision {
    name: "Goiás",
    code: "BR-GO",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-GO",
};

pub const BR_MA: Subdivision = Subdivision {
    name: "Maranhão",
    code: "BR-MA",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-MA",
};

pub const BR_MG: Subdivision = Subdivision {
    name: "Minas Gerais",
    code: "BR-MG",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-MG",
};

pub const BR_MS: Subdivision = Subdivision {
    name: "Mato Grosso do Sul",
    code: "BR-MS",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-MS",
};

pub const BR_MT: Subdivision = Subdivision {
    name: "Mato Grosso",
    code: "BR-MT",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-MT",
};

pub const BR_PA: Subdivision = Subdivision {
    name: "Pará",
    code: "BR-PA",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-PA",
};

pub const BR_PB: Subdivision = Subdivision {
    name: "Paraíba",
    code: "BR-PB",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-PB",
};

pub const BR_PE: Subdivision = Subdivision {
    name: "Pernambuco",
    code: "BR-PE",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-PE",
};

pub const BR_PI: Subdivision = Subdivision {
    name: "Piauí",
    code: "BR-PI",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-PI",
};

pub const BR_PR: Subdivision = Subdivision {
    name: "Paraná",
    code: "BR-PR",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-PR",
};

pub const BR_RJ: Subdivision = Subdivision {
    name: "Rio de Janeiro",
    code: "BR-RJ",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-RJ",
};

pub const BR_RN: Subdivision = Subdivision {
    name: "Rio Grande do Norte",
    code: "BR-RN",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-RN",
};

pub const BR_RO: Subdivision = Subdivision {
    name: "Rondônia",
    code: "BR-RO",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-RO",
};

pub const BR_RR: Subdivision = Subdivision {
    name: "Roraima",
    code: "BR-RR",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-RR",
};

pub const BR_RS: Subdivision = Subdivision {
    name: "Rio Grande do Sul",
    code: "BR-RS",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-RS",
};

pub const BR_SC: Subdivision = Subdivision {
    name: "Santa Catarina",
    code: "BR-SC",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-SC",
};

pub const BR_SE: Subdivision = Subdivision {
    name: "Sergipe",
    code: "BR-SE",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-SE",
};

pub const BR_SP: Subdivision = Subdivision {
    name: "São Paulo",
    code: "BR-SP",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-SP",
};

pub const BR_TO: Subdivision = Subdivision {
    name: "Tocantins",
    code: "BR-TO",
    subdivision_type: "State",
    country_name: "Brazil",
    country_code: "BR",
    region_code: "BR-TO",
};

pub const IO_IO: Subdivision = Subdivision {
    name: "British Indian Ocean Territory",
    code: "IO-IO",
    subdivision_type: "Country",
    country_name: "British Indian Ocean Territory",
    country_code: "IO",
    region_code: "IO-IO",
};

pub const BN_BE: Subdivision = Subdivision {
    name: "Belait",
    code: "BN-BE",
    subdivision_type: "District",
    country_name: "Brunei Darussalam",
    country_code: "BN",
    region_code: "BN-BE",
};

pub const BN_BM: Subdivision = Subdivision {
    name: "Brunei-Muara",
    code: "BN-BM",
    subdivision_type: "District",
    country_name: "Brunei Darussalam",
    country_code: "BN",
    region_code: "BN-BM",
};

pub const BN_TE: Subdivision = Subdivision {
    name: "Temburong",
    code: "BN-TE",
    subdivision_type: "District",
    country_name: "Brunei Darussalam",
    country_code: "BN",
    region_code: "BN-TE",
};

pub const BN_TU: Subdivision = Subdivision {
    name: "Tutong",
    code: "BN-TU",
    subdivision_type: "District",
    country_name: "Brunei Darussalam",
    country_code: "BN",
    region_code: "BN-TU",
};

pub const BG_01: Subdivision = Subdivision {
    name: "Blagoevgrad",
    code: "BG-01",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-01",
};

pub const BG_02: Subdivision = Subdivision {
    name: "Burgas",
    code: "BG-02",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-02",
};

pub const BG_03: Subdivision = Subdivision {
    name: "Varna",
    code: "BG-03",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-03",
};

pub const BG_04: Subdivision = Subdivision {
    name: "Veliko Tarnovo",
    code: "BG-04",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-04",
};

pub const BG_05: Subdivision = Subdivision {
    name: "Vidin",
    code: "BG-05",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-05",
};

pub const BG_06: Subdivision = Subdivision {
    name: "Vratsa",
    code: "BG-06",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-06",
};

pub const BG_07: Subdivision = Subdivision {
    name: "Gabrovo",
    code: "BG-07",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-07",
};

pub const BG_08: Subdivision = Subdivision {
    name: "Dobrich",
    code: "BG-08",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-08",
};

pub const BG_09: Subdivision = Subdivision {
    name: "Kardzhali",
    code: "BG-09",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-09",
};

pub const BG_10: Subdivision = Subdivision {
    name: "Kyustendil",
    code: "BG-10",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-10",
};

pub const BG_11: Subdivision = Subdivision {
    name: "Lovech",
    code: "BG-11",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-11",
};

pub const BG_12: Subdivision = Subdivision {
    name: "Montana",
    code: "BG-12",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-12",
};

pub const BG_13: Subdivision = Subdivision {
    name: "Pazardzhik",
    code: "BG-13",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-13",
};

pub const BG_14: Subdivision = Subdivision {
    name: "Pernik",
    code: "BG-14",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-14",
};

pub const BG_15: Subdivision = Subdivision {
    name: "Pleven",
    code: "BG-15",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-15",
};

pub const BG_16: Subdivision = Subdivision {
    name: "Plovdiv",
    code: "BG-16",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-16",
};

pub const BG_17: Subdivision = Subdivision {
    name: "Razgrad",
    code: "BG-17",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-17",
};

pub const BG_18: Subdivision = Subdivision {
    name: "Ruse",
    code: "BG-18",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-18",
};

pub const BG_19: Subdivision = Subdivision {
    name: "Silistra",
    code: "BG-19",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-19",
};

pub const BG_20: Subdivision = Subdivision {
    name: "Sliven",
    code: "BG-20",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-20",
};

pub const BG_21: Subdivision = Subdivision {
    name: "Smolyan",
    code: "BG-21",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-21",
};

pub const BG_22: Subdivision = Subdivision {
    name: "Sofia",
    code: "BG-22",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-22",
};

pub const BG_23: Subdivision = Subdivision {
    name: "Sofia",
    code: "BG-23",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-23",
};

pub const BG_24: Subdivision = Subdivision {
    name: "Stara Zagora",
    code: "BG-24",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-24",
};

pub const BG_25: Subdivision = Subdivision {
    name: "Targovishte",
    code: "BG-25",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-25",
};

pub const BG_26: Subdivision = Subdivision {
    name: "Haskovo",
    code: "BG-26",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-26",
};

pub const BG_27: Subdivision = Subdivision {
    name: "Shumen",
    code: "BG-27",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-27",
};

pub const BG_28: Subdivision = Subdivision {
    name: "Yambol",
    code: "BG-28",
    subdivision_type: "Region",
    country_name: "Bulgaria",
    country_code: "BG",
    region_code: "BG-28",
};

pub const BF_01: Subdivision = Subdivision {
    name: "Boucle du Mouhoun",
    code: "BF-01",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-01",
};

pub const BF_02: Subdivision = Subdivision {
    name: "Cascades",
    code: "BF-02",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-02",
};

pub const BF_03: Subdivision = Subdivision {
    name: "Centre",
    code: "BF-03",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-03",
};

pub const BF_04: Subdivision = Subdivision {
    name: "Centre-Est",
    code: "BF-04",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-04",
};

pub const BF_05: Subdivision = Subdivision {
    name: "Centre-Nord",
    code: "BF-05",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-05",
};

pub const BF_06: Subdivision = Subdivision {
    name: "Centre-Ouest",
    code: "BF-06",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-06",
};

pub const BF_07: Subdivision = Subdivision {
    name: "Centre-Sud",
    code: "BF-07",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-07",
};

pub const BF_08: Subdivision = Subdivision {
    name: "Est",
    code: "BF-08",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-08",
};

pub const BF_09: Subdivision = Subdivision {
    name: "Hauts-Bassins",
    code: "BF-09",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-09",
};

pub const BF_10: Subdivision = Subdivision {
    name: "Nord",
    code: "BF-10",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-10",
};

pub const BF_11: Subdivision = Subdivision {
    name: "Plateau-Central",
    code: "BF-11",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-11",
};

pub const BF_12: Subdivision = Subdivision {
    name: "Sahel",
    code: "BF-12",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-12",
};

pub const BF_13: Subdivision = Subdivision {
    name: "Sud-Ouest",
    code: "BF-13",
    subdivision_type: "Region",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-13",
};

pub const BF_BAL: Subdivision = Subdivision {
    name: "Balé",
    code: "BF-BAL",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BAL",
};

pub const BF_BAM: Subdivision = Subdivision {
    name: "Bam",
    code: "BF-BAM",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BAM",
};

pub const BF_BAN: Subdivision = Subdivision {
    name: "Banwa",
    code: "BF-BAN",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BAN",
};

pub const BF_BAZ: Subdivision = Subdivision {
    name: "Bazèga",
    code: "BF-BAZ",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BAZ",
};

pub const BF_BGR: Subdivision = Subdivision {
    name: "Bougouriba",
    code: "BF-BGR",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BGR",
};

pub const BF_BLG: Subdivision = Subdivision {
    name: "Boulgou",
    code: "BF-BLG",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BLG",
};

pub const BF_BLK: Subdivision = Subdivision {
    name: "Boulkiemdé",
    code: "BF-BLK",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-BLK",
};

pub const BF_COM: Subdivision = Subdivision {
    name: "Comoé",
    code: "BF-COM",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-COM",
};

pub const BF_GAN: Subdivision = Subdivision {
    name: "Ganzourgou",
    code: "BF-GAN",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-GAN",
};

pub const BF_GNA: Subdivision = Subdivision {
    name: "Gnagna",
    code: "BF-GNA",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-GNA",
};

pub const BF_GOU: Subdivision = Subdivision {
    name: "Gourma",
    code: "BF-GOU",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-GOU",
};

pub const BF_HOU: Subdivision = Subdivision {
    name: "Houet",
    code: "BF-HOU",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-HOU",
};

pub const BF_IOB: Subdivision = Subdivision {
    name: "Ioba",
    code: "BF-IOB",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-IOB",
};

pub const BF_KAD: Subdivision = Subdivision {
    name: "Kadiogo",
    code: "BF-KAD",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KAD",
};

pub const BF_KEN: Subdivision = Subdivision {
    name: "Kénédougou",
    code: "BF-KEN",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KEN",
};

pub const BF_KMD: Subdivision = Subdivision {
    name: "Komondjari",
    code: "BF-KMD",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KMD",
};

pub const BF_KMP: Subdivision = Subdivision {
    name: "Kompienga",
    code: "BF-KMP",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KMP",
};

pub const BF_KOP: Subdivision = Subdivision {
    name: "Koulpélogo",
    code: "BF-KOP",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KOP",
};

pub const BF_KOS: Subdivision = Subdivision {
    name: "Kossi",
    code: "BF-KOS",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KOS",
};

pub const BF_KOT: Subdivision = Subdivision {
    name: "Kouritenga",
    code: "BF-KOT",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KOT",
};

pub const BF_KOW: Subdivision = Subdivision {
    name: "Kourwéogo",
    code: "BF-KOW",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-KOW",
};

pub const BF_LER: Subdivision = Subdivision {
    name: "Léraba",
    code: "BF-LER",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-LER",
};

pub const BF_LOR: Subdivision = Subdivision {
    name: "Loroum",
    code: "BF-LOR",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-LOR",
};

pub const BF_MOU: Subdivision = Subdivision {
    name: "Mouhoun",
    code: "BF-MOU",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-MOU",
};

pub const BF_NAM: Subdivision = Subdivision {
    name: "Namentenga",
    code: "BF-NAM",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-NAM",
};

pub const BF_NAO: Subdivision = Subdivision {
    name: "Nahouri",
    code: "BF-NAO",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-NAO",
};

pub const BF_NAY: Subdivision = Subdivision {
    name: "Nayala",
    code: "BF-NAY",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-NAY",
};

pub const BF_NOU: Subdivision = Subdivision {
    name: "Noumbiel",
    code: "BF-NOU",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-NOU",
};

pub const BF_OUB: Subdivision = Subdivision {
    name: "Oubritenga",
    code: "BF-OUB",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-OUB",
};

pub const BF_OUD: Subdivision = Subdivision {
    name: "Oudalan",
    code: "BF-OUD",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-OUD",
};

pub const BF_PAS: Subdivision = Subdivision {
    name: "Passoré",
    code: "BF-PAS",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-PAS",
};

pub const BF_PON: Subdivision = Subdivision {
    name: "Poni",
    code: "BF-PON",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-PON",
};

pub const BF_SEN: Subdivision = Subdivision {
    name: "Séno",
    code: "BF-SEN",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-SEN",
};

pub const BF_SIS: Subdivision = Subdivision {
    name: "Sissili",
    code: "BF-SIS",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-SIS",
};

pub const BF_SMT: Subdivision = Subdivision {
    name: "Sanmatenga",
    code: "BF-SMT",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-SMT",
};

pub const BF_SNG: Subdivision = Subdivision {
    name: "Sanguié",
    code: "BF-SNG",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-SNG",
};

pub const BF_SOM: Subdivision = Subdivision {
    name: "Soum",
    code: "BF-SOM",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-SOM",
};

pub const BF_SOR: Subdivision = Subdivision {
    name: "Sourou",
    code: "BF-SOR",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-SOR",
};

pub const BF_TAP: Subdivision = Subdivision {
    name: "Tapoa",
    code: "BF-TAP",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-TAP",
};

pub const BF_TUI: Subdivision = Subdivision {
    name: "Tui",
    code: "BF-TUI",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-TUI",
};

pub const BF_YAG: Subdivision = Subdivision {
    name: "Yagha",
    code: "BF-YAG",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-YAG",
};

pub const BF_YAT: Subdivision = Subdivision {
    name: "Yatenga",
    code: "BF-YAT",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-YAT",
};

pub const BF_ZIR: Subdivision = Subdivision {
    name: "Ziro",
    code: "BF-ZIR",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-ZIR",
};

pub const BF_ZON: Subdivision = Subdivision {
    name: "Zondoma",
    code: "BF-ZON",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-ZON",
};

pub const BF_ZOU: Subdivision = Subdivision {
    name: "Zoundwéogo",
    code: "BF-ZOU",
    subdivision_type: "Province",
    country_name: "Burkina Faso",
    country_code: "BF",
    region_code: "BF-ZOU",
};

pub const BI_BB: Subdivision = Subdivision {
    name: "Bubanza",
    code: "BI-BB",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-BB",
};

pub const BI_BL: Subdivision = Subdivision {
    name: "Bujumbura Rural",
    code: "BI-BL",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-BL",
};

pub const BI_BM: Subdivision = Subdivision {
    name: "Bujumbura Mairie",
    code: "BI-BM",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-BM",
};

pub const BI_BR: Subdivision = Subdivision {
    name: "Bururi",
    code: "BI-BR",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-BR",
};

pub const BI_CA: Subdivision = Subdivision {
    name: "Cankuzo",
    code: "BI-CA",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-CA",
};

pub const BI_CI: Subdivision = Subdivision {
    name: "Cibitoke",
    code: "BI-CI",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-CI",
};

pub const BI_GI: Subdivision = Subdivision {
    name: "Gitega",
    code: "BI-GI",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-GI",
};

pub const BI_KI: Subdivision = Subdivision {
    name: "Kirundo",
    code: "BI-KI",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-KI",
};

pub const BI_KR: Subdivision = Subdivision {
    name: "Karuzi",
    code: "BI-KR",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-KR",
};

pub const BI_KY: Subdivision = Subdivision {
    name: "Kayanza",
    code: "BI-KY",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-KY",
};

pub const BI_MA: Subdivision = Subdivision {
    name: "Makamba",
    code: "BI-MA",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-MA",
};

pub const BI_MU: Subdivision = Subdivision {
    name: "Muramvya",
    code: "BI-MU",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-MU",
};

pub const BI_MW: Subdivision = Subdivision {
    name: "Mwaro",
    code: "BI-MW",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-MW",
};

pub const BI_MY: Subdivision = Subdivision {
    name: "Muyinga",
    code: "BI-MY",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-MY",
};

pub const BI_NG: Subdivision = Subdivision {
    name: "Ngozi",
    code: "BI-NG",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-NG",
};

pub const BI_RT: Subdivision = Subdivision {
    name: "Rutana",
    code: "BI-RT",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-RT",
};

pub const BI_RY: Subdivision = Subdivision {
    name: "Ruyigi",
    code: "BI-RY",
    subdivision_type: "Province",
    country_name: "Burundi",
    country_code: "BI",
    region_code: "BI-RY",
};

pub const KH_10: Subdivision = Subdivision {
    name: "Krâchéh",
    code: "KH-10",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-10",
};

pub const KH_11: Subdivision = Subdivision {
    name: "Môndól Kiri",
    code: "KH-11",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-11",
};

pub const KH_12: Subdivision = Subdivision {
    name: "Phnom Penh",
    code: "KH-12",
    subdivision_type: "Autonomous municipality",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-12",
};

pub const KH_13: Subdivision = Subdivision {
    name: "Preăh Vihéar",
    code: "KH-13",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-13",
};

pub const KH_14: Subdivision = Subdivision {
    name: "Prey Veaeng",
    code: "KH-14",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-14",
};

pub const KH_15: Subdivision = Subdivision {
    name: "Pousaat",
    code: "KH-15",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-15",
};

pub const KH_16: Subdivision = Subdivision {
    name: "Rotanak Kiri",
    code: "KH-16",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-16",
};

pub const KH_17: Subdivision = Subdivision {
    name: "Siem Reab",
    code: "KH-17",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-17",
};

pub const KH_18: Subdivision = Subdivision {
    name: "Krŏng Preăh Sihanouk",
    code: "KH-18",
    subdivision_type: "Autonomous municipality",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-18",
};

pub const KH_19: Subdivision = Subdivision {
    name: "Stoĕng Trêng",
    code: "KH-19",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-19",
};

pub const KH_1: Subdivision = Subdivision {
    name: "Bântéay Méanchey",
    code: "KH-1",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-1",
};

pub const KH_20: Subdivision = Subdivision {
    name: "Svaay Rieng",
    code: "KH-20",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-20",
};

pub const KH_21: Subdivision = Subdivision {
    name: "Taakaev",
    code: "KH-21",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-21",
};

pub const KH_22: Subdivision = Subdivision {
    name: "Ŏtdâr Méanchey",
    code: "KH-22",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-22",
};

pub const KH_23: Subdivision = Subdivision {
    name: "Krong Kaeb",
    code: "KH-23",
    subdivision_type: "Autonomous municipality",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-23",
};

pub const KH_24: Subdivision = Subdivision {
    name: "Krŏng Pailĭn",
    code: "KH-24",
    subdivision_type: "Autonomous municipality",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-24",
};

pub const KH_2: Subdivision = Subdivision {
    name: "Baat Dambang",
    code: "KH-2",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-2",
};

pub const KH_3: Subdivision = Subdivision {
    name: "Kampong Chaam",
    code: "KH-3",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-3",
};

pub const KH_4: Subdivision = Subdivision {
    name: "Kampong Chhnang",
    code: "KH-4",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-4",
};

pub const KH_5: Subdivision = Subdivision {
    name: "Kâmpóng Spœ",
    code: "KH-5",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-5",
};

pub const KH_6: Subdivision = Subdivision {
    name: "Kâmpóng Thum",
    code: "KH-6",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-6",
};

pub const KH_7: Subdivision = Subdivision {
    name: "Kampot",
    code: "KH-7",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-7",
};

pub const KH_8: Subdivision = Subdivision {
    name: "Kandaal",
    code: "KH-8",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-8",
};

pub const KH_9: Subdivision = Subdivision {
    name: "Kaoh Kong",
    code: "KH-9",
    subdivision_type: "Province",
    country_name: "Cambodia",
    country_code: "KH",
    region_code: "KH-9",
};

pub const CM_AD: Subdivision = Subdivision {
    name: "Adamaoua",
    code: "CM-AD",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-AD",
};

pub const CM_CE: Subdivision = Subdivision {
    name: "Centre",
    code: "CM-CE",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-CE",
};

pub const CM_EN: Subdivision = Subdivision {
    name: "Extrême-Nord",
    code: "CM-EN",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-EN",
};

pub const CM_ES: Subdivision = Subdivision {
    name: "East",
    code: "CM-ES",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-ES",
};

pub const CM_LT: Subdivision = Subdivision {
    name: "Littoral",
    code: "CM-LT",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-LT",
};

pub const CM_NO: Subdivision = Subdivision {
    name: "Nord",
    code: "CM-NO",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-NO",
};

pub const CM_NW: Subdivision = Subdivision {
    name: "Nord-Ouest",
    code: "CM-NW",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-NW",
};

pub const CM_OU: Subdivision = Subdivision {
    name: "Ouest",
    code: "CM-OU",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-OU",
};

pub const CM_SU: Subdivision = Subdivision {
    name: "South",
    code: "CM-SU",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-SU",
};

pub const CM_SW: Subdivision = Subdivision {
    name: "South-West",
    code: "CM-SW",
    subdivision_type: "Region",
    country_name: "Cameroon",
    country_code: "CM",
    region_code: "CM-SW",
};

pub const CA_AB: Subdivision = Subdivision {
    name: "Alberta",
    code: "CA-AB",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-AB",
};

pub const CA_BC: Subdivision = Subdivision {
    name: "British Columbia",
    code: "CA-BC",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-BC",
};

pub const CA_MB: Subdivision = Subdivision {
    name: "Manitoba",
    code: "CA-MB",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-MB",
};

pub const CA_NB: Subdivision = Subdivision {
    name: "New Brunswick",
    code: "CA-NB",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-NB",
};

pub const CA_NL: Subdivision = Subdivision {
    name: "Newfoundland and Labrador",
    code: "CA-NL",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-NL",
};

pub const CA_NS: Subdivision = Subdivision {
    name: "Nova Scotia",
    code: "CA-NS",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-NS",
};

pub const CA_NT: Subdivision = Subdivision {
    name: "Northwest Territories",
    code: "CA-NT",
    subdivision_type: "Territory",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-NT",
};

pub const CA_NU: Subdivision = Subdivision {
    name: "Nunavut",
    code: "CA-NU",
    subdivision_type: "Territory",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-NU",
};

pub const CA_ON: Subdivision = Subdivision {
    name: "Ontario",
    code: "CA-ON",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-ON",
};

pub const CA_PE: Subdivision = Subdivision {
    name: "Prince Edward Island",
    code: "CA-PE",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-PE",
};

pub const CA_QC: Subdivision = Subdivision {
    name: "Quebec",
    code: "CA-QC",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-QC",
};

pub const CA_SK: Subdivision = Subdivision {
    name: "Saskatchewan",
    code: "CA-SK",
    subdivision_type: "Province",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-SK",
};

pub const CA_YT: Subdivision = Subdivision {
    name: "Yukon",
    code: "CA-YT",
    subdivision_type: "Territory",
    country_name: "Canada",
    country_code: "CA",
    region_code: "CA-YT",
};

pub const CV_B: Subdivision = Subdivision {
    name: "Ilhas de Barlavento",
    code: "CV-B",
    subdivision_type: "Geographical region",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-B",
};

pub const CV_BR: Subdivision = Subdivision {
    name: "Brava",
    code: "CV-BR",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-BR",
};

pub const CV_BV: Subdivision = Subdivision {
    name: "Boa Vista",
    code: "CV-BV",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-BV",
};

pub const CV_CA: Subdivision = Subdivision {
    name: "Santa Catarina",
    code: "CV-CA",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-CA",
};

pub const CV_CF: Subdivision = Subdivision {
    name: "Santa Catarina do Fogo",
    code: "CV-CF",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-CF",
};

pub const CV_CR: Subdivision = Subdivision {
    name: "Santa Cruz",
    code: "CV-CR",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-CR",
};

pub const CV_MA: Subdivision = Subdivision {
    name: "Maio",
    code: "CV-MA",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-MA",
};

pub const CV_MO: Subdivision = Subdivision {
    name: "Mosteiros",
    code: "CV-MO",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-MO",
};

pub const CV_PA: Subdivision = Subdivision {
    name: "Paul",
    code: "CV-PA",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-PA",
};

pub const CV_PN: Subdivision = Subdivision {
    name: "Porto Novo",
    code: "CV-PN",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-PN",
};

pub const CV_PR: Subdivision = Subdivision {
    name: "Praia",
    code: "CV-PR",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-PR",
};

pub const CV_RB: Subdivision = Subdivision {
    name: "Ribeira Brava",
    code: "CV-RB",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-RB",
};

pub const CV_RG: Subdivision = Subdivision {
    name: "Ribeira Grande",
    code: "CV-RG",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-RG",
};

pub const CV_RS: Subdivision = Subdivision {
    name: "Ribeira Grande de Santiago",
    code: "CV-RS",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-RS",
};

pub const CV_SD: Subdivision = Subdivision {
    name: "São Domingos",
    code: "CV-SD",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SD",
};

pub const CV_SF: Subdivision = Subdivision {
    name: "São Filipe",
    code: "CV-SF",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SF",
};

pub const CV_S: Subdivision = Subdivision {
    name: "Ilhas de Sotavento",
    code: "CV-S",
    subdivision_type: "Geographical region",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-S",
};

pub const CV_SL: Subdivision = Subdivision {
    name: "Sal",
    code: "CV-SL",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SL",
};

pub const CV_SM: Subdivision = Subdivision {
    name: "São Miguel",
    code: "CV-SM",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SM",
};

pub const CV_SO: Subdivision = Subdivision {
    name: "São Lourenço dos Órgãos",
    code: "CV-SO",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SO",
};

pub const CV_SS: Subdivision = Subdivision {
    name: "São Salvador do Mundo",
    code: "CV-SS",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SS",
};

pub const CV_SV: Subdivision = Subdivision {
    name: "São Vicente",
    code: "CV-SV",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-SV",
};

pub const CV_TA: Subdivision = Subdivision {
    name: "Tarrafal",
    code: "CV-TA",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-TA",
};

pub const CV_TS: Subdivision = Subdivision {
    name: "Tarrafal de São Nicolau",
    code: "CV-TS",
    subdivision_type: "Municipality",
    country_name: "Cape Verde",
    country_code: "CV",
    region_code: "CV-TS",
};

pub const KY_KY: Subdivision = Subdivision {
    name: "Cayman Islands",
    code: "KY-KY",
    subdivision_type: "Country",
    country_name: "Cayman Islands",
    country_code: "KY",
    region_code: "KY-KY",
};

pub const XC_BA: Subdivision = Subdivision {
    name: "Barcelona",
    code: "XC-BA",
    subdivision_type: "Province",
    country_name: "Catalonia",
    country_code: "XC",
    region_code: "XC-BA",
};

pub const XC_GI: Subdivision = Subdivision {
    name: "Girona",
    code: "XC-GI",
    subdivision_type: "Province",
    country_name: "Catalonia",
    country_code: "XC",
    region_code: "XC-GI",
};

pub const XC_LL: Subdivision = Subdivision {
    name: "Lleida",
    code: "XC-LL",
    subdivision_type: "Province",
    country_name: "Catalonia",
    country_code: "XC",
    region_code: "XC-LL",
};

pub const XC_TA: Subdivision = Subdivision {
    name: "Tarragona",
    code: "XC-TA",
    subdivision_type: "Province",
    country_name: "Catalonia",
    country_code: "XC",
    region_code: "XC-TA",
};

pub const CF_AC: Subdivision = Subdivision {
    name: "Ouham",
    code: "CF-AC",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-AC",
};

pub const CF_BB: Subdivision = Subdivision {
    name: "Bamïngï-Bangoran",
    code: "CF-BB",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-BB",
};

pub const CF_BGF: Subdivision = Subdivision {
    name: "Bangî",
    code: "CF-BGF",
    subdivision_type: "Commune",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-BGF",
};

pub const CF_BK: Subdivision = Subdivision {
    name: "Basse-Kotto",
    code: "CF-BK",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-BK",
};

pub const CF_HK: Subdivision = Subdivision {
    name: "Haute-Kotto",
    code: "CF-HK",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-HK",
};

pub const CF_HM: Subdivision = Subdivision {
    name: "Haut-Mbomou",
    code: "CF-HM",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-HM",
};

pub const CF_HS: Subdivision = Subdivision {
    name: "Haute-Sangha / Mambéré-Kadéï",
    code: "CF-HS",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-HS",
};

pub const CF_KB: Subdivision = Subdivision {
    name: "Gïrïbïngï",
    code: "CF-KB",
    subdivision_type: "Economic prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-KB",
};

pub const CF_KG: Subdivision = Subdivision {
    name: "Kemö-Gïrïbïngï",
    code: "CF-KG",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-KG",
};

pub const CF_LB: Subdivision = Subdivision {
    name: "Lobaye",
    code: "CF-LB",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-LB",
};

pub const CF_MB: Subdivision = Subdivision {
    name: "Mbomou",
    code: "CF-MB",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-MB",
};

pub const CF_MP: Subdivision = Subdivision {
    name: "Ömbëlä-Pökö",
    code: "CF-MP",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-MP",
};

pub const CF_NM: Subdivision = Subdivision {
    name: "Nana-Mambéré",
    code: "CF-NM",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-NM",
};

pub const CF_OP: Subdivision = Subdivision {
    name: "Ouham-Pendé",
    code: "CF-OP",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-OP",
};

pub const CF_SE: Subdivision = Subdivision {
    name: "Sangä",
    code: "CF-SE",
    subdivision_type: "Economic prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-SE",
};

pub const CF_UK: Subdivision = Subdivision {
    name: "Ouaka",
    code: "CF-UK",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-UK",
};

pub const CF_VK: Subdivision = Subdivision {
    name: "Vakaga",
    code: "CF-VK",
    subdivision_type: "Prefecture",
    country_name: "Central African Republic",
    country_code: "CF",
    region_code: "CF-VK",
};

pub const TD_BA: Subdivision = Subdivision {
    name: "Al Baţḩah",
    code: "TD-BA",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-BA",
};

pub const TD_BG: Subdivision = Subdivision {
    name: "Baḩr al Ghazāl",
    code: "TD-BG",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-BG",
};

pub const TD_BO: Subdivision = Subdivision {
    name: "Borkou",
    code: "TD-BO",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-BO",
};

pub const TD_CB: Subdivision = Subdivision {
    name: "Chari-Baguirmi",
    code: "TD-CB",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-CB",
};

pub const TD_EE: Subdivision = Subdivision {
    name: "Ennedi-Est",
    code: "TD-EE",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-EE",
};

pub const TD_EO: Subdivision = Subdivision {
    name: "Ennedi-Ouest",
    code: "TD-EO",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-EO",
};

pub const TD_GR: Subdivision = Subdivision {
    name: "Guéra",
    code: "TD-GR",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-GR",
};

pub const TD_HL: Subdivision = Subdivision {
    name: "Hadjer Lamis",
    code: "TD-HL",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-HL",
};

pub const TD_KA: Subdivision = Subdivision {
    name: "Kanem",
    code: "TD-KA",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-KA",
};

pub const TD_LC: Subdivision = Subdivision {
    name: "Al Buḩayrah",
    code: "TD-LC",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-LC",
};

pub const TD_LO: Subdivision = Subdivision {
    name: "Logone-Occidental",
    code: "TD-LO",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-LO",
};

pub const TD_LR: Subdivision = Subdivision {
    name: "Logone-Oriental",
    code: "TD-LR",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-LR",
};

pub const TD_MA: Subdivision = Subdivision {
    name: "Mandoul",
    code: "TD-MA",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-MA",
};

pub const TD_MC: Subdivision = Subdivision {
    name: "Moyen-Chari",
    code: "TD-MC",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-MC",
};

pub const TD_ME: Subdivision = Subdivision {
    name: "Mayo-Kebbi-Est",
    code: "TD-ME",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-ME",
};

pub const TD_MO: Subdivision = Subdivision {
    name: "Mayo-Kebbi-Ouest",
    code: "TD-MO",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-MO",
};

pub const TD_ND: Subdivision = Subdivision {
    name: "Madīnat Injamīnā",
    code: "TD-ND",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-ND",
};

pub const TD_OD: Subdivision = Subdivision {
    name: "Ouaddaï",
    code: "TD-OD",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-OD",
};

pub const TD_SA: Subdivision = Subdivision {
    name: "Salamat",
    code: "TD-SA",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-SA",
};

pub const TD_SI: Subdivision = Subdivision {
    name: "Sila",
    code: "TD-SI",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-SI",
};

pub const TD_TA: Subdivision = Subdivision {
    name: "Tandjilé",
    code: "TD-TA",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-TA",
};

pub const TD_TI: Subdivision = Subdivision {
    name: "Tibastī",
    code: "TD-TI",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-TI",
};

pub const TD_WF: Subdivision = Subdivision {
    name: "Wadi Fira",
    code: "TD-WF",
    subdivision_type: "Region",
    country_name: "Chad",
    country_code: "TD",
    region_code: "TD-WF",
};

pub const CL_AI: Subdivision = Subdivision {
    name: "Aysén",
    code: "CL-AI",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-AI",
};

pub const CL_AN: Subdivision = Subdivision {
    name: "Antofagasta",
    code: "CL-AN",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-AN",
};

pub const CL_AP: Subdivision = Subdivision {
    name: "Arica y Parinacota",
    code: "CL-AP",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-AP",
};

pub const CL_AR: Subdivision = Subdivision {
    name: "Araucanía",
    code: "CL-AR",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-AR",
};

pub const CL_AT: Subdivision = Subdivision {
    name: "Atacama",
    code: "CL-AT",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-AT",
};

pub const CL_BI: Subdivision = Subdivision {
    name: "Biobío",
    code: "CL-BI",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-BI",
};

pub const CL_CO: Subdivision = Subdivision {
    name: "Coquimbo",
    code: "CL-CO",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-CO",
};

pub const CL_LI: Subdivision = Subdivision {
    name: "Libertador General Bernardo O'Higgins",
    code: "CL-LI",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-LI",
};

pub const CL_LL: Subdivision = Subdivision {
    name: "Los Lagos",
    code: "CL-LL",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-LL",
};

pub const CL_LR: Subdivision = Subdivision {
    name: "Los Ríos",
    code: "CL-LR",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-LR",
};

pub const CL_MA: Subdivision = Subdivision {
    name: "Magallanes",
    code: "CL-MA",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-MA",
};

pub const CL_ML: Subdivision = Subdivision {
    name: "Maule",
    code: "CL-ML",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-ML",
};

pub const CL_RM: Subdivision = Subdivision {
    name: "Región Metropolitana de Santiago",
    code: "CL-RM",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-RM",
};

pub const CL_TA: Subdivision = Subdivision {
    name: "Tarapacá",
    code: "CL-TA",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-TA",
};

pub const CL_VS: Subdivision = Subdivision {
    name: "Valparaíso",
    code: "CL-VS",
    subdivision_type: "Region",
    country_name: "Chile",
    country_code: "CL",
    region_code: "CL-VS",
};

pub const CN_11: Subdivision = Subdivision {
    name: "Beijing",
    code: "CN-11",
    subdivision_type: "Municipality",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-11",
};

pub const CN_12: Subdivision = Subdivision {
    name: "Tianjin",
    code: "CN-12",
    subdivision_type: "Municipality",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-12",
};

pub const CN_13: Subdivision = Subdivision {
    name: "Hebei",
    code: "CN-13",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-13",
};

pub const CN_14: Subdivision = Subdivision {
    name: "Shanxi",
    code: "CN-14",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-14",
};

pub const CN_15: Subdivision = Subdivision {
    name: "Nei Mongol",
    code: "CN-15",
    subdivision_type: "Autonomous region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-15",
};

pub const CN_21: Subdivision = Subdivision {
    name: "Liaoning",
    code: "CN-21",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-21",
};

pub const CN_22: Subdivision = Subdivision {
    name: "Jilin",
    code: "CN-22",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-22",
};

pub const CN_23: Subdivision = Subdivision {
    name: "Heilongjiang",
    code: "CN-23",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-23",
};

pub const CN_31: Subdivision = Subdivision {
    name: "Shanghai",
    code: "CN-31",
    subdivision_type: "Municipality",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-31",
};

pub const CN_32: Subdivision = Subdivision {
    name: "Jiangsu",
    code: "CN-32",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-32",
};

pub const CN_33: Subdivision = Subdivision {
    name: "Zhejiang",
    code: "CN-33",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-33",
};

pub const CN_34: Subdivision = Subdivision {
    name: "Anhui",
    code: "CN-34",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-34",
};

pub const CN_35: Subdivision = Subdivision {
    name: "Fujian",
    code: "CN-35",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-35",
};

pub const CN_36: Subdivision = Subdivision {
    name: "Jiangxi",
    code: "CN-36",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-36",
};

pub const CN_37: Subdivision = Subdivision {
    name: "Shandong",
    code: "CN-37",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-37",
};

pub const CN_41: Subdivision = Subdivision {
    name: "Henan",
    code: "CN-41",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-41",
};

pub const CN_42: Subdivision = Subdivision {
    name: "Hubei",
    code: "CN-42",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-42",
};

pub const CN_43: Subdivision = Subdivision {
    name: "Hunan",
    code: "CN-43",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-43",
};

pub const CN_44: Subdivision = Subdivision {
    name: "Guangdong",
    code: "CN-44",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-44",
};

pub const CN_45: Subdivision = Subdivision {
    name: "Guangxi",
    code: "CN-45",
    subdivision_type: "Autonomous region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-45",
};

pub const CN_46: Subdivision = Subdivision {
    name: "Hainan",
    code: "CN-46",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-46",
};

pub const CN_50: Subdivision = Subdivision {
    name: "Chongqing",
    code: "CN-50",
    subdivision_type: "Municipality",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-50",
};

pub const CN_51: Subdivision = Subdivision {
    name: "Sichuan",
    code: "CN-51",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-51",
};

pub const CN_52: Subdivision = Subdivision {
    name: "Guizhou",
    code: "CN-52",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-52",
};

pub const CN_53: Subdivision = Subdivision {
    name: "Yunnan",
    code: "CN-53",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-53",
};

pub const CN_54: Subdivision = Subdivision {
    name: "Xizang",
    code: "CN-54",
    subdivision_type: "Autonomous region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-54",
};

pub const CN_61: Subdivision = Subdivision {
    name: "Shaanxi",
    code: "CN-61",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-61",
};

pub const CN_62: Subdivision = Subdivision {
    name: "Gansu",
    code: "CN-62",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-62",
};

pub const CN_63: Subdivision = Subdivision {
    name: "Qinghai",
    code: "CN-63",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-63",
};

pub const CN_64: Subdivision = Subdivision {
    name: "Ningxia",
    code: "CN-64",
    subdivision_type: "Autonomous region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-64",
};

pub const CN_65: Subdivision = Subdivision {
    name: "Xinjiang",
    code: "CN-65",
    subdivision_type: "Autonomous region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-65",
};

pub const CN_71: Subdivision = Subdivision {
    name: "Taiwan",
    code: "CN-71",
    subdivision_type: "Province",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-71",
};

pub const CN_91: Subdivision = Subdivision {
    name: "Hong Kong",
    code: "CN-91",
    subdivision_type: "Special administrative region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-91",
};

pub const CN_92: Subdivision = Subdivision {
    name: "Aomen",
    code: "CN-92",
    subdivision_type: "Special administrative region",
    country_name: "China",
    country_code: "CN",
    region_code: "CN-92",
};

pub const CX_CX: Subdivision = Subdivision {
    name: "Christmas Island",
    code: "CX-CX",
    subdivision_type: "Country",
    country_name: "Christmas Island",
    country_code: "CX",
    region_code: "CX-CX",
};

pub const CC_CC: Subdivision = Subdivision {
    name: "Cocos  Islands",
    code: "CC-CC",
    subdivision_type: "Country",
    country_name: "Cocos  Islands",
    country_code: "CC",
    region_code: "CC-CC",
};

pub const CO_AMA: Subdivision = Subdivision {
    name: "Amazonas",
    code: "CO-AMA",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-AMA",
};

pub const CO_ANT: Subdivision = Subdivision {
    name: "Antioquia",
    code: "CO-ANT",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-ANT",
};

pub const CO_ARA: Subdivision = Subdivision {
    name: "Arauca",
    code: "CO-ARA",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-ARA",
};

pub const CO_ATL: Subdivision = Subdivision {
    name: "Atlántico",
    code: "CO-ATL",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-ATL",
};

pub const CO_BOL: Subdivision = Subdivision {
    name: "Bolívar",
    code: "CO-BOL",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-BOL",
};

pub const CO_BOY: Subdivision = Subdivision {
    name: "Boyacá",
    code: "CO-BOY",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-BOY",
};

pub const CO_CAL: Subdivision = Subdivision {
    name: "Caldas",
    code: "CO-CAL",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CAL",
};

pub const CO_CAQ: Subdivision = Subdivision {
    name: "Caquetá",
    code: "CO-CAQ",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CAQ",
};

pub const CO_CAS: Subdivision = Subdivision {
    name: "Casanare",
    code: "CO-CAS",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CAS",
};

pub const CO_CAU: Subdivision = Subdivision {
    name: "Cauca",
    code: "CO-CAU",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CAU",
};

pub const CO_CES: Subdivision = Subdivision {
    name: "Cesar",
    code: "CO-CES",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CES",
};

pub const CO_CHO: Subdivision = Subdivision {
    name: "Chocó",
    code: "CO-CHO",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CHO",
};

pub const CO_COR: Subdivision = Subdivision {
    name: "Córdoba",
    code: "CO-COR",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-COR",
};

pub const CO_CUN: Subdivision = Subdivision {
    name: "Cundinamarca",
    code: "CO-CUN",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-CUN",
};

pub const CO_DC: Subdivision = Subdivision {
    name: "Distrito Capital de Bogotá",
    code: "CO-DC",
    subdivision_type: "Capital district",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-DC",
};

pub const CO_GUA: Subdivision = Subdivision {
    name: "Guainía",
    code: "CO-GUA",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-GUA",
};

pub const CO_GUV: Subdivision = Subdivision {
    name: "Guaviare",
    code: "CO-GUV",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-GUV",
};

pub const CO_HUI: Subdivision = Subdivision {
    name: "Huila",
    code: "CO-HUI",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-HUI",
};

pub const CO_LAG: Subdivision = Subdivision {
    name: "La Guajira",
    code: "CO-LAG",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-LAG",
};

pub const CO_MAG: Subdivision = Subdivision {
    name: "Magdalena",
    code: "CO-MAG",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-MAG",
};

pub const CO_MET: Subdivision = Subdivision {
    name: "Meta",
    code: "CO-MET",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-MET",
};

pub const CO_NAR: Subdivision = Subdivision {
    name: "Nariño",
    code: "CO-NAR",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-NAR",
};

pub const CO_NSA: Subdivision = Subdivision {
    name: "Norte de Santander",
    code: "CO-NSA",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-NSA",
};

pub const CO_PUT: Subdivision = Subdivision {
    name: "Putumayo",
    code: "CO-PUT",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-PUT",
};

pub const CO_QUI: Subdivision = Subdivision {
    name: "Quindío",
    code: "CO-QUI",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-QUI",
};

pub const CO_RIS: Subdivision = Subdivision {
    name: "Risaralda",
    code: "CO-RIS",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-RIS",
};

pub const CO_SAN: Subdivision = Subdivision {
    name: "Santander",
    code: "CO-SAN",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-SAN",
};

pub const CO_SAP: Subdivision = Subdivision {
    name: "San Andrés, Providencia y Santa Catalina",
    code: "CO-SAP",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-SAP",
};

pub const CO_SUC: Subdivision = Subdivision {
    name: "Sucre",
    code: "CO-SUC",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-SUC",
};

pub const CO_TOL: Subdivision = Subdivision {
    name: "Tolima",
    code: "CO-TOL",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-TOL",
};

pub const CO_VAC: Subdivision = Subdivision {
    name: "Valle del Cauca",
    code: "CO-VAC",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-VAC",
};

pub const CO_VAU: Subdivision = Subdivision {
    name: "Vaupés",
    code: "CO-VAU",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-VAU",
};

pub const CO_VID: Subdivision = Subdivision {
    name: "Vichada",
    code: "CO-VID",
    subdivision_type: "Department",
    country_name: "Colombia",
    country_code: "CO",
    region_code: "CO-VID",
};

pub const KM_A: Subdivision = Subdivision {
    name: "Andjouân",
    code: "KM-A",
    subdivision_type: "Island",
    country_name: "Comoros",
    country_code: "KM",
    region_code: "KM-A",
};

pub const KM_G: Subdivision = Subdivision {
    name: "Andjazîdja",
    code: "KM-G",
    subdivision_type: "Island",
    country_name: "Comoros",
    country_code: "KM",
    region_code: "KM-G",
};

pub const KM_M: Subdivision = Subdivision {
    name: "Mohéli",
    code: "KM-M",
    subdivision_type: "Island",
    country_name: "Comoros",
    country_code: "KM",
    region_code: "KM-M",
};

pub const CG_11: Subdivision = Subdivision {
    name: "Bouenza",
    code: "CG-11",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-11",
};

pub const CG_12: Subdivision = Subdivision {
    name: "Pool",
    code: "CG-12",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-12",
};

pub const CG_13: Subdivision = Subdivision {
    name: "Sangha",
    code: "CG-13",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-13",
};

pub const CG_14: Subdivision = Subdivision {
    name: "Plateaux",
    code: "CG-14",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-14",
};

pub const CG_15: Subdivision = Subdivision {
    name: "Cuvette-Ouest",
    code: "CG-15",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-15",
};

pub const CG_16: Subdivision = Subdivision {
    name: "Pointe-Noire",
    code: "CG-16",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-16",
};

pub const CG_2: Subdivision = Subdivision {
    name: "Lékoumou",
    code: "CG-2",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-2",
};

pub const CG_5: Subdivision = Subdivision {
    name: "Kouilou",
    code: "CG-5",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-5",
};

pub const CG_7: Subdivision = Subdivision {
    name: "Likouala",
    code: "CG-7",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-7",
};

pub const CG_8: Subdivision = Subdivision {
    name: "Cuvette",
    code: "CG-8",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-8",
};

pub const CG_9: Subdivision = Subdivision {
    name: "Niari",
    code: "CG-9",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-9",
};

pub const CG_BZV: Subdivision = Subdivision {
    name: "Brazzaville",
    code: "CG-BZV",
    subdivision_type: "Department",
    country_name: "Congo",
    country_code: "CG",
    region_code: "CG-BZV",
};

pub const CD_BC: Subdivision = Subdivision {
    name: "Bas-Congo",
    code: "CD-BC",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-BC",
};

pub const CD_BN: Subdivision = Subdivision {
    name: "Bandundu",
    code: "CD-BN",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-BN",
};

pub const CD_EQ: Subdivision = Subdivision {
    name: "Équateur",
    code: "CD-EQ",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-EQ",
};

pub const CD_KA: Subdivision = Subdivision {
    name: "Katanga",
    code: "CD-KA",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-KA",
};

pub const CD_KE: Subdivision = Subdivision {
    name: "Kasai-Oriental",
    code: "CD-KE",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-KE",
};

pub const CD_KN: Subdivision = Subdivision {
    name: "Kinshasa",
    code: "CD-KN",
    subdivision_type: "City",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-KN",
};

pub const CD_KW: Subdivision = Subdivision {
    name: "Kasai-Occidental",
    code: "CD-KW",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-KW",
};

pub const CD_MA: Subdivision = Subdivision {
    name: "Maniema",
    code: "CD-MA",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-MA",
};

pub const CD_NK: Subdivision = Subdivision {
    name: "Nord-Kivu",
    code: "CD-NK",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-NK",
};

pub const CD_OR: Subdivision = Subdivision {
    name: "Orientale",
    code: "CD-OR",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-OR",
};

pub const CD_SK: Subdivision = Subdivision {
    name: "Sud-Kivu",
    code: "CD-SK",
    subdivision_type: "Province",
    country_name: "Congo, The Democratic Republic Of The",
    country_code: "CD",
    region_code: "CD-SK",
};

pub const CK_CK: Subdivision = Subdivision {
    name: "Cook Islands",
    code: "CK-CK",
    subdivision_type: "Country",
    country_name: "Cook Islands",
    country_code: "CK",
    region_code: "CK-CK",
};

pub const CR_A: Subdivision = Subdivision {
    name: "Alajuela",
    code: "CR-A",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-A",
};

pub const CR_C: Subdivision = Subdivision {
    name: "Cartago",
    code: "CR-C",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-C",
};

pub const CR_G: Subdivision = Subdivision {
    name: "Guanacaste",
    code: "CR-G",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-G",
};

pub const CR_H: Subdivision = Subdivision {
    name: "Heredia",
    code: "CR-H",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-H",
};

pub const CR_L: Subdivision = Subdivision {
    name: "Limón",
    code: "CR-L",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-L",
};

pub const CR_P: Subdivision = Subdivision {
    name: "Puntarenas",
    code: "CR-P",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-P",
};

pub const CR_SJ: Subdivision = Subdivision {
    name: "San José",
    code: "CR-SJ",
    subdivision_type: "Province",
    country_name: "Costa Rica",
    country_code: "CR",
    region_code: "CR-SJ",
};

pub const CI_01: Subdivision = Subdivision {
    name: "Lagunes",
    code: "CI-01",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-01",
};

pub const CI_02: Subdivision = Subdivision {
    name: "Haut-Sassandra",
    code: "CI-02",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-02",
};

pub const CI_03: Subdivision = Subdivision {
    name: "Savanes",
    code: "CI-03",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-03",
};

pub const CI_04: Subdivision = Subdivision {
    name: "Vallée du Bandama",
    code: "CI-04",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-04",
};

pub const CI_05: Subdivision = Subdivision {
    name: "Moyen-Comoé",
    code: "CI-05",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-05",
};

pub const CI_06: Subdivision = Subdivision {
    name: "18 Montagnes",
    code: "CI-06",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-06",
};

pub const CI_07: Subdivision = Subdivision {
    name: "Lacs",
    code: "CI-07",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-07",
};

pub const CI_08: Subdivision = Subdivision {
    name: "Zanzan",
    code: "CI-08",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-08",
};

pub const CI_09: Subdivision = Subdivision {
    name: "Bas-Sassandra",
    code: "CI-09",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-09",
};

pub const CI_10: Subdivision = Subdivision {
    name: "Denguélé",
    code: "CI-10",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-10",
};

pub const CI_11: Subdivision = Subdivision {
    name: "Nzi-Comoé",
    code: "CI-11",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-11",
};

pub const CI_12: Subdivision = Subdivision {
    name: "Marahoué",
    code: "CI-12",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-12",
};

pub const CI_13: Subdivision = Subdivision {
    name: "Sud-Comoé",
    code: "CI-13",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-13",
};

pub const CI_14: Subdivision = Subdivision {
    name: "Worodougou",
    code: "CI-14",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-14",
};

pub const CI_15: Subdivision = Subdivision {
    name: "Sud-Bandama",
    code: "CI-15",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-15",
};

pub const CI_16: Subdivision = Subdivision {
    name: "Agnébi",
    code: "CI-16",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-16",
};

pub const CI_17: Subdivision = Subdivision {
    name: "Bafing",
    code: "CI-17",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-17",
};

pub const CI_18: Subdivision = Subdivision {
    name: "Fromager",
    code: "CI-18",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-18",
};

pub const CI_19: Subdivision = Subdivision {
    name: "Moyen-Cavally",
    code: "CI-19",
    subdivision_type: "Region",
    country_name: "Côte D'Ivoire",
    country_code: "CI",
    region_code: "CI-19",
};

pub const HR_01: Subdivision = Subdivision {
    name: "Zagrebačka županija",
    code: "HR-01",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-01",
};

pub const HR_02: Subdivision = Subdivision {
    name: "Krapinsko-zagorska županija",
    code: "HR-02",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-02",
};

pub const HR_03: Subdivision = Subdivision {
    name: "Sisačko-moslavačka županija",
    code: "HR-03",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-03",
};

pub const HR_04: Subdivision = Subdivision {
    name: "Karlovačka županija",
    code: "HR-04",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-04",
};

pub const HR_05: Subdivision = Subdivision {
    name: "Varaždinska županija",
    code: "HR-05",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-05",
};

pub const HR_06: Subdivision = Subdivision {
    name: "Koprivničko-križevačka županija",
    code: "HR-06",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-06",
};

pub const HR_07: Subdivision = Subdivision {
    name: "Bjelovarsko-bilogorska županija",
    code: "HR-07",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-07",
};

pub const HR_08: Subdivision = Subdivision {
    name: "Primorsko-goranska županija",
    code: "HR-08",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-08",
};

pub const HR_09: Subdivision = Subdivision {
    name: "Ličko-senjska županija",
    code: "HR-09",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-09",
};

pub const HR_10: Subdivision = Subdivision {
    name: "Virovitičko-podravska županija",
    code: "HR-10",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-10",
};

pub const HR_11: Subdivision = Subdivision {
    name: "Požeško-slavonska županija",
    code: "HR-11",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-11",
};

pub const HR_12: Subdivision = Subdivision {
    name: "Brodsko-posavska županija",
    code: "HR-12",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-12",
};

pub const HR_13: Subdivision = Subdivision {
    name: "Zadarska županija",
    code: "HR-13",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-13",
};

pub const HR_14: Subdivision = Subdivision {
    name: "Osječko-baranjska županija",
    code: "HR-14",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-14",
};

pub const HR_15: Subdivision = Subdivision {
    name: "Šibensko-kninska županija",
    code: "HR-15",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-15",
};

pub const HR_16: Subdivision = Subdivision {
    name: "Vukovarsko-srijemska županija",
    code: "HR-16",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-16",
};

pub const HR_17: Subdivision = Subdivision {
    name: "Splitsko-dalmatinska županija",
    code: "HR-17",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-17",
};

pub const HR_18: Subdivision = Subdivision {
    name: "Istarska županija",
    code: "HR-18",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-18",
};

pub const HR_19: Subdivision = Subdivision {
    name: "Dubrovačko-neretvanska županija",
    code: "HR-19",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-19",
};

pub const HR_20: Subdivision = Subdivision {
    name: "Međimurska županija",
    code: "HR-20",
    subdivision_type: "County",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-20",
};

pub const HR_21: Subdivision = Subdivision {
    name: "Grad Zagreb",
    code: "HR-21",
    subdivision_type: "City",
    country_name: "Croatia",
    country_code: "HR",
    region_code: "HR-21",
};

pub const CU_01: Subdivision = Subdivision {
    name: "Pinar del Río",
    code: "CU-01",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-01",
};

pub const CU_03: Subdivision = Subdivision {
    name: "La Habana",
    code: "CU-03",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-03",
};

pub const CU_04: Subdivision = Subdivision {
    name: "Matanzas",
    code: "CU-04",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-04",
};

pub const CU_05: Subdivision = Subdivision {
    name: "Villa Clara",
    code: "CU-05",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-05",
};

pub const CU_06: Subdivision = Subdivision {
    name: "Cienfuegos",
    code: "CU-06",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-06",
};

pub const CU_07: Subdivision = Subdivision {
    name: "Sancti Spíritus",
    code: "CU-07",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-07",
};

pub const CU_08: Subdivision = Subdivision {
    name: "Ciego de Ávila",
    code: "CU-08",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-08",
};

pub const CU_09: Subdivision = Subdivision {
    name: "Camagüey",
    code: "CU-09",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-09",
};

pub const CU_10: Subdivision = Subdivision {
    name: "Las Tunas",
    code: "CU-10",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-10",
};

pub const CU_11: Subdivision = Subdivision {
    name: "Holguín",
    code: "CU-11",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-11",
};

pub const CU_12: Subdivision = Subdivision {
    name: "Granma",
    code: "CU-12",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-12",
};

pub const CU_13: Subdivision = Subdivision {
    name: "Santiago de Cuba",
    code: "CU-13",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-13",
};

pub const CU_14: Subdivision = Subdivision {
    name: "Guantánamo",
    code: "CU-14",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-14",
};

pub const CU_15: Subdivision = Subdivision {
    name: "Artemisa",
    code: "CU-15",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-15",
};

pub const CU_16: Subdivision = Subdivision {
    name: "Mayabeque",
    code: "CU-16",
    subdivision_type: "Province",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-16",
};

pub const CU_99: Subdivision = Subdivision {
    name: "Isla de la Juventud",
    code: "CU-99",
    subdivision_type: "Special municipality",
    country_name: "Cuba",
    country_code: "CU",
    region_code: "CU-99",
};

pub const CW_CW: Subdivision = Subdivision {
    name: "Curaçao",
    code: "CW-CW",
    subdivision_type: "Country",
    country_name: "Curaçao",
    country_code: "CW",
    region_code: "CW-CW",
};

pub const CY_01: Subdivision = Subdivision {
    name: "Lefkoşa",
    code: "CY-01",
    subdivision_type: "District",
    country_name: "Cyprus",
    country_code: "CY",
    region_code: "CY-01",
};

pub const CY_02: Subdivision = Subdivision {
    name: "Lemesos",
    code: "CY-02",
    subdivision_type: "District",
    country_name: "Cyprus",
    country_code: "CY",
    region_code: "CY-02",
};

pub const CY_03: Subdivision = Subdivision {
    name: "Larnaka",
    code: "CY-03",
    subdivision_type: "District",
    country_name: "Cyprus",
    country_code: "CY",
    region_code: "CY-03",
};

pub const CY_04: Subdivision = Subdivision {
    name: "Ammochostos",
    code: "CY-04",
    subdivision_type: "District",
    country_name: "Cyprus",
    country_code: "CY",
    region_code: "CY-04",
};

pub const CY_05: Subdivision = Subdivision {
    name: "Baf",
    code: "CY-05",
    subdivision_type: "District",
    country_name: "Cyprus",
    country_code: "CY",
    region_code: "CY-05",
};

pub const CY_06: Subdivision = Subdivision {
    name: "Girne",
    code: "CY-06",
    subdivision_type: "District",
    country_name: "Cyprus",
    country_code: "CY",
    region_code: "CY-06",
};

pub const CZ_101: Subdivision = Subdivision {
    name: "Praha 1",
    code: "CZ-101",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-101",
};

pub const CZ_102: Subdivision = Subdivision {
    name: "Praha 2",
    code: "CZ-102",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-102",
};

pub const CZ_103: Subdivision = Subdivision {
    name: "Praha 3",
    code: "CZ-103",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-103",
};

pub const CZ_104: Subdivision = Subdivision {
    name: "Praha 4",
    code: "CZ-104",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-104",
};

pub const CZ_105: Subdivision = Subdivision {
    name: "Praha 5",
    code: "CZ-105",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-105",
};

pub const CZ_106: Subdivision = Subdivision {
    name: "Praha 6",
    code: "CZ-106",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-106",
};

pub const CZ_107: Subdivision = Subdivision {
    name: "Praha 7",
    code: "CZ-107",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-107",
};

pub const CZ_108: Subdivision = Subdivision {
    name: "Praha 8",
    code: "CZ-108",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-108",
};

pub const CZ_109: Subdivision = Subdivision {
    name: "Praha 9",
    code: "CZ-109",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-109",
};

pub const CZ_10A: Subdivision = Subdivision {
    name: "Praha 10",
    code: "CZ-10A",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-10A",
};

pub const CZ_10B: Subdivision = Subdivision {
    name: "Praha 11",
    code: "CZ-10B",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-10B",
};

pub const CZ_10C: Subdivision = Subdivision {
    name: "Praha 12",
    code: "CZ-10C",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-10C",
};

pub const CZ_10D: Subdivision = Subdivision {
    name: "Praha 13",
    code: "CZ-10D",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-10D",
};

pub const CZ_10E: Subdivision = Subdivision {
    name: "Praha 14",
    code: "CZ-10E",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-10E",
};

pub const CZ_10F: Subdivision = Subdivision {
    name: "Praha 15",
    code: "CZ-10F",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-10F",
};

pub const CZ_201: Subdivision = Subdivision {
    name: "Benešov",
    code: "CZ-201",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-201",
};

pub const CZ_202: Subdivision = Subdivision {
    name: "Beroun",
    code: "CZ-202",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-202",
};

pub const CZ_203: Subdivision = Subdivision {
    name: "Kladno",
    code: "CZ-203",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-203",
};

pub const CZ_204: Subdivision = Subdivision {
    name: "Kolín",
    code: "CZ-204",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-204",
};

pub const CZ_205: Subdivision = Subdivision {
    name: "Kutná Hora",
    code: "CZ-205",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-205",
};

pub const CZ_206: Subdivision = Subdivision {
    name: "Mělník",
    code: "CZ-206",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-206",
};

pub const CZ_207: Subdivision = Subdivision {
    name: "Mladá Boleslav",
    code: "CZ-207",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-207",
};

pub const CZ_208: Subdivision = Subdivision {
    name: "Nymburk",
    code: "CZ-208",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-208",
};

pub const CZ_209: Subdivision = Subdivision {
    name: "Praha-východ",
    code: "CZ-209",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-209",
};

pub const CZ_20A: Subdivision = Subdivision {
    name: "Praha-západ",
    code: "CZ-20A",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-20A",
};

pub const CZ_20B: Subdivision = Subdivision {
    name: "Příbram",
    code: "CZ-20B",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-20B",
};

pub const CZ_20C: Subdivision = Subdivision {
    name: "Rakovník",
    code: "CZ-20C",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-20C",
};

pub const CZ_311: Subdivision = Subdivision {
    name: "České Budějovice",
    code: "CZ-311",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-311",
};

pub const CZ_312: Subdivision = Subdivision {
    name: "Český Krumlov",
    code: "CZ-312",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-312",
};

pub const CZ_313: Subdivision = Subdivision {
    name: "Jindřichův Hradec",
    code: "CZ-313",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-313",
};

pub const CZ_314: Subdivision = Subdivision {
    name: "Písek",
    code: "CZ-314",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-314",
};

pub const CZ_315: Subdivision = Subdivision {
    name: "Prachatice",
    code: "CZ-315",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-315",
};

pub const CZ_316: Subdivision = Subdivision {
    name: "Strakonice",
    code: "CZ-316",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-316",
};

pub const CZ_317: Subdivision = Subdivision {
    name: "Tábor",
    code: "CZ-317",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-317",
};

pub const CZ_321: Subdivision = Subdivision {
    name: "Domažlice",
    code: "CZ-321",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-321",
};

pub const CZ_322: Subdivision = Subdivision {
    name: "Klatovy",
    code: "CZ-322",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-322",
};

pub const CZ_323: Subdivision = Subdivision {
    name: "Plzeň-město",
    code: "CZ-323",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-323",
};

pub const CZ_324: Subdivision = Subdivision {
    name: "Plzeň-jih",
    code: "CZ-324",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-324",
};

pub const CZ_325: Subdivision = Subdivision {
    name: "Plzeň-sever",
    code: "CZ-325",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-325",
};

pub const CZ_326: Subdivision = Subdivision {
    name: "Rokycany",
    code: "CZ-326",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-326",
};

pub const CZ_327: Subdivision = Subdivision {
    name: "Tachov",
    code: "CZ-327",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-327",
};

pub const CZ_411: Subdivision = Subdivision {
    name: "Cheb",
    code: "CZ-411",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-411",
};

pub const CZ_412: Subdivision = Subdivision {
    name: "Karlovy Vary",
    code: "CZ-412",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-412",
};

pub const CZ_413: Subdivision = Subdivision {
    name: "Sokolov",
    code: "CZ-413",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-413",
};

pub const CZ_421: Subdivision = Subdivision {
    name: "Děčín",
    code: "CZ-421",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-421",
};

pub const CZ_422: Subdivision = Subdivision {
    name: "Chomutov",
    code: "CZ-422",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-422",
};

pub const CZ_423: Subdivision = Subdivision {
    name: "Litoměřice",
    code: "CZ-423",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-423",
};

pub const CZ_424: Subdivision = Subdivision {
    name: "Louny",
    code: "CZ-424",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-424",
};

pub const CZ_425: Subdivision = Subdivision {
    name: "Most",
    code: "CZ-425",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-425",
};

pub const CZ_426: Subdivision = Subdivision {
    name: "Teplice",
    code: "CZ-426",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-426",
};

pub const CZ_427: Subdivision = Subdivision {
    name: "Ústí nad Labem",
    code: "CZ-427",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-427",
};

pub const CZ_511: Subdivision = Subdivision {
    name: "Česká Lípa",
    code: "CZ-511",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-511",
};

pub const CZ_512: Subdivision = Subdivision {
    name: "Jablonec nad Nisou",
    code: "CZ-512",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-512",
};

pub const CZ_513: Subdivision = Subdivision {
    name: "Liberec",
    code: "CZ-513",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-513",
};

pub const CZ_514: Subdivision = Subdivision {
    name: "Semily",
    code: "CZ-514",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-514",
};

pub const CZ_521: Subdivision = Subdivision {
    name: "Hradec Králové",
    code: "CZ-521",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-521",
};

pub const CZ_522: Subdivision = Subdivision {
    name: "Jičín",
    code: "CZ-522",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-522",
};

pub const CZ_523: Subdivision = Subdivision {
    name: "Náchod",
    code: "CZ-523",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-523",
};

pub const CZ_524: Subdivision = Subdivision {
    name: "Rychnov nad Kněžnou",
    code: "CZ-524",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-524",
};

pub const CZ_525: Subdivision = Subdivision {
    name: "Trutnov",
    code: "CZ-525",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-525",
};

pub const CZ_531: Subdivision = Subdivision {
    name: "Chrudim",
    code: "CZ-531",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-531",
};

pub const CZ_532: Subdivision = Subdivision {
    name: "Pardubice",
    code: "CZ-532",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-532",
};

pub const CZ_533: Subdivision = Subdivision {
    name: "Svitavy",
    code: "CZ-533",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-533",
};

pub const CZ_534: Subdivision = Subdivision {
    name: "Ústí nad Orlicí",
    code: "CZ-534",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-534",
};

pub const CZ_611: Subdivision = Subdivision {
    name: "Havlíčkův Brod",
    code: "CZ-611",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-611",
};

pub const CZ_612: Subdivision = Subdivision {
    name: "Jihlava",
    code: "CZ-612",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-612",
};

pub const CZ_613: Subdivision = Subdivision {
    name: "Pelhřimov",
    code: "CZ-613",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-613",
};

pub const CZ_614: Subdivision = Subdivision {
    name: "Třebíč",
    code: "CZ-614",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-614",
};

pub const CZ_615: Subdivision = Subdivision {
    name: "Žd'ár nad Sázavou",
    code: "CZ-615",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-615",
};

pub const CZ_621: Subdivision = Subdivision {
    name: "Blansko",
    code: "CZ-621",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-621",
};

pub const CZ_622: Subdivision = Subdivision {
    name: "Brno-město",
    code: "CZ-622",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-622",
};

pub const CZ_623: Subdivision = Subdivision {
    name: "Brno-venkov",
    code: "CZ-623",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-623",
};

pub const CZ_624: Subdivision = Subdivision {
    name: "Břeclav",
    code: "CZ-624",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-624",
};

pub const CZ_625: Subdivision = Subdivision {
    name: "Hodonín",
    code: "CZ-625",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-625",
};

pub const CZ_626: Subdivision = Subdivision {
    name: "Vyškov",
    code: "CZ-626",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-626",
};

pub const CZ_627: Subdivision = Subdivision {
    name: "Znojmo",
    code: "CZ-627",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-627",
};

pub const CZ_711: Subdivision = Subdivision {
    name: "Jeseník",
    code: "CZ-711",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-711",
};

pub const CZ_712: Subdivision = Subdivision {
    name: "Olomouc",
    code: "CZ-712",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-712",
};

pub const CZ_713: Subdivision = Subdivision {
    name: "Prostĕjov",
    code: "CZ-713",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-713",
};

pub const CZ_714: Subdivision = Subdivision {
    name: "Přerov",
    code: "CZ-714",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-714",
};

pub const CZ_715: Subdivision = Subdivision {
    name: "Šumperk",
    code: "CZ-715",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-715",
};

pub const CZ_721: Subdivision = Subdivision {
    name: "Kromĕříž",
    code: "CZ-721",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-721",
};

pub const CZ_722: Subdivision = Subdivision {
    name: "Uherské Hradištĕ",
    code: "CZ-722",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-722",
};

pub const CZ_723: Subdivision = Subdivision {
    name: "Vsetín",
    code: "CZ-723",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-723",
};

pub const CZ_724: Subdivision = Subdivision {
    name: "Zlín",
    code: "CZ-724",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-724",
};

pub const CZ_801: Subdivision = Subdivision {
    name: "Bruntál",
    code: "CZ-801",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-801",
};

pub const CZ_802: Subdivision = Subdivision {
    name: "Frýdek Místek",
    code: "CZ-802",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-802",
};

pub const CZ_803: Subdivision = Subdivision {
    name: "Karviná",
    code: "CZ-803",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-803",
};

pub const CZ_804: Subdivision = Subdivision {
    name: "Nový Jičín",
    code: "CZ-804",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-804",
};

pub const CZ_805: Subdivision = Subdivision {
    name: "Opava",
    code: "CZ-805",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-805",
};

pub const CZ_806: Subdivision = Subdivision {
    name: "Ostrava město",
    code: "CZ-806",
    subdivision_type: "District",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-806",
};

pub const CZ_JC: Subdivision = Subdivision {
    name: "Jihočeský kraj",
    code: "CZ-JC",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-JC",
};

pub const CZ_JM: Subdivision = Subdivision {
    name: "Jihomoravský kraj",
    code: "CZ-JM",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-JM",
};

pub const CZ_KA: Subdivision = Subdivision {
    name: "Karlovarský kraj",
    code: "CZ-KA",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-KA",
};

pub const CZ_KR: Subdivision = Subdivision {
    name: "Královéhradecký kraj",
    code: "CZ-KR",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-KR",
};

pub const CZ_LI: Subdivision = Subdivision {
    name: "Liberecký kraj",
    code: "CZ-LI",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-LI",
};

pub const CZ_MO: Subdivision = Subdivision {
    name: "Moravskoslezský kraj",
    code: "CZ-MO",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-MO",
};

pub const CZ_OL: Subdivision = Subdivision {
    name: "Olomoucký kraj",
    code: "CZ-OL",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-OL",
};

pub const CZ_PA: Subdivision = Subdivision {
    name: "Pardubický kraj",
    code: "CZ-PA",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-PA",
};

pub const CZ_PL: Subdivision = Subdivision {
    name: "Plzeňský kraj",
    code: "CZ-PL",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-PL",
};

pub const CZ_PR: Subdivision = Subdivision {
    name: "Praha, hlavní mešto",
    code: "CZ-PR",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-PR",
};

pub const CZ_ST: Subdivision = Subdivision {
    name: "Středočeský kraj",
    code: "CZ-ST",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-ST",
};

pub const CZ_US: Subdivision = Subdivision {
    name: "Ústecký kraj",
    code: "CZ-US",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-US",
};

pub const CZ_VY: Subdivision = Subdivision {
    name: "Vysočina",
    code: "CZ-VY",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-VY",
};

pub const CZ_ZL: Subdivision = Subdivision {
    name: "Zlínský kraj",
    code: "CZ-ZL",
    subdivision_type: "Region",
    country_name: "Czech Republic",
    country_code: "CZ",
    region_code: "CZ-ZL",
};

pub const DK_81: Subdivision = Subdivision {
    name: "Nordjylland",
    code: "DK-81",
    subdivision_type: "Region",
    country_name: "Denmark",
    country_code: "DK",
    region_code: "DK-81",
};

pub const DK_82: Subdivision = Subdivision {
    name: "Midtjylland",
    code: "DK-82",
    subdivision_type: "Region",
    country_name: "Denmark",
    country_code: "DK",
    region_code: "DK-82",
};

pub const DK_83: Subdivision = Subdivision {
    name: "Syddanmark",
    code: "DK-83",
    subdivision_type: "Region",
    country_name: "Denmark",
    country_code: "DK",
    region_code: "DK-83",
};

pub const DK_84: Subdivision = Subdivision {
    name: "Hovedstaden",
    code: "DK-84",
    subdivision_type: "Region",
    country_name: "Denmark",
    country_code: "DK",
    region_code: "DK-84",
};

pub const DK_85: Subdivision = Subdivision {
    name: "Sjælland",
    code: "DK-85",
    subdivision_type: "Region",
    country_name: "Denmark",
    country_code: "DK",
    region_code: "DK-85",
};

pub const DJ_AR: Subdivision = Subdivision {
    name: "Arta",
    code: "DJ-AR",
    subdivision_type: "Region",
    country_name: "Djibouti",
    country_code: "DJ",
    region_code: "DJ-AR",
};

pub const DJ_AS: Subdivision = Subdivision {
    name: "Ali Sabieh",
    code: "DJ-AS",
    subdivision_type: "Region",
    country_name: "Djibouti",
    country_code: "DJ",
    region_code: "DJ-AS",
};

pub const DJ_DI: Subdivision = Subdivision {
    name: "Dikhīl",
    code: "DJ-DI",
    subdivision_type: "Region",
    country_name: "Djibouti",
    country_code: "DJ",
    region_code: "DJ-DI",
};

pub const DJ_DJ: Subdivision = Subdivision {
    name: "Djibouti",
    code: "DJ-DJ",
    subdivision_type: "City",
    country_name: "Djibouti",
    country_code: "DJ",
    region_code: "DJ-DJ",
};

pub const DJ_OB: Subdivision = Subdivision {
    name: "Obock",
    code: "DJ-OB",
    subdivision_type: "Region",
    country_name: "Djibouti",
    country_code: "DJ",
    region_code: "DJ-OB",
};

pub const DJ_TA: Subdivision = Subdivision {
    name: "Tadjourah",
    code: "DJ-TA",
    subdivision_type: "Region",
    country_name: "Djibouti",
    country_code: "DJ",
    region_code: "DJ-TA",
};

pub const DM_02: Subdivision = Subdivision {
    name: "Saint Andrew",
    code: "DM-02",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-02",
};

pub const DM_03: Subdivision = Subdivision {
    name: "Saint David",
    code: "DM-03",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-03",
};

pub const DM_04: Subdivision = Subdivision {
    name: "Saint George",
    code: "DM-04",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-04",
};

pub const DM_05: Subdivision = Subdivision {
    name: "Saint John",
    code: "DM-05",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-05",
};

pub const DM_06: Subdivision = Subdivision {
    name: "Saint Joseph",
    code: "DM-06",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-06",
};

pub const DM_07: Subdivision = Subdivision {
    name: "Saint Luke",
    code: "DM-07",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-07",
};

pub const DM_08: Subdivision = Subdivision {
    name: "Saint Mark",
    code: "DM-08",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-08",
};

pub const DM_09: Subdivision = Subdivision {
    name: "Saint Patrick",
    code: "DM-09",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-09",
};

pub const DM_10: Subdivision = Subdivision {
    name: "Saint Paul",
    code: "DM-10",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-10",
};

pub const DM_11: Subdivision = Subdivision {
    name: "Saint Peter",
    code: "DM-11",
    subdivision_type: "Parish",
    country_name: "Dominica",
    country_code: "DM",
    region_code: "DM-11",
};

pub const DO_01: Subdivision = Subdivision {
    name: "Distrito Nacional",
    code: "DO-01",
    subdivision_type: "District",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-01",
};

pub const DO_02: Subdivision = Subdivision {
    name: "Azua",
    code: "DO-02",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-02",
};

pub const DO_03: Subdivision = Subdivision {
    name: "Baoruco",
    code: "DO-03",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-03",
};

pub const DO_04: Subdivision = Subdivision {
    name: "Barahona",
    code: "DO-04",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-04",
};

pub const DO_05: Subdivision = Subdivision {
    name: "Dajabón",
    code: "DO-05",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-05",
};

pub const DO_06: Subdivision = Subdivision {
    name: "Duarte",
    code: "DO-06",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-06",
};

pub const DO_07: Subdivision = Subdivision {
    name: "La Estrelleta",
    code: "DO-07",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-07",
};

pub const DO_08: Subdivision = Subdivision {
    name: "El Seibo",
    code: "DO-08",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-08",
};

pub const DO_09: Subdivision = Subdivision {
    name: "Espaillat",
    code: "DO-09",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-09",
};

pub const DO_10: Subdivision = Subdivision {
    name: "Independencia",
    code: "DO-10",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-10",
};

pub const DO_11: Subdivision = Subdivision {
    name: "La Altagracia",
    code: "DO-11",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-11",
};

pub const DO_12: Subdivision = Subdivision {
    name: "La Romana",
    code: "DO-12",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-12",
};

pub const DO_13: Subdivision = Subdivision {
    name: "La Vega",
    code: "DO-13",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-13",
};

pub const DO_14: Subdivision = Subdivision {
    name: "María Trinidad Sánchez",
    code: "DO-14",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-14",
};

pub const DO_15: Subdivision = Subdivision {
    name: "Monte Cristi",
    code: "DO-15",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-15",
};

pub const DO_16: Subdivision = Subdivision {
    name: "Pedernales",
    code: "DO-16",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-16",
};

pub const DO_17: Subdivision = Subdivision {
    name: "Peravia",
    code: "DO-17",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-17",
};

pub const DO_18: Subdivision = Subdivision {
    name: "Puerto Plata",
    code: "DO-18",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-18",
};

pub const DO_19: Subdivision = Subdivision {
    name: "Hermanas Mirabal",
    code: "DO-19",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-19",
};

pub const DO_20: Subdivision = Subdivision {
    name: "Samaná",
    code: "DO-20",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-20",
};

pub const DO_21: Subdivision = Subdivision {
    name: "San Cristóbal",
    code: "DO-21",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-21",
};

pub const DO_22: Subdivision = Subdivision {
    name: "San Juan",
    code: "DO-22",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-22",
};

pub const DO_23: Subdivision = Subdivision {
    name: "San Pedro de Macorís",
    code: "DO-23",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-23",
};

pub const DO_24: Subdivision = Subdivision {
    name: "Sánchez Ramírez",
    code: "DO-24",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-24",
};

pub const DO_25: Subdivision = Subdivision {
    name: "Santiago",
    code: "DO-25",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-25",
};

pub const DO_26: Subdivision = Subdivision {
    name: "Santiago Rodríguez",
    code: "DO-26",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-26",
};

pub const DO_27: Subdivision = Subdivision {
    name: "Valverde",
    code: "DO-27",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-27",
};

pub const DO_28: Subdivision = Subdivision {
    name: "Monseñor Nouel",
    code: "DO-28",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-28",
};

pub const DO_29: Subdivision = Subdivision {
    name: "Monte Plata",
    code: "DO-29",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-29",
};

pub const DO_30: Subdivision = Subdivision {
    name: "Hato Mayor",
    code: "DO-30",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-30",
};

pub const DO_31: Subdivision = Subdivision {
    name: "San José de Ocoa",
    code: "DO-31",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-31",
};

pub const DO_32: Subdivision = Subdivision {
    name: "Santo Domingo",
    code: "DO-32",
    subdivision_type: "Province",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-32",
};

pub const DO_33: Subdivision = Subdivision {
    name: "Cibao Nordeste",
    code: "DO-33",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-33",
};

pub const DO_34: Subdivision = Subdivision {
    name: "Cibao Noroeste",
    code: "DO-34",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-34",
};

pub const DO_35: Subdivision = Subdivision {
    name: "Cibao Norte",
    code: "DO-35",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-35",
};

pub const DO_36: Subdivision = Subdivision {
    name: "Cibao Sur",
    code: "DO-36",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-36",
};

pub const DO_37: Subdivision = Subdivision {
    name: "El Valle",
    code: "DO-37",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-37",
};

pub const DO_38: Subdivision = Subdivision {
    name: "Enriquillo",
    code: "DO-38",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-38",
};

pub const DO_39: Subdivision = Subdivision {
    name: "Higuamo",
    code: "DO-39",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-39",
};

pub const DO_40: Subdivision = Subdivision {
    name: "Ozama",
    code: "DO-40",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-40",
};

pub const DO_41: Subdivision = Subdivision {
    name: "Valdesia",
    code: "DO-41",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-41",
};

pub const DO_42: Subdivision = Subdivision {
    name: "Yuma",
    code: "DO-42",
    subdivision_type: "Region",
    country_name: "Dominican Republic",
    country_code: "DO",
    region_code: "DO-42",
};

pub const EC_A: Subdivision = Subdivision {
    name: "Azuay",
    code: "EC-A",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-A",
};

pub const EC_B: Subdivision = Subdivision {
    name: "Bolívar",
    code: "EC-B",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-B",
};

pub const EC_C: Subdivision = Subdivision {
    name: "Carchi",
    code: "EC-C",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-C",
};

pub const EC_D: Subdivision = Subdivision {
    name: "Orellana",
    code: "EC-D",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-D",
};

pub const EC_E: Subdivision = Subdivision {
    name: "Esmeraldas",
    code: "EC-E",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-E",
};

pub const EC_F: Subdivision = Subdivision {
    name: "Cañar",
    code: "EC-F",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-F",
};

pub const EC_G: Subdivision = Subdivision {
    name: "Guayas",
    code: "EC-G",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-G",
};

pub const EC_H: Subdivision = Subdivision {
    name: "Chimborazo",
    code: "EC-H",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-H",
};

pub const EC_I: Subdivision = Subdivision {
    name: "Imbabura",
    code: "EC-I",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-I",
};

pub const EC_L: Subdivision = Subdivision {
    name: "Loja",
    code: "EC-L",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-L",
};

pub const EC_M: Subdivision = Subdivision {
    name: "Manabí",
    code: "EC-M",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-M",
};

pub const EC_N: Subdivision = Subdivision {
    name: "Napo",
    code: "EC-N",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-N",
};

pub const EC_O: Subdivision = Subdivision {
    name: "El Oro",
    code: "EC-O",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-O",
};

pub const EC_P: Subdivision = Subdivision {
    name: "Pichincha",
    code: "EC-P",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-P",
};

pub const EC_R: Subdivision = Subdivision {
    name: "Los Ríos",
    code: "EC-R",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-R",
};

pub const EC_SD: Subdivision = Subdivision {
    name: "Santo Domingo de los Tsáchilas",
    code: "EC-SD",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-SD",
};

pub const EC_SE: Subdivision = Subdivision {
    name: "Santa Elena",
    code: "EC-SE",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-SE",
};

pub const EC_S: Subdivision = Subdivision {
    name: "Morona-Santiago",
    code: "EC-S",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-S",
};

pub const EC_T: Subdivision = Subdivision {
    name: "Tungurahua",
    code: "EC-T",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-T",
};

pub const EC_U: Subdivision = Subdivision {
    name: "Sucumbíos",
    code: "EC-U",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-U",
};

pub const EC_W: Subdivision = Subdivision {
    name: "Galápagos",
    code: "EC-W",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-W",
};

pub const EC_X: Subdivision = Subdivision {
    name: "Cotopaxi",
    code: "EC-X",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-X",
};

pub const EC_Y: Subdivision = Subdivision {
    name: "Pastaza",
    code: "EC-Y",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-Y",
};

pub const EC_Z: Subdivision = Subdivision {
    name: "Zamora-Chinchipe",
    code: "EC-Z",
    subdivision_type: "Province",
    country_name: "Ecuador",
    country_code: "EC",
    region_code: "EC-Z",
};

pub const EG_ALX: Subdivision = Subdivision {
    name: "Al Iskandarīyah",
    code: "EG-ALX",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-ALX",
};

pub const EG_ASN: Subdivision = Subdivision {
    name: "Aswān",
    code: "EG-ASN",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-ASN",
};

pub const EG_AST: Subdivision = Subdivision {
    name: "Asyūţ",
    code: "EG-AST",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-AST",
};

pub const EG_BA: Subdivision = Subdivision {
    name: "Al Baḩr al Aḩmar",
    code: "EG-BA",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-BA",
};

pub const EG_BH: Subdivision = Subdivision {
    name: "Al Buḩayrah",
    code: "EG-BH",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-BH",
};

pub const EG_BNS: Subdivision = Subdivision {
    name: "Banī Suwayf",
    code: "EG-BNS",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-BNS",
};

pub const EG_C: Subdivision = Subdivision {
    name: "Al Qāhirah",
    code: "EG-C",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-C",
};

pub const EG_DK: Subdivision = Subdivision {
    name: "Ad Daqahlīyah",
    code: "EG-DK",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-DK",
};

pub const EG_DT: Subdivision = Subdivision {
    name: "Dumyāţ",
    code: "EG-DT",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-DT",
};

pub const EG_FYM: Subdivision = Subdivision {
    name: "Al Fayyūm",
    code: "EG-FYM",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-FYM",
};

pub const EG_GH: Subdivision = Subdivision {
    name: "Al Gharbīyah",
    code: "EG-GH",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-GH",
};

pub const EG_GZ: Subdivision = Subdivision {
    name: "Al Jīzah",
    code: "EG-GZ",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-GZ",
};

pub const EG_IS: Subdivision = Subdivision {
    name: "Al Ismā'īlīyah",
    code: "EG-IS",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-IS",
};

pub const EG_JS: Subdivision = Subdivision {
    name: "Janūb Sīnā'",
    code: "EG-JS",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-JS",
};

pub const EG_KB: Subdivision = Subdivision {
    name: "Al Qalyūbīyah",
    code: "EG-KB",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-KB",
};

pub const EG_KFS: Subdivision = Subdivision {
    name: "Kafr ash Shaykh",
    code: "EG-KFS",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-KFS",
};

pub const EG_KN: Subdivision = Subdivision {
    name: "Qinā",
    code: "EG-KN",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-KN",
};

pub const EG_LX: Subdivision = Subdivision {
    name: "Al Uqşur",
    code: "EG-LX",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-LX",
};

pub const EG_MN: Subdivision = Subdivision {
    name: "Al Minyā",
    code: "EG-MN",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-MN",
};

pub const EG_MNF: Subdivision = Subdivision {
    name: "Al Minūfīyah",
    code: "EG-MNF",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-MNF",
};

pub const EG_MT: Subdivision = Subdivision {
    name: "Maţrūḩ",
    code: "EG-MT",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-MT",
};

pub const EG_PTS: Subdivision = Subdivision {
    name: "Būr Sa‘īd",
    code: "EG-PTS",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-PTS",
};

pub const EG_SHG: Subdivision = Subdivision {
    name: "Sūhāj",
    code: "EG-SHG",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-SHG",
};

pub const EG_SHR: Subdivision = Subdivision {
    name: "Ash Sharqīyah",
    code: "EG-SHR",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-SHR",
};

pub const EG_SIN: Subdivision = Subdivision {
    name: "Shamāl Sīnā'",
    code: "EG-SIN",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-SIN",
};

pub const EG_SUZ: Subdivision = Subdivision {
    name: "As Suways",
    code: "EG-SUZ",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-SUZ",
};

pub const EG_WAD: Subdivision = Subdivision {
    name: "Al Wādī al Jadīd",
    code: "EG-WAD",
    subdivision_type: "Governorate",
    country_name: "Egypt",
    country_code: "EG",
    region_code: "EG-WAD",
};

pub const SV_AH: Subdivision = Subdivision {
    name: "Ahuachapán",
    code: "SV-AH",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-AH",
};

pub const SV_CA: Subdivision = Subdivision {
    name: "Cabañas",
    code: "SV-CA",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-CA",
};

pub const SV_CH: Subdivision = Subdivision {
    name: "Chalatenango",
    code: "SV-CH",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-CH",
};

pub const SV_CU: Subdivision = Subdivision {
    name: "Cuscatlán",
    code: "SV-CU",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-CU",
};

pub const SV_LI: Subdivision = Subdivision {
    name: "La Libertad",
    code: "SV-LI",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-LI",
};

pub const SV_MO: Subdivision = Subdivision {
    name: "Morazán",
    code: "SV-MO",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-MO",
};

pub const SV_PA: Subdivision = Subdivision {
    name: "La Paz",
    code: "SV-PA",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-PA",
};

pub const SV_SA: Subdivision = Subdivision {
    name: "Santa Ana",
    code: "SV-SA",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-SA",
};

pub const SV_SM: Subdivision = Subdivision {
    name: "San Miguel",
    code: "SV-SM",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-SM",
};

pub const SV_SO: Subdivision = Subdivision {
    name: "Sonsonate",
    code: "SV-SO",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-SO",
};

pub const SV_SS: Subdivision = Subdivision {
    name: "San Salvador",
    code: "SV-SS",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-SS",
};

pub const SV_SV: Subdivision = Subdivision {
    name: "San Vicente",
    code: "SV-SV",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-SV",
};

pub const SV_UN: Subdivision = Subdivision {
    name: "La Unión",
    code: "SV-UN",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-UN",
};

pub const SV_US: Subdivision = Subdivision {
    name: "Usulután",
    code: "SV-US",
    subdivision_type: "Department",
    country_name: "El Salvador",
    country_code: "SV",
    region_code: "SV-US",
};

pub const GQ_AN: Subdivision = Subdivision {
    name: "Annobón",
    code: "GQ-AN",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-AN",
};

pub const GQ_BN: Subdivision = Subdivision {
    name: "Bioko Nord",
    code: "GQ-BN",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-BN",
};

pub const GQ_BS: Subdivision = Subdivision {
    name: "Bioko Sud",
    code: "GQ-BS",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-BS",
};

pub const GQ_C: Subdivision = Subdivision {
    name: "Região Continental",
    code: "GQ-C",
    subdivision_type: "Region",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-C",
};

pub const GQ_CS: Subdivision = Subdivision {
    name: "Centro Sud",
    code: "GQ-CS",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-CS",
};

pub const GQ_I: Subdivision = Subdivision {
    name: "Região Insular",
    code: "GQ-I",
    subdivision_type: "Region",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-I",
};

pub const GQ_KN: Subdivision = Subdivision {
    name: "Kié-Ntem",
    code: "GQ-KN",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-KN",
};

pub const GQ_LI: Subdivision = Subdivision {
    name: "Litoral",
    code: "GQ-LI",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-LI",
};

pub const GQ_WN: Subdivision = Subdivision {
    name: "Wele-Nzas",
    code: "GQ-WN",
    subdivision_type: "Province",
    country_name: "Equatorial Guinea",
    country_code: "GQ",
    region_code: "GQ-WN",
};

pub const ER_AN: Subdivision = Subdivision {
    name: "Ansabā",
    code: "ER-AN",
    subdivision_type: "Region",
    country_name: "Eritrea",
    country_code: "ER",
    region_code: "ER-AN",
};

pub const ER_DK: Subdivision = Subdivision {
    name: "Debubawi K’eyyĭḥ Baḥri",
    code: "ER-DK",
    subdivision_type: "Region",
    country_name: "Eritrea",
    country_code: "ER",
    region_code: "ER-DK",
};

pub const ER_DU: Subdivision = Subdivision {
    name: "Al Janūbī",
    code: "ER-DU",
    subdivision_type: "Region",
    country_name: "Eritrea",
    country_code: "ER",
    region_code: "ER-DU",
};

pub const ER_GB: Subdivision = Subdivision {
    name: "Gash-Barka",
    code: "ER-GB",
    subdivision_type: "Region",
    country_name: "Eritrea",
    country_code: "ER",
    region_code: "ER-GB",
};

pub const ER_MA: Subdivision = Subdivision {
    name: "Al Awsaţ",
    code: "ER-MA",
    subdivision_type: "Region",
    country_name: "Eritrea",
    country_code: "ER",
    region_code: "ER-MA",
};

pub const ER_SK: Subdivision = Subdivision {
    name: "Semienawi K’eyyĭḥ Baḥri",
    code: "ER-SK",
    subdivision_type: "Region",
    country_name: "Eritrea",
    country_code: "ER",
    region_code: "ER-SK",
};

pub const EE_37: Subdivision = Subdivision {
    name: "Harjumaa",
    code: "EE-37",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-37",
};

pub const EE_39: Subdivision = Subdivision {
    name: "Hiiumaa",
    code: "EE-39",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-39",
};

pub const EE_44: Subdivision = Subdivision {
    name: "Ida-Virumaa",
    code: "EE-44",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-44",
};

pub const EE_49: Subdivision = Subdivision {
    name: "Jõgevamaa",
    code: "EE-49",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-49",
};

pub const EE_51: Subdivision = Subdivision {
    name: "Järvamaa",
    code: "EE-51",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-51",
};

pub const EE_57: Subdivision = Subdivision {
    name: "Läänemaa",
    code: "EE-57",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-57",
};

pub const EE_59: Subdivision = Subdivision {
    name: "Lääne-Virumaa",
    code: "EE-59",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-59",
};

pub const EE_65: Subdivision = Subdivision {
    name: "Põlvamaa",
    code: "EE-65",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-65",
};

pub const EE_67: Subdivision = Subdivision {
    name: "Pärnumaa",
    code: "EE-67",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-67",
};

pub const EE_70: Subdivision = Subdivision {
    name: "Raplamaa",
    code: "EE-70",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-70",
};

pub const EE_74: Subdivision = Subdivision {
    name: "Saaremaa",
    code: "EE-74",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-74",
};

pub const EE_78: Subdivision = Subdivision {
    name: "Tartumaa",
    code: "EE-78",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-78",
};

pub const EE_82: Subdivision = Subdivision {
    name: "Valgamaa",
    code: "EE-82",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-82",
};

pub const EE_84: Subdivision = Subdivision {
    name: "Viljandimaa",
    code: "EE-84",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-84",
};

pub const EE_86: Subdivision = Subdivision {
    name: "Võrumaa",
    code: "EE-86",
    subdivision_type: "County",
    country_name: "Estonia",
    country_code: "EE",
    region_code: "EE-86",
};

pub const ET_AA: Subdivision = Subdivision {
    name: "Addis Ababa",
    code: "ET-AA",
    subdivision_type: "Administration",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-AA",
};

pub const ET_AF: Subdivision = Subdivision {
    name: "Afar",
    code: "ET-AF",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-AF",
};

pub const ET_AM: Subdivision = Subdivision {
    name: "Amara",
    code: "ET-AM",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-AM",
};

pub const ET_BE: Subdivision = Subdivision {
    name: "Benshangul-Gumaz",
    code: "ET-BE",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-BE",
};

pub const ET_DD: Subdivision = Subdivision {
    name: "Dire Dawa",
    code: "ET-DD",
    subdivision_type: "Administration",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-DD",
};

pub const ET_GA: Subdivision = Subdivision {
    name: "Gambēla Hizboch",
    code: "ET-GA",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-GA",
};

pub const ET_HA: Subdivision = Subdivision {
    name: "Harari People",
    code: "ET-HA",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-HA",
};

pub const ET_OR: Subdivision = Subdivision {
    name: "Oromia",
    code: "ET-OR",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-OR",
};

pub const ET_SN: Subdivision = Subdivision {
    name: "Southern Nations, Nationalities and Peoples",
    code: "ET-SN",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-SN",
};

pub const ET_SO: Subdivision = Subdivision {
    name: "Somali",
    code: "ET-SO",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-SO",
};

pub const ET_TI: Subdivision = Subdivision {
    name: "Tigrai",
    code: "ET-TI",
    subdivision_type: "State",
    country_name: "Ethiopia",
    country_code: "ET",
    region_code: "ET-TI",
};

pub const FK_FK: Subdivision = Subdivision {
    name: "Falkland Islands",
    code: "FK-FK",
    subdivision_type: "Country",
    country_name: "Falkland Islands",
    country_code: "FK",
    region_code: "FK-FK",
};

pub const FO_FO: Subdivision = Subdivision {
    name: "Faroe Islands",
    code: "FO-FO",
    subdivision_type: "Country",
    country_name: "Faroe Islands",
    country_code: "FO",
    region_code: "FO-FO",
};

pub const FJ_01: Subdivision = Subdivision {
    name: "Ba",
    code: "FJ-01",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-01",
};

pub const FJ_02: Subdivision = Subdivision {
    name: "Bua",
    code: "FJ-02",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-02",
};

pub const FJ_03: Subdivision = Subdivision {
    name: "Cakaudrove",
    code: "FJ-03",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-03",
};

pub const FJ_04: Subdivision = Subdivision {
    name: "Kadavu",
    code: "FJ-04",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-04",
};

pub const FJ_05: Subdivision = Subdivision {
    name: "Lau",
    code: "FJ-05",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-05",
};

pub const FJ_06: Subdivision = Subdivision {
    name: "Lomaiviti",
    code: "FJ-06",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-06",
};

pub const FJ_07: Subdivision = Subdivision {
    name: "Macuata",
    code: "FJ-07",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-07",
};

pub const FJ_08: Subdivision = Subdivision {
    name: "Nadroga and Navosa",
    code: "FJ-08",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-08",
};

pub const FJ_09: Subdivision = Subdivision {
    name: "Naitasiri",
    code: "FJ-09",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-09",
};

pub const FJ_10: Subdivision = Subdivision {
    name: "Namosi",
    code: "FJ-10",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-10",
};

pub const FJ_11: Subdivision = Subdivision {
    name: "Ra",
    code: "FJ-11",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-11",
};

pub const FJ_12: Subdivision = Subdivision {
    name: "Rewa",
    code: "FJ-12",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-12",
};

pub const FJ_13: Subdivision = Subdivision {
    name: "Serua",
    code: "FJ-13",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-13",
};

pub const FJ_14: Subdivision = Subdivision {
    name: "Tailevu",
    code: "FJ-14",
    subdivision_type: "Province",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-14",
};

pub const FJ_C: Subdivision = Subdivision {
    name: "Central",
    code: "FJ-C",
    subdivision_type: "Division",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-C",
};

pub const FJ_E: Subdivision = Subdivision {
    name: "Eastern",
    code: "FJ-E",
    subdivision_type: "Division",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-E",
};

pub const FJ_N: Subdivision = Subdivision {
    name: "Northern",
    code: "FJ-N",
    subdivision_type: "Division",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-N",
};

pub const FJ_R: Subdivision = Subdivision {
    name: "Rotuma",
    code: "FJ-R",
    subdivision_type: "Dependency",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-R",
};

pub const FJ_W: Subdivision = Subdivision {
    name: "Western",
    code: "FJ-W",
    subdivision_type: "Division",
    country_name: "Fiji",
    country_code: "FJ",
    region_code: "FJ-W",
};

pub const FI_01: Subdivision = Subdivision {
    name: "Ahvenanmaan maakunta",
    code: "FI-01",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-01",
};

pub const FI_02: Subdivision = Subdivision {
    name: "Etelä-Karjala",
    code: "FI-02",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-02",
};

pub const FI_03: Subdivision = Subdivision {
    name: "Etelä-Pohjanmaa",
    code: "FI-03",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-03",
};

pub const FI_04: Subdivision = Subdivision {
    name: "Etelä-Savo",
    code: "FI-04",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-04",
};

pub const FI_05: Subdivision = Subdivision {
    name: "Kainuu",
    code: "FI-05",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-05",
};

pub const FI_06: Subdivision = Subdivision {
    name: "Egentliga Tavastland",
    code: "FI-06",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-06",
};

pub const FI_07: Subdivision = Subdivision {
    name: "Keski-Pohjanmaa",
    code: "FI-07",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-07",
};

pub const FI_08: Subdivision = Subdivision {
    name: "Keski-Suomi",
    code: "FI-08",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-08",
};

pub const FI_09: Subdivision = Subdivision {
    name: "Kymenlaakso",
    code: "FI-09",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-09",
};

pub const FI_10: Subdivision = Subdivision {
    name: "Lappi",
    code: "FI-10",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-10",
};

pub const FI_11: Subdivision = Subdivision {
    name: "Birkaland",
    code: "FI-11",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-11",
};

pub const FI_12: Subdivision = Subdivision {
    name: "Österbotten",
    code: "FI-12",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-12",
};

pub const FI_13: Subdivision = Subdivision {
    name: "Norra Karelen",
    code: "FI-13",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-13",
};

pub const FI_14: Subdivision = Subdivision {
    name: "Norra Österbotten",
    code: "FI-14",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-14",
};

pub const FI_15: Subdivision = Subdivision {
    name: "Norra Savolax",
    code: "FI-15",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-15",
};

pub const FI_16: Subdivision = Subdivision {
    name: "Päijänne-Tavastland",
    code: "FI-16",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-16",
};

pub const FI_17: Subdivision = Subdivision {
    name: "Satakunda",
    code: "FI-17",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-17",
};

pub const FI_18: Subdivision = Subdivision {
    name: "Nyland",
    code: "FI-18",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-18",
};

pub const FI_19: Subdivision = Subdivision {
    name: "Egentliga Finland",
    code: "FI-19",
    subdivision_type: "Region",
    country_name: "Finland",
    country_code: "FI",
    region_code: "FI-19",
};

pub const FR_01: Subdivision = Subdivision {
    name: "Ain",
    code: "FR-01",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-01",
};

pub const FR_02: Subdivision = Subdivision {
    name: "Aisne",
    code: "FR-02",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-02",
};

pub const FR_03: Subdivision = Subdivision {
    name: "Allier",
    code: "FR-03",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-03",
};

pub const FR_04: Subdivision = Subdivision {
    name: "Alpes-de-Haute-Provence",
    code: "FR-04",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-04",
};

pub const FR_05: Subdivision = Subdivision {
    name: "Hautes-Alpes",
    code: "FR-05",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-05",
};

pub const FR_06: Subdivision = Subdivision {
    name: "Alpes-Maritimes",
    code: "FR-06",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-06",
};

pub const FR_07: Subdivision = Subdivision {
    name: "Ardèche",
    code: "FR-07",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-07",
};

pub const FR_08: Subdivision = Subdivision {
    name: "Ardennes",
    code: "FR-08",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-08",
};

pub const FR_09: Subdivision = Subdivision {
    name: "Ariège",
    code: "FR-09",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-09",
};

pub const FR_10: Subdivision = Subdivision {
    name: "Aube",
    code: "FR-10",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-10",
};

pub const FR_11: Subdivision = Subdivision {
    name: "Aude",
    code: "FR-11",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-11",
};

pub const FR_12: Subdivision = Subdivision {
    name: "Aveyron",
    code: "FR-12",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-12",
};

pub const FR_13: Subdivision = Subdivision {
    name: "Bouches-du-Rhône",
    code: "FR-13",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-13",
};

pub const FR_14: Subdivision = Subdivision {
    name: "Calvados",
    code: "FR-14",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-14",
};

pub const FR_15: Subdivision = Subdivision {
    name: "Cantal",
    code: "FR-15",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-15",
};

pub const FR_16: Subdivision = Subdivision {
    name: "Charente",
    code: "FR-16",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-16",
};

pub const FR_17: Subdivision = Subdivision {
    name: "Charente-Maritime",
    code: "FR-17",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-17",
};

pub const FR_18: Subdivision = Subdivision {
    name: "Cher",
    code: "FR-18",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-18",
};

pub const FR_19: Subdivision = Subdivision {
    name: "Corrèze",
    code: "FR-19",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-19",
};

pub const FR_21: Subdivision = Subdivision {
    name: "Côte-d'Or",
    code: "FR-21",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-21",
};

pub const FR_22: Subdivision = Subdivision {
    name: "Côtes-d'Armor",
    code: "FR-22",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-22",
};

pub const FR_23: Subdivision = Subdivision {
    name: "Creuse",
    code: "FR-23",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-23",
};

pub const FR_24: Subdivision = Subdivision {
    name: "Dordogne",
    code: "FR-24",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-24",
};

pub const FR_25: Subdivision = Subdivision {
    name: "Doubs",
    code: "FR-25",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-25",
};

pub const FR_26: Subdivision = Subdivision {
    name: "Drôme",
    code: "FR-26",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-26",
};

pub const FR_27: Subdivision = Subdivision {
    name: "Eure",
    code: "FR-27",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-27",
};

pub const FR_28: Subdivision = Subdivision {
    name: "Eure-et-Loir",
    code: "FR-28",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-28",
};

pub const FR_29: Subdivision = Subdivision {
    name: "Finistère",
    code: "FR-29",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-29",
};

pub const FR_2A: Subdivision = Subdivision {
    name: "Corse-du-Sud",
    code: "FR-2A",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-2A",
};

pub const FR_2B: Subdivision = Subdivision {
    name: "Haute-Corse",
    code: "FR-2B",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-2B",
};

pub const FR_30: Subdivision = Subdivision {
    name: "Gard",
    code: "FR-30",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-30",
};

pub const FR_31: Subdivision = Subdivision {
    name: "Haute-Garonne",
    code: "FR-31",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-31",
};

pub const FR_32: Subdivision = Subdivision {
    name: "Gers",
    code: "FR-32",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-32",
};

pub const FR_33: Subdivision = Subdivision {
    name: "Gironde",
    code: "FR-33",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-33",
};

pub const FR_34: Subdivision = Subdivision {
    name: "Hérault",
    code: "FR-34",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-34",
};

pub const FR_35: Subdivision = Subdivision {
    name: "Ille-et-Vilaine",
    code: "FR-35",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-35",
};

pub const FR_36: Subdivision = Subdivision {
    name: "Indre",
    code: "FR-36",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-36",
};

pub const FR_37: Subdivision = Subdivision {
    name: "Indre-et-Loire",
    code: "FR-37",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-37",
};

pub const FR_38: Subdivision = Subdivision {
    name: "Isère",
    code: "FR-38",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-38",
};

pub const FR_39: Subdivision = Subdivision {
    name: "Jura",
    code: "FR-39",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-39",
};

pub const FR_40: Subdivision = Subdivision {
    name: "Landes",
    code: "FR-40",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-40",
};

pub const FR_41: Subdivision = Subdivision {
    name: "Loir-et-Cher",
    code: "FR-41",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-41",
};

pub const FR_42: Subdivision = Subdivision {
    name: "Loire",
    code: "FR-42",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-42",
};

pub const FR_43: Subdivision = Subdivision {
    name: "Haute-Loire",
    code: "FR-43",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-43",
};

pub const FR_44: Subdivision = Subdivision {
    name: "Loire-Atlantique",
    code: "FR-44",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-44",
};

pub const FR_45: Subdivision = Subdivision {
    name: "Loiret",
    code: "FR-45",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-45",
};

pub const FR_46: Subdivision = Subdivision {
    name: "Lot",
    code: "FR-46",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-46",
};

pub const FR_47: Subdivision = Subdivision {
    name: "Lot-et-Garonne",
    code: "FR-47",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-47",
};

pub const FR_48: Subdivision = Subdivision {
    name: "Lozère",
    code: "FR-48",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-48",
};

pub const FR_49: Subdivision = Subdivision {
    name: "Maine-et-Loire",
    code: "FR-49",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-49",
};

pub const FR_50: Subdivision = Subdivision {
    name: "Manche",
    code: "FR-50",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-50",
};

pub const FR_51: Subdivision = Subdivision {
    name: "Marne",
    code: "FR-51",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-51",
};

pub const FR_52: Subdivision = Subdivision {
    name: "Haute-Marne",
    code: "FR-52",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-52",
};

pub const FR_53: Subdivision = Subdivision {
    name: "Mayenne",
    code: "FR-53",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-53",
};

pub const FR_54: Subdivision = Subdivision {
    name: "Meurthe-et-Moselle",
    code: "FR-54",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-54",
};

pub const FR_55: Subdivision = Subdivision {
    name: "Meuse",
    code: "FR-55",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-55",
};

pub const FR_56: Subdivision = Subdivision {
    name: "Morbihan",
    code: "FR-56",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-56",
};

pub const FR_57: Subdivision = Subdivision {
    name: "Moselle",
    code: "FR-57",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-57",
};

pub const FR_58: Subdivision = Subdivision {
    name: "Nièvre",
    code: "FR-58",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-58",
};

pub const FR_59: Subdivision = Subdivision {
    name: "Nord",
    code: "FR-59",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-59",
};

pub const FR_60: Subdivision = Subdivision {
    name: "Oise",
    code: "FR-60",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-60",
};

pub const FR_61: Subdivision = Subdivision {
    name: "Orne",
    code: "FR-61",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-61",
};

pub const FR_62: Subdivision = Subdivision {
    name: "Pas-de-Calais",
    code: "FR-62",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-62",
};

pub const FR_63: Subdivision = Subdivision {
    name: "Puy-de-Dôme",
    code: "FR-63",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-63",
};

pub const FR_64: Subdivision = Subdivision {
    name: "Pyrénées-Atlantiques",
    code: "FR-64",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-64",
};

pub const FR_65: Subdivision = Subdivision {
    name: "Hautes-Pyrénées",
    code: "FR-65",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-65",
};

pub const FR_66: Subdivision = Subdivision {
    name: "Pyrénées-Orientales",
    code: "FR-66",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-66",
};

pub const FR_67: Subdivision = Subdivision {
    name: "Bas-Rhin",
    code: "FR-67",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-67",
};

pub const FR_68: Subdivision = Subdivision {
    name: "Haut-Rhin",
    code: "FR-68",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-68",
};

pub const FR_69: Subdivision = Subdivision {
    name: "Rhône",
    code: "FR-69",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-69",
};

pub const FR_70: Subdivision = Subdivision {
    name: "Haute-Saône",
    code: "FR-70",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-70",
};

pub const FR_71: Subdivision = Subdivision {
    name: "Saône-et-Loire",
    code: "FR-71",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-71",
};

pub const FR_72: Subdivision = Subdivision {
    name: "Sarthe",
    code: "FR-72",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-72",
};

pub const FR_73: Subdivision = Subdivision {
    name: "Savoie",
    code: "FR-73",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-73",
};

pub const FR_74: Subdivision = Subdivision {
    name: "Haute-Savoie",
    code: "FR-74",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-74",
};

pub const FR_75: Subdivision = Subdivision {
    name: "Paris",
    code: "FR-75",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-75",
};

pub const FR_76: Subdivision = Subdivision {
    name: "Seine-Maritime",
    code: "FR-76",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-76",
};

pub const FR_77: Subdivision = Subdivision {
    name: "Seine-et-Marne",
    code: "FR-77",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-77",
};

pub const FR_78: Subdivision = Subdivision {
    name: "Yvelines",
    code: "FR-78",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-78",
};

pub const FR_79: Subdivision = Subdivision {
    name: "Deux-Sèvres",
    code: "FR-79",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-79",
};

pub const FR_80: Subdivision = Subdivision {
    name: "Somme",
    code: "FR-80",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-80",
};

pub const FR_81: Subdivision = Subdivision {
    name: "Tarn",
    code: "FR-81",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-81",
};

pub const FR_82: Subdivision = Subdivision {
    name: "Tarn-et-Garonne",
    code: "FR-82",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-82",
};

pub const FR_83: Subdivision = Subdivision {
    name: "Var",
    code: "FR-83",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-83",
};

pub const FR_84: Subdivision = Subdivision {
    name: "Vaucluse",
    code: "FR-84",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-84",
};

pub const FR_85: Subdivision = Subdivision {
    name: "Vendée",
    code: "FR-85",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-85",
};

pub const FR_86: Subdivision = Subdivision {
    name: "Vienne",
    code: "FR-86",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-86",
};

pub const FR_87: Subdivision = Subdivision {
    name: "Haute-Vienne",
    code: "FR-87",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-87",
};

pub const FR_88: Subdivision = Subdivision {
    name: "Vosges",
    code: "FR-88",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-88",
};

pub const FR_89: Subdivision = Subdivision {
    name: "Yonne",
    code: "FR-89",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-89",
};

pub const FR_90: Subdivision = Subdivision {
    name: "Territoire de Belfort",
    code: "FR-90",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-90",
};

pub const FR_91: Subdivision = Subdivision {
    name: "Essonne",
    code: "FR-91",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-91",
};

pub const FR_92: Subdivision = Subdivision {
    name: "Hauts-de-Seine",
    code: "FR-92",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-92",
};

pub const FR_93: Subdivision = Subdivision {
    name: "Seine-Saint-Denis",
    code: "FR-93",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-93",
};

pub const FR_94: Subdivision = Subdivision {
    name: "Val-de-Marne",
    code: "FR-94",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-94",
};

pub const FR_95: Subdivision = Subdivision {
    name: "Val-d'Oise",
    code: "FR-95",
    subdivision_type: "Metropolitan department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-95",
};

pub const FR_A: Subdivision = Subdivision {
    name: "Alsace",
    code: "FR-A",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-A",
};

pub const FR_B: Subdivision = Subdivision {
    name: "Aquitaine",
    code: "FR-B",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-B",
};

pub const FR_BL: Subdivision = Subdivision {
    name: "Saint-Barthélemy",
    code: "FR-BL",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-BL",
};

pub const FR_C: Subdivision = Subdivision {
    name: "Auvergne",
    code: "FR-C",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-C",
};

pub const FR_CP: Subdivision = Subdivision {
    name: "Clipperton",
    code: "FR-CP",
    subdivision_type: "Dependency",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-CP",
};

pub const FR_D: Subdivision = Subdivision {
    name: "Bourgogne",
    code: "FR-D",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-D",
};

pub const FR_E: Subdivision = Subdivision {
    name: "Bretagne",
    code: "FR-E",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-E",
};

pub const FR_F: Subdivision = Subdivision {
    name: "Centre",
    code: "FR-F",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-F",
};

pub const FR_G: Subdivision = Subdivision {
    name: "Champagne-Ardenne",
    code: "FR-G",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-G",
};

pub const FR_GF: Subdivision = Subdivision {
    name: "Guyane",
    code: "FR-GF",
    subdivision_type: "Overseas department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-GF",
};

pub const FR_GP: Subdivision = Subdivision {
    name: "Guadeloupe",
    code: "FR-GP",
    subdivision_type: "Overseas department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-GP",
};

pub const FR_H: Subdivision = Subdivision {
    name: "Corse",
    code: "FR-H",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-H",
};

pub const FR_I: Subdivision = Subdivision {
    name: "Franche-Comté",
    code: "FR-I",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-I",
};

pub const FR_J: Subdivision = Subdivision {
    name: "Île-de-France",
    code: "FR-J",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-J",
};

pub const FR_K: Subdivision = Subdivision {
    name: "Languedoc-Roussillon",
    code: "FR-K",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-K",
};

pub const FR_L: Subdivision = Subdivision {
    name: "Limousin",
    code: "FR-L",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-L",
};

pub const FR_MF: Subdivision = Subdivision {
    name: "Saint-Martin",
    code: "FR-MF",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-MF",
};

pub const FR_M: Subdivision = Subdivision {
    name: "Lorraine",
    code: "FR-M",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-M",
};

pub const FR_MQ: Subdivision = Subdivision {
    name: "Martinique",
    code: "FR-MQ",
    subdivision_type: "Overseas department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-MQ",
};

pub const FR_NC: Subdivision = Subdivision {
    name: "Nouvelle-Calédonie",
    code: "FR-NC",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-NC",
};

pub const FR_N: Subdivision = Subdivision {
    name: "Midi-Pyrénées",
    code: "FR-N",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-N",
};

pub const FR_O: Subdivision = Subdivision {
    name: "Nord-Pas-de-Calais",
    code: "FR-O",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-O",
};

pub const FR_P: Subdivision = Subdivision {
    name: "Basse-Normandie",
    code: "FR-P",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-P",
};

pub const FR_PF: Subdivision = Subdivision {
    name: "Polynésie française",
    code: "FR-PF",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-PF",
};

pub const FR_PM: Subdivision = Subdivision {
    name: "Saint-Pierre-et-Miquelon",
    code: "FR-PM",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-PM",
};

pub const FR_Q: Subdivision = Subdivision {
    name: "Haute-Normandie",
    code: "FR-Q",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-Q",
};

pub const FR_RE: Subdivision = Subdivision {
    name: "La Réunion",
    code: "FR-RE",
    subdivision_type: "Overseas department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-RE",
};

pub const FR_R: Subdivision = Subdivision {
    name: "Pays-de-la-Loire",
    code: "FR-R",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-R",
};

pub const FR_S: Subdivision = Subdivision {
    name: "Picardie",
    code: "FR-S",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-S",
};

pub const FR_TF: Subdivision = Subdivision {
    name: "Terres australes françaises",
    code: "FR-TF",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-TF",
};

pub const FR_T: Subdivision = Subdivision {
    name: "Poitou-Charentes",
    code: "FR-T",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-T",
};

pub const FR_U: Subdivision = Subdivision {
    name: "Provence-Alpes-Côte-d'Azur",
    code: "FR-U",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-U",
};

pub const FR_V: Subdivision = Subdivision {
    name: "Rhône-Alpes",
    code: "FR-V",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-V",
};

pub const FR_WF: Subdivision = Subdivision {
    name: "Wallis-et-Futuna",
    code: "FR-WF",
    subdivision_type: "Overseas territorial collectivity",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-WF",
};

pub const FR_YT: Subdivision = Subdivision {
    name: "Mayotte",
    code: "FR-YT",
    subdivision_type: "Overseas department",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-YT",
};

pub const FR_ARA: Subdivision = Subdivision {
    name: "Auvergne-Rhône-Alpes",
    code: "FR-ARA",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-ARA",
};

pub const FR_BFC: Subdivision = Subdivision {
    name: "Bourgogne-Franche-Comté",
    code: "FR-BFC",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-BFC",
};

pub const FR_BRE: Subdivision = Subdivision {
    name: "Bretagne",
    code: "FR-BRE",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-BRE",
};

pub const FR_CVL: Subdivision = Subdivision {
    name: "Centre-Val de Loire",
    code: "FR-CVL",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-CVL",
};

pub const FR_20R: Subdivision = Subdivision {
    name: "Corse",
    code: "FR-20R",
    subdivision_type: "Metropolitan collectivity with special status",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-20R",
};

pub const FR_GES: Subdivision = Subdivision {
    name: "Grand-Est",
    code: "FR-GES",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-GES",
};

pub const FR_HDF: Subdivision = Subdivision {
    name: "Hauts-de-France",
    code: "FR-HDF",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-HDF",
};

pub const FR_IDF: Subdivision = Subdivision {
    name: "Île-de-France",
    code: "FR-IDF",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-IDF",
};

pub const FR_NOR: Subdivision = Subdivision {
    name: "Normandie",
    code: "FR-NOR",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-NOR",
};

pub const FR_NAQ: Subdivision = Subdivision {
    name: "Nouvelle-Aquitaine",
    code: "FR-NAQ",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-NAQ",
};

pub const FR_OCC: Subdivision = Subdivision {
    name: "Occitanie",
    code: "FR-OCC",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-OCC",
};

pub const FR_PDL: Subdivision = Subdivision {
    name: "Pays-de-la-Loire",
    code: "FR-PDL",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-PDL",
};

pub const FR_PAC: Subdivision = Subdivision {
    name: "Provence-Alpes-Côte-d’Azur",
    code: "FR-PAC",
    subdivision_type: "Metropolitan region",
    country_name: "France",
    country_code: "FR",
    region_code: "FR-PAC",
};

pub const GF_GF: Subdivision = Subdivision {
    name: "French Guiana",
    code: "GF-GF",
    subdivision_type: "Country",
    country_name: "French Guiana",
    country_code: "GF",
    region_code: "GF-GF",
};

pub const PF_PF: Subdivision = Subdivision {
    name: "French Polynesia",
    code: "PF-PF",
    subdivision_type: "Country",
    country_name: "French Polynesia",
    country_code: "PF",
    region_code: "PF-PF",
};

pub const TF_TF: Subdivision = Subdivision {
    name: "French Southern Territories",
    code: "TF-TF",
    subdivision_type: "Country",
    country_name: "French Southern Territories",
    country_code: "TF",
    region_code: "TF-TF",
};

pub const GA_1: Subdivision = Subdivision {
    name: "Estuaire",
    code: "GA-1",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-1",
};

pub const GA_2: Subdivision = Subdivision {
    name: "Haut-Ogooué",
    code: "GA-2",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-2",
};

pub const GA_3: Subdivision = Subdivision {
    name: "Moyen-Ogooué",
    code: "GA-3",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-3",
};

pub const GA_4: Subdivision = Subdivision {
    name: "Ngounié",
    code: "GA-4",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-4",
};

pub const GA_5: Subdivision = Subdivision {
    name: "Nyanga",
    code: "GA-5",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-5",
};

pub const GA_6: Subdivision = Subdivision {
    name: "Ogooué-Ivindo",
    code: "GA-6",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-6",
};

pub const GA_7: Subdivision = Subdivision {
    name: "Ogooué-Lolo",
    code: "GA-7",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-7",
};

pub const GA_8: Subdivision = Subdivision {
    name: "Ogooué-Maritime",
    code: "GA-8",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-8",
};

pub const GA_9: Subdivision = Subdivision {
    name: "Woleu-Ntem",
    code: "GA-9",
    subdivision_type: "Province",
    country_name: "Gabon",
    country_code: "GA",
    region_code: "GA-9",
};

pub const GM_B: Subdivision = Subdivision {
    name: "Banjul",
    code: "GM-B",
    subdivision_type: "City",
    country_name: "Gambia",
    country_code: "GM",
    region_code: "GM-B",
};

pub const GM_L: Subdivision = Subdivision {
    name: "Lower River",
    code: "GM-L",
    subdivision_type: "Division",
    country_name: "Gambia",
    country_code: "GM",
    region_code: "GM-L",
};

pub const GM_M: Subdivision = Subdivision {
    name: "Central River",
    code: "GM-M",
    subdivision_type: "Division",
    country_name: "Gambia",
    country_code: "GM",
    region_code: "GM-M",
};

pub const GM_N: Subdivision = Subdivision {
    name: "North Bank",
    code: "GM-N",
    subdivision_type: "Division",
    country_name: "Gambia",
    country_code: "GM",
    region_code: "GM-N",
};

pub const GM_U: Subdivision = Subdivision {
    name: "Upper River",
    code: "GM-U",
    subdivision_type: "Division",
    country_name: "Gambia",
    country_code: "GM",
    region_code: "GM-U",
};

pub const GM_W: Subdivision = Subdivision {
    name: "Western",
    code: "GM-W",
    subdivision_type: "Division",
    country_name: "Gambia",
    country_code: "GM",
    region_code: "GM-W",
};

pub const GE_AB: Subdivision = Subdivision {
    name: "Abkhazia",
    code: "GE-AB",
    subdivision_type: "Autonomous republic",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-AB",
};

pub const GE_AJ: Subdivision = Subdivision {
    name: "Ajaria",
    code: "GE-AJ",
    subdivision_type: "Autonomous republic",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-AJ",
};

pub const GE_GU: Subdivision = Subdivision {
    name: "Guria",
    code: "GE-GU",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-GU",
};

pub const GE_IM: Subdivision = Subdivision {
    name: "Imereti",
    code: "GE-IM",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-IM",
};

pub const GE_KA: Subdivision = Subdivision {
    name: "K'akheti",
    code: "GE-KA",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-KA",
};

pub const GE_KK: Subdivision = Subdivision {
    name: "Kvemo Kartli",
    code: "GE-KK",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-KK",
};

pub const GE_MM: Subdivision = Subdivision {
    name: "Mtskheta-Mtianeti",
    code: "GE-MM",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-MM",
};

pub const GE_RL: Subdivision = Subdivision {
    name: "Rach'a-Lechkhumi-Kvemo Svaneti",
    code: "GE-RL",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-RL",
};

pub const GE_SJ: Subdivision = Subdivision {
    name: "Samtskhe-Javakheti",
    code: "GE-SJ",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-SJ",
};

pub const GE_SK: Subdivision = Subdivision {
    name: "Shida Kartli",
    code: "GE-SK",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-SK",
};

pub const GE_SZ: Subdivision = Subdivision {
    name: "Samegrelo-Zemo Svaneti",
    code: "GE-SZ",
    subdivision_type: "Region",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-SZ",
};

pub const GE_TB: Subdivision = Subdivision {
    name: "Tbilisi",
    code: "GE-TB",
    subdivision_type: "City",
    country_name: "Georgia",
    country_code: "GE",
    region_code: "GE-TB",
};

pub const DE_BB: Subdivision = Subdivision {
    name: "Brandenburg",
    code: "DE-BB",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-BB",
};

pub const DE_BE: Subdivision = Subdivision {
    name: "Berlin",
    code: "DE-BE",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-BE",
};

pub const DE_BW: Subdivision = Subdivision {
    name: "Baden-Württemberg",
    code: "DE-BW",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-BW",
};

pub const DE_BY: Subdivision = Subdivision {
    name: "Bayern",
    code: "DE-BY",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-BY",
};

pub const DE_HB: Subdivision = Subdivision {
    name: "Bremen",
    code: "DE-HB",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-HB",
};

pub const DE_HE: Subdivision = Subdivision {
    name: "Hessen",
    code: "DE-HE",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-HE",
};

pub const DE_HH: Subdivision = Subdivision {
    name: "Hamburg",
    code: "DE-HH",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-HH",
};

pub const DE_MV: Subdivision = Subdivision {
    name: "Mecklenburg-Vorpommern",
    code: "DE-MV",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-MV",
};

pub const DE_NI: Subdivision = Subdivision {
    name: "Niedersachsen",
    code: "DE-NI",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-NI",
};

pub const DE_NW: Subdivision = Subdivision {
    name: "Nordrhein-Westfalen",
    code: "DE-NW",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-NW",
};

pub const DE_RP: Subdivision = Subdivision {
    name: "Rheinland-Pfalz",
    code: "DE-RP",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-RP",
};

pub const DE_SH: Subdivision = Subdivision {
    name: "Schleswig-Holstein",
    code: "DE-SH",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-SH",
};

pub const DE_SL: Subdivision = Subdivision {
    name: "Saarland",
    code: "DE-SL",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-SL",
};

pub const DE_SN: Subdivision = Subdivision {
    name: "Sachsen",
    code: "DE-SN",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-SN",
};

pub const DE_ST: Subdivision = Subdivision {
    name: "Sachsen-Anhalt",
    code: "DE-ST",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-ST",
};

pub const DE_TH: Subdivision = Subdivision {
    name: "Thüringen",
    code: "DE-TH",
    subdivision_type: "Länder",
    country_name: "Germany",
    country_code: "DE",
    region_code: "DE-TH",
};

pub const GH_AA: Subdivision = Subdivision {
    name: "Greater Accra",
    code: "GH-AA",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-AA",
};

pub const GH_AH: Subdivision = Subdivision {
    name: "Ashanti",
    code: "GH-AH",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-AH",
};

pub const GH_BA: Subdivision = Subdivision {
    name: "Brong-Ahafo",
    code: "GH-BA",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-BA",
};

pub const GH_CP: Subdivision = Subdivision {
    name: "Central",
    code: "GH-CP",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-CP",
};

pub const GH_EP: Subdivision = Subdivision {
    name: "Eastern",
    code: "GH-EP",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-EP",
};

pub const GH_NP: Subdivision = Subdivision {
    name: "Northern",
    code: "GH-NP",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-NP",
};

pub const GH_TV: Subdivision = Subdivision {
    name: "Volta",
    code: "GH-TV",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-TV",
};

pub const GH_UE: Subdivision = Subdivision {
    name: "Upper East",
    code: "GH-UE",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-UE",
};

pub const GH_UW: Subdivision = Subdivision {
    name: "Upper West",
    code: "GH-UW",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-UW",
};

pub const GH_WP: Subdivision = Subdivision {
    name: "Western",
    code: "GH-WP",
    subdivision_type: "Region",
    country_name: "Ghana",
    country_code: "GH",
    region_code: "GH-WP",
};

pub const GI_GI: Subdivision = Subdivision {
    name: "Gibraltar",
    code: "GI-GI",
    subdivision_type: "Country",
    country_name: "Gibraltar",
    country_code: "GI",
    region_code: "GI-GI",
};

pub const GR_01: Subdivision = Subdivision {
    name: "Aitoloakarnanía",
    code: "GR-01",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-01",
};

pub const GR_03: Subdivision = Subdivision {
    name: "Voiotía",
    code: "GR-03",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-03",
};

pub const GR_04: Subdivision = Subdivision {
    name: "Évvoia",
    code: "GR-04",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-04",
};

pub const GR_05: Subdivision = Subdivision {
    name: "Evrytanía",
    code: "GR-05",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-05",
};

pub const GR_06: Subdivision = Subdivision {
    name: "Fthiótida",
    code: "GR-06",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-06",
};

pub const GR_07: Subdivision = Subdivision {
    name: "Fokída",
    code: "GR-07",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-07",
};

pub const GR_11: Subdivision = Subdivision {
    name: "Argolída",
    code: "GR-11",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-11",
};

pub const GR_12: Subdivision = Subdivision {
    name: "Arkadía",
    code: "GR-12",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-12",
};

pub const GR_13: Subdivision = Subdivision {
    name: "Achaḯa",
    code: "GR-13",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-13",
};

pub const GR_14: Subdivision = Subdivision {
    name: "Ileía",
    code: "GR-14",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-14",
};

pub const GR_15: Subdivision = Subdivision {
    name: "Korinthía",
    code: "GR-15",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-15",
};

pub const GR_16: Subdivision = Subdivision {
    name: "Lakonía",
    code: "GR-16",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-16",
};

pub const GR_17: Subdivision = Subdivision {
    name: "Messinía",
    code: "GR-17",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-17",
};

pub const GR_21: Subdivision = Subdivision {
    name: "Zákynthos",
    code: "GR-21",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-21",
};

pub const GR_22: Subdivision = Subdivision {
    name: "Kérkyra",
    code: "GR-22",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-22",
};

pub const GR_23: Subdivision = Subdivision {
    name: "Kefallinía",
    code: "GR-23",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-23",
};

pub const GR_24: Subdivision = Subdivision {
    name: "Lefkáda",
    code: "GR-24",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-24",
};

pub const GR_31: Subdivision = Subdivision {
    name: "Árta",
    code: "GR-31",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-31",
};

pub const GR_32: Subdivision = Subdivision {
    name: "Thesprotía",
    code: "GR-32",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-32",
};

pub const GR_33: Subdivision = Subdivision {
    name: "Ioánnina",
    code: "GR-33",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-33",
};

pub const GR_34: Subdivision = Subdivision {
    name: "Préveza",
    code: "GR-34",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-34",
};

pub const GR_41: Subdivision = Subdivision {
    name: "Kardítsa",
    code: "GR-41",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-41",
};

pub const GR_42: Subdivision = Subdivision {
    name: "Lárisa",
    code: "GR-42",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-42",
};

pub const GR_43: Subdivision = Subdivision {
    name: "Magnisía",
    code: "GR-43",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-43",
};

pub const GR_44: Subdivision = Subdivision {
    name: "Tríkala",
    code: "GR-44",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-44",
};

pub const GR_51: Subdivision = Subdivision {
    name: "Grevená",
    code: "GR-51",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-51",
};

pub const GR_52: Subdivision = Subdivision {
    name: "Dráma",
    code: "GR-52",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-52",
};

pub const GR_53: Subdivision = Subdivision {
    name: "Imathía",
    code: "GR-53",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-53",
};

pub const GR_54: Subdivision = Subdivision {
    name: "Thessaloníki",
    code: "GR-54",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-54",
};

pub const GR_55: Subdivision = Subdivision {
    name: "Kavála",
    code: "GR-55",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-55",
};

pub const GR_56: Subdivision = Subdivision {
    name: "Kastoriá",
    code: "GR-56",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-56",
};

pub const GR_57: Subdivision = Subdivision {
    name: "Kilkís",
    code: "GR-57",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-57",
};

pub const GR_58: Subdivision = Subdivision {
    name: "Kozáni",
    code: "GR-58",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-58",
};

pub const GR_59: Subdivision = Subdivision {
    name: "Pélla",
    code: "GR-59",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-59",
};

pub const GR_61: Subdivision = Subdivision {
    name: "Piería",
    code: "GR-61",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-61",
};

pub const GR_62: Subdivision = Subdivision {
    name: "Sérres",
    code: "GR-62",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-62",
};

pub const GR_63: Subdivision = Subdivision {
    name: "Flórina",
    code: "GR-63",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-63",
};

pub const GR_64: Subdivision = Subdivision {
    name: "Chalkidikí",
    code: "GR-64",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-64",
};

pub const GR_69: Subdivision = Subdivision {
    name: "Ágion Óros",
    code: "GR-69",
    subdivision_type: "Self-governed part",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-69",
};

pub const GR_71: Subdivision = Subdivision {
    name: "Évros",
    code: "GR-71",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-71",
};

pub const GR_72: Subdivision = Subdivision {
    name: "Xánthi",
    code: "GR-72",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-72",
};

pub const GR_73: Subdivision = Subdivision {
    name: "Rodópi",
    code: "GR-73",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-73",
};

pub const GR_81: Subdivision = Subdivision {
    name: "Dodekánisa",
    code: "GR-81",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-81",
};

pub const GR_82: Subdivision = Subdivision {
    name: "Kykládes",
    code: "GR-82",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-82",
};

pub const GR_83: Subdivision = Subdivision {
    name: "Lésvos",
    code: "GR-83",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-83",
};

pub const GR_84: Subdivision = Subdivision {
    name: "Sámos",
    code: "GR-84",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-84",
};

pub const GR_85: Subdivision = Subdivision {
    name: "Chíos",
    code: "GR-85",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-85",
};

pub const GR_91: Subdivision = Subdivision {
    name: "Irakleío",
    code: "GR-91",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-91",
};

pub const GR_92: Subdivision = Subdivision {
    name: "Lasíthi",
    code: "GR-92",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-92",
};

pub const GR_93: Subdivision = Subdivision {
    name: "Rethýmnis",
    code: "GR-93",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-93",
};

pub const GR_94: Subdivision = Subdivision {
    name: "Chaniá",
    code: "GR-94",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-94",
};

pub const GR_A1: Subdivision = Subdivision {
    name: "Attikí",
    code: "GR-A1",
    subdivision_type: "Department",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-A1",
};

pub const GR_A: Subdivision = Subdivision {
    name: "Anatolikí Makedonía kai Thráki",
    code: "GR-A",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-A",
};

pub const GR_B: Subdivision = Subdivision {
    name: "Kentrikí Makedonía",
    code: "GR-B",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-B",
};

pub const GR_C: Subdivision = Subdivision {
    name: "Dytikí Makedonía",
    code: "GR-C",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-C",
};

pub const GR_D: Subdivision = Subdivision {
    name: "Ípeiros",
    code: "GR-D",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-D",
};

pub const GR_E: Subdivision = Subdivision {
    name: "Thessalía",
    code: "GR-E",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-E",
};

pub const GR_F: Subdivision = Subdivision {
    name: "Ionía Nísia",
    code: "GR-F",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-F",
};

pub const GR_G: Subdivision = Subdivision {
    name: "Dytikí Elláda",
    code: "GR-G",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-G",
};

pub const GR_H: Subdivision = Subdivision {
    name: "Stereá Elláda",
    code: "GR-H",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-H",
};

pub const GR_I: Subdivision = Subdivision {
    name: "Attikí",
    code: "GR-I",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-I",
};

pub const GR_J: Subdivision = Subdivision {
    name: "Peloponnísos",
    code: "GR-J",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-J",
};

pub const GR_K: Subdivision = Subdivision {
    name: "Voreío Aigaío",
    code: "GR-K",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-K",
};

pub const GR_L: Subdivision = Subdivision {
    name: "Notío Aigaío",
    code: "GR-L",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-L",
};

pub const GR_M: Subdivision = Subdivision {
    name: "Krítí",
    code: "GR-M",
    subdivision_type: "Administrative region",
    country_name: "Greece",
    country_code: "GR",
    region_code: "GR-M",
};

pub const GL_KU: Subdivision = Subdivision {
    name: "Kommune Kujalleq",
    code: "GL-KU",
    subdivision_type: "Municipality",
    country_name: "Greenland",
    country_code: "GL",
    region_code: "GL-KU",
};

pub const GL_QA: Subdivision = Subdivision {
    name: "Qaasuitsup Kommunia",
    code: "GL-QA",
    subdivision_type: "Municipality",
    country_name: "Greenland",
    country_code: "GL",
    region_code: "GL-QA",
};

pub const GL_QE: Subdivision = Subdivision {
    name: "Qeqqata Kommunia",
    code: "GL-QE",
    subdivision_type: "Municipality",
    country_name: "Greenland",
    country_code: "GL",
    region_code: "GL-QE",
};

pub const GL_SM: Subdivision = Subdivision {
    name: "Kommuneqarfik Sermersooq",
    code: "GL-SM",
    subdivision_type: "Municipality",
    country_name: "Greenland",
    country_code: "GL",
    region_code: "GL-SM",
};

pub const GD_01: Subdivision = Subdivision {
    name: "Saint Andrew",
    code: "GD-01",
    subdivision_type: "Parish",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-01",
};

pub const GD_02: Subdivision = Subdivision {
    name: "Saint David",
    code: "GD-02",
    subdivision_type: "Parish",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-02",
};

pub const GD_03: Subdivision = Subdivision {
    name: "Saint George",
    code: "GD-03",
    subdivision_type: "Parish",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-03",
};

pub const GD_04: Subdivision = Subdivision {
    name: "Saint John",
    code: "GD-04",
    subdivision_type: "Parish",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-04",
};

pub const GD_05: Subdivision = Subdivision {
    name: "Saint Mark",
    code: "GD-05",
    subdivision_type: "Parish",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-05",
};

pub const GD_06: Subdivision = Subdivision {
    name: "Saint Patrick",
    code: "GD-06",
    subdivision_type: "Parish",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-06",
};

pub const GD_10: Subdivision = Subdivision {
    name: "Southern Grenadine Islands",
    code: "GD-10",
    subdivision_type: "Dependency",
    country_name: "Grenada",
    country_code: "GD",
    region_code: "GD-10",
};

pub const GP_GP: Subdivision = Subdivision {
    name: "Guadeloupe",
    code: "GP-GP",
    subdivision_type: "Country",
    country_name: "Guadeloupe",
    country_code: "GP",
    region_code: "GP-GP",
};

pub const GU_GU: Subdivision = Subdivision {
    name: "Guam",
    code: "GU-GU",
    subdivision_type: "Country",
    country_name: "Guam",
    country_code: "GU",
    region_code: "GU-GU",
};

pub const GT_AV: Subdivision = Subdivision {
    name: "Alta Verapaz",
    code: "GT-AV",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-AV",
};

pub const GT_BV: Subdivision = Subdivision {
    name: "Baja Verapaz",
    code: "GT-BV",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-BV",
};

pub const GT_CM: Subdivision = Subdivision {
    name: "Chimaltenango",
    code: "GT-CM",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-CM",
};

pub const GT_CQ: Subdivision = Subdivision {
    name: "Chiquimula",
    code: "GT-CQ",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-CQ",
};

pub const GT_ES: Subdivision = Subdivision {
    name: "Escuintla",
    code: "GT-ES",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-ES",
};

pub const GT_GU: Subdivision = Subdivision {
    name: "Guatemala",
    code: "GT-GU",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-GU",
};

pub const GT_HU: Subdivision = Subdivision {
    name: "Huehuetenango",
    code: "GT-HU",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-HU",
};

pub const GT_IZ: Subdivision = Subdivision {
    name: "Izabal",
    code: "GT-IZ",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-IZ",
};

pub const GT_JA: Subdivision = Subdivision {
    name: "Jalapa",
    code: "GT-JA",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-JA",
};

pub const GT_JU: Subdivision = Subdivision {
    name: "Jutiapa",
    code: "GT-JU",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-JU",
};

pub const GT_PE: Subdivision = Subdivision {
    name: "Petén",
    code: "GT-PE",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-PE",
};

pub const GT_PR: Subdivision = Subdivision {
    name: "El Progreso",
    code: "GT-PR",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-PR",
};

pub const GT_QC: Subdivision = Subdivision {
    name: "Quiché",
    code: "GT-QC",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-QC",
};

pub const GT_QZ: Subdivision = Subdivision {
    name: "Quetzaltenango",
    code: "GT-QZ",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-QZ",
};

pub const GT_RE: Subdivision = Subdivision {
    name: "Retalhuleu",
    code: "GT-RE",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-RE",
};

pub const GT_SA: Subdivision = Subdivision {
    name: "Sacatepéquez",
    code: "GT-SA",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-SA",
};

pub const GT_SM: Subdivision = Subdivision {
    name: "San Marcos",
    code: "GT-SM",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-SM",
};

pub const GT_SO: Subdivision = Subdivision {
    name: "Sololá",
    code: "GT-SO",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-SO",
};

pub const GT_SR: Subdivision = Subdivision {
    name: "Santa Rosa",
    code: "GT-SR",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-SR",
};

pub const GT_SU: Subdivision = Subdivision {
    name: "Suchitepéquez",
    code: "GT-SU",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-SU",
};

pub const GT_TO: Subdivision = Subdivision {
    name: "Totonicapán",
    code: "GT-TO",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-TO",
};

pub const GT_ZA: Subdivision = Subdivision {
    name: "Zacapa",
    code: "GT-ZA",
    subdivision_type: "Department",
    country_name: "Guatemala",
    country_code: "GT",
    region_code: "GT-ZA",
};

pub const GG_GG: Subdivision = Subdivision {
    name: "Guernsey",
    code: "GG-GG",
    subdivision_type: "Country",
    country_name: "Guernsey",
    country_code: "GG",
    region_code: "GG-GG",
};

pub const GW_BA: Subdivision = Subdivision {
    name: "Bafatá",
    code: "GW-BA",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-BA",
};

pub const GW_BL: Subdivision = Subdivision {
    name: "Bolama",
    code: "GW-BL",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-BL",
};

pub const GW_BM: Subdivision = Subdivision {
    name: "Biombo",
    code: "GW-BM",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-BM",
};

pub const GW_BS: Subdivision = Subdivision {
    name: "Bissau",
    code: "GW-BS",
    subdivision_type: "Autonomous sector",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-BS",
};

pub const GW_CA: Subdivision = Subdivision {
    name: "Cacheu",
    code: "GW-CA",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-CA",
};

pub const GW_GA: Subdivision = Subdivision {
    name: "Gabú",
    code: "GW-GA",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-GA",
};

pub const GW_L: Subdivision = Subdivision {
    name: "Leste",
    code: "GW-L",
    subdivision_type: "Province",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-L",
};

pub const GW_N: Subdivision = Subdivision {
    name: "Norte",
    code: "GW-N",
    subdivision_type: "Province",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-N",
};

pub const GW_OI: Subdivision = Subdivision {
    name: "Oio",
    code: "GW-OI",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-OI",
};

pub const GW_QU: Subdivision = Subdivision {
    name: "Quinara",
    code: "GW-QU",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-QU",
};

pub const GW_S: Subdivision = Subdivision {
    name: "Sul",
    code: "GW-S",
    subdivision_type: "Province",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-S",
};

pub const GW_TO: Subdivision = Subdivision {
    name: "Tombali",
    code: "GW-TO",
    subdivision_type: "Region",
    country_name: "Guinea-Bissau",
    country_code: "GW",
    region_code: "GW-TO",
};

pub const GN_B: Subdivision = Subdivision {
    name: "Boké",
    code: "GN-B",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-B",
};

pub const GN_BE: Subdivision = Subdivision {
    name: "Beyla",
    code: "GN-BE",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-BE",
};

pub const GN_BF: Subdivision = Subdivision {
    name: "Boffa",
    code: "GN-BF",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-BF",
};

pub const GN_BK: Subdivision = Subdivision {
    name: "Boké",
    code: "GN-BK",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-BK",
};

pub const GN_C: Subdivision = Subdivision {
    name: "Conakry",
    code: "GN-C",
    subdivision_type: "Governorate",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-C",
};

pub const GN_CO: Subdivision = Subdivision {
    name: "Coyah",
    code: "GN-CO",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-CO",
};

pub const GN_DB: Subdivision = Subdivision {
    name: "Dabola",
    code: "GN-DB",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-DB",
};

pub const GN_DI: Subdivision = Subdivision {
    name: "Dinguiraye",
    code: "GN-DI",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-DI",
};

pub const GN_D: Subdivision = Subdivision {
    name: "Kindia",
    code: "GN-D",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-D",
};

pub const GN_DL: Subdivision = Subdivision {
    name: "Dalaba",
    code: "GN-DL",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-DL",
};

pub const GN_DU: Subdivision = Subdivision {
    name: "Dubréka",
    code: "GN-DU",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-DU",
};

pub const GN_FA: Subdivision = Subdivision {
    name: "Faranah",
    code: "GN-FA",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-FA",
};

pub const GN_F: Subdivision = Subdivision {
    name: "Faranah",
    code: "GN-F",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-F",
};

pub const GN_FO: Subdivision = Subdivision {
    name: "Forécariah",
    code: "GN-FO",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-FO",
};

pub const GN_FR: Subdivision = Subdivision {
    name: "Fria",
    code: "GN-FR",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-FR",
};

pub const GN_GA: Subdivision = Subdivision {
    name: "Gaoual",
    code: "GN-GA",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-GA",
};

pub const GN_GU: Subdivision = Subdivision {
    name: "Guékédou",
    code: "GN-GU",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-GU",
};

pub const GN_KA: Subdivision = Subdivision {
    name: "Kankan",
    code: "GN-KA",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KA",
};

pub const GN_KB: Subdivision = Subdivision {
    name: "Koubia",
    code: "GN-KB",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KB",
};

pub const GN_KD: Subdivision = Subdivision {
    name: "Kindia",
    code: "GN-KD",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KD",
};

pub const GN_KE: Subdivision = Subdivision {
    name: "Kérouané",
    code: "GN-KE",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KE",
};

pub const GN_K: Subdivision = Subdivision {
    name: "Kankan",
    code: "GN-K",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-K",
};

pub const GN_KN: Subdivision = Subdivision {
    name: "Koundara",
    code: "GN-KN",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KN",
};

pub const GN_KO: Subdivision = Subdivision {
    name: "Kouroussa",
    code: "GN-KO",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KO",
};

pub const GN_KS: Subdivision = Subdivision {
    name: "Kissidougou",
    code: "GN-KS",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-KS",
};

pub const GN_LA: Subdivision = Subdivision {
    name: "Labé",
    code: "GN-LA",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-LA",
};

pub const GN_LE: Subdivision = Subdivision {
    name: "Lélouma",
    code: "GN-LE",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-LE",
};

pub const GN_L: Subdivision = Subdivision {
    name: "Labé",
    code: "GN-L",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-L",
};

pub const GN_LO: Subdivision = Subdivision {
    name: "Lola",
    code: "GN-LO",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-LO",
};

pub const GN_MC: Subdivision = Subdivision {
    name: "Macenta",
    code: "GN-MC",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-MC",
};

pub const GN_MD: Subdivision = Subdivision {
    name: "Mandiana",
    code: "GN-MD",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-MD",
};

pub const GN_ML: Subdivision = Subdivision {
    name: "Mali",
    code: "GN-ML",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-ML",
};

pub const GN_M: Subdivision = Subdivision {
    name: "Mamou",
    code: "GN-M",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-M",
};

pub const GN_MM: Subdivision = Subdivision {
    name: "Mamou",
    code: "GN-MM",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-MM",
};

pub const GN_N: Subdivision = Subdivision {
    name: "Nzérékoré",
    code: "GN-N",
    subdivision_type: "Administrative region",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-N",
};

pub const GN_NZ: Subdivision = Subdivision {
    name: "Nzérékoré",
    code: "GN-NZ",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-NZ",
};

pub const GN_PI: Subdivision = Subdivision {
    name: "Pita",
    code: "GN-PI",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-PI",
};

pub const GN_SI: Subdivision = Subdivision {
    name: "Siguiri",
    code: "GN-SI",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-SI",
};

pub const GN_TE: Subdivision = Subdivision {
    name: "Télimélé",
    code: "GN-TE",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-TE",
};

pub const GN_TO: Subdivision = Subdivision {
    name: "Tougué",
    code: "GN-TO",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-TO",
};

pub const GN_YO: Subdivision = Subdivision {
    name: "Yomou",
    code: "GN-YO",
    subdivision_type: "Prefecture",
    country_name: "Guinea",
    country_code: "GN",
    region_code: "GN-YO",
};

pub const GY_BA: Subdivision = Subdivision {
    name: "Barima-Waini",
    code: "GY-BA",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-BA",
};

pub const GY_CU: Subdivision = Subdivision {
    name: "Cuyuni-Mazaruni",
    code: "GY-CU",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-CU",
};

pub const GY_DE: Subdivision = Subdivision {
    name: "Demerara-Mahaica",
    code: "GY-DE",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-DE",
};

pub const GY_EB: Subdivision = Subdivision {
    name: "East Berbice-Corentyne",
    code: "GY-EB",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-EB",
};

pub const GY_ES: Subdivision = Subdivision {
    name: "Essequibo Islands-West Demerara",
    code: "GY-ES",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-ES",
};

pub const GY_MA: Subdivision = Subdivision {
    name: "Mahaica-Berbice",
    code: "GY-MA",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-MA",
};

pub const GY_PM: Subdivision = Subdivision {
    name: "Pomeroon-Supenaam",
    code: "GY-PM",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-PM",
};

pub const GY_PT: Subdivision = Subdivision {
    name: "Potaro-Siparuni",
    code: "GY-PT",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-PT",
};

pub const GY_UD: Subdivision = Subdivision {
    name: "Upper Demerara-Berbice",
    code: "GY-UD",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-UD",
};

pub const GY_UT: Subdivision = Subdivision {
    name: "Upper Takutu-Upper Essequibo",
    code: "GY-UT",
    subdivision_type: "Region",
    country_name: "Guyana",
    country_code: "GY",
    region_code: "GY-UT",
};

pub const HT_AR: Subdivision = Subdivision {
    name: "Artibonite",
    code: "HT-AR",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-AR",
};

pub const HT_CE: Subdivision = Subdivision {
    name: "Centre",
    code: "HT-CE",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-CE",
};

pub const HT_GA: Subdivision = Subdivision {
    name: "Grandans",
    code: "HT-GA",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-GA",
};

pub const HT_ND: Subdivision = Subdivision {
    name: "Nò",
    code: "HT-ND",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-ND",
};

pub const HT_NE: Subdivision = Subdivision {
    name: "Nòdès",
    code: "HT-NE",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-NE",
};

pub const HT_NI: Subdivision = Subdivision {
    name: "Nip",
    code: "HT-NI",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-NI",
};

pub const HT_NO: Subdivision = Subdivision {
    name: "Nòdwès",
    code: "HT-NO",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-NO",
};

pub const HT_OU: Subdivision = Subdivision {
    name: "Lwès",
    code: "HT-OU",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-OU",
};

pub const HT_SD: Subdivision = Subdivision {
    name: "Sid",
    code: "HT-SD",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-SD",
};

pub const HT_SE: Subdivision = Subdivision {
    name: "Sidès",
    code: "HT-SE",
    subdivision_type: "Department",
    country_name: "Haiti",
    country_code: "HT",
    region_code: "HT-SE",
};

pub const HM_HM: Subdivision = Subdivision {
    name: "Heard & McDonald Islands",
    code: "HM-HM",
    subdivision_type: "Country",
    country_name: "Heard & McDonald Islands",
    country_code: "HM",
    region_code: "HM-HM",
};

pub const HN_AT: Subdivision = Subdivision {
    name: "Atlántida",
    code: "HN-AT",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-AT",
};

pub const HN_CH: Subdivision = Subdivision {
    name: "Choluteca",
    code: "HN-CH",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-CH",
};

pub const HN_CL: Subdivision = Subdivision {
    name: "Colón",
    code: "HN-CL",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-CL",
};

pub const HN_CM: Subdivision = Subdivision {
    name: "Comayagua",
    code: "HN-CM",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-CM",
};

pub const HN_CP: Subdivision = Subdivision {
    name: "Copán",
    code: "HN-CP",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-CP",
};

pub const HN_CR: Subdivision = Subdivision {
    name: "Cortés",
    code: "HN-CR",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-CR",
};

pub const HN_EP: Subdivision = Subdivision {
    name: "El Paraíso",
    code: "HN-EP",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-EP",
};

pub const HN_FM: Subdivision = Subdivision {
    name: "Francisco Morazán",
    code: "HN-FM",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-FM",
};

pub const HN_GD: Subdivision = Subdivision {
    name: "Gracias a Dios",
    code: "HN-GD",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-GD",
};

pub const HN_IB: Subdivision = Subdivision {
    name: "Islas de la Bahía",
    code: "HN-IB",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-IB",
};

pub const HN_IN: Subdivision = Subdivision {
    name: "Intibucá",
    code: "HN-IN",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-IN",
};

pub const HN_LE: Subdivision = Subdivision {
    name: "Lempira",
    code: "HN-LE",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-LE",
};

pub const HN_LP: Subdivision = Subdivision {
    name: "La Paz",
    code: "HN-LP",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-LP",
};

pub const HN_OC: Subdivision = Subdivision {
    name: "Ocotepeque",
    code: "HN-OC",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-OC",
};

pub const HN_OL: Subdivision = Subdivision {
    name: "Olancho",
    code: "HN-OL",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-OL",
};

pub const HN_SB: Subdivision = Subdivision {
    name: "Santa Bárbara",
    code: "HN-SB",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-SB",
};

pub const HN_VA: Subdivision = Subdivision {
    name: "Valle",
    code: "HN-VA",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-VA",
};

pub const HN_YO: Subdivision = Subdivision {
    name: "Yoro",
    code: "HN-YO",
    subdivision_type: "Department",
    country_name: "Honduras",
    country_code: "HN",
    region_code: "HN-YO",
};

pub const HK_HK: Subdivision = Subdivision {
    name: "Hong Kong",
    code: "HK-HK",
    subdivision_type: "Country",
    country_name: "Hong Kong",
    country_code: "HK",
    region_code: "HK-HK",
};

pub const HU_BA: Subdivision = Subdivision {
    name: "Baranya",
    code: "HU-BA",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-BA",
};

pub const HU_BC: Subdivision = Subdivision {
    name: "Békéscsaba",
    code: "HU-BC",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-BC",
};

pub const HU_BE: Subdivision = Subdivision {
    name: "Békés",
    code: "HU-BE",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-BE",
};

pub const HU_BK: Subdivision = Subdivision {
    name: "Bács-Kiskun",
    code: "HU-BK",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-BK",
};

pub const HU_BU: Subdivision = Subdivision {
    name: "Budapest",
    code: "HU-BU",
    subdivision_type: "Capital city",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-BU",
};

pub const HU_BZ: Subdivision = Subdivision {
    name: "Borsod-Abaúj-Zemplén",
    code: "HU-BZ",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-BZ",
};

pub const HU_CS: Subdivision = Subdivision {
    name: "Csongrád",
    code: "HU-CS",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-CS",
};

pub const HU_DE: Subdivision = Subdivision {
    name: "Debrecen",
    code: "HU-DE",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-DE",
};

pub const HU_DU: Subdivision = Subdivision {
    name: "Dunaújváros",
    code: "HU-DU",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-DU",
};

pub const HU_EG: Subdivision = Subdivision {
    name: "Eger",
    code: "HU-EG",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-EG",
};

pub const HU_ER: Subdivision = Subdivision {
    name: "Érd",
    code: "HU-ER",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-ER",
};

pub const HU_FE: Subdivision = Subdivision {
    name: "Fejér",
    code: "HU-FE",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-FE",
};

pub const HU_GS: Subdivision = Subdivision {
    name: "Győr-Moson-Sopron",
    code: "HU-GS",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-GS",
};

pub const HU_GY: Subdivision = Subdivision {
    name: "Győr",
    code: "HU-GY",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-GY",
};

pub const HU_HB: Subdivision = Subdivision {
    name: "Hajdú-Bihar",
    code: "HU-HB",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-HB",
};

pub const HU_HE: Subdivision = Subdivision {
    name: "Heves",
    code: "HU-HE",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-HE",
};

pub const HU_HV: Subdivision = Subdivision {
    name: "Hódmezővásárhely",
    code: "HU-HV",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-HV",
};

pub const HU_JN: Subdivision = Subdivision {
    name: "Jász-Nagykun-Szolnok",
    code: "HU-JN",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-JN",
};

pub const HU_KE: Subdivision = Subdivision {
    name: "Komárom-Esztergom",
    code: "HU-KE",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-KE",
};

pub const HU_KM: Subdivision = Subdivision {
    name: "Kecskemét",
    code: "HU-KM",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-KM",
};

pub const HU_KV: Subdivision = Subdivision {
    name: "Kaposvár",
    code: "HU-KV",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-KV",
};

pub const HU_MI: Subdivision = Subdivision {
    name: "Miskolc",
    code: "HU-MI",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-MI",
};

pub const HU_NK: Subdivision = Subdivision {
    name: "Nagykanizsa",
    code: "HU-NK",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-NK",
};

pub const HU_NO: Subdivision = Subdivision {
    name: "Nógrád",
    code: "HU-NO",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-NO",
};

pub const HU_NY: Subdivision = Subdivision {
    name: "Nyíregyháza",
    code: "HU-NY",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-NY",
};

pub const HU_PE: Subdivision = Subdivision {
    name: "Pest",
    code: "HU-PE",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-PE",
};

pub const HU_PS: Subdivision = Subdivision {
    name: "Pécs",
    code: "HU-PS",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-PS",
};

pub const HU_SD: Subdivision = Subdivision {
    name: "Szeged",
    code: "HU-SD",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SD",
};

pub const HU_SF: Subdivision = Subdivision {
    name: "Székesfehérvár",
    code: "HU-SF",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SF",
};

pub const HU_SH: Subdivision = Subdivision {
    name: "Szombathely",
    code: "HU-SH",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SH",
};

pub const HU_SK: Subdivision = Subdivision {
    name: "Szolnok",
    code: "HU-SK",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SK",
};

pub const HU_SN: Subdivision = Subdivision {
    name: "Sopron",
    code: "HU-SN",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SN",
};

pub const HU_SO: Subdivision = Subdivision {
    name: "Somogy",
    code: "HU-SO",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SO",
};

pub const HU_SS: Subdivision = Subdivision {
    name: "Szekszárd",
    code: "HU-SS",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SS",
};

pub const HU_ST: Subdivision = Subdivision {
    name: "Salgótarján",
    code: "HU-ST",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-ST",
};

pub const HU_SZ: Subdivision = Subdivision {
    name: "Szabolcs-Szatmár-Bereg",
    code: "HU-SZ",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-SZ",
};

pub const HU_TB: Subdivision = Subdivision {
    name: "Tatabánya",
    code: "HU-TB",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-TB",
};

pub const HU_TO: Subdivision = Subdivision {
    name: "Tolna",
    code: "HU-TO",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-TO",
};

pub const HU_VA: Subdivision = Subdivision {
    name: "Vas",
    code: "HU-VA",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-VA",
};

pub const HU_VE: Subdivision = Subdivision {
    name: "Veszprém",
    code: "HU-VE",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-VE",
};

pub const HU_VM: Subdivision = Subdivision {
    name: "Veszprém",
    code: "HU-VM",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-VM",
};

pub const HU_ZA: Subdivision = Subdivision {
    name: "Zala",
    code: "HU-ZA",
    subdivision_type: "County",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-ZA",
};

pub const HU_ZE: Subdivision = Subdivision {
    name: "Zalaegerszeg",
    code: "HU-ZE",
    subdivision_type: "City of county right",
    country_name: "Hungary",
    country_code: "HU",
    region_code: "HU-ZE",
};

pub const IS_0: Subdivision = Subdivision {
    name: "Reykjavík",
    code: "IS-0",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-0",
};

pub const IS_1: Subdivision = Subdivision {
    name: "Höfuðborgarsvæði utan Reykjavíkur",
    code: "IS-1",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-1",
};

pub const IS_2: Subdivision = Subdivision {
    name: "Suðurnes",
    code: "IS-2",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-2",
};

pub const IS_3: Subdivision = Subdivision {
    name: "Vesturland",
    code: "IS-3",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-3",
};

pub const IS_4: Subdivision = Subdivision {
    name: "Vestfirðir",
    code: "IS-4",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-4",
};

pub const IS_5: Subdivision = Subdivision {
    name: "Norðurland vestra",
    code: "IS-5",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-5",
};

pub const IS_6: Subdivision = Subdivision {
    name: "Norðurland eystra",
    code: "IS-6",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-6",
};

pub const IS_7: Subdivision = Subdivision {
    name: "Austurland",
    code: "IS-7",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-7",
};

pub const IS_8: Subdivision = Subdivision {
    name: "Suðurland",
    code: "IS-8",
    subdivision_type: "Region",
    country_name: "Iceland",
    country_code: "IS",
    region_code: "IS-8",
};

pub const IN_AN: Subdivision = Subdivision {
    name: "Andaman and Nicobar Islands",
    code: "IN-AN",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-AN",
};

pub const IN_AP: Subdivision = Subdivision {
    name: "Andhra Pradesh",
    code: "IN-AP",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-AP",
};

pub const IN_AR: Subdivision = Subdivision {
    name: "Arunachal Pradesh",
    code: "IN-AR",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-AR",
};

pub const IN_AS: Subdivision = Subdivision {
    name: "Assam",
    code: "IN-AS",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-AS",
};

pub const IN_BR: Subdivision = Subdivision {
    name: "Bihar",
    code: "IN-BR",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-BR",
};

pub const IN_CH: Subdivision = Subdivision {
    name: "Chandigarh",
    code: "IN-CH",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-CH",
};

pub const IN_CT: Subdivision = Subdivision {
    name: "Chhattisgarh",
    code: "IN-CT",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-CT",
};

pub const IN_DD: Subdivision = Subdivision {
    name: "Daman and Diu",
    code: "IN-DD",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-DD",
};

pub const IN_DL: Subdivision = Subdivision {
    name: "Delhi",
    code: "IN-DL",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-DL",
};

pub const IN_DN: Subdivision = Subdivision {
    name: "Dadra and Nagar Haveli",
    code: "IN-DN",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-DN",
};

pub const IN_GA: Subdivision = Subdivision {
    name: "Goa",
    code: "IN-GA",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-GA",
};

pub const IN_GJ: Subdivision = Subdivision {
    name: "Gujarat",
    code: "IN-GJ",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-GJ",
};

pub const IN_HP: Subdivision = Subdivision {
    name: "Himachal Pradesh",
    code: "IN-HP",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-HP",
};

pub const IN_HR: Subdivision = Subdivision {
    name: "Haryana",
    code: "IN-HR",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-HR",
};

pub const IN_JH: Subdivision = Subdivision {
    name: "Jharkhand",
    code: "IN-JH",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-JH",
};

pub const IN_JK: Subdivision = Subdivision {
    name: "Jammu and Kashmir",
    code: "IN-JK",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-JK",
};

pub const IN_KA: Subdivision = Subdivision {
    name: "Karnataka",
    code: "IN-KA",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-KA",
};

pub const IN_KL: Subdivision = Subdivision {
    name: "Kerala",
    code: "IN-KL",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-KL",
};

pub const IN_LD: Subdivision = Subdivision {
    name: "Lakshadweep",
    code: "IN-LD",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-LD",
};

pub const IN_MH: Subdivision = Subdivision {
    name: "Maharashtra",
    code: "IN-MH",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-MH",
};

pub const IN_ML: Subdivision = Subdivision {
    name: "Meghalaya",
    code: "IN-ML",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-ML",
};

pub const IN_MN: Subdivision = Subdivision {
    name: "Manipur",
    code: "IN-MN",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-MN",
};

pub const IN_MP: Subdivision = Subdivision {
    name: "Madhya Pradesh",
    code: "IN-MP",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-MP",
};

pub const IN_MZ: Subdivision = Subdivision {
    name: "Mizoram",
    code: "IN-MZ",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-MZ",
};

pub const IN_NL: Subdivision = Subdivision {
    name: "Nagaland",
    code: "IN-NL",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-NL",
};

pub const IN_OR: Subdivision = Subdivision {
    name: "Odisha",
    code: "IN-OR",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-OR",
};

pub const IN_PB: Subdivision = Subdivision {
    name: "Punjab",
    code: "IN-PB",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-PB",
};

pub const IN_PY: Subdivision = Subdivision {
    name: "Puducherry",
    code: "IN-PY",
    subdivision_type: "Union territory",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-PY",
};

pub const IN_RJ: Subdivision = Subdivision {
    name: "Rajasthan",
    code: "IN-RJ",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-RJ",
};

pub const IN_SK: Subdivision = Subdivision {
    name: "Sikkim",
    code: "IN-SK",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-SK",
};

pub const IN_TG: Subdivision = Subdivision {
    name: "Telangana",
    code: "IN-TG",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-TG",
};

pub const IN_TN: Subdivision = Subdivision {
    name: "Tamil Nadu",
    code: "IN-TN",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-TN",
};

pub const IN_TR: Subdivision = Subdivision {
    name: "Tripura",
    code: "IN-TR",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-TR",
};

pub const IN_UP: Subdivision = Subdivision {
    name: "Uttar Pradesh",
    code: "IN-UP",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-UP",
};

pub const IN_UT: Subdivision = Subdivision {
    name: "Uttarakhand",
    code: "IN-UT",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-UT",
};

pub const IN_WB: Subdivision = Subdivision {
    name: "West Bengal",
    code: "IN-WB",
    subdivision_type: "State",
    country_name: "India",
    country_code: "IN",
    region_code: "IN-WB",
};

pub const ID_AC: Subdivision = Subdivision {
    name: "Aceh",
    code: "ID-AC",
    subdivision_type: "Autonomous province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-AC",
};

pub const ID_BA: Subdivision = Subdivision {
    name: "Bali",
    code: "ID-BA",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-BA",
};

pub const ID_BB: Subdivision = Subdivision {
    name: "Bangka Belitung",
    code: "ID-BB",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-BB",
};

pub const ID_BE: Subdivision = Subdivision {
    name: "Bengkulu",
    code: "ID-BE",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-BE",
};

pub const ID_BT: Subdivision = Subdivision {
    name: "Banten",
    code: "ID-BT",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-BT",
};

pub const ID_GO: Subdivision = Subdivision {
    name: "Gorontalo",
    code: "ID-GO",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-GO",
};

pub const ID_JA: Subdivision = Subdivision {
    name: "Jambi",
    code: "ID-JA",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-JA",
};

pub const ID_JB: Subdivision = Subdivision {
    name: "Jawa Barat",
    code: "ID-JB",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-JB",
};

pub const ID_JI: Subdivision = Subdivision {
    name: "Jawa Timur",
    code: "ID-JI",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-JI",
};

pub const ID_JK: Subdivision = Subdivision {
    name: "Jakarta Raya",
    code: "ID-JK",
    subdivision_type: "Special district",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-JK",
};

pub const ID_JT: Subdivision = Subdivision {
    name: "Jawa Tengah",
    code: "ID-JT",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-JT",
};

pub const ID_JW: Subdivision = Subdivision {
    name: "Jawa",
    code: "ID-JW",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-JW",
};

pub const ID_KA: Subdivision = Subdivision {
    name: "Kalimantan",
    code: "ID-KA",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KA",
};

pub const ID_KB: Subdivision = Subdivision {
    name: "Kalimantan Barat",
    code: "ID-KB",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KB",
};

pub const ID_KI: Subdivision = Subdivision {
    name: "Kalimantan Timur",
    code: "ID-KI",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KI",
};

pub const ID_KR: Subdivision = Subdivision {
    name: "Kepulauan Riau",
    code: "ID-KR",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KR",
};

pub const ID_KS: Subdivision = Subdivision {
    name: "Kalimantan Selatan",
    code: "ID-KS",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KS",
};

pub const ID_KT: Subdivision = Subdivision {
    name: "Kalimantan Tengah",
    code: "ID-KT",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KT",
};

pub const ID_KU: Subdivision = Subdivision {
    name: "Kalimantan Utara",
    code: "ID-KU",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-KU",
};

pub const ID_LA: Subdivision = Subdivision {
    name: "Lampung",
    code: "ID-LA",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-LA",
};

pub const ID_MA: Subdivision = Subdivision {
    name: "Maluku",
    code: "ID-MA",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-MA",
};

pub const ID_ML: Subdivision = Subdivision {
    name: "Maluku",
    code: "ID-ML",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-ML",
};

pub const ID_MU: Subdivision = Subdivision {
    name: "Maluku Utara",
    code: "ID-MU",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-MU",
};

pub const ID_NB: Subdivision = Subdivision {
    name: "Nusa Tenggara Barat",
    code: "ID-NB",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-NB",
};

pub const ID_NT: Subdivision = Subdivision {
    name: "Nusa Tenggara Timur",
    code: "ID-NT",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-NT",
};

pub const ID_NU: Subdivision = Subdivision {
    name: "Nusa Tenggara",
    code: "ID-NU",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-NU",
};

pub const ID_PA: Subdivision = Subdivision {
    name: "Papua",
    code: "ID-PA",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-PA",
};

pub const ID_PB: Subdivision = Subdivision {
    name: "Papua Barat",
    code: "ID-PB",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-PB",
};

pub const ID_PP: Subdivision = Subdivision {
    name: "Papua",
    code: "ID-PP",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-PP",
};

pub const ID_RI: Subdivision = Subdivision {
    name: "Riau",
    code: "ID-RI",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-RI",
};

pub const ID_SA: Subdivision = Subdivision {
    name: "Sulawesi Utara",
    code: "ID-SA",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SA",
};

pub const ID_SB: Subdivision = Subdivision {
    name: "Sumatera Barat",
    code: "ID-SB",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SB",
};

pub const ID_SG: Subdivision = Subdivision {
    name: "Sulawesi Tenggara",
    code: "ID-SG",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SG",
};

pub const ID_SL: Subdivision = Subdivision {
    name: "Sulawesi",
    code: "ID-SL",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SL",
};

pub const ID_SM: Subdivision = Subdivision {
    name: "Sumatera",
    code: "ID-SM",
    subdivision_type: "Geographical unit",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SM",
};

pub const ID_SN: Subdivision = Subdivision {
    name: "Sulawesi Selatan",
    code: "ID-SN",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SN",
};

pub const ID_SR: Subdivision = Subdivision {
    name: "Sulawesi Barat",
    code: "ID-SR",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SR",
};

pub const ID_SS: Subdivision = Subdivision {
    name: "Sumatera Selatan",
    code: "ID-SS",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SS",
};

pub const ID_ST: Subdivision = Subdivision {
    name: "Sulawesi Tengah",
    code: "ID-ST",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-ST",
};

pub const ID_SU: Subdivision = Subdivision {
    name: "Sumatera Utara",
    code: "ID-SU",
    subdivision_type: "Province",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-SU",
};

pub const ID_YO: Subdivision = Subdivision {
    name: "Yogyakarta",
    code: "ID-YO",
    subdivision_type: "Special region",
    country_name: "Indonesia",
    country_code: "ID",
    region_code: "ID-YO",
};

pub const IR_01: Subdivision = Subdivision {
    name: "Āz̄arbāyjān-e Sharqī",
    code: "IR-01",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-01",
};

pub const IR_02: Subdivision = Subdivision {
    name: "Āz̄arbāyjān-e Gharbī",
    code: "IR-02",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-02",
};

pub const IR_03: Subdivision = Subdivision {
    name: "Ardabīl",
    code: "IR-03",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-03",
};

pub const IR_04: Subdivision = Subdivision {
    name: "Eşfahān",
    code: "IR-04",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-04",
};

pub const IR_05: Subdivision = Subdivision {
    name: "Īlām",
    code: "IR-05",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-05",
};

pub const IR_06: Subdivision = Subdivision {
    name: "Būshehr",
    code: "IR-06",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-06",
};

pub const IR_07: Subdivision = Subdivision {
    name: "Tehrān",
    code: "IR-07",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-07",
};

pub const IR_08: Subdivision = Subdivision {
    name: "Chahār Maḩāll va Bakhtīārī",
    code: "IR-08",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-08",
};

pub const IR_10: Subdivision = Subdivision {
    name: "Khūzestān",
    code: "IR-10",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-10",
};

pub const IR_11: Subdivision = Subdivision {
    name: "Zanjān",
    code: "IR-11",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-11",
};

pub const IR_12: Subdivision = Subdivision {
    name: "Semnān",
    code: "IR-12",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-12",
};

pub const IR_13: Subdivision = Subdivision {
    name: "Sīstān va Balūchestān",
    code: "IR-13",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-13",
};

pub const IR_14: Subdivision = Subdivision {
    name: "Fārs",
    code: "IR-14",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-14",
};

pub const IR_15: Subdivision = Subdivision {
    name: "Kermān",
    code: "IR-15",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-15",
};

pub const IR_16: Subdivision = Subdivision {
    name: "Kordestān",
    code: "IR-16",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-16",
};

pub const IR_17: Subdivision = Subdivision {
    name: "Kermānshāh",
    code: "IR-17",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-17",
};

pub const IR_18: Subdivision = Subdivision {
    name: "Kohgīlūyeh va Būyer Aḩmad",
    code: "IR-18",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-18",
};

pub const IR_19: Subdivision = Subdivision {
    name: "Gīlān",
    code: "IR-19",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-19",
};

pub const IR_20: Subdivision = Subdivision {
    name: "Lorestān",
    code: "IR-20",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-20",
};

pub const IR_21: Subdivision = Subdivision {
    name: "Māzandarān",
    code: "IR-21",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-21",
};

pub const IR_22: Subdivision = Subdivision {
    name: "Markazī",
    code: "IR-22",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-22",
};

pub const IR_23: Subdivision = Subdivision {
    name: "Hormozgān",
    code: "IR-23",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-23",
};

pub const IR_24: Subdivision = Subdivision {
    name: "Hamadān",
    code: "IR-24",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-24",
};

pub const IR_25: Subdivision = Subdivision {
    name: "Yazd",
    code: "IR-25",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-25",
};

pub const IR_26: Subdivision = Subdivision {
    name: "Qom",
    code: "IR-26",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-26",
};

pub const IR_27: Subdivision = Subdivision {
    name: "Golestān",
    code: "IR-27",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-27",
};

pub const IR_28: Subdivision = Subdivision {
    name: "Qazvīn",
    code: "IR-28",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-28",
};

pub const IR_29: Subdivision = Subdivision {
    name: "Khorāsān-e Janūbī",
    code: "IR-29",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-29",
};

pub const IR_30: Subdivision = Subdivision {
    name: "Khorāsān-e Razavī",
    code: "IR-30",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-30",
};

pub const IR_31: Subdivision = Subdivision {
    name: "Khorāsān-e Shemālī",
    code: "IR-31",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-31",
};

pub const IR_32: Subdivision = Subdivision {
    name: "Alborz",
    code: "IR-32",
    subdivision_type: "Province",
    country_name: "Iran",
    country_code: "IR",
    region_code: "IR-32",
};

pub const IQ_AN: Subdivision = Subdivision {
    name: "Al Anbār",
    code: "IQ-AN",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-AN",
};

pub const IQ_AR: Subdivision = Subdivision {
    name: "Arbīl",
    code: "IQ-AR",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-AR",
};

pub const IQ_BA: Subdivision = Subdivision {
    name: "Al Başrah",
    code: "IQ-BA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-BA",
};

pub const IQ_BB: Subdivision = Subdivision {
    name: "Bābil",
    code: "IQ-BB",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-BB",
};

pub const IQ_BG: Subdivision = Subdivision {
    name: "Baghdād",
    code: "IQ-BG",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-BG",
};

pub const IQ_DA: Subdivision = Subdivision {
    name: "Dahūk",
    code: "IQ-DA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-DA",
};

pub const IQ_DI: Subdivision = Subdivision {
    name: "Diyālá",
    code: "IQ-DI",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-DI",
};

pub const IQ_DQ: Subdivision = Subdivision {
    name: "Dhī Qār",
    code: "IQ-DQ",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-DQ",
};

pub const IQ_KA: Subdivision = Subdivision {
    name: "Karbalā'",
    code: "IQ-KA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-KA",
};

pub const IQ_KI: Subdivision = Subdivision {
    name: "Kirkūk",
    code: "IQ-KI",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-KI",
};

pub const IQ_MA: Subdivision = Subdivision {
    name: "Maysān",
    code: "IQ-MA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-MA",
};

pub const IQ_MU: Subdivision = Subdivision {
    name: "Al Muthanná",
    code: "IQ-MU",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-MU",
};

pub const IQ_NA: Subdivision = Subdivision {
    name: "An Najaf",
    code: "IQ-NA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-NA",
};

pub const IQ_NI: Subdivision = Subdivision {
    name: "Nīnawá",
    code: "IQ-NI",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-NI",
};

pub const IQ_QA: Subdivision = Subdivision {
    name: "Al Qādisīyah",
    code: "IQ-QA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-QA",
};

pub const IQ_SD: Subdivision = Subdivision {
    name: "Şalāḩ ad Dīn",
    code: "IQ-SD",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-SD",
};

pub const IQ_SU: Subdivision = Subdivision {
    name: "As Sulaymānīyah",
    code: "IQ-SU",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-SU",
};

pub const IQ_WA: Subdivision = Subdivision {
    name: "Wāsiţ",
    code: "IQ-WA",
    subdivision_type: "Governorate",
    country_name: "Iraq",
    country_code: "IQ",
    region_code: "IQ-WA",
};

pub const IE_C: Subdivision = Subdivision {
    name: "Connacht",
    code: "IE-C",
    subdivision_type: "Province",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-C",
};

pub const IE_CE: Subdivision = Subdivision {
    name: "An Clár",
    code: "IE-CE",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-CE",
};

pub const IE_CN: Subdivision = Subdivision {
    name: "An Cabhán",
    code: "IE-CN",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-CN",
};

pub const IE_CO: Subdivision = Subdivision {
    name: "Corcaigh",
    code: "IE-CO",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-CO",
};

pub const IE_CW: Subdivision = Subdivision {
    name: "Carlow",
    code: "IE-CW",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-CW",
};

pub const IE_D: Subdivision = Subdivision {
    name: "Baile Átha Cliath",
    code: "IE-D",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-D",
};

pub const IE_DL: Subdivision = Subdivision {
    name: "Donegal",
    code: "IE-DL",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-DL",
};

pub const IE_G: Subdivision = Subdivision {
    name: "Gaillimh",
    code: "IE-G",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-G",
};

pub const IE_KE: Subdivision = Subdivision {
    name: "Cill Dara",
    code: "IE-KE",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-KE",
};

pub const IE_KK: Subdivision = Subdivision {
    name: "Cill Chainnigh",
    code: "IE-KK",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-KK",
};

pub const IE_KY: Subdivision = Subdivision {
    name: "Ciarraí",
    code: "IE-KY",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-KY",
};

pub const IE_LD: Subdivision = Subdivision {
    name: "An Longfort",
    code: "IE-LD",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-LD",
};

pub const IE_LH: Subdivision = Subdivision {
    name: "Louth",
    code: "IE-LH",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-LH",
};

pub const IE_LK: Subdivision = Subdivision {
    name: "Limerick",
    code: "IE-LK",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-LK",
};

pub const IE_L: Subdivision = Subdivision {
    name: "Laighin",
    code: "IE-L",
    subdivision_type: "Province",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-L",
};

pub const IE_LM: Subdivision = Subdivision {
    name: "Leitrim",
    code: "IE-LM",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-LM",
};

pub const IE_LS: Subdivision = Subdivision {
    name: "Laois",
    code: "IE-LS",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-LS",
};

pub const IE_M: Subdivision = Subdivision {
    name: "An Mhumhain",
    code: "IE-M",
    subdivision_type: "Province",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-M",
};

pub const IE_MH: Subdivision = Subdivision {
    name: "An Mhí",
    code: "IE-MH",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-MH",
};

pub const IE_MN: Subdivision = Subdivision {
    name: "Monaghan",
    code: "IE-MN",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-MN",
};

pub const IE_MO: Subdivision = Subdivision {
    name: "Maigh Eo",
    code: "IE-MO",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-MO",
};

pub const IE_OY: Subdivision = Subdivision {
    name: "Offaly",
    code: "IE-OY",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-OY",
};

pub const IE_RN: Subdivision = Subdivision {
    name: "Ros Comáin",
    code: "IE-RN",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-RN",
};

pub const IE_SO: Subdivision = Subdivision {
    name: "Sligeach",
    code: "IE-SO",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-SO",
};

pub const IE_TA: Subdivision = Subdivision {
    name: "Tiobraid Árann",
    code: "IE-TA",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-TA",
};

pub const IE_U: Subdivision = Subdivision {
    name: "Ulaidh",
    code: "IE-U",
    subdivision_type: "Province",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-U",
};

pub const IE_WD: Subdivision = Subdivision {
    name: "Port Láirge",
    code: "IE-WD",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-WD",
};

pub const IE_WH: Subdivision = Subdivision {
    name: "An Iarmhí",
    code: "IE-WH",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-WH",
};

pub const IE_WW: Subdivision = Subdivision {
    name: "Cill Mhantáin",
    code: "IE-WW",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-WW",
};

pub const IE_WX: Subdivision = Subdivision {
    name: "Loch Garman",
    code: "IE-WX",
    subdivision_type: "County",
    country_name: "Ireland",
    country_code: "IE",
    region_code: "IE-WX",
};

pub const IM_IM: Subdivision = Subdivision {
    name: "Isle of Main",
    code: "IM-IM",
    subdivision_type: "Country",
    country_name: "Isle of Man",
    country_code: "IM",
    region_code: "IM-IM",
};

pub const IL_D: Subdivision = Subdivision {
    name: "Southern District",
    code: "IL-D",
    subdivision_type: "District",
    country_name: "Israel",
    country_code: "IL",
    region_code: "IL-D",
};

pub const IL_HA: Subdivision = Subdivision {
    name: "Haifa",
    code: "IL-HA",
    subdivision_type: "District",
    country_name: "Israel",
    country_code: "IL",
    region_code: "IL-HA",
};

pub const IL_JM: Subdivision = Subdivision {
    name: "Jerusalem",
    code: "IL-JM",
    subdivision_type: "District",
    country_name: "Israel",
    country_code: "IL",
    region_code: "IL-JM",
};

pub const IL_M: Subdivision = Subdivision {
    name: "Central District",
    code: "IL-M",
    subdivision_type: "District",
    country_name: "Israel",
    country_code: "IL",
    region_code: "IL-M",
};

pub const IL_TA: Subdivision = Subdivision {
    name: "Tel Aviv",
    code: "IL-TA",
    subdivision_type: "District",
    country_name: "Israel",
    country_code: "IL",
    region_code: "IL-TA",
};

pub const IL_Z: Subdivision = Subdivision {
    name: "Northern District",
    code: "IL-Z",
    subdivision_type: "District",
    country_name: "Israel",
    country_code: "IL",
    region_code: "IL-Z",
};

pub const IT_21: Subdivision = Subdivision {
    name: "Piemonte",
    code: "IT-21",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-21",
};

pub const IT_23: Subdivision = Subdivision {
    name: "Val d'Aoste",
    code: "IT-23",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-23",
};

pub const IT_25: Subdivision = Subdivision {
    name: "Lombardia",
    code: "IT-25",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-25",
};

pub const IT_32: Subdivision = Subdivision {
    name: "Trentino-Alto Adige",
    code: "IT-32",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-32",
};

pub const IT_34: Subdivision = Subdivision {
    name: "Veneto",
    code: "IT-34",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-34",
};

pub const IT_36: Subdivision = Subdivision {
    name: "Friuli-Venezia Giulia",
    code: "IT-36",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-36",
};

pub const IT_42: Subdivision = Subdivision {
    name: "Liguria",
    code: "IT-42",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-42",
};

pub const IT_45: Subdivision = Subdivision {
    name: "Emilia-Romagna",
    code: "IT-45",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-45",
};

pub const IT_52: Subdivision = Subdivision {
    name: "Toscana",
    code: "IT-52",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-52",
};

pub const IT_55: Subdivision = Subdivision {
    name: "Umbria",
    code: "IT-55",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-55",
};

pub const IT_57: Subdivision = Subdivision {
    name: "Marche",
    code: "IT-57",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-57",
};

pub const IT_62: Subdivision = Subdivision {
    name: "Lazio",
    code: "IT-62",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-62",
};

pub const IT_65: Subdivision = Subdivision {
    name: "Abruzzo",
    code: "IT-65",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-65",
};

pub const IT_67: Subdivision = Subdivision {
    name: "Molise",
    code: "IT-67",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-67",
};

pub const IT_72: Subdivision = Subdivision {
    name: "Campania",
    code: "IT-72",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-72",
};

pub const IT_75: Subdivision = Subdivision {
    name: "Puglia",
    code: "IT-75",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-75",
};

pub const IT_77: Subdivision = Subdivision {
    name: "Basilicata",
    code: "IT-77",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-77",
};

pub const IT_78: Subdivision = Subdivision {
    name: "Calabria",
    code: "IT-78",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-78",
};

pub const IT_82: Subdivision = Subdivision {
    name: "Sicilia",
    code: "IT-82",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-82",
};

pub const IT_88: Subdivision = Subdivision {
    name: "Sardegna",
    code: "IT-88",
    subdivision_type: "Region",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-88",
};

pub const IT_AG: Subdivision = Subdivision {
    name: "Agrigento",
    code: "IT-AG",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AG",
};

pub const IT_AL: Subdivision = Subdivision {
    name: "Alessandria",
    code: "IT-AL",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AL",
};

pub const IT_AN: Subdivision = Subdivision {
    name: "Ancona",
    code: "IT-AN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AN",
};

pub const IT_AO: Subdivision = Subdivision {
    name: "Aosta",
    code: "IT-AO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AO",
};

pub const IT_AP: Subdivision = Subdivision {
    name: "Ascoli Piceno",
    code: "IT-AP",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AP",
};

pub const IT_AQ: Subdivision = Subdivision {
    name: "L'Aquila",
    code: "IT-AQ",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AQ",
};

pub const IT_AR: Subdivision = Subdivision {
    name: "Arezzo",
    code: "IT-AR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AR",
};

pub const IT_AT: Subdivision = Subdivision {
    name: "Asti",
    code: "IT-AT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AT",
};

pub const IT_AV: Subdivision = Subdivision {
    name: "Avellino",
    code: "IT-AV",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-AV",
};

pub const IT_BA: Subdivision = Subdivision {
    name: "Bari",
    code: "IT-BA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BA",
};

pub const IT_BG: Subdivision = Subdivision {
    name: "Bergamo",
    code: "IT-BG",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BG",
};

pub const IT_BI: Subdivision = Subdivision {
    name: "Biella",
    code: "IT-BI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BI",
};

pub const IT_BL: Subdivision = Subdivision {
    name: "Belluno",
    code: "IT-BL",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BL",
};

pub const IT_BN: Subdivision = Subdivision {
    name: "Benevento",
    code: "IT-BN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BN",
};

pub const IT_BO: Subdivision = Subdivision {
    name: "Bologna",
    code: "IT-BO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BO",
};

pub const IT_BR: Subdivision = Subdivision {
    name: "Brindisi",
    code: "IT-BR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BR",
};

pub const IT_BS: Subdivision = Subdivision {
    name: "Brescia",
    code: "IT-BS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BS",
};

pub const IT_BT: Subdivision = Subdivision {
    name: "Barletta-Andria-Trani",
    code: "IT-BT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BT",
};

pub const IT_BZ: Subdivision = Subdivision {
    name: "Bolzano",
    code: "IT-BZ",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-BZ",
};

pub const IT_CA: Subdivision = Subdivision {
    name: "Cagliari",
    code: "IT-CA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CA",
};

pub const IT_CB: Subdivision = Subdivision {
    name: "Campobasso",
    code: "IT-CB",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CB",
};

pub const IT_CE: Subdivision = Subdivision {
    name: "Caserta",
    code: "IT-CE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CE",
};

pub const IT_CH: Subdivision = Subdivision {
    name: "Chieti",
    code: "IT-CH",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CH",
};

pub const IT_CI: Subdivision = Subdivision {
    name: "Carbonia-Iglesias",
    code: "IT-CI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CI",
};

pub const IT_CL: Subdivision = Subdivision {
    name: "Caltanissetta",
    code: "IT-CL",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CL",
};

pub const IT_CN: Subdivision = Subdivision {
    name: "Cuneo",
    code: "IT-CN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CN",
};

pub const IT_CO: Subdivision = Subdivision {
    name: "Como",
    code: "IT-CO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CO",
};

pub const IT_CR: Subdivision = Subdivision {
    name: "Cremona",
    code: "IT-CR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CR",
};

pub const IT_CS: Subdivision = Subdivision {
    name: "Cosenza",
    code: "IT-CS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CS",
};

pub const IT_CT: Subdivision = Subdivision {
    name: "Catania",
    code: "IT-CT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CT",
};

pub const IT_CZ: Subdivision = Subdivision {
    name: "Catanzaro",
    code: "IT-CZ",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-CZ",
};

pub const IT_EN: Subdivision = Subdivision {
    name: "Enna",
    code: "IT-EN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-EN",
};

pub const IT_FC: Subdivision = Subdivision {
    name: "Forlì-Cesena",
    code: "IT-FC",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-FC",
};

pub const IT_FE: Subdivision = Subdivision {
    name: "Ferrara",
    code: "IT-FE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-FE",
};

pub const IT_FG: Subdivision = Subdivision {
    name: "Foggia",
    code: "IT-FG",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-FG",
};

pub const IT_FI: Subdivision = Subdivision {
    name: "Firenze",
    code: "IT-FI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-FI",
};

pub const IT_FM: Subdivision = Subdivision {
    name: "Fermo",
    code: "IT-FM",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-FM",
};

pub const IT_FR: Subdivision = Subdivision {
    name: "Frosinone",
    code: "IT-FR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-FR",
};

pub const IT_GE: Subdivision = Subdivision {
    name: "Genova",
    code: "IT-GE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-GE",
};

pub const IT_GO: Subdivision = Subdivision {
    name: "Gorizia",
    code: "IT-GO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-GO",
};

pub const IT_GR: Subdivision = Subdivision {
    name: "Grosseto",
    code: "IT-GR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-GR",
};

pub const IT_IM: Subdivision = Subdivision {
    name: "Imperia",
    code: "IT-IM",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-IM",
};

pub const IT_IS: Subdivision = Subdivision {
    name: "Isernia",
    code: "IT-IS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-IS",
};

pub const IT_KR: Subdivision = Subdivision {
    name: "Crotone",
    code: "IT-KR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-KR",
};

pub const IT_LC: Subdivision = Subdivision {
    name: "Lecco",
    code: "IT-LC",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-LC",
};

pub const IT_LE: Subdivision = Subdivision {
    name: "Lecce",
    code: "IT-LE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-LE",
};

pub const IT_LI: Subdivision = Subdivision {
    name: "Livorno",
    code: "IT-LI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-LI",
};

pub const IT_LO: Subdivision = Subdivision {
    name: "Lodi",
    code: "IT-LO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-LO",
};

pub const IT_LT: Subdivision = Subdivision {
    name: "Latina",
    code: "IT-LT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-LT",
};

pub const IT_LU: Subdivision = Subdivision {
    name: "Lucca",
    code: "IT-LU",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-LU",
};

pub const IT_MB: Subdivision = Subdivision {
    name: "Monza e Brianza",
    code: "IT-MB",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MB",
};

pub const IT_MC: Subdivision = Subdivision {
    name: "Macerata",
    code: "IT-MC",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MC",
};

pub const IT_ME: Subdivision = Subdivision {
    name: "Messina",
    code: "IT-ME",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-ME",
};

pub const IT_MI: Subdivision = Subdivision {
    name: "Milano",
    code: "IT-MI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MI",
};

pub const IT_MN: Subdivision = Subdivision {
    name: "Mantova",
    code: "IT-MN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MN",
};

pub const IT_MO: Subdivision = Subdivision {
    name: "Modena",
    code: "IT-MO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MO",
};

pub const IT_MS: Subdivision = Subdivision {
    name: "Massa-Carrara",
    code: "IT-MS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MS",
};

pub const IT_MT: Subdivision = Subdivision {
    name: "Matera",
    code: "IT-MT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-MT",
};

pub const IT_NA: Subdivision = Subdivision {
    name: "Napoli",
    code: "IT-NA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-NA",
};

pub const IT_NO: Subdivision = Subdivision {
    name: "Novara",
    code: "IT-NO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-NO",
};

pub const IT_NU: Subdivision = Subdivision {
    name: "Nuoro",
    code: "IT-NU",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-NU",
};

pub const IT_OG: Subdivision = Subdivision {
    name: "Ogliastra",
    code: "IT-OG",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-OG",
};

pub const IT_OR: Subdivision = Subdivision {
    name: "Oristano",
    code: "IT-OR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-OR",
};

pub const IT_OT: Subdivision = Subdivision {
    name: "Olbia-Tempio",
    code: "IT-OT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-OT",
};

pub const IT_PA: Subdivision = Subdivision {
    name: "Palermo",
    code: "IT-PA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PA",
};

pub const IT_PC: Subdivision = Subdivision {
    name: "Piacenza",
    code: "IT-PC",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PC",
};

pub const IT_PD: Subdivision = Subdivision {
    name: "Padova",
    code: "IT-PD",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PD",
};

pub const IT_PE: Subdivision = Subdivision {
    name: "Pescara",
    code: "IT-PE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PE",
};

pub const IT_PG: Subdivision = Subdivision {
    name: "Perugia",
    code: "IT-PG",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PG",
};

pub const IT_PI: Subdivision = Subdivision {
    name: "Pisa",
    code: "IT-PI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PI",
};

pub const IT_PN: Subdivision = Subdivision {
    name: "Pordenone",
    code: "IT-PN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PN",
};

pub const IT_PO: Subdivision = Subdivision {
    name: "Prato",
    code: "IT-PO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PO",
};

pub const IT_PR: Subdivision = Subdivision {
    name: "Parma",
    code: "IT-PR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PR",
};

pub const IT_PT: Subdivision = Subdivision {
    name: "Pistoia",
    code: "IT-PT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PT",
};

pub const IT_PU: Subdivision = Subdivision {
    name: "Pesaro e Urbino",
    code: "IT-PU",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PU",
};

pub const IT_PV: Subdivision = Subdivision {
    name: "Pavia",
    code: "IT-PV",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PV",
};

pub const IT_PZ: Subdivision = Subdivision {
    name: "Potenza",
    code: "IT-PZ",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-PZ",
};

pub const IT_RA: Subdivision = Subdivision {
    name: "Ravenna",
    code: "IT-RA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RA",
};

pub const IT_RC: Subdivision = Subdivision {
    name: "Reggio Calabria",
    code: "IT-RC",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RC",
};

pub const IT_RE: Subdivision = Subdivision {
    name: "Reggio Emilia",
    code: "IT-RE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RE",
};

pub const IT_RG: Subdivision = Subdivision {
    name: "Ragusa",
    code: "IT-RG",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RG",
};

pub const IT_RI: Subdivision = Subdivision {
    name: "Rieti",
    code: "IT-RI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RI",
};

pub const IT_RM: Subdivision = Subdivision {
    name: "Roma",
    code: "IT-RM",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RM",
};

pub const IT_RN: Subdivision = Subdivision {
    name: "Rimini",
    code: "IT-RN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RN",
};

pub const IT_RO: Subdivision = Subdivision {
    name: "Rovigo",
    code: "IT-RO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-RO",
};

pub const IT_SA: Subdivision = Subdivision {
    name: "Salerno",
    code: "IT-SA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SA",
};

pub const IT_SI: Subdivision = Subdivision {
    name: "Siena",
    code: "IT-SI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SI",
};

pub const IT_SO: Subdivision = Subdivision {
    name: "Sondrio",
    code: "IT-SO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SO",
};

pub const IT_SP: Subdivision = Subdivision {
    name: "La Spezia",
    code: "IT-SP",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SP",
};

pub const IT_SR: Subdivision = Subdivision {
    name: "Siracusa",
    code: "IT-SR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SR",
};

pub const IT_SS: Subdivision = Subdivision {
    name: "Sassari",
    code: "IT-SS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SS",
};

pub const IT_SV: Subdivision = Subdivision {
    name: "Savona",
    code: "IT-SV",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-SV",
};

pub const IT_TA: Subdivision = Subdivision {
    name: "Taranto",
    code: "IT-TA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TA",
};

pub const IT_TE: Subdivision = Subdivision {
    name: "Teramo",
    code: "IT-TE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TE",
};

pub const IT_TN: Subdivision = Subdivision {
    name: "Trento",
    code: "IT-TN",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TN",
};

pub const IT_TO: Subdivision = Subdivision {
    name: "Torino",
    code: "IT-TO",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TO",
};

pub const IT_TP: Subdivision = Subdivision {
    name: "Trapani",
    code: "IT-TP",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TP",
};

pub const IT_TR: Subdivision = Subdivision {
    name: "Terni",
    code: "IT-TR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TR",
};

pub const IT_TS: Subdivision = Subdivision {
    name: "Trieste",
    code: "IT-TS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TS",
};

pub const IT_TV: Subdivision = Subdivision {
    name: "Treviso",
    code: "IT-TV",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-TV",
};

pub const IT_UD: Subdivision = Subdivision {
    name: "Udine",
    code: "IT-UD",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-UD",
};

pub const IT_VA: Subdivision = Subdivision {
    name: "Varese",
    code: "IT-VA",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VA",
};

pub const IT_VB: Subdivision = Subdivision {
    name: "Verbano-Cusio-Ossola",
    code: "IT-VB",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VB",
};

pub const IT_VC: Subdivision = Subdivision {
    name: "Vercelli",
    code: "IT-VC",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VC",
};

pub const IT_VE: Subdivision = Subdivision {
    name: "Venezia",
    code: "IT-VE",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VE",
};

pub const IT_VI: Subdivision = Subdivision {
    name: "Vicenza",
    code: "IT-VI",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VI",
};

pub const IT_VR: Subdivision = Subdivision {
    name: "Verona",
    code: "IT-VR",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VR",
};

pub const IT_VS: Subdivision = Subdivision {
    name: "Medio Campidano",
    code: "IT-VS",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VS",
};

pub const IT_VT: Subdivision = Subdivision {
    name: "Viterbo",
    code: "IT-VT",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VT",
};

pub const IT_VV: Subdivision = Subdivision {
    name: "Vibo Valentia",
    code: "IT-VV",
    subdivision_type: "Province",
    country_name: "Italy",
    country_code: "IT",
    region_code: "IT-VV",
};

pub const JM_01: Subdivision = Subdivision {
    name: "Kingston",
    code: "JM-01",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-01",
};

pub const JM_02: Subdivision = Subdivision {
    name: "Saint Andrew",
    code: "JM-02",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-02",
};

pub const JM_03: Subdivision = Subdivision {
    name: "Saint Thomas",
    code: "JM-03",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-03",
};

pub const JM_04: Subdivision = Subdivision {
    name: "Portland",
    code: "JM-04",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-04",
};

pub const JM_05: Subdivision = Subdivision {
    name: "Saint Mary",
    code: "JM-05",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-05",
};

pub const JM_06: Subdivision = Subdivision {
    name: "Saint Ann",
    code: "JM-06",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-06",
};

pub const JM_07: Subdivision = Subdivision {
    name: "Trelawny",
    code: "JM-07",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-07",
};

pub const JM_08: Subdivision = Subdivision {
    name: "Saint James",
    code: "JM-08",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-08",
};

pub const JM_09: Subdivision = Subdivision {
    name: "Hanover",
    code: "JM-09",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-09",
};

pub const JM_10: Subdivision = Subdivision {
    name: "Westmoreland",
    code: "JM-10",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-10",
};

pub const JM_11: Subdivision = Subdivision {
    name: "Saint Elizabeth",
    code: "JM-11",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-11",
};

pub const JM_12: Subdivision = Subdivision {
    name: "Manchester",
    code: "JM-12",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-12",
};

pub const JM_13: Subdivision = Subdivision {
    name: "Clarendon",
    code: "JM-13",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-13",
};

pub const JM_14: Subdivision = Subdivision {
    name: "Saint Catherine",
    code: "JM-14",
    subdivision_type: "Parish",
    country_name: "Jamaica",
    country_code: "JM",
    region_code: "JM-14",
};

pub const JP_01: Subdivision = Subdivision {
    name: "Hokkaido",
    code: "JP-01",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-01",
};

pub const JP_02: Subdivision = Subdivision {
    name: "Aomori",
    code: "JP-02",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-02",
};

pub const JP_03: Subdivision = Subdivision {
    name: "Iwate",
    code: "JP-03",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-03",
};

pub const JP_04: Subdivision = Subdivision {
    name: "Miyagi",
    code: "JP-04",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-04",
};

pub const JP_05: Subdivision = Subdivision {
    name: "Akita",
    code: "JP-05",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-05",
};

pub const JP_06: Subdivision = Subdivision {
    name: "Yamagata",
    code: "JP-06",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-06",
};

pub const JP_07: Subdivision = Subdivision {
    name: "Fukushima",
    code: "JP-07",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-07",
};

pub const JP_08: Subdivision = Subdivision {
    name: "Ibaraki",
    code: "JP-08",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-08",
};

pub const JP_09: Subdivision = Subdivision {
    name: "Tochigi",
    code: "JP-09",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-09",
};

pub const JP_10: Subdivision = Subdivision {
    name: "Gunma",
    code: "JP-10",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-10",
};

pub const JP_11: Subdivision = Subdivision {
    name: "Saitama",
    code: "JP-11",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-11",
};

pub const JP_12: Subdivision = Subdivision {
    name: "Chiba",
    code: "JP-12",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-12",
};

pub const JP_13: Subdivision = Subdivision {
    name: "Tôkyô",
    code: "JP-13",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-13",
};

pub const JP_14: Subdivision = Subdivision {
    name: "Kanagawa",
    code: "JP-14",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-14",
};

pub const JP_15: Subdivision = Subdivision {
    name: "Niigata",
    code: "JP-15",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-15",
};

pub const JP_16: Subdivision = Subdivision {
    name: "Toyama",
    code: "JP-16",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-16",
};

pub const JP_17: Subdivision = Subdivision {
    name: "Ishikawa",
    code: "JP-17",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-17",
};

pub const JP_18: Subdivision = Subdivision {
    name: "Fukui",
    code: "JP-18",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-18",
};

pub const JP_19: Subdivision = Subdivision {
    name: "Yamanashi",
    code: "JP-19",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-19",
};

pub const JP_20: Subdivision = Subdivision {
    name: "Nagano",
    code: "JP-20",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-20",
};

pub const JP_21: Subdivision = Subdivision {
    name: "Gifu",
    code: "JP-21",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-21",
};

pub const JP_22: Subdivision = Subdivision {
    name: "Shizuoka",
    code: "JP-22",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-22",
};

pub const JP_23: Subdivision = Subdivision {
    name: "Aichi",
    code: "JP-23",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-23",
};

pub const JP_24: Subdivision = Subdivision {
    name: "Mie",
    code: "JP-24",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-24",
};

pub const JP_25: Subdivision = Subdivision {
    name: "Shiga",
    code: "JP-25",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-25",
};

pub const JP_26: Subdivision = Subdivision {
    name: "Kyoto",
    code: "JP-26",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-26",
};

pub const JP_27: Subdivision = Subdivision {
    name: "Osaka",
    code: "JP-27",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-27",
};

pub const JP_28: Subdivision = Subdivision {
    name: "Hyogo",
    code: "JP-28",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-28",
};

pub const JP_29: Subdivision = Subdivision {
    name: "Nara",
    code: "JP-29",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-29",
};

pub const JP_30: Subdivision = Subdivision {
    name: "Wakayama",
    code: "JP-30",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-30",
};

pub const JP_31: Subdivision = Subdivision {
    name: "Tottori",
    code: "JP-31",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-31",
};

pub const JP_32: Subdivision = Subdivision {
    name: "Shimane",
    code: "JP-32",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-32",
};

pub const JP_33: Subdivision = Subdivision {
    name: "Okayama",
    code: "JP-33",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-33",
};

pub const JP_34: Subdivision = Subdivision {
    name: "Hiroshima",
    code: "JP-34",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-34",
};

pub const JP_35: Subdivision = Subdivision {
    name: "Yamaguchi",
    code: "JP-35",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-35",
};

pub const JP_36: Subdivision = Subdivision {
    name: "Tokushima",
    code: "JP-36",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-36",
};

pub const JP_37: Subdivision = Subdivision {
    name: "Kagawa",
    code: "JP-37",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-37",
};

pub const JP_38: Subdivision = Subdivision {
    name: "Ehime",
    code: "JP-38",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-38",
};

pub const JP_39: Subdivision = Subdivision {
    name: "Kochi",
    code: "JP-39",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-39",
};

pub const JP_40: Subdivision = Subdivision {
    name: "Fukuoka",
    code: "JP-40",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-40",
};

pub const JP_41: Subdivision = Subdivision {
    name: "Saga",
    code: "JP-41",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-41",
};

pub const JP_42: Subdivision = Subdivision {
    name: "Nagasaki",
    code: "JP-42",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-42",
};

pub const JP_43: Subdivision = Subdivision {
    name: "Kumamoto",
    code: "JP-43",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-43",
};

pub const JP_44: Subdivision = Subdivision {
    name: "Oita",
    code: "JP-44",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-44",
};

pub const JP_45: Subdivision = Subdivision {
    name: "Miyazaki",
    code: "JP-45",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-45",
};

pub const JP_46: Subdivision = Subdivision {
    name: "Kagoshima",
    code: "JP-46",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-46",
};

pub const JP_47: Subdivision = Subdivision {
    name: "Okinawa",
    code: "JP-47",
    subdivision_type: "Prefecture",
    country_name: "Japan",
    country_code: "JP",
    region_code: "JP-47",
};

pub const JE_JE: Subdivision = Subdivision {
    name: "Jersey",
    code: "JE-JE",
    subdivision_type: "Country",
    country_name: "Jersey",
    country_code: "JE",
    region_code: "JE-JE",
};

pub const JO_AJ: Subdivision = Subdivision {
    name: "ʽAjlūn",
    code: "JO-AJ",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-AJ",
};

pub const JO_AM: Subdivision = Subdivision {
    name: "‘Ammān",
    code: "JO-AM",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-AM",
};

pub const JO_AQ: Subdivision = Subdivision {
    name: "Al ʽAqabah",
    code: "JO-AQ",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-AQ",
};

pub const JO_AT: Subdivision = Subdivision {
    name: "Aţ Ţafīlah",
    code: "JO-AT",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-AT",
};

pub const JO_AZ: Subdivision = Subdivision {
    name: "Az Zarqā'",
    code: "JO-AZ",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-AZ",
};

pub const JO_BA: Subdivision = Subdivision {
    name: "Al Balqā'",
    code: "JO-BA",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-BA",
};

pub const JO_IR: Subdivision = Subdivision {
    name: "Irbid",
    code: "JO-IR",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-IR",
};

pub const JO_JA: Subdivision = Subdivision {
    name: "Jarash",
    code: "JO-JA",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-JA",
};

pub const JO_KA: Subdivision = Subdivision {
    name: "Al Karak",
    code: "JO-KA",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-KA",
};

pub const JO_MA: Subdivision = Subdivision {
    name: "Al Mafraq",
    code: "JO-MA",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-MA",
};

pub const JO_MD: Subdivision = Subdivision {
    name: "Mādabā",
    code: "JO-MD",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-MD",
};

pub const JO_MN: Subdivision = Subdivision {
    name: "Ma'ān",
    code: "JO-MN",
    subdivision_type: "Governorate",
    country_name: "Jordan",
    country_code: "JO",
    region_code: "JO-MN",
};

pub const KZ_AKM: Subdivision = Subdivision {
    name: "Akmolinskaja oblast'",
    code: "KZ-AKM",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-AKM",
};

pub const KZ_AKT: Subdivision = Subdivision {
    name: "Aktjubinskaja oblast'",
    code: "KZ-AKT",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-AKT",
};

pub const KZ_ALA: Subdivision = Subdivision {
    name: "Almaty",
    code: "KZ-ALA",
    subdivision_type: "City",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-ALA",
};

pub const KZ_ALM: Subdivision = Subdivision {
    name: "Almatinskaja oblast'",
    code: "KZ-ALM",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-ALM",
};

pub const KZ_AST: Subdivision = Subdivision {
    name: "Astana",
    code: "KZ-AST",
    subdivision_type: "City",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-AST",
};

pub const KZ_ATY: Subdivision = Subdivision {
    name: "Atyraū oblysy",
    code: "KZ-ATY",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-ATY",
};

pub const KZ_KAR: Subdivision = Subdivision {
    name: "Karagandinskaja oblast'",
    code: "KZ-KAR",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-KAR",
};

pub const KZ_KUS: Subdivision = Subdivision {
    name: "Kostanajskaja oblast'",
    code: "KZ-KUS",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-KUS",
};

pub const KZ_KZY: Subdivision = Subdivision {
    name: "Kyzylordinskaja oblast'",
    code: "KZ-KZY",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-KZY",
};

pub const KZ_MAN: Subdivision = Subdivision {
    name: "Mangghystaū oblysy",
    code: "KZ-MAN",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-MAN",
};

pub const KZ_PAV: Subdivision = Subdivision {
    name: "Pavlodar oblysy",
    code: "KZ-PAV",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-PAV",
};

pub const KZ_SEV: Subdivision = Subdivision {
    name: "Severo-Kazahstanskaja oblast'",
    code: "KZ-SEV",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-SEV",
};

pub const KZ_VOS: Subdivision = Subdivision {
    name: "Shyghys Qazaqstan oblysy",
    code: "KZ-VOS",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-VOS",
};

pub const KZ_YUZ: Subdivision = Subdivision {
    name: "Južno-Kazahstanskaja oblast'",
    code: "KZ-YUZ",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-YUZ",
};

pub const KZ_ZAP: Subdivision = Subdivision {
    name: "Batys Qazaqstan oblysy",
    code: "KZ-ZAP",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-ZAP",
};

pub const KZ_ZHA: Subdivision = Subdivision {
    name: "Žambylskaja oblast'",
    code: "KZ-ZHA",
    subdivision_type: "Region",
    country_name: "Kazakhstan",
    country_code: "KZ",
    region_code: "KZ-ZHA",
};

pub const KE_01: Subdivision = Subdivision {
    name: "Baringo",
    code: "KE-01",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-01",
};

pub const KE_02: Subdivision = Subdivision {
    name: "Bomet",
    code: "KE-02",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-02",
};

pub const KE_03: Subdivision = Subdivision {
    name: "Bungoma",
    code: "KE-03",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-03",
};

pub const KE_04: Subdivision = Subdivision {
    name: "Busia",
    code: "KE-04",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-04",
};

pub const KE_05: Subdivision = Subdivision {
    name: "Elgeyo/Marakwet",
    code: "KE-05",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-05",
};

pub const KE_06: Subdivision = Subdivision {
    name: "Embu",
    code: "KE-06",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-06",
};

pub const KE_07: Subdivision = Subdivision {
    name: "Garissa",
    code: "KE-07",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-07",
};

pub const KE_08: Subdivision = Subdivision {
    name: "Homa Bay",
    code: "KE-08",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-08",
};

pub const KE_09: Subdivision = Subdivision {
    name: "Isiolo",
    code: "KE-09",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-09",
};

pub const KE_10: Subdivision = Subdivision {
    name: "Kajiado",
    code: "KE-10",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-10",
};

pub const KE_11: Subdivision = Subdivision {
    name: "Kakamega",
    code: "KE-11",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-11",
};

pub const KE_12: Subdivision = Subdivision {
    name: "Kericho",
    code: "KE-12",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-12",
};

pub const KE_13: Subdivision = Subdivision {
    name: "Kiambu",
    code: "KE-13",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-13",
};

pub const KE_14: Subdivision = Subdivision {
    name: "Kilifi",
    code: "KE-14",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-14",
};

pub const KE_15: Subdivision = Subdivision {
    name: "Kirinyaga",
    code: "KE-15",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-15",
};

pub const KE_16: Subdivision = Subdivision {
    name: "Kisii",
    code: "KE-16",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-16",
};

pub const KE_17: Subdivision = Subdivision {
    name: "Kisumu",
    code: "KE-17",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-17",
};

pub const KE_18: Subdivision = Subdivision {
    name: "Kitui",
    code: "KE-18",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-18",
};

pub const KE_19: Subdivision = Subdivision {
    name: "Kwale",
    code: "KE-19",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-19",
};

pub const KE_20: Subdivision = Subdivision {
    name: "Laikipia",
    code: "KE-20",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-20",
};

pub const KE_21: Subdivision = Subdivision {
    name: "Lamu",
    code: "KE-21",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-21",
};

pub const KE_22: Subdivision = Subdivision {
    name: "Machakos",
    code: "KE-22",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-22",
};

pub const KE_23: Subdivision = Subdivision {
    name: "Makueni",
    code: "KE-23",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-23",
};

pub const KE_24: Subdivision = Subdivision {
    name: "Mandera",
    code: "KE-24",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-24",
};

pub const KE_25: Subdivision = Subdivision {
    name: "Marsabit",
    code: "KE-25",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-25",
};

pub const KE_26: Subdivision = Subdivision {
    name: "Meru",
    code: "KE-26",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-26",
};

pub const KE_27: Subdivision = Subdivision {
    name: "Migori",
    code: "KE-27",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-27",
};

pub const KE_28: Subdivision = Subdivision {
    name: "Mombasa",
    code: "KE-28",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-28",
};

pub const KE_29: Subdivision = Subdivision {
    name: "Murang'a",
    code: "KE-29",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-29",
};

pub const KE_30: Subdivision = Subdivision {
    name: "Nairobi City",
    code: "KE-30",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-30",
};

pub const KE_31: Subdivision = Subdivision {
    name: "Nakuru",
    code: "KE-31",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-31",
};

pub const KE_32: Subdivision = Subdivision {
    name: "Nandi",
    code: "KE-32",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-32",
};

pub const KE_33: Subdivision = Subdivision {
    name: "Narok",
    code: "KE-33",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-33",
};

pub const KE_34: Subdivision = Subdivision {
    name: "Nyamira",
    code: "KE-34",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-34",
};

pub const KE_35: Subdivision = Subdivision {
    name: "Nyandarua",
    code: "KE-35",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-35",
};

pub const KE_36: Subdivision = Subdivision {
    name: "Nyeri",
    code: "KE-36",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-36",
};

pub const KE_37: Subdivision = Subdivision {
    name: "Samburu",
    code: "KE-37",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-37",
};

pub const KE_38: Subdivision = Subdivision {
    name: "Siaya",
    code: "KE-38",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-38",
};

pub const KE_39: Subdivision = Subdivision {
    name: "Taita/Taveta",
    code: "KE-39",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-39",
};

pub const KE_40: Subdivision = Subdivision {
    name: "Tana River",
    code: "KE-40",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-40",
};

pub const KE_41: Subdivision = Subdivision {
    name: "Tharaka-Nithi",
    code: "KE-41",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-41",
};

pub const KE_42: Subdivision = Subdivision {
    name: "Trans Nzoia",
    code: "KE-42",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-42",
};

pub const KE_43: Subdivision = Subdivision {
    name: "Turkana",
    code: "KE-43",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-43",
};

pub const KE_44: Subdivision = Subdivision {
    name: "Uasin Gishu",
    code: "KE-44",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-44",
};

pub const KE_45: Subdivision = Subdivision {
    name: "Vihiga",
    code: "KE-45",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-45",
};

pub const KE_46: Subdivision = Subdivision {
    name: "Wajir",
    code: "KE-46",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-46",
};

pub const KE_47: Subdivision = Subdivision {
    name: "West Pokot",
    code: "KE-47",
    subdivision_type: "County",
    country_name: "Kenya",
    country_code: "KE",
    region_code: "KE-47",
};

pub const KI_G: Subdivision = Subdivision {
    name: "Gilbert Islands",
    code: "KI-G",
    subdivision_type: "Group of islands",
    country_name: "Kiribati",
    country_code: "KI",
    region_code: "KI-G",
};

pub const KI_L: Subdivision = Subdivision {
    name: "Line Islands",
    code: "KI-L",
    subdivision_type: "Group of islands",
    country_name: "Kiribati",
    country_code: "KI",
    region_code: "KI-L",
};

pub const KI_P: Subdivision = Subdivision {
    name: "Phoenix Islands",
    code: "KI-P",
    subdivision_type: "Group of islands",
    country_name: "Kiribati",
    country_code: "KI",
    region_code: "KI-P",
};

pub const KP_01: Subdivision = Subdivision {
    name: "P'yǒngyang",
    code: "KP-01",
    subdivision_type: "Capital city",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-01",
};

pub const KP_02: Subdivision = Subdivision {
    name: "P'yǒngan-namdo",
    code: "KP-02",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-02",
};

pub const KP_03: Subdivision = Subdivision {
    name: "P'yǒngan-bukto",
    code: "KP-03",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-03",
};

pub const KP_04: Subdivision = Subdivision {
    name: "Chagang-do",
    code: "KP-04",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-04",
};

pub const KP_05: Subdivision = Subdivision {
    name: "Hwanghae-namdo",
    code: "KP-05",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-05",
};

pub const KP_06: Subdivision = Subdivision {
    name: "Hwanghae-bukto",
    code: "KP-06",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-06",
};

pub const KP_07: Subdivision = Subdivision {
    name: "Kangweonto",
    code: "KP-07",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-07",
};

pub const KP_08: Subdivision = Subdivision {
    name: "Hamgyǒng-namdo",
    code: "KP-08",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-08",
};

pub const KP_09: Subdivision = Subdivision {
    name: "Hamgyǒng-bukto",
    code: "KP-09",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-09",
};

pub const KP_10: Subdivision = Subdivision {
    name: "Ryangkangto",
    code: "KP-10",
    subdivision_type: "Province",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-10",
};

pub const KP_13: Subdivision = Subdivision {
    name: "Nasǒn",
    code: "KP-13",
    subdivision_type: "Special city",
    country_name: "Korea, Democratic People's Republic Of",
    country_code: "KP",
    region_code: "KP-13",
};

pub const KR_11: Subdivision = Subdivision {
    name: "Seoul-T'ǔkpyǒlshi",
    code: "KR-11",
    subdivision_type: "Special city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-11",
};

pub const KR_26: Subdivision = Subdivision {
    name: "Busan Gwang'yeogsi",
    code: "KR-26",
    subdivision_type: "Metropolitan city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-26",
};

pub const KR_27: Subdivision = Subdivision {
    name: "Daegu Gwang'yeogsi",
    code: "KR-27",
    subdivision_type: "Metropolitan city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-27",
};

pub const KR_28: Subdivision = Subdivision {
    name: "Inch'ǒn-Kwangyǒkshi",
    code: "KR-28",
    subdivision_type: "Metropolitan city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-28",
};

pub const KR_29: Subdivision = Subdivision {
    name: "Gwangju Gwang'yeogsi",
    code: "KR-29",
    subdivision_type: "Metropolitan city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-29",
};

pub const KR_30: Subdivision = Subdivision {
    name: "Daejeon Gwang'yeogsi",
    code: "KR-30",
    subdivision_type: "Metropolitan city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-30",
};

pub const KR_31: Subdivision = Subdivision {
    name: "Ulsan Gwang'yeogsi",
    code: "KR-31",
    subdivision_type: "Metropolitan city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-31",
};

pub const KR_41: Subdivision = Subdivision {
    name: "Gyeonggido",
    code: "KR-41",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-41",
};

pub const KR_42: Subdivision = Subdivision {
    name: "Gang'weondo",
    code: "KR-42",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-42",
};

pub const KR_43: Subdivision = Subdivision {
    name: "Ch'ungch'ǒngbuk-do",
    code: "KR-43",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-43",
};

pub const KR_44: Subdivision = Subdivision {
    name: "Ch'ungch'ǒngnam-do",
    code: "KR-44",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-44",
};

pub const KR_45: Subdivision = Subdivision {
    name: "Chǒllabuk-do",
    code: "KR-45",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-45",
};

pub const KR_46: Subdivision = Subdivision {
    name: "Chǒllanam-do",
    code: "KR-46",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-46",
};

pub const KR_47: Subdivision = Subdivision {
    name: "Gyeongsangbugdo",
    code: "KR-47",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-47",
};

pub const KR_48: Subdivision = Subdivision {
    name: "Gyeongsangnamdo",
    code: "KR-48",
    subdivision_type: "Province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-48",
};

pub const KR_49: Subdivision = Subdivision {
    name: "Cheju-do",
    code: "KR-49",
    subdivision_type: "Special self-governing province",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-49",
};

pub const KR_50: Subdivision = Subdivision {
    name: "Sejong",
    code: "KR-50",
    subdivision_type: "Special self-governing city",
    country_name: "Korea, Republic of",
    country_code: "KR",
    region_code: "KR-50",
};

pub const KW_AH: Subdivision = Subdivision {
    name: "Al Aḩmadī",
    code: "KW-AH",
    subdivision_type: "Governorate",
    country_name: "Kuwait",
    country_code: "KW",
    region_code: "KW-AH",
};

pub const KW_FA: Subdivision = Subdivision {
    name: "Al Farwānīyah",
    code: "KW-FA",
    subdivision_type: "Governorate",
    country_name: "Kuwait",
    country_code: "KW",
    region_code: "KW-FA",
};

pub const KW_HA: Subdivision = Subdivision {
    name: "Ḩawallī",
    code: "KW-HA",
    subdivision_type: "Governorate",
    country_name: "Kuwait",
    country_code: "KW",
    region_code: "KW-HA",
};

pub const KW_JA: Subdivision = Subdivision {
    name: "Al Jahrā’",
    code: "KW-JA",
    subdivision_type: "Governorate",
    country_name: "Kuwait",
    country_code: "KW",
    region_code: "KW-JA",
};

pub const KW_KU: Subdivision = Subdivision {
    name: "Al Kuwayt",
    code: "KW-KU",
    subdivision_type: "Governorate",
    country_name: "Kuwait",
    country_code: "KW",
    region_code: "KW-KU",
};

pub const KW_MU: Subdivision = Subdivision {
    name: "Mubārak al Kabīr",
    code: "KW-MU",
    subdivision_type: "Governorate",
    country_name: "Kuwait",
    country_code: "KW",
    region_code: "KW-MU",
};

pub const KG_B: Subdivision = Subdivision {
    name: "Batken",
    code: "KG-B",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-B",
};

pub const KG_C: Subdivision = Subdivision {
    name: "Chü",
    code: "KG-C",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-C",
};

pub const KG_GB: Subdivision = Subdivision {
    name: "Bishkek",
    code: "KG-GB",
    subdivision_type: "City",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-GB",
};

pub const KG_GO: Subdivision = Subdivision {
    name: "Gorod Oš",
    code: "KG-GO",
    subdivision_type: "City",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-GO",
};

pub const KG_J: Subdivision = Subdivision {
    name: "Džalal-Abadskaja oblast'",
    code: "KG-J",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-J",
};

pub const KG_N: Subdivision = Subdivision {
    name: "Naryn",
    code: "KG-N",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-N",
};

pub const KG_O: Subdivision = Subdivision {
    name: "Osh",
    code: "KG-O",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-O",
};

pub const KG_T: Subdivision = Subdivision {
    name: "Talas",
    code: "KG-T",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-T",
};

pub const KG_Y: Subdivision = Subdivision {
    name: "Issyk-Kul'skaja oblast'",
    code: "KG-Y",
    subdivision_type: "Region",
    country_name: "Kyrgyzstan",
    country_code: "KG",
    region_code: "KG-Y",
};

pub const LA_AT: Subdivision = Subdivision {
    name: "Attapu",
    code: "LA-AT",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-AT",
};

pub const LA_BK: Subdivision = Subdivision {
    name: "Bokèo",
    code: "LA-BK",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-BK",
};

pub const LA_BL: Subdivision = Subdivision {
    name: "Bolikhamxai",
    code: "LA-BL",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-BL",
};

pub const LA_CH: Subdivision = Subdivision {
    name: "Champasak",
    code: "LA-CH",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-CH",
};

pub const LA_HO: Subdivision = Subdivision {
    name: "Houaphan",
    code: "LA-HO",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-HO",
};

pub const LA_KH: Subdivision = Subdivision {
    name: "Khammouan",
    code: "LA-KH",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-KH",
};

pub const LA_LM: Subdivision = Subdivision {
    name: "Louang Namtha",
    code: "LA-LM",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-LM",
};

pub const LA_LP: Subdivision = Subdivision {
    name: "Louangphabang",
    code: "LA-LP",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-LP",
};

pub const LA_OU: Subdivision = Subdivision {
    name: "Oudomsai",
    code: "LA-OU",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-OU",
};

pub const LA_PH: Subdivision = Subdivision {
    name: "Phôngsali",
    code: "LA-PH",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-PH",
};

pub const LA_SL: Subdivision = Subdivision {
    name: "Salavan",
    code: "LA-SL",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-SL",
};

pub const LA_SV: Subdivision = Subdivision {
    name: "Savannakhét",
    code: "LA-SV",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-SV",
};

pub const LA_VI: Subdivision = Subdivision {
    name: "Vientiane",
    code: "LA-VI",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-VI",
};

pub const LA_VT: Subdivision = Subdivision {
    name: "Vientiane",
    code: "LA-VT",
    subdivision_type: "Prefecture",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-VT",
};

pub const LA_XA: Subdivision = Subdivision {
    name: "Sayaboury",
    code: "LA-XA",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-XA",
};

pub const LA_XE: Subdivision = Subdivision {
    name: "Sékong",
    code: "LA-XE",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-XE",
};

pub const LA_XI: Subdivision = Subdivision {
    name: "Xiangkhouang",
    code: "LA-XI",
    subdivision_type: "Province",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-XI",
};

pub const LA_XN: Subdivision = Subdivision {
    name: "Xaisômboun",
    code: "LA-XN",
    subdivision_type: "Special zone",
    country_name: "Laos",
    country_code: "LA",
    region_code: "LA-XN",
};

pub const LV_001: Subdivision = Subdivision {
    name: "Aglonas novads",
    code: "LV-001",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-001",
};

pub const LV_002: Subdivision = Subdivision {
    name: "Aizkraukles novads",
    code: "LV-002",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-002",
};

pub const LV_003: Subdivision = Subdivision {
    name: "Aizputes novads",
    code: "LV-003",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-003",
};

pub const LV_004: Subdivision = Subdivision {
    name: "Aknīstes novads",
    code: "LV-004",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-004",
};

pub const LV_005: Subdivision = Subdivision {
    name: "Alojas novads",
    code: "LV-005",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-005",
};

pub const LV_006: Subdivision = Subdivision {
    name: "Alsungas novads",
    code: "LV-006",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-006",
};

pub const LV_007: Subdivision = Subdivision {
    name: "Alūksnes novads",
    code: "LV-007",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-007",
};

pub const LV_008: Subdivision = Subdivision {
    name: "Amatas novads",
    code: "LV-008",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-008",
};

pub const LV_009: Subdivision = Subdivision {
    name: "Apes novads",
    code: "LV-009",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-009",
};

pub const LV_010: Subdivision = Subdivision {
    name: "Auces novads",
    code: "LV-010",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-010",
};

pub const LV_011: Subdivision = Subdivision {
    name: "Ādažu novads",
    code: "LV-011",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-011",
};

pub const LV_012: Subdivision = Subdivision {
    name: "Babītes novads",
    code: "LV-012",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-012",
};

pub const LV_013: Subdivision = Subdivision {
    name: "Baldones novads",
    code: "LV-013",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-013",
};

pub const LV_014: Subdivision = Subdivision {
    name: "Baltinavas novads",
    code: "LV-014",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-014",
};

pub const LV_015: Subdivision = Subdivision {
    name: "Balvu novads",
    code: "LV-015",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-015",
};

pub const LV_016: Subdivision = Subdivision {
    name: "Bauskas novads",
    code: "LV-016",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-016",
};

pub const LV_017: Subdivision = Subdivision {
    name: "Beverīnas novads",
    code: "LV-017",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-017",
};

pub const LV_018: Subdivision = Subdivision {
    name: "Brocēnu novads",
    code: "LV-018",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-018",
};

pub const LV_019: Subdivision = Subdivision {
    name: "Burtnieku novads",
    code: "LV-019",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-019",
};

pub const LV_020: Subdivision = Subdivision {
    name: "Carnikavas novads",
    code: "LV-020",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-020",
};

pub const LV_021: Subdivision = Subdivision {
    name: "Cesvaines novads",
    code: "LV-021",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-021",
};

pub const LV_022: Subdivision = Subdivision {
    name: "Cēsu novads",
    code: "LV-022",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-022",
};

pub const LV_023: Subdivision = Subdivision {
    name: "Ciblas novads",
    code: "LV-023",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-023",
};

pub const LV_024: Subdivision = Subdivision {
    name: "Dagdas novads",
    code: "LV-024",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-024",
};

pub const LV_025: Subdivision = Subdivision {
    name: "Daugavpils novads",
    code: "LV-025",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-025",
};

pub const LV_026: Subdivision = Subdivision {
    name: "Dobeles novads",
    code: "LV-026",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-026",
};

pub const LV_027: Subdivision = Subdivision {
    name: "Dundagas novads",
    code: "LV-027",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-027",
};

pub const LV_028: Subdivision = Subdivision {
    name: "Durbes novads",
    code: "LV-028",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-028",
};

pub const LV_029: Subdivision = Subdivision {
    name: "Engures novads",
    code: "LV-029",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-029",
};

pub const LV_030: Subdivision = Subdivision {
    name: "Ērgļu novads",
    code: "LV-030",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-030",
};

pub const LV_031: Subdivision = Subdivision {
    name: "Garkalnes novads",
    code: "LV-031",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-031",
};

pub const LV_032: Subdivision = Subdivision {
    name: "Grobiņas novads",
    code: "LV-032",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-032",
};

pub const LV_033: Subdivision = Subdivision {
    name: "Gulbenes novads",
    code: "LV-033",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-033",
};

pub const LV_034: Subdivision = Subdivision {
    name: "Iecavas novads",
    code: "LV-034",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-034",
};

pub const LV_035: Subdivision = Subdivision {
    name: "Ikšķiles novads",
    code: "LV-035",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-035",
};

pub const LV_036: Subdivision = Subdivision {
    name: "Ilūkstes novads",
    code: "LV-036",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-036",
};

pub const LV_037: Subdivision = Subdivision {
    name: "Inčukalna novads",
    code: "LV-037",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-037",
};

pub const LV_038: Subdivision = Subdivision {
    name: "Jaunjelgavas novads",
    code: "LV-038",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-038",
};

pub const LV_039: Subdivision = Subdivision {
    name: "Jaunpiebalgas novads",
    code: "LV-039",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-039",
};

pub const LV_040: Subdivision = Subdivision {
    name: "Jaunpils novads",
    code: "LV-040",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-040",
};

pub const LV_041: Subdivision = Subdivision {
    name: "Jelgavas novads",
    code: "LV-041",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-041",
};

pub const LV_042: Subdivision = Subdivision {
    name: "Jēkabpils novads",
    code: "LV-042",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-042",
};

pub const LV_043: Subdivision = Subdivision {
    name: "Kandavas novads",
    code: "LV-043",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-043",
};

pub const LV_044: Subdivision = Subdivision {
    name: "Kārsavas novads",
    code: "LV-044",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-044",
};

pub const LV_045: Subdivision = Subdivision {
    name: "Kocēnu novads",
    code: "LV-045",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-045",
};

pub const LV_046: Subdivision = Subdivision {
    name: "Kokneses novads",
    code: "LV-046",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-046",
};

pub const LV_047: Subdivision = Subdivision {
    name: "Krāslavas novads",
    code: "LV-047",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-047",
};

pub const LV_048: Subdivision = Subdivision {
    name: "Krimuldas novads",
    code: "LV-048",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-048",
};

pub const LV_049: Subdivision = Subdivision {
    name: "Krustpils novads",
    code: "LV-049",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-049",
};

pub const LV_050: Subdivision = Subdivision {
    name: "Kuldīgas novads",
    code: "LV-050",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-050",
};

pub const LV_051: Subdivision = Subdivision {
    name: "Ķeguma novads",
    code: "LV-051",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-051",
};

pub const LV_052: Subdivision = Subdivision {
    name: "Ķekavas novads",
    code: "LV-052",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-052",
};

pub const LV_053: Subdivision = Subdivision {
    name: "Lielvārdes novads",
    code: "LV-053",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-053",
};

pub const LV_054: Subdivision = Subdivision {
    name: "Limbažu novads",
    code: "LV-054",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-054",
};

pub const LV_055: Subdivision = Subdivision {
    name: "Līgatnes novads",
    code: "LV-055",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-055",
};

pub const LV_056: Subdivision = Subdivision {
    name: "Līvānu novads",
    code: "LV-056",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-056",
};

pub const LV_057: Subdivision = Subdivision {
    name: "Lubānas novads",
    code: "LV-057",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-057",
};

pub const LV_058: Subdivision = Subdivision {
    name: "Ludzas novads",
    code: "LV-058",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-058",
};

pub const LV_059: Subdivision = Subdivision {
    name: "Madonas novads",
    code: "LV-059",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-059",
};

pub const LV_060: Subdivision = Subdivision {
    name: "Mazsalacas novads",
    code: "LV-060",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-060",
};

pub const LV_061: Subdivision = Subdivision {
    name: "Mālpils novads",
    code: "LV-061",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-061",
};

pub const LV_062: Subdivision = Subdivision {
    name: "Mārupes novads",
    code: "LV-062",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-062",
};

pub const LV_063: Subdivision = Subdivision {
    name: "Mērsraga novads",
    code: "LV-063",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-063",
};

pub const LV_064: Subdivision = Subdivision {
    name: "Naukšēnu novads",
    code: "LV-064",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-064",
};

pub const LV_065: Subdivision = Subdivision {
    name: "Neretas novads",
    code: "LV-065",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-065",
};

pub const LV_066: Subdivision = Subdivision {
    name: "Nīcas novads",
    code: "LV-066",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-066",
};

pub const LV_067: Subdivision = Subdivision {
    name: "Ogres novads",
    code: "LV-067",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-067",
};

pub const LV_068: Subdivision = Subdivision {
    name: "Olaines novads",
    code: "LV-068",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-068",
};

pub const LV_069: Subdivision = Subdivision {
    name: "Ozolnieku novads",
    code: "LV-069",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-069",
};

pub const LV_070: Subdivision = Subdivision {
    name: "Pārgaujas novads",
    code: "LV-070",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-070",
};

pub const LV_071: Subdivision = Subdivision {
    name: "Pāvilostas novads",
    code: "LV-071",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-071",
};

pub const LV_072: Subdivision = Subdivision {
    name: "Pļaviņu novads",
    code: "LV-072",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-072",
};

pub const LV_073: Subdivision = Subdivision {
    name: "Preiļu novads",
    code: "LV-073",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-073",
};

pub const LV_074: Subdivision = Subdivision {
    name: "Priekules novads",
    code: "LV-074",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-074",
};

pub const LV_075: Subdivision = Subdivision {
    name: "Priekuļu novads",
    code: "LV-075",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-075",
};

pub const LV_076: Subdivision = Subdivision {
    name: "Raunas novads",
    code: "LV-076",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-076",
};

pub const LV_077: Subdivision = Subdivision {
    name: "Rēzeknes novads",
    code: "LV-077",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-077",
};

pub const LV_078: Subdivision = Subdivision {
    name: "Riebiņu novads",
    code: "LV-078",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-078",
};

pub const LV_079: Subdivision = Subdivision {
    name: "Rojas novads",
    code: "LV-079",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-079",
};

pub const LV_080: Subdivision = Subdivision {
    name: "Ropažu novads",
    code: "LV-080",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-080",
};

pub const LV_081: Subdivision = Subdivision {
    name: "Rucavas novads",
    code: "LV-081",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-081",
};

pub const LV_082: Subdivision = Subdivision {
    name: "Rugāju novads",
    code: "LV-082",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-082",
};

pub const LV_083: Subdivision = Subdivision {
    name: "Rundāles novads",
    code: "LV-083",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-083",
};

pub const LV_084: Subdivision = Subdivision {
    name: "Rūjienas novads",
    code: "LV-084",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-084",
};

pub const LV_085: Subdivision = Subdivision {
    name: "Salas novads",
    code: "LV-085",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-085",
};

pub const LV_086: Subdivision = Subdivision {
    name: "Salacgrīvas novads",
    code: "LV-086",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-086",
};

pub const LV_087: Subdivision = Subdivision {
    name: "Salaspils novads",
    code: "LV-087",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-087",
};

pub const LV_088: Subdivision = Subdivision {
    name: "Saldus novads",
    code: "LV-088",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-088",
};

pub const LV_089: Subdivision = Subdivision {
    name: "Saulkrastu novads",
    code: "LV-089",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-089",
};

pub const LV_090: Subdivision = Subdivision {
    name: "Sējas novads",
    code: "LV-090",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-090",
};

pub const LV_091: Subdivision = Subdivision {
    name: "Siguldas novads",
    code: "LV-091",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-091",
};

pub const LV_092: Subdivision = Subdivision {
    name: "Skrīveru novads",
    code: "LV-092",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-092",
};

pub const LV_093: Subdivision = Subdivision {
    name: "Skrundas novads",
    code: "LV-093",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-093",
};

pub const LV_094: Subdivision = Subdivision {
    name: "Smiltenes novads",
    code: "LV-094",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-094",
};

pub const LV_095: Subdivision = Subdivision {
    name: "Stopiņu novads",
    code: "LV-095",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-095",
};

pub const LV_096: Subdivision = Subdivision {
    name: "Strenču novads",
    code: "LV-096",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-096",
};

pub const LV_097: Subdivision = Subdivision {
    name: "Talsu novads",
    code: "LV-097",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-097",
};

pub const LV_098: Subdivision = Subdivision {
    name: "Tērvetes novads",
    code: "LV-098",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-098",
};

pub const LV_099: Subdivision = Subdivision {
    name: "Tukuma novads",
    code: "LV-099",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-099",
};

pub const LV_100: Subdivision = Subdivision {
    name: "Vaiņodes novads",
    code: "LV-100",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-100",
};

pub const LV_101: Subdivision = Subdivision {
    name: "Valkas novads",
    code: "LV-101",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-101",
};

pub const LV_102: Subdivision = Subdivision {
    name: "Varakļānu novads",
    code: "LV-102",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-102",
};

pub const LV_103: Subdivision = Subdivision {
    name: "Vārkavas novads",
    code: "LV-103",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-103",
};

pub const LV_104: Subdivision = Subdivision {
    name: "Vecpiebalgas novads",
    code: "LV-104",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-104",
};

pub const LV_105: Subdivision = Subdivision {
    name: "Vecumnieku novads",
    code: "LV-105",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-105",
};

pub const LV_106: Subdivision = Subdivision {
    name: "Ventspils novads",
    code: "LV-106",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-106",
};

pub const LV_107: Subdivision = Subdivision {
    name: "Viesītes novads",
    code: "LV-107",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-107",
};

pub const LV_108: Subdivision = Subdivision {
    name: "Viļakas novads",
    code: "LV-108",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-108",
};

pub const LV_109: Subdivision = Subdivision {
    name: "Viļānu novads",
    code: "LV-109",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-109",
};

pub const LV_110: Subdivision = Subdivision {
    name: "Zilupes novads",
    code: "LV-110",
    subdivision_type: "Municipality",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-110",
};

pub const LV_DGV: Subdivision = Subdivision {
    name: "Daugavpils",
    code: "LV-DGV",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-DGV",
};

pub const LV_JEL: Subdivision = Subdivision {
    name: "Jelgava",
    code: "LV-JEL",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-JEL",
};

pub const LV_JKB: Subdivision = Subdivision {
    name: "Jēkabpils",
    code: "LV-JKB",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-JKB",
};

pub const LV_JUR: Subdivision = Subdivision {
    name: "Jūrmala",
    code: "LV-JUR",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-JUR",
};

pub const LV_LPX: Subdivision = Subdivision {
    name: "Liepāja",
    code: "LV-LPX",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-LPX",
};

pub const LV_REZ: Subdivision = Subdivision {
    name: "Rēzekne",
    code: "LV-REZ",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-REZ",
};

pub const LV_RIX: Subdivision = Subdivision {
    name: "Rīga",
    code: "LV-RIX",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-RIX",
};

pub const LV_VEN: Subdivision = Subdivision {
    name: "Ventspils",
    code: "LV-VEN",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-VEN",
};

pub const LV_VMR: Subdivision = Subdivision {
    name: "Valmiera",
    code: "LV-VMR",
    subdivision_type: "Republican city",
    country_name: "Latvia",
    country_code: "LV",
    region_code: "LV-VMR",
};

pub const LB_AK: Subdivision = Subdivision {
    name: "Aakkâr",
    code: "LB-AK",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-AK",
};

pub const LB_AS: Subdivision = Subdivision {
    name: "Ash Shimāl",
    code: "LB-AS",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-AS",
};

pub const LB_BA: Subdivision = Subdivision {
    name: "Bayrūt",
    code: "LB-BA",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-BA",
};

pub const LB_BH: Subdivision = Subdivision {
    name: "Baalbek-Hermel",
    code: "LB-BH",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-BH",
};

pub const LB_BI: Subdivision = Subdivision {
    name: "Al Biqā‘",
    code: "LB-BI",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-BI",
};

pub const LB_JA: Subdivision = Subdivision {
    name: "Al Janūb",
    code: "LB-JA",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-JA",
};

pub const LB_JL: Subdivision = Subdivision {
    name: "Jabal Lubnān",
    code: "LB-JL",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-JL",
};

pub const LB_NA: Subdivision = Subdivision {
    name: "An Nabaţīyah",
    code: "LB-NA",
    subdivision_type: "Governorate",
    country_name: "Lebanon",
    country_code: "LB",
    region_code: "LB-NA",
};

pub const LS_A: Subdivision = Subdivision {
    name: "Maseru",
    code: "LS-A",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-A",
};

pub const LS_B: Subdivision = Subdivision {
    name: "Butha-Buthe",
    code: "LS-B",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-B",
};

pub const LS_C: Subdivision = Subdivision {
    name: "Leribe",
    code: "LS-C",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-C",
};

pub const LS_D: Subdivision = Subdivision {
    name: "Berea",
    code: "LS-D",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-D",
};

pub const LS_E: Subdivision = Subdivision {
    name: "Mafeteng",
    code: "LS-E",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-E",
};

pub const LS_F: Subdivision = Subdivision {
    name: "Mohale's Hoek",
    code: "LS-F",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-F",
};

pub const LS_G: Subdivision = Subdivision {
    name: "Quthing",
    code: "LS-G",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-G",
};

pub const LS_H: Subdivision = Subdivision {
    name: "Qacha's Nek",
    code: "LS-H",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-H",
};

pub const LS_J: Subdivision = Subdivision {
    name: "Mokhotlong",
    code: "LS-J",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-J",
};

pub const LS_K: Subdivision = Subdivision {
    name: "Thaba-Tseka",
    code: "LS-K",
    subdivision_type: "District",
    country_name: "Lesotho",
    country_code: "LS",
    region_code: "LS-K",
};

pub const LR_BG: Subdivision = Subdivision {
    name: "Bong",
    code: "LR-BG",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-BG",
};

pub const LR_BM: Subdivision = Subdivision {
    name: "Bomi",
    code: "LR-BM",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-BM",
};

pub const LR_CM: Subdivision = Subdivision {
    name: "Grand Cape Mount",
    code: "LR-CM",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-CM",
};

pub const LR_GB: Subdivision = Subdivision {
    name: "Grand Bassa",
    code: "LR-GB",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-GB",
};

pub const LR_GG: Subdivision = Subdivision {
    name: "Grand Gedeh",
    code: "LR-GG",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-GG",
};

pub const LR_GK: Subdivision = Subdivision {
    name: "Grand Kru",
    code: "LR-GK",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-GK",
};

pub const LR_GP: Subdivision = Subdivision {
    name: "Gbarpolu",
    code: "LR-GP",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-GP",
};

pub const LR_LO: Subdivision = Subdivision {
    name: "Lofa",
    code: "LR-LO",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-LO",
};

pub const LR_MG: Subdivision = Subdivision {
    name: "Margibi",
    code: "LR-MG",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-MG",
};

pub const LR_MO: Subdivision = Subdivision {
    name: "Montserrado",
    code: "LR-MO",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-MO",
};

pub const LR_MY: Subdivision = Subdivision {
    name: "Maryland",
    code: "LR-MY",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-MY",
};

pub const LR_NI: Subdivision = Subdivision {
    name: "Nimba",
    code: "LR-NI",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-NI",
};

pub const LR_RG: Subdivision = Subdivision {
    name: "River Gee",
    code: "LR-RG",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-RG",
};

pub const LR_RI: Subdivision = Subdivision {
    name: "Rivercess",
    code: "LR-RI",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-RI",
};

pub const LR_SI: Subdivision = Subdivision {
    name: "Sinoe",
    code: "LR-SI",
    subdivision_type: "County",
    country_name: "Liberia",
    country_code: "LR",
    region_code: "LR-SI",
};

pub const LY_BA: Subdivision = Subdivision {
    name: "Banghāzī",
    code: "LY-BA",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-BA",
};

pub const LY_BU: Subdivision = Subdivision {
    name: "Al Buţnān",
    code: "LY-BU",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-BU",
};

pub const LY_DR: Subdivision = Subdivision {
    name: "Darnah",
    code: "LY-DR",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-DR",
};

pub const LY_GT: Subdivision = Subdivision {
    name: "Ghāt",
    code: "LY-GT",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-GT",
};

pub const LY_JA: Subdivision = Subdivision {
    name: "Al Jabal al Akhḑar",
    code: "LY-JA",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-JA",
};

pub const LY_JG: Subdivision = Subdivision {
    name: "Al Jabal al Gharbī",
    code: "LY-JG",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-JG",
};

pub const LY_JI: Subdivision = Subdivision {
    name: "Al Jifārah",
    code: "LY-JI",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-JI",
};

pub const LY_JU: Subdivision = Subdivision {
    name: "Al Jufrah",
    code: "LY-JU",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-JU",
};

pub const LY_KF: Subdivision = Subdivision {
    name: "Al Kufrah",
    code: "LY-KF",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-KF",
};

pub const LY_MB: Subdivision = Subdivision {
    name: "Al Marqab",
    code: "LY-MB",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-MB",
};

pub const LY_MI: Subdivision = Subdivision {
    name: "Mişrātah",
    code: "LY-MI",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-MI",
};

pub const LY_MJ: Subdivision = Subdivision {
    name: "Al Marj",
    code: "LY-MJ",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-MJ",
};

pub const LY_MQ: Subdivision = Subdivision {
    name: "Murzuq",
    code: "LY-MQ",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-MQ",
};

pub const LY_NL: Subdivision = Subdivision {
    name: "Nālūt",
    code: "LY-NL",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-NL",
};

pub const LY_NQ: Subdivision = Subdivision {
    name: "An Nuqāţ al Khams",
    code: "LY-NQ",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-NQ",
};

pub const LY_SB: Subdivision = Subdivision {
    name: "Sabhā",
    code: "LY-SB",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-SB",
};

pub const LY_SR: Subdivision = Subdivision {
    name: "Surt",
    code: "LY-SR",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-SR",
};

pub const LY_TB: Subdivision = Subdivision {
    name: "Ţarābulus",
    code: "LY-TB",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-TB",
};

pub const LY_WA: Subdivision = Subdivision {
    name: "Al Wāḩāt",
    code: "LY-WA",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-WA",
};

pub const LY_WD: Subdivision = Subdivision {
    name: "Wādī al Ḩayāt",
    code: "LY-WD",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-WD",
};

pub const LY_WS: Subdivision = Subdivision {
    name: "Wādī ash Shāţi’",
    code: "LY-WS",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-WS",
};

pub const LY_ZA: Subdivision = Subdivision {
    name: "Az Zāwiyah",
    code: "LY-ZA",
    subdivision_type: "Popularate",
    country_name: "Libya",
    country_code: "LY",
    region_code: "LY-ZA",
};

pub const LI_01: Subdivision = Subdivision {
    name: "Balzers",
    code: "LI-01",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-01",
};

pub const LI_02: Subdivision = Subdivision {
    name: "Eschen",
    code: "LI-02",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-02",
};

pub const LI_03: Subdivision = Subdivision {
    name: "Gamprin",
    code: "LI-03",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-03",
};

pub const LI_04: Subdivision = Subdivision {
    name: "Mauren",
    code: "LI-04",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-04",
};

pub const LI_05: Subdivision = Subdivision {
    name: "Planken",
    code: "LI-05",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-05",
};

pub const LI_06: Subdivision = Subdivision {
    name: "Ruggell",
    code: "LI-06",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-06",
};

pub const LI_07: Subdivision = Subdivision {
    name: "Schaan",
    code: "LI-07",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-07",
};

pub const LI_08: Subdivision = Subdivision {
    name: "Schellenberg",
    code: "LI-08",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-08",
};

pub const LI_09: Subdivision = Subdivision {
    name: "Triesen",
    code: "LI-09",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-09",
};

pub const LI_10: Subdivision = Subdivision {
    name: "Triesenberg",
    code: "LI-10",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-10",
};

pub const LI_11: Subdivision = Subdivision {
    name: "Vaduz",
    code: "LI-11",
    subdivision_type: "Commune",
    country_name: "Liechtenstein",
    country_code: "LI",
    region_code: "LI-11",
};

pub const LT_01: Subdivision = Subdivision {
    name: "Akmenė",
    code: "LT-01",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-01",
};

pub const LT_02: Subdivision = Subdivision {
    name: "Alytaus miestas",
    code: "LT-02",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-02",
};

pub const LT_03: Subdivision = Subdivision {
    name: "Alytus",
    code: "LT-03",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-03",
};

pub const LT_04: Subdivision = Subdivision {
    name: "Anykščiai",
    code: "LT-04",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-04",
};

pub const LT_05: Subdivision = Subdivision {
    name: "Birštono",
    code: "LT-05",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-05",
};

pub const LT_06: Subdivision = Subdivision {
    name: "Biržai",
    code: "LT-06",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-06",
};

pub const LT_07: Subdivision = Subdivision {
    name: "Druskininkai",
    code: "LT-07",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-07",
};

pub const LT_08: Subdivision = Subdivision {
    name: "Elektrénai",
    code: "LT-08",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-08",
};

pub const LT_09: Subdivision = Subdivision {
    name: "Ignalina",
    code: "LT-09",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-09",
};

pub const LT_10: Subdivision = Subdivision {
    name: "Jonava",
    code: "LT-10",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-10",
};

pub const LT_11: Subdivision = Subdivision {
    name: "Joniškis",
    code: "LT-11",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-11",
};

pub const LT_12: Subdivision = Subdivision {
    name: "Jurbarkas",
    code: "LT-12",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-12",
};

pub const LT_13: Subdivision = Subdivision {
    name: "Kaišiadorys",
    code: "LT-13",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-13",
};

pub const LT_14: Subdivision = Subdivision {
    name: "Kalvarijos",
    code: "LT-14",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-14",
};

pub const LT_15: Subdivision = Subdivision {
    name: "Kauno miestas",
    code: "LT-15",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-15",
};

pub const LT_16: Subdivision = Subdivision {
    name: "Kaunas",
    code: "LT-16",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-16",
};

pub const LT_17: Subdivision = Subdivision {
    name: "Kazlų Rūdos",
    code: "LT-17",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-17",
};

pub const LT_18: Subdivision = Subdivision {
    name: "Kėdainiai",
    code: "LT-18",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-18",
};

pub const LT_19: Subdivision = Subdivision {
    name: "Kelmė",
    code: "LT-19",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-19",
};

pub const LT_20: Subdivision = Subdivision {
    name: "Klaipėdos miestas",
    code: "LT-20",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-20",
};

pub const LT_21: Subdivision = Subdivision {
    name: "Klaipėda",
    code: "LT-21",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-21",
};

pub const LT_22: Subdivision = Subdivision {
    name: "Kretinga",
    code: "LT-22",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-22",
};

pub const LT_23: Subdivision = Subdivision {
    name: "Kupiškis",
    code: "LT-23",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-23",
};

pub const LT_24: Subdivision = Subdivision {
    name: "Lazdijai",
    code: "LT-24",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-24",
};

pub const LT_25: Subdivision = Subdivision {
    name: "Marijampolė",
    code: "LT-25",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-25",
};

pub const LT_26: Subdivision = Subdivision {
    name: "Mažeikiai",
    code: "LT-26",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-26",
};

pub const LT_27: Subdivision = Subdivision {
    name: "Molėtai",
    code: "LT-27",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-27",
};

pub const LT_28: Subdivision = Subdivision {
    name: "Neringa",
    code: "LT-28",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-28",
};

pub const LT_29: Subdivision = Subdivision {
    name: "Pagégiai",
    code: "LT-29",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-29",
};

pub const LT_30: Subdivision = Subdivision {
    name: "Pakruojis",
    code: "LT-30",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-30",
};

pub const LT_31: Subdivision = Subdivision {
    name: "Palangos miestas",
    code: "LT-31",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-31",
};

pub const LT_32: Subdivision = Subdivision {
    name: "Panevėžio miestas",
    code: "LT-32",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-32",
};

pub const LT_33: Subdivision = Subdivision {
    name: "Panevėžys",
    code: "LT-33",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-33",
};

pub const LT_34: Subdivision = Subdivision {
    name: "Pasvalys",
    code: "LT-34",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-34",
};

pub const LT_35: Subdivision = Subdivision {
    name: "Plungė",
    code: "LT-35",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-35",
};

pub const LT_36: Subdivision = Subdivision {
    name: "Prienai",
    code: "LT-36",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-36",
};

pub const LT_37: Subdivision = Subdivision {
    name: "Radviliškis",
    code: "LT-37",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-37",
};

pub const LT_38: Subdivision = Subdivision {
    name: "Raseiniai",
    code: "LT-38",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-38",
};

pub const LT_39: Subdivision = Subdivision {
    name: "Rietavo",
    code: "LT-39",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-39",
};

pub const LT_40: Subdivision = Subdivision {
    name: "Rokiškis",
    code: "LT-40",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-40",
};

pub const LT_41: Subdivision = Subdivision {
    name: "Šakiai",
    code: "LT-41",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-41",
};

pub const LT_42: Subdivision = Subdivision {
    name: "Šalčininkai",
    code: "LT-42",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-42",
};

pub const LT_43: Subdivision = Subdivision {
    name: "Šiaulių miestas",
    code: "LT-43",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-43",
};

pub const LT_44: Subdivision = Subdivision {
    name: "Šiauliai",
    code: "LT-44",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-44",
};

pub const LT_45: Subdivision = Subdivision {
    name: "Šilalė",
    code: "LT-45",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-45",
};

pub const LT_46: Subdivision = Subdivision {
    name: "Šilutė",
    code: "LT-46",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-46",
};

pub const LT_47: Subdivision = Subdivision {
    name: "Širvintos",
    code: "LT-47",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-47",
};

pub const LT_48: Subdivision = Subdivision {
    name: "Skuodas",
    code: "LT-48",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-48",
};

pub const LT_49: Subdivision = Subdivision {
    name: "Švenčionys",
    code: "LT-49",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-49",
};

pub const LT_50: Subdivision = Subdivision {
    name: "Tauragė",
    code: "LT-50",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-50",
};

pub const LT_51: Subdivision = Subdivision {
    name: "Telšiai",
    code: "LT-51",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-51",
};

pub const LT_52: Subdivision = Subdivision {
    name: "Trakai",
    code: "LT-52",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-52",
};

pub const LT_53: Subdivision = Subdivision {
    name: "Ukmergė",
    code: "LT-53",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-53",
};

pub const LT_54: Subdivision = Subdivision {
    name: "Utena",
    code: "LT-54",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-54",
};

pub const LT_55: Subdivision = Subdivision {
    name: "Varėna",
    code: "LT-55",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-55",
};

pub const LT_56: Subdivision = Subdivision {
    name: "Vilkaviškis",
    code: "LT-56",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-56",
};

pub const LT_57: Subdivision = Subdivision {
    name: "Vilniaus miestas",
    code: "LT-57",
    subdivision_type: "City municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-57",
};

pub const LT_58: Subdivision = Subdivision {
    name: "Vilnius",
    code: "LT-58",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-58",
};

pub const LT_59: Subdivision = Subdivision {
    name: "Visaginas",
    code: "LT-59",
    subdivision_type: "Municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-59",
};

pub const LT_60: Subdivision = Subdivision {
    name: "Zarasai",
    code: "LT-60",
    subdivision_type: "District municipality",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-60",
};

pub const LT_AL: Subdivision = Subdivision {
    name: "Alytaus apskritis",
    code: "LT-AL",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-AL",
};

pub const LT_KL: Subdivision = Subdivision {
    name: "Klaipėdos apskritis",
    code: "LT-KL",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-KL",
};

pub const LT_KU: Subdivision = Subdivision {
    name: "Kauno apskritis",
    code: "LT-KU",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-KU",
};

pub const LT_MR: Subdivision = Subdivision {
    name: "Marijampolės apskritis",
    code: "LT-MR",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-MR",
};

pub const LT_PN: Subdivision = Subdivision {
    name: "Panevėžio apskritis",
    code: "LT-PN",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-PN",
};

pub const LT_SA: Subdivision = Subdivision {
    name: "Šiaulių apskritis",
    code: "LT-SA",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-SA",
};

pub const LT_TA: Subdivision = Subdivision {
    name: "Tauragės apskritis",
    code: "LT-TA",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-TA",
};

pub const LT_TE: Subdivision = Subdivision {
    name: "Telšių apskritis",
    code: "LT-TE",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-TE",
};

pub const LT_UT: Subdivision = Subdivision {
    name: "Utenos apskritis",
    code: "LT-UT",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-UT",
};

pub const LT_VL: Subdivision = Subdivision {
    name: "Vilniaus apskritis",
    code: "LT-VL",
    subdivision_type: "County",
    country_name: "Lithuania",
    country_code: "LT",
    region_code: "LT-VL",
};

pub const LU_D: Subdivision = Subdivision {
    name: "Diekirch",
    code: "LU-D",
    subdivision_type: "District",
    country_name: "Luxembourg",
    country_code: "LU",
    region_code: "LU-D",
};

pub const LU_G: Subdivision = Subdivision {
    name: "Grevenmacher",
    code: "LU-G",
    subdivision_type: "District",
    country_name: "Luxembourg",
    country_code: "LU",
    region_code: "LU-G",
};

pub const LU_L: Subdivision = Subdivision {
    name: "Lëtzebuerg",
    code: "LU-L",
    subdivision_type: "District",
    country_name: "Luxembourg",
    country_code: "LU",
    region_code: "LU-L",
};

pub const MO_MO: Subdivision = Subdivision {
    name: "Macau",
    code: "MO-MO",
    subdivision_type: "Country",
    country_name: "Macau",
    country_code: "MO",
    region_code: "MO-MO",
};

pub const MK_01: Subdivision = Subdivision {
    name: "Aerodrom †",
    code: "MK-01",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-01",
};

pub const MK_02: Subdivision = Subdivision {
    name: "Aračinovo",
    code: "MK-02",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-02",
};

pub const MK_03: Subdivision = Subdivision {
    name: "Berovo",
    code: "MK-03",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-03",
};

pub const MK_04: Subdivision = Subdivision {
    name: "Bitola",
    code: "MK-04",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-04",
};

pub const MK_05: Subdivision = Subdivision {
    name: "Bogdanci",
    code: "MK-05",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-05",
};

pub const MK_06: Subdivision = Subdivision {
    name: "Bogovinje",
    code: "MK-06",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-06",
};

pub const MK_07: Subdivision = Subdivision {
    name: "Bosilovo",
    code: "MK-07",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-07",
};

pub const MK_08: Subdivision = Subdivision {
    name: "Brvenica",
    code: "MK-08",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-08",
};

pub const MK_09: Subdivision = Subdivision {
    name: "Butel †",
    code: "MK-09",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-09",
};

pub const MK_10: Subdivision = Subdivision {
    name: "Valandovo",
    code: "MK-10",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-10",
};

pub const MK_11: Subdivision = Subdivision {
    name: "Vasilevo",
    code: "MK-11",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-11",
};

pub const MK_12: Subdivision = Subdivision {
    name: "Vevčani",
    code: "MK-12",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-12",
};

pub const MK_13: Subdivision = Subdivision {
    name: "Veles",
    code: "MK-13",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-13",
};

pub const MK_14: Subdivision = Subdivision {
    name: "Vinica",
    code: "MK-14",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-14",
};

pub const MK_15: Subdivision = Subdivision {
    name: "Vraneštica",
    code: "MK-15",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-15",
};

pub const MK_16: Subdivision = Subdivision {
    name: "Vrapčište",
    code: "MK-16",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-16",
};

pub const MK_17: Subdivision = Subdivision {
    name: "Gazi Baba †",
    code: "MK-17",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-17",
};

pub const MK_18: Subdivision = Subdivision {
    name: "Gevgelija",
    code: "MK-18",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-18",
};

pub const MK_19: Subdivision = Subdivision {
    name: "Gostivar",
    code: "MK-19",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-19",
};

pub const MK_20: Subdivision = Subdivision {
    name: "Gradsko",
    code: "MK-20",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-20",
};

pub const MK_21: Subdivision = Subdivision {
    name: "Debar",
    code: "MK-21",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-21",
};

pub const MK_22: Subdivision = Subdivision {
    name: "Debarca",
    code: "MK-22",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-22",
};

pub const MK_23: Subdivision = Subdivision {
    name: "Delčevo",
    code: "MK-23",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-23",
};

pub const MK_24: Subdivision = Subdivision {
    name: "Demir Kapija",
    code: "MK-24",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-24",
};

pub const MK_25: Subdivision = Subdivision {
    name: "Demir Hisar",
    code: "MK-25",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-25",
};

pub const MK_26: Subdivision = Subdivision {
    name: "Dojran",
    code: "MK-26",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-26",
};

pub const MK_27: Subdivision = Subdivision {
    name: "Dolneni",
    code: "MK-27",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-27",
};

pub const MK_28: Subdivision = Subdivision {
    name: "Drugovo",
    code: "MK-28",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-28",
};

pub const MK_29: Subdivision = Subdivision {
    name: "Gjorče Petrov †",
    code: "MK-29",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-29",
};

pub const MK_30: Subdivision = Subdivision {
    name: "Želino",
    code: "MK-30",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-30",
};

pub const MK_31: Subdivision = Subdivision {
    name: "Zajas",
    code: "MK-31",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-31",
};

pub const MK_32: Subdivision = Subdivision {
    name: "Zelenikovo",
    code: "MK-32",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-32",
};

pub const MK_33: Subdivision = Subdivision {
    name: "Zrnovci",
    code: "MK-33",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-33",
};

pub const MK_34: Subdivision = Subdivision {
    name: "Ilinden",
    code: "MK-34",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-34",
};

pub const MK_35: Subdivision = Subdivision {
    name: "Jegunovce",
    code: "MK-35",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-35",
};

pub const MK_36: Subdivision = Subdivision {
    name: "Kavadarci",
    code: "MK-36",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-36",
};

pub const MK_37: Subdivision = Subdivision {
    name: "Karbinci",
    code: "MK-37",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-37",
};

pub const MK_38: Subdivision = Subdivision {
    name: "Karpoš †",
    code: "MK-38",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-38",
};

pub const MK_39: Subdivision = Subdivision {
    name: "Kisela Voda †",
    code: "MK-39",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-39",
};

pub const MK_40: Subdivision = Subdivision {
    name: "Kičevo",
    code: "MK-40",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-40",
};

pub const MK_41: Subdivision = Subdivision {
    name: "Konče",
    code: "MK-41",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-41",
};

pub const MK_42: Subdivision = Subdivision {
    name: "Kočani",
    code: "MK-42",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-42",
};

pub const MK_43: Subdivision = Subdivision {
    name: "Kratovo",
    code: "MK-43",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-43",
};

pub const MK_44: Subdivision = Subdivision {
    name: "Kriva Palanka",
    code: "MK-44",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-44",
};

pub const MK_45: Subdivision = Subdivision {
    name: "Krivogaštani",
    code: "MK-45",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-45",
};

pub const MK_46: Subdivision = Subdivision {
    name: "Kruševo",
    code: "MK-46",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-46",
};

pub const MK_47: Subdivision = Subdivision {
    name: "Kumanovo",
    code: "MK-47",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-47",
};

pub const MK_48: Subdivision = Subdivision {
    name: "Lipkovo",
    code: "MK-48",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-48",
};

pub const MK_49: Subdivision = Subdivision {
    name: "Lozovo",
    code: "MK-49",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-49",
};

pub const MK_50: Subdivision = Subdivision {
    name: "Mavrovo i Rostuša",
    code: "MK-50",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-50",
};

pub const MK_51: Subdivision = Subdivision {
    name: "Makedonska Kamenica",
    code: "MK-51",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-51",
};

pub const MK_52: Subdivision = Subdivision {
    name: "Makedonski Brod",
    code: "MK-52",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-52",
};

pub const MK_53: Subdivision = Subdivision {
    name: "Mogila",
    code: "MK-53",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-53",
};

pub const MK_54: Subdivision = Subdivision {
    name: "Negotino",
    code: "MK-54",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-54",
};

pub const MK_55: Subdivision = Subdivision {
    name: "Novaci",
    code: "MK-55",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-55",
};

pub const MK_56: Subdivision = Subdivision {
    name: "Novo Selo",
    code: "MK-56",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-56",
};

pub const MK_57: Subdivision = Subdivision {
    name: "Oslomej",
    code: "MK-57",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-57",
};

pub const MK_58: Subdivision = Subdivision {
    name: "Ohrid",
    code: "MK-58",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-58",
};

pub const MK_59: Subdivision = Subdivision {
    name: "Petrovec",
    code: "MK-59",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-59",
};

pub const MK_60: Subdivision = Subdivision {
    name: "Pehčevo",
    code: "MK-60",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-60",
};

pub const MK_61: Subdivision = Subdivision {
    name: "Plasnica",
    code: "MK-61",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-61",
};

pub const MK_62: Subdivision = Subdivision {
    name: "Prilep",
    code: "MK-62",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-62",
};

pub const MK_63: Subdivision = Subdivision {
    name: "Probištip",
    code: "MK-63",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-63",
};

pub const MK_64: Subdivision = Subdivision {
    name: "Radoviš",
    code: "MK-64",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-64",
};

pub const MK_65: Subdivision = Subdivision {
    name: "Rankovce",
    code: "MK-65",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-65",
};

pub const MK_66: Subdivision = Subdivision {
    name: "Resen",
    code: "MK-66",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-66",
};

pub const MK_67: Subdivision = Subdivision {
    name: "Rosoman",
    code: "MK-67",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-67",
};

pub const MK_68: Subdivision = Subdivision {
    name: "Saraj †",
    code: "MK-68",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-68",
};

pub const MK_69: Subdivision = Subdivision {
    name: "Sveti Nikole",
    code: "MK-69",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-69",
};

pub const MK_70: Subdivision = Subdivision {
    name: "Sopište",
    code: "MK-70",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-70",
};

pub const MK_71: Subdivision = Subdivision {
    name: "Staro Nagoričane",
    code: "MK-71",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-71",
};

pub const MK_72: Subdivision = Subdivision {
    name: "Struga",
    code: "MK-72",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-72",
};

pub const MK_73: Subdivision = Subdivision {
    name: "Strumica",
    code: "MK-73",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-73",
};

pub const MK_74: Subdivision = Subdivision {
    name: "Studeničani",
    code: "MK-74",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-74",
};

pub const MK_75: Subdivision = Subdivision {
    name: "Tearce",
    code: "MK-75",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-75",
};

pub const MK_76: Subdivision = Subdivision {
    name: "Tetovo",
    code: "MK-76",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-76",
};

pub const MK_77: Subdivision = Subdivision {
    name: "Centar †",
    code: "MK-77",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-77",
};

pub const MK_78: Subdivision = Subdivision {
    name: "Centar Župa",
    code: "MK-78",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-78",
};

pub const MK_79: Subdivision = Subdivision {
    name: "Čair †",
    code: "MK-79",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-79",
};

pub const MK_80: Subdivision = Subdivision {
    name: "Čaška",
    code: "MK-80",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-80",
};

pub const MK_81: Subdivision = Subdivision {
    name: "Češinovo-Obleševo",
    code: "MK-81",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-81",
};

pub const MK_82: Subdivision = Subdivision {
    name: "Čučer Sandevo",
    code: "MK-82",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-82",
};

pub const MK_83: Subdivision = Subdivision {
    name: "Štip",
    code: "MK-83",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-83",
};

pub const MK_84: Subdivision = Subdivision {
    name: "Šuto Orizari †",
    code: "MK-84",
    subdivision_type: "Municipality",
    country_name: "Macedonia, the Former Yugoslav Republic Of",
    country_code: "MK",
    region_code: "MK-84",
};

pub const MG_A: Subdivision = Subdivision {
    name: "Toamasina",
    code: "MG-A",
    subdivision_type: "Province",
    country_name: "Madagascar",
    country_code: "MG",
    region_code: "MG-A",
};

pub const MG_D: Subdivision = Subdivision {
    name: "Antsiranana",
    code: "MG-D",
    subdivision_type: "Province",
    country_name: "Madagascar",
    country_code: "MG",
    region_code: "MG-D",
};

pub const MG_F: Subdivision = Subdivision {
    name: "Fianarantsoa",
    code: "MG-F",
    subdivision_type: "Province",
    country_name: "Madagascar",
    country_code: "MG",
    region_code: "MG-F",
};

pub const MG_M: Subdivision = Subdivision {
    name: "Mahajanga",
    code: "MG-M",
    subdivision_type: "Province",
    country_name: "Madagascar",
    country_code: "MG",
    region_code: "MG-M",
};

pub const MG_T: Subdivision = Subdivision {
    name: "Antananarivo",
    code: "MG-T",
    subdivision_type: "Province",
    country_name: "Madagascar",
    country_code: "MG",
    region_code: "MG-T",
};

pub const MG_U: Subdivision = Subdivision {
    name: "Toliara",
    code: "MG-U",
    subdivision_type: "Province",
    country_name: "Madagascar",
    country_code: "MG",
    region_code: "MG-U",
};

pub const MW_BA: Subdivision = Subdivision {
    name: "Balaka",
    code: "MW-BA",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-BA",
};

pub const MW_BL: Subdivision = Subdivision {
    name: "Blantyre",
    code: "MW-BL",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-BL",
};

pub const MW_C: Subdivision = Subdivision {
    name: "Central Region",
    code: "MW-C",
    subdivision_type: "Region",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-C",
};

pub const MW_CK: Subdivision = Subdivision {
    name: "Chikwawa",
    code: "MW-CK",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-CK",
};

pub const MW_CR: Subdivision = Subdivision {
    name: "Chiradzulu",
    code: "MW-CR",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-CR",
};

pub const MW_CT: Subdivision = Subdivision {
    name: "Chitipa",
    code: "MW-CT",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-CT",
};

pub const MW_DE: Subdivision = Subdivision {
    name: "Dedza",
    code: "MW-DE",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-DE",
};

pub const MW_DO: Subdivision = Subdivision {
    name: "Dowa",
    code: "MW-DO",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-DO",
};

pub const MW_KR: Subdivision = Subdivision {
    name: "Karonga",
    code: "MW-KR",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-KR",
};

pub const MW_KS: Subdivision = Subdivision {
    name: "Kasungu",
    code: "MW-KS",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-KS",
};

pub const MW_LI: Subdivision = Subdivision {
    name: "Lilongwe",
    code: "MW-LI",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-LI",
};

pub const MW_LK: Subdivision = Subdivision {
    name: "Likoma",
    code: "MW-LK",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-LK",
};

pub const MW_MC: Subdivision = Subdivision {
    name: "Mchinji",
    code: "MW-MC",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-MC",
};

pub const MW_MG: Subdivision = Subdivision {
    name: "Mangochi",
    code: "MW-MG",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-MG",
};

pub const MW_MH: Subdivision = Subdivision {
    name: "Machinga",
    code: "MW-MH",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-MH",
};

pub const MW_MU: Subdivision = Subdivision {
    name: "Mulanje",
    code: "MW-MU",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-MU",
};

pub const MW_MW: Subdivision = Subdivision {
    name: "Mwanza",
    code: "MW-MW",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-MW",
};

pub const MW_MZ: Subdivision = Subdivision {
    name: "Mzimba",
    code: "MW-MZ",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-MZ",
};

pub const MW_NB: Subdivision = Subdivision {
    name: "Nkhata Bay",
    code: "MW-NB",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-NB",
};

pub const MW_N: Subdivision = Subdivision {
    name: "Chakumpoto",
    code: "MW-N",
    subdivision_type: "Region",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-N",
};

pub const MW_NE: Subdivision = Subdivision {
    name: "Neno",
    code: "MW-NE",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-NE",
};

pub const MW_NI: Subdivision = Subdivision {
    name: "Ntchisi",
    code: "MW-NI",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-NI",
};

pub const MW_NK: Subdivision = Subdivision {
    name: "Nkhotakota",
    code: "MW-NK",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-NK",
};

pub const MW_NS: Subdivision = Subdivision {
    name: "Nsanje",
    code: "MW-NS",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-NS",
};

pub const MW_NU: Subdivision = Subdivision {
    name: "Ntcheu",
    code: "MW-NU",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-NU",
};

pub const MW_PH: Subdivision = Subdivision {
    name: "Phalombe",
    code: "MW-PH",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-PH",
};

pub const MW_RU: Subdivision = Subdivision {
    name: "Rumphi",
    code: "MW-RU",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-RU",
};

pub const MW_SA: Subdivision = Subdivision {
    name: "Salima",
    code: "MW-SA",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-SA",
};

pub const MW_S: Subdivision = Subdivision {
    name: "Chakumwera",
    code: "MW-S",
    subdivision_type: "Region",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-S",
};

pub const MW_TH: Subdivision = Subdivision {
    name: "Thyolo",
    code: "MW-TH",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-TH",
};

pub const MW_ZO: Subdivision = Subdivision {
    name: "Zomba",
    code: "MW-ZO",
    subdivision_type: "District",
    country_name: "Malawi",
    country_code: "MW",
    region_code: "MW-ZO",
};

pub const MY_01: Subdivision = Subdivision {
    name: "Johor",
    code: "MY-01",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-01",
};

pub const MY_02: Subdivision = Subdivision {
    name: "Kedah",
    code: "MY-02",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-02",
};

pub const MY_03: Subdivision = Subdivision {
    name: "Kelantan",
    code: "MY-03",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-03",
};

pub const MY_04: Subdivision = Subdivision {
    name: "Melaka",
    code: "MY-04",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-04",
};

pub const MY_05: Subdivision = Subdivision {
    name: "Negeri Sembilan",
    code: "MY-05",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-05",
};

pub const MY_06: Subdivision = Subdivision {
    name: "Pahang",
    code: "MY-06",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-06",
};

pub const MY_07: Subdivision = Subdivision {
    name: "Pulau Pinang",
    code: "MY-07",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-07",
};

pub const MY_08: Subdivision = Subdivision {
    name: "Perak",
    code: "MY-08",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-08",
};

pub const MY_09: Subdivision = Subdivision {
    name: "Perlis",
    code: "MY-09",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-09",
};

pub const MY_10: Subdivision = Subdivision {
    name: "Selangor",
    code: "MY-10",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-10",
};

pub const MY_11: Subdivision = Subdivision {
    name: "Terengganu",
    code: "MY-11",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-11",
};

pub const MY_12: Subdivision = Subdivision {
    name: "Sabah",
    code: "MY-12",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-12",
};

pub const MY_13: Subdivision = Subdivision {
    name: "Sarawak",
    code: "MY-13",
    subdivision_type: "State",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-13",
};

pub const MY_14: Subdivision = Subdivision {
    name: "Wilayah Persekutuan Kuala Lumpur",
    code: "MY-14",
    subdivision_type: "Federal territory",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-14",
};

pub const MY_15: Subdivision = Subdivision {
    name: "Wilayah Persekutuan Labuan",
    code: "MY-15",
    subdivision_type: "Federal territory",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-15",
};

pub const MY_16: Subdivision = Subdivision {
    name: "Wilayah Persekutuan Putrajaya",
    code: "MY-16",
    subdivision_type: "Federal territory",
    country_name: "Malaysia",
    country_code: "MY",
    region_code: "MY-16",
};

pub const MV_00: Subdivision = Subdivision {
    name: "Alifu Dhaalu",
    code: "MV-00",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-00",
};

pub const MV_01: Subdivision = Subdivision {
    name: "Addu Atholhu",
    code: "MV-01",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-01",
};

pub const MV_02: Subdivision = Subdivision {
    name: "Alifu Alifu",
    code: "MV-02",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-02",
};

pub const MV_03: Subdivision = Subdivision {
    name: "Faadhippolhu",
    code: "MV-03",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-03",
};

pub const MV_04: Subdivision = Subdivision {
    name: "Felidhu Atholhu",
    code: "MV-04",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-04",
};

pub const MV_05: Subdivision = Subdivision {
    name: "Hadhdhunmathi",
    code: "MV-05",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-05",
};

pub const MV_07: Subdivision = Subdivision {
    name: "Haa Alifu",
    code: "MV-07",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-07",
};

pub const MV_08: Subdivision = Subdivision {
    name: "Kolhumadulu",
    code: "MV-08",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-08",
};

pub const MV_12: Subdivision = Subdivision {
    name: "Meemu",
    code: "MV-12",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-12",
};

pub const MV_13: Subdivision = Subdivision {
    name: "Maalhosmadulu Uthuruburi",
    code: "MV-13",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-13",
};

pub const MV_14: Subdivision = Subdivision {
    name: "Faafu",
    code: "MV-14",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-14",
};

pub const MV_17: Subdivision = Subdivision {
    name: "Dhaalu",
    code: "MV-17",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-17",
};

pub const MV_20: Subdivision = Subdivision {
    name: "Baa",
    code: "MV-20",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-20",
};

pub const MV_23: Subdivision = Subdivision {
    name: "Haa Dhaalu",
    code: "MV-23",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-23",
};

pub const MV_24: Subdivision = Subdivision {
    name: "Miladhunmadulu Uthuruburi",
    code: "MV-24",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-24",
};

pub const MV_25: Subdivision = Subdivision {
    name: "Miladhunmadulu Dhekunuburi",
    code: "MV-25",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-25",
};

pub const MV_26: Subdivision = Subdivision {
    name: "Kaafu",
    code: "MV-26",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-26",
};

pub const MV_27: Subdivision = Subdivision {
    name: "Gaafu Alifu",
    code: "MV-27",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-27",
};

pub const MV_28: Subdivision = Subdivision {
    name: "Gaafu Dhaalu",
    code: "MV-28",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-28",
};

pub const MV_29: Subdivision = Subdivision {
    name: "Fuvammulah",
    code: "MV-29",
    subdivision_type: "Administrative atoll",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-29",
};

pub const MV_CE: Subdivision = Subdivision {
    name: "Central",
    code: "MV-CE",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-CE",
};

pub const MV_MLE: Subdivision = Subdivision {
    name: "Maale",
    code: "MV-MLE",
    subdivision_type: "Capital",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-MLE",
};

pub const MV_NC: Subdivision = Subdivision {
    name: "Medhu-Uthuru",
    code: "MV-NC",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-NC",
};

pub const MV_NO: Subdivision = Subdivision {
    name: "North",
    code: "MV-NO",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-NO",
};

pub const MV_SC: Subdivision = Subdivision {
    name: "Medhu-Dhekunu",
    code: "MV-SC",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-SC",
};

pub const MV_SU: Subdivision = Subdivision {
    name: "Dhekunu",
    code: "MV-SU",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-SU",
};

pub const MV_UN: Subdivision = Subdivision {
    name: "Mathi-Uthuru",
    code: "MV-UN",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-UN",
};

pub const MV_US: Subdivision = Subdivision {
    name: "Mathi-Dhekunu",
    code: "MV-US",
    subdivision_type: "Province",
    country_name: "Maldives",
    country_code: "MV",
    region_code: "MV-US",
};

pub const ML_1: Subdivision = Subdivision {
    name: "Kayes",
    code: "ML-1",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-1",
};

pub const ML_2: Subdivision = Subdivision {
    name: "Koulikoro",
    code: "ML-2",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-2",
};

pub const ML_3: Subdivision = Subdivision {
    name: "Sikasso",
    code: "ML-3",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-3",
};

pub const ML_4: Subdivision = Subdivision {
    name: "Ségou",
    code: "ML-4",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-4",
};

pub const ML_5: Subdivision = Subdivision {
    name: "Mopti",
    code: "ML-5",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-5",
};

pub const ML_6: Subdivision = Subdivision {
    name: "Tombouctou",
    code: "ML-6",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-6",
};

pub const ML_7: Subdivision = Subdivision {
    name: "Gao",
    code: "ML-7",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-7",
};

pub const ML_8: Subdivision = Subdivision {
    name: "Kidal",
    code: "ML-8",
    subdivision_type: "Region",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-8",
};

pub const ML_BKO: Subdivision = Subdivision {
    name: "Bamako",
    code: "ML-BKO",
    subdivision_type: "District",
    country_name: "Mali",
    country_code: "ML",
    region_code: "ML-BKO",
};

pub const MT_01: Subdivision = Subdivision {
    name: "Attard",
    code: "MT-01",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-01",
};

pub const MT_02: Subdivision = Subdivision {
    name: "Balzan",
    code: "MT-02",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-02",
};

pub const MT_03: Subdivision = Subdivision {
    name: "Birgu",
    code: "MT-03",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-03",
};

pub const MT_04: Subdivision = Subdivision {
    name: "Birkirkara",
    code: "MT-04",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-04",
};

pub const MT_05: Subdivision = Subdivision {
    name: "Birżebbuġa",
    code: "MT-05",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-05",
};

pub const MT_06: Subdivision = Subdivision {
    name: "Bormla",
    code: "MT-06",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-06",
};

pub const MT_07: Subdivision = Subdivision {
    name: "Dingli",
    code: "MT-07",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-07",
};

pub const MT_08: Subdivision = Subdivision {
    name: "Fgura",
    code: "MT-08",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-08",
};

pub const MT_09: Subdivision = Subdivision {
    name: "Floriana",
    code: "MT-09",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-09",
};

pub const MT_10: Subdivision = Subdivision {
    name: "Fontana",
    code: "MT-10",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-10",
};

pub const MT_11: Subdivision = Subdivision {
    name: "Gudja",
    code: "MT-11",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-11",
};

pub const MT_12: Subdivision = Subdivision {
    name: "Gżira",
    code: "MT-12",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-12",
};

pub const MT_13: Subdivision = Subdivision {
    name: "Għajnsielem",
    code: "MT-13",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-13",
};

pub const MT_14: Subdivision = Subdivision {
    name: "Għarb",
    code: "MT-14",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-14",
};

pub const MT_15: Subdivision = Subdivision {
    name: "Għargħur",
    code: "MT-15",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-15",
};

pub const MT_16: Subdivision = Subdivision {
    name: "Għasri",
    code: "MT-16",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-16",
};

pub const MT_17: Subdivision = Subdivision {
    name: "Għaxaq",
    code: "MT-17",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-17",
};

pub const MT_18: Subdivision = Subdivision {
    name: "Ħamrun",
    code: "MT-18",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-18",
};

pub const MT_19: Subdivision = Subdivision {
    name: "Iklin",
    code: "MT-19",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-19",
};

pub const MT_20: Subdivision = Subdivision {
    name: "Isla",
    code: "MT-20",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-20",
};

pub const MT_21: Subdivision = Subdivision {
    name: "Kalkara",
    code: "MT-21",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-21",
};

pub const MT_22: Subdivision = Subdivision {
    name: "Kerċem",
    code: "MT-22",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-22",
};

pub const MT_23: Subdivision = Subdivision {
    name: "Kirkop",
    code: "MT-23",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-23",
};

pub const MT_24: Subdivision = Subdivision {
    name: "Lija",
    code: "MT-24",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-24",
};

pub const MT_25: Subdivision = Subdivision {
    name: "Luqa",
    code: "MT-25",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-25",
};

pub const MT_26: Subdivision = Subdivision {
    name: "Marsa",
    code: "MT-26",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-26",
};

pub const MT_27: Subdivision = Subdivision {
    name: "Marsaskala",
    code: "MT-27",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-27",
};

pub const MT_28: Subdivision = Subdivision {
    name: "Marsaxlokk",
    code: "MT-28",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-28",
};

pub const MT_29: Subdivision = Subdivision {
    name: "Mdina",
    code: "MT-29",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-29",
};

pub const MT_30: Subdivision = Subdivision {
    name: "Mellieħa",
    code: "MT-30",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-30",
};

pub const MT_31: Subdivision = Subdivision {
    name: "Mġarr",
    code: "MT-31",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-31",
};

pub const MT_32: Subdivision = Subdivision {
    name: "Mosta",
    code: "MT-32",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-32",
};

pub const MT_33: Subdivision = Subdivision {
    name: "Mqabba",
    code: "MT-33",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-33",
};

pub const MT_34: Subdivision = Subdivision {
    name: "Msida",
    code: "MT-34",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-34",
};

pub const MT_35: Subdivision = Subdivision {
    name: "Mtarfa",
    code: "MT-35",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-35",
};

pub const MT_36: Subdivision = Subdivision {
    name: "Munxar",
    code: "MT-36",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-36",
};

pub const MT_37: Subdivision = Subdivision {
    name: "Nadur",
    code: "MT-37",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-37",
};

pub const MT_38: Subdivision = Subdivision {
    name: "Naxxar",
    code: "MT-38",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-38",
};

pub const MT_39: Subdivision = Subdivision {
    name: "Paola",
    code: "MT-39",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-39",
};

pub const MT_40: Subdivision = Subdivision {
    name: "Pembroke",
    code: "MT-40",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-40",
};

pub const MT_41: Subdivision = Subdivision {
    name: "Pietà",
    code: "MT-41",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-41",
};

pub const MT_42: Subdivision = Subdivision {
    name: "Qala",
    code: "MT-42",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-42",
};

pub const MT_43: Subdivision = Subdivision {
    name: "Qormi",
    code: "MT-43",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-43",
};

pub const MT_44: Subdivision = Subdivision {
    name: "Qrendi",
    code: "MT-44",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-44",
};

pub const MT_45: Subdivision = Subdivision {
    name: "Rabat Gozo",
    code: "MT-45",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-45",
};

pub const MT_46: Subdivision = Subdivision {
    name: "Rabat Malta",
    code: "MT-46",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-46",
};

pub const MT_47: Subdivision = Subdivision {
    name: "Safi",
    code: "MT-47",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-47",
};

pub const MT_48: Subdivision = Subdivision {
    name: "Saint Julian’s",
    code: "MT-48",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-48",
};

pub const MT_49: Subdivision = Subdivision {
    name: "Saint John",
    code: "MT-49",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-49",
};

pub const MT_50: Subdivision = Subdivision {
    name: "Saint Lawrence",
    code: "MT-50",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-50",
};

pub const MT_51: Subdivision = Subdivision {
    name: "Saint Paul’s Bay",
    code: "MT-51",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-51",
};

pub const MT_52: Subdivision = Subdivision {
    name: "Sannat",
    code: "MT-52",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-52",
};

pub const MT_53: Subdivision = Subdivision {
    name: "Saint Lucia’s",
    code: "MT-53",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-53",
};

pub const MT_54: Subdivision = Subdivision {
    name: "Santa Venera",
    code: "MT-54",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-54",
};

pub const MT_55: Subdivision = Subdivision {
    name: "Siġġiewi",
    code: "MT-55",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-55",
};

pub const MT_56: Subdivision = Subdivision {
    name: "Sliema",
    code: "MT-56",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-56",
};

pub const MT_57: Subdivision = Subdivision {
    name: "Swieqi",
    code: "MT-57",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-57",
};

pub const MT_58: Subdivision = Subdivision {
    name: "Ta’ Xbiex",
    code: "MT-58",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-58",
};

pub const MT_59: Subdivision = Subdivision {
    name: "Tarxien",
    code: "MT-59",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-59",
};

pub const MT_60: Subdivision = Subdivision {
    name: "Valletta",
    code: "MT-60",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-60",
};

pub const MT_61: Subdivision = Subdivision {
    name: "Xagħra",
    code: "MT-61",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-61",
};

pub const MT_62: Subdivision = Subdivision {
    name: "Xewkija",
    code: "MT-62",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-62",
};

pub const MT_63: Subdivision = Subdivision {
    name: "Xgħajra",
    code: "MT-63",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-63",
};

pub const MT_64: Subdivision = Subdivision {
    name: "Żabbar",
    code: "MT-64",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-64",
};

pub const MT_65: Subdivision = Subdivision {
    name: "Żebbuġ Gozo",
    code: "MT-65",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-65",
};

pub const MT_66: Subdivision = Subdivision {
    name: "Żebbuġ Malta",
    code: "MT-66",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-66",
};

pub const MT_67: Subdivision = Subdivision {
    name: "Żejtun",
    code: "MT-67",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-67",
};

pub const MT_68: Subdivision = Subdivision {
    name: "Żurrieq",
    code: "MT-68",
    subdivision_type: "Local council",
    country_name: "Malta",
    country_code: "MT",
    region_code: "MT-68",
};

pub const MH_ALK: Subdivision = Subdivision {
    name: "Ailuk",
    code: "MH-ALK",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-ALK",
};

pub const MH_ALL: Subdivision = Subdivision {
    name: "Ailinglaplap",
    code: "MH-ALL",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-ALL",
};

pub const MH_ARN: Subdivision = Subdivision {
    name: "Arno",
    code: "MH-ARN",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-ARN",
};

pub const MH_AUR: Subdivision = Subdivision {
    name: "Aur",
    code: "MH-AUR",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-AUR",
};

pub const MH_EBO: Subdivision = Subdivision {
    name: "Ebon",
    code: "MH-EBO",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-EBO",
};

pub const MH_ENI: Subdivision = Subdivision {
    name: "Enewetak and Ujelang",
    code: "MH-ENI",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-ENI",
};

pub const MH_JAB: Subdivision = Subdivision {
    name: "Jabat",
    code: "MH-JAB",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-JAB",
};

pub const MH_JAL: Subdivision = Subdivision {
    name: "Jaluit",
    code: "MH-JAL",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-JAL",
};

pub const MH_KIL: Subdivision = Subdivision {
    name: "Bikini and Kili",
    code: "MH-KIL",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-KIL",
};

pub const MH_KWA: Subdivision = Subdivision {
    name: "Kwajalein",
    code: "MH-KWA",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-KWA",
};

pub const MH_LAE: Subdivision = Subdivision {
    name: "Lae",
    code: "MH-LAE",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-LAE",
};

pub const MH_LIB: Subdivision = Subdivision {
    name: "Lib",
    code: "MH-LIB",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-LIB",
};

pub const MH_LIK: Subdivision = Subdivision {
    name: "Likiep",
    code: "MH-LIK",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-LIK",
};

pub const MH_L: Subdivision = Subdivision {
    name: "Ralik chain",
    code: "MH-L",
    subdivision_type: "Chain",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-L",
};

pub const MH_MAJ: Subdivision = Subdivision {
    name: "Majuro",
    code: "MH-MAJ",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-MAJ",
};

pub const MH_MAL: Subdivision = Subdivision {
    name: "Maloelap",
    code: "MH-MAL",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-MAL",
};

pub const MH_MEJ: Subdivision = Subdivision {
    name: "Mejit",
    code: "MH-MEJ",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-MEJ",
};

pub const MH_MIL: Subdivision = Subdivision {
    name: "Mili",
    code: "MH-MIL",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-MIL",
};

pub const MH_NMK: Subdivision = Subdivision {
    name: "Namdrik",
    code: "MH-NMK",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-NMK",
};

pub const MH_NMU: Subdivision = Subdivision {
    name: "Namu",
    code: "MH-NMU",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-NMU",
};

pub const MH_RON: Subdivision = Subdivision {
    name: "Rongelap",
    code: "MH-RON",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-RON",
};

pub const MH_T: Subdivision = Subdivision {
    name: "Ratak chain",
    code: "MH-T",
    subdivision_type: "Chain",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-T",
};

pub const MH_UJA: Subdivision = Subdivision {
    name: "Ujae",
    code: "MH-UJA",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-UJA",
};

pub const MH_UTI: Subdivision = Subdivision {
    name: "Utrik",
    code: "MH-UTI",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-UTI",
};

pub const MH_WTH: Subdivision = Subdivision {
    name: "Wotho",
    code: "MH-WTH",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-WTH",
};

pub const MH_WTJ: Subdivision = Subdivision {
    name: "Wotje",
    code: "MH-WTJ",
    subdivision_type: "Municipality",
    country_name: "Marshall Islands",
    country_code: "MH",
    region_code: "MH-WTJ",
};

pub const MQ_MQ: Subdivision = Subdivision {
    name: "Martinique",
    code: "MQ-MQ",
    subdivision_type: "Country",
    country_name: "Martinique",
    country_code: "MQ",
    region_code: "MQ-MQ",
};

pub const MR_01: Subdivision = Subdivision {
    name: "Hodh ech Chargui",
    code: "MR-01",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-01",
};

pub const MR_02: Subdivision = Subdivision {
    name: "Hodh el Gharbi",
    code: "MR-02",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-02",
};

pub const MR_03: Subdivision = Subdivision {
    name: "Assaba",
    code: "MR-03",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-03",
};

pub const MR_04: Subdivision = Subdivision {
    name: "Gorgol",
    code: "MR-04",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-04",
};

pub const MR_05: Subdivision = Subdivision {
    name: "Brakna",
    code: "MR-05",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-05",
};

pub const MR_06: Subdivision = Subdivision {
    name: "Trarza",
    code: "MR-06",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-06",
};

pub const MR_07: Subdivision = Subdivision {
    name: "Adrar",
    code: "MR-07",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-07",
};

pub const MR_08: Subdivision = Subdivision {
    name: "Dakhlet Nouâdhibou",
    code: "MR-08",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-08",
};

pub const MR_09: Subdivision = Subdivision {
    name: "Tagant",
    code: "MR-09",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-09",
};

pub const MR_10: Subdivision = Subdivision {
    name: "Guidimaka",
    code: "MR-10",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-10",
};

pub const MR_11: Subdivision = Subdivision {
    name: "Tiris Zemmour",
    code: "MR-11",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-11",
};

pub const MR_12: Subdivision = Subdivision {
    name: "Inchiri",
    code: "MR-12",
    subdivision_type: "Region",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-12",
};

pub const MR_NKC: Subdivision = Subdivision {
    name: "Nouakchott",
    code: "MR-NKC",
    subdivision_type: "District",
    country_name: "Mauritania",
    country_code: "MR",
    region_code: "MR-NKC",
};

pub const MU_AG: Subdivision = Subdivision {
    name: "Agalega Islands",
    code: "MU-AG",
    subdivision_type: "Dependency",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-AG",
};

pub const MU_BL: Subdivision = Subdivision {
    name: "Black River",
    code: "MU-BL",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-BL",
};

pub const MU_BR: Subdivision = Subdivision {
    name: "Beau Bassin-Rose Hill",
    code: "MU-BR",
    subdivision_type: "City",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-BR",
};

pub const MU_CC: Subdivision = Subdivision {
    name: "Cargados Carajos Shoals",
    code: "MU-CC",
    subdivision_type: "Dependency",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-CC",
};

pub const MU_CU: Subdivision = Subdivision {
    name: "Curepipe",
    code: "MU-CU",
    subdivision_type: "City",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-CU",
};

pub const MU_FL: Subdivision = Subdivision {
    name: "Flacq",
    code: "MU-FL",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-FL",
};

pub const MU_GP: Subdivision = Subdivision {
    name: "Grand Port",
    code: "MU-GP",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-GP",
};

pub const MU_MO: Subdivision = Subdivision {
    name: "Moka",
    code: "MU-MO",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-MO",
};

pub const MU_PA: Subdivision = Subdivision {
    name: "Pamplemousses",
    code: "MU-PA",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-PA",
};

pub const MU_PL: Subdivision = Subdivision {
    name: "Port Louis",
    code: "MU-PL",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-PL",
};

pub const MU_PU: Subdivision = Subdivision {
    name: "Port Louis",
    code: "MU-PU",
    subdivision_type: "City",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-PU",
};

pub const MU_PW: Subdivision = Subdivision {
    name: "Plaines Wilhems",
    code: "MU-PW",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-PW",
};

pub const MU_QB: Subdivision = Subdivision {
    name: "Quatre Bornes",
    code: "MU-QB",
    subdivision_type: "City",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-QB",
};

pub const MU_RO: Subdivision = Subdivision {
    name: "Rodrigues Island",
    code: "MU-RO",
    subdivision_type: "Dependency",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-RO",
};

pub const MU_RR: Subdivision = Subdivision {
    name: "Rivière du Rempart",
    code: "MU-RR",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-RR",
};

pub const MU_SA: Subdivision = Subdivision {
    name: "Savanne",
    code: "MU-SA",
    subdivision_type: "District",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-SA",
};

pub const MU_VP: Subdivision = Subdivision {
    name: "Vacoas-Phoenix",
    code: "MU-VP",
    subdivision_type: "City",
    country_name: "Mauritius",
    country_code: "MU",
    region_code: "MU-VP",
};

pub const YT_YT: Subdivision = Subdivision {
    name: "Mayotte",
    code: "YT-YT",
    subdivision_type: "Country",
    country_name: "Mayotte",
    country_code: "YT",
    region_code: "YT-YT",
};

pub const MX_AGU: Subdivision = Subdivision {
    name: "Aguascalientes",
    code: "MX-AGU",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-AGU",
};

pub const MX_BCN: Subdivision = Subdivision {
    name: "Baja California",
    code: "MX-BCN",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-BCN",
};

pub const MX_BCS: Subdivision = Subdivision {
    name: "Baja California Sur",
    code: "MX-BCS",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-BCS",
};

pub const MX_CAM: Subdivision = Subdivision {
    name: "Campeche",
    code: "MX-CAM",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-CAM",
};

pub const MX_CHH: Subdivision = Subdivision {
    name: "Chihuahua",
    code: "MX-CHH",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-CHH",
};

pub const MX_CHP: Subdivision = Subdivision {
    name: "Chiapas",
    code: "MX-CHP",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-CHP",
};

pub const MX_COA: Subdivision = Subdivision {
    name: "Coahuila de Zaragoza",
    code: "MX-COA",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-COA",
};

pub const MX_COL: Subdivision = Subdivision {
    name: "Colima",
    code: "MX-COL",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-COL",
};

pub const MX_CMX: Subdivision = Subdivision {
    name: "Ciudad de México",
    code: "MX-CMX",
    subdivision_type: "Federal District",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-CMX",
};

pub const MX_DUR: Subdivision = Subdivision {
    name: "Durango",
    code: "MX-DUR",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-DUR",
};

pub const MX_GRO: Subdivision = Subdivision {
    name: "Guerrero",
    code: "MX-GRO",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-GRO",
};

pub const MX_GUA: Subdivision = Subdivision {
    name: "Guanajuato",
    code: "MX-GUA",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-GUA",
};

pub const MX_HID: Subdivision = Subdivision {
    name: "Hidalgo",
    code: "MX-HID",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-HID",
};

pub const MX_JAL: Subdivision = Subdivision {
    name: "Jalisco",
    code: "MX-JAL",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-JAL",
};

pub const MX_MEX: Subdivision = Subdivision {
    name: "México",
    code: "MX-MEX",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-MEX",
};

pub const MX_MIC: Subdivision = Subdivision {
    name: "Michoacán de Ocampo",
    code: "MX-MIC",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-MIC",
};

pub const MX_MOR: Subdivision = Subdivision {
    name: "Morelos",
    code: "MX-MOR",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-MOR",
};

pub const MX_NAY: Subdivision = Subdivision {
    name: "Nayarit",
    code: "MX-NAY",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-NAY",
};

pub const MX_NLE: Subdivision = Subdivision {
    name: "Nuevo León",
    code: "MX-NLE",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-NLE",
};

pub const MX_OAX: Subdivision = Subdivision {
    name: "Oaxaca",
    code: "MX-OAX",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-OAX",
};

pub const MX_PUE: Subdivision = Subdivision {
    name: "Puebla",
    code: "MX-PUE",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-PUE",
};

pub const MX_QUE: Subdivision = Subdivision {
    name: "Querétaro",
    code: "MX-QUE",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-QUE",
};

pub const MX_ROO: Subdivision = Subdivision {
    name: "Quintana Roo",
    code: "MX-ROO",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-ROO",
};

pub const MX_SIN: Subdivision = Subdivision {
    name: "Sinaloa",
    code: "MX-SIN",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-SIN",
};

pub const MX_SLP: Subdivision = Subdivision {
    name: "San Luis Potosí",
    code: "MX-SLP",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-SLP",
};

pub const MX_SON: Subdivision = Subdivision {
    name: "Sonora",
    code: "MX-SON",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-SON",
};

pub const MX_TAB: Subdivision = Subdivision {
    name: "Tabasco",
    code: "MX-TAB",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-TAB",
};

pub const MX_TAM: Subdivision = Subdivision {
    name: "Tamaulipas",
    code: "MX-TAM",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-TAM",
};

pub const MX_TLA: Subdivision = Subdivision {
    name: "Tlaxcala",
    code: "MX-TLA",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-TLA",
};

pub const MX_VER: Subdivision = Subdivision {
    name: "Veracruz de Ignacio de la Llave",
    code: "MX-VER",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-VER",
};

pub const MX_YUC: Subdivision = Subdivision {
    name: "Yucatán",
    code: "MX-YUC",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-YUC",
};

pub const MX_ZAC: Subdivision = Subdivision {
    name: "Zacatecas",
    code: "MX-ZAC",
    subdivision_type: "State",
    country_name: "Mexico",
    country_code: "MX",
    region_code: "MX-ZAC",
};

pub const FM_KSA: Subdivision = Subdivision {
    name: "Kosrae",
    code: "FM-KSA",
    subdivision_type: "State",
    country_name: "Micronesia, Federated States Of",
    country_code: "FM",
    region_code: "FM-KSA",
};

pub const FM_PNI: Subdivision = Subdivision {
    name: "Pohnpei",
    code: "FM-PNI",
    subdivision_type: "State",
    country_name: "Micronesia, Federated States Of",
    country_code: "FM",
    region_code: "FM-PNI",
};

pub const FM_TRK: Subdivision = Subdivision {
    name: "Chuuk",
    code: "FM-TRK",
    subdivision_type: "State",
    country_name: "Micronesia, Federated States Of",
    country_code: "FM",
    region_code: "FM-TRK",
};

pub const FM_YAP: Subdivision = Subdivision {
    name: "Yap",
    code: "FM-YAP",
    subdivision_type: "State",
    country_name: "Micronesia, Federated States Of",
    country_code: "FM",
    region_code: "FM-YAP",
};

pub const MD_AN: Subdivision = Subdivision {
    name: "Anenii Noi",
    code: "MD-AN",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-AN",
};

pub const MD_BA: Subdivision = Subdivision {
    name: "Bălţi",
    code: "MD-BA",
    subdivision_type: "City",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-BA",
};

pub const MD_BD: Subdivision = Subdivision {
    name: "Bender",
    code: "MD-BD",
    subdivision_type: "City",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-BD",
};

pub const MD_BR: Subdivision = Subdivision {
    name: "Briceni",
    code: "MD-BR",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-BR",
};

pub const MD_BS: Subdivision = Subdivision {
    name: "Basarabeasca",
    code: "MD-BS",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-BS",
};

pub const MD_CA: Subdivision = Subdivision {
    name: "Cahul",
    code: "MD-CA",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CA",
};

pub const MD_CL: Subdivision = Subdivision {
    name: "Călăraşi",
    code: "MD-CL",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CL",
};

pub const MD_CM: Subdivision = Subdivision {
    name: "Cimişlia",
    code: "MD-CM",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CM",
};

pub const MD_CR: Subdivision = Subdivision {
    name: "Criuleni",
    code: "MD-CR",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CR",
};

pub const MD_CS: Subdivision = Subdivision {
    name: "Căuşeni",
    code: "MD-CS",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CS",
};

pub const MD_CT: Subdivision = Subdivision {
    name: "Cantemir",
    code: "MD-CT",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CT",
};

pub const MD_CU: Subdivision = Subdivision {
    name: "Chişinău",
    code: "MD-CU",
    subdivision_type: "City",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-CU",
};

pub const MD_DO: Subdivision = Subdivision {
    name: "Donduşeni",
    code: "MD-DO",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-DO",
};

pub const MD_DR: Subdivision = Subdivision {
    name: "Drochia",
    code: "MD-DR",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-DR",
};

pub const MD_DU: Subdivision = Subdivision {
    name: "Dubăsari",
    code: "MD-DU",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-DU",
};

pub const MD_ED: Subdivision = Subdivision {
    name: "Edineţ",
    code: "MD-ED",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-ED",
};

pub const MD_FA: Subdivision = Subdivision {
    name: "Făleşti",
    code: "MD-FA",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-FA",
};

pub const MD_FL: Subdivision = Subdivision {
    name: "Floreşti",
    code: "MD-FL",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-FL",
};

pub const MD_GA: Subdivision = Subdivision {
    name: "Găgăuzia, Unitatea teritorială autonomă",
    code: "MD-GA",
    subdivision_type: "Autonomous territorial unit",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-GA",
};

pub const MD_GL: Subdivision = Subdivision {
    name: "Glodeni",
    code: "MD-GL",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-GL",
};

pub const MD_HI: Subdivision = Subdivision {
    name: "Hînceşti",
    code: "MD-HI",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-HI",
};

pub const MD_IA: Subdivision = Subdivision {
    name: "Ialoveni",
    code: "MD-IA",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-IA",
};

pub const MD_LE: Subdivision = Subdivision {
    name: "Leova",
    code: "MD-LE",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-LE",
};

pub const MD_NI: Subdivision = Subdivision {
    name: "Nisporeni",
    code: "MD-NI",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-NI",
};

pub const MD_OC: Subdivision = Subdivision {
    name: "Ocniţa",
    code: "MD-OC",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-OC",
};

pub const MD_OR: Subdivision = Subdivision {
    name: "Orhei",
    code: "MD-OR",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-OR",
};

pub const MD_RE: Subdivision = Subdivision {
    name: "Rezina",
    code: "MD-RE",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-RE",
};

pub const MD_RI: Subdivision = Subdivision {
    name: "Rîşcani",
    code: "MD-RI",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-RI",
};

pub const MD_SD: Subdivision = Subdivision {
    name: "Şoldăneşti",
    code: "MD-SD",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-SD",
};

pub const MD_SI: Subdivision = Subdivision {
    name: "Sîngerei",
    code: "MD-SI",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-SI",
};

pub const MD_SN: Subdivision = Subdivision {
    name: "Stînga Nistrului, unitatea teritorială din",
    code: "MD-SN",
    subdivision_type: "Territorial unit",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-SN",
};

pub const MD_SO: Subdivision = Subdivision {
    name: "Soroca",
    code: "MD-SO",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-SO",
};

pub const MD_ST: Subdivision = Subdivision {
    name: "Străşeni",
    code: "MD-ST",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-ST",
};

pub const MD_SV: Subdivision = Subdivision {
    name: "Ştefan Vodă",
    code: "MD-SV",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-SV",
};

pub const MD_TA: Subdivision = Subdivision {
    name: "Taraclia",
    code: "MD-TA",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-TA",
};

pub const MD_TE: Subdivision = Subdivision {
    name: "Teleneşti",
    code: "MD-TE",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-TE",
};

pub const MD_UN: Subdivision = Subdivision {
    name: "Ungheni",
    code: "MD-UN",
    subdivision_type: "District",
    country_name: "Moldova",
    country_code: "MD",
    region_code: "MD-UN",
};

pub const MC_CL: Subdivision = Subdivision {
    name: "La Colle",
    code: "MC-CL",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-CL",
};

pub const MC_CO: Subdivision = Subdivision {
    name: "La Condamine",
    code: "MC-CO",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-CO",
};

pub const MC_FO: Subdivision = Subdivision {
    name: "Fontvieille",
    code: "MC-FO",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-FO",
};

pub const MC_GA: Subdivision = Subdivision {
    name: "La Gare",
    code: "MC-GA",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-GA",
};

pub const MC_JE: Subdivision = Subdivision {
    name: "Jardin Exotique",
    code: "MC-JE",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-JE",
};

pub const MC_LA: Subdivision = Subdivision {
    name: "Larvotto",
    code: "MC-LA",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-LA",
};

pub const MC_MA: Subdivision = Subdivision {
    name: "Malbousquet",
    code: "MC-MA",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-MA",
};

pub const MC_MC: Subdivision = Subdivision {
    name: "Monte-Carlo",
    code: "MC-MC",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-MC",
};

pub const MC_MG: Subdivision = Subdivision {
    name: "Moneghetti",
    code: "MC-MG",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-MG",
};

pub const MC_MO: Subdivision = Subdivision {
    name: "Monaco-Ville",
    code: "MC-MO",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-MO",
};

pub const MC_MU: Subdivision = Subdivision {
    name: "Moulins",
    code: "MC-MU",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-MU",
};

pub const MC_PH: Subdivision = Subdivision {
    name: "Port-Hercule",
    code: "MC-PH",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-PH",
};

pub const MC_SD: Subdivision = Subdivision {
    name: "Sainte-Dévote",
    code: "MC-SD",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-SD",
};

pub const MC_SO: Subdivision = Subdivision {
    name: "La Source",
    code: "MC-SO",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-SO",
};

pub const MC_SP: Subdivision = Subdivision {
    name: "Spélugues",
    code: "MC-SP",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-SP",
};

pub const MC_SR: Subdivision = Subdivision {
    name: "Saint-Roman",
    code: "MC-SR",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-SR",
};

pub const MC_VR: Subdivision = Subdivision {
    name: "Vallon de la Rousse",
    code: "MC-VR",
    subdivision_type: "Quarter",
    country_name: "Monaco",
    country_code: "MC",
    region_code: "MC-VR",
};

pub const MN_035: Subdivision = Subdivision {
    name: "Orhon",
    code: "MN-035",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-035",
};

pub const MN_037: Subdivision = Subdivision {
    name: "Darhan uul",
    code: "MN-037",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-037",
};

pub const MN_039: Subdivision = Subdivision {
    name: "Hentiy",
    code: "MN-039",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-039",
};

pub const MN_041: Subdivision = Subdivision {
    name: "Hövsgöl",
    code: "MN-041",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-041",
};

pub const MN_043: Subdivision = Subdivision {
    name: "Hovd",
    code: "MN-043",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-043",
};

pub const MN_046: Subdivision = Subdivision {
    name: "Uvs",
    code: "MN-046",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-046",
};

pub const MN_047: Subdivision = Subdivision {
    name: "Töv",
    code: "MN-047",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-047",
};

pub const MN_049: Subdivision = Subdivision {
    name: "Selenge",
    code: "MN-049",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-049",
};

pub const MN_051: Subdivision = Subdivision {
    name: "Sühbaatar",
    code: "MN-051",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-051",
};

pub const MN_053: Subdivision = Subdivision {
    name: "Ömnögovĭ",
    code: "MN-053",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-053",
};

pub const MN_055: Subdivision = Subdivision {
    name: "Övörhangay",
    code: "MN-055",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-055",
};

pub const MN_057: Subdivision = Subdivision {
    name: "Dzavhan",
    code: "MN-057",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-057",
};

pub const MN_059: Subdivision = Subdivision {
    name: "Dundgovĭ",
    code: "MN-059",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-059",
};

pub const MN_061: Subdivision = Subdivision {
    name: "Dornod",
    code: "MN-061",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-061",
};

pub const MN_063: Subdivision = Subdivision {
    name: "Dornogovĭ",
    code: "MN-063",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-063",
};

pub const MN_064: Subdivision = Subdivision {
    name: "Govĭ-Sümber",
    code: "MN-064",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-064",
};

pub const MN_065: Subdivision = Subdivision {
    name: "Govĭ-Altay",
    code: "MN-065",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-065",
};

pub const MN_067: Subdivision = Subdivision {
    name: "Bulgan",
    code: "MN-067",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-067",
};

pub const MN_069: Subdivision = Subdivision {
    name: "Bayanhongor",
    code: "MN-069",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-069",
};

pub const MN_071: Subdivision = Subdivision {
    name: "Bayan-Ölgiy",
    code: "MN-071",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-071",
};

pub const MN_073: Subdivision = Subdivision {
    name: "Arhangay",
    code: "MN-073",
    subdivision_type: "Province",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-073",
};

pub const MN_1: Subdivision = Subdivision {
    name: "Ulaanbaatar",
    code: "MN-1",
    subdivision_type: "Capital city",
    country_name: "Mongolia",
    country_code: "MN",
    region_code: "MN-1",
};

pub const ME_01: Subdivision = Subdivision {
    name: "Andrijevica",
    code: "ME-01",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-01",
};

pub const ME_02: Subdivision = Subdivision {
    name: "Bar",
    code: "ME-02",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-02",
};

pub const ME_03: Subdivision = Subdivision {
    name: "Berane",
    code: "ME-03",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-03",
};

pub const ME_04: Subdivision = Subdivision {
    name: "Bijelo Polje",
    code: "ME-04",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-04",
};

pub const ME_05: Subdivision = Subdivision {
    name: "Budva",
    code: "ME-05",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-05",
};

pub const ME_06: Subdivision = Subdivision {
    name: "Cetinje",
    code: "ME-06",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-06",
};

pub const ME_07: Subdivision = Subdivision {
    name: "Danilovgrad",
    code: "ME-07",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-07",
};

pub const ME_08: Subdivision = Subdivision {
    name: "Herceg-Novi",
    code: "ME-08",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-08",
};

pub const ME_09: Subdivision = Subdivision {
    name: "Kolašin",
    code: "ME-09",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-09",
};

pub const ME_10: Subdivision = Subdivision {
    name: "Kotor",
    code: "ME-10",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-10",
};

pub const ME_11: Subdivision = Subdivision {
    name: "Mojkovac",
    code: "ME-11",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-11",
};

pub const ME_12: Subdivision = Subdivision {
    name: "Nikšić",
    code: "ME-12",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-12",
};

pub const ME_13: Subdivision = Subdivision {
    name: "Plav",
    code: "ME-13",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-13",
};

pub const ME_14: Subdivision = Subdivision {
    name: "Pljevlja",
    code: "ME-14",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-14",
};

pub const ME_15: Subdivision = Subdivision {
    name: "Plužine",
    code: "ME-15",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-15",
};

pub const ME_16: Subdivision = Subdivision {
    name: "Podgorica",
    code: "ME-16",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-16",
};

pub const ME_17: Subdivision = Subdivision {
    name: "Rožaje",
    code: "ME-17",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-17",
};

pub const ME_18: Subdivision = Subdivision {
    name: "Šavnik",
    code: "ME-18",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-18",
};

pub const ME_19: Subdivision = Subdivision {
    name: "Tivat",
    code: "ME-19",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-19",
};

pub const ME_20: Subdivision = Subdivision {
    name: "Ulcinj",
    code: "ME-20",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-20",
};

pub const ME_21: Subdivision = Subdivision {
    name: "Žabljak",
    code: "ME-21",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-21",
};

pub const ME_22: Subdivision = Subdivision {
    name: "Gusinje",
    code: "ME-22",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-22",
};

pub const ME_23: Subdivision = Subdivision {
    name: "Petnjica",
    code: "ME-23",
    subdivision_type: "Municipality",
    country_name: "Montenegro",
    country_code: "ME",
    region_code: "ME-23",
};

pub const MS_MS: Subdivision = Subdivision {
    name: "Montserrat",
    code: "MS-MS",
    subdivision_type: "Country",
    country_name: "Montserrat",
    country_code: "MS",
    region_code: "MS-MS",
};

pub const MA_01: Subdivision = Subdivision {
    name: "Tanger-Tétouan",
    code: "MA-01",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-01",
};

pub const MA_02: Subdivision = Subdivision {
    name: "Gharb-Chrarda-Beni Hssen",
    code: "MA-02",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-02",
};

pub const MA_03: Subdivision = Subdivision {
    name: "Taza-Al Hoceima-Taounate",
    code: "MA-03",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-03",
};

pub const MA_04: Subdivision = Subdivision {
    name: "L'Oriental",
    code: "MA-04",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-04",
};

pub const MA_05: Subdivision = Subdivision {
    name: "Fès-Boulemane",
    code: "MA-05",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-05",
};

pub const MA_06: Subdivision = Subdivision {
    name: "Meknès-Tafilalet",
    code: "MA-06",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-06",
};

pub const MA_07: Subdivision = Subdivision {
    name: "Rabat-Salé-Zemmour-Zaer",
    code: "MA-07",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-07",
};

pub const MA_08: Subdivision = Subdivision {
    name: "Grand Casablanca",
    code: "MA-08",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-08",
};

pub const MA_09: Subdivision = Subdivision {
    name: "Chaouia-Ouardigha",
    code: "MA-09",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-09",
};

pub const MA_10: Subdivision = Subdivision {
    name: "Doukkala-Abda",
    code: "MA-10",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-10",
};

pub const MA_11: Subdivision = Subdivision {
    name: "Marrakech-Tensift-Al Haouz",
    code: "MA-11",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-11",
};

pub const MA_12: Subdivision = Subdivision {
    name: "Tadla-Azilal",
    code: "MA-12",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-12",
};

pub const MA_13: Subdivision = Subdivision {
    name: "Sous-Massa-Draa",
    code: "MA-13",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-13",
};

pub const MA_14: Subdivision = Subdivision {
    name: "Guelmim-Es Semara",
    code: "MA-14",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-14",
};

pub const MA_15: Subdivision = Subdivision {
    name: "Laâyoune-Boujdour-Sakia el Hamra",
    code: "MA-15",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-15",
};

pub const MA_16: Subdivision = Subdivision {
    name: "Oued ed Dahab-Lagouira",
    code: "MA-16",
    subdivision_type: "Region",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-16",
};

pub const MA_AGD: Subdivision = Subdivision {
    name: "Agadir-Ida-Outanane",
    code: "MA-AGD",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-AGD",
};

pub const MA_AOU: Subdivision = Subdivision {
    name: "Aousserd",
    code: "MA-AOU",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-AOU",
};

pub const MA_ASZ: Subdivision = Subdivision {
    name: "Assa-Zag",
    code: "MA-ASZ",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-ASZ",
};

pub const MA_AZI: Subdivision = Subdivision {
    name: "Azilal",
    code: "MA-AZI",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-AZI",
};

pub const MA_BEM: Subdivision = Subdivision {
    name: "Beni Mellal",
    code: "MA-BEM",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-BEM",
};

pub const MA_BER: Subdivision = Subdivision {
    name: "Berkane",
    code: "MA-BER",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-BER",
};

pub const MA_BES: Subdivision = Subdivision {
    name: "Ben Slimane",
    code: "MA-BES",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-BES",
};

pub const MA_BOD: Subdivision = Subdivision {
    name: "Boujdour",
    code: "MA-BOD",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-BOD",
};

pub const MA_BOM: Subdivision = Subdivision {
    name: "Boulemane",
    code: "MA-BOM",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-BOM",
};

pub const MA_CAS: Subdivision = Subdivision {
    name: "Casablanca",
    code: "MA-CAS",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-CAS",
};

pub const MA_CHE: Subdivision = Subdivision {
    name: "Chefchaouen",
    code: "MA-CHE",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-CHE",
};

pub const MA_CHI: Subdivision = Subdivision {
    name: "Chichaoua",
    code: "MA-CHI",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-CHI",
};

pub const MA_CHT: Subdivision = Subdivision {
    name: "Chtouka-Ait Baha",
    code: "MA-CHT",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-CHT",
};

pub const MA_ERR: Subdivision = Subdivision {
    name: "Errachidia",
    code: "MA-ERR",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-ERR",
};

pub const MA_ESI: Subdivision = Subdivision {
    name: "Essaouira",
    code: "MA-ESI",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-ESI",
};

pub const MA_ESM: Subdivision = Subdivision {
    name: "Es Smara",
    code: "MA-ESM",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-ESM",
};

pub const MA_FAH: Subdivision = Subdivision {
    name: "Fahs-Beni Makada",
    code: "MA-FAH",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-FAH",
};

pub const MA_FES: Subdivision = Subdivision {
    name: "Fès-Dar-Dbibegh",
    code: "MA-FES",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-FES",
};

pub const MA_FIG: Subdivision = Subdivision {
    name: "Figuig",
    code: "MA-FIG",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-FIG",
};

pub const MA_GUE: Subdivision = Subdivision {
    name: "Guelmim",
    code: "MA-GUE",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-GUE",
};

pub const MA_HAJ: Subdivision = Subdivision {
    name: "El Hajeb",
    code: "MA-HAJ",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-HAJ",
};

pub const MA_HAO: Subdivision = Subdivision {
    name: "Al Haouz",
    code: "MA-HAO",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-HAO",
};

pub const MA_HOC: Subdivision = Subdivision {
    name: "Al Hoceïma",
    code: "MA-HOC",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-HOC",
};

pub const MA_IFR: Subdivision = Subdivision {
    name: "Ifrane",
    code: "MA-IFR",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-IFR",
};

pub const MA_INE: Subdivision = Subdivision {
    name: "Inezgane-Ait Melloul",
    code: "MA-INE",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-INE",
};

pub const MA_JDI: Subdivision = Subdivision {
    name: "El Jadida",
    code: "MA-JDI",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-JDI",
};

pub const MA_JRA: Subdivision = Subdivision {
    name: "Jrada",
    code: "MA-JRA",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-JRA",
};

pub const MA_KEN: Subdivision = Subdivision {
    name: "Kénitra",
    code: "MA-KEN",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-KEN",
};

pub const MA_KES: Subdivision = Subdivision {
    name: "Kelaat es Sraghna",
    code: "MA-KES",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-KES",
};

pub const MA_KHE: Subdivision = Subdivision {
    name: "Khemisset",
    code: "MA-KHE",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-KHE",
};

pub const MA_KHN: Subdivision = Subdivision {
    name: "Khenifra",
    code: "MA-KHN",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-KHN",
};

pub const MA_KHO: Subdivision = Subdivision {
    name: "Khouribga",
    code: "MA-KHO",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-KHO",
};

pub const MA_LAA: Subdivision = Subdivision {
    name: "Laâyoune",
    code: "MA-LAA",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-LAA",
};

pub const MA_LAR: Subdivision = Subdivision {
    name: "Larache",
    code: "MA-LAR",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-LAR",
};

pub const MA_MED: Subdivision = Subdivision {
    name: "Médiouna",
    code: "MA-MED",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-MED",
};

pub const MA_MEK: Subdivision = Subdivision {
    name: "Meknès",
    code: "MA-MEK",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-MEK",
};

pub const MA_MMD: Subdivision = Subdivision {
    name: "Marrakech-Medina",
    code: "MA-MMD",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-MMD",
};

pub const MA_MMN: Subdivision = Subdivision {
    name: "Marrakech-Menara",
    code: "MA-MMN",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-MMN",
};

pub const MA_MOH: Subdivision = Subdivision {
    name: "Mohammadia",
    code: "MA-MOH",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-MOH",
};

pub const MA_MOU: Subdivision = Subdivision {
    name: "Moulay Yacoub",
    code: "MA-MOU",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-MOU",
};

pub const MA_NAD: Subdivision = Subdivision {
    name: "Nador",
    code: "MA-NAD",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-NAD",
};

pub const MA_NOU: Subdivision = Subdivision {
    name: "Nouaceur",
    code: "MA-NOU",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-NOU",
};

pub const MA_OUA: Subdivision = Subdivision {
    name: "Ouarzazate",
    code: "MA-OUA",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-OUA",
};

pub const MA_OUD: Subdivision = Subdivision {
    name: "Oued ed Dahab",
    code: "MA-OUD",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-OUD",
};

pub const MA_OUJ: Subdivision = Subdivision {
    name: "Oujda-Angad",
    code: "MA-OUJ",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-OUJ",
};

pub const MA_RAB: Subdivision = Subdivision {
    name: "Rabat",
    code: "MA-RAB",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-RAB",
};

pub const MA_SAF: Subdivision = Subdivision {
    name: "Safi",
    code: "MA-SAF",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SAF",
};

pub const MA_SAL: Subdivision = Subdivision {
    name: "Salé",
    code: "MA-SAL",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SAL",
};

pub const MA_SEF: Subdivision = Subdivision {
    name: "Sefrou",
    code: "MA-SEF",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SEF",
};

pub const MA_SET: Subdivision = Subdivision {
    name: "Settat",
    code: "MA-SET",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SET",
};

pub const MA_SIK: Subdivision = Subdivision {
    name: "Sidi Kacem",
    code: "MA-SIK",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SIK",
};

pub const MA_SKH: Subdivision = Subdivision {
    name: "Skhirate-Témara",
    code: "MA-SKH",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SKH",
};

pub const MA_SYB: Subdivision = Subdivision {
    name: "Sidi Youssef Ben Ali",
    code: "MA-SYB",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-SYB",
};

pub const MA_TAI: Subdivision = Subdivision {
    name: "Taourirt",
    code: "MA-TAI",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TAI",
};

pub const MA_TAO: Subdivision = Subdivision {
    name: "Taounate",
    code: "MA-TAO",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TAO",
};

pub const MA_TAR: Subdivision = Subdivision {
    name: "Taroudant",
    code: "MA-TAR",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TAR",
};

pub const MA_TAT: Subdivision = Subdivision {
    name: "Tata",
    code: "MA-TAT",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TAT",
};

pub const MA_TAZ: Subdivision = Subdivision {
    name: "Taza",
    code: "MA-TAZ",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TAZ",
};

pub const MA_TET: Subdivision = Subdivision {
    name: "Tétouan",
    code: "MA-TET",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TET",
};

pub const MA_TIZ: Subdivision = Subdivision {
    name: "Tiznit",
    code: "MA-TIZ",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TIZ",
};

pub const MA_TNG: Subdivision = Subdivision {
    name: "Tanger-Assilah",
    code: "MA-TNG",
    subdivision_type: "Prefecture",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TNG",
};

pub const MA_TNT: Subdivision = Subdivision {
    name: "Tan-Tan",
    code: "MA-TNT",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-TNT",
};

pub const MA_ZAG: Subdivision = Subdivision {
    name: "Zagora",
    code: "MA-ZAG",
    subdivision_type: "Province",
    country_name: "Morocco",
    country_code: "MA",
    region_code: "MA-ZAG",
};

pub const MZ_A: Subdivision = Subdivision {
    name: "Niassa",
    code: "MZ-A",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-A",
};

pub const MZ_B: Subdivision = Subdivision {
    name: "Manica",
    code: "MZ-B",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-B",
};

pub const MZ_G: Subdivision = Subdivision {
    name: "Gaza",
    code: "MZ-G",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-G",
};

pub const MZ_I: Subdivision = Subdivision {
    name: "Inhambane",
    code: "MZ-I",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-I",
};

pub const MZ_L: Subdivision = Subdivision {
    name: "Maputo",
    code: "MZ-L",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-L",
};

pub const MZ_MPM: Subdivision = Subdivision {
    name: "Maputo",
    code: "MZ-MPM",
    subdivision_type: "City",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-MPM",
};

pub const MZ_N: Subdivision = Subdivision {
    name: "Nampula",
    code: "MZ-N",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-N",
};

pub const MZ_P: Subdivision = Subdivision {
    name: "Cabo Delgado",
    code: "MZ-P",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-P",
};

pub const MZ_Q: Subdivision = Subdivision {
    name: "Zambézia",
    code: "MZ-Q",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-Q",
};

pub const MZ_S: Subdivision = Subdivision {
    name: "Sofala",
    code: "MZ-S",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-S",
};

pub const MZ_T: Subdivision = Subdivision {
    name: "Tete",
    code: "MZ-T",
    subdivision_type: "Province",
    country_name: "Mozambique",
    country_code: "MZ",
    region_code: "MZ-T",
};

pub const MM_01: Subdivision = Subdivision {
    name: "Sagaing",
    code: "MM-01",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-01",
};

pub const MM_02: Subdivision = Subdivision {
    name: "Bago",
    code: "MM-02",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-02",
};

pub const MM_03: Subdivision = Subdivision {
    name: "Magway",
    code: "MM-03",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-03",
};

pub const MM_04: Subdivision = Subdivision {
    name: "Mandalay",
    code: "MM-04",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-04",
};

pub const MM_05: Subdivision = Subdivision {
    name: "Taninthayi",
    code: "MM-05",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-05",
};

pub const MM_06: Subdivision = Subdivision {
    name: "Yangon",
    code: "MM-06",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-06",
};

pub const MM_07: Subdivision = Subdivision {
    name: "Ayeyawady",
    code: "MM-07",
    subdivision_type: "Region",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-07",
};

pub const MM_11: Subdivision = Subdivision {
    name: "Kachin",
    code: "MM-11",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-11",
};

pub const MM_12: Subdivision = Subdivision {
    name: "Kayah",
    code: "MM-12",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-12",
};

pub const MM_13: Subdivision = Subdivision {
    name: "Kayin",
    code: "MM-13",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-13",
};

pub const MM_14: Subdivision = Subdivision {
    name: "Chin",
    code: "MM-14",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-14",
};

pub const MM_15: Subdivision = Subdivision {
    name: "Mon",
    code: "MM-15",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-15",
};

pub const MM_16: Subdivision = Subdivision {
    name: "Rakhine",
    code: "MM-16",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-16",
};

pub const MM_17: Subdivision = Subdivision {
    name: "Shan",
    code: "MM-17",
    subdivision_type: "State",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-17",
};

pub const MM_18: Subdivision = Subdivision {
    name: "Nay Pyi Taw",
    code: "MM-18",
    subdivision_type: "Union territory",
    country_name: "Myanmar",
    country_code: "MM",
    region_code: "MM-18",
};

pub const NA_CA: Subdivision = Subdivision {
    name: "Zambezi",
    code: "NA-CA",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-CA",
};

pub const NA_ER: Subdivision = Subdivision {
    name: "Erongo",
    code: "NA-ER",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-ER",
};

pub const NA_HA: Subdivision = Subdivision {
    name: "Hardap",
    code: "NA-HA",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-HA",
};

pub const NA_KA: Subdivision = Subdivision {
    name: "Karas",
    code: "NA-KA",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-KA",
};

pub const NA_KE: Subdivision = Subdivision {
    name: "Kavango East",
    code: "NA-KE",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-KE",
};

pub const NA_KH: Subdivision = Subdivision {
    name: "Khomas",
    code: "NA-KH",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-KH",
};

pub const NA_KU: Subdivision = Subdivision {
    name: "Kunene",
    code: "NA-KU",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-KU",
};

pub const NA_KW: Subdivision = Subdivision {
    name: "Kavango West",
    code: "NA-KW",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-KW",
};

pub const NA_OD: Subdivision = Subdivision {
    name: "Otjozondjupa",
    code: "NA-OD",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-OD",
};

pub const NA_OH: Subdivision = Subdivision {
    name: "Omaheke",
    code: "NA-OH",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-OH",
};

pub const NA_ON: Subdivision = Subdivision {
    name: "Oshana",
    code: "NA-ON",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-ON",
};

pub const NA_OS: Subdivision = Subdivision {
    name: "Omusati",
    code: "NA-OS",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-OS",
};

pub const NA_OT: Subdivision = Subdivision {
    name: "Oshikoto",
    code: "NA-OT",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-OT",
};

pub const NA_OW: Subdivision = Subdivision {
    name: "Ohangwena",
    code: "NA-OW",
    subdivision_type: "Region",
    country_name: "Namibia",
    country_code: "NA",
    region_code: "NA-OW",
};

pub const NR_01: Subdivision = Subdivision {
    name: "Aiwo",
    code: "NR-01",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-01",
};

pub const NR_02: Subdivision = Subdivision {
    name: "Anabar",
    code: "NR-02",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-02",
};

pub const NR_03: Subdivision = Subdivision {
    name: "Anetan",
    code: "NR-03",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-03",
};

pub const NR_04: Subdivision = Subdivision {
    name: "Anibare",
    code: "NR-04",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-04",
};

pub const NR_05: Subdivision = Subdivision {
    name: "Baiti",
    code: "NR-05",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-05",
};

pub const NR_06: Subdivision = Subdivision {
    name: "Boe",
    code: "NR-06",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-06",
};

pub const NR_07: Subdivision = Subdivision {
    name: "Buada",
    code: "NR-07",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-07",
};

pub const NR_08: Subdivision = Subdivision {
    name: "Denigomodu",
    code: "NR-08",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-08",
};

pub const NR_09: Subdivision = Subdivision {
    name: "Ewa",
    code: "NR-09",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-09",
};

pub const NR_10: Subdivision = Subdivision {
    name: "Ijuw",
    code: "NR-10",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-10",
};

pub const NR_11: Subdivision = Subdivision {
    name: "Meneng",
    code: "NR-11",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-11",
};

pub const NR_12: Subdivision = Subdivision {
    name: "Nibok",
    code: "NR-12",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-12",
};

pub const NR_13: Subdivision = Subdivision {
    name: "Uaboe",
    code: "NR-13",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-13",
};

pub const NR_14: Subdivision = Subdivision {
    name: "Yaren",
    code: "NR-14",
    subdivision_type: "District",
    country_name: "Nauru",
    country_code: "NR",
    region_code: "NR-14",
};

pub const NP_1: Subdivision = Subdivision {
    name: "Central",
    code: "NP-1",
    subdivision_type: "Development region",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-1",
};

pub const NP_2: Subdivision = Subdivision {
    name: "Madhya Pashchimanchal",
    code: "NP-2",
    subdivision_type: "Development region",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-2",
};

pub const NP_3: Subdivision = Subdivision {
    name: "Pashchimanchal",
    code: "NP-3",
    subdivision_type: "Development region",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-3",
};

pub const NP_4: Subdivision = Subdivision {
    name: "Eastern",
    code: "NP-4",
    subdivision_type: "Development region",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-4",
};

pub const NP_5: Subdivision = Subdivision {
    name: "Far Western",
    code: "NP-5",
    subdivision_type: "Development region",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-5",
};

pub const NP_BA: Subdivision = Subdivision {
    name: "Bagmati",
    code: "NP-BA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-BA",
};

pub const NP_BH: Subdivision = Subdivision {
    name: "Bheri",
    code: "NP-BH",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-BH",
};

pub const NP_DH: Subdivision = Subdivision {
    name: "Dhawalagiri",
    code: "NP-DH",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-DH",
};

pub const NP_GA: Subdivision = Subdivision {
    name: "Gandaki",
    code: "NP-GA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-GA",
};

pub const NP_JA: Subdivision = Subdivision {
    name: "Janakpur",
    code: "NP-JA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-JA",
};

pub const NP_KA: Subdivision = Subdivision {
    name: "Karnali",
    code: "NP-KA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-KA",
};

pub const NP_KO: Subdivision = Subdivision {
    name: "Kosi",
    code: "NP-KO",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-KO",
};

pub const NP_LU: Subdivision = Subdivision {
    name: "Lumbini",
    code: "NP-LU",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-LU",
};

pub const NP_MA: Subdivision = Subdivision {
    name: "Mahakali",
    code: "NP-MA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-MA",
};

pub const NP_ME: Subdivision = Subdivision {
    name: "Mechi",
    code: "NP-ME",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-ME",
};

pub const NP_NA: Subdivision = Subdivision {
    name: "Narayani",
    code: "NP-NA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-NA",
};

pub const NP_RA: Subdivision = Subdivision {
    name: "Rapti",
    code: "NP-RA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-RA",
};

pub const NP_SA: Subdivision = Subdivision {
    name: "Sagarmatha",
    code: "NP-SA",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-SA",
};

pub const NP_SE: Subdivision = Subdivision {
    name: "Seti",
    code: "NP-SE",
    subdivision_type: "Zone",
    country_name: "Nepal",
    country_code: "NP",
    region_code: "NP-SE",
};

pub const NL_AW: Subdivision = Subdivision {
    name: "Aruba",
    code: "NL-AW",
    subdivision_type: "Country",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-AW",
};

pub const NL_BQ1: Subdivision = Subdivision {
    name: "Bonaire",
    code: "NL-BQ1",
    subdivision_type: "Special municipality",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-BQ1",
};

pub const NL_BQ2: Subdivision = Subdivision {
    name: "Saba",
    code: "NL-BQ2",
    subdivision_type: "Special municipality",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-BQ2",
};

pub const NL_BQ3: Subdivision = Subdivision {
    name: "Sint Eustatius",
    code: "NL-BQ3",
    subdivision_type: "Special municipality",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-BQ3",
};

pub const NL_CW: Subdivision = Subdivision {
    name: "Curaçao",
    code: "NL-CW",
    subdivision_type: "Country",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-CW",
};

pub const NL_DR: Subdivision = Subdivision {
    name: "Drenthe",
    code: "NL-DR",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-DR",
};

pub const NL_FL: Subdivision = Subdivision {
    name: "Flevoland",
    code: "NL-FL",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-FL",
};

pub const NL_FR: Subdivision = Subdivision {
    name: "Fryslân",
    code: "NL-FR",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-FR",
};

pub const NL_GE: Subdivision = Subdivision {
    name: "Gelderland",
    code: "NL-GE",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-GE",
};

pub const NL_GR: Subdivision = Subdivision {
    name: "Groningen",
    code: "NL-GR",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-GR",
};

pub const NL_LI: Subdivision = Subdivision {
    name: "Limburg",
    code: "NL-LI",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-LI",
};

pub const NL_NB: Subdivision = Subdivision {
    name: "Noord-Brabant",
    code: "NL-NB",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-NB",
};

pub const NL_NH: Subdivision = Subdivision {
    name: "Noord-Holland",
    code: "NL-NH",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-NH",
};

pub const NL_OV: Subdivision = Subdivision {
    name: "Overijssel",
    code: "NL-OV",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-OV",
};

pub const NL_SX: Subdivision = Subdivision {
    name: "Sint Maarten",
    code: "NL-SX",
    subdivision_type: "Country",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-SX",
};

pub const NL_UT: Subdivision = Subdivision {
    name: "Utrecht",
    code: "NL-UT",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-UT",
};

pub const NL_ZE: Subdivision = Subdivision {
    name: "Zeeland",
    code: "NL-ZE",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-ZE",
};

pub const NL_ZH: Subdivision = Subdivision {
    name: "Zuid-Holland",
    code: "NL-ZH",
    subdivision_type: "Province",
    country_name: "Netherlands",
    country_code: "NL",
    region_code: "NL-ZH",
};

pub const NC_NC: Subdivision = Subdivision {
    name: "New Caledonia",
    code: "NC-NC",
    subdivision_type: "Country",
    country_name: "New Caledonia",
    country_code: "NC",
    region_code: "NC-NC",
};

pub const NZ_AUK: Subdivision = Subdivision {
    name: "Auckland",
    code: "NZ-AUK",
    subdivision_type: "Unitary authority",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-AUK",
};

pub const NZ_BOP: Subdivision = Subdivision {
    name: "Bay of Plenty",
    code: "NZ-BOP",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-BOP",
};

pub const NZ_CAN: Subdivision = Subdivision {
    name: "Canterbury",
    code: "NZ-CAN",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-CAN",
};

pub const NZ_CIT: Subdivision = Subdivision {
    name: "Chatham Islands Territory",
    code: "NZ-CIT",
    subdivision_type: "Special island authority",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-CIT",
};

pub const NZ_GIS: Subdivision = Subdivision {
    name: "Gisborne District",
    code: "NZ-GIS",
    subdivision_type: "Unitary authority",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-GIS",
};

pub const NZ_HKB: Subdivision = Subdivision {
    name: "Hawke's Bay",
    code: "NZ-HKB",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-HKB",
};

pub const NZ_MBH: Subdivision = Subdivision {
    name: "Marlborough District",
    code: "NZ-MBH",
    subdivision_type: "Unitary authority",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-MBH",
};

pub const NZ_MWT: Subdivision = Subdivision {
    name: "Manawatu-Wanganui",
    code: "NZ-MWT",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-MWT",
};

pub const NZ_N: Subdivision = Subdivision {
    name: "North Island",
    code: "NZ-N",
    subdivision_type: "Island",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-N",
};

pub const NZ_NSN: Subdivision = Subdivision {
    name: "Nelson City",
    code: "NZ-NSN",
    subdivision_type: "Unitary authority",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-NSN",
};

pub const NZ_NTL: Subdivision = Subdivision {
    name: "Northland",
    code: "NZ-NTL",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-NTL",
};

pub const NZ_OTA: Subdivision = Subdivision {
    name: "Otago",
    code: "NZ-OTA",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-OTA",
};

pub const NZ_S: Subdivision = Subdivision {
    name: "South Island",
    code: "NZ-S",
    subdivision_type: "Island",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-S",
};

pub const NZ_STL: Subdivision = Subdivision {
    name: "Southland",
    code: "NZ-STL",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-STL",
};

pub const NZ_TAS: Subdivision = Subdivision {
    name: "Tasman District",
    code: "NZ-TAS",
    subdivision_type: "Unitary authority",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-TAS",
};

pub const NZ_TKI: Subdivision = Subdivision {
    name: "Taranaki",
    code: "NZ-TKI",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-TKI",
};

pub const NZ_WGN: Subdivision = Subdivision {
    name: "Wellington",
    code: "NZ-WGN",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-WGN",
};

pub const NZ_WKO: Subdivision = Subdivision {
    name: "Waikato",
    code: "NZ-WKO",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-WKO",
};

pub const NZ_WTC: Subdivision = Subdivision {
    name: "West Coast",
    code: "NZ-WTC",
    subdivision_type: "Regional council",
    country_name: "New Zealand",
    country_code: "NZ",
    region_code: "NZ-WTC",
};

pub const NI_AN: Subdivision = Subdivision {
    name: "Atlántico Norte",
    code: "NI-AN",
    subdivision_type: "Autonomous region",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-AN",
};

pub const NI_AS: Subdivision = Subdivision {
    name: "Atlántico Sur",
    code: "NI-AS",
    subdivision_type: "Autonomous region",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-AS",
};

pub const NI_BO: Subdivision = Subdivision {
    name: "Boaco",
    code: "NI-BO",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-BO",
};

pub const NI_CA: Subdivision = Subdivision {
    name: "Carazo",
    code: "NI-CA",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-CA",
};

pub const NI_CI: Subdivision = Subdivision {
    name: "Chinandega",
    code: "NI-CI",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-CI",
};

pub const NI_CO: Subdivision = Subdivision {
    name: "Chontales",
    code: "NI-CO",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-CO",
};

pub const NI_ES: Subdivision = Subdivision {
    name: "Estelí",
    code: "NI-ES",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-ES",
};

pub const NI_GR: Subdivision = Subdivision {
    name: "Granada",
    code: "NI-GR",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-GR",
};

pub const NI_JI: Subdivision = Subdivision {
    name: "Jinotega",
    code: "NI-JI",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-JI",
};

pub const NI_LE: Subdivision = Subdivision {
    name: "León",
    code: "NI-LE",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-LE",
};

pub const NI_MD: Subdivision = Subdivision {
    name: "Madriz",
    code: "NI-MD",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-MD",
};

pub const NI_MN: Subdivision = Subdivision {
    name: "Managua",
    code: "NI-MN",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-MN",
};

pub const NI_MS: Subdivision = Subdivision {
    name: "Masaya",
    code: "NI-MS",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-MS",
};

pub const NI_MT: Subdivision = Subdivision {
    name: "Matagalpa",
    code: "NI-MT",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-MT",
};

pub const NI_NS: Subdivision = Subdivision {
    name: "Nueva Segovia",
    code: "NI-NS",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-NS",
};

pub const NI_RI: Subdivision = Subdivision {
    name: "Rivas",
    code: "NI-RI",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-RI",
};

pub const NI_SJ: Subdivision = Subdivision {
    name: "Río San Juan",
    code: "NI-SJ",
    subdivision_type: "Department",
    country_name: "Nicaragua",
    country_code: "NI",
    region_code: "NI-SJ",
};

pub const NG_AB: Subdivision = Subdivision {
    name: "Abia",
    code: "NG-AB",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-AB",
};

pub const NG_AD: Subdivision = Subdivision {
    name: "Adamawa",
    code: "NG-AD",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-AD",
};

pub const NG_AK: Subdivision = Subdivision {
    name: "Akwa Ibom",
    code: "NG-AK",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-AK",
};

pub const NG_AN: Subdivision = Subdivision {
    name: "Anambra",
    code: "NG-AN",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-AN",
};

pub const NG_BA: Subdivision = Subdivision {
    name: "Bauchi",
    code: "NG-BA",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-BA",
};

pub const NG_BE: Subdivision = Subdivision {
    name: "Benue",
    code: "NG-BE",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-BE",
};

pub const NG_BO: Subdivision = Subdivision {
    name: "Borno",
    code: "NG-BO",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-BO",
};

pub const NG_BY: Subdivision = Subdivision {
    name: "Bayelsa",
    code: "NG-BY",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-BY",
};

pub const NG_CR: Subdivision = Subdivision {
    name: "Cross River",
    code: "NG-CR",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-CR",
};

pub const NG_DE: Subdivision = Subdivision {
    name: "Delta",
    code: "NG-DE",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-DE",
};

pub const NG_EB: Subdivision = Subdivision {
    name: "Ebonyi",
    code: "NG-EB",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-EB",
};

pub const NG_ED: Subdivision = Subdivision {
    name: "Edo",
    code: "NG-ED",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-ED",
};

pub const NG_EK: Subdivision = Subdivision {
    name: "Ekiti",
    code: "NG-EK",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-EK",
};

pub const NG_EN: Subdivision = Subdivision {
    name: "Enugu",
    code: "NG-EN",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-EN",
};

pub const NG_FC: Subdivision = Subdivision {
    name: "Abuja Federal Capital Territory",
    code: "NG-FC",
    subdivision_type: "Capital territory",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-FC",
};

pub const NG_GO: Subdivision = Subdivision {
    name: "Gombe",
    code: "NG-GO",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-GO",
};

pub const NG_IM: Subdivision = Subdivision {
    name: "Imo",
    code: "NG-IM",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-IM",
};

pub const NG_JI: Subdivision = Subdivision {
    name: "Jigawa",
    code: "NG-JI",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-JI",
};

pub const NG_KD: Subdivision = Subdivision {
    name: "Kaduna",
    code: "NG-KD",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-KD",
};

pub const NG_KE: Subdivision = Subdivision {
    name: "Kebbi",
    code: "NG-KE",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-KE",
};

pub const NG_KN: Subdivision = Subdivision {
    name: "Kano",
    code: "NG-KN",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-KN",
};

pub const NG_KO: Subdivision = Subdivision {
    name: "Kogi",
    code: "NG-KO",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-KO",
};

pub const NG_KT: Subdivision = Subdivision {
    name: "Katsina",
    code: "NG-KT",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-KT",
};

pub const NG_KW: Subdivision = Subdivision {
    name: "Kwara",
    code: "NG-KW",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-KW",
};

pub const NG_LA: Subdivision = Subdivision {
    name: "Lagos",
    code: "NG-LA",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-LA",
};

pub const NG_NA: Subdivision = Subdivision {
    name: "Nasarawa",
    code: "NG-NA",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-NA",
};

pub const NG_NI: Subdivision = Subdivision {
    name: "Niger",
    code: "NG-NI",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-NI",
};

pub const NG_OG: Subdivision = Subdivision {
    name: "Ogun",
    code: "NG-OG",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-OG",
};

pub const NG_ON: Subdivision = Subdivision {
    name: "Ondo",
    code: "NG-ON",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-ON",
};

pub const NG_OS: Subdivision = Subdivision {
    name: "Osun",
    code: "NG-OS",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-OS",
};

pub const NG_OY: Subdivision = Subdivision {
    name: "Oyo",
    code: "NG-OY",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-OY",
};

pub const NG_PL: Subdivision = Subdivision {
    name: "Plateau",
    code: "NG-PL",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-PL",
};

pub const NG_RI: Subdivision = Subdivision {
    name: "Rivers",
    code: "NG-RI",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-RI",
};

pub const NG_SO: Subdivision = Subdivision {
    name: "Sokoto",
    code: "NG-SO",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-SO",
};

pub const NG_TA: Subdivision = Subdivision {
    name: "Taraba",
    code: "NG-TA",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-TA",
};

pub const NG_YO: Subdivision = Subdivision {
    name: "Yobe",
    code: "NG-YO",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-YO",
};

pub const NG_ZA: Subdivision = Subdivision {
    name: "Zamfara",
    code: "NG-ZA",
    subdivision_type: "State",
    country_name: "Nigeria",
    country_code: "NG",
    region_code: "NG-ZA",
};

pub const NE_1: Subdivision = Subdivision {
    name: "Agadez",
    code: "NE-1",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-1",
};

pub const NE_2: Subdivision = Subdivision {
    name: "Diffa",
    code: "NE-2",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-2",
};

pub const NE_3: Subdivision = Subdivision {
    name: "Dosso",
    code: "NE-3",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-3",
};

pub const NE_4: Subdivision = Subdivision {
    name: "Maradi",
    code: "NE-4",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-4",
};

pub const NE_5: Subdivision = Subdivision {
    name: "Tahoua",
    code: "NE-5",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-5",
};

pub const NE_6: Subdivision = Subdivision {
    name: "Tillabéri",
    code: "NE-6",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-6",
};

pub const NE_7: Subdivision = Subdivision {
    name: "Zinder",
    code: "NE-7",
    subdivision_type: "Region",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-7",
};

pub const NE_8: Subdivision = Subdivision {
    name: "Niamey",
    code: "NE-8",
    subdivision_type: "Urban community",
    country_name: "Niger",
    country_code: "NE",
    region_code: "NE-8",
};

pub const NU_NU: Subdivision = Subdivision {
    name: "Niue",
    code: "NU-NU",
    subdivision_type: "Country",
    country_name: "Niue",
    country_code: "NU",
    region_code: "NU-NU",
};

pub const NF_NF: Subdivision = Subdivision {
    name: "Norfolk Island",
    code: "NF-NF",
    subdivision_type: "Country",
    country_name: "Norfolk Island",
    country_code: "NF",
    region_code: "NF-NF",
};

pub const MP_MP: Subdivision = Subdivision {
    name: "Northern Mariana Islands",
    code: "MP-MP",
    subdivision_type: "Country",
    country_name: "Northern Mariana Islands",
    country_code: "MP",
    region_code: "MP-MP",
};

pub const NO_01: Subdivision = Subdivision {
    name: "Østfold",
    code: "NO-01",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-01",
};

pub const NO_02: Subdivision = Subdivision {
    name: "Akershus",
    code: "NO-02",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-02",
};

pub const NO_03: Subdivision = Subdivision {
    name: "Oslo",
    code: "NO-03",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-03",
};

pub const NO_04: Subdivision = Subdivision {
    name: "Hedmark",
    code: "NO-04",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-04",
};

pub const NO_05: Subdivision = Subdivision {
    name: "Oppland",
    code: "NO-05",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-05",
};

pub const NO_06: Subdivision = Subdivision {
    name: "Buskerud",
    code: "NO-06",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-06",
};

pub const NO_07: Subdivision = Subdivision {
    name: "Vestfold",
    code: "NO-07",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-07",
};

pub const NO_08: Subdivision = Subdivision {
    name: "Telemark",
    code: "NO-08",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-08",
};

pub const NO_09: Subdivision = Subdivision {
    name: "Aust-Agder",
    code: "NO-09",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-09",
};

pub const NO_10: Subdivision = Subdivision {
    name: "Vest-Agder",
    code: "NO-10",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-10",
};

pub const NO_11: Subdivision = Subdivision {
    name: "Rogaland",
    code: "NO-11",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-11",
};

pub const NO_12: Subdivision = Subdivision {
    name: "Hordaland",
    code: "NO-12",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-12",
};

pub const NO_14: Subdivision = Subdivision {
    name: "Sogn og Fjordane",
    code: "NO-14",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-14",
};

pub const NO_15: Subdivision = Subdivision {
    name: "Møre og Romsdal",
    code: "NO-15",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-15",
};

pub const NO_16: Subdivision = Subdivision {
    name: "Sør-Trøndelag",
    code: "NO-16",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-16",
};

pub const NO_17: Subdivision = Subdivision {
    name: "Nord-Trøndelag",
    code: "NO-17",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-17",
};

pub const NO_18: Subdivision = Subdivision {
    name: "Nordland",
    code: "NO-18",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-18",
};

pub const NO_19: Subdivision = Subdivision {
    name: "Romsa",
    code: "NO-19",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-19",
};

pub const NO_20: Subdivision = Subdivision {
    name: "Finnmark",
    code: "NO-20",
    subdivision_type: "County",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-20",
};

pub const NO_21: Subdivision = Subdivision {
    name: "Svalbard",
    code: "NO-21",
    subdivision_type: "Arctic region",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-21",
};

pub const NO_22: Subdivision = Subdivision {
    name: "Jan Mayen",
    code: "NO-22",
    subdivision_type: "Arctic region",
    country_name: "Norway",
    country_code: "NO",
    region_code: "NO-22",
};

pub const OM_BA: Subdivision = Subdivision {
    name: "Al Bāţinah",
    code: "OM-BA",
    subdivision_type: "Region",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-BA",
};

pub const OM_BU: Subdivision = Subdivision {
    name: "Al Buraymī",
    code: "OM-BU",
    subdivision_type: "Governorate",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-BU",
};

pub const OM_DA: Subdivision = Subdivision {
    name: "Ad Dākhilīyah",
    code: "OM-DA",
    subdivision_type: "Region",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-DA",
};

pub const OM_MA: Subdivision = Subdivision {
    name: "Masqaţ",
    code: "OM-MA",
    subdivision_type: "Governorate",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-MA",
};

pub const OM_MU: Subdivision = Subdivision {
    name: "Musandam",
    code: "OM-MU",
    subdivision_type: "Governorate",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-MU",
};

pub const OM_SH: Subdivision = Subdivision {
    name: "Ash Sharqīyah",
    code: "OM-SH",
    subdivision_type: "Region",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-SH",
};

pub const OM_WU: Subdivision = Subdivision {
    name: "Al Wusţá",
    code: "OM-WU",
    subdivision_type: "Region",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-WU",
};

pub const OM_ZA: Subdivision = Subdivision {
    name: "Az̧ Z̧āhirah",
    code: "OM-ZA",
    subdivision_type: "Region",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-ZA",
};

pub const OM_ZU: Subdivision = Subdivision {
    name: "Z̧ufār",
    code: "OM-ZU",
    subdivision_type: "Governorate",
    country_name: "Oman",
    country_code: "OM",
    region_code: "OM-ZU",
};

pub const PK_BA: Subdivision = Subdivision {
    name: "Balochistan",
    code: "PK-BA",
    subdivision_type: "Province",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-BA",
};

pub const PK_GB: Subdivision = Subdivision {
    name: "Gilgit-Baltistān",
    code: "PK-GB",
    subdivision_type: "Pakistan administered area",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-GB",
};

pub const PK_IS: Subdivision = Subdivision {
    name: "Islāmābād",
    code: "PK-IS",
    subdivision_type: "Federal capital territory",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-IS",
};

pub const PK_JK: Subdivision = Subdivision {
    name: "Azad Kashmir",
    code: "PK-JK",
    subdivision_type: "Pakistan administered area",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-JK",
};

pub const PK_KP: Subdivision = Subdivision {
    name: "Khaībar Pakhtūnkhwā",
    code: "PK-KP",
    subdivision_type: "Province",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-KP",
};

pub const PK_PB: Subdivision = Subdivision {
    name: "Panjāb",
    code: "PK-PB",
    subdivision_type: "Province",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-PB",
};

pub const PK_SD: Subdivision = Subdivision {
    name: "Sindh",
    code: "PK-SD",
    subdivision_type: "Province",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-SD",
};

pub const PK_TA: Subdivision = Subdivision {
    name: "Federally Administered Tribal Areas",
    code: "PK-TA",
    subdivision_type: "Territory",
    country_name: "Pakistan",
    country_code: "PK",
    region_code: "PK-TA",
};

pub const PW_002: Subdivision = Subdivision {
    name: "Aimeliik",
    code: "PW-002",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-002",
};

pub const PW_004: Subdivision = Subdivision {
    name: "Airai",
    code: "PW-004",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-004",
};

pub const PW_010: Subdivision = Subdivision {
    name: "Angaur",
    code: "PW-010",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-010",
};

pub const PW_050: Subdivision = Subdivision {
    name: "Hatobohei",
    code: "PW-050",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-050",
};

pub const PW_100: Subdivision = Subdivision {
    name: "Kayangel",
    code: "PW-100",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-100",
};

pub const PW_150: Subdivision = Subdivision {
    name: "Koror",
    code: "PW-150",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-150",
};

pub const PW_212: Subdivision = Subdivision {
    name: "Melekeok",
    code: "PW-212",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-212",
};

pub const PW_214: Subdivision = Subdivision {
    name: "Ngaraard",
    code: "PW-214",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-214",
};

pub const PW_218: Subdivision = Subdivision {
    name: "Ngarchelong",
    code: "PW-218",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-218",
};

pub const PW_222: Subdivision = Subdivision {
    name: "Ngardmau",
    code: "PW-222",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-222",
};

pub const PW_224: Subdivision = Subdivision {
    name: "Ngatpang",
    code: "PW-224",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-224",
};

pub const PW_226: Subdivision = Subdivision {
    name: "Ngchesar",
    code: "PW-226",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-226",
};

pub const PW_227: Subdivision = Subdivision {
    name: "Ngeremlengui",
    code: "PW-227",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-227",
};

pub const PW_228: Subdivision = Subdivision {
    name: "Ngiwal",
    code: "PW-228",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-228",
};

pub const PW_350: Subdivision = Subdivision {
    name: "Peleliu",
    code: "PW-350",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-350",
};

pub const PW_370: Subdivision = Subdivision {
    name: "Sonsorol",
    code: "PW-370",
    subdivision_type: "State",
    country_name: "Palau",
    country_code: "PW",
    region_code: "PW-370",
};

pub const PS_BTH: Subdivision = Subdivision {
    name: "Bayt Laḩm",
    code: "PS-BTH",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-BTH",
};

pub const PS_DEB: Subdivision = Subdivision {
    name: "Dayr al Balaḩ",
    code: "PS-DEB",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-DEB",
};

pub const PS_GZA: Subdivision = Subdivision {
    name: "Gaza",
    code: "PS-GZA",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-GZA",
};

pub const PS_HBN: Subdivision = Subdivision {
    name: "Al Khalīl",
    code: "PS-HBN",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-HBN",
};

pub const PS_JEM: Subdivision = Subdivision {
    name: "Al Quds",
    code: "PS-JEM",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-JEM",
};

pub const PS_JEN: Subdivision = Subdivision {
    name: "Janīn",
    code: "PS-JEN",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-JEN",
};

pub const PS_JRH: Subdivision = Subdivision {
    name: "Arīḩā wa al Aghwār",
    code: "PS-JRH",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-JRH",
};

pub const PS_KYS: Subdivision = Subdivision {
    name: "Khan Yunis",
    code: "PS-KYS",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-KYS",
};

pub const PS_NBS: Subdivision = Subdivision {
    name: "Nablus",
    code: "PS-NBS",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-NBS",
};

pub const PS_NGZ: Subdivision = Subdivision {
    name: "North Gaza",
    code: "PS-NGZ",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-NGZ",
};

pub const PS_QQA: Subdivision = Subdivision {
    name: "Qalqilya",
    code: "PS-QQA",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-QQA",
};

pub const PS_RBH: Subdivision = Subdivision {
    name: "Ramallah",
    code: "PS-RBH",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-RBH",
};

pub const PS_RFH: Subdivision = Subdivision {
    name: "Rafah",
    code: "PS-RFH",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-RFH",
};

pub const PS_SLT: Subdivision = Subdivision {
    name: "Salfit",
    code: "PS-SLT",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-SLT",
};

pub const PS_TBS: Subdivision = Subdivision {
    name: "Tubas",
    code: "PS-TBS",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-TBS",
};

pub const PS_TKM: Subdivision = Subdivision {
    name: "Tulkarm",
    code: "PS-TKM",
    subdivision_type: "Governorate",
    country_name: "Palestine",
    country_code: "PS",
    region_code: "PS-TKM",
};

pub const PA_10: Subdivision = Subdivision {
    name: "Panamá Oeste",
    code: "PA-10",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-10",
};

pub const PA_1: Subdivision = Subdivision {
    name: "Bocas del Toro",
    code: "PA-1",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-1",
};

pub const PA_2: Subdivision = Subdivision {
    name: "Coclé",
    code: "PA-2",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-2",
};

pub const PA_3: Subdivision = Subdivision {
    name: "Colón",
    code: "PA-3",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-3",
};

pub const PA_4: Subdivision = Subdivision {
    name: "Chiriquí",
    code: "PA-4",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-4",
};

pub const PA_5: Subdivision = Subdivision {
    name: "Darién",
    code: "PA-5",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-5",
};

pub const PA_6: Subdivision = Subdivision {
    name: "Herrera",
    code: "PA-6",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-6",
};

pub const PA_7: Subdivision = Subdivision {
    name: "Los Santos",
    code: "PA-7",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-7",
};

pub const PA_8: Subdivision = Subdivision {
    name: "Panamá",
    code: "PA-8",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-8",
};

pub const PA_9: Subdivision = Subdivision {
    name: "Veraguas",
    code: "PA-9",
    subdivision_type: "Province",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-9",
};

pub const PA_EM: Subdivision = Subdivision {
    name: "Emberá",
    code: "PA-EM",
    subdivision_type: "Indigenous region",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-EM",
};

pub const PA_KY: Subdivision = Subdivision {
    name: "Kuna Yala",
    code: "PA-KY",
    subdivision_type: "Indigenous region",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-KY",
};

pub const PA_NB: Subdivision = Subdivision {
    name: "Ngöbe-Buglé",
    code: "PA-NB",
    subdivision_type: "Indigenous region",
    country_name: "Panama",
    country_code: "PA",
    region_code: "PA-NB",
};

pub const PG_CPK: Subdivision = Subdivision {
    name: "Chimbu",
    code: "PG-CPK",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-CPK",
};

pub const PG_CPM: Subdivision = Subdivision {
    name: "Central",
    code: "PG-CPM",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-CPM",
};

pub const PG_EBR: Subdivision = Subdivision {
    name: "East New Britain",
    code: "PG-EBR",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-EBR",
};

pub const PG_EHG: Subdivision = Subdivision {
    name: "Eastern Highlands",
    code: "PG-EHG",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-EHG",
};

pub const PG_EPW: Subdivision = Subdivision {
    name: "Enga",
    code: "PG-EPW",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-EPW",
};

pub const PG_ESW: Subdivision = Subdivision {
    name: "East Sepik",
    code: "PG-ESW",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-ESW",
};

pub const PG_GPK: Subdivision = Subdivision {
    name: "Gulf",
    code: "PG-GPK",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-GPK",
};

pub const PG_HLA: Subdivision = Subdivision {
    name: "Hela",
    code: "PG-HLA",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-HLA",
};

pub const PG_JWK: Subdivision = Subdivision {
    name: "Jiwaka",
    code: "PG-JWK",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-JWK",
};

pub const PG_MBA: Subdivision = Subdivision {
    name: "Milne Bay",
    code: "PG-MBA",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-MBA",
};

pub const PG_MPL: Subdivision = Subdivision {
    name: "Morobe",
    code: "PG-MPL",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-MPL",
};

pub const PG_MPM: Subdivision = Subdivision {
    name: "Madang",
    code: "PG-MPM",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-MPM",
};

pub const PG_MRL: Subdivision = Subdivision {
    name: "Manus",
    code: "PG-MRL",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-MRL",
};

pub const PG_NCD: Subdivision = Subdivision {
    name: "National Capital District",
    code: "PG-NCD",
    subdivision_type: "District",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-NCD",
};

pub const PG_NIK: Subdivision = Subdivision {
    name: "New Ireland",
    code: "PG-NIK",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-NIK",
};

pub const PG_NPP: Subdivision = Subdivision {
    name: "Northern",
    code: "PG-NPP",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-NPP",
};

pub const PG_NSB: Subdivision = Subdivision {
    name: "Bougainville",
    code: "PG-NSB",
    subdivision_type: "Autonomous region",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-NSB",
};

pub const PG_SAN: Subdivision = Subdivision {
    name: "West Sepik",
    code: "PG-SAN",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-SAN",
};

pub const PG_SHM: Subdivision = Subdivision {
    name: "Southern Highlands",
    code: "PG-SHM",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-SHM",
};

pub const PG_WBK: Subdivision = Subdivision {
    name: "West New Britain",
    code: "PG-WBK",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-WBK",
};

pub const PG_WHM: Subdivision = Subdivision {
    name: "Western Highlands",
    code: "PG-WHM",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-WHM",
};

pub const PG_WPD: Subdivision = Subdivision {
    name: "Western",
    code: "PG-WPD",
    subdivision_type: "Province",
    country_name: "Papua New Guinea",
    country_code: "PG",
    region_code: "PG-WPD",
};

pub const PY_10: Subdivision = Subdivision {
    name: "Alto Paraná",
    code: "PY-10",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-10",
};

pub const PY_11: Subdivision = Subdivision {
    name: "Central",
    code: "PY-11",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-11",
};

pub const PY_12: Subdivision = Subdivision {
    name: "Ñeembucú",
    code: "PY-12",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-12",
};

pub const PY_13: Subdivision = Subdivision {
    name: "Amambay",
    code: "PY-13",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-13",
};

pub const PY_14: Subdivision = Subdivision {
    name: "Canindeyú",
    code: "PY-14",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-14",
};

pub const PY_15: Subdivision = Subdivision {
    name: "Presidente Hayes",
    code: "PY-15",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-15",
};

pub const PY_16: Subdivision = Subdivision {
    name: "Alto Paraguay",
    code: "PY-16",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-16",
};

pub const PY_19: Subdivision = Subdivision {
    name: "Boquerón",
    code: "PY-19",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-19",
};

pub const PY_1: Subdivision = Subdivision {
    name: "Concepción",
    code: "PY-1",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-1",
};

pub const PY_2: Subdivision = Subdivision {
    name: "San Pedro",
    code: "PY-2",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-2",
};

pub const PY_3: Subdivision = Subdivision {
    name: "Cordillera",
    code: "PY-3",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-3",
};

pub const PY_4: Subdivision = Subdivision {
    name: "Guairá",
    code: "PY-4",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-4",
};

pub const PY_5: Subdivision = Subdivision {
    name: "Caaguazú",
    code: "PY-5",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-5",
};

pub const PY_6: Subdivision = Subdivision {
    name: "Caazapá",
    code: "PY-6",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-6",
};

pub const PY_7: Subdivision = Subdivision {
    name: "Itapúa",
    code: "PY-7",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-7",
};

pub const PY_8: Subdivision = Subdivision {
    name: "Misiones",
    code: "PY-8",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-8",
};

pub const PY_9: Subdivision = Subdivision {
    name: "Paraguarí",
    code: "PY-9",
    subdivision_type: "Department",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-9",
};

pub const PY_ASU: Subdivision = Subdivision {
    name: "Asunción",
    code: "PY-ASU",
    subdivision_type: "Capital",
    country_name: "Paraguay",
    country_code: "PY",
    region_code: "PY-ASU",
};

pub const PE_AMA: Subdivision = Subdivision {
    name: "Amarumayu",
    code: "PE-AMA",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-AMA",
};

pub const PE_ANC: Subdivision = Subdivision {
    name: "Ancash",
    code: "PE-ANC",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-ANC",
};

pub const PE_APU: Subdivision = Subdivision {
    name: "Apurímac",
    code: "PE-APU",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-APU",
};

pub const PE_ARE: Subdivision = Subdivision {
    name: "Arequipa",
    code: "PE-ARE",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-ARE",
};

pub const PE_AYA: Subdivision = Subdivision {
    name: "Ayacucho",
    code: "PE-AYA",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-AYA",
};

pub const PE_CAJ: Subdivision = Subdivision {
    name: "Cajamarca",
    code: "PE-CAJ",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-CAJ",
};

pub const PE_CAL: Subdivision = Subdivision {
    name: "El Callao",
    code: "PE-CAL",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-CAL",
};

pub const PE_CUS: Subdivision = Subdivision {
    name: "Cusco",
    code: "PE-CUS",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-CUS",
};

pub const PE_HUC: Subdivision = Subdivision {
    name: "Huánuco",
    code: "PE-HUC",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-HUC",
};

pub const PE_HUV: Subdivision = Subdivision {
    name: "Huancavelica",
    code: "PE-HUV",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-HUV",
};

pub const PE_ICA: Subdivision = Subdivision {
    name: "Ica",
    code: "PE-ICA",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-ICA",
};

pub const PE_JUN: Subdivision = Subdivision {
    name: "Hunin",
    code: "PE-JUN",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-JUN",
};

pub const PE_LAL: Subdivision = Subdivision {
    name: "La Libertad",
    code: "PE-LAL",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-LAL",
};

pub const PE_LAM: Subdivision = Subdivision {
    name: "Lambayeque",
    code: "PE-LAM",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-LAM",
};

pub const PE_LIM: Subdivision = Subdivision {
    name: "Lima",
    code: "PE-LIM",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-LIM",
};

pub const PE_LMA: Subdivision = Subdivision {
    name: "Lima hatun llaqta",
    code: "PE-LMA",
    subdivision_type: "Municipality",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-LMA",
};

pub const PE_LOR: Subdivision = Subdivision {
    name: "Loreto",
    code: "PE-LOR",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-LOR",
};

pub const PE_MDD: Subdivision = Subdivision {
    name: "Madre de Dios",
    code: "PE-MDD",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-MDD",
};

pub const PE_MOQ: Subdivision = Subdivision {
    name: "Moquegua",
    code: "PE-MOQ",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-MOQ",
};

pub const PE_PAS: Subdivision = Subdivision {
    name: "Pasco",
    code: "PE-PAS",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-PAS",
};

pub const PE_PIU: Subdivision = Subdivision {
    name: "Piura",
    code: "PE-PIU",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-PIU",
};

pub const PE_PUN: Subdivision = Subdivision {
    name: "Puno",
    code: "PE-PUN",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-PUN",
};

pub const PE_SAM: Subdivision = Subdivision {
    name: "San Martin",
    code: "PE-SAM",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-SAM",
};

pub const PE_TAC: Subdivision = Subdivision {
    name: "Tacna",
    code: "PE-TAC",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-TAC",
};

pub const PE_TUM: Subdivision = Subdivision {
    name: "Tumbes",
    code: "PE-TUM",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-TUM",
};

pub const PE_UCA: Subdivision = Subdivision {
    name: "Ucayali",
    code: "PE-UCA",
    subdivision_type: "Region",
    country_name: "Peru",
    country_code: "PE",
    region_code: "PE-UCA",
};

pub const PH_00: Subdivision = Subdivision {
    name: "National Capital Region",
    code: "PH-00",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-00",
};

pub const PH_01: Subdivision = Subdivision {
    name: "Ilocos",
    code: "PH-01",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-01",
};

pub const PH_02: Subdivision = Subdivision {
    name: "Cagayan Valley",
    code: "PH-02",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-02",
};

pub const PH_03: Subdivision = Subdivision {
    name: "Central Luzon",
    code: "PH-03",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-03",
};

pub const PH_05: Subdivision = Subdivision {
    name: "Bicol",
    code: "PH-05",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-05",
};

pub const PH_06: Subdivision = Subdivision {
    name: "Rehiyon ng Kanlurang Bisaya",
    code: "PH-06",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-06",
};

pub const PH_07: Subdivision = Subdivision {
    name: "Central Visayas",
    code: "PH-07",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-07",
};

pub const PH_08: Subdivision = Subdivision {
    name: "Eastern Visayas",
    code: "PH-08",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-08",
};

pub const PH_09: Subdivision = Subdivision {
    name: "Rehiyon ng Tangway ng Sambuwangga",
    code: "PH-09",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-09",
};

pub const PH_10: Subdivision = Subdivision {
    name: "Northern Mindanao",
    code: "PH-10",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-10",
};

pub const PH_11: Subdivision = Subdivision {
    name: "Davao",
    code: "PH-11",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-11",
};

pub const PH_12: Subdivision = Subdivision {
    name: "Rehiyon ng Soccsksargen",
    code: "PH-12",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-12",
};

pub const PH_13: Subdivision = Subdivision {
    name: "Caraga",
    code: "PH-13",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-13",
};

pub const PH_14: Subdivision = Subdivision {
    name: "Autonomous Region in Muslim Mindanao",
    code: "PH-14",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-14",
};

pub const PH_15: Subdivision = Subdivision {
    name: "Cordillera Administrative Region",
    code: "PH-15",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-15",
};

pub const PH_40: Subdivision = Subdivision {
    name: "Calabarzon",
    code: "PH-40",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-40",
};

pub const PH_41: Subdivision = Subdivision {
    name: "Mimaropa",
    code: "PH-41",
    subdivision_type: "Region",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-41",
};

pub const PH_ABR: Subdivision = Subdivision {
    name: "Abra",
    code: "PH-ABR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ABR",
};

pub const PH_AGN: Subdivision = Subdivision {
    name: "Agusan del Norte",
    code: "PH-AGN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-AGN",
};

pub const PH_AGS: Subdivision = Subdivision {
    name: "Agusan del Sur",
    code: "PH-AGS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-AGS",
};

pub const PH_AKL: Subdivision = Subdivision {
    name: "Aklan",
    code: "PH-AKL",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-AKL",
};

pub const PH_ALB: Subdivision = Subdivision {
    name: "Albay",
    code: "PH-ALB",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ALB",
};

pub const PH_ANT: Subdivision = Subdivision {
    name: "Antike",
    code: "PH-ANT",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ANT",
};

pub const PH_APA: Subdivision = Subdivision {
    name: "Apayao",
    code: "PH-APA",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-APA",
};

pub const PH_AUR: Subdivision = Subdivision {
    name: "Aurora",
    code: "PH-AUR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-AUR",
};

pub const PH_BAN: Subdivision = Subdivision {
    name: "Bataan",
    code: "PH-BAN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BAN",
};

pub const PH_BAS: Subdivision = Subdivision {
    name: "Basilan",
    code: "PH-BAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BAS",
};

pub const PH_BEN: Subdivision = Subdivision {
    name: "Benget",
    code: "PH-BEN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BEN",
};

pub const PH_BIL: Subdivision = Subdivision {
    name: "Biliran",
    code: "PH-BIL",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BIL",
};

pub const PH_BOH: Subdivision = Subdivision {
    name: "Bohol",
    code: "PH-BOH",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BOH",
};

pub const PH_BTG: Subdivision = Subdivision {
    name: "Batangas",
    code: "PH-BTG",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BTG",
};

pub const PH_BTN: Subdivision = Subdivision {
    name: "Batanes",
    code: "PH-BTN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BTN",
};

pub const PH_BUK: Subdivision = Subdivision {
    name: "Bukidnon",
    code: "PH-BUK",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BUK",
};

pub const PH_BUL: Subdivision = Subdivision {
    name: "Bulacan",
    code: "PH-BUL",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-BUL",
};

pub const PH_CAG: Subdivision = Subdivision {
    name: "Cagayan",
    code: "PH-CAG",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAG",
};

pub const PH_CAM: Subdivision = Subdivision {
    name: "Camiguin",
    code: "PH-CAM",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAM",
};

pub const PH_CAN: Subdivision = Subdivision {
    name: "Camarines Norte",
    code: "PH-CAN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAN",
};

pub const PH_CAP: Subdivision = Subdivision {
    name: "Capiz",
    code: "PH-CAP",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAP",
};

pub const PH_CAS: Subdivision = Subdivision {
    name: "Camarines Sur",
    code: "PH-CAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAS",
};

pub const PH_CAT: Subdivision = Subdivision {
    name: "Catanduanes",
    code: "PH-CAT",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAT",
};

pub const PH_CAV: Subdivision = Subdivision {
    name: "Cavite",
    code: "PH-CAV",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CAV",
};

pub const PH_CEB: Subdivision = Subdivision {
    name: "Cebu",
    code: "PH-CEB",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-CEB",
};

pub const PH_COM: Subdivision = Subdivision {
    name: "Compostela Valley",
    code: "PH-COM",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-COM",
};

pub const PH_DAO: Subdivision = Subdivision {
    name: "Davao Oriental",
    code: "PH-DAO",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-DAO",
};

pub const PH_DAS: Subdivision = Subdivision {
    name: "Davao del Sur",
    code: "PH-DAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-DAS",
};

pub const PH_DAV: Subdivision = Subdivision {
    name: "Davao del Norte",
    code: "PH-DAV",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-DAV",
};

pub const PH_DIN: Subdivision = Subdivision {
    name: "Dinagat Islands",
    code: "PH-DIN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-DIN",
};

pub const PH_EAS: Subdivision = Subdivision {
    name: "Eastern Samar",
    code: "PH-EAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-EAS",
};

pub const PH_GUI: Subdivision = Subdivision {
    name: "Gimaras",
    code: "PH-GUI",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-GUI",
};

pub const PH_IFU: Subdivision = Subdivision {
    name: "Ifugao",
    code: "PH-IFU",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-IFU",
};

pub const PH_ILI: Subdivision = Subdivision {
    name: "Iloilo",
    code: "PH-ILI",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ILI",
};

pub const PH_ILN: Subdivision = Subdivision {
    name: "Hilagang Iloko",
    code: "PH-ILN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ILN",
};

pub const PH_ILS: Subdivision = Subdivision {
    name: "Ilocos Sur",
    code: "PH-ILS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ILS",
};

pub const PH_ISA: Subdivision = Subdivision {
    name: "Isabela",
    code: "PH-ISA",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ISA",
};

pub const PH_KAL: Subdivision = Subdivision {
    name: "Kalinga",
    code: "PH-KAL",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-KAL",
};

pub const PH_LAG: Subdivision = Subdivision {
    name: "Laguna",
    code: "PH-LAG",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-LAG",
};

pub const PH_LAN: Subdivision = Subdivision {
    name: "Hilagang Lanaw",
    code: "PH-LAN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-LAN",
};

pub const PH_LAS: Subdivision = Subdivision {
    name: "Lanao del Sur",
    code: "PH-LAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-LAS",
};

pub const PH_LEY: Subdivision = Subdivision {
    name: "Leyte",
    code: "PH-LEY",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-LEY",
};

pub const PH_LUN: Subdivision = Subdivision {
    name: "La Union",
    code: "PH-LUN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-LUN",
};

pub const PH_MAD: Subdivision = Subdivision {
    name: "Marinduke",
    code: "PH-MAD",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MAD",
};

pub const PH_MAG: Subdivision = Subdivision {
    name: "Magindanaw",
    code: "PH-MAG",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MAG",
};

pub const PH_MAS: Subdivision = Subdivision {
    name: "Masbate",
    code: "PH-MAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MAS",
};

pub const PH_MDC: Subdivision = Subdivision {
    name: "Kanlurang Mindoro",
    code: "PH-MDC",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MDC",
};

pub const PH_MDR: Subdivision = Subdivision {
    name: "Mindoro Oriental",
    code: "PH-MDR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MDR",
};

pub const PH_MOU: Subdivision = Subdivision {
    name: "Lalawigang Bulubundukin",
    code: "PH-MOU",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MOU",
};

pub const PH_MSC: Subdivision = Subdivision {
    name: "Kanlurang Misamis",
    code: "PH-MSC",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MSC",
};

pub const PH_MSR: Subdivision = Subdivision {
    name: "Misamis Oriental",
    code: "PH-MSR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-MSR",
};

pub const PH_NCO: Subdivision = Subdivision {
    name: "Cotabato",
    code: "PH-NCO",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-NCO",
};

pub const PH_NEC: Subdivision = Subdivision {
    name: "Kanlurang Negros",
    code: "PH-NEC",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-NEC",
};

pub const PH_NER: Subdivision = Subdivision {
    name: "Negros Oriental",
    code: "PH-NER",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-NER",
};

pub const PH_NSA: Subdivision = Subdivision {
    name: "Hilagang Samar",
    code: "PH-NSA",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-NSA",
};

pub const PH_NUE: Subdivision = Subdivision {
    name: "Nueva Ecija",
    code: "PH-NUE",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-NUE",
};

pub const PH_NUV: Subdivision = Subdivision {
    name: "Nueva Vizcaya",
    code: "PH-NUV",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-NUV",
};

pub const PH_PAM: Subdivision = Subdivision {
    name: "Pampanga",
    code: "PH-PAM",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-PAM",
};

pub const PH_PAN: Subdivision = Subdivision {
    name: "Pangasinan",
    code: "PH-PAN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-PAN",
};

pub const PH_PLW: Subdivision = Subdivision {
    name: "Palawan",
    code: "PH-PLW",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-PLW",
};

pub const PH_QUE: Subdivision = Subdivision {
    name: "Keson",
    code: "PH-QUE",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-QUE",
};

pub const PH_QUI: Subdivision = Subdivision {
    name: "Kirino",
    code: "PH-QUI",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-QUI",
};

pub const PH_RIZ: Subdivision = Subdivision {
    name: "Risal",
    code: "PH-RIZ",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-RIZ",
};

pub const PH_ROM: Subdivision = Subdivision {
    name: "Romblon",
    code: "PH-ROM",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ROM",
};

pub const PH_SAR: Subdivision = Subdivision {
    name: "Sarangani",
    code: "PH-SAR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SAR",
};

pub const PH_SCO: Subdivision = Subdivision {
    name: "South Cotabato",
    code: "PH-SCO",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SCO",
};

pub const PH_SIG: Subdivision = Subdivision {
    name: "Sikihor",
    code: "PH-SIG",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SIG",
};

pub const PH_SLE: Subdivision = Subdivision {
    name: "Katimogang Leyte",
    code: "PH-SLE",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SLE",
};

pub const PH_SLU: Subdivision = Subdivision {
    name: "Sulu",
    code: "PH-SLU",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SLU",
};

pub const PH_SOR: Subdivision = Subdivision {
    name: "Sorsogon",
    code: "PH-SOR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SOR",
};

pub const PH_SUK: Subdivision = Subdivision {
    name: "Sultan Kudarat",
    code: "PH-SUK",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SUK",
};

pub const PH_SUN: Subdivision = Subdivision {
    name: "Hilagang Surigaw",
    code: "PH-SUN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SUN",
};

pub const PH_SUR: Subdivision = Subdivision {
    name: "Surigao del Sur",
    code: "PH-SUR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-SUR",
};

pub const PH_TAR: Subdivision = Subdivision {
    name: "Tarlac",
    code: "PH-TAR",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-TAR",
};

pub const PH_TAW: Subdivision = Subdivision {
    name: "Tawi-Tawi",
    code: "PH-TAW",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-TAW",
};

pub const PH_WSA: Subdivision = Subdivision {
    name: "Samar",
    code: "PH-WSA",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-WSA",
};

pub const PH_ZAN: Subdivision = Subdivision {
    name: "Hilagang Sambuwangga",
    code: "PH-ZAN",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ZAN",
};

pub const PH_ZAS: Subdivision = Subdivision {
    name: "Timog Sambuwangga",
    code: "PH-ZAS",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ZAS",
};

pub const PH_ZMB: Subdivision = Subdivision {
    name: "Sambales",
    code: "PH-ZMB",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ZMB",
};

pub const PH_ZSI: Subdivision = Subdivision {
    name: "Sambuwangga Sibugay",
    code: "PH-ZSI",
    subdivision_type: "Province",
    country_name: "Philippines",
    country_code: "PH",
    region_code: "PH-ZSI",
};

pub const PN_PN: Subdivision = Subdivision {
    name: "Pitcairn",
    code: "PN-PN",
    subdivision_type: "Country",
    country_name: "Pitcairn",
    country_code: "PN",
    region_code: "PN-PN",
};

pub const PL_DS: Subdivision = Subdivision {
    name: "Dolnośląskie",
    code: "PL-DS",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-DS",
};

pub const PL_KP: Subdivision = Subdivision {
    name: "Kujawsko-pomorskie",
    code: "PL-KP",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-KP",
};

pub const PL_LB: Subdivision = Subdivision {
    name: "Lubuskie",
    code: "PL-LB",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-LB",
};

pub const PL_LD: Subdivision = Subdivision {
    name: "Łódzkie",
    code: "PL-LD",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-LD",
};

pub const PL_LU: Subdivision = Subdivision {
    name: "Lubelskie",
    code: "PL-LU",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-LU",
};

pub const PL_MA: Subdivision = Subdivision {
    name: "Małopolskie",
    code: "PL-MA",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-MA",
};

pub const PL_MZ: Subdivision = Subdivision {
    name: "Mazowieckie",
    code: "PL-MZ",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-MZ",
};

pub const PL_OP: Subdivision = Subdivision {
    name: "Opolskie",
    code: "PL-OP",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-OP",
};

pub const PL_PD: Subdivision = Subdivision {
    name: "Podlaskie",
    code: "PL-PD",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-PD",
};

pub const PL_PK: Subdivision = Subdivision {
    name: "Podkarpackie",
    code: "PL-PK",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-PK",
};

pub const PL_PM: Subdivision = Subdivision {
    name: "Pomorskie",
    code: "PL-PM",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-PM",
};

pub const PL_SK: Subdivision = Subdivision {
    name: "Świętokrzyskie",
    code: "PL-SK",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-SK",
};

pub const PL_SL: Subdivision = Subdivision {
    name: "Śląskie",
    code: "PL-SL",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-SL",
};

pub const PL_WN: Subdivision = Subdivision {
    name: "Warmińsko-mazurskie",
    code: "PL-WN",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-WN",
};

pub const PL_WP: Subdivision = Subdivision {
    name: "Wielkopolskie",
    code: "PL-WP",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-WP",
};

pub const PL_ZP: Subdivision = Subdivision {
    name: "Zachodniopomorskie",
    code: "PL-ZP",
    subdivision_type: "Province",
    country_name: "Poland",
    country_code: "PL",
    region_code: "PL-ZP",
};

pub const PT_01: Subdivision = Subdivision {
    name: "Aveiro",
    code: "PT-01",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-01",
};

pub const PT_02: Subdivision = Subdivision {
    name: "Beja",
    code: "PT-02",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-02",
};

pub const PT_03: Subdivision = Subdivision {
    name: "Braga",
    code: "PT-03",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-03",
};

pub const PT_04: Subdivision = Subdivision {
    name: "Bragança",
    code: "PT-04",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-04",
};

pub const PT_05: Subdivision = Subdivision {
    name: "Castelo Branco",
    code: "PT-05",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-05",
};

pub const PT_06: Subdivision = Subdivision {
    name: "Coimbra",
    code: "PT-06",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-06",
};

pub const PT_07: Subdivision = Subdivision {
    name: "Évora",
    code: "PT-07",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-07",
};

pub const PT_08: Subdivision = Subdivision {
    name: "Faro",
    code: "PT-08",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-08",
};

pub const PT_09: Subdivision = Subdivision {
    name: "Guarda",
    code: "PT-09",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-09",
};

pub const PT_10: Subdivision = Subdivision {
    name: "Leiria",
    code: "PT-10",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-10",
};

pub const PT_11: Subdivision = Subdivision {
    name: "Lisboa",
    code: "PT-11",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-11",
};

pub const PT_12: Subdivision = Subdivision {
    name: "Portalegre",
    code: "PT-12",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-12",
};

pub const PT_13: Subdivision = Subdivision {
    name: "Porto",
    code: "PT-13",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-13",
};

pub const PT_14: Subdivision = Subdivision {
    name: "Santarém",
    code: "PT-14",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-14",
};

pub const PT_15: Subdivision = Subdivision {
    name: "Setúbal",
    code: "PT-15",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-15",
};

pub const PT_16: Subdivision = Subdivision {
    name: "Viana do Castelo",
    code: "PT-16",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-16",
};

pub const PT_17: Subdivision = Subdivision {
    name: "Vila Real",
    code: "PT-17",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-17",
};

pub const PT_18: Subdivision = Subdivision {
    name: "Viseu",
    code: "PT-18",
    subdivision_type: "District",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-18",
};

pub const PT_20: Subdivision = Subdivision {
    name: "Região Autónoma dos Açores",
    code: "PT-20",
    subdivision_type: "Autonomous region",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-20",
};

pub const PT_30: Subdivision = Subdivision {
    name: "Região Autónoma da Madeira",
    code: "PT-30",
    subdivision_type: "Autonomous region",
    country_name: "Portugal",
    country_code: "PT",
    region_code: "PT-30",
};

pub const PR_PR: Subdivision = Subdivision {
    name: "Puerto Rico",
    code: "PR-PR",
    subdivision_type: "Country",
    country_name: "Puerto Rico",
    country_code: "PR",
    region_code: "PR-PR",
};

pub const QA_DA: Subdivision = Subdivision {
    name: "Ad Dawḩah",
    code: "QA-DA",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-DA",
};

pub const QA_KH: Subdivision = Subdivision {
    name: "Al Khawr wa adh Dhakhīrah",
    code: "QA-KH",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-KH",
};

pub const QA_MS: Subdivision = Subdivision {
    name: "Ash Shamāl",
    code: "QA-MS",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-MS",
};

pub const QA_RA: Subdivision = Subdivision {
    name: "Ar Rayyān",
    code: "QA-RA",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-RA",
};

pub const QA_US: Subdivision = Subdivision {
    name: "Umm Şalāl",
    code: "QA-US",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-US",
};

pub const QA_WA: Subdivision = Subdivision {
    name: "Al Wakrah",
    code: "QA-WA",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-WA",
};

pub const QA_ZA: Subdivision = Subdivision {
    name: "Az̧ Z̧a‘āyin",
    code: "QA-ZA",
    subdivision_type: "Municipality",
    country_name: "Qatar",
    country_code: "QA",
    region_code: "QA-ZA",
};

pub const RE_RE: Subdivision = Subdivision {
    name: "Reunion",
    code: "RE-RE",
    subdivision_type: "Country",
    country_name: "Reunion",
    country_code: "RE",
    region_code: "RE-RE",
};

pub const RO_AB: Subdivision = Subdivision {
    name: "Alba",
    code: "RO-AB",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-AB",
};

pub const RO_AG: Subdivision = Subdivision {
    name: "Argeş",
    code: "RO-AG",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-AG",
};

pub const RO_AR: Subdivision = Subdivision {
    name: "Arad",
    code: "RO-AR",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-AR",
};

pub const RO_B: Subdivision = Subdivision {
    name: "Bucureşti",
    code: "RO-B",
    subdivision_type: "Municipality",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-B",
};

pub const RO_BC: Subdivision = Subdivision {
    name: "Bacău",
    code: "RO-BC",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BC",
};

pub const RO_BH: Subdivision = Subdivision {
    name: "Bihor",
    code: "RO-BH",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BH",
};

pub const RO_BN: Subdivision = Subdivision {
    name: "Bistriţa-Năsăud",
    code: "RO-BN",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BN",
};

pub const RO_BR: Subdivision = Subdivision {
    name: "Brăila",
    code: "RO-BR",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BR",
};

pub const RO_BT: Subdivision = Subdivision {
    name: "Botoşani",
    code: "RO-BT",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BT",
};

pub const RO_BV: Subdivision = Subdivision {
    name: "Braşov",
    code: "RO-BV",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BV",
};

pub const RO_BZ: Subdivision = Subdivision {
    name: "Buzău",
    code: "RO-BZ",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-BZ",
};

pub const RO_CJ: Subdivision = Subdivision {
    name: "Cluj",
    code: "RO-CJ",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-CJ",
};

pub const RO_CL: Subdivision = Subdivision {
    name: "Călăraşi",
    code: "RO-CL",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-CL",
};

pub const RO_CS: Subdivision = Subdivision {
    name: "Caraş-Severin",
    code: "RO-CS",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-CS",
};

pub const RO_CT: Subdivision = Subdivision {
    name: "Constanţa",
    code: "RO-CT",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-CT",
};

pub const RO_CV: Subdivision = Subdivision {
    name: "Covasna",
    code: "RO-CV",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-CV",
};

pub const RO_DB: Subdivision = Subdivision {
    name: "Dâmboviţa",
    code: "RO-DB",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-DB",
};

pub const RO_DJ: Subdivision = Subdivision {
    name: "Dolj",
    code: "RO-DJ",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-DJ",
};

pub const RO_GJ: Subdivision = Subdivision {
    name: "Gorj",
    code: "RO-GJ",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-GJ",
};

pub const RO_GL: Subdivision = Subdivision {
    name: "Galaţi",
    code: "RO-GL",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-GL",
};

pub const RO_GR: Subdivision = Subdivision {
    name: "Giurgiu",
    code: "RO-GR",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-GR",
};

pub const RO_HD: Subdivision = Subdivision {
    name: "Hunedoara",
    code: "RO-HD",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-HD",
};

pub const RO_HR: Subdivision = Subdivision {
    name: "Harghita",
    code: "RO-HR",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-HR",
};

pub const RO_IF: Subdivision = Subdivision {
    name: "Ilfov",
    code: "RO-IF",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-IF",
};

pub const RO_IL: Subdivision = Subdivision {
    name: "Ialomiţa",
    code: "RO-IL",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-IL",
};

pub const RO_IS: Subdivision = Subdivision {
    name: "Iaşi",
    code: "RO-IS",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-IS",
};

pub const RO_MH: Subdivision = Subdivision {
    name: "Mehedinţi",
    code: "RO-MH",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-MH",
};

pub const RO_MM: Subdivision = Subdivision {
    name: "Maramureş",
    code: "RO-MM",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-MM",
};

pub const RO_MS: Subdivision = Subdivision {
    name: "Mureş",
    code: "RO-MS",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-MS",
};

pub const RO_NT: Subdivision = Subdivision {
    name: "Neamţ",
    code: "RO-NT",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-NT",
};

pub const RO_OT: Subdivision = Subdivision {
    name: "Olt",
    code: "RO-OT",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-OT",
};

pub const RO_PH: Subdivision = Subdivision {
    name: "Prahova",
    code: "RO-PH",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-PH",
};

pub const RO_SB: Subdivision = Subdivision {
    name: "Sibiu",
    code: "RO-SB",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-SB",
};

pub const RO_SJ: Subdivision = Subdivision {
    name: "Sălaj",
    code: "RO-SJ",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-SJ",
};

pub const RO_SM: Subdivision = Subdivision {
    name: "Satu Mare",
    code: "RO-SM",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-SM",
};

pub const RO_SV: Subdivision = Subdivision {
    name: "Suceava",
    code: "RO-SV",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-SV",
};

pub const RO_TL: Subdivision = Subdivision {
    name: "Tulcea",
    code: "RO-TL",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-TL",
};

pub const RO_TM: Subdivision = Subdivision {
    name: "Timiş",
    code: "RO-TM",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-TM",
};

pub const RO_TR: Subdivision = Subdivision {
    name: "Teleorman",
    code: "RO-TR",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-TR",
};

pub const RO_VL: Subdivision = Subdivision {
    name: "Vâlcea",
    code: "RO-VL",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-VL",
};

pub const RO_VN: Subdivision = Subdivision {
    name: "Vrancea",
    code: "RO-VN",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-VN",
};

pub const RO_VS: Subdivision = Subdivision {
    name: "Vaslui",
    code: "RO-VS",
    subdivision_type: "Department",
    country_name: "Romania",
    country_code: "RO",
    region_code: "RO-VS",
};

pub const RU_AD: Subdivision = Subdivision {
    name: "Adygeja, Respublika",
    code: "RU-AD",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-AD",
};

pub const RU_AL: Subdivision = Subdivision {
    name: "Altaj, Respublika",
    code: "RU-AL",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-AL",
};

pub const RU_ALT: Subdivision = Subdivision {
    name: "Altajskij kraj",
    code: "RU-ALT",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ALT",
};

pub const RU_AMU: Subdivision = Subdivision {
    name: "Amurskaja oblast'",
    code: "RU-AMU",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-AMU",
};

pub const RU_ARK: Subdivision = Subdivision {
    name: "Arhangel'skaja oblast'",
    code: "RU-ARK",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ARK",
};

pub const RU_AST: Subdivision = Subdivision {
    name: "Astrahanskaja oblast'",
    code: "RU-AST",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-AST",
};

pub const RU_BA: Subdivision = Subdivision {
    name: "Bashkortostan, Respublika",
    code: "RU-BA",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-BA",
};

pub const RU_BEL: Subdivision = Subdivision {
    name: "Belgorodskaja oblast'",
    code: "RU-BEL",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-BEL",
};

pub const RU_BRY: Subdivision = Subdivision {
    name: "Brjanskaja oblast'",
    code: "RU-BRY",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-BRY",
};

pub const RU_BU: Subdivision = Subdivision {
    name: "Burjatija, Respublika",
    code: "RU-BU",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-BU",
};

pub const RU_CE: Subdivision = Subdivision {
    name: "Čečenskaja Respublika",
    code: "RU-CE",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-CE",
};

pub const RU_CHE: Subdivision = Subdivision {
    name: "Čeljabinskaja oblast'",
    code: "RU-CHE",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-CHE",
};

pub const RU_CHU: Subdivision = Subdivision {
    name: "Chukotskiy avtonomnyy okrug",
    code: "RU-CHU",
    subdivision_type: "Autonomous district",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-CHU",
};

pub const RU_CU: Subdivision = Subdivision {
    name: "Chuvashskaya Respublika",
    code: "RU-CU",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-CU",
};

pub const RU_DA: Subdivision = Subdivision {
    name: "Dagestan, Respublika",
    code: "RU-DA",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-DA",
};

pub const RU_IN: Subdivision = Subdivision {
    name: "Ingušetija, Respublika",
    code: "RU-IN",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-IN",
};

pub const RU_IRK: Subdivision = Subdivision {
    name: "Irkutskaja oblast'",
    code: "RU-IRK",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-IRK",
};

pub const RU_IVA: Subdivision = Subdivision {
    name: "Ivanovskaja oblast'",
    code: "RU-IVA",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-IVA",
};

pub const RU_KAM: Subdivision = Subdivision {
    name: "Kamčatskij kraj",
    code: "RU-KAM",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KAM",
};

pub const RU_KB: Subdivision = Subdivision {
    name: "Kabardino-Balkarskaja Respublika",
    code: "RU-KB",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KB",
};

pub const RU_KC: Subdivision = Subdivision {
    name: "Karačaevo-Čerkesskaja Respublika",
    code: "RU-KC",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KC",
};

pub const RU_KDA: Subdivision = Subdivision {
    name: "Krasnodarskij kraj",
    code: "RU-KDA",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KDA",
};

pub const RU_KEM: Subdivision = Subdivision {
    name: "Kemerovskaja oblast'",
    code: "RU-KEM",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KEM",
};

pub const RU_KGD: Subdivision = Subdivision {
    name: "Kaliningradskaja oblast'",
    code: "RU-KGD",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KGD",
};

pub const RU_KGN: Subdivision = Subdivision {
    name: "Kurganskaja oblast'",
    code: "RU-KGN",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KGN",
};

pub const RU_KHA: Subdivision = Subdivision {
    name: "Habarovskij kraj",
    code: "RU-KHA",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KHA",
};

pub const RU_KHM: Subdivision = Subdivision {
    name: "Hanty-Mansijskij avtonomnyj okrug",
    code: "RU-KHM",
    subdivision_type: "Autonomous district",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KHM",
};

pub const RU_KIR: Subdivision = Subdivision {
    name: "Kirovskaja oblast'",
    code: "RU-KIR",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KIR",
};

pub const RU_KK: Subdivision = Subdivision {
    name: "Hakasija, Respublika",
    code: "RU-KK",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KK",
};

pub const RU_KL: Subdivision = Subdivision {
    name: "Kalmykija, Respublika",
    code: "RU-KL",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KL",
};

pub const RU_KLU: Subdivision = Subdivision {
    name: "Kaluzhskaya oblast'",
    code: "RU-KLU",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KLU",
};

pub const RU_KO: Subdivision = Subdivision {
    name: "Komi, Respublika",
    code: "RU-KO",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KO",
};

pub const RU_KOS: Subdivision = Subdivision {
    name: "Kostromskaja oblast'",
    code: "RU-KOS",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KOS",
};

pub const RU_KR: Subdivision = Subdivision {
    name: "Karelija, Respublika",
    code: "RU-KR",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KR",
};

pub const RU_KRS: Subdivision = Subdivision {
    name: "Kurskaja oblast'",
    code: "RU-KRS",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KRS",
};

pub const RU_KYA: Subdivision = Subdivision {
    name: "Krasnojarskij kraj",
    code: "RU-KYA",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-KYA",
};

pub const RU_LEN: Subdivision = Subdivision {
    name: "Leningradskaja oblast'",
    code: "RU-LEN",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-LEN",
};

pub const RU_LIP: Subdivision = Subdivision {
    name: "Lipeckaja oblast'",
    code: "RU-LIP",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-LIP",
};

pub const RU_MAG: Subdivision = Subdivision {
    name: "Magadanskaja oblast'",
    code: "RU-MAG",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-MAG",
};

pub const RU_ME: Subdivision = Subdivision {
    name: "Marij Èl, Respublika",
    code: "RU-ME",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ME",
};

pub const RU_MO: Subdivision = Subdivision {
    name: "Mordovija, Respublika",
    code: "RU-MO",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-MO",
};

pub const RU_MOS: Subdivision = Subdivision {
    name: "Moskovskaja oblast'",
    code: "RU-MOS",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-MOS",
};

pub const RU_MOW: Subdivision = Subdivision {
    name: "Moskva",
    code: "RU-MOW",
    subdivision_type: "Autonomous city",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-MOW",
};

pub const RU_MUR: Subdivision = Subdivision {
    name: "Murmanskaja oblast'",
    code: "RU-MUR",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-MUR",
};

pub const RU_NEN: Subdivision = Subdivision {
    name: "Neneckij avtonomnyj okrug",
    code: "RU-NEN",
    subdivision_type: "Autonomous district",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-NEN",
};

pub const RU_NGR: Subdivision = Subdivision {
    name: "Novgorodskaja oblast'",
    code: "RU-NGR",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-NGR",
};

pub const RU_NIZ: Subdivision = Subdivision {
    name: "Nižegorodskaja oblast'",
    code: "RU-NIZ",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-NIZ",
};

pub const RU_NVS: Subdivision = Subdivision {
    name: "Novosibirskaja oblast'",
    code: "RU-NVS",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-NVS",
};

pub const RU_OMS: Subdivision = Subdivision {
    name: "Omskaja oblast'",
    code: "RU-OMS",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-OMS",
};

pub const RU_ORE: Subdivision = Subdivision {
    name: "Orenburgskaja oblast'",
    code: "RU-ORE",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ORE",
};

pub const RU_ORL: Subdivision = Subdivision {
    name: "Orlovskaja oblast'",
    code: "RU-ORL",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ORL",
};

pub const RU_PER: Subdivision = Subdivision {
    name: "Permskij kraj",
    code: "RU-PER",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-PER",
};

pub const RU_PNZ: Subdivision = Subdivision {
    name: "Penzenskaja oblast'",
    code: "RU-PNZ",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-PNZ",
};

pub const RU_PRI: Subdivision = Subdivision {
    name: "Primorskij kraj",
    code: "RU-PRI",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-PRI",
};

pub const RU_PSK: Subdivision = Subdivision {
    name: "Pskovskaja oblast'",
    code: "RU-PSK",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-PSK",
};

pub const RU_ROS: Subdivision = Subdivision {
    name: "Rostovskaja oblast'",
    code: "RU-ROS",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ROS",
};

pub const RU_RYA: Subdivision = Subdivision {
    name: "Rjazanskaja oblast'",
    code: "RU-RYA",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-RYA",
};

pub const RU_SAK: Subdivision = Subdivision {
    name: "Sahalinskaja oblast'",
    code: "RU-SAK",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SAK",
};

pub const RU_SAM: Subdivision = Subdivision {
    name: "Samarskaja oblast'",
    code: "RU-SAM",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SAM",
};

pub const RU_SAR: Subdivision = Subdivision {
    name: "Saratovskaja oblast'",
    code: "RU-SAR",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SAR",
};

pub const RU_SA: Subdivision = Subdivision {
    name: "Saha, Respublika",
    code: "RU-SA",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SA",
};

pub const RU_SE: Subdivision = Subdivision {
    name: "Severnaja Osetija, Respublika",
    code: "RU-SE",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SE",
};

pub const RU_SMO: Subdivision = Subdivision {
    name: "Smolenskaja oblast'",
    code: "RU-SMO",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SMO",
};

pub const RU_SPE: Subdivision = Subdivision {
    name: "Sankt-Peterburg",
    code: "RU-SPE",
    subdivision_type: "Autonomous city",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SPE",
};

pub const RU_STA: Subdivision = Subdivision {
    name: "Stavropol'skij kraj",
    code: "RU-STA",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-STA",
};

pub const RU_SVE: Subdivision = Subdivision {
    name: "Sverdlovskaja oblast'",
    code: "RU-SVE",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-SVE",
};

pub const RU_TAM: Subdivision = Subdivision {
    name: "Tambovskaja oblast'",
    code: "RU-TAM",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TAM",
};

pub const RU_TA: Subdivision = Subdivision {
    name: "Tatarstan, Respublika",
    code: "RU-TA",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TA",
};

pub const RU_TOM: Subdivision = Subdivision {
    name: "Tomskaja oblast'",
    code: "RU-TOM",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TOM",
};

pub const RU_TUL: Subdivision = Subdivision {
    name: "Tul'skaja oblast'",
    code: "RU-TUL",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TUL",
};

pub const RU_TVE: Subdivision = Subdivision {
    name: "Tverskaja oblast'",
    code: "RU-TVE",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TVE",
};

pub const RU_TY: Subdivision = Subdivision {
    name: "Tyva, Respublika",
    code: "RU-TY",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TY",
};

pub const RU_TYU: Subdivision = Subdivision {
    name: "Tjumenskaja oblast'",
    code: "RU-TYU",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-TYU",
};

pub const RU_UD: Subdivision = Subdivision {
    name: "Udmurtskaja Respublika",
    code: "RU-UD",
    subdivision_type: "Republic",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-UD",
};

pub const RU_ULY: Subdivision = Subdivision {
    name: "Ul'janovskaja oblast'",
    code: "RU-ULY",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ULY",
};

pub const RU_VGG: Subdivision = Subdivision {
    name: "Volgogradskaja oblast'",
    code: "RU-VGG",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-VGG",
};

pub const RU_VLA: Subdivision = Subdivision {
    name: "Vladimirskaja oblast'",
    code: "RU-VLA",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-VLA",
};

pub const RU_VLG: Subdivision = Subdivision {
    name: "Vologodskaja oblast'",
    code: "RU-VLG",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-VLG",
};

pub const RU_VOR: Subdivision = Subdivision {
    name: "Voronezhskaya oblast'",
    code: "RU-VOR",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-VOR",
};

pub const RU_YAN: Subdivision = Subdivision {
    name: "Jamalo-Neneckij avtonomnyj okrug",
    code: "RU-YAN",
    subdivision_type: "Autonomous district",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-YAN",
};

pub const RU_YAR: Subdivision = Subdivision {
    name: "Jaroslavskaja oblast'",
    code: "RU-YAR",
    subdivision_type: "Administrative region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-YAR",
};

pub const RU_YEV: Subdivision = Subdivision {
    name: "Evrejskaja avtonomnaja oblast'",
    code: "RU-YEV",
    subdivision_type: "Autonomous region",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-YEV",
};

pub const RU_ZAB: Subdivision = Subdivision {
    name: "Zabajkal'skij kraj",
    code: "RU-ZAB",
    subdivision_type: "Administrative territory",
    country_name: "Russia",
    country_code: "RU",
    region_code: "RU-ZAB",
};

pub const RW_01: Subdivision = Subdivision {
    name: "Ville de Kigal",
    code: "RW-01",
    subdivision_type: "Town council",
    country_name: "Rwanda",
    country_code: "RW",
    region_code: "RW-01",
};

pub const RW_02: Subdivision = Subdivision {
    name: "Est",
    code: "RW-02",
    subdivision_type: "Province",
    country_name: "Rwanda",
    country_code: "RW",
    region_code: "RW-02",
};

pub const RW_03: Subdivision = Subdivision {
    name: "Nord",
    code: "RW-03",
    subdivision_type: "Province",
    country_name: "Rwanda",
    country_code: "RW",
    region_code: "RW-03",
};

pub const RW_04: Subdivision = Subdivision {
    name: "Ouest",
    code: "RW-04",
    subdivision_type: "Province",
    country_name: "Rwanda",
    country_code: "RW",
    region_code: "RW-04",
};

pub const RW_05: Subdivision = Subdivision {
    name: "Sud",
    code: "RW-05",
    subdivision_type: "Province",
    country_name: "Rwanda",
    country_code: "RW",
    region_code: "RW-05",
};

pub const SH_AC: Subdivision = Subdivision {
    name: "Ascension",
    code: "SH-AC",
    subdivision_type: "Geographical entity",
    country_name: "Saint Helena, Ascension and Tristan Da Cunha",
    country_code: "SH",
    region_code: "SH-AC",
};

pub const SH_HL: Subdivision = Subdivision {
    name: "Saint Helena",
    code: "SH-HL",
    subdivision_type: "Geographical entity",
    country_name: "Saint Helena, Ascension and Tristan Da Cunha",
    country_code: "SH",
    region_code: "SH-HL",
};

pub const SH_TA: Subdivision = Subdivision {
    name: "Tristan da Cunha",
    code: "SH-TA",
    subdivision_type: "Geographical entity",
    country_name: "Saint Helena, Ascension and Tristan Da Cunha",
    country_code: "SH",
    region_code: "SH-TA",
};

pub const KN_01: Subdivision = Subdivision {
    name: "Christ Church Nichola Town",
    code: "KN-01",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-01",
};

pub const KN_02: Subdivision = Subdivision {
    name: "Saint Anne Sandy Point",
    code: "KN-02",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-02",
};

pub const KN_03: Subdivision = Subdivision {
    name: "Saint George Basseterre",
    code: "KN-03",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-03",
};

pub const KN_04: Subdivision = Subdivision {
    name: "Saint George Gingerland",
    code: "KN-04",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-04",
};

pub const KN_05: Subdivision = Subdivision {
    name: "Saint James Windward",
    code: "KN-05",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-05",
};

pub const KN_06: Subdivision = Subdivision {
    name: "Saint John Capisterre",
    code: "KN-06",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-06",
};

pub const KN_07: Subdivision = Subdivision {
    name: "Saint John Figtree",
    code: "KN-07",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-07",
};

pub const KN_08: Subdivision = Subdivision {
    name: "Saint Mary Cayon",
    code: "KN-08",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-08",
};

pub const KN_09: Subdivision = Subdivision {
    name: "Saint Paul Capisterre",
    code: "KN-09",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-09",
};

pub const KN_10: Subdivision = Subdivision {
    name: "Saint Paul Charlestown",
    code: "KN-10",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-10",
};

pub const KN_11: Subdivision = Subdivision {
    name: "Saint Peter Basseterre",
    code: "KN-11",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-11",
};

pub const KN_12: Subdivision = Subdivision {
    name: "Saint Thomas Lowland",
    code: "KN-12",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-12",
};

pub const KN_13: Subdivision = Subdivision {
    name: "Saint Thomas Middle Island",
    code: "KN-13",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-13",
};

pub const KN_15: Subdivision = Subdivision {
    name: "Trinity Palmetto Point",
    code: "KN-15",
    subdivision_type: "Parish",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-15",
};

pub const KN_K: Subdivision = Subdivision {
    name: "Saint Kitts",
    code: "KN-K",
    subdivision_type: "State",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-K",
};

pub const KN_N: Subdivision = Subdivision {
    name: "Nevis",
    code: "KN-N",
    subdivision_type: "State",
    country_name: "Saint Kitts And Nevis",
    country_code: "KN",
    region_code: "KN-N",
};

pub const LC_01: Subdivision = Subdivision {
    name: "Anse la Raye",
    code: "LC-01",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-01",
};

pub const LC_02: Subdivision = Subdivision {
    name: "Castries",
    code: "LC-02",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-02",
};

pub const LC_03: Subdivision = Subdivision {
    name: "Choiseul",
    code: "LC-03",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-03",
};

pub const LC_05: Subdivision = Subdivision {
    name: "Dennery",
    code: "LC-05",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-05",
};

pub const LC_06: Subdivision = Subdivision {
    name: "Gros Islet",
    code: "LC-06",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-06",
};

pub const LC_07: Subdivision = Subdivision {
    name: "Laborie",
    code: "LC-07",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-07",
};

pub const LC_08: Subdivision = Subdivision {
    name: "Micoud",
    code: "LC-08",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-08",
};

pub const LC_10: Subdivision = Subdivision {
    name: "Soufrière",
    code: "LC-10",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-10",
};

pub const LC_11: Subdivision = Subdivision {
    name: "Vieux Fort",
    code: "LC-11",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-11",
};

pub const LC_12: Subdivision = Subdivision {
    name: "Canaries",
    code: "LC-12",
    subdivision_type: "District",
    country_name: "Saint Lucia",
    country_code: "LC",
    region_code: "LC-12",
};

pub const VC_01: Subdivision = Subdivision {
    name: "Charlotte",
    code: "VC-01",
    subdivision_type: "Parish",
    country_name: "Saint Vincent And The Grenadines",
    country_code: "VC",
    region_code: "VC-01",
};

pub const VC_02: Subdivision = Subdivision {
    name: "Saint Andrew",
    code: "VC-02",
    subdivision_type: "Parish",
    country_name: "Saint Vincent And The Grenadines",
    country_code: "VC",
    region_code: "VC-02",
};

pub const VC_03: Subdivision = Subdivision {
    name: "Saint David",
    code: "VC-03",
    subdivision_type: "Parish",
    country_name: "Saint Vincent And The Grenadines",
    country_code: "VC",
    region_code: "VC-03",
};

pub const VC_04: Subdivision = Subdivision {
    name: "Saint George",
    code: "VC-04",
    subdivision_type: "Parish",
    country_name: "Saint Vincent And The Grenadines",
    country_code: "VC",
    region_code: "VC-04",
};

pub const VC_05: Subdivision = Subdivision {
    name: "Saint Patrick",
    code: "VC-05",
    subdivision_type: "Parish",
    country_name: "Saint Vincent And The Grenadines",
    country_code: "VC",
    region_code: "VC-05",
};

pub const VC_06: Subdivision = Subdivision {
    name: "Grenadines",
    code: "VC-06",
    subdivision_type: "Parish",
    country_name: "Saint Vincent And The Grenadines",
    country_code: "VC",
    region_code: "VC-06",
};

pub const WS_AA: Subdivision = Subdivision {
    name: "A'ana",
    code: "WS-AA",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-AA",
};

pub const WS_AL: Subdivision = Subdivision {
    name: "Aiga-i-le-Tai",
    code: "WS-AL",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-AL",
};

pub const WS_AT: Subdivision = Subdivision {
    name: "Atua",
    code: "WS-AT",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-AT",
};

pub const WS_FA: Subdivision = Subdivision {
    name: "Fa'asaleleaga",
    code: "WS-FA",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-FA",
};

pub const WS_GE: Subdivision = Subdivision {
    name: "Gaga'emauga",
    code: "WS-GE",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-GE",
};

pub const WS_GI: Subdivision = Subdivision {
    name: "Gagaifomauga",
    code: "WS-GI",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-GI",
};

pub const WS_PA: Subdivision = Subdivision {
    name: "Palauli",
    code: "WS-PA",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-PA",
};

pub const WS_SA: Subdivision = Subdivision {
    name: "Satupa'itea",
    code: "WS-SA",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-SA",
};

pub const WS_TU: Subdivision = Subdivision {
    name: "Tuamasaga",
    code: "WS-TU",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-TU",
};

pub const WS_VF: Subdivision = Subdivision {
    name: "Va'a-o-Fonoti",
    code: "WS-VF",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-VF",
};

pub const WS_VS: Subdivision = Subdivision {
    name: "Vaisigano",
    code: "WS-VS",
    subdivision_type: "District",
    country_name: "Samoa",
    country_code: "WS",
    region_code: "WS-VS",
};

pub const SM_01: Subdivision = Subdivision {
    name: "Acquaviva",
    code: "SM-01",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-01",
};

pub const SM_02: Subdivision = Subdivision {
    name: "Chiesanuova",
    code: "SM-02",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-02",
};

pub const SM_03: Subdivision = Subdivision {
    name: "Domagnano",
    code: "SM-03",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-03",
};

pub const SM_04: Subdivision = Subdivision {
    name: "Faetano",
    code: "SM-04",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-04",
};

pub const SM_05: Subdivision = Subdivision {
    name: "Fiorentino",
    code: "SM-05",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-05",
};

pub const SM_06: Subdivision = Subdivision {
    name: "Borgo Maggiore",
    code: "SM-06",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-06",
};

pub const SM_07: Subdivision = Subdivision {
    name: "San Marino",
    code: "SM-07",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-07",
};

pub const SM_08: Subdivision = Subdivision {
    name: "Montegiardino",
    code: "SM-08",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-08",
};

pub const SM_09: Subdivision = Subdivision {
    name: "Serravalle",
    code: "SM-09",
    subdivision_type: "Municipality",
    country_name: "San Marino",
    country_code: "SM",
    region_code: "SM-09",
};

pub const ST_P: Subdivision = Subdivision {
    name: "Príncipe",
    code: "ST-P",
    subdivision_type: "Province",
    country_name: "Sao Tome and Principe",
    country_code: "ST",
    region_code: "ST-P",
};

pub const ST_S: Subdivision = Subdivision {
    name: "São Tomé",
    code: "ST-S",
    subdivision_type: "Province",
    country_name: "Sao Tome and Principe",
    country_code: "ST",
    region_code: "ST-S",
};

pub const SA_01: Subdivision = Subdivision {
    name: "Ar Riyāḑ",
    code: "SA-01",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-01",
};

pub const SA_02: Subdivision = Subdivision {
    name: "Makkah al Mukarramah",
    code: "SA-02",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-02",
};

pub const SA_03: Subdivision = Subdivision {
    name: "Al Madīnah al Munawwarah",
    code: "SA-03",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-03",
};

pub const SA_04: Subdivision = Subdivision {
    name: "Ash Sharqīyah",
    code: "SA-04",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-04",
};

pub const SA_05: Subdivision = Subdivision {
    name: "Al Qaşīm",
    code: "SA-05",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-05",
};

pub const SA_06: Subdivision = Subdivision {
    name: "Ḩā'il",
    code: "SA-06",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-06",
};

pub const SA_07: Subdivision = Subdivision {
    name: "Tabūk",
    code: "SA-07",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-07",
};

pub const SA_08: Subdivision = Subdivision {
    name: "Al Ḩudūd ash Shamālīyah",
    code: "SA-08",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-08",
};

pub const SA_09: Subdivision = Subdivision {
    name: "Jāzān",
    code: "SA-09",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-09",
};

pub const SA_10: Subdivision = Subdivision {
    name: "Najrān",
    code: "SA-10",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-10",
};

pub const SA_11: Subdivision = Subdivision {
    name: "Al Bāḩah",
    code: "SA-11",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-11",
};

pub const SA_12: Subdivision = Subdivision {
    name: "Al Jawf",
    code: "SA-12",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-12",
};

pub const SA_14: Subdivision = Subdivision {
    name: "'Asīr",
    code: "SA-14",
    subdivision_type: "Province",
    country_name: "Saudi Arabia",
    country_code: "SA",
    region_code: "SA-14",
};

pub const SN_DB: Subdivision = Subdivision {
    name: "Diourbel",
    code: "SN-DB",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-DB",
};

pub const SN_DK: Subdivision = Subdivision {
    name: "Dakar",
    code: "SN-DK",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-DK",
};

pub const SN_FK: Subdivision = Subdivision {
    name: "Fatick",
    code: "SN-FK",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-FK",
};

pub const SN_KA: Subdivision = Subdivision {
    name: "Kaffrine",
    code: "SN-KA",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-KA",
};

pub const SN_KD: Subdivision = Subdivision {
    name: "Kolda",
    code: "SN-KD",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-KD",
};

pub const SN_KE: Subdivision = Subdivision {
    name: "Kédougou",
    code: "SN-KE",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-KE",
};

pub const SN_KL: Subdivision = Subdivision {
    name: "Kaolack",
    code: "SN-KL",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-KL",
};

pub const SN_LG: Subdivision = Subdivision {
    name: "Louga",
    code: "SN-LG",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-LG",
};

pub const SN_MT: Subdivision = Subdivision {
    name: "Matam",
    code: "SN-MT",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-MT",
};

pub const SN_SE: Subdivision = Subdivision {
    name: "Sédhiou",
    code: "SN-SE",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-SE",
};

pub const SN_SL: Subdivision = Subdivision {
    name: "Saint-Louis",
    code: "SN-SL",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-SL",
};

pub const SN_TC: Subdivision = Subdivision {
    name: "Tambacounda",
    code: "SN-TC",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-TC",
};

pub const SN_TH: Subdivision = Subdivision {
    name: "Thiès",
    code: "SN-TH",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-TH",
};

pub const SN_ZG: Subdivision = Subdivision {
    name: "Ziguinchor",
    code: "SN-ZG",
    subdivision_type: "Region",
    country_name: "Senegal",
    country_code: "SN",
    region_code: "SN-ZG",
};

pub const RS_00: Subdivision = Subdivision {
    name: "Beograd",
    code: "RS-00",
    subdivision_type: "City",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-00",
};

pub const RS_01: Subdivision = Subdivision {
    name: "Severnobački okrug",
    code: "RS-01",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-01",
};

pub const RS_02: Subdivision = Subdivision {
    name: "Srednjebanatski okrug",
    code: "RS-02",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-02",
};

pub const RS_03: Subdivision = Subdivision {
    name: "Severnobanatski okrug",
    code: "RS-03",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-03",
};

pub const RS_04: Subdivision = Subdivision {
    name: "Južnobanatski okrug",
    code: "RS-04",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-04",
};

pub const RS_05: Subdivision = Subdivision {
    name: "Zapadnobački okrug",
    code: "RS-05",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-05",
};

pub const RS_06: Subdivision = Subdivision {
    name: "Južnobački okrug",
    code: "RS-06",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-06",
};

pub const RS_07: Subdivision = Subdivision {
    name: "Sremski okrug",
    code: "RS-07",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-07",
};

pub const RS_08: Subdivision = Subdivision {
    name: "Mačvanski okrug",
    code: "RS-08",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-08",
};

pub const RS_09: Subdivision = Subdivision {
    name: "Kolubarski okrug",
    code: "RS-09",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-09",
};

pub const RS_10: Subdivision = Subdivision {
    name: "Podunavski okrug",
    code: "RS-10",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-10",
};

pub const RS_11: Subdivision = Subdivision {
    name: "Braničevski okrug",
    code: "RS-11",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-11",
};

pub const RS_12: Subdivision = Subdivision {
    name: "Šumadijski okrug",
    code: "RS-12",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-12",
};

pub const RS_13: Subdivision = Subdivision {
    name: "Pomoravski okrug",
    code: "RS-13",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-13",
};

pub const RS_14: Subdivision = Subdivision {
    name: "Borski okrug",
    code: "RS-14",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-14",
};

pub const RS_15: Subdivision = Subdivision {
    name: "Zaječarski okrug",
    code: "RS-15",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-15",
};

pub const RS_16: Subdivision = Subdivision {
    name: "Zlatiborski okrug",
    code: "RS-16",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-16",
};

pub const RS_17: Subdivision = Subdivision {
    name: "Moravički okrug",
    code: "RS-17",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-17",
};

pub const RS_18: Subdivision = Subdivision {
    name: "Raški okrug",
    code: "RS-18",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-18",
};

pub const RS_19: Subdivision = Subdivision {
    name: "Rasinski okrug",
    code: "RS-19",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-19",
};

pub const RS_20: Subdivision = Subdivision {
    name: "Nišavski okrug",
    code: "RS-20",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-20",
};

pub const RS_21: Subdivision = Subdivision {
    name: "Toplički okrug",
    code: "RS-21",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-21",
};

pub const RS_22: Subdivision = Subdivision {
    name: "Pirotski okrug",
    code: "RS-22",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-22",
};

pub const RS_23: Subdivision = Subdivision {
    name: "Jablanički okrug",
    code: "RS-23",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-23",
};

pub const RS_24: Subdivision = Subdivision {
    name: "Pčinjski okrug",
    code: "RS-24",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-24",
};

pub const RS_25: Subdivision = Subdivision {
    name: "Kosovski okrug",
    code: "RS-25",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-25",
};

pub const RS_26: Subdivision = Subdivision {
    name: "Pećki okrug",
    code: "RS-26",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-26",
};

pub const RS_27: Subdivision = Subdivision {
    name: "Prizrenski okrug",
    code: "RS-27",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-27",
};

pub const RS_28: Subdivision = Subdivision {
    name: "Kosovsko-Mitrovački okrug",
    code: "RS-28",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-28",
};

pub const RS_29: Subdivision = Subdivision {
    name: "Kosovsko-Pomoravski okrug",
    code: "RS-29",
    subdivision_type: "District",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-29",
};

pub const RS_KM: Subdivision = Subdivision {
    name: "Kosovo-Metohija",
    code: "RS-KM",
    subdivision_type: "Autonomous province",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-KM",
};

pub const RS_VO: Subdivision = Subdivision {
    name: "Vojvodina",
    code: "RS-VO",
    subdivision_type: "Autonomous province",
    country_name: "Serbia",
    country_code: "RS",
    region_code: "RS-VO",
};

pub const SC_01: Subdivision = Subdivision {
    name: "Anse aux Pins",
    code: "SC-01",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-01",
};

pub const SC_02: Subdivision = Subdivision {
    name: "Ans Bwalo",
    code: "SC-02",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-02",
};

pub const SC_03: Subdivision = Subdivision {
    name: "Anse Etoile",
    code: "SC-03",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-03",
};

pub const SC_04: Subdivision = Subdivision {
    name: "Au Cap",
    code: "SC-04",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-04",
};

pub const SC_05: Subdivision = Subdivision {
    name: "Anse Royale",
    code: "SC-05",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-05",
};

pub const SC_06: Subdivision = Subdivision {
    name: "Baie Lazare",
    code: "SC-06",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-06",
};

pub const SC_07: Subdivision = Subdivision {
    name: "Baie Sainte-Anne",
    code: "SC-07",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-07",
};

pub const SC_08: Subdivision = Subdivision {
    name: "Beau Vallon",
    code: "SC-08",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-08",
};

pub const SC_09: Subdivision = Subdivision {
    name: "Bel Air",
    code: "SC-09",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-09",
};

pub const SC_10: Subdivision = Subdivision {
    name: "Bel Ombre",
    code: "SC-10",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-10",
};

pub const SC_11: Subdivision = Subdivision {
    name: "Cascade",
    code: "SC-11",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-11",
};

pub const SC_12: Subdivision = Subdivision {
    name: "Glacis",
    code: "SC-12",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-12",
};

pub const SC_13: Subdivision = Subdivision {
    name: "Grand'Anse Mahé",
    code: "SC-13",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-13",
};

pub const SC_14: Subdivision = Subdivision {
    name: "Grand'Anse Praslin",
    code: "SC-14",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-14",
};

pub const SC_15: Subdivision = Subdivision {
    name: "Ladig",
    code: "SC-15",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-15",
};

pub const SC_16: Subdivision = Subdivision {
    name: "English River",
    code: "SC-16",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-16",
};

pub const SC_17: Subdivision = Subdivision {
    name: "Mon Bikston",
    code: "SC-17",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-17",
};

pub const SC_18: Subdivision = Subdivision {
    name: "Mon Fleri",
    code: "SC-18",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-18",
};

pub const SC_19: Subdivision = Subdivision {
    name: "Plaisance",
    code: "SC-19",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-19",
};

pub const SC_20: Subdivision = Subdivision {
    name: "Pointe La Rue",
    code: "SC-20",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-20",
};

pub const SC_21: Subdivision = Subdivision {
    name: "Porglo",
    code: "SC-21",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-21",
};

pub const SC_22: Subdivision = Subdivision {
    name: "Saint Louis",
    code: "SC-22",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-22",
};

pub const SC_23: Subdivision = Subdivision {
    name: "Takamaka",
    code: "SC-23",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-23",
};

pub const SC_24: Subdivision = Subdivision {
    name: "Lemamel",
    code: "SC-24",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-24",
};

pub const SC_25: Subdivision = Subdivision {
    name: "Roche Caïman",
    code: "SC-25",
    subdivision_type: "District",
    country_name: "Seychelles",
    country_code: "SC",
    region_code: "SC-25",
};

pub const GS_GS: Subdivision = Subdivision {
    name: "S.Georgia & S.Sandwich Islands",
    code: "GS-GS",
    subdivision_type: "Country",
    country_name: "S.Georgia & S.Sandwich Islands",
    country_code: "GS",
    region_code: "GS-GS",
};

pub const SL_E: Subdivision = Subdivision {
    name: "Eastern",
    code: "SL-E",
    subdivision_type: "Province",
    country_name: "Sierra Leone",
    country_code: "SL",
    region_code: "SL-E",
};

pub const SL_N: Subdivision = Subdivision {
    name: "Northern",
    code: "SL-N",
    subdivision_type: "Province",
    country_name: "Sierra Leone",
    country_code: "SL",
    region_code: "SL-N",
};

pub const SL_S: Subdivision = Subdivision {
    name: "Southern",
    code: "SL-S",
    subdivision_type: "Province",
    country_name: "Sierra Leone",
    country_code: "SL",
    region_code: "SL-S",
};

pub const SL_W: Subdivision = Subdivision {
    name: "Western Area",
    code: "SL-W",
    subdivision_type: "Area",
    country_name: "Sierra Leone",
    country_code: "SL",
    region_code: "SL-W",
};

pub const SG_01: Subdivision = Subdivision {
    name: "Central Singapore",
    code: "SG-01",
    subdivision_type: "District",
    country_name: "Singapore",
    country_code: "SG",
    region_code: "SG-01",
};

pub const SG_02: Subdivision = Subdivision {
    name: "North East",
    code: "SG-02",
    subdivision_type: "District",
    country_name: "Singapore",
    country_code: "SG",
    region_code: "SG-02",
};

pub const SG_03: Subdivision = Subdivision {
    name: "North West",
    code: "SG-03",
    subdivision_type: "District",
    country_name: "Singapore",
    country_code: "SG",
    region_code: "SG-03",
};

pub const SG_04: Subdivision = Subdivision {
    name: "South East",
    code: "SG-04",
    subdivision_type: "District",
    country_name: "Singapore",
    country_code: "SG",
    region_code: "SG-04",
};

pub const SG_05: Subdivision = Subdivision {
    name: "South West",
    code: "SG-05",
    subdivision_type: "District",
    country_name: "Singapore",
    country_code: "SG",
    region_code: "SG-05",
};

pub const SK_BC: Subdivision = Subdivision {
    name: "Banskobystrický kraj",
    code: "SK-BC",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-BC",
};

pub const SK_BL: Subdivision = Subdivision {
    name: "Bratislavský kraj",
    code: "SK-BL",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-BL",
};

pub const SK_KI: Subdivision = Subdivision {
    name: "Košický kraj",
    code: "SK-KI",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-KI",
};

pub const SK_NI: Subdivision = Subdivision {
    name: "Nitriansky kraj",
    code: "SK-NI",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-NI",
};

pub const SK_PV: Subdivision = Subdivision {
    name: "Prešovský kraj",
    code: "SK-PV",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-PV",
};

pub const SK_TA: Subdivision = Subdivision {
    name: "Trnavský kraj",
    code: "SK-TA",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-TA",
};

pub const SK_TC: Subdivision = Subdivision {
    name: "Trenčiansky kraj",
    code: "SK-TC",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-TC",
};

pub const SK_ZI: Subdivision = Subdivision {
    name: "Žilinský kraj",
    code: "SK-ZI",
    subdivision_type: "Region",
    country_name: "Slovakia",
    country_code: "SK",
    region_code: "SK-ZI",
};

pub const SI_001: Subdivision = Subdivision {
    name: "Ajdovščina",
    code: "SI-001",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-001",
};

pub const SI_002: Subdivision = Subdivision {
    name: "Beltinci",
    code: "SI-002",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-002",
};

pub const SI_003: Subdivision = Subdivision {
    name: "Bled",
    code: "SI-003",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-003",
};

pub const SI_004: Subdivision = Subdivision {
    name: "Bohinj",
    code: "SI-004",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-004",
};

pub const SI_005: Subdivision = Subdivision {
    name: "Borovnica",
    code: "SI-005",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-005",
};

pub const SI_006: Subdivision = Subdivision {
    name: "Bovec",
    code: "SI-006",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-006",
};

pub const SI_007: Subdivision = Subdivision {
    name: "Brda",
    code: "SI-007",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-007",
};

pub const SI_008: Subdivision = Subdivision {
    name: "Brezovica",
    code: "SI-008",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-008",
};

pub const SI_009: Subdivision = Subdivision {
    name: "Brežice",
    code: "SI-009",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-009",
};

pub const SI_010: Subdivision = Subdivision {
    name: "Tišina",
    code: "SI-010",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-010",
};

pub const SI_011: Subdivision = Subdivision {
    name: "Celje",
    code: "SI-011",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-011",
};

pub const SI_012: Subdivision = Subdivision {
    name: "Cerklje na Gorenjskem",
    code: "SI-012",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-012",
};

pub const SI_013: Subdivision = Subdivision {
    name: "Cerknica",
    code: "SI-013",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-013",
};

pub const SI_014: Subdivision = Subdivision {
    name: "Cerkno",
    code: "SI-014",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-014",
};

pub const SI_015: Subdivision = Subdivision {
    name: "Črenšovci",
    code: "SI-015",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-015",
};

pub const SI_016: Subdivision = Subdivision {
    name: "Črna na Koroškem",
    code: "SI-016",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-016",
};

pub const SI_017: Subdivision = Subdivision {
    name: "Črnomelj",
    code: "SI-017",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-017",
};

pub const SI_018: Subdivision = Subdivision {
    name: "Destrnik",
    code: "SI-018",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-018",
};

pub const SI_019: Subdivision = Subdivision {
    name: "Divača",
    code: "SI-019",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-019",
};

pub const SI_020: Subdivision = Subdivision {
    name: "Dobrepolje",
    code: "SI-020",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-020",
};

pub const SI_021: Subdivision = Subdivision {
    name: "Dobrova-Polhov Gradec",
    code: "SI-021",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-021",
};

pub const SI_022: Subdivision = Subdivision {
    name: "Dol pri Ljubljani",
    code: "SI-022",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-022",
};

pub const SI_023: Subdivision = Subdivision {
    name: "Domžale",
    code: "SI-023",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-023",
};

pub const SI_024: Subdivision = Subdivision {
    name: "Dornava",
    code: "SI-024",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-024",
};

pub const SI_025: Subdivision = Subdivision {
    name: "Dravograd",
    code: "SI-025",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-025",
};

pub const SI_026: Subdivision = Subdivision {
    name: "Duplek",
    code: "SI-026",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-026",
};

pub const SI_027: Subdivision = Subdivision {
    name: "Gorenja vas-Poljane",
    code: "SI-027",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-027",
};

pub const SI_028: Subdivision = Subdivision {
    name: "Gorišnica",
    code: "SI-028",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-028",
};

pub const SI_029: Subdivision = Subdivision {
    name: "Gornja Radgona",
    code: "SI-029",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-029",
};

pub const SI_030: Subdivision = Subdivision {
    name: "Gornji Grad",
    code: "SI-030",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-030",
};

pub const SI_031: Subdivision = Subdivision {
    name: "Gornji Petrovci",
    code: "SI-031",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-031",
};

pub const SI_032: Subdivision = Subdivision {
    name: "Grosuplje",
    code: "SI-032",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-032",
};

pub const SI_033: Subdivision = Subdivision {
    name: "Šalovci",
    code: "SI-033",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-033",
};

pub const SI_034: Subdivision = Subdivision {
    name: "Hrastnik",
    code: "SI-034",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-034",
};

pub const SI_035: Subdivision = Subdivision {
    name: "Hrpelje-Kozina",
    code: "SI-035",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-035",
};

pub const SI_036: Subdivision = Subdivision {
    name: "Idrija",
    code: "SI-036",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-036",
};

pub const SI_037: Subdivision = Subdivision {
    name: "Ig",
    code: "SI-037",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-037",
};

pub const SI_038: Subdivision = Subdivision {
    name: "Ilirska Bistrica",
    code: "SI-038",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-038",
};

pub const SI_039: Subdivision = Subdivision {
    name: "Ivančna Gorica",
    code: "SI-039",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-039",
};

pub const SI_040: Subdivision = Subdivision {
    name: "Izola",
    code: "SI-040",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-040",
};

pub const SI_041: Subdivision = Subdivision {
    name: "Jesenice",
    code: "SI-041",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-041",
};

pub const SI_042: Subdivision = Subdivision {
    name: "Juršinci",
    code: "SI-042",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-042",
};

pub const SI_043: Subdivision = Subdivision {
    name: "Kamnik",
    code: "SI-043",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-043",
};

pub const SI_044: Subdivision = Subdivision {
    name: "Kanal",
    code: "SI-044",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-044",
};

pub const SI_045: Subdivision = Subdivision {
    name: "Kidričevo",
    code: "SI-045",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-045",
};

pub const SI_046: Subdivision = Subdivision {
    name: "Kobarid",
    code: "SI-046",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-046",
};

pub const SI_047: Subdivision = Subdivision {
    name: "Kobilje",
    code: "SI-047",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-047",
};

pub const SI_048: Subdivision = Subdivision {
    name: "Kočevje",
    code: "SI-048",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-048",
};

pub const SI_049: Subdivision = Subdivision {
    name: "Komen",
    code: "SI-049",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-049",
};

pub const SI_050: Subdivision = Subdivision {
    name: "Koper",
    code: "SI-050",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-050",
};

pub const SI_051: Subdivision = Subdivision {
    name: "Kozje",
    code: "SI-051",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-051",
};

pub const SI_052: Subdivision = Subdivision {
    name: "Kranj",
    code: "SI-052",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-052",
};

pub const SI_053: Subdivision = Subdivision {
    name: "Kranjska Gora",
    code: "SI-053",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-053",
};

pub const SI_054: Subdivision = Subdivision {
    name: "Krško",
    code: "SI-054",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-054",
};

pub const SI_055: Subdivision = Subdivision {
    name: "Kungota",
    code: "SI-055",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-055",
};

pub const SI_056: Subdivision = Subdivision {
    name: "Kuzma",
    code: "SI-056",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-056",
};

pub const SI_057: Subdivision = Subdivision {
    name: "Laško",
    code: "SI-057",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-057",
};

pub const SI_058: Subdivision = Subdivision {
    name: "Lenart",
    code: "SI-058",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-058",
};

pub const SI_059: Subdivision = Subdivision {
    name: "Lendava",
    code: "SI-059",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-059",
};

pub const SI_060: Subdivision = Subdivision {
    name: "Litija",
    code: "SI-060",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-060",
};

pub const SI_061: Subdivision = Subdivision {
    name: "Ljubljana",
    code: "SI-061",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-061",
};

pub const SI_062: Subdivision = Subdivision {
    name: "Ljubno",
    code: "SI-062",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-062",
};

pub const SI_063: Subdivision = Subdivision {
    name: "Ljutomer",
    code: "SI-063",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-063",
};

pub const SI_064: Subdivision = Subdivision {
    name: "Logatec",
    code: "SI-064",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-064",
};

pub const SI_065: Subdivision = Subdivision {
    name: "Loška Dolina",
    code: "SI-065",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-065",
};

pub const SI_066: Subdivision = Subdivision {
    name: "Loški Potok",
    code: "SI-066",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-066",
};

pub const SI_067: Subdivision = Subdivision {
    name: "Luče",
    code: "SI-067",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-067",
};

pub const SI_068: Subdivision = Subdivision {
    name: "Lukovica",
    code: "SI-068",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-068",
};

pub const SI_069: Subdivision = Subdivision {
    name: "Majšperk",
    code: "SI-069",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-069",
};

pub const SI_070: Subdivision = Subdivision {
    name: "Maribor",
    code: "SI-070",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-070",
};

pub const SI_071: Subdivision = Subdivision {
    name: "Medvode",
    code: "SI-071",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-071",
};

pub const SI_072: Subdivision = Subdivision {
    name: "Mengeš",
    code: "SI-072",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-072",
};

pub const SI_073: Subdivision = Subdivision {
    name: "Metlika",
    code: "SI-073",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-073",
};

pub const SI_074: Subdivision = Subdivision {
    name: "Mežica",
    code: "SI-074",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-074",
};

pub const SI_075: Subdivision = Subdivision {
    name: "Miren-Kostanjevica",
    code: "SI-075",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-075",
};

pub const SI_076: Subdivision = Subdivision {
    name: "Mislinja",
    code: "SI-076",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-076",
};

pub const SI_077: Subdivision = Subdivision {
    name: "Moravče",
    code: "SI-077",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-077",
};

pub const SI_078: Subdivision = Subdivision {
    name: "Moravske Toplice",
    code: "SI-078",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-078",
};

pub const SI_079: Subdivision = Subdivision {
    name: "Mozirje",
    code: "SI-079",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-079",
};

pub const SI_080: Subdivision = Subdivision {
    name: "Murska Sobota",
    code: "SI-080",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-080",
};

pub const SI_081: Subdivision = Subdivision {
    name: "Muta",
    code: "SI-081",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-081",
};

pub const SI_082: Subdivision = Subdivision {
    name: "Naklo",
    code: "SI-082",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-082",
};

pub const SI_083: Subdivision = Subdivision {
    name: "Nazarje",
    code: "SI-083",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-083",
};

pub const SI_084: Subdivision = Subdivision {
    name: "Nova Gorica",
    code: "SI-084",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-084",
};

pub const SI_085: Subdivision = Subdivision {
    name: "Novo Mesto",
    code: "SI-085",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-085",
};

pub const SI_086: Subdivision = Subdivision {
    name: "Odranci",
    code: "SI-086",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-086",
};

pub const SI_087: Subdivision = Subdivision {
    name: "Ormož",
    code: "SI-087",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-087",
};

pub const SI_088: Subdivision = Subdivision {
    name: "Osilnica",
    code: "SI-088",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-088",
};

pub const SI_089: Subdivision = Subdivision {
    name: "Pesnica",
    code: "SI-089",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-089",
};

pub const SI_090: Subdivision = Subdivision {
    name: "Piran",
    code: "SI-090",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-090",
};

pub const SI_091: Subdivision = Subdivision {
    name: "Pivka",
    code: "SI-091",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-091",
};

pub const SI_092: Subdivision = Subdivision {
    name: "Podčetrtek",
    code: "SI-092",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-092",
};

pub const SI_093: Subdivision = Subdivision {
    name: "Podvelka",
    code: "SI-093",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-093",
};

pub const SI_094: Subdivision = Subdivision {
    name: "Postojna",
    code: "SI-094",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-094",
};

pub const SI_095: Subdivision = Subdivision {
    name: "Preddvor",
    code: "SI-095",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-095",
};

pub const SI_096: Subdivision = Subdivision {
    name: "Ptuj",
    code: "SI-096",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-096",
};

pub const SI_097: Subdivision = Subdivision {
    name: "Puconci",
    code: "SI-097",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-097",
};

pub const SI_098: Subdivision = Subdivision {
    name: "Rače-Fram",
    code: "SI-098",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-098",
};

pub const SI_099: Subdivision = Subdivision {
    name: "Radeče",
    code: "SI-099",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-099",
};

pub const SI_100: Subdivision = Subdivision {
    name: "Radenci",
    code: "SI-100",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-100",
};

pub const SI_101: Subdivision = Subdivision {
    name: "Radlje ob Dravi",
    code: "SI-101",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-101",
};

pub const SI_102: Subdivision = Subdivision {
    name: "Radovljica",
    code: "SI-102",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-102",
};

pub const SI_103: Subdivision = Subdivision {
    name: "Ravne na Koroškem",
    code: "SI-103",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-103",
};

pub const SI_104: Subdivision = Subdivision {
    name: "Ribnica",
    code: "SI-104",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-104",
};

pub const SI_105: Subdivision = Subdivision {
    name: "Rogašovci",
    code: "SI-105",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-105",
};

pub const SI_106: Subdivision = Subdivision {
    name: "Rogaška Slatina",
    code: "SI-106",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-106",
};

pub const SI_107: Subdivision = Subdivision {
    name: "Rogatec",
    code: "SI-107",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-107",
};

pub const SI_108: Subdivision = Subdivision {
    name: "Ruše",
    code: "SI-108",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-108",
};

pub const SI_109: Subdivision = Subdivision {
    name: "Semič",
    code: "SI-109",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-109",
};

pub const SI_110: Subdivision = Subdivision {
    name: "Sevnica",
    code: "SI-110",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-110",
};

pub const SI_111: Subdivision = Subdivision {
    name: "Sežana",
    code: "SI-111",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-111",
};

pub const SI_112: Subdivision = Subdivision {
    name: "Slovenj Gradec",
    code: "SI-112",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-112",
};

pub const SI_113: Subdivision = Subdivision {
    name: "Slovenska Bistrica",
    code: "SI-113",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-113",
};

pub const SI_114: Subdivision = Subdivision {
    name: "Slovenske Konjice",
    code: "SI-114",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-114",
};

pub const SI_115: Subdivision = Subdivision {
    name: "Starše",
    code: "SI-115",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-115",
};

pub const SI_116: Subdivision = Subdivision {
    name: "Sveti Jurij",
    code: "SI-116",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-116",
};

pub const SI_117: Subdivision = Subdivision {
    name: "Šenčur",
    code: "SI-117",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-117",
};

pub const SI_118: Subdivision = Subdivision {
    name: "Šentilj",
    code: "SI-118",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-118",
};

pub const SI_119: Subdivision = Subdivision {
    name: "Šentjernej",
    code: "SI-119",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-119",
};

pub const SI_120: Subdivision = Subdivision {
    name: "Šentjur",
    code: "SI-120",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-120",
};

pub const SI_121: Subdivision = Subdivision {
    name: "Škocjan",
    code: "SI-121",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-121",
};

pub const SI_122: Subdivision = Subdivision {
    name: "Škofja Loka",
    code: "SI-122",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-122",
};

pub const SI_123: Subdivision = Subdivision {
    name: "Škofljica",
    code: "SI-123",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-123",
};

pub const SI_124: Subdivision = Subdivision {
    name: "Šmarje pri Jelšah",
    code: "SI-124",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-124",
};

pub const SI_125: Subdivision = Subdivision {
    name: "Šmartno ob Paki",
    code: "SI-125",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-125",
};

pub const SI_126: Subdivision = Subdivision {
    name: "Šoštanj",
    code: "SI-126",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-126",
};

pub const SI_127: Subdivision = Subdivision {
    name: "Štore",
    code: "SI-127",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-127",
};

pub const SI_128: Subdivision = Subdivision {
    name: "Tolmin",
    code: "SI-128",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-128",
};

pub const SI_129: Subdivision = Subdivision {
    name: "Trbovlje",
    code: "SI-129",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-129",
};

pub const SI_130: Subdivision = Subdivision {
    name: "Trebnje",
    code: "SI-130",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-130",
};

pub const SI_131: Subdivision = Subdivision {
    name: "Tržič",
    code: "SI-131",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-131",
};

pub const SI_132: Subdivision = Subdivision {
    name: "Turnišče",
    code: "SI-132",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-132",
};

pub const SI_133: Subdivision = Subdivision {
    name: "Velenje",
    code: "SI-133",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-133",
};

pub const SI_134: Subdivision = Subdivision {
    name: "Velike Lašče",
    code: "SI-134",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-134",
};

pub const SI_135: Subdivision = Subdivision {
    name: "Videm",
    code: "SI-135",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-135",
};

pub const SI_136: Subdivision = Subdivision {
    name: "Vipava",
    code: "SI-136",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-136",
};

pub const SI_137: Subdivision = Subdivision {
    name: "Vitanje",
    code: "SI-137",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-137",
};

pub const SI_138: Subdivision = Subdivision {
    name: "Vodice",
    code: "SI-138",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-138",
};

pub const SI_139: Subdivision = Subdivision {
    name: "Vojnik",
    code: "SI-139",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-139",
};

pub const SI_140: Subdivision = Subdivision {
    name: "Vrhnika",
    code: "SI-140",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-140",
};

pub const SI_141: Subdivision = Subdivision {
    name: "Vuzenica",
    code: "SI-141",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-141",
};

pub const SI_142: Subdivision = Subdivision {
    name: "Zagorje ob Savi",
    code: "SI-142",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-142",
};

pub const SI_143: Subdivision = Subdivision {
    name: "Zavrč",
    code: "SI-143",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-143",
};

pub const SI_144: Subdivision = Subdivision {
    name: "Zreče",
    code: "SI-144",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-144",
};

pub const SI_146: Subdivision = Subdivision {
    name: "Železniki",
    code: "SI-146",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-146",
};

pub const SI_147: Subdivision = Subdivision {
    name: "Žiri",
    code: "SI-147",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-147",
};

pub const SI_148: Subdivision = Subdivision {
    name: "Benedikt",
    code: "SI-148",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-148",
};

pub const SI_149: Subdivision = Subdivision {
    name: "Bistrica ob Sotli",
    code: "SI-149",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-149",
};

pub const SI_150: Subdivision = Subdivision {
    name: "Bloke",
    code: "SI-150",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-150",
};

pub const SI_151: Subdivision = Subdivision {
    name: "Braslovče",
    code: "SI-151",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-151",
};

pub const SI_152: Subdivision = Subdivision {
    name: "Cankova",
    code: "SI-152",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-152",
};

pub const SI_153: Subdivision = Subdivision {
    name: "Cerkvenjak",
    code: "SI-153",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-153",
};

pub const SI_154: Subdivision = Subdivision {
    name: "Dobje",
    code: "SI-154",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-154",
};

pub const SI_155: Subdivision = Subdivision {
    name: "Dobrna",
    code: "SI-155",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-155",
};

pub const SI_156: Subdivision = Subdivision {
    name: "Dobrovnik",
    code: "SI-156",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-156",
};

pub const SI_157: Subdivision = Subdivision {
    name: "Dolenjske Toplice",
    code: "SI-157",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-157",
};

pub const SI_158: Subdivision = Subdivision {
    name: "Grad",
    code: "SI-158",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-158",
};

pub const SI_159: Subdivision = Subdivision {
    name: "Hajdina",
    code: "SI-159",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-159",
};

pub const SI_160: Subdivision = Subdivision {
    name: "Hoče-Slivnica",
    code: "SI-160",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-160",
};

pub const SI_161: Subdivision = Subdivision {
    name: "Hodoš",
    code: "SI-161",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-161",
};

pub const SI_162: Subdivision = Subdivision {
    name: "Horjul",
    code: "SI-162",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-162",
};

pub const SI_163: Subdivision = Subdivision {
    name: "Jezersko",
    code: "SI-163",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-163",
};

pub const SI_164: Subdivision = Subdivision {
    name: "Komenda",
    code: "SI-164",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-164",
};

pub const SI_165: Subdivision = Subdivision {
    name: "Kostel",
    code: "SI-165",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-165",
};

pub const SI_166: Subdivision = Subdivision {
    name: "Križevci",
    code: "SI-166",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-166",
};

pub const SI_167: Subdivision = Subdivision {
    name: "Lovrenc na Pohorju",
    code: "SI-167",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-167",
};

pub const SI_168: Subdivision = Subdivision {
    name: "Markovci",
    code: "SI-168",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-168",
};

pub const SI_169: Subdivision = Subdivision {
    name: "Miklavž na Dravskem Polju",
    code: "SI-169",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-169",
};

pub const SI_170: Subdivision = Subdivision {
    name: "Mirna Peč",
    code: "SI-170",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-170",
};

pub const SI_171: Subdivision = Subdivision {
    name: "Oplotnica",
    code: "SI-171",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-171",
};

pub const SI_172: Subdivision = Subdivision {
    name: "Podlehnik",
    code: "SI-172",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-172",
};

pub const SI_173: Subdivision = Subdivision {
    name: "Polzela",
    code: "SI-173",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-173",
};

pub const SI_174: Subdivision = Subdivision {
    name: "Prebold",
    code: "SI-174",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-174",
};

pub const SI_175: Subdivision = Subdivision {
    name: "Prevalje",
    code: "SI-175",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-175",
};

pub const SI_176: Subdivision = Subdivision {
    name: "Razkrižje",
    code: "SI-176",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-176",
};

pub const SI_177: Subdivision = Subdivision {
    name: "Ribnica na Pohorju",
    code: "SI-177",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-177",
};

pub const SI_178: Subdivision = Subdivision {
    name: "Selnica ob Dravi",
    code: "SI-178",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-178",
};

pub const SI_179: Subdivision = Subdivision {
    name: "Sodražica",
    code: "SI-179",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-179",
};

pub const SI_180: Subdivision = Subdivision {
    name: "Solčava",
    code: "SI-180",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-180",
};

pub const SI_181: Subdivision = Subdivision {
    name: "Sveta Ana",
    code: "SI-181",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-181",
};

pub const SI_182: Subdivision = Subdivision {
    name: "Sveti Andraž v Slovenskih Goricah",
    code: "SI-182",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-182",
};

pub const SI_183: Subdivision = Subdivision {
    name: "Šempeter-Vrtojba",
    code: "SI-183",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-183",
};

pub const SI_184: Subdivision = Subdivision {
    name: "Tabor",
    code: "SI-184",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-184",
};

pub const SI_185: Subdivision = Subdivision {
    name: "Trnovska Vas",
    code: "SI-185",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-185",
};

pub const SI_186: Subdivision = Subdivision {
    name: "Trzin",
    code: "SI-186",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-186",
};

pub const SI_187: Subdivision = Subdivision {
    name: "Velika Polana",
    code: "SI-187",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-187",
};

pub const SI_188: Subdivision = Subdivision {
    name: "Veržej",
    code: "SI-188",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-188",
};

pub const SI_189: Subdivision = Subdivision {
    name: "Vransko",
    code: "SI-189",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-189",
};

pub const SI_190: Subdivision = Subdivision {
    name: "Žalec",
    code: "SI-190",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-190",
};

pub const SI_191: Subdivision = Subdivision {
    name: "Žetale",
    code: "SI-191",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-191",
};

pub const SI_192: Subdivision = Subdivision {
    name: "Žirovnica",
    code: "SI-192",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-192",
};

pub const SI_193: Subdivision = Subdivision {
    name: "Žužemberk",
    code: "SI-193",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-193",
};

pub const SI_194: Subdivision = Subdivision {
    name: "Šmartno pri Litiji",
    code: "SI-194",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-194",
};

pub const SI_195: Subdivision = Subdivision {
    name: "Apače",
    code: "SI-195",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-195",
};

pub const SI_196: Subdivision = Subdivision {
    name: "Cirkulane",
    code: "SI-196",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-196",
};

pub const SI_197: Subdivision = Subdivision {
    name: "Kosanjevica na Krki",
    code: "SI-197",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-197",
};

pub const SI_198: Subdivision = Subdivision {
    name: "Makole",
    code: "SI-198",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-198",
};

pub const SI_199: Subdivision = Subdivision {
    name: "Mokronog-Trebelno",
    code: "SI-199",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-199",
};

pub const SI_200: Subdivision = Subdivision {
    name: "Poljčane",
    code: "SI-200",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-200",
};

pub const SI_201: Subdivision = Subdivision {
    name: "Renče-Vogrsko",
    code: "SI-201",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-201",
};

pub const SI_202: Subdivision = Subdivision {
    name: "Središče ob Dravi",
    code: "SI-202",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-202",
};

pub const SI_203: Subdivision = Subdivision {
    name: "Straža",
    code: "SI-203",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-203",
};

pub const SI_204: Subdivision = Subdivision {
    name: "Sveta Trojica v Slovenskih Goricah",
    code: "SI-204",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-204",
};

pub const SI_205: Subdivision = Subdivision {
    name: "Sveti Tomaž",
    code: "SI-205",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-205",
};

pub const SI_206: Subdivision = Subdivision {
    name: "Šmarješke Toplice",
    code: "SI-206",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-206",
};

pub const SI_207: Subdivision = Subdivision {
    name: "Gorje",
    code: "SI-207",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-207",
};

pub const SI_208: Subdivision = Subdivision {
    name: "Log-Dragomer",
    code: "SI-208",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-208",
};

pub const SI_209: Subdivision = Subdivision {
    name: "Rečica ob Savinji",
    code: "SI-209",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-209",
};

pub const SI_210: Subdivision = Subdivision {
    name: "Sveti Jurij v Slovenskih Goricah",
    code: "SI-210",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-210",
};

pub const SI_211: Subdivision = Subdivision {
    name: "Šentrupert",
    code: "SI-211",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-211",
};

pub const SI_212: Subdivision = Subdivision {
    name: "Mirna",
    code: "SI-212",
    subdivision_type: "Commune",
    country_name: "Slovenia",
    country_code: "SI",
    region_code: "SI-212",
};

pub const SB_CE: Subdivision = Subdivision {
    name: "Central",
    code: "SB-CE",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-CE",
};

pub const SB_CH: Subdivision = Subdivision {
    name: "Choiseul",
    code: "SB-CH",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-CH",
};

pub const SB_CT: Subdivision = Subdivision {
    name: "Capital Territory",
    code: "SB-CT",
    subdivision_type: "Capital territory",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-CT",
};

pub const SB_GU: Subdivision = Subdivision {
    name: "Guadalcanal",
    code: "SB-GU",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-GU",
};

pub const SB_IS: Subdivision = Subdivision {
    name: "Isabel",
    code: "SB-IS",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-IS",
};

pub const SB_MK: Subdivision = Subdivision {
    name: "Makira-Ulawa",
    code: "SB-MK",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-MK",
};

pub const SB_ML: Subdivision = Subdivision {
    name: "Malaita",
    code: "SB-ML",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-ML",
};

pub const SB_RB: Subdivision = Subdivision {
    name: "Rennell and Bellona",
    code: "SB-RB",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-RB",
};

pub const SB_TE: Subdivision = Subdivision {
    name: "Temotu",
    code: "SB-TE",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-TE",
};

pub const SB_WE: Subdivision = Subdivision {
    name: "Western",
    code: "SB-WE",
    subdivision_type: "Province",
    country_name: "Solomon Islands",
    country_code: "SB",
    region_code: "SB-WE",
};

pub const SO_AW: Subdivision = Subdivision {
    name: "Awdal",
    code: "SO-AW",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-AW",
};

pub const SO_BK: Subdivision = Subdivision {
    name: "Bakool",
    code: "SO-BK",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-BK",
};

pub const SO_BN: Subdivision = Subdivision {
    name: "Banaadir",
    code: "SO-BN",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-BN",
};

pub const SO_BR: Subdivision = Subdivision {
    name: "Bari",
    code: "SO-BR",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-BR",
};

pub const SO_BY: Subdivision = Subdivision {
    name: "Bay",
    code: "SO-BY",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-BY",
};

pub const SO_GA: Subdivision = Subdivision {
    name: "Galguduud",
    code: "SO-GA",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-GA",
};

pub const SO_GE: Subdivision = Subdivision {
    name: "Gedo",
    code: "SO-GE",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-GE",
};

pub const SO_HI: Subdivision = Subdivision {
    name: "Hiiraan",
    code: "SO-HI",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-HI",
};

pub const SO_JD: Subdivision = Subdivision {
    name: "Jubbada Dhexe",
    code: "SO-JD",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-JD",
};

pub const SO_JH: Subdivision = Subdivision {
    name: "Jubbada Hoose",
    code: "SO-JH",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-JH",
};

pub const SO_MU: Subdivision = Subdivision {
    name: "Mudug",
    code: "SO-MU",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-MU",
};

pub const SO_NU: Subdivision = Subdivision {
    name: "Nugaal",
    code: "SO-NU",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-NU",
};

pub const SO_SA: Subdivision = Subdivision {
    name: "Sanaag",
    code: "SO-SA",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-SA",
};

pub const SO_SD: Subdivision = Subdivision {
    name: "Shabeellaha Dhexe",
    code: "SO-SD",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-SD",
};

pub const SO_SH: Subdivision = Subdivision {
    name: "Shabeellaha Hoose",
    code: "SO-SH",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-SH",
};

pub const SO_SO: Subdivision = Subdivision {
    name: "Sool",
    code: "SO-SO",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-SO",
};

pub const SO_TO: Subdivision = Subdivision {
    name: "Togdheer",
    code: "SO-TO",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-TO",
};

pub const SO_WO: Subdivision = Subdivision {
    name: "Woqooyi Galbeed",
    code: "SO-WO",
    subdivision_type: "Region",
    country_name: "Somalia",
    country_code: "SO",
    region_code: "SO-WO",
};

pub const ZA_EC: Subdivision = Subdivision {
    name: "Eastern Cape",
    code: "ZA-EC",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-EC",
};

pub const ZA_FS: Subdivision = Subdivision {
    name: "Foreisetata",
    code: "ZA-FS",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-FS",
};

pub const ZA_GT: Subdivision = Subdivision {
    name: "Gauteng",
    code: "ZA-GT",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-GT",
};

pub const ZA_LP: Subdivision = Subdivision {
    name: "Limpopo",
    code: "ZA-LP",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-LP",
};

pub const ZA_MP: Subdivision = Subdivision {
    name: "iMpumalanga",
    code: "ZA-MP",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-MP",
};

pub const ZA_NC: Subdivision = Subdivision {
    name: "iTlhagwini-Kapa",
    code: "ZA-NC",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-NC",
};

pub const ZA_NL: Subdivision = Subdivision {
    name: "GaZulu-Natala",
    code: "ZA-NL",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-NL",
};

pub const ZA_NW: Subdivision = Subdivision {
    name: "Bokone Bophirima",
    code: "ZA-NW",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-NW",
};

pub const ZA_WC: Subdivision = Subdivision {
    name: "iTjhingalanga-Kapa",
    code: "ZA-WC",
    subdivision_type: "Province",
    country_name: "South Africa",
    country_code: "ZA",
    region_code: "ZA-WC",
};

pub const SS_BN: Subdivision = Subdivision {
    name: "Northern Bahr el Ghazal",
    code: "SS-BN",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-BN",
};

pub const SS_BW: Subdivision = Subdivision {
    name: "Western Bahr el Ghazal",
    code: "SS-BW",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-BW",
};

pub const SS_EC: Subdivision = Subdivision {
    name: "Central Equatoria",
    code: "SS-EC",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-EC",
};

pub const SS_EE: Subdivision = Subdivision {
    name: "Eastern Equatoria",
    code: "SS-EE",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-EE",
};

pub const SS_EW: Subdivision = Subdivision {
    name: "Western Equatoria",
    code: "SS-EW",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-EW",
};

pub const SS_JG: Subdivision = Subdivision {
    name: "Jonglei",
    code: "SS-JG",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-JG",
};

pub const SS_LK: Subdivision = Subdivision {
    name: "Lakes",
    code: "SS-LK",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-LK",
};

pub const SS_NU: Subdivision = Subdivision {
    name: "Upper Nile",
    code: "SS-NU",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-NU",
};

pub const SS_UY: Subdivision = Subdivision {
    name: "Unity",
    code: "SS-UY",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-UY",
};

pub const SS_WR: Subdivision = Subdivision {
    name: "Warrap",
    code: "SS-WR",
    subdivision_type: "State",
    country_name: "South Sudan",
    country_code: "SS",
    region_code: "SS-WR",
};

pub const ES_A: Subdivision = Subdivision {
    name: "Alacant*",
    code: "ES-A",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-A",
};

pub const ES_AB: Subdivision = Subdivision {
    name: "Albacete",
    code: "ES-AB",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-AB",
};

pub const ES_AL: Subdivision = Subdivision {
    name: "Almería",
    code: "ES-AL",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-AL",
};

pub const ES_AN: Subdivision = Subdivision {
    name: "Andalucía",
    code: "ES-AN",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-AN",
};

pub const ES_AR: Subdivision = Subdivision {
    name: "Aragón",
    code: "ES-AR",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-AR",
};

pub const ES_AS: Subdivision = Subdivision {
    name: "Asturias, Principado de",
    code: "ES-AS",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-AS",
};

pub const ES_AV: Subdivision = Subdivision {
    name: "Ávila",
    code: "ES-AV",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-AV",
};

pub const ES_BA: Subdivision = Subdivision {
    name: "Badajoz",
    code: "ES-BA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-BA",
};

pub const ES_B: Subdivision = Subdivision {
    name: "Barcelona",
    code: "ES-B",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-B",
};

pub const ES_BI: Subdivision = Subdivision {
    name: "Bizkaia*",
    code: "ES-BI",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-BI",
};

pub const ES_BU: Subdivision = Subdivision {
    name: "Burgos",
    code: "ES-BU",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-BU",
};

pub const ES_CA: Subdivision = Subdivision {
    name: "Cádiz",
    code: "ES-CA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CA",
};

pub const ES_C: Subdivision = Subdivision {
    name: "A Coruña",
    code: "ES-C",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-C",
};

pub const ES_CB: Subdivision = Subdivision {
    name: "Cantabria",
    code: "ES-CB",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CB",
};

pub const ES_CC: Subdivision = Subdivision {
    name: "Cáceres",
    code: "ES-CC",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CC",
};

pub const ES_CE: Subdivision = Subdivision {
    name: "Ceuta",
    code: "ES-CE",
    subdivision_type: "Autonomous city in north africa",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CE",
};

pub const ES_CL: Subdivision = Subdivision {
    name: "Castilla y León",
    code: "ES-CL",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CL",
};

pub const ES_CM: Subdivision = Subdivision {
    name: "Castilla-La Mancha",
    code: "ES-CM",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CM",
};

pub const ES_CN: Subdivision = Subdivision {
    name: "Canarias",
    code: "ES-CN",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CN",
};

pub const ES_CO: Subdivision = Subdivision {
    name: "Córdoba",
    code: "ES-CO",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CO",
};

pub const ES_CR: Subdivision = Subdivision {
    name: "Ciudad Real",
    code: "ES-CR",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CR",
};

pub const ES_CS: Subdivision = Subdivision {
    name: "Castelló*",
    code: "ES-CS",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CS",
};

pub const ES_CT: Subdivision = Subdivision {
    name: "Catalunya",
    code: "ES-CT",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CT",
};

pub const ES_CU: Subdivision = Subdivision {
    name: "Cuenca",
    code: "ES-CU",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-CU",
};

pub const ES_EX: Subdivision = Subdivision {
    name: "Extremadura",
    code: "ES-EX",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-EX",
};

pub const ES_GA: Subdivision = Subdivision {
    name: "Galicia",
    code: "ES-GA",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-GA",
};

pub const ES_GC: Subdivision = Subdivision {
    name: "Las Palmas",
    code: "ES-GC",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-GC",
};

pub const ES_GI: Subdivision = Subdivision {
    name: "Girona",
    code: "ES-GI",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-GI",
};

pub const ES_GR: Subdivision = Subdivision {
    name: "Granada",
    code: "ES-GR",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-GR",
};

pub const ES_GU: Subdivision = Subdivision {
    name: "Guadalajara",
    code: "ES-GU",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-GU",
};

pub const ES_H: Subdivision = Subdivision {
    name: "Huelva",
    code: "ES-H",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-H",
};

pub const ES_HU: Subdivision = Subdivision {
    name: "Huesca",
    code: "ES-HU",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-HU",
};

pub const ES_IB: Subdivision = Subdivision {
    name: "Illes Balears",
    code: "ES-IB",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-IB",
};

pub const ES_J: Subdivision = Subdivision {
    name: "Jaén",
    code: "ES-J",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-J",
};

pub const ES_LE: Subdivision = Subdivision {
    name: "León",
    code: "ES-LE",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-LE",
};

pub const ES_L: Subdivision = Subdivision {
    name: "Lleida",
    code: "ES-L",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-L",
};

pub const ES_LO: Subdivision = Subdivision {
    name: "La Rioja",
    code: "ES-LO",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-LO",
};

pub const ES_LU: Subdivision = Subdivision {
    name: "Lugo",
    code: "ES-LU",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-LU",
};

pub const ES_MA: Subdivision = Subdivision {
    name: "Málaga",
    code: "ES-MA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-MA",
};

pub const ES_MC: Subdivision = Subdivision {
    name: "Murcia, Región de",
    code: "ES-MC",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-MC",
};

pub const ES_MD: Subdivision = Subdivision {
    name: "Madrid, Comunidad de",
    code: "ES-MD",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-MD",
};

pub const ES_ML: Subdivision = Subdivision {
    name: "Melilla",
    code: "ES-ML",
    subdivision_type: "Autonomous city in north africa",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-ML",
};

pub const ES_M: Subdivision = Subdivision {
    name: "Madrid",
    code: "ES-M",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-M",
};

pub const ES_MU: Subdivision = Subdivision {
    name: "Murcia",
    code: "ES-MU",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-MU",
};

pub const ES_NA: Subdivision = Subdivision {
    name: "Nafarroa*",
    code: "ES-NA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-NA",
};

pub const ES_NC: Subdivision = Subdivision {
    name: "Nafarroako Foru Komunitatea*",
    code: "ES-NC",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-NC",
};

pub const ES_O: Subdivision = Subdivision {
    name: "Asturias",
    code: "ES-O",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-O",
};

pub const ES_OR: Subdivision = Subdivision {
    name: "Ourense",
    code: "ES-OR",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-OR",
};

pub const ES_PM: Subdivision = Subdivision {
    name: "Balears",
    code: "ES-PM",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-PM",
};

pub const ES_PO: Subdivision = Subdivision {
    name: "Pontevedra",
    code: "ES-PO",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-PO",
};

pub const ES_P: Subdivision = Subdivision {
    name: "Palencia",
    code: "ES-P",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-P",
};

pub const ES_PV: Subdivision = Subdivision {
    name: "Euskal Herria",
    code: "ES-PV",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-PV",
};

pub const ES_RI: Subdivision = Subdivision {
    name: "La Rioja",
    code: "ES-RI",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-RI",
};

pub const ES_SA: Subdivision = Subdivision {
    name: "Salamanca",
    code: "ES-SA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-SA",
};

pub const ES_S: Subdivision = Subdivision {
    name: "Cantabria",
    code: "ES-S",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-S",
};

pub const ES_SE: Subdivision = Subdivision {
    name: "Sevilla",
    code: "ES-SE",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-SE",
};

pub const ES_SG: Subdivision = Subdivision {
    name: "Segovia",
    code: "ES-SG",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-SG",
};

pub const ES_SO: Subdivision = Subdivision {
    name: "Soria",
    code: "ES-SO",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-SO",
};

pub const ES_SS: Subdivision = Subdivision {
    name: "Gipuzkoa*",
    code: "ES-SS",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-SS",
};

pub const ES_TE: Subdivision = Subdivision {
    name: "Teruel",
    code: "ES-TE",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-TE",
};

pub const ES_TF: Subdivision = Subdivision {
    name: "Santa Cruz de Tenerife",
    code: "ES-TF",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-TF",
};

pub const ES_TO: Subdivision = Subdivision {
    name: "Toledo",
    code: "ES-TO",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-TO",
};

pub const ES_T: Subdivision = Subdivision {
    name: "Tarragona",
    code: "ES-T",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-T",
};

pub const ES_VA: Subdivision = Subdivision {
    name: "Valladolid",
    code: "ES-VA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-VA",
};

pub const ES_VC: Subdivision = Subdivision {
    name: "Valenciana, Comunidad",
    code: "ES-VC",
    subdivision_type: "Autonomous community",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-VC",
};

pub const ES_VI: Subdivision = Subdivision {
    name: "Álava",
    code: "ES-VI",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-VI",
};

pub const ES_V: Subdivision = Subdivision {
    name: "Valencia",
    code: "ES-V",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-V",
};

pub const ES_ZA: Subdivision = Subdivision {
    name: "Zamora",
    code: "ES-ZA",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-ZA",
};

pub const ES_Z: Subdivision = Subdivision {
    name: "Zaragoza",
    code: "ES-Z",
    subdivision_type: "Province",
    country_name: "Spain",
    country_code: "ES",
    region_code: "ES-Z",
};

pub const LK_11: Subdivision = Subdivision {
    name: "Colombo",
    code: "LK-11",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-11",
};

pub const LK_12: Subdivision = Subdivision {
    name: "Gampaha",
    code: "LK-12",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-12",
};

pub const LK_13: Subdivision = Subdivision {
    name: "Kalutara",
    code: "LK-13",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-13",
};

pub const LK_1: Subdivision = Subdivision {
    name: "Basnāhira paḷāta",
    code: "LK-1",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-1",
};

pub const LK_21: Subdivision = Subdivision {
    name: "Kandy",
    code: "LK-21",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-21",
};

pub const LK_22: Subdivision = Subdivision {
    name: "Matale",
    code: "LK-22",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-22",
};

pub const LK_23: Subdivision = Subdivision {
    name: "Nuvara Ĕliya",
    code: "LK-23",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-23",
};

pub const LK_2: Subdivision = Subdivision {
    name: "Central Province",
    code: "LK-2",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-2",
};

pub const LK_31: Subdivision = Subdivision {
    name: "Gālla",
    code: "LK-31",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-31",
};

pub const LK_32: Subdivision = Subdivision {
    name: "Matara",
    code: "LK-32",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-32",
};

pub const LK_33: Subdivision = Subdivision {
    name: "Ampāntōṭṭai",
    code: "LK-33",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-33",
};

pub const LK_3: Subdivision = Subdivision {
    name: "Dakuṇu paḷāta",
    code: "LK-3",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-3",
};

pub const LK_41: Subdivision = Subdivision {
    name: "Jaffna",
    code: "LK-41",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-41",
};

pub const LK_42: Subdivision = Subdivision {
    name: "Kilinochchi",
    code: "LK-42",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-42",
};

pub const LK_43: Subdivision = Subdivision {
    name: "Mannar",
    code: "LK-43",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-43",
};

pub const LK_44: Subdivision = Subdivision {
    name: "Vavuniya",
    code: "LK-44",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-44",
};

pub const LK_45: Subdivision = Subdivision {
    name: "Mulativ",
    code: "LK-45",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-45",
};

pub const LK_4: Subdivision = Subdivision {
    name: "Northern Province",
    code: "LK-4",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-4",
};

pub const LK_51: Subdivision = Subdivision {
    name: "Batticaloa",
    code: "LK-51",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-51",
};

pub const LK_52: Subdivision = Subdivision {
    name: "Ampara",
    code: "LK-52",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-52",
};

pub const LK_53: Subdivision = Subdivision {
    name: "Tirukŏṇamalai",
    code: "LK-53",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-53",
};

pub const LK_5: Subdivision = Subdivision {
    name: "Eastern Province",
    code: "LK-5",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-5",
};

pub const LK_61: Subdivision = Subdivision {
    name: "Kuruṇægala",
    code: "LK-61",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-61",
};

pub const LK_62: Subdivision = Subdivision {
    name: "Puttalam",
    code: "LK-62",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-62",
};

pub const LK_6: Subdivision = Subdivision {
    name: "North Western Province",
    code: "LK-6",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-6",
};

pub const LK_71: Subdivision = Subdivision {
    name: "Anuradhapura",
    code: "LK-71",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-71",
};

pub const LK_72: Subdivision = Subdivision {
    name: "Pŏlaṉṉaṛuvai",
    code: "LK-72",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-72",
};

pub const LK_7: Subdivision = Subdivision {
    name: "North Central Province",
    code: "LK-7",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-7",
};

pub const LK_81: Subdivision = Subdivision {
    name: "Badulla",
    code: "LK-81",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-81",
};

pub const LK_82: Subdivision = Subdivision {
    name: "Monaragala",
    code: "LK-82",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-82",
};

pub const LK_8: Subdivision = Subdivision {
    name: "Ūvā mākāṇam",
    code: "LK-8",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-8",
};

pub const LK_91: Subdivision = Subdivision {
    name: "Irattiṉapuri",
    code: "LK-91",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-91",
};

pub const LK_92: Subdivision = Subdivision {
    name: "Kægalla",
    code: "LK-92",
    subdivision_type: "District",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-92",
};

pub const LK_9: Subdivision = Subdivision {
    name: "Chappirakamuva mākāṇam",
    code: "LK-9",
    subdivision_type: "Province",
    country_name: "Sri Lanka",
    country_code: "LK",
    region_code: "LK-9",
};

pub const BL_BL: Subdivision = Subdivision {
    name: "St. Barthélemy",
    code: "BL-BL",
    subdivision_type: "Country",
    country_name: "St. Barthélemy",
    country_code: "BL",
    region_code: "BL-BL",
};

pub const SX_SX: Subdivision = Subdivision {
    name: "St. Maarten",
    code: "SX-SX",
    subdivision_type: "Country",
    country_name: "St. Maarten",
    country_code: "SX",
    region_code: "SX-SX",
};

pub const MF_MF: Subdivision = Subdivision {
    name: "St. Martin",
    code: "MF-MF",
    subdivision_type: "Country",
    country_name: "St. Martin",
    country_code: "MF",
    region_code: "MF-MF",
};

pub const PM_PM: Subdivision = Subdivision {
    name: "St. Pierre & Miquelon",
    code: "PM-PM",
    subdivision_type: "Country",
    country_name: "St. Pierre & Miquelon",
    country_code: "PM",
    region_code: "PM-PM",
};

pub const SD_DC: Subdivision = Subdivision {
    name: "Central Darfur",
    code: "SD-DC",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-DC",
};

pub const SD_DE: Subdivision = Subdivision {
    name: "East Darfur",
    code: "SD-DE",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-DE",
};

pub const SD_DN: Subdivision = Subdivision {
    name: "North Darfur",
    code: "SD-DN",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-DN",
};

pub const SD_DS: Subdivision = Subdivision {
    name: "Janūb Dārfūr",
    code: "SD-DS",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-DS",
};

pub const SD_DW: Subdivision = Subdivision {
    name: "Gharb Dārfūr",
    code: "SD-DW",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-DW",
};

pub const SD_GD: Subdivision = Subdivision {
    name: "Al Qaḑārif",
    code: "SD-GD",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-GD",
};

pub const SD_GZ: Subdivision = Subdivision {
    name: "Al Jazīrah",
    code: "SD-GZ",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-GZ",
};

pub const SD_KA: Subdivision = Subdivision {
    name: "Kassala",
    code: "SD-KA",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-KA",
};

pub const SD_KH: Subdivision = Subdivision {
    name: "Al Kharţūm",
    code: "SD-KH",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-KH",
};

pub const SD_KN: Subdivision = Subdivision {
    name: "North Kordofan",
    code: "SD-KN",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-KN",
};

pub const SD_KS: Subdivision = Subdivision {
    name: "Janūb Kurdufān",
    code: "SD-KS",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-KS",
};

pub const SD_NB: Subdivision = Subdivision {
    name: "An Nīl al Azraq",
    code: "SD-NB",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-NB",
};

pub const SD_NO: Subdivision = Subdivision {
    name: "Ash Shamālīyah",
    code: "SD-NO",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-NO",
};

pub const SD_NR: Subdivision = Subdivision {
    name: "An Nīl",
    code: "SD-NR",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-NR",
};

pub const SD_NW: Subdivision = Subdivision {
    name: "An Nīl al Abyaḑ",
    code: "SD-NW",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-NW",
};

pub const SD_RS: Subdivision = Subdivision {
    name: "Al Baḩr al Aḩmar",
    code: "SD-RS",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-RS",
};

pub const SD_SI: Subdivision = Subdivision {
    name: "Sennar",
    code: "SD-SI",
    subdivision_type: "State",
    country_name: "Sudan",
    country_code: "SD",
    region_code: "SD-SI",
};

pub const SR_BR: Subdivision = Subdivision {
    name: "Brokopondo",
    code: "SR-BR",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-BR",
};

pub const SR_CM: Subdivision = Subdivision {
    name: "Commewijne",
    code: "SR-CM",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-CM",
};

pub const SR_CR: Subdivision = Subdivision {
    name: "Coronie",
    code: "SR-CR",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-CR",
};

pub const SR_MA: Subdivision = Subdivision {
    name: "Marowijne",
    code: "SR-MA",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-MA",
};

pub const SR_NI: Subdivision = Subdivision {
    name: "Nickerie",
    code: "SR-NI",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-NI",
};

pub const SR_PM: Subdivision = Subdivision {
    name: "Paramaribo",
    code: "SR-PM",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-PM",
};

pub const SR_PR: Subdivision = Subdivision {
    name: "Para",
    code: "SR-PR",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-PR",
};

pub const SR_SA: Subdivision = Subdivision {
    name: "Saramacca",
    code: "SR-SA",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-SA",
};

pub const SR_SI: Subdivision = Subdivision {
    name: "Sipaliwini",
    code: "SR-SI",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-SI",
};

pub const SR_WA: Subdivision = Subdivision {
    name: "Wanica",
    code: "SR-WA",
    subdivision_type: "District",
    country_name: "Suriname",
    country_code: "SR",
    region_code: "SR-WA",
};

pub const SJ_SJ: Subdivision = Subdivision {
    name: "Svalbard & Jan Mayen Islands",
    code: "SJ-SJ",
    subdivision_type: "Country",
    country_name: "Svalbard & Jan Mayen Islands",
    country_code: "SJ",
    region_code: "SJ-SJ",
};

pub const SZ_HH: Subdivision = Subdivision {
    name: "Hhohho",
    code: "SZ-HH",
    subdivision_type: "District",
    country_name: "Swaziland",
    country_code: "SZ",
    region_code: "SZ-HH",
};

pub const SZ_LU: Subdivision = Subdivision {
    name: "Lubombo",
    code: "SZ-LU",
    subdivision_type: "District",
    country_name: "Swaziland",
    country_code: "SZ",
    region_code: "SZ-LU",
};

pub const SZ_MA: Subdivision = Subdivision {
    name: "Manzini",
    code: "SZ-MA",
    subdivision_type: "District",
    country_name: "Swaziland",
    country_code: "SZ",
    region_code: "SZ-MA",
};

pub const SZ_SH: Subdivision = Subdivision {
    name: "Shiselweni",
    code: "SZ-SH",
    subdivision_type: "District",
    country_name: "Swaziland",
    country_code: "SZ",
    region_code: "SZ-SH",
};

pub const SE_AB: Subdivision = Subdivision {
    name: "Stockholms län",
    code: "SE-AB",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-AB",
};

pub const SE_AC: Subdivision = Subdivision {
    name: "Västerbottens län",
    code: "SE-AC",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-AC",
};

pub const SE_BD: Subdivision = Subdivision {
    name: "Norrbottens län",
    code: "SE-BD",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-BD",
};

pub const SE_C: Subdivision = Subdivision {
    name: "Uppsala län",
    code: "SE-C",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-C",
};

pub const SE_D: Subdivision = Subdivision {
    name: "Södermanlands län",
    code: "SE-D",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-D",
};

pub const SE_E: Subdivision = Subdivision {
    name: "Östergötlands län",
    code: "SE-E",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-E",
};

pub const SE_F: Subdivision = Subdivision {
    name: "Jönköpings län",
    code: "SE-F",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-F",
};

pub const SE_G: Subdivision = Subdivision {
    name: "Kronobergs län",
    code: "SE-G",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-G",
};

pub const SE_H: Subdivision = Subdivision {
    name: "Kalmar län",
    code: "SE-H",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-H",
};

pub const SE_I: Subdivision = Subdivision {
    name: "Gotlands län",
    code: "SE-I",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-I",
};

pub const SE_K: Subdivision = Subdivision {
    name: "Blekinge län",
    code: "SE-K",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-K",
};

pub const SE_M: Subdivision = Subdivision {
    name: "Skåne län",
    code: "SE-M",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-M",
};

pub const SE_N: Subdivision = Subdivision {
    name: "Hallands län",
    code: "SE-N",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-N",
};

pub const SE_O: Subdivision = Subdivision {
    name: "Västra Götalands län",
    code: "SE-O",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-O",
};

pub const SE_S: Subdivision = Subdivision {
    name: "Värmlands län",
    code: "SE-S",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-S",
};

pub const SE_T: Subdivision = Subdivision {
    name: "Örebro län",
    code: "SE-T",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-T",
};

pub const SE_U: Subdivision = Subdivision {
    name: "Västmanlands län",
    code: "SE-U",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-U",
};

pub const SE_W: Subdivision = Subdivision {
    name: "Dalarnas län",
    code: "SE-W",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-W",
};

pub const SE_X: Subdivision = Subdivision {
    name: "Gävleborgs län",
    code: "SE-X",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-X",
};

pub const SE_Y: Subdivision = Subdivision {
    name: "Västernorrlands län",
    code: "SE-Y",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-Y",
};

pub const SE_Z: Subdivision = Subdivision {
    name: "Jämtlands län",
    code: "SE-Z",
    subdivision_type: "County",
    country_name: "Sweden",
    country_code: "SE",
    region_code: "SE-Z",
};

pub const CH_AG: Subdivision = Subdivision {
    name: "Aargau",
    code: "CH-AG",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-AG",
};

pub const CH_AI: Subdivision = Subdivision {
    name: "Appenzell Innerrhoden",
    code: "CH-AI",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-AI",
};

pub const CH_AR: Subdivision = Subdivision {
    name: "Appenzell Ausserrhoden",
    code: "CH-AR",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-AR",
};

pub const CH_BE: Subdivision = Subdivision {
    name: "Bern",
    code: "CH-BE",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-BE",
};

pub const CH_BL: Subdivision = Subdivision {
    name: "Basel-Landschaft",
    code: "CH-BL",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-BL",
};

pub const CH_BS: Subdivision = Subdivision {
    name: "Basel-Stadt",
    code: "CH-BS",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-BS",
};

pub const CH_FR: Subdivision = Subdivision {
    name: "Freiburg",
    code: "CH-FR",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-FR",
};

pub const CH_GE: Subdivision = Subdivision {
    name: "Genève",
    code: "CH-GE",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-GE",
};

pub const CH_GL: Subdivision = Subdivision {
    name: "Glarus",
    code: "CH-GL",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-GL",
};

pub const CH_GR: Subdivision = Subdivision {
    name: "Graubünden",
    code: "CH-GR",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-GR",
};

pub const CH_JU: Subdivision = Subdivision {
    name: "Jura",
    code: "CH-JU",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-JU",
};

pub const CH_LU: Subdivision = Subdivision {
    name: "Luzern",
    code: "CH-LU",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-LU",
};

pub const CH_NE: Subdivision = Subdivision {
    name: "Neuchâtel",
    code: "CH-NE",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-NE",
};

pub const CH_NW: Subdivision = Subdivision {
    name: "Nidwalden",
    code: "CH-NW",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-NW",
};

pub const CH_OW: Subdivision = Subdivision {
    name: "Obwalden",
    code: "CH-OW",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-OW",
};

pub const CH_SG: Subdivision = Subdivision {
    name: "Sankt Gallen",
    code: "CH-SG",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-SG",
};

pub const CH_SH: Subdivision = Subdivision {
    name: "Schaffhausen",
    code: "CH-SH",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-SH",
};

pub const CH_SO: Subdivision = Subdivision {
    name: "Solothurn",
    code: "CH-SO",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-SO",
};

pub const CH_SZ: Subdivision = Subdivision {
    name: "Schwyz",
    code: "CH-SZ",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-SZ",
};

pub const CH_TG: Subdivision = Subdivision {
    name: "Thurgau",
    code: "CH-TG",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-TG",
};

pub const CH_TI: Subdivision = Subdivision {
    name: "Ticino",
    code: "CH-TI",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-TI",
};

pub const CH_UR: Subdivision = Subdivision {
    name: "Uri",
    code: "CH-UR",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-UR",
};

pub const CH_VD: Subdivision = Subdivision {
    name: "Vaud",
    code: "CH-VD",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-VD",
};

pub const CH_VS: Subdivision = Subdivision {
    name: "Valais",
    code: "CH-VS",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-VS",
};

pub const CH_ZG: Subdivision = Subdivision {
    name: "Zug",
    code: "CH-ZG",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-ZG",
};

pub const CH_ZH: Subdivision = Subdivision {
    name: "Zürich",
    code: "CH-ZH",
    subdivision_type: "Canton",
    country_name: "Switzerland",
    country_code: "CH",
    region_code: "CH-ZH",
};

pub const SY_DI: Subdivision = Subdivision {
    name: "Dimashq",
    code: "SY-DI",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-DI",
};

pub const SY_DR: Subdivision = Subdivision {
    name: "Dar'ā",
    code: "SY-DR",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-DR",
};

pub const SY_DY: Subdivision = Subdivision {
    name: "Dayr az Zawr",
    code: "SY-DY",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-DY",
};

pub const SY_HA: Subdivision = Subdivision {
    name: "Al Ḩasakah",
    code: "SY-HA",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-HA",
};

pub const SY_HI: Subdivision = Subdivision {
    name: "Ḩimş",
    code: "SY-HI",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-HI",
};

pub const SY_HL: Subdivision = Subdivision {
    name: "Ḩalab",
    code: "SY-HL",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-HL",
};

pub const SY_HM: Subdivision = Subdivision {
    name: "Ḩamāh",
    code: "SY-HM",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-HM",
};

pub const SY_ID: Subdivision = Subdivision {
    name: "Idlib",
    code: "SY-ID",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-ID",
};

pub const SY_LA: Subdivision = Subdivision {
    name: "Al Lādhiqīyah",
    code: "SY-LA",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-LA",
};

pub const SY_QU: Subdivision = Subdivision {
    name: "Al Qunayţirah",
    code: "SY-QU",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-QU",
};

pub const SY_RA: Subdivision = Subdivision {
    name: "Ar Raqqah",
    code: "SY-RA",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-RA",
};

pub const SY_RD: Subdivision = Subdivision {
    name: "Rīf Dimashq",
    code: "SY-RD",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-RD",
};

pub const SY_SU: Subdivision = Subdivision {
    name: "As Suwaydā'",
    code: "SY-SU",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-SU",
};

pub const SY_TA: Subdivision = Subdivision {
    name: "Ţarţūs",
    code: "SY-TA",
    subdivision_type: "Province",
    country_name: "Syria",
    country_code: "SY",
    region_code: "SY-TA",
};

pub const TW_CHA: Subdivision = Subdivision {
    name: "Changhua",
    code: "TW-CHA",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-CHA",
};

pub const TW_CYI: Subdivision = Subdivision {
    name: "Chiayi",
    code: "TW-CYI",
    subdivision_type: "Municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-CYI",
};

pub const TW_CYQ: Subdivision = Subdivision {
    name: "Chiayi",
    code: "TW-CYQ",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-CYQ",
};

pub const TW_HSQ: Subdivision = Subdivision {
    name: "Hsinchu",
    code: "TW-HSQ",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-HSQ",
};

pub const TW_HSZ: Subdivision = Subdivision {
    name: "Hsinchu",
    code: "TW-HSZ",
    subdivision_type: "Municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-HSZ",
};

pub const TW_HUA: Subdivision = Subdivision {
    name: "Hualien",
    code: "TW-HUA",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-HUA",
};

pub const TW_ILA: Subdivision = Subdivision {
    name: "Ilan",
    code: "TW-ILA",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-ILA",
};

pub const TW_KEE: Subdivision = Subdivision {
    name: "Keelung",
    code: "TW-KEE",
    subdivision_type: "Municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-KEE",
};

pub const TW_KHH: Subdivision = Subdivision {
    name: "Kaohsiung",
    code: "TW-KHH",
    subdivision_type: "Special municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-KHH",
};

pub const TW_KHQ: Subdivision = Subdivision {
    name: "Kaohsiung",
    code: "TW-KHQ",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-KHQ",
};

pub const TW_MIA: Subdivision = Subdivision {
    name: "Miaoli",
    code: "TW-MIA",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-MIA",
};

pub const TW_NAN: Subdivision = Subdivision {
    name: "Nantou",
    code: "TW-NAN",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-NAN",
};

pub const TW_PEN: Subdivision = Subdivision {
    name: "Penghu",
    code: "TW-PEN",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-PEN",
};

pub const TW_PIF: Subdivision = Subdivision {
    name: "Pingtung",
    code: "TW-PIF",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-PIF",
};

pub const TW_TAO: Subdivision = Subdivision {
    name: "Taoyuan",
    code: "TW-TAO",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TAO",
};

pub const TW_TNN: Subdivision = Subdivision {
    name: "Tainan",
    code: "TW-TNN",
    subdivision_type: "Municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TNN",
};

pub const TW_TNQ: Subdivision = Subdivision {
    name: "Tainan",
    code: "TW-TNQ",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TNQ",
};

pub const TW_TPE: Subdivision = Subdivision {
    name: "Taipei",
    code: "TW-TPE",
    subdivision_type: "Special municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TPE",
};

pub const TW_TPQ: Subdivision = Subdivision {
    name: "Taipei",
    code: "TW-TPQ",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TPQ",
};

pub const TW_TTT: Subdivision = Subdivision {
    name: "Taitung",
    code: "TW-TTT",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TTT",
};

pub const TW_TXG: Subdivision = Subdivision {
    name: "Taichung",
    code: "TW-TXG",
    subdivision_type: "Municipality",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TXG",
};

pub const TW_TXQ: Subdivision = Subdivision {
    name: "Taichung",
    code: "TW-TXQ",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-TXQ",
};

pub const TW_YUN: Subdivision = Subdivision {
    name: "Yunlin",
    code: "TW-YUN",
    subdivision_type: "District",
    country_name: "Taiwan",
    country_code: "TW",
    region_code: "TW-YUN",
};

pub const TJ_DU: Subdivision = Subdivision {
    name: "Dushanbe",
    code: "TJ-DU",
    subdivision_type: "Capital territory",
    country_name: "Tajikistan",
    country_code: "TJ",
    region_code: "TJ-DU",
};

pub const TJ_GB: Subdivision = Subdivision {
    name: "Kŭhistoni Badakhshon",
    code: "TJ-GB",
    subdivision_type: "Autonomous region",
    country_name: "Tajikistan",
    country_code: "TJ",
    region_code: "TJ-GB",
};

pub const TJ_KT: Subdivision = Subdivision {
    name: "Khatlon",
    code: "TJ-KT",
    subdivision_type: "Region",
    country_name: "Tajikistan",
    country_code: "TJ",
    region_code: "TJ-KT",
};

pub const TJ_SU: Subdivision = Subdivision {
    name: "Sughd",
    code: "TJ-SU",
    subdivision_type: "Region",
    country_name: "Tajikistan",
    country_code: "TJ",
    region_code: "TJ-SU",
};

pub const TZ_01: Subdivision = Subdivision {
    name: "Arusha",
    code: "TZ-01",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-01",
};

pub const TZ_02: Subdivision = Subdivision {
    name: "Dar es Salaam",
    code: "TZ-02",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-02",
};

pub const TZ_03: Subdivision = Subdivision {
    name: "Dodoma",
    code: "TZ-03",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-03",
};

pub const TZ_04: Subdivision = Subdivision {
    name: "Iringa",
    code: "TZ-04",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-04",
};

pub const TZ_05: Subdivision = Subdivision {
    name: "Kagera",
    code: "TZ-05",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-05",
};

pub const TZ_06: Subdivision = Subdivision {
    name: "Kaskazini Pemba",
    code: "TZ-06",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-06",
};

pub const TZ_07: Subdivision = Subdivision {
    name: "Kaskazini Unguja",
    code: "TZ-07",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-07",
};

pub const TZ_08: Subdivision = Subdivision {
    name: "Kigoma",
    code: "TZ-08",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-08",
};

pub const TZ_09: Subdivision = Subdivision {
    name: "Kilimanjaro",
    code: "TZ-09",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-09",
};

pub const TZ_10: Subdivision = Subdivision {
    name: "Kusini Pemba",
    code: "TZ-10",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-10",
};

pub const TZ_11: Subdivision = Subdivision {
    name: "Kusini Unguja",
    code: "TZ-11",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-11",
};

pub const TZ_12: Subdivision = Subdivision {
    name: "Lindi",
    code: "TZ-12",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-12",
};

pub const TZ_13: Subdivision = Subdivision {
    name: "Mara",
    code: "TZ-13",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-13",
};

pub const TZ_14: Subdivision = Subdivision {
    name: "Mbeya",
    code: "TZ-14",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-14",
};

pub const TZ_15: Subdivision = Subdivision {
    name: "Mjini Magharibi",
    code: "TZ-15",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-15",
};

pub const TZ_16: Subdivision = Subdivision {
    name: "Morogoro",
    code: "TZ-16",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-16",
};

pub const TZ_17: Subdivision = Subdivision {
    name: "Mtwara",
    code: "TZ-17",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-17",
};

pub const TZ_18: Subdivision = Subdivision {
    name: "Mwanza",
    code: "TZ-18",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-18",
};

pub const TZ_19: Subdivision = Subdivision {
    name: "Coast",
    code: "TZ-19",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-19",
};

pub const TZ_20: Subdivision = Subdivision {
    name: "Rukwa",
    code: "TZ-20",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-20",
};

pub const TZ_21: Subdivision = Subdivision {
    name: "Ruvuma",
    code: "TZ-21",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-21",
};

pub const TZ_22: Subdivision = Subdivision {
    name: "Shinyanga",
    code: "TZ-22",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-22",
};

pub const TZ_23: Subdivision = Subdivision {
    name: "Singida",
    code: "TZ-23",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-23",
};

pub const TZ_24: Subdivision = Subdivision {
    name: "Tabora",
    code: "TZ-24",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-24",
};

pub const TZ_25: Subdivision = Subdivision {
    name: "Tanga",
    code: "TZ-25",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-25",
};

pub const TZ_26: Subdivision = Subdivision {
    name: "Manyara",
    code: "TZ-26",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-26",
};

pub const TZ_27: Subdivision = Subdivision {
    name: "Geita",
    code: "TZ-27",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-27",
};

pub const TZ_28: Subdivision = Subdivision {
    name: "Katavi",
    code: "TZ-28",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-28",
};

pub const TZ_29: Subdivision = Subdivision {
    name: "Njombe",
    code: "TZ-29",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-29",
};

pub const TZ_30: Subdivision = Subdivision {
    name: "Simiyu",
    code: "TZ-30",
    subdivision_type: "Region",
    country_name: "Tanzania",
    country_code: "TZ",
    region_code: "TZ-30",
};

pub const TH_10: Subdivision = Subdivision {
    name: "Krung Thep Maha Nakhon",
    code: "TH-10",
    subdivision_type: "Metropolitan administration",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-10",
};

pub const TH_11: Subdivision = Subdivision {
    name: "Samut Prakan",
    code: "TH-11",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-11",
};

pub const TH_12: Subdivision = Subdivision {
    name: "Nonthaburi",
    code: "TH-12",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-12",
};

pub const TH_13: Subdivision = Subdivision {
    name: "Pathum Thani",
    code: "TH-13",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-13",
};

pub const TH_14: Subdivision = Subdivision {
    name: "Phra Nakhon Si Ayutthaya",
    code: "TH-14",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-14",
};

pub const TH_15: Subdivision = Subdivision {
    name: "Ang Thong",
    code: "TH-15",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-15",
};

pub const TH_16: Subdivision = Subdivision {
    name: "Lop Buri",
    code: "TH-16",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-16",
};

pub const TH_17: Subdivision = Subdivision {
    name: "Sing Buri",
    code: "TH-17",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-17",
};

pub const TH_18: Subdivision = Subdivision {
    name: "Chai Nat",
    code: "TH-18",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-18",
};

pub const TH_19: Subdivision = Subdivision {
    name: "Saraburi",
    code: "TH-19",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-19",
};

pub const TH_20: Subdivision = Subdivision {
    name: "Chon Buri",
    code: "TH-20",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-20",
};

pub const TH_21: Subdivision = Subdivision {
    name: "Rayong",
    code: "TH-21",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-21",
};

pub const TH_22: Subdivision = Subdivision {
    name: "Chanthaburi",
    code: "TH-22",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-22",
};

pub const TH_23: Subdivision = Subdivision {
    name: "Trat",
    code: "TH-23",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-23",
};

pub const TH_24: Subdivision = Subdivision {
    name: "Chachoengsao",
    code: "TH-24",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-24",
};

pub const TH_25: Subdivision = Subdivision {
    name: "Prachin Buri",
    code: "TH-25",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-25",
};

pub const TH_26: Subdivision = Subdivision {
    name: "Nakhon Nayok",
    code: "TH-26",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-26",
};

pub const TH_27: Subdivision = Subdivision {
    name: "Sa Kaeo",
    code: "TH-27",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-27",
};

pub const TH_30: Subdivision = Subdivision {
    name: "Nakhon Ratchasima",
    code: "TH-30",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-30",
};

pub const TH_31: Subdivision = Subdivision {
    name: "Buri Ram",
    code: "TH-31",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-31",
};

pub const TH_32: Subdivision = Subdivision {
    name: "Surin",
    code: "TH-32",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-32",
};

pub const TH_33: Subdivision = Subdivision {
    name: "Si Sa Ket",
    code: "TH-33",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-33",
};

pub const TH_34: Subdivision = Subdivision {
    name: "Ubon Ratchathani",
    code: "TH-34",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-34",
};

pub const TH_35: Subdivision = Subdivision {
    name: "Yasothon",
    code: "TH-35",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-35",
};

pub const TH_36: Subdivision = Subdivision {
    name: "Chaiyaphum",
    code: "TH-36",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-36",
};

pub const TH_37: Subdivision = Subdivision {
    name: "Amnat Charoen",
    code: "TH-37",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-37",
};

pub const TH_38: Subdivision = Subdivision {
    name: "Bueng Kan",
    code: "TH-38",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-38",
};

pub const TH_39: Subdivision = Subdivision {
    name: "Nong Bua Lam Phu",
    code: "TH-39",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-39",
};

pub const TH_40: Subdivision = Subdivision {
    name: "Khon Kaen",
    code: "TH-40",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-40",
};

pub const TH_41: Subdivision = Subdivision {
    name: "Udon Thani",
    code: "TH-41",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-41",
};

pub const TH_42: Subdivision = Subdivision {
    name: "Loei",
    code: "TH-42",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-42",
};

pub const TH_43: Subdivision = Subdivision {
    name: "Nong Khai",
    code: "TH-43",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-43",
};

pub const TH_44: Subdivision = Subdivision {
    name: "Maha Sarakham",
    code: "TH-44",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-44",
};

pub const TH_45: Subdivision = Subdivision {
    name: "Roi Et",
    code: "TH-45",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-45",
};

pub const TH_46: Subdivision = Subdivision {
    name: "Kalasin",
    code: "TH-46",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-46",
};

pub const TH_47: Subdivision = Subdivision {
    name: "Sakon Nakhon",
    code: "TH-47",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-47",
};

pub const TH_48: Subdivision = Subdivision {
    name: "Nakhon Phanom",
    code: "TH-48",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-48",
};

pub const TH_49: Subdivision = Subdivision {
    name: "Mukdahan",
    code: "TH-49",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-49",
};

pub const TH_50: Subdivision = Subdivision {
    name: "Chiang Mai",
    code: "TH-50",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-50",
};

pub const TH_51: Subdivision = Subdivision {
    name: "Lamphun",
    code: "TH-51",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-51",
};

pub const TH_52: Subdivision = Subdivision {
    name: "Lampang",
    code: "TH-52",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-52",
};

pub const TH_53: Subdivision = Subdivision {
    name: "Uttaradit",
    code: "TH-53",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-53",
};

pub const TH_54: Subdivision = Subdivision {
    name: "Phrae",
    code: "TH-54",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-54",
};

pub const TH_55: Subdivision = Subdivision {
    name: "Nan",
    code: "TH-55",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-55",
};

pub const TH_56: Subdivision = Subdivision {
    name: "Phayao",
    code: "TH-56",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-56",
};

pub const TH_57: Subdivision = Subdivision {
    name: "Chiang Rai",
    code: "TH-57",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-57",
};

pub const TH_58: Subdivision = Subdivision {
    name: "Mae Hong Son",
    code: "TH-58",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-58",
};

pub const TH_60: Subdivision = Subdivision {
    name: "Nakhon Sawan",
    code: "TH-60",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-60",
};

pub const TH_61: Subdivision = Subdivision {
    name: "Uthai Thani",
    code: "TH-61",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-61",
};

pub const TH_62: Subdivision = Subdivision {
    name: "Kamphaeng Phet",
    code: "TH-62",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-62",
};

pub const TH_63: Subdivision = Subdivision {
    name: "Tak",
    code: "TH-63",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-63",
};

pub const TH_64: Subdivision = Subdivision {
    name: "Sukhothai",
    code: "TH-64",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-64",
};

pub const TH_65: Subdivision = Subdivision {
    name: "Phitsanulok",
    code: "TH-65",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-65",
};

pub const TH_66: Subdivision = Subdivision {
    name: "Phichit",
    code: "TH-66",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-66",
};

pub const TH_67: Subdivision = Subdivision {
    name: "Phetchabun",
    code: "TH-67",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-67",
};

pub const TH_70: Subdivision = Subdivision {
    name: "Ratchaburi",
    code: "TH-70",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-70",
};

pub const TH_71: Subdivision = Subdivision {
    name: "Kanchanaburi",
    code: "TH-71",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-71",
};

pub const TH_72: Subdivision = Subdivision {
    name: "Suphan Buri",
    code: "TH-72",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-72",
};

pub const TH_73: Subdivision = Subdivision {
    name: "Nakhon Pathom",
    code: "TH-73",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-73",
};

pub const TH_74: Subdivision = Subdivision {
    name: "Samut Sakhon",
    code: "TH-74",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-74",
};

pub const TH_75: Subdivision = Subdivision {
    name: "Samut Songkhram",
    code: "TH-75",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-75",
};

pub const TH_76: Subdivision = Subdivision {
    name: "Phetchaburi",
    code: "TH-76",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-76",
};

pub const TH_77: Subdivision = Subdivision {
    name: "Prachuap Khiri Khan",
    code: "TH-77",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-77",
};

pub const TH_80: Subdivision = Subdivision {
    name: "Nakhon Si Thammarat",
    code: "TH-80",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-80",
};

pub const TH_81: Subdivision = Subdivision {
    name: "Krabi",
    code: "TH-81",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-81",
};

pub const TH_82: Subdivision = Subdivision {
    name: "Phangnga",
    code: "TH-82",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-82",
};

pub const TH_83: Subdivision = Subdivision {
    name: "Phuket",
    code: "TH-83",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-83",
};

pub const TH_84: Subdivision = Subdivision {
    name: "Surat Thani",
    code: "TH-84",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-84",
};

pub const TH_85: Subdivision = Subdivision {
    name: "Ranong",
    code: "TH-85",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-85",
};

pub const TH_86: Subdivision = Subdivision {
    name: "Chumphon",
    code: "TH-86",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-86",
};

pub const TH_90: Subdivision = Subdivision {
    name: "Songkhla",
    code: "TH-90",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-90",
};

pub const TH_91: Subdivision = Subdivision {
    name: "Satun",
    code: "TH-91",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-91",
};

pub const TH_92: Subdivision = Subdivision {
    name: "Trang",
    code: "TH-92",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-92",
};

pub const TH_93: Subdivision = Subdivision {
    name: "Phatthalung",
    code: "TH-93",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-93",
};

pub const TH_94: Subdivision = Subdivision {
    name: "Pattani",
    code: "TH-94",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-94",
};

pub const TH_95: Subdivision = Subdivision {
    name: "Yala",
    code: "TH-95",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-95",
};

pub const TH_96: Subdivision = Subdivision {
    name: "Narathiwat",
    code: "TH-96",
    subdivision_type: "Province",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-96",
};

pub const TH_S: Subdivision = Subdivision {
    name: "Phatthaya",
    code: "TH-S",
    subdivision_type: "Special administrative city",
    country_name: "Thailand",
    country_code: "TH",
    region_code: "TH-S",
};

pub const TL_AL: Subdivision = Subdivision {
    name: "Aileu",
    code: "TL-AL",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-AL",
};

pub const TL_AN: Subdivision = Subdivision {
    name: "Ainaro",
    code: "TL-AN",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-AN",
};

pub const TL_BA: Subdivision = Subdivision {
    name: "Baucau",
    code: "TL-BA",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-BA",
};

pub const TL_BO: Subdivision = Subdivision {
    name: "Bobonaro",
    code: "TL-BO",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-BO",
};

pub const TL_CO: Subdivision = Subdivision {
    name: "Cova Lima",
    code: "TL-CO",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-CO",
};

pub const TL_DI: Subdivision = Subdivision {
    name: "Díli",
    code: "TL-DI",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-DI",
};

pub const TL_ER: Subdivision = Subdivision {
    name: "Ermera",
    code: "TL-ER",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-ER",
};

pub const TL_LA: Subdivision = Subdivision {
    name: "Lautém",
    code: "TL-LA",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-LA",
};

pub const TL_LI: Subdivision = Subdivision {
    name: "Likisá",
    code: "TL-LI",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-LI",
};

pub const TL_MF: Subdivision = Subdivision {
    name: "Manufahi",
    code: "TL-MF",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-MF",
};

pub const TL_MT: Subdivision = Subdivision {
    name: "Manatuto",
    code: "TL-MT",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-MT",
};

pub const TL_OE: Subdivision = Subdivision {
    name: "Oecussi",
    code: "TL-OE",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-OE",
};

pub const TL_VI: Subdivision = Subdivision {
    name: "Vikeke",
    code: "TL-VI",
    subdivision_type: "District",
    country_name: "East Timor",
    country_code: "TL",
    region_code: "TL-VI",
};

pub const TG_C: Subdivision = Subdivision {
    name: "Centre",
    code: "TG-C",
    subdivision_type: "Region",
    country_name: "Togo",
    country_code: "TG",
    region_code: "TG-C",
};

pub const TG_K: Subdivision = Subdivision {
    name: "Kara",
    code: "TG-K",
    subdivision_type: "Region",
    country_name: "Togo",
    country_code: "TG",
    region_code: "TG-K",
};

pub const TG_M: Subdivision = Subdivision {
    name: "Maritime",
    code: "TG-M",
    subdivision_type: "Region",
    country_name: "Togo",
    country_code: "TG",
    region_code: "TG-M",
};

pub const TG_P: Subdivision = Subdivision {
    name: "Plateaux",
    code: "TG-P",
    subdivision_type: "Region",
    country_name: "Togo",
    country_code: "TG",
    region_code: "TG-P",
};

pub const TG_S: Subdivision = Subdivision {
    name: "Savannes",
    code: "TG-S",
    subdivision_type: "Region",
    country_name: "Togo",
    country_code: "TG",
    region_code: "TG-S",
};

pub const TK_TK: Subdivision = Subdivision {
    name: "Tokelau",
    code: "TK-TK",
    subdivision_type: "Country",
    country_name: "Tokelau",
    country_code: "TK",
    region_code: "TK-TK",
};

pub const TO_01: Subdivision = Subdivision {
    name: "'Eua",
    code: "TO-01",
    subdivision_type: "Division",
    country_name: "Tonga",
    country_code: "TO",
    region_code: "TO-01",
};

pub const TO_02: Subdivision = Subdivision {
    name: "Ha'apai",
    code: "TO-02",
    subdivision_type: "Division",
    country_name: "Tonga",
    country_code: "TO",
    region_code: "TO-02",
};

pub const TO_03: Subdivision = Subdivision {
    name: "Niuas",
    code: "TO-03",
    subdivision_type: "Division",
    country_name: "Tonga",
    country_code: "TO",
    region_code: "TO-03",
};

pub const TO_04: Subdivision = Subdivision {
    name: "Tongatapu",
    code: "TO-04",
    subdivision_type: "Division",
    country_name: "Tonga",
    country_code: "TO",
    region_code: "TO-04",
};

pub const TO_05: Subdivision = Subdivision {
    name: "Vava'u",
    code: "TO-05",
    subdivision_type: "Division",
    country_name: "Tonga",
    country_code: "TO",
    region_code: "TO-05",
};

pub const TT_ARI: Subdivision = Subdivision {
    name: "Arima",
    code: "TT-ARI",
    subdivision_type: "Municipality",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-ARI",
};

pub const TT_CHA: Subdivision = Subdivision {
    name: "Chaguanas",
    code: "TT-CHA",
    subdivision_type: "Municipality",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-CHA",
};

pub const TT_CTT: Subdivision = Subdivision {
    name: "Couva-Tabaquite-Talparo",
    code: "TT-CTT",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-CTT",
};

pub const TT_DMN: Subdivision = Subdivision {
    name: "Diego Martin",
    code: "TT-DMN",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-DMN",
};

pub const TT_ETO: Subdivision = Subdivision {
    name: "Eastern Tobago",
    code: "TT-ETO",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-ETO",
};

pub const TT_PED: Subdivision = Subdivision {
    name: "Penal-Debe",
    code: "TT-PED",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-PED",
};

pub const TT_POS: Subdivision = Subdivision {
    name: "Port of Spain",
    code: "TT-POS",
    subdivision_type: "Municipality",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-POS",
};

pub const TT_PRT: Subdivision = Subdivision {
    name: "Princes Town",
    code: "TT-PRT",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-PRT",
};

pub const TT_PTF: Subdivision = Subdivision {
    name: "Point Fortin",
    code: "TT-PTF",
    subdivision_type: "Municipality",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-PTF",
};

pub const TT_RCM: Subdivision = Subdivision {
    name: "Rio Claro-Mayaro",
    code: "TT-RCM",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-RCM",
};

pub const TT_SFO: Subdivision = Subdivision {
    name: "San Fernando",
    code: "TT-SFO",
    subdivision_type: "Municipality",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-SFO",
};

pub const TT_SGE: Subdivision = Subdivision {
    name: "Sangre Grande",
    code: "TT-SGE",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-SGE",
};

pub const TT_SIP: Subdivision = Subdivision {
    name: "Siparia",
    code: "TT-SIP",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-SIP",
};

pub const TT_SJL: Subdivision = Subdivision {
    name: "San Juan-Laventille",
    code: "TT-SJL",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-SJL",
};

pub const TT_TUP: Subdivision = Subdivision {
    name: "Tunapuna-Piarco",
    code: "TT-TUP",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-TUP",
};

pub const TT_WTO: Subdivision = Subdivision {
    name: "Western Tobago",
    code: "TT-WTO",
    subdivision_type: "Region",
    country_name: "Trinidad and Tobago",
    country_code: "TT",
    region_code: "TT-WTO",
};

pub const TN_11: Subdivision = Subdivision {
    name: "Tunis",
    code: "TN-11",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-11",
};

pub const TN_12: Subdivision = Subdivision {
    name: "Ariana",
    code: "TN-12",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-12",
};

pub const TN_13: Subdivision = Subdivision {
    name: "Ben Arous",
    code: "TN-13",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-13",
};

pub const TN_14: Subdivision = Subdivision {
    name: "La Manouba",
    code: "TN-14",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-14",
};

pub const TN_21: Subdivision = Subdivision {
    name: "Nabeul",
    code: "TN-21",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-21",
};

pub const TN_22: Subdivision = Subdivision {
    name: "Zaghouan",
    code: "TN-22",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-22",
};

pub const TN_23: Subdivision = Subdivision {
    name: "Bizerte",
    code: "TN-23",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-23",
};

pub const TN_31: Subdivision = Subdivision {
    name: "Béja",
    code: "TN-31",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-31",
};

pub const TN_32: Subdivision = Subdivision {
    name: "Jendouba",
    code: "TN-32",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-32",
};

pub const TN_33: Subdivision = Subdivision {
    name: "Le Kef",
    code: "TN-33",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-33",
};

pub const TN_34: Subdivision = Subdivision {
    name: "Siliana",
    code: "TN-34",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-34",
};

pub const TN_41: Subdivision = Subdivision {
    name: "Kairouan",
    code: "TN-41",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-41",
};

pub const TN_42: Subdivision = Subdivision {
    name: "Kasserine",
    code: "TN-42",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-42",
};

pub const TN_43: Subdivision = Subdivision {
    name: "Sidi Bouzid",
    code: "TN-43",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-43",
};

pub const TN_51: Subdivision = Subdivision {
    name: "Sousse",
    code: "TN-51",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-51",
};

pub const TN_52: Subdivision = Subdivision {
    name: "Monastir",
    code: "TN-52",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-52",
};

pub const TN_53: Subdivision = Subdivision {
    name: "Mahdia",
    code: "TN-53",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-53",
};

pub const TN_61: Subdivision = Subdivision {
    name: "Sfax",
    code: "TN-61",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-61",
};

pub const TN_71: Subdivision = Subdivision {
    name: "Gafsa",
    code: "TN-71",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-71",
};

pub const TN_72: Subdivision = Subdivision {
    name: "Tozeur",
    code: "TN-72",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-72",
};

pub const TN_73: Subdivision = Subdivision {
    name: "Kebili",
    code: "TN-73",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-73",
};

pub const TN_81: Subdivision = Subdivision {
    name: "Gabès",
    code: "TN-81",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-81",
};

pub const TN_82: Subdivision = Subdivision {
    name: "Medenine",
    code: "TN-82",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-82",
};

pub const TN_83: Subdivision = Subdivision {
    name: "Tataouine",
    code: "TN-83",
    subdivision_type: "Governorate",
    country_name: "Tunisia",
    country_code: "TN",
    region_code: "TN-83",
};

pub const TR_01: Subdivision = Subdivision {
    name: "Adana",
    code: "TR-01",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-01",
};

pub const TR_02: Subdivision = Subdivision {
    name: "Adıyaman",
    code: "TR-02",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-02",
};

pub const TR_03: Subdivision = Subdivision {
    name: "Afyonkarahisar",
    code: "TR-03",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-03",
};

pub const TR_04: Subdivision = Subdivision {
    name: "Ağrı",
    code: "TR-04",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-04",
};

pub const TR_05: Subdivision = Subdivision {
    name: "Amasya",
    code: "TR-05",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-05",
};

pub const TR_06: Subdivision = Subdivision {
    name: "Ankara",
    code: "TR-06",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-06",
};

pub const TR_07: Subdivision = Subdivision {
    name: "Antalya",
    code: "TR-07",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-07",
};

pub const TR_08: Subdivision = Subdivision {
    name: "Artvin",
    code: "TR-08",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-08",
};

pub const TR_09: Subdivision = Subdivision {
    name: "Aydın",
    code: "TR-09",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-09",
};

pub const TR_10: Subdivision = Subdivision {
    name: "Balıkesir",
    code: "TR-10",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-10",
};

pub const TR_11: Subdivision = Subdivision {
    name: "Bilecik",
    code: "TR-11",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-11",
};

pub const TR_12: Subdivision = Subdivision {
    name: "Bingöl",
    code: "TR-12",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-12",
};

pub const TR_13: Subdivision = Subdivision {
    name: "Bitlis",
    code: "TR-13",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-13",
};

pub const TR_14: Subdivision = Subdivision {
    name: "Bolu",
    code: "TR-14",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-14",
};

pub const TR_15: Subdivision = Subdivision {
    name: "Burdur",
    code: "TR-15",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-15",
};

pub const TR_16: Subdivision = Subdivision {
    name: "Bursa",
    code: "TR-16",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-16",
};

pub const TR_17: Subdivision = Subdivision {
    name: "Çanakkale",
    code: "TR-17",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-17",
};

pub const TR_18: Subdivision = Subdivision {
    name: "Çankırı",
    code: "TR-18",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-18",
};

pub const TR_19: Subdivision = Subdivision {
    name: "Çorum",
    code: "TR-19",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-19",
};

pub const TR_20: Subdivision = Subdivision {
    name: "Denizli",
    code: "TR-20",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-20",
};

pub const TR_21: Subdivision = Subdivision {
    name: "Diyarbakır",
    code: "TR-21",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-21",
};

pub const TR_22: Subdivision = Subdivision {
    name: "Edirne",
    code: "TR-22",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-22",
};

pub const TR_23: Subdivision = Subdivision {
    name: "Elazığ",
    code: "TR-23",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-23",
};

pub const TR_24: Subdivision = Subdivision {
    name: "Erzincan",
    code: "TR-24",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-24",
};

pub const TR_25: Subdivision = Subdivision {
    name: "Erzurum",
    code: "TR-25",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-25",
};

pub const TR_26: Subdivision = Subdivision {
    name: "Eskişehir",
    code: "TR-26",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-26",
};

pub const TR_27: Subdivision = Subdivision {
    name: "Gaziantep",
    code: "TR-27",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-27",
};

pub const TR_28: Subdivision = Subdivision {
    name: "Giresun",
    code: "TR-28",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-28",
};

pub const TR_29: Subdivision = Subdivision {
    name: "Gümüşhane",
    code: "TR-29",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-29",
};

pub const TR_30: Subdivision = Subdivision {
    name: "Hakkâri",
    code: "TR-30",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-30",
};

pub const TR_31: Subdivision = Subdivision {
    name: "Hatay",
    code: "TR-31",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-31",
};

pub const TR_32: Subdivision = Subdivision {
    name: "Isparta",
    code: "TR-32",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-32",
};

pub const TR_33: Subdivision = Subdivision {
    name: "Mersin",
    code: "TR-33",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-33",
};

pub const TR_34: Subdivision = Subdivision {
    name: "İstanbul",
    code: "TR-34",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-34",
};

pub const TR_35: Subdivision = Subdivision {
    name: "İzmir",
    code: "TR-35",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-35",
};

pub const TR_36: Subdivision = Subdivision {
    name: "Kars",
    code: "TR-36",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-36",
};

pub const TR_37: Subdivision = Subdivision {
    name: "Kastamonu",
    code: "TR-37",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-37",
};

pub const TR_38: Subdivision = Subdivision {
    name: "Kayseri",
    code: "TR-38",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-38",
};

pub const TR_39: Subdivision = Subdivision {
    name: "Kırklareli",
    code: "TR-39",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-39",
};

pub const TR_40: Subdivision = Subdivision {
    name: "Kırşehir",
    code: "TR-40",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-40",
};

pub const TR_41: Subdivision = Subdivision {
    name: "Kocaeli",
    code: "TR-41",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-41",
};

pub const TR_42: Subdivision = Subdivision {
    name: "Konya",
    code: "TR-42",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-42",
};

pub const TR_43: Subdivision = Subdivision {
    name: "Kütahya",
    code: "TR-43",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-43",
};

pub const TR_44: Subdivision = Subdivision {
    name: "Malatya",
    code: "TR-44",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-44",
};

pub const TR_45: Subdivision = Subdivision {
    name: "Manisa",
    code: "TR-45",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-45",
};

pub const TR_46: Subdivision = Subdivision {
    name: "Kahramanmaraş",
    code: "TR-46",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-46",
};

pub const TR_47: Subdivision = Subdivision {
    name: "Mardin",
    code: "TR-47",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-47",
};

pub const TR_48: Subdivision = Subdivision {
    name: "Muğla",
    code: "TR-48",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-48",
};

pub const TR_49: Subdivision = Subdivision {
    name: "Muş",
    code: "TR-49",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-49",
};

pub const TR_50: Subdivision = Subdivision {
    name: "Nevşehir",
    code: "TR-50",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-50",
};

pub const TR_51: Subdivision = Subdivision {
    name: "Niğde",
    code: "TR-51",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-51",
};

pub const TR_52: Subdivision = Subdivision {
    name: "Ordu",
    code: "TR-52",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-52",
};

pub const TR_53: Subdivision = Subdivision {
    name: "Rize",
    code: "TR-53",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-53",
};

pub const TR_54: Subdivision = Subdivision {
    name: "Sakarya",
    code: "TR-54",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-54",
};

pub const TR_55: Subdivision = Subdivision {
    name: "Samsun",
    code: "TR-55",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-55",
};

pub const TR_56: Subdivision = Subdivision {
    name: "Siirt",
    code: "TR-56",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-56",
};

pub const TR_57: Subdivision = Subdivision {
    name: "Sinop",
    code: "TR-57",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-57",
};

pub const TR_58: Subdivision = Subdivision {
    name: "Sivas",
    code: "TR-58",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-58",
};

pub const TR_59: Subdivision = Subdivision {
    name: "Tekirdağ",
    code: "TR-59",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-59",
};

pub const TR_60: Subdivision = Subdivision {
    name: "Tokat",
    code: "TR-60",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-60",
};

pub const TR_61: Subdivision = Subdivision {
    name: "Trabzon",
    code: "TR-61",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-61",
};

pub const TR_62: Subdivision = Subdivision {
    name: "Tunceli",
    code: "TR-62",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-62",
};

pub const TR_63: Subdivision = Subdivision {
    name: "Şanlıurfa",
    code: "TR-63",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-63",
};

pub const TR_64: Subdivision = Subdivision {
    name: "Uşak",
    code: "TR-64",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-64",
};

pub const TR_65: Subdivision = Subdivision {
    name: "Van",
    code: "TR-65",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-65",
};

pub const TR_66: Subdivision = Subdivision {
    name: "Yozgat",
    code: "TR-66",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-66",
};

pub const TR_67: Subdivision = Subdivision {
    name: "Zonguldak",
    code: "TR-67",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-67",
};

pub const TR_68: Subdivision = Subdivision {
    name: "Aksaray",
    code: "TR-68",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-68",
};

pub const TR_69: Subdivision = Subdivision {
    name: "Bayburt",
    code: "TR-69",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-69",
};

pub const TR_70: Subdivision = Subdivision {
    name: "Karaman",
    code: "TR-70",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-70",
};

pub const TR_71: Subdivision = Subdivision {
    name: "Kırıkkale",
    code: "TR-71",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-71",
};

pub const TR_72: Subdivision = Subdivision {
    name: "Batman",
    code: "TR-72",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-72",
};

pub const TR_73: Subdivision = Subdivision {
    name: "Şırnak",
    code: "TR-73",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-73",
};

pub const TR_74: Subdivision = Subdivision {
    name: "Bartın",
    code: "TR-74",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-74",
};

pub const TR_75: Subdivision = Subdivision {
    name: "Ardahan",
    code: "TR-75",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-75",
};

pub const TR_76: Subdivision = Subdivision {
    name: "Iğdır",
    code: "TR-76",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-76",
};

pub const TR_77: Subdivision = Subdivision {
    name: "Yalova",
    code: "TR-77",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-77",
};

pub const TR_78: Subdivision = Subdivision {
    name: "Karabük",
    code: "TR-78",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-78",
};

pub const TR_79: Subdivision = Subdivision {
    name: "Kilis",
    code: "TR-79",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-79",
};

pub const TR_80: Subdivision = Subdivision {
    name: "Osmaniye",
    code: "TR-80",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-80",
};

pub const TR_81: Subdivision = Subdivision {
    name: "Düzce",
    code: "TR-81",
    subdivision_type: "Province",
    country_name: "Turkey",
    country_code: "TR",
    region_code: "TR-81",
};

pub const TM_A: Subdivision = Subdivision {
    name: "Ahal",
    code: "TM-A",
    subdivision_type: "Region",
    country_name: "Turkmenistan",
    country_code: "TM",
    region_code: "TM-A",
};

pub const TM_B: Subdivision = Subdivision {
    name: "Balkan",
    code: "TM-B",
    subdivision_type: "Region",
    country_name: "Turkmenistan",
    country_code: "TM",
    region_code: "TM-B",
};

pub const TM_D: Subdivision = Subdivision {
    name: "Daşoguz",
    code: "TM-D",
    subdivision_type: "Region",
    country_name: "Turkmenistan",
    country_code: "TM",
    region_code: "TM-D",
};

pub const TM_L: Subdivision = Subdivision {
    name: "Lebap",
    code: "TM-L",
    subdivision_type: "Region",
    country_name: "Turkmenistan",
    country_code: "TM",
    region_code: "TM-L",
};

pub const TM_M: Subdivision = Subdivision {
    name: "Mary",
    code: "TM-M",
    subdivision_type: "Region",
    country_name: "Turkmenistan",
    country_code: "TM",
    region_code: "TM-M",
};

pub const TM_S: Subdivision = Subdivision {
    name: "Aşgabat",
    code: "TM-S",
    subdivision_type: "City",
    country_name: "Turkmenistan",
    country_code: "TM",
    region_code: "TM-S",
};

pub const TC_TC: Subdivision = Subdivision {
    name: "Turks & Caicos Islands",
    code: "TC-TC",
    subdivision_type: "Country",
    country_name: "Turks & Caicos Islands",
    country_code: "TC",
    region_code: "TC-TC",
};

pub const TV_FUN: Subdivision = Subdivision {
    name: "Funafuti",
    code: "TV-FUN",
    subdivision_type: "Town council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-FUN",
};

pub const TV_NIT: Subdivision = Subdivision {
    name: "Niutao",
    code: "TV-NIT",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-NIT",
};

pub const TV_NKF: Subdivision = Subdivision {
    name: "Nukufetau",
    code: "TV-NKF",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-NKF",
};

pub const TV_NKL: Subdivision = Subdivision {
    name: "Nukulaelae",
    code: "TV-NKL",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-NKL",
};

pub const TV_NMA: Subdivision = Subdivision {
    name: "Nanumea",
    code: "TV-NMA",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-NMA",
};

pub const TV_NMG: Subdivision = Subdivision {
    name: "Nanumanga",
    code: "TV-NMG",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-NMG",
};

pub const TV_NUI: Subdivision = Subdivision {
    name: "Nui",
    code: "TV-NUI",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-NUI",
};

pub const TV_VAI: Subdivision = Subdivision {
    name: "Vaitupu",
    code: "TV-VAI",
    subdivision_type: "Island council",
    country_name: "Tuvalu",
    country_code: "TV",
    region_code: "TV-VAI",
};

pub const UG_101: Subdivision = Subdivision {
    name: "Kalangala",
    code: "UG-101",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-101",
};

pub const UG_102: Subdivision = Subdivision {
    name: "Kampala",
    code: "UG-102",
    subdivision_type: "City",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-102",
};

pub const UG_103: Subdivision = Subdivision {
    name: "Kiboga",
    code: "UG-103",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-103",
};

pub const UG_104: Subdivision = Subdivision {
    name: "Luwero",
    code: "UG-104",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-104",
};

pub const UG_105: Subdivision = Subdivision {
    name: "Masaka",
    code: "UG-105",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-105",
};

pub const UG_106: Subdivision = Subdivision {
    name: "Mpigi",
    code: "UG-106",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-106",
};

pub const UG_107: Subdivision = Subdivision {
    name: "Mubende",
    code: "UG-107",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-107",
};

pub const UG_108: Subdivision = Subdivision {
    name: "Mukono",
    code: "UG-108",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-108",
};

pub const UG_109: Subdivision = Subdivision {
    name: "Nakasongola",
    code: "UG-109",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-109",
};

pub const UG_110: Subdivision = Subdivision {
    name: "Rakai",
    code: "UG-110",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-110",
};

pub const UG_111: Subdivision = Subdivision {
    name: "Sembabule",
    code: "UG-111",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-111",
};

pub const UG_112: Subdivision = Subdivision {
    name: "Kayunga",
    code: "UG-112",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-112",
};

pub const UG_113: Subdivision = Subdivision {
    name: "Wakiso",
    code: "UG-113",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-113",
};

pub const UG_114: Subdivision = Subdivision {
    name: "Mityana",
    code: "UG-114",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-114",
};

pub const UG_115: Subdivision = Subdivision {
    name: "Nakaseke",
    code: "UG-115",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-115",
};

pub const UG_116: Subdivision = Subdivision {
    name: "Lyantonde",
    code: "UG-116",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-116",
};

pub const UG_117: Subdivision = Subdivision {
    name: "Buikwe",
    code: "UG-117",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-117",
};

pub const UG_118: Subdivision = Subdivision {
    name: "Bukomansibi",
    code: "UG-118",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-118",
};

pub const UG_119: Subdivision = Subdivision {
    name: "Butambala",
    code: "UG-119",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-119",
};

pub const UG_120: Subdivision = Subdivision {
    name: "Buvuma",
    code: "UG-120",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-120",
};

pub const UG_121: Subdivision = Subdivision {
    name: "Gomba",
    code: "UG-121",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-121",
};

pub const UG_122: Subdivision = Subdivision {
    name: "Kalungu",
    code: "UG-122",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-122",
};

pub const UG_123: Subdivision = Subdivision {
    name: "Kyankwanzi",
    code: "UG-123",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-123",
};

pub const UG_124: Subdivision = Subdivision {
    name: "Lwengo",
    code: "UG-124",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-124",
};

pub const UG_201: Subdivision = Subdivision {
    name: "Bugiri",
    code: "UG-201",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-201",
};

pub const UG_202: Subdivision = Subdivision {
    name: "Busia",
    code: "UG-202",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-202",
};

pub const UG_203: Subdivision = Subdivision {
    name: "Iganga",
    code: "UG-203",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-203",
};

pub const UG_204: Subdivision = Subdivision {
    name: "Jinja",
    code: "UG-204",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-204",
};

pub const UG_205: Subdivision = Subdivision {
    name: "Kamuli",
    code: "UG-205",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-205",
};

pub const UG_206: Subdivision = Subdivision {
    name: "Kapchorwa",
    code: "UG-206",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-206",
};

pub const UG_207: Subdivision = Subdivision {
    name: "Katakwi",
    code: "UG-207",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-207",
};

pub const UG_208: Subdivision = Subdivision {
    name: "Kumi",
    code: "UG-208",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-208",
};

pub const UG_209: Subdivision = Subdivision {
    name: "Mbale",
    code: "UG-209",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-209",
};

pub const UG_210: Subdivision = Subdivision {
    name: "Pallisa",
    code: "UG-210",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-210",
};

pub const UG_211: Subdivision = Subdivision {
    name: "Soroti",
    code: "UG-211",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-211",
};

pub const UG_212: Subdivision = Subdivision {
    name: "Tororo",
    code: "UG-212",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-212",
};

pub const UG_213: Subdivision = Subdivision {
    name: "Kaberamaido",
    code: "UG-213",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-213",
};

pub const UG_214: Subdivision = Subdivision {
    name: "Mayuge",
    code: "UG-214",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-214",
};

pub const UG_215: Subdivision = Subdivision {
    name: "Sironko",
    code: "UG-215",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-215",
};

pub const UG_216: Subdivision = Subdivision {
    name: "Amuria",
    code: "UG-216",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-216",
};

pub const UG_217: Subdivision = Subdivision {
    name: "Budaka",
    code: "UG-217",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-217",
};

pub const UG_218: Subdivision = Subdivision {
    name: "Bukwa",
    code: "UG-218",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-218",
};

pub const UG_219: Subdivision = Subdivision {
    name: "Butaleja",
    code: "UG-219",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-219",
};

pub const UG_220: Subdivision = Subdivision {
    name: "Kaliro",
    code: "UG-220",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-220",
};

pub const UG_221: Subdivision = Subdivision {
    name: "Manafwa",
    code: "UG-221",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-221",
};

pub const UG_222: Subdivision = Subdivision {
    name: "Namutumba",
    code: "UG-222",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-222",
};

pub const UG_223: Subdivision = Subdivision {
    name: "Bududa",
    code: "UG-223",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-223",
};

pub const UG_224: Subdivision = Subdivision {
    name: "Bukedea",
    code: "UG-224",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-224",
};

pub const UG_225: Subdivision = Subdivision {
    name: "Bulambuli",
    code: "UG-225",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-225",
};

pub const UG_226: Subdivision = Subdivision {
    name: "Buyende",
    code: "UG-226",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-226",
};

pub const UG_227: Subdivision = Subdivision {
    name: "Kibuku",
    code: "UG-227",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-227",
};

pub const UG_228: Subdivision = Subdivision {
    name: "Kween",
    code: "UG-228",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-228",
};

pub const UG_229: Subdivision = Subdivision {
    name: "Luuka",
    code: "UG-229",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-229",
};

pub const UG_230: Subdivision = Subdivision {
    name: "Namayingo",
    code: "UG-230",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-230",
};

pub const UG_231: Subdivision = Subdivision {
    name: "Ngora",
    code: "UG-231",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-231",
};

pub const UG_232: Subdivision = Subdivision {
    name: "Serere",
    code: "UG-232",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-232",
};

pub const UG_301: Subdivision = Subdivision {
    name: "Adjumani",
    code: "UG-301",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-301",
};

pub const UG_302: Subdivision = Subdivision {
    name: "Apac",
    code: "UG-302",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-302",
};

pub const UG_303: Subdivision = Subdivision {
    name: "Arua",
    code: "UG-303",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-303",
};

pub const UG_304: Subdivision = Subdivision {
    name: "Gulu",
    code: "UG-304",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-304",
};

pub const UG_305: Subdivision = Subdivision {
    name: "Kitgum",
    code: "UG-305",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-305",
};

pub const UG_306: Subdivision = Subdivision {
    name: "Kotido",
    code: "UG-306",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-306",
};

pub const UG_307: Subdivision = Subdivision {
    name: "Lira",
    code: "UG-307",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-307",
};

pub const UG_308: Subdivision = Subdivision {
    name: "Moroto",
    code: "UG-308",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-308",
};

pub const UG_309: Subdivision = Subdivision {
    name: "Moyo",
    code: "UG-309",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-309",
};

pub const UG_310: Subdivision = Subdivision {
    name: "Nebbi",
    code: "UG-310",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-310",
};

pub const UG_311: Subdivision = Subdivision {
    name: "Nakapiripirit",
    code: "UG-311",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-311",
};

pub const UG_312: Subdivision = Subdivision {
    name: "Pader",
    code: "UG-312",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-312",
};

pub const UG_313: Subdivision = Subdivision {
    name: "Yumbe",
    code: "UG-313",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-313",
};

pub const UG_314: Subdivision = Subdivision {
    name: "Amolatar",
    code: "UG-314",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-314",
};

pub const UG_315: Subdivision = Subdivision {
    name: "Kaabong",
    code: "UG-315",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-315",
};

pub const UG_316: Subdivision = Subdivision {
    name: "Koboko",
    code: "UG-316",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-316",
};

pub const UG_317: Subdivision = Subdivision {
    name: "Abim",
    code: "UG-317",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-317",
};

pub const UG_318: Subdivision = Subdivision {
    name: "Dokolo",
    code: "UG-318",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-318",
};

pub const UG_319: Subdivision = Subdivision {
    name: "Amuru",
    code: "UG-319",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-319",
};

pub const UG_320: Subdivision = Subdivision {
    name: "Maracha",
    code: "UG-320",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-320",
};

pub const UG_321: Subdivision = Subdivision {
    name: "Oyam",
    code: "UG-321",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-321",
};

pub const UG_322: Subdivision = Subdivision {
    name: "Agago",
    code: "UG-322",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-322",
};

pub const UG_323: Subdivision = Subdivision {
    name: "Alebtong",
    code: "UG-323",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-323",
};

pub const UG_324: Subdivision = Subdivision {
    name: "Amudat",
    code: "UG-324",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-324",
};

pub const UG_325: Subdivision = Subdivision {
    name: "Buhweju",
    code: "UG-325",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-325",
};

pub const UG_326: Subdivision = Subdivision {
    name: "Kole",
    code: "UG-326",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-326",
};

pub const UG_327: Subdivision = Subdivision {
    name: "Lamwo",
    code: "UG-327",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-327",
};

pub const UG_328: Subdivision = Subdivision {
    name: "Napak",
    code: "UG-328",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-328",
};

pub const UG_329: Subdivision = Subdivision {
    name: "Nwoya",
    code: "UG-329",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-329",
};

pub const UG_330: Subdivision = Subdivision {
    name: "Otuke",
    code: "UG-330",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-330",
};

pub const UG_331: Subdivision = Subdivision {
    name: "Zombo",
    code: "UG-331",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-331",
};

pub const UG_401: Subdivision = Subdivision {
    name: "Bundibugyo",
    code: "UG-401",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-401",
};

pub const UG_402: Subdivision = Subdivision {
    name: "Bushenyi",
    code: "UG-402",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-402",
};

pub const UG_403: Subdivision = Subdivision {
    name: "Hoima",
    code: "UG-403",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-403",
};

pub const UG_404: Subdivision = Subdivision {
    name: "Kabale",
    code: "UG-404",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-404",
};

pub const UG_405: Subdivision = Subdivision {
    name: "Kabarole",
    code: "UG-405",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-405",
};

pub const UG_406: Subdivision = Subdivision {
    name: "Kasese",
    code: "UG-406",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-406",
};

pub const UG_407: Subdivision = Subdivision {
    name: "Kibaale",
    code: "UG-407",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-407",
};

pub const UG_408: Subdivision = Subdivision {
    name: "Kisoro",
    code: "UG-408",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-408",
};

pub const UG_409: Subdivision = Subdivision {
    name: "Masindi",
    code: "UG-409",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-409",
};

pub const UG_410: Subdivision = Subdivision {
    name: "Mbarara",
    code: "UG-410",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-410",
};

pub const UG_411: Subdivision = Subdivision {
    name: "Ntungamo",
    code: "UG-411",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-411",
};

pub const UG_412: Subdivision = Subdivision {
    name: "Rukungiri",
    code: "UG-412",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-412",
};

pub const UG_413: Subdivision = Subdivision {
    name: "Kamwenge",
    code: "UG-413",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-413",
};

pub const UG_414: Subdivision = Subdivision {
    name: "Kanungu",
    code: "UG-414",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-414",
};

pub const UG_415: Subdivision = Subdivision {
    name: "Kyenjojo",
    code: "UG-415",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-415",
};

pub const UG_416: Subdivision = Subdivision {
    name: "Ibanda",
    code: "UG-416",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-416",
};

pub const UG_417: Subdivision = Subdivision {
    name: "Isingiro",
    code: "UG-417",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-417",
};

pub const UG_418: Subdivision = Subdivision {
    name: "Kiruhura",
    code: "UG-418",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-418",
};

pub const UG_419: Subdivision = Subdivision {
    name: "Buliisa",
    code: "UG-419",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-419",
};

pub const UG_420: Subdivision = Subdivision {
    name: "Kiryandongo",
    code: "UG-420",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-420",
};

pub const UG_421: Subdivision = Subdivision {
    name: "Kyegegwa",
    code: "UG-421",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-421",
};

pub const UG_422: Subdivision = Subdivision {
    name: "Mitooma",
    code: "UG-422",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-422",
};

pub const UG_423: Subdivision = Subdivision {
    name: "Ntoroko",
    code: "UG-423",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-423",
};

pub const UG_424: Subdivision = Subdivision {
    name: "Rubirizi",
    code: "UG-424",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-424",
};

pub const UG_425: Subdivision = Subdivision {
    name: "Sheema",
    code: "UG-425",
    subdivision_type: "District",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-425",
};

pub const UG_C: Subdivision = Subdivision {
    name: "Central",
    code: "UG-C",
    subdivision_type: "Geographical region",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-C",
};

pub const UG_E: Subdivision = Subdivision {
    name: "Eastern",
    code: "UG-E",
    subdivision_type: "Geographical region",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-E",
};

pub const UG_N: Subdivision = Subdivision {
    name: "Northern",
    code: "UG-N",
    subdivision_type: "Geographical region",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-N",
};

pub const UG_W: Subdivision = Subdivision {
    name: "Western",
    code: "UG-W",
    subdivision_type: "Geographical region",
    country_name: "Uganda",
    country_code: "UG",
    region_code: "UG-W",
};

pub const UA_05: Subdivision = Subdivision {
    name: "Vinnytska oblast",
    code: "UA-05",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-05",
};

pub const UA_07: Subdivision = Subdivision {
    name: "Volynska oblast",
    code: "UA-07",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-07",
};

pub const UA_09: Subdivision = Subdivision {
    name: "Luhanska oblast",
    code: "UA-09",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-09",
};

pub const UA_12: Subdivision = Subdivision {
    name: "Dnipropetrovska oblast",
    code: "UA-12",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-12",
};

pub const UA_14: Subdivision = Subdivision {
    name: "Donetska oblast",
    code: "UA-14",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-14",
};

pub const UA_18: Subdivision = Subdivision {
    name: "Zhytomyrska oblast",
    code: "UA-18",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-18",
};

pub const UA_21: Subdivision = Subdivision {
    name: "Zakarpatska oblast",
    code: "UA-21",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-21",
};

pub const UA_23: Subdivision = Subdivision {
    name: "Zaporizka oblast",
    code: "UA-23",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-23",
};

pub const UA_26: Subdivision = Subdivision {
    name: "Ivano-Frankivska oblast",
    code: "UA-26",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-26",
};

pub const UA_30: Subdivision = Subdivision {
    name: "Kyiv",
    code: "UA-30",
    subdivision_type: "City",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-30",
};

pub const UA_32: Subdivision = Subdivision {
    name: "Kyivska oblast",
    code: "UA-32",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-32",
};

pub const UA_35: Subdivision = Subdivision {
    name: "Kirovohradska oblast",
    code: "UA-35",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-35",
};

pub const UA_40: Subdivision = Subdivision {
    name: "Sevastopol",
    code: "UA-40",
    subdivision_type: "City",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-40",
};

pub const UA_43: Subdivision = Subdivision {
    name: "Avtonomna Respublika Krym",
    code: "UA-43",
    subdivision_type: "Republic",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-43",
};

pub const UA_46: Subdivision = Subdivision {
    name: "Lvivska oblast",
    code: "UA-46",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-46",
};

pub const UA_48: Subdivision = Subdivision {
    name: "Mykolaivska oblast",
    code: "UA-48",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-48",
};

pub const UA_51: Subdivision = Subdivision {
    name: "Odeska oblast",
    code: "UA-51",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-51",
};

pub const UA_53: Subdivision = Subdivision {
    name: "Poltavska oblast",
    code: "UA-53",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-53",
};

pub const UA_56: Subdivision = Subdivision {
    name: "Rivnenska oblast",
    code: "UA-56",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-56",
};

pub const UA_59: Subdivision = Subdivision {
    name: "Sumska oblast",
    code: "UA-59",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-59",
};

pub const UA_61: Subdivision = Subdivision {
    name: "Ternopilska oblast",
    code: "UA-61",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-61",
};

pub const UA_63: Subdivision = Subdivision {
    name: "Kharkivska oblast",
    code: "UA-63",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-63",
};

pub const UA_65: Subdivision = Subdivision {
    name: "Khersonska oblast",
    code: "UA-65",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-65",
};

pub const UA_68: Subdivision = Subdivision {
    name: "Khmelnytska oblast",
    code: "UA-68",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-68",
};

pub const UA_71: Subdivision = Subdivision {
    name: "Cherkaska oblast",
    code: "UA-71",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-71",
};

pub const UA_74: Subdivision = Subdivision {
    name: "Chernihivska oblast",
    code: "UA-74",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-74",
};

pub const UA_77: Subdivision = Subdivision {
    name: "Chernivetska oblast",
    code: "UA-77",
    subdivision_type: "Region",
    country_name: "Ukraine",
    country_code: "UA",
    region_code: "UA-77",
};

pub const AE_AJ: Subdivision = Subdivision {
    name: "'Ajmān",
    code: "AE-AJ",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-AJ",
};

pub const AE_AZ: Subdivision = Subdivision {
    name: "Abū Z̧aby",
    code: "AE-AZ",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-AZ",
};

pub const AE_DU: Subdivision = Subdivision {
    name: "Dubayy",
    code: "AE-DU",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-DU",
};

pub const AE_FU: Subdivision = Subdivision {
    name: "Al Fujayrah",
    code: "AE-FU",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-FU",
};

pub const AE_RK: Subdivision = Subdivision {
    name: "Ra's al Khaymah",
    code: "AE-RK",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-RK",
};

pub const AE_SH: Subdivision = Subdivision {
    name: "Ash Shāriqah",
    code: "AE-SH",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-SH",
};

pub const AE_UQ: Subdivision = Subdivision {
    name: "Umm al Qaywayn",
    code: "AE-UQ",
    subdivision_type: "Emirate",
    country_name: "United Arab Emirates",
    country_code: "AE",
    region_code: "AE-UQ",
};

pub const GB_ABD: Subdivision = Subdivision {
    name: "Aberdeenshire",
    code: "GB-ABD",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ABD",
};

pub const GB_ABE: Subdivision = Subdivision {
    name: "Aberdeen City",
    code: "GB-ABE",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ABE",
};

pub const GB_AGB: Subdivision = Subdivision {
    name: "Argyll and Bute",
    code: "GB-AGB",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-AGB",
};

pub const GB_AGY: Subdivision = Subdivision {
    name: "Isle of Anglesey",
    code: "GB-AGY",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-AGY",
};

pub const GB_ANS: Subdivision = Subdivision {
    name: "Angus",
    code: "GB-ANS",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ANS",
};

pub const GB_ANT: Subdivision = Subdivision {
    name: "Antrim",
    code: "GB-ANT",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ANT",
};

pub const GB_ARD: Subdivision = Subdivision {
    name: "Ards",
    code: "GB-ARD",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ARD",
};

pub const GB_ARM: Subdivision = Subdivision {
    name: "Armagh",
    code: "GB-ARM",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ARM",
};

pub const GB_BAS: Subdivision = Subdivision {
    name: "Bath and North East Somerset",
    code: "GB-BAS",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BAS",
};

pub const GB_BBD: Subdivision = Subdivision {
    name: "Blackburn with Darwen",
    code: "GB-BBD",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BBD",
};

pub const GB_BDF: Subdivision = Subdivision {
    name: "Bedford",
    code: "GB-BDF",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BDF",
};

pub const GB_BDG: Subdivision = Subdivision {
    name: "Barking and Dagenham",
    code: "GB-BDG",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BDG",
};

pub const GB_BEN: Subdivision = Subdivision {
    name: "Brent",
    code: "GB-BEN",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BEN",
};

pub const GB_BEX: Subdivision = Subdivision {
    name: "Bexley",
    code: "GB-BEX",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BEX",
};

pub const GB_BFS: Subdivision = Subdivision {
    name: "Belfast",
    code: "GB-BFS",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BFS",
};

pub const GB_BGE: Subdivision = Subdivision {
    name: "Bridgend",
    code: "GB-BGE",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BGE",
};

pub const GB_BGW: Subdivision = Subdivision {
    name: "Blaenau Gwent",
    code: "GB-BGW",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BGW",
};

pub const GB_BIR: Subdivision = Subdivision {
    name: "Birmingham",
    code: "GB-BIR",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BIR",
};

pub const GB_BKM: Subdivision = Subdivision {
    name: "Buckinghamshire",
    code: "GB-BKM",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BKM",
};

pub const GB_BLA: Subdivision = Subdivision {
    name: "Ballymena",
    code: "GB-BLA",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BLA",
};

pub const GB_BLY: Subdivision = Subdivision {
    name: "Ballymoney",
    code: "GB-BLY",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BLY",
};

pub const GB_BMH: Subdivision = Subdivision {
    name: "Bournemouth",
    code: "GB-BMH",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BMH",
};

pub const GB_BNB: Subdivision = Subdivision {
    name: "Banbridge",
    code: "GB-BNB",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BNB",
};

pub const GB_BNE: Subdivision = Subdivision {
    name: "Barnet",
    code: "GB-BNE",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BNE",
};

pub const GB_BNH: Subdivision = Subdivision {
    name: "Brighton and Hove",
    code: "GB-BNH",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BNH",
};

pub const GB_BNS: Subdivision = Subdivision {
    name: "Barnsley",
    code: "GB-BNS",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BNS",
};

pub const GB_BOL: Subdivision = Subdivision {
    name: "Bolton",
    code: "GB-BOL",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BOL",
};

pub const GB_BPL: Subdivision = Subdivision {
    name: "Blackpool",
    code: "GB-BPL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BPL",
};

pub const GB_BRC: Subdivision = Subdivision {
    name: "Bracknell Forest",
    code: "GB-BRC",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BRC",
};

pub const GB_BRD: Subdivision = Subdivision {
    name: "Bradford",
    code: "GB-BRD",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BRD",
};

pub const GB_BRY: Subdivision = Subdivision {
    name: "Bromley",
    code: "GB-BRY",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BRY",
};

pub const GB_BST: Subdivision = Subdivision {
    name: "Bristol, City of",
    code: "GB-BST",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BST",
};

pub const GB_BUR: Subdivision = Subdivision {
    name: "Bury",
    code: "GB-BUR",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-BUR",
};

pub const GB_CAM: Subdivision = Subdivision {
    name: "Cambridgeshire",
    code: "GB-CAM",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CAM",
};

pub const GB_CAY: Subdivision = Subdivision {
    name: "Caerphilly",
    code: "GB-CAY",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CAY",
};

pub const GB_CBF: Subdivision = Subdivision {
    name: "Central Bedfordshire",
    code: "GB-CBF",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CBF",
};

pub const GB_CGN: Subdivision = Subdivision {
    name: "Ceredigion",
    code: "GB-CGN",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CGN",
};

pub const GB_CGV: Subdivision = Subdivision {
    name: "Craigavon",
    code: "GB-CGV",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CGV",
};

pub const GB_CHE: Subdivision = Subdivision {
    name: "Cheshire East",
    code: "GB-CHE",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CHE",
};

pub const GB_CHW: Subdivision = Subdivision {
    name: "Cheshire West and Chester",
    code: "GB-CHW",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CHW",
};

pub const GB_CKF: Subdivision = Subdivision {
    name: "Carrickfergus",
    code: "GB-CKF",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CKF",
};

pub const GB_CKT: Subdivision = Subdivision {
    name: "Cookstown",
    code: "GB-CKT",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CKT",
};

pub const GB_CLD: Subdivision = Subdivision {
    name: "Calderdale",
    code: "GB-CLD",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CLD",
};

pub const GB_CLK: Subdivision = Subdivision {
    name: "Clackmannanshire",
    code: "GB-CLK",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CLK",
};

pub const GB_CLR: Subdivision = Subdivision {
    name: "Coleraine",
    code: "GB-CLR",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CLR",
};

pub const GB_CMA: Subdivision = Subdivision {
    name: "Cumbria",
    code: "GB-CMA",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CMA",
};

pub const GB_CMD: Subdivision = Subdivision {
    name: "Camden",
    code: "GB-CMD",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CMD",
};

pub const GB_CMN: Subdivision = Subdivision {
    name: "Carmarthenshire",
    code: "GB-CMN",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CMN",
};

pub const GB_CON: Subdivision = Subdivision {
    name: "Cornwall",
    code: "GB-CON",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CON",
};

pub const GB_COV: Subdivision = Subdivision {
    name: "Coventry",
    code: "GB-COV",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-COV",
};

pub const GB_CRF: Subdivision = Subdivision {
    name: "Cardiff",
    code: "GB-CRF",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CRF",
};

pub const GB_CRY: Subdivision = Subdivision {
    name: "Croydon",
    code: "GB-CRY",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CRY",
};

pub const GB_CSR: Subdivision = Subdivision {
    name: "Castlereagh",
    code: "GB-CSR",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CSR",
};

pub const GB_CWY: Subdivision = Subdivision {
    name: "Conwy",
    code: "GB-CWY",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-CWY",
};

pub const GB_DAL: Subdivision = Subdivision {
    name: "Darlington",
    code: "GB-DAL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DAL",
};

pub const GB_DBY: Subdivision = Subdivision {
    name: "Derbyshire",
    code: "GB-DBY",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DBY",
};

pub const GB_DEN: Subdivision = Subdivision {
    name: "Denbighshire",
    code: "GB-DEN",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DEN",
};

pub const GB_DER: Subdivision = Subdivision {
    name: "Derby",
    code: "GB-DER",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DER",
};

pub const GB_DEV: Subdivision = Subdivision {
    name: "Devon",
    code: "GB-DEV",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DEV",
};

pub const GB_DGN: Subdivision = Subdivision {
    name: "Dungannon and South Tyrone",
    code: "GB-DGN",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DGN",
};

pub const GB_DGY: Subdivision = Subdivision {
    name: "Dumfries and Galloway",
    code: "GB-DGY",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DGY",
};

pub const GB_DNC: Subdivision = Subdivision {
    name: "Doncaster",
    code: "GB-DNC",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DNC",
};

pub const GB_DND: Subdivision = Subdivision {
    name: "Dundee City",
    code: "GB-DND",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DND",
};

pub const GB_DOR: Subdivision = Subdivision {
    name: "Dorset",
    code: "GB-DOR",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DOR",
};

pub const GB_DOW: Subdivision = Subdivision {
    name: "Down",
    code: "GB-DOW",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DOW",
};

pub const GB_DRY: Subdivision = Subdivision {
    name: "Derry",
    code: "GB-DRY",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DRY",
};

pub const GB_DUD: Subdivision = Subdivision {
    name: "Dudley",
    code: "GB-DUD",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DUD",
};

pub const GB_DUR: Subdivision = Subdivision {
    name: "Durham County",
    code: "GB-DUR",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-DUR",
};

pub const GB_EAL: Subdivision = Subdivision {
    name: "Ealing",
    code: "GB-EAL",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-EAL",
};

pub const GB_EAW: Subdivision = Subdivision {
    name: "England and Wales",
    code: "GB-EAW",
    subdivision_type: "Nation",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-EAW",
};

pub const GB_EAY: Subdivision = Subdivision {
    name: "East Ayrshire",
    code: "GB-EAY",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-EAY",
};

pub const GB_EDH: Subdivision = Subdivision {
    name: "Edinburgh, City of",
    code: "GB-EDH",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-EDH",
};

pub const GB_EDU: Subdivision = Subdivision {
    name: "East Dunbartonshire",
    code: "GB-EDU",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-EDU",
};

pub const GB_ELN: Subdivision = Subdivision {
    name: "East Lothian",
    code: "GB-ELN",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ELN",
};

pub const GB_ELS: Subdivision = Subdivision {
    name: "Eilean Siar",
    code: "GB-ELS",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ELS",
};

pub const GB_ENF: Subdivision = Subdivision {
    name: "Enfield",
    code: "GB-ENF",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ENF",
};

pub const GB_ENG: Subdivision = Subdivision {
    name: "England",
    code: "GB-ENG",
    subdivision_type: "Country",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ENG",
};

pub const GB_ERW: Subdivision = Subdivision {
    name: "East Renfrewshire",
    code: "GB-ERW",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ERW",
};

pub const GB_ERY: Subdivision = Subdivision {
    name: "East Riding of Yorkshire",
    code: "GB-ERY",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ERY",
};

pub const GB_ESS: Subdivision = Subdivision {
    name: "Essex",
    code: "GB-ESS",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ESS",
};

pub const GB_ESX: Subdivision = Subdivision {
    name: "East Sussex",
    code: "GB-ESX",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ESX",
};

pub const GB_FAL: Subdivision = Subdivision {
    name: "Falkirk",
    code: "GB-FAL",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-FAL",
};

pub const GB_FER: Subdivision = Subdivision {
    name: "Fermanagh",
    code: "GB-FER",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-FER",
};

pub const GB_FIF: Subdivision = Subdivision {
    name: "Fife",
    code: "GB-FIF",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-FIF",
};

pub const GB_FLN: Subdivision = Subdivision {
    name: "Flintshire",
    code: "GB-FLN",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-FLN",
};

pub const GB_GAT: Subdivision = Subdivision {
    name: "Gateshead",
    code: "GB-GAT",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-GAT",
};

pub const GB_GBN: Subdivision = Subdivision {
    name: "Great Britain",
    code: "GB-GBN",
    subdivision_type: "Nation",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-GBN",
};

pub const GB_GLG: Subdivision = Subdivision {
    name: "Glasgow City",
    code: "GB-GLG",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-GLG",
};

pub const GB_GLS: Subdivision = Subdivision {
    name: "Gloucestershire",
    code: "GB-GLS",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-GLS",
};

pub const GB_GRE: Subdivision = Subdivision {
    name: "Greenwich",
    code: "GB-GRE",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-GRE",
};

pub const GB_GWN: Subdivision = Subdivision {
    name: "Gwynedd",
    code: "GB-GWN",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-GWN",
};

pub const GB_HAL: Subdivision = Subdivision {
    name: "Halton",
    code: "GB-HAL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HAL",
};

pub const GB_HAM: Subdivision = Subdivision {
    name: "Hampshire",
    code: "GB-HAM",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HAM",
};

pub const GB_HAV: Subdivision = Subdivision {
    name: "Havering",
    code: "GB-HAV",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HAV",
};

pub const GB_HCK: Subdivision = Subdivision {
    name: "Hackney",
    code: "GB-HCK",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HCK",
};

pub const GB_HEF: Subdivision = Subdivision {
    name: "Herefordshire",
    code: "GB-HEF",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HEF",
};

pub const GB_HIL: Subdivision = Subdivision {
    name: "Hillingdon",
    code: "GB-HIL",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HIL",
};

pub const GB_HLD: Subdivision = Subdivision {
    name: "Highland",
    code: "GB-HLD",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HLD",
};

pub const GB_HMF: Subdivision = Subdivision {
    name: "Hammersmith and Fulham",
    code: "GB-HMF",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HMF",
};

pub const GB_HNS: Subdivision = Subdivision {
    name: "Hounslow",
    code: "GB-HNS",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HNS",
};

pub const GB_HPL: Subdivision = Subdivision {
    name: "Hartlepool",
    code: "GB-HPL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HPL",
};

pub const GB_HRT: Subdivision = Subdivision {
    name: "Hertfordshire",
    code: "GB-HRT",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HRT",
};

pub const GB_HRW: Subdivision = Subdivision {
    name: "Harrow",
    code: "GB-HRW",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HRW",
};

pub const GB_HRY: Subdivision = Subdivision {
    name: "Haringey",
    code: "GB-HRY",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-HRY",
};

pub const GB_IOS: Subdivision = Subdivision {
    name: "Isles of Scilly",
    code: "GB-IOS",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-IOS",
};

pub const GB_IOW: Subdivision = Subdivision {
    name: "Isle of Wight",
    code: "GB-IOW",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-IOW",
};

pub const GB_ISL: Subdivision = Subdivision {
    name: "Islington",
    code: "GB-ISL",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ISL",
};

pub const GB_IVC: Subdivision = Subdivision {
    name: "Inverclyde",
    code: "GB-IVC",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-IVC",
};

pub const GB_KEC: Subdivision = Subdivision {
    name: "Kensington and Chelsea",
    code: "GB-KEC",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-KEC",
};

pub const GB_KEN: Subdivision = Subdivision {
    name: "Kent",
    code: "GB-KEN",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-KEN",
};

pub const GB_KHL: Subdivision = Subdivision {
    name: "Kingston upon Hull",
    code: "GB-KHL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-KHL",
};

pub const GB_KIR: Subdivision = Subdivision {
    name: "Kirklees",
    code: "GB-KIR",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-KIR",
};

pub const GB_KTT: Subdivision = Subdivision {
    name: "Kingston upon Thames",
    code: "GB-KTT",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-KTT",
};

pub const GB_KWL: Subdivision = Subdivision {
    name: "Knowsley",
    code: "GB-KWL",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-KWL",
};

pub const GB_LAN: Subdivision = Subdivision {
    name: "Lancashire",
    code: "GB-LAN",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LAN",
};

pub const GB_LBH: Subdivision = Subdivision {
    name: "Lambeth",
    code: "GB-LBH",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LBH",
};

pub const GB_LCE: Subdivision = Subdivision {
    name: "Leicester",
    code: "GB-LCE",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LCE",
};

pub const GB_LDS: Subdivision = Subdivision {
    name: "Leeds",
    code: "GB-LDS",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LDS",
};

pub const GB_LEC: Subdivision = Subdivision {
    name: "Leicestershire",
    code: "GB-LEC",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LEC",
};

pub const GB_LEW: Subdivision = Subdivision {
    name: "Lewisham",
    code: "GB-LEW",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LEW",
};

pub const GB_LIN: Subdivision = Subdivision {
    name: "Lincolnshire",
    code: "GB-LIN",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LIN",
};

pub const GB_LIV: Subdivision = Subdivision {
    name: "Liverpool",
    code: "GB-LIV",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LIV",
};

pub const GB_LMV: Subdivision = Subdivision {
    name: "Limavady",
    code: "GB-LMV",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LMV",
};

pub const GB_LND: Subdivision = Subdivision {
    name: "London, City of",
    code: "GB-LND",
    subdivision_type: "City corporation",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LND",
};

pub const GB_LRN: Subdivision = Subdivision {
    name: "Larne",
    code: "GB-LRN",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LRN",
};

pub const GB_LSB: Subdivision = Subdivision {
    name: "Lisburn",
    code: "GB-LSB",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LSB",
};

pub const GB_LUT: Subdivision = Subdivision {
    name: "Luton",
    code: "GB-LUT",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-LUT",
};

pub const GB_MAN: Subdivision = Subdivision {
    name: "Manchester",
    code: "GB-MAN",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MAN",
};

pub const GB_MDB: Subdivision = Subdivision {
    name: "Middlesbrough",
    code: "GB-MDB",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MDB",
};

pub const GB_MDW: Subdivision = Subdivision {
    name: "Medway",
    code: "GB-MDW",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MDW",
};

pub const GB_MFT: Subdivision = Subdivision {
    name: "Magherafelt",
    code: "GB-MFT",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MFT",
};

pub const GB_MIK: Subdivision = Subdivision {
    name: "Milton Keynes",
    code: "GB-MIK",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MIK",
};

pub const GB_MLN: Subdivision = Subdivision {
    name: "Midlothian",
    code: "GB-MLN",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MLN",
};

pub const GB_MON: Subdivision = Subdivision {
    name: "Monmouthshire",
    code: "GB-MON",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MON",
};

pub const GB_MRT: Subdivision = Subdivision {
    name: "Merton",
    code: "GB-MRT",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MRT",
};

pub const GB_MRY: Subdivision = Subdivision {
    name: "Moray",
    code: "GB-MRY",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MRY",
};

pub const GB_MTY: Subdivision = Subdivision {
    name: "Merthyr Tydfil",
    code: "GB-MTY",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MTY",
};

pub const GB_MYL: Subdivision = Subdivision {
    name: "Moyle",
    code: "GB-MYL",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-MYL",
};

pub const GB_NAY: Subdivision = Subdivision {
    name: "North Ayrshire",
    code: "GB-NAY",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NAY",
};

pub const GB_NBL: Subdivision = Subdivision {
    name: "Northumberland",
    code: "GB-NBL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NBL",
};

pub const GB_NDN: Subdivision = Subdivision {
    name: "North Down",
    code: "GB-NDN",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NDN",
};

pub const GB_NEL: Subdivision = Subdivision {
    name: "North East Lincolnshire",
    code: "GB-NEL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NEL",
};

pub const GB_NET: Subdivision = Subdivision {
    name: "Newcastle upon Tyne",
    code: "GB-NET",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NET",
};

pub const GB_NFK: Subdivision = Subdivision {
    name: "Norfolk",
    code: "GB-NFK",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NFK",
};

pub const GB_NGM: Subdivision = Subdivision {
    name: "Nottingham",
    code: "GB-NGM",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NGM",
};

pub const GB_NIR: Subdivision = Subdivision {
    name: "Northern Ireland",
    code: "GB-NIR",
    subdivision_type: "Province",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NIR",
};

pub const GB_NLK: Subdivision = Subdivision {
    name: "North Lanarkshire",
    code: "GB-NLK",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NLK",
};

pub const GB_NLN: Subdivision = Subdivision {
    name: "North Lincolnshire",
    code: "GB-NLN",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NLN",
};

pub const GB_NSM: Subdivision = Subdivision {
    name: "North Somerset",
    code: "GB-NSM",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NSM",
};

pub const GB_NTA: Subdivision = Subdivision {
    name: "Newtownabbey",
    code: "GB-NTA",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NTA",
};

pub const GB_NTH: Subdivision = Subdivision {
    name: "Northamptonshire",
    code: "GB-NTH",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NTH",
};

pub const GB_NTL: Subdivision = Subdivision {
    name: "Neath Port Talbot",
    code: "GB-NTL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NTL",
};

pub const GB_NTT: Subdivision = Subdivision {
    name: "Nottinghamshire",
    code: "GB-NTT",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NTT",
};

pub const GB_NTY: Subdivision = Subdivision {
    name: "North Tyneside",
    code: "GB-NTY",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NTY",
};

pub const GB_NWM: Subdivision = Subdivision {
    name: "Newham",
    code: "GB-NWM",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NWM",
};

pub const GB_NWP: Subdivision = Subdivision {
    name: "Newport",
    code: "GB-NWP",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NWP",
};

pub const GB_NYK: Subdivision = Subdivision {
    name: "North Yorkshire",
    code: "GB-NYK",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NYK",
};

pub const GB_NYM: Subdivision = Subdivision {
    name: "Newry and Mourne",
    code: "GB-NYM",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-NYM",
};

pub const GB_OLD: Subdivision = Subdivision {
    name: "Oldham",
    code: "GB-OLD",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-OLD",
};

pub const GB_OMH: Subdivision = Subdivision {
    name: "Omagh",
    code: "GB-OMH",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-OMH",
};

pub const GB_ORK: Subdivision = Subdivision {
    name: "Orkney Islands",
    code: "GB-ORK",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ORK",
};

pub const GB_OXF: Subdivision = Subdivision {
    name: "Oxfordshire",
    code: "GB-OXF",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-OXF",
};

pub const GB_PEM: Subdivision = Subdivision {
    name: "Pembrokeshire",
    code: "GB-PEM",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-PEM",
};

pub const GB_PKN: Subdivision = Subdivision {
    name: "Perth and Kinross",
    code: "GB-PKN",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-PKN",
};

pub const GB_PLY: Subdivision = Subdivision {
    name: "Plymouth",
    code: "GB-PLY",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-PLY",
};

pub const GB_POL: Subdivision = Subdivision {
    name: "Poole",
    code: "GB-POL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-POL",
};

pub const GB_POR: Subdivision = Subdivision {
    name: "Portsmouth",
    code: "GB-POR",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-POR",
};

pub const GB_POW: Subdivision = Subdivision {
    name: "Powys",
    code: "GB-POW",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-POW",
};

pub const GB_PTE: Subdivision = Subdivision {
    name: "Peterborough",
    code: "GB-PTE",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-PTE",
};

pub const GB_RCC: Subdivision = Subdivision {
    name: "Redcar and Cleveland",
    code: "GB-RCC",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RCC",
};

pub const GB_RCH: Subdivision = Subdivision {
    name: "Rochdale",
    code: "GB-RCH",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RCH",
};

pub const GB_RCT: Subdivision = Subdivision {
    name: "Rhondda, Cynon, Taff",
    code: "GB-RCT",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RCT",
};

pub const GB_RDB: Subdivision = Subdivision {
    name: "Redbridge",
    code: "GB-RDB",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RDB",
};

pub const GB_RDG: Subdivision = Subdivision {
    name: "Reading",
    code: "GB-RDG",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RDG",
};

pub const GB_RFW: Subdivision = Subdivision {
    name: "Renfrewshire",
    code: "GB-RFW",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RFW",
};

pub const GB_RIC: Subdivision = Subdivision {
    name: "Richmond upon Thames",
    code: "GB-RIC",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RIC",
};

pub const GB_ROT: Subdivision = Subdivision {
    name: "Rotherham",
    code: "GB-ROT",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ROT",
};

pub const GB_RUT: Subdivision = Subdivision {
    name: "Rutland",
    code: "GB-RUT",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-RUT",
};

pub const GB_SAW: Subdivision = Subdivision {
    name: "Sandwell",
    code: "GB-SAW",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SAW",
};

pub const GB_SAY: Subdivision = Subdivision {
    name: "South Ayrshire",
    code: "GB-SAY",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SAY",
};

pub const GB_SCB: Subdivision = Subdivision {
    name: "Scottish Borders, The",
    code: "GB-SCB",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SCB",
};

pub const GB_SCT: Subdivision = Subdivision {
    name: "Scotland",
    code: "GB-SCT",
    subdivision_type: "Country",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SCT",
};

pub const GB_SFK: Subdivision = Subdivision {
    name: "Suffolk",
    code: "GB-SFK",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SFK",
};

pub const GB_SFT: Subdivision = Subdivision {
    name: "Sefton",
    code: "GB-SFT",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SFT",
};

pub const GB_SGC: Subdivision = Subdivision {
    name: "South Gloucestershire",
    code: "GB-SGC",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SGC",
};

pub const GB_SHF: Subdivision = Subdivision {
    name: "Sheffield",
    code: "GB-SHF",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SHF",
};

pub const GB_SHN: Subdivision = Subdivision {
    name: "St. Helens",
    code: "GB-SHN",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SHN",
};

pub const GB_SHR: Subdivision = Subdivision {
    name: "Shropshire",
    code: "GB-SHR",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SHR",
};

pub const GB_SKP: Subdivision = Subdivision {
    name: "Stockport",
    code: "GB-SKP",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SKP",
};

pub const GB_SLF: Subdivision = Subdivision {
    name: "Salford",
    code: "GB-SLF",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SLF",
};

pub const GB_SLG: Subdivision = Subdivision {
    name: "Slough",
    code: "GB-SLG",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SLG",
};

pub const GB_SLK: Subdivision = Subdivision {
    name: "South Lanarkshire",
    code: "GB-SLK",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SLK",
};

pub const GB_SND: Subdivision = Subdivision {
    name: "Sunderland",
    code: "GB-SND",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SND",
};

pub const GB_SOL: Subdivision = Subdivision {
    name: "Solihull",
    code: "GB-SOL",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SOL",
};

pub const GB_SOM: Subdivision = Subdivision {
    name: "Somerset",
    code: "GB-SOM",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SOM",
};

pub const GB_SOS: Subdivision = Subdivision {
    name: "Southend-on-Sea",
    code: "GB-SOS",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SOS",
};

pub const GB_SRY: Subdivision = Subdivision {
    name: "Surrey",
    code: "GB-SRY",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SRY",
};

pub const GB_STB: Subdivision = Subdivision {
    name: "Strabane",
    code: "GB-STB",
    subdivision_type: "District council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STB",
};

pub const GB_STE: Subdivision = Subdivision {
    name: "Stoke-on-Trent",
    code: "GB-STE",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STE",
};

pub const GB_STG: Subdivision = Subdivision {
    name: "Stirling",
    code: "GB-STG",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STG",
};

pub const GB_STH: Subdivision = Subdivision {
    name: "Southampton",
    code: "GB-STH",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STH",
};

pub const GB_STN: Subdivision = Subdivision {
    name: "Sutton",
    code: "GB-STN",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STN",
};

pub const GB_STS: Subdivision = Subdivision {
    name: "Staffordshire",
    code: "GB-STS",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STS",
};

pub const GB_STT: Subdivision = Subdivision {
    name: "Stockton-on-Tees",
    code: "GB-STT",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STT",
};

pub const GB_STY: Subdivision = Subdivision {
    name: "South Tyneside",
    code: "GB-STY",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-STY",
};

pub const GB_SWA: Subdivision = Subdivision {
    name: "Swansea",
    code: "GB-SWA",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SWA",
};

pub const GB_SWD: Subdivision = Subdivision {
    name: "Swindon",
    code: "GB-SWD",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SWD",
};

pub const GB_SWK: Subdivision = Subdivision {
    name: "Southwark",
    code: "GB-SWK",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-SWK",
};

pub const GB_TAM: Subdivision = Subdivision {
    name: "Tameside",
    code: "GB-TAM",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-TAM",
};

pub const GB_TFW: Subdivision = Subdivision {
    name: "Telford and Wrekin",
    code: "GB-TFW",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-TFW",
};

pub const GB_THR: Subdivision = Subdivision {
    name: "Thurrock",
    code: "GB-THR",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-THR",
};

pub const GB_TOB: Subdivision = Subdivision {
    name: "Torbay",
    code: "GB-TOB",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-TOB",
};

pub const GB_TOF: Subdivision = Subdivision {
    name: "Torfaen",
    code: "GB-TOF",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-TOF",
};

pub const GB_TRF: Subdivision = Subdivision {
    name: "Trafford",
    code: "GB-TRF",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-TRF",
};

pub const GB_TWH: Subdivision = Subdivision {
    name: "Tower Hamlets",
    code: "GB-TWH",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-TWH",
};

pub const GB_UKM: Subdivision = Subdivision {
    name: "United Kingdom",
    code: "GB-UKM",
    subdivision_type: "Nation",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-UKM",
};

pub const GB_VGL: Subdivision = Subdivision {
    name: "Vale of Glamorgan, The",
    code: "GB-VGL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-VGL",
};

pub const GB_WAR: Subdivision = Subdivision {
    name: "Warwickshire",
    code: "GB-WAR",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WAR",
};

pub const GB_WBK: Subdivision = Subdivision {
    name: "West Berkshire",
    code: "GB-WBK",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WBK",
};

pub const GB_WDU: Subdivision = Subdivision {
    name: "West Dunbartonshire",
    code: "GB-WDU",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WDU",
};

pub const GB_WFT: Subdivision = Subdivision {
    name: "Waltham Forest",
    code: "GB-WFT",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WFT",
};

pub const GB_WGN: Subdivision = Subdivision {
    name: "Wigan",
    code: "GB-WGN",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WGN",
};

pub const GB_WIL: Subdivision = Subdivision {
    name: "Wiltshire",
    code: "GB-WIL",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WIL",
};

pub const GB_WKF: Subdivision = Subdivision {
    name: "Wakefield",
    code: "GB-WKF",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WKF",
};

pub const GB_WLL: Subdivision = Subdivision {
    name: "Walsall",
    code: "GB-WLL",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WLL",
};

pub const GB_WLN: Subdivision = Subdivision {
    name: "West Lothian",
    code: "GB-WLN",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WLN",
};

pub const GB_WLS: Subdivision = Subdivision {
    name: "Wales",
    code: "GB-WLS",
    subdivision_type: "Country",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WLS",
};

pub const GB_WLV: Subdivision = Subdivision {
    name: "Wolverhampton",
    code: "GB-WLV",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WLV",
};

pub const GB_WND: Subdivision = Subdivision {
    name: "Wandsworth",
    code: "GB-WND",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WND",
};

pub const GB_WNM: Subdivision = Subdivision {
    name: "Windsor and Maidenhead",
    code: "GB-WNM",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WNM",
};

pub const GB_WOK: Subdivision = Subdivision {
    name: "Wokingham",
    code: "GB-WOK",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WOK",
};

pub const GB_WOR: Subdivision = Subdivision {
    name: "Worcestershire",
    code: "GB-WOR",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WOR",
};

pub const GB_WRL: Subdivision = Subdivision {
    name: "Wirral",
    code: "GB-WRL",
    subdivision_type: "Metropolitan district",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WRL",
};

pub const GB_WRT: Subdivision = Subdivision {
    name: "Warrington",
    code: "GB-WRT",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WRT",
};

pub const GB_WRX: Subdivision = Subdivision {
    name: "Wrexham",
    code: "GB-WRX",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WRX",
};

pub const GB_WSM: Subdivision = Subdivision {
    name: "Westminster",
    code: "GB-WSM",
    subdivision_type: "London borough",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WSM",
};

pub const GB_WSX: Subdivision = Subdivision {
    name: "West Sussex",
    code: "GB-WSX",
    subdivision_type: "Two-tier county",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-WSX",
};

pub const GB_YOR: Subdivision = Subdivision {
    name: "York",
    code: "GB-YOR",
    subdivision_type: "Unitary authority",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-YOR",
};

pub const GB_ZET: Subdivision = Subdivision {
    name: "Shetland Islands",
    code: "GB-ZET",
    subdivision_type: "Council area",
    country_name: "United Kingdom",
    country_code: "GB",
    region_code: "GB-ZET",
};

pub const UM_67: Subdivision = Subdivision {
    name: "Johnston Atoll",
    code: "UM-67",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-67",
};

pub const UM_71: Subdivision = Subdivision {
    name: "Midway Islands",
    code: "UM-71",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-71",
};

pub const UM_76: Subdivision = Subdivision {
    name: "Navassa Island",
    code: "UM-76",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-76",
};

pub const UM_79: Subdivision = Subdivision {
    name: "Wake Island",
    code: "UM-79",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-79",
};

pub const UM_81: Subdivision = Subdivision {
    name: "Baker Island",
    code: "UM-81",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-81",
};

pub const UM_84: Subdivision = Subdivision {
    name: "Howland Island",
    code: "UM-84",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-84",
};

pub const UM_86: Subdivision = Subdivision {
    name: "Jarvis Island",
    code: "UM-86",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-86",
};

pub const UM_89: Subdivision = Subdivision {
    name: "Kingman Reef",
    code: "UM-89",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-89",
};

pub const UM_95: Subdivision = Subdivision {
    name: "Palmyra Atoll",
    code: "UM-95",
    subdivision_type: "Islands, groups of islands",
    country_name: "United States Minor Outlying Islands",
    country_code: "UM",
    region_code: "UM-95",
};

pub const US_AK: Subdivision = Subdivision {
    name: "Alaska",
    code: "US-AK",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-AK",
};

pub const US_AL: Subdivision = Subdivision {
    name: "Alabama",
    code: "US-AL",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-AL",
};

pub const US_AR: Subdivision = Subdivision {
    name: "Arkansas",
    code: "US-AR",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-AR",
};

pub const US_AS: Subdivision = Subdivision {
    name: "American Samoa",
    code: "US-AS",
    subdivision_type: "Outlying area",
    country_name: "United States",
    country_code: "US",
    region_code: "US-AS",
};

pub const US_AZ: Subdivision = Subdivision {
    name: "Arizona",
    code: "US-AZ",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-AZ",
};

pub const US_CA: Subdivision = Subdivision {
    name: "California",
    code: "US-CA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-CA",
};

pub const US_CO: Subdivision = Subdivision {
    name: "Colorado",
    code: "US-CO",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-CO",
};

pub const US_CT: Subdivision = Subdivision {
    name: "Connecticut",
    code: "US-CT",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-CT",
};

pub const US_DC: Subdivision = Subdivision {
    name: "District of Columbia",
    code: "US-DC",
    subdivision_type: "District",
    country_name: "United States",
    country_code: "US",
    region_code: "US-DC",
};

pub const US_DE: Subdivision = Subdivision {
    name: "Delaware",
    code: "US-DE",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-DE",
};

pub const US_FL: Subdivision = Subdivision {
    name: "Florida",
    code: "US-FL",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-FL",
};

pub const US_GA: Subdivision = Subdivision {
    name: "Georgia",
    code: "US-GA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-GA",
};

pub const US_GU: Subdivision = Subdivision {
    name: "Guam",
    code: "US-GU",
    subdivision_type: "Outlying area",
    country_name: "United States",
    country_code: "US",
    region_code: "US-GU",
};

pub const US_HI: Subdivision = Subdivision {
    name: "Hawaii",
    code: "US-HI",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-HI",
};

pub const US_IA: Subdivision = Subdivision {
    name: "Iowa",
    code: "US-IA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-IA",
};

pub const US_ID: Subdivision = Subdivision {
    name: "Idaho",
    code: "US-ID",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-ID",
};

pub const US_IL: Subdivision = Subdivision {
    name: "Illinois",
    code: "US-IL",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-IL",
};

pub const US_IN: Subdivision = Subdivision {
    name: "Indiana",
    code: "US-IN",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-IN",
};

pub const US_KS: Subdivision = Subdivision {
    name: "Kansas",
    code: "US-KS",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-KS",
};

pub const US_KY: Subdivision = Subdivision {
    name: "Kentucky",
    code: "US-KY",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-KY",
};

pub const US_LA: Subdivision = Subdivision {
    name: "Louisiana",
    code: "US-LA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-LA",
};

pub const US_MA: Subdivision = Subdivision {
    name: "Massachusetts",
    code: "US-MA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MA",
};

pub const US_MD: Subdivision = Subdivision {
    name: "Maryland",
    code: "US-MD",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MD",
};

pub const US_ME: Subdivision = Subdivision {
    name: "Maine",
    code: "US-ME",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-ME",
};

pub const US_MI: Subdivision = Subdivision {
    name: "Michigan",
    code: "US-MI",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MI",
};

pub const US_MN: Subdivision = Subdivision {
    name: "Minnesota",
    code: "US-MN",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MN",
};

pub const US_MO: Subdivision = Subdivision {
    name: "Missouri",
    code: "US-MO",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MO",
};

pub const US_MP: Subdivision = Subdivision {
    name: "Northern Mariana Islands",
    code: "US-MP",
    subdivision_type: "Outlying area",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MP",
};

pub const US_MS: Subdivision = Subdivision {
    name: "Mississippi",
    code: "US-MS",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MS",
};

pub const US_MT: Subdivision = Subdivision {
    name: "Montana",
    code: "US-MT",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-MT",
};

pub const US_NC: Subdivision = Subdivision {
    name: "North Carolina",
    code: "US-NC",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NC",
};

pub const US_ND: Subdivision = Subdivision {
    name: "North Dakota",
    code: "US-ND",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-ND",
};

pub const US_NE: Subdivision = Subdivision {
    name: "Nebraska",
    code: "US-NE",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NE",
};

pub const US_NH: Subdivision = Subdivision {
    name: "New Hampshire",
    code: "US-NH",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NH",
};

pub const US_NJ: Subdivision = Subdivision {
    name: "New Jersey",
    code: "US-NJ",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NJ",
};

pub const US_NM: Subdivision = Subdivision {
    name: "New Mexico",
    code: "US-NM",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NM",
};

pub const US_NV: Subdivision = Subdivision {
    name: "Nevada",
    code: "US-NV",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NV",
};

pub const US_NY: Subdivision = Subdivision {
    name: "New York",
    code: "US-NY",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-NY",
};

pub const US_OH: Subdivision = Subdivision {
    name: "Ohio",
    code: "US-OH",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-OH",
};

pub const US_OK: Subdivision = Subdivision {
    name: "Oklahoma",
    code: "US-OK",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-OK",
};

pub const US_OR: Subdivision = Subdivision {
    name: "Oregon",
    code: "US-OR",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-OR",
};

pub const US_PA: Subdivision = Subdivision {
    name: "Pennsylvania",
    code: "US-PA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-PA",
};

pub const US_PR: Subdivision = Subdivision {
    name: "Puerto Rico",
    code: "US-PR",
    subdivision_type: "Outlying area",
    country_name: "United States",
    country_code: "US",
    region_code: "US-PR",
};

pub const US_RI: Subdivision = Subdivision {
    name: "Rhode Island",
    code: "US-RI",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-RI",
};

pub const US_SC: Subdivision = Subdivision {
    name: "South Carolina",
    code: "US-SC",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-SC",
};

pub const US_SD: Subdivision = Subdivision {
    name: "South Dakota",
    code: "US-SD",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-SD",
};

pub const US_TN: Subdivision = Subdivision {
    name: "Tennessee",
    code: "US-TN",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-TN",
};

pub const US_TX: Subdivision = Subdivision {
    name: "Texas",
    code: "US-TX",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-TX",
};

pub const US_UM: Subdivision = Subdivision {
    name: "United States Minor Outlying Islands",
    code: "US-UM",
    subdivision_type: "Outlying area",
    country_name: "United States",
    country_code: "US",
    region_code: "US-UM",
};

pub const US_UT: Subdivision = Subdivision {
    name: "Utah",
    code: "US-UT",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-UT",
};

pub const US_VA: Subdivision = Subdivision {
    name: "Virginia",
    code: "US-VA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-VA",
};

pub const US_VI: Subdivision = Subdivision {
    name: "Virgin Islands, U.S.",
    code: "US-VI",
    subdivision_type: "Outlying area",
    country_name: "United States",
    country_code: "US",
    region_code: "US-VI",
};

pub const US_VT: Subdivision = Subdivision {
    name: "Vermont",
    code: "US-VT",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-VT",
};

pub const US_WA: Subdivision = Subdivision {
    name: "Washington",
    code: "US-WA",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-WA",
};

pub const US_WI: Subdivision = Subdivision {
    name: "Wisconsin",
    code: "US-WI",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-WI",
};

pub const US_WV: Subdivision = Subdivision {
    name: "West Virginia",
    code: "US-WV",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-WV",
};

pub const US_WY: Subdivision = Subdivision {
    name: "Wyoming",
    code: "US-WY",
    subdivision_type: "State",
    country_name: "United States",
    country_code: "US",
    region_code: "US-WY",
};

pub const UY_AR: Subdivision = Subdivision {
    name: "Artigas",
    code: "UY-AR",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-AR",
};

pub const UY_CA: Subdivision = Subdivision {
    name: "Canelones",
    code: "UY-CA",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-CA",
};

pub const UY_CL: Subdivision = Subdivision {
    name: "Cerro Largo",
    code: "UY-CL",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-CL",
};

pub const UY_CO: Subdivision = Subdivision {
    name: "Colonia",
    code: "UY-CO",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-CO",
};

pub const UY_DU: Subdivision = Subdivision {
    name: "Durazno",
    code: "UY-DU",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-DU",
};

pub const UY_FD: Subdivision = Subdivision {
    name: "Florida",
    code: "UY-FD",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-FD",
};

pub const UY_FS: Subdivision = Subdivision {
    name: "Flores",
    code: "UY-FS",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-FS",
};

pub const UY_LA: Subdivision = Subdivision {
    name: "Lavalleja",
    code: "UY-LA",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-LA",
};

pub const UY_MA: Subdivision = Subdivision {
    name: "Maldonado",
    code: "UY-MA",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-MA",
};

pub const UY_MO: Subdivision = Subdivision {
    name: "Montevideo",
    code: "UY-MO",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-MO",
};

pub const UY_PA: Subdivision = Subdivision {
    name: "Paysandú",
    code: "UY-PA",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-PA",
};

pub const UY_RN: Subdivision = Subdivision {
    name: "Río Negro",
    code: "UY-RN",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-RN",
};

pub const UY_RO: Subdivision = Subdivision {
    name: "Rocha",
    code: "UY-RO",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-RO",
};

pub const UY_RV: Subdivision = Subdivision {
    name: "Rivera",
    code: "UY-RV",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-RV",
};

pub const UY_SA: Subdivision = Subdivision {
    name: "Salto",
    code: "UY-SA",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-SA",
};

pub const UY_SJ: Subdivision = Subdivision {
    name: "San José",
    code: "UY-SJ",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-SJ",
};

pub const UY_SO: Subdivision = Subdivision {
    name: "Soriano",
    code: "UY-SO",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-SO",
};

pub const UY_TA: Subdivision = Subdivision {
    name: "Tacuarembó",
    code: "UY-TA",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-TA",
};

pub const UY_TT: Subdivision = Subdivision {
    name: "Treinta y Tres",
    code: "UY-TT",
    subdivision_type: "Department",
    country_name: "Uruguay",
    country_code: "UY",
    region_code: "UY-TT",
};

pub const UZ_AN: Subdivision = Subdivision {
    name: "Andijon",
    code: "UZ-AN",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-AN",
};

pub const UZ_BU: Subdivision = Subdivision {
    name: "Buxoro",
    code: "UZ-BU",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-BU",
};

pub const UZ_FA: Subdivision = Subdivision {
    name: "Farg‘ona",
    code: "UZ-FA",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-FA",
};

pub const UZ_JI: Subdivision = Subdivision {
    name: "Jizzax",
    code: "UZ-JI",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-JI",
};

pub const UZ_NG: Subdivision = Subdivision {
    name: "Namangan",
    code: "UZ-NG",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-NG",
};

pub const UZ_NW: Subdivision = Subdivision {
    name: "Navoiy",
    code: "UZ-NW",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-NW",
};

pub const UZ_QA: Subdivision = Subdivision {
    name: "Qashqadaryo",
    code: "UZ-QA",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-QA",
};

pub const UZ_QR: Subdivision = Subdivision {
    name: "Qoraqalpog‘iston Respublikasi",
    code: "UZ-QR",
    subdivision_type: "Republic",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-QR",
};

pub const UZ_SA: Subdivision = Subdivision {
    name: "Samarqand",
    code: "UZ-SA",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-SA",
};

pub const UZ_SI: Subdivision = Subdivision {
    name: "Sirdaryo",
    code: "UZ-SI",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-SI",
};

pub const UZ_SU: Subdivision = Subdivision {
    name: "Surxondaryo",
    code: "UZ-SU",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-SU",
};

pub const UZ_TK: Subdivision = Subdivision {
    name: "Toshkent",
    code: "UZ-TK",
    subdivision_type: "City",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-TK",
};

pub const UZ_TO: Subdivision = Subdivision {
    name: "Toshkent",
    code: "UZ-TO",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-TO",
};

pub const UZ_XO: Subdivision = Subdivision {
    name: "Xorazm",
    code: "UZ-XO",
    subdivision_type: "Region",
    country_name: "Uzbekistan",
    country_code: "UZ",
    region_code: "UZ-XO",
};

pub const VU_MAP: Subdivision = Subdivision {
    name: "Malampa",
    code: "VU-MAP",
    subdivision_type: "Province",
    country_name: "Vanuatu",
    country_code: "VU",
    region_code: "VU-MAP",
};

pub const VU_PAM: Subdivision = Subdivision {
    name: "Pénama",
    code: "VU-PAM",
    subdivision_type: "Province",
    country_name: "Vanuatu",
    country_code: "VU",
    region_code: "VU-PAM",
};

pub const VU_SAM: Subdivision = Subdivision {
    name: "Sanma",
    code: "VU-SAM",
    subdivision_type: "Province",
    country_name: "Vanuatu",
    country_code: "VU",
    region_code: "VU-SAM",
};

pub const VU_SEE: Subdivision = Subdivision {
    name: "Shéfa",
    code: "VU-SEE",
    subdivision_type: "Province",
    country_name: "Vanuatu",
    country_code: "VU",
    region_code: "VU-SEE",
};

pub const VU_TAE: Subdivision = Subdivision {
    name: "Taféa",
    code: "VU-TAE",
    subdivision_type: "Province",
    country_name: "Vanuatu",
    country_code: "VU",
    region_code: "VU-TAE",
};

pub const VU_TOB: Subdivision = Subdivision {
    name: "Torba",
    code: "VU-TOB",
    subdivision_type: "Province",
    country_name: "Vanuatu",
    country_code: "VU",
    region_code: "VU-TOB",
};

pub const VA_VA: Subdivision = Subdivision {
    name: "Vatican City",
    code: "VA-VA",
    subdivision_type: "Country",
    country_name: "Vatican City",
    country_code: "VA",
    region_code: "VA-VA",
};

pub const VE_A: Subdivision = Subdivision {
    name: "Distrito Capital",
    code: "VE-A",
    subdivision_type: "Capital district",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-A",
};

pub const VE_B: Subdivision = Subdivision {
    name: "Anzoátegui",
    code: "VE-B",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-B",
};

pub const VE_C: Subdivision = Subdivision {
    name: "Apure",
    code: "VE-C",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-C",
};

pub const VE_D: Subdivision = Subdivision {
    name: "Aragua",
    code: "VE-D",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-D",
};

pub const VE_E: Subdivision = Subdivision {
    name: "Barinas",
    code: "VE-E",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-E",
};

pub const VE_F: Subdivision = Subdivision {
    name: "Bolívar",
    code: "VE-F",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-F",
};

pub const VE_G: Subdivision = Subdivision {
    name: "Carabobo",
    code: "VE-G",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-G",
};

pub const VE_H: Subdivision = Subdivision {
    name: "Cojedes",
    code: "VE-H",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-H",
};

pub const VE_I: Subdivision = Subdivision {
    name: "Falcón",
    code: "VE-I",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-I",
};

pub const VE_J: Subdivision = Subdivision {
    name: "Guárico",
    code: "VE-J",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-J",
};

pub const VE_K: Subdivision = Subdivision {
    name: "Lara",
    code: "VE-K",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-K",
};

pub const VE_L: Subdivision = Subdivision {
    name: "Mérida",
    code: "VE-L",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-L",
};

pub const VE_M: Subdivision = Subdivision {
    name: "Miranda",
    code: "VE-M",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-M",
};

pub const VE_N: Subdivision = Subdivision {
    name: "Monagas",
    code: "VE-N",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-N",
};

pub const VE_O: Subdivision = Subdivision {
    name: "Nueva Esparta",
    code: "VE-O",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-O",
};

pub const VE_P: Subdivision = Subdivision {
    name: "Portuguesa",
    code: "VE-P",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-P",
};

pub const VE_R: Subdivision = Subdivision {
    name: "Sucre",
    code: "VE-R",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-R",
};

pub const VE_S: Subdivision = Subdivision {
    name: "Táchira",
    code: "VE-S",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-S",
};

pub const VE_T: Subdivision = Subdivision {
    name: "Trujillo",
    code: "VE-T",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-T",
};

pub const VE_U: Subdivision = Subdivision {
    name: "Yaracuy",
    code: "VE-U",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-U",
};

pub const VE_V: Subdivision = Subdivision {
    name: "Zulia",
    code: "VE-V",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-V",
};

pub const VE_W: Subdivision = Subdivision {
    name: "Dependencias Federales",
    code: "VE-W",
    subdivision_type: "Federal dependency",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-W",
};

pub const VE_X: Subdivision = Subdivision {
    name: "Vargas",
    code: "VE-X",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-X",
};

pub const VE_Y: Subdivision = Subdivision {
    name: "Delta Amacuro",
    code: "VE-Y",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-Y",
};

pub const VE_Z: Subdivision = Subdivision {
    name: "Amazonas",
    code: "VE-Z",
    subdivision_type: "State",
    country_name: "Venezuela",
    country_code: "VE",
    region_code: "VE-Z",
};

pub const VN_01: Subdivision = Subdivision {
    name: "Lai Châu",
    code: "VN-01",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-01",
};

pub const VN_02: Subdivision = Subdivision {
    name: "Lào Cai",
    code: "VN-02",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-02",
};

pub const VN_03: Subdivision = Subdivision {
    name: "Hà Giang",
    code: "VN-03",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-03",
};

pub const VN_04: Subdivision = Subdivision {
    name: "Cao Bằng",
    code: "VN-04",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-04",
};

pub const VN_05: Subdivision = Subdivision {
    name: "Sơn La",
    code: "VN-05",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-05",
};

pub const VN_06: Subdivision = Subdivision {
    name: "Yên Bái",
    code: "VN-06",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-06",
};

pub const VN_07: Subdivision = Subdivision {
    name: "Tuyên Quang",
    code: "VN-07",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-07",
};

pub const VN_09: Subdivision = Subdivision {
    name: "Lạng Sơn",
    code: "VN-09",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-09",
};

pub const VN_13: Subdivision = Subdivision {
    name: "Quảng Ninh",
    code: "VN-13",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-13",
};

pub const VN_14: Subdivision = Subdivision {
    name: "Hòa Bình",
    code: "VN-14",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-14",
};

pub const VN_18: Subdivision = Subdivision {
    name: "Ninh Bình",
    code: "VN-18",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-18",
};

pub const VN_20: Subdivision = Subdivision {
    name: "Thái Bình",
    code: "VN-20",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-20",
};

pub const VN_21: Subdivision = Subdivision {
    name: "Thanh Hóa",
    code: "VN-21",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-21",
};

pub const VN_22: Subdivision = Subdivision {
    name: "Nghệ An",
    code: "VN-22",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-22",
};

pub const VN_23: Subdivision = Subdivision {
    name: "Hà Tĩnh",
    code: "VN-23",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-23",
};

pub const VN_24: Subdivision = Subdivision {
    name: "Quảng Bình",
    code: "VN-24",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-24",
};

pub const VN_25: Subdivision = Subdivision {
    name: "Quảng Trị",
    code: "VN-25",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-25",
};

pub const VN_26: Subdivision = Subdivision {
    name: "Thừa Thiên-Huế",
    code: "VN-26",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-26",
};

pub const VN_27: Subdivision = Subdivision {
    name: "Quảng Nam",
    code: "VN-27",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-27",
};

pub const VN_28: Subdivision = Subdivision {
    name: "Kon Tum",
    code: "VN-28",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-28",
};

pub const VN_29: Subdivision = Subdivision {
    name: "Quảng Ngãi",
    code: "VN-29",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-29",
};

pub const VN_30: Subdivision = Subdivision {
    name: "Gia Lai",
    code: "VN-30",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-30",
};

pub const VN_31: Subdivision = Subdivision {
    name: "Bình Định",
    code: "VN-31",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-31",
};

pub const VN_32: Subdivision = Subdivision {
    name: "Phú Yên",
    code: "VN-32",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-32",
};

pub const VN_33: Subdivision = Subdivision {
    name: "Đắk Lắk",
    code: "VN-33",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-33",
};

pub const VN_34: Subdivision = Subdivision {
    name: "Khánh Hòa",
    code: "VN-34",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-34",
};

pub const VN_35: Subdivision = Subdivision {
    name: "Lâm Ðồng",
    code: "VN-35",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-35",
};

pub const VN_36: Subdivision = Subdivision {
    name: "Ninh Thuận",
    code: "VN-36",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-36",
};

pub const VN_37: Subdivision = Subdivision {
    name: "Tây Ninh",
    code: "VN-37",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-37",
};

pub const VN_39: Subdivision = Subdivision {
    name: "Ðồng Nai",
    code: "VN-39",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-39",
};

pub const VN_40: Subdivision = Subdivision {
    name: "Bình Thuận",
    code: "VN-40",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-40",
};

pub const VN_41: Subdivision = Subdivision {
    name: "Long An",
    code: "VN-41",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-41",
};

pub const VN_43: Subdivision = Subdivision {
    name: "Bà Rịa - Vũng Tàu",
    code: "VN-43",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-43",
};

pub const VN_44: Subdivision = Subdivision {
    name: "An Giang",
    code: "VN-44",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-44",
};

pub const VN_45: Subdivision = Subdivision {
    name: "Ðồng Tháp",
    code: "VN-45",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-45",
};

pub const VN_46: Subdivision = Subdivision {
    name: "Tiền Giang",
    code: "VN-46",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-46",
};

pub const VN_47: Subdivision = Subdivision {
    name: "Kiến Giang",
    code: "VN-47",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-47",
};

pub const VN_49: Subdivision = Subdivision {
    name: "Vĩnh Long",
    code: "VN-49",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-49",
};

pub const VN_50: Subdivision = Subdivision {
    name: "Bến Tre",
    code: "VN-50",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-50",
};

pub const VN_51: Subdivision = Subdivision {
    name: "Trà Vinh",
    code: "VN-51",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-51",
};

pub const VN_52: Subdivision = Subdivision {
    name: "Sóc Trăng",
    code: "VN-52",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-52",
};

pub const VN_53: Subdivision = Subdivision {
    name: "Bắc Kạn",
    code: "VN-53",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-53",
};

pub const VN_54: Subdivision = Subdivision {
    name: "Bắc Giang",
    code: "VN-54",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-54",
};

pub const VN_55: Subdivision = Subdivision {
    name: "Bạc Liêu",
    code: "VN-55",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-55",
};

pub const VN_56: Subdivision = Subdivision {
    name: "Bắc Ninh",
    code: "VN-56",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-56",
};

pub const VN_57: Subdivision = Subdivision {
    name: "Bình Dương",
    code: "VN-57",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-57",
};

pub const VN_58: Subdivision = Subdivision {
    name: "Bình Phước",
    code: "VN-58",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-58",
};

pub const VN_59: Subdivision = Subdivision {
    name: "Cà Mau",
    code: "VN-59",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-59",
};

pub const VN_61: Subdivision = Subdivision {
    name: "Hải Dương",
    code: "VN-61",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-61",
};

pub const VN_63: Subdivision = Subdivision {
    name: "Hà Nam",
    code: "VN-63",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-63",
};

pub const VN_66: Subdivision = Subdivision {
    name: "Hưng Yên",
    code: "VN-66",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-66",
};

pub const VN_67: Subdivision = Subdivision {
    name: "Nam Ðịnh",
    code: "VN-67",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-67",
};

pub const VN_68: Subdivision = Subdivision {
    name: "Phú Thọ",
    code: "VN-68",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-68",
};

pub const VN_69: Subdivision = Subdivision {
    name: "Thái Nguyên",
    code: "VN-69",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-69",
};

pub const VN_70: Subdivision = Subdivision {
    name: "Vĩnh Phúc",
    code: "VN-70",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-70",
};

pub const VN_71: Subdivision = Subdivision {
    name: "Điện Biên",
    code: "VN-71",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-71",
};

pub const VN_72: Subdivision = Subdivision {
    name: "Đắk Nông",
    code: "VN-72",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-72",
};

pub const VN_73: Subdivision = Subdivision {
    name: "Hậu Giang",
    code: "VN-73",
    subdivision_type: "Province",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-73",
};

pub const VN_CT: Subdivision = Subdivision {
    name: "Can Tho",
    code: "VN-CT",
    subdivision_type: "Municipality",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-CT",
};

pub const VN_DN: Subdivision = Subdivision {
    name: "Da Nang",
    code: "VN-DN",
    subdivision_type: "Municipality",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-DN",
};

pub const VN_HN: Subdivision = Subdivision {
    name: "Ha Noi",
    code: "VN-HN",
    subdivision_type: "Municipality",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-HN",
};

pub const VN_HP: Subdivision = Subdivision {
    name: "Hai Phong",
    code: "VN-HP",
    subdivision_type: "Municipality",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-HP",
};

pub const VN_SG: Subdivision = Subdivision {
    name: "Ho Chi Minh",
    code: "VN-SG",
    subdivision_type: "Municipality",
    country_name: "Viet Nam",
    country_code: "VN",
    region_code: "VN-SG",
};

pub const VG_VG: Subdivision = Subdivision {
    name: "Virgin Islands",
    code: "VG-VG",
    subdivision_type: "Country",
    country_name: "Virgin Islands",
    country_code: "VG",
    region_code: "VG-VG",
};

pub const VI_VI: Subdivision = Subdivision {
    name: "Virgin Islands",
    code: "VI-VI",
    subdivision_type: "Country",
    country_name: "Virgin Islands",
    country_code: "VI",
    region_code: "VI-VI",
};

pub const WF_WF: Subdivision = Subdivision {
    name: "Wallis & Futuna Islands",
    code: "WF-WF",
    subdivision_type: "Country",
    country_name: "Wallis & Futuna Islands",
    country_code: "WF",
    region_code: "WF-WF",
};

pub const EH_EH: Subdivision = Subdivision {
    name: "Western Sahara",
    code: "EH-EH",
    subdivision_type: "Country",
    country_name: "Western Sahara",
    country_code: "EH",
    region_code: "EH-EH",
};

pub const YE_AB: Subdivision = Subdivision {
    name: "Abyān",
    code: "YE-AB",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-AB",
};

pub const YE_AD: Subdivision = Subdivision {
    name: "'Adan",
    code: "YE-AD",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-AD",
};

pub const YE_AM: Subdivision = Subdivision {
    name: "'Amrān",
    code: "YE-AM",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-AM",
};

pub const YE_BA: Subdivision = Subdivision {
    name: "Al Bayḑā'",
    code: "YE-BA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-BA",
};

pub const YE_DA: Subdivision = Subdivision {
    name: "Aḑ Ḑāli'",
    code: "YE-DA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-DA",
};

pub const YE_DH: Subdivision = Subdivision {
    name: "Dhamār",
    code: "YE-DH",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-DH",
};

pub const YE_HD: Subdivision = Subdivision {
    name: "Ḩaḑramawt",
    code: "YE-HD",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-HD",
};

pub const YE_HJ: Subdivision = Subdivision {
    name: "Ḩajjah",
    code: "YE-HJ",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-HJ",
};

pub const YE_HU: Subdivision = Subdivision {
    name: "Al Ḩudaydah",
    code: "YE-HU",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-HU",
};

pub const YE_IB: Subdivision = Subdivision {
    name: "Ibb",
    code: "YE-IB",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-IB",
};

pub const YE_JA: Subdivision = Subdivision {
    name: "Al Jawf",
    code: "YE-JA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-JA",
};

pub const YE_LA: Subdivision = Subdivision {
    name: "Laḩij",
    code: "YE-LA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-LA",
};

pub const YE_MA: Subdivision = Subdivision {
    name: "Ma'rib",
    code: "YE-MA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-MA",
};

pub const YE_MR: Subdivision = Subdivision {
    name: "Al Mahrah",
    code: "YE-MR",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-MR",
};

pub const YE_MW: Subdivision = Subdivision {
    name: "Al Maḩwīt",
    code: "YE-MW",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-MW",
};

pub const YE_RA: Subdivision = Subdivision {
    name: "Raymah",
    code: "YE-RA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-RA",
};

pub const YE_SA: Subdivision = Subdivision {
    name: "Amānat al ‘Āşimah",
    code: "YE-SA",
    subdivision_type: "Municipality",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-SA",
};

pub const YE_SD: Subdivision = Subdivision {
    name: "Şāʻdah",
    code: "YE-SD",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-SD",
};

pub const YE_SH: Subdivision = Subdivision {
    name: "Shabwah",
    code: "YE-SH",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-SH",
};

pub const YE_SN: Subdivision = Subdivision {
    name: "Şanʻā'",
    code: "YE-SN",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-SN",
};

pub const YE_TA: Subdivision = Subdivision {
    name: "Tāʻizz",
    code: "YE-TA",
    subdivision_type: "Governorate",
    country_name: "Yemen",
    country_code: "YE",
    region_code: "YE-TA",
};

pub const ZM_01: Subdivision = Subdivision {
    name: "Western",
    code: "ZM-01",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-01",
};

pub const ZM_02: Subdivision = Subdivision {
    name: "Central",
    code: "ZM-02",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-02",
};

pub const ZM_03: Subdivision = Subdivision {
    name: "Eastern",
    code: "ZM-03",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-03",
};

pub const ZM_04: Subdivision = Subdivision {
    name: "Luapula",
    code: "ZM-04",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-04",
};

pub const ZM_05: Subdivision = Subdivision {
    name: "Northern",
    code: "ZM-05",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-05",
};

pub const ZM_06: Subdivision = Subdivision {
    name: "North-Western",
    code: "ZM-06",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-06",
};

pub const ZM_07: Subdivision = Subdivision {
    name: "Southern",
    code: "ZM-07",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-07",
};

pub const ZM_08: Subdivision = Subdivision {
    name: "Copperbelt",
    code: "ZM-08",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-08",
};

pub const ZM_09: Subdivision = Subdivision {
    name: "Lusaka",
    code: "ZM-09",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-09",
};

pub const ZM_10: Subdivision = Subdivision {
    name: "Muchinga",
    code: "ZM-10",
    subdivision_type: "Province",
    country_name: "Zambia",
    country_code: "ZM",
    region_code: "ZM-10",
};

pub const ZW_BU: Subdivision = Subdivision {
    name: "Bulawayo",
    code: "ZW-BU",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-BU",
};

pub const ZW_HA: Subdivision = Subdivision {
    name: "Harare",
    code: "ZW-HA",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-HA",
};

pub const ZW_MA: Subdivision = Subdivision {
    name: "Manicaland",
    code: "ZW-MA",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MA",
};

pub const ZW_MC: Subdivision = Subdivision {
    name: "Mashonaland Central",
    code: "ZW-MC",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MC",
};

pub const ZW_ME: Subdivision = Subdivision {
    name: "Mashonaland East",
    code: "ZW-ME",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-ME",
};

pub const ZW_MI: Subdivision = Subdivision {
    name: "Midlands",
    code: "ZW-MI",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MI",
};

pub const ZW_MN: Subdivision = Subdivision {
    name: "Matabeleland North",
    code: "ZW-MN",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MN",
};

pub const ZW_MS: Subdivision = Subdivision {
    name: "Matabeleland South",
    code: "ZW-MS",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MS",
};

pub const ZW_MV: Subdivision = Subdivision {
    name: "Masvingo",
    code: "ZW-MV",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MV",
};

pub const ZW_MW: Subdivision = Subdivision {
    name: "Mashonaland West",
    code: "ZW-MW",
    subdivision_type: "Province",
    country_name: "Zimbabwe",
    country_code: "ZW",
    region_code: "ZW-MW",
};


///Subdivision map with  Code key 
pub const SUBDIVISION_MAP: Map<&str, Subdivision> = phf_map! {

"AF-BAL" => AF_BAL,
"AF-BAM" => AF_BAM,
"AF-BDG" => AF_BDG,
"AF-BDS" => AF_BDS,
"AF-BGL" => AF_BGL,
"AF-DAY" => AF_DAY,
"AF-FRA" => AF_FRA,
"AF-FYB" => AF_FYB,
"AF-GHA" => AF_GHA,
"AF-GHO" => AF_GHO,
"AF-HEL" => AF_HEL,
"AF-HER" => AF_HER,
"AF-JOW" => AF_JOW,
"AF-KAB" => AF_KAB,
"AF-KAN" => AF_KAN,
"AF-KAP" => AF_KAP,
"AF-KDZ" => AF_KDZ,
"AF-KHO" => AF_KHO,
"AF-KNR" => AF_KNR,
"AF-LAG" => AF_LAG,
"AF-LOG" => AF_LOG,
"AF-NAN" => AF_NAN,
"AF-NIM" => AF_NIM,
"AF-NUR" => AF_NUR,
"AF-PAN" => AF_PAN,
"AF-PAR" => AF_PAR,
"AF-PIA" => AF_PIA,
"AF-PKA" => AF_PKA,
"AF-SAM" => AF_SAM,
"AF-SAR" => AF_SAR,
"AF-TAK" => AF_TAK,
"AF-URU" => AF_URU,
"AF-WAR" => AF_WAR,
"AF-ZAB" => AF_ZAB,
"AX-AX" => AX_AX,
"AL-01" => AL_01,
"AL-02" => AL_02,
"AL-03" => AL_03,
"AL-04" => AL_04,
"AL-05" => AL_05,
"AL-06" => AL_06,
"AL-07" => AL_07,
"AL-08" => AL_08,
"AL-09" => AL_09,
"AL-10" => AL_10,
"AL-11" => AL_11,
"AL-12" => AL_12,
"AL-BR" => AL_BR,
"AL-BU" => AL_BU,
"AL-DI" => AL_DI,
"AL-DL" => AL_DL,
"AL-DR" => AL_DR,
"AL-DV" => AL_DV,
"AL-EL" => AL_EL,
"AL-ER" => AL_ER,
"AL-FR" => AL_FR,
"AL-GJ" => AL_GJ,
"AL-GR" => AL_GR,
"AL-HA" => AL_HA,
"AL-KA" => AL_KA,
"AL-KB" => AL_KB,
"AL-KC" => AL_KC,
"AL-KO" => AL_KO,
"AL-KR" => AL_KR,
"AL-KU" => AL_KU,
"AL-LB" => AL_LB,
"AL-LE" => AL_LE,
"AL-LU" => AL_LU,
"AL-MK" => AL_MK,
"AL-MM" => AL_MM,
"AL-MR" => AL_MR,
"AL-MT" => AL_MT,
"AL-PG" => AL_PG,
"AL-PQ" => AL_PQ,
"AL-PR" => AL_PR,
"AL-PU" => AL_PU,
"AL-SH" => AL_SH,
"AL-SK" => AL_SK,
"AL-SR" => AL_SR,
"AL-TE" => AL_TE,
"AL-TP" => AL_TP,
"AL-TR" => AL_TR,
"AL-VL" => AL_VL,
"DZ-01" => DZ_01,
"DZ-02" => DZ_02,
"DZ-03" => DZ_03,
"DZ-04" => DZ_04,
"DZ-05" => DZ_05,
"DZ-06" => DZ_06,
"DZ-07" => DZ_07,
"DZ-08" => DZ_08,
"DZ-09" => DZ_09,
"DZ-10" => DZ_10,
"DZ-11" => DZ_11,
"DZ-12" => DZ_12,
"DZ-13" => DZ_13,
"DZ-14" => DZ_14,
"DZ-15" => DZ_15,
"DZ-16" => DZ_16,
"DZ-17" => DZ_17,
"DZ-18" => DZ_18,
"DZ-19" => DZ_19,
"DZ-20" => DZ_20,
"DZ-21" => DZ_21,
"DZ-22" => DZ_22,
"DZ-23" => DZ_23,
"DZ-24" => DZ_24,
"DZ-25" => DZ_25,
"DZ-26" => DZ_26,
"DZ-27" => DZ_27,
"DZ-28" => DZ_28,
"DZ-29" => DZ_29,
"DZ-30" => DZ_30,
"DZ-31" => DZ_31,
"DZ-32" => DZ_32,
"DZ-33" => DZ_33,
"DZ-34" => DZ_34,
"DZ-35" => DZ_35,
"DZ-36" => DZ_36,
"DZ-37" => DZ_37,
"DZ-38" => DZ_38,
"DZ-39" => DZ_39,
"DZ-40" => DZ_40,
"DZ-41" => DZ_41,
"DZ-42" => DZ_42,
"DZ-43" => DZ_43,
"DZ-44" => DZ_44,
"DZ-45" => DZ_45,
"DZ-46" => DZ_46,
"DZ-47" => DZ_47,
"DZ-48" => DZ_48,
"AS-AS" => AS_AS,
"AD-02" => AD_02,
"AD-03" => AD_03,
"AD-04" => AD_04,
"AD-05" => AD_05,
"AD-06" => AD_06,
"AD-07" => AD_07,
"AD-08" => AD_08,
"AO-BGO" => AO_BGO,
"AO-BGU" => AO_BGU,
"AO-BIE" => AO_BIE,
"AO-CAB" => AO_CAB,
"AO-CCU" => AO_CCU,
"AO-CNN" => AO_CNN,
"AO-CNO" => AO_CNO,
"AO-CUS" => AO_CUS,
"AO-HUA" => AO_HUA,
"AO-HUI" => AO_HUI,
"AO-LNO" => AO_LNO,
"AO-LSU" => AO_LSU,
"AO-LUA" => AO_LUA,
"AO-MAL" => AO_MAL,
"AO-MOX" => AO_MOX,
"AO-NAM" => AO_NAM,
"AO-UIG" => AO_UIG,
"AO-ZAI" => AO_ZAI,
"AI-AI" => AI_AI,
"AQ-AQ" => AQ_AQ,
"AG-03" => AG_03,
"AG-04" => AG_04,
"AG-05" => AG_05,
"AG-06" => AG_06,
"AG-07" => AG_07,
"AG-08" => AG_08,
"AG-10" => AG_10,
"AG-11" => AG_11,
"AR-A" => AR_A,
"AR-B" => AR_B,
"AR-C" => AR_C,
"AR-D" => AR_D,
"AR-E" => AR_E,
"AR-F" => AR_F,
"AR-G" => AR_G,
"AR-H" => AR_H,
"AR-J" => AR_J,
"AR-K" => AR_K,
"AR-L" => AR_L,
"AR-M" => AR_M,
"AR-N" => AR_N,
"AR-P" => AR_P,
"AR-Q" => AR_Q,
"AR-R" => AR_R,
"AR-S" => AR_S,
"AR-T" => AR_T,
"AR-U" => AR_U,
"AR-V" => AR_V,
"AR-W" => AR_W,
"AR-X" => AR_X,
"AR-Y" => AR_Y,
"AR-Z" => AR_Z,
"AM-AG" => AM_AG,
"AM-AR" => AM_AR,
"AM-AV" => AM_AV,
"AM-ER" => AM_ER,
"AM-GR" => AM_GR,
"AM-KT" => AM_KT,
"AM-LO" => AM_LO,
"AM-SH" => AM_SH,
"AM-SU" => AM_SU,
"AM-TV" => AM_TV,
"AM-VD" => AM_VD,
"AW-AW" => AW_AW,
"AU-ACT" => AU_ACT,
"AU-NSW" => AU_NSW,
"AU-NT" => AU_NT,
"AU-QLD" => AU_QLD,
"AU-SA" => AU_SA,
"AU-TAS" => AU_TAS,
"AU-VIC" => AU_VIC,
"AU-WA" => AU_WA,
"AT-1" => AT_1,
"AT-2" => AT_2,
"AT-3" => AT_3,
"AT-4" => AT_4,
"AT-5" => AT_5,
"AT-6" => AT_6,
"AT-7" => AT_7,
"AT-8" => AT_8,
"AT-9" => AT_9,
"AZ-ABS" => AZ_ABS,
"AZ-AGA" => AZ_AGA,
"AZ-AGC" => AZ_AGC,
"AZ-AGM" => AZ_AGM,
"AZ-AGS" => AZ_AGS,
"AZ-AGU" => AZ_AGU,
"AZ-AST" => AZ_AST,
"AZ-BA" => AZ_BA,
"AZ-BAB" => AZ_BAB,
"AZ-BAL" => AZ_BAL,
"AZ-BAR" => AZ_BAR,
"AZ-BEY" => AZ_BEY,
"AZ-BIL" => AZ_BIL,
"AZ-CAB" => AZ_CAB,
"AZ-CAL" => AZ_CAL,
"AZ-CUL" => AZ_CUL,
"AZ-DAS" => AZ_DAS,
"AZ-FUZ" => AZ_FUZ,
"AZ-GAD" => AZ_GAD,
"AZ-GA" => AZ_GA,
"AZ-GOR" => AZ_GOR,
"AZ-GOY" => AZ_GOY,
"AZ-GYG" => AZ_GYG,
"AZ-HAC" => AZ_HAC,
"AZ-IMI" => AZ_IMI,
"AZ-ISM" => AZ_ISM,
"AZ-KAL" => AZ_KAL,
"AZ-KAN" => AZ_KAN,
"AZ-KUR" => AZ_KUR,
"AZ-LAC" => AZ_LAC,
"AZ-LA" => AZ_LA,
"AZ-LAN" => AZ_LAN,
"AZ-LER" => AZ_LER,
"AZ-MAS" => AZ_MAS,
"AZ-MI" => AZ_MI,
"AZ-NA" => AZ_NA,
"AZ-NEF" => AZ_NEF,
"AZ-NV" => AZ_NV,
"AZ-NX" => AZ_NX,
"AZ-OGU" => AZ_OGU,
"AZ-ORD" => AZ_ORD,
"AZ-QAB" => AZ_QAB,
"AZ-QAX" => AZ_QAX,
"AZ-QAZ" => AZ_QAZ,
"AZ-QBA" => AZ_QBA,
"AZ-QBI" => AZ_QBI,
"AZ-QOB" => AZ_QOB,
"AZ-QUS" => AZ_QUS,
"AZ-SAB" => AZ_SAB,
"AZ-SAD" => AZ_SAD,
"AZ-SAH" => AZ_SAH,
"AZ-SAK" => AZ_SAK,
"AZ-SAL" => AZ_SAL,
"AZ-SAR" => AZ_SAR,
"AZ-SA" => AZ_SA,
"AZ-SAT" => AZ_SAT,
"AZ-SBN" => AZ_SBN,
"AZ-SIY" => AZ_SIY,
"AZ-SKR" => AZ_SKR,
"AZ-SMI" => AZ_SMI,
"AZ-SM" => AZ_SM,
"AZ-SMX" => AZ_SMX,
"AZ-SR" => AZ_SR,
"AZ-SUS" => AZ_SUS,
"AZ-TAR" => AZ_TAR,
"AZ-TOV" => AZ_TOV,
"AZ-UCA" => AZ_UCA,
"AZ-XAC" => AZ_XAC,
"AZ-XA" => AZ_XA,
"AZ-XCI" => AZ_XCI,
"AZ-XIZ" => AZ_XIZ,
"AZ-XVD" => AZ_XVD,
"AZ-YAR" => AZ_YAR,
"AZ-YEV" => AZ_YEV,
"AZ-YE" => AZ_YE,
"AZ-ZAN" => AZ_ZAN,
"AZ-ZAQ" => AZ_ZAQ,
"AZ-ZAR" => AZ_ZAR,
"BS-AK" => BS_AK,
"BS-BI" => BS_BI,
"BS-BP" => BS_BP,
"BS-BY" => BS_BY,
"BS-CE" => BS_CE,
"BS-CI" => BS_CI,
"BS-CK" => BS_CK,
"BS-CO" => BS_CO,
"BS-CS" => BS_CS,
"BS-EG" => BS_EG,
"BS-EX" => BS_EX,
"BS-FP" => BS_FP,
"BS-GC" => BS_GC,
"BS-HI" => BS_HI,
"BS-HT" => BS_HT,
"BS-IN" => BS_IN,
"BS-LI" => BS_LI,
"BS-MC" => BS_MC,
"BS-MG" => BS_MG,
"BS-MI" => BS_MI,
"BS-NE" => BS_NE,
"BS-NO" => BS_NO,
"BS-NS" => BS_NS,
"BS-RC" => BS_RC,
"BS-RI" => BS_RI,
"BS-SA" => BS_SA,
"BS-SE" => BS_SE,
"BS-SO" => BS_SO,
"BS-SS" => BS_SS,
"BS-SW" => BS_SW,
"BS-WG" => BS_WG,
"BH-13" => BH_13,
"BH-14" => BH_14,
"BH-15" => BH_15,
"BH-16" => BH_16,
"BH-17" => BH_17,
"BD-01" => BD_01,
"BD-02" => BD_02,
"BD-03" => BD_03,
"BD-04" => BD_04,
"BD-05" => BD_05,
"BD-06" => BD_06,
"BD-07" => BD_07,
"BD-08" => BD_08,
"BD-09" => BD_09,
"BD-10" => BD_10,
"BD-11" => BD_11,
"BD-12" => BD_12,
"BD-13" => BD_13,
"BD-14" => BD_14,
"BD-15" => BD_15,
"BD-16" => BD_16,
"BD-17" => BD_17,
"BD-18" => BD_18,
"BD-19" => BD_19,
"BD-20" => BD_20,
"BD-21" => BD_21,
"BD-22" => BD_22,
"BD-23" => BD_23,
"BD-24" => BD_24,
"BD-25" => BD_25,
"BD-26" => BD_26,
"BD-27" => BD_27,
"BD-28" => BD_28,
"BD-29" => BD_29,
"BD-30" => BD_30,
"BD-31" => BD_31,
"BD-32" => BD_32,
"BD-33" => BD_33,
"BD-34" => BD_34,
"BD-35" => BD_35,
"BD-36" => BD_36,
"BD-37" => BD_37,
"BD-38" => BD_38,
"BD-39" => BD_39,
"BD-40" => BD_40,
"BD-41" => BD_41,
"BD-42" => BD_42,
"BD-43" => BD_43,
"BD-44" => BD_44,
"BD-45" => BD_45,
"BD-46" => BD_46,
"BD-47" => BD_47,
"BD-48" => BD_48,
"BD-49" => BD_49,
"BD-50" => BD_50,
"BD-51" => BD_51,
"BD-52" => BD_52,
"BD-53" => BD_53,
"BD-54" => BD_54,
"BD-55" => BD_55,
"BD-56" => BD_56,
"BD-57" => BD_57,
"BD-58" => BD_58,
"BD-59" => BD_59,
"BD-60" => BD_60,
"BD-61" => BD_61,
"BD-62" => BD_62,
"BD-63" => BD_63,
"BD-64" => BD_64,
"BD-A" => BD_A,
"BD-B" => BD_B,
"BD-C" => BD_C,
"BD-D" => BD_D,
"BD-E" => BD_E,
"BD-F" => BD_F,
"BD-G" => BD_G,
"BB-01" => BB_01,
"BB-02" => BB_02,
"BB-03" => BB_03,
"BB-04" => BB_04,
"BB-05" => BB_05,
"BB-06" => BB_06,
"BB-07" => BB_07,
"BB-08" => BB_08,
"BB-09" => BB_09,
"BB-10" => BB_10,
"BB-11" => BB_11,
"BY-BR" => BY_BR,
"BY-HM" => BY_HM,
"BY-HO" => BY_HO,
"BY-HR" => BY_HR,
"BY-MA" => BY_MA,
"BY-MI" => BY_MI,
"BY-VI" => BY_VI,
"BE-BRU" => BE_BRU,
"BE-VAN" => BE_VAN,
"BE-VBR" => BE_VBR,
"BE-VLG" => BE_VLG,
"BE-VLI" => BE_VLI,
"BE-VOV" => BE_VOV,
"BE-VWV" => BE_VWV,
"BE-WAL" => BE_WAL,
"BE-WBR" => BE_WBR,
"BE-WHT" => BE_WHT,
"BE-WLG" => BE_WLG,
"BE-WLX" => BE_WLX,
"BE-WNA" => BE_WNA,
"BZ-BZ" => BZ_BZ,
"BZ-CY" => BZ_CY,
"BZ-CZL" => BZ_CZL,
"BZ-OW" => BZ_OW,
"BZ-SC" => BZ_SC,
"BZ-TOL" => BZ_TOL,
"BJ-AK" => BJ_AK,
"BJ-AL" => BJ_AL,
"BJ-AQ" => BJ_AQ,
"BJ-BO" => BJ_BO,
"BJ-CO" => BJ_CO,
"BJ-DO" => BJ_DO,
"BJ-KO" => BJ_KO,
"BJ-LI" => BJ_LI,
"BJ-MO" => BJ_MO,
"BJ-OU" => BJ_OU,
"BJ-PL" => BJ_PL,
"BJ-ZO" => BJ_ZO,
"BM-BM" => BM_BM,
"BT-11" => BT_11,
"BT-12" => BT_12,
"BT-13" => BT_13,
"BT-14" => BT_14,
"BT-15" => BT_15,
"BT-21" => BT_21,
"BT-22" => BT_22,
"BT-23" => BT_23,
"BT-24" => BT_24,
"BT-31" => BT_31,
"BT-32" => BT_32,
"BT-33" => BT_33,
"BT-34" => BT_34,
"BT-41" => BT_41,
"BT-42" => BT_42,
"BT-43" => BT_43,
"BT-44" => BT_44,
"BT-45" => BT_45,
"BT-GA" => BT_GA,
"BT-TY" => BT_TY,
"BO-B" => BO_B,
"BO-C" => BO_C,
"BO-H" => BO_H,
"BO-L" => BO_L,
"BO-N" => BO_N,
"BO-O" => BO_O,
"BO-P" => BO_P,
"BO-S" => BO_S,
"BO-T" => BO_T,
"BQ-BO" => BQ_BO,
"BQ-SA" => BQ_SA,
"BQ-SE" => BQ_SE,
"BA-01" => BA_01,
"BA-02" => BA_02,
"BA-03" => BA_03,
"BA-04" => BA_04,
"BA-05" => BA_05,
"BA-06" => BA_06,
"BA-07" => BA_07,
"BA-08" => BA_08,
"BA-09" => BA_09,
"BA-10" => BA_10,
"BA-BIH" => BA_BIH,
"BA-BRC" => BA_BRC,
"BA-SRP" => BA_SRP,
"BW-CE" => BW_CE,
"BW-CH" => BW_CH,
"BW-FR" => BW_FR,
"BW-GA" => BW_GA,
"BW-GH" => BW_GH,
"BW-JW" => BW_JW,
"BW-KG" => BW_KG,
"BW-KL" => BW_KL,
"BW-KW" => BW_KW,
"BW-LO" => BW_LO,
"BW-NE" => BW_NE,
"BW-NW" => BW_NW,
"BW-SE" => BW_SE,
"BW-SO" => BW_SO,
"BW-SP" => BW_SP,
"BW-ST" => BW_ST,
"BV-BV" => BV_BV,
"BR-AC" => BR_AC,
"BR-AL" => BR_AL,
"BR-AM" => BR_AM,
"BR-AP" => BR_AP,
"BR-BA" => BR_BA,
"BR-CE" => BR_CE,
"BR-DF" => BR_DF,
"BR-ES" => BR_ES,
"BR-GO" => BR_GO,
"BR-MA" => BR_MA,
"BR-MG" => BR_MG,
"BR-MS" => BR_MS,
"BR-MT" => BR_MT,
"BR-PA" => BR_PA,
"BR-PB" => BR_PB,
"BR-PE" => BR_PE,
"BR-PI" => BR_PI,
"BR-PR" => BR_PR,
"BR-RJ" => BR_RJ,
"BR-RN" => BR_RN,
"BR-RO" => BR_RO,
"BR-RR" => BR_RR,
"BR-RS" => BR_RS,
"BR-SC" => BR_SC,
"BR-SE" => BR_SE,
"BR-SP" => BR_SP,
"BR-TO" => BR_TO,
"IO-IO" => IO_IO,
"BN-BE" => BN_BE,
"BN-BM" => BN_BM,
"BN-TE" => BN_TE,
"BN-TU" => BN_TU,
"BG-01" => BG_01,
"BG-02" => BG_02,
"BG-03" => BG_03,
"BG-04" => BG_04,
"BG-05" => BG_05,
"BG-06" => BG_06,
"BG-07" => BG_07,
"BG-08" => BG_08,
"BG-09" => BG_09,
"BG-10" => BG_10,
"BG-11" => BG_11,
"BG-12" => BG_12,
"BG-13" => BG_13,
"BG-14" => BG_14,
"BG-15" => BG_15,
"BG-16" => BG_16,
"BG-17" => BG_17,
"BG-18" => BG_18,
"BG-19" => BG_19,
"BG-20" => BG_20,
"BG-21" => BG_21,
"BG-22" => BG_22,
"BG-23" => BG_23,
"BG-24" => BG_24,
"BG-25" => BG_25,
"BG-26" => BG_26,
"BG-27" => BG_27,
"BG-28" => BG_28,
"BF-01" => BF_01,
"BF-02" => BF_02,
"BF-03" => BF_03,
"BF-04" => BF_04,
"BF-05" => BF_05,
"BF-06" => BF_06,
"BF-07" => BF_07,
"BF-08" => BF_08,
"BF-09" => BF_09,
"BF-10" => BF_10,
"BF-11" => BF_11,
"BF-12" => BF_12,
"BF-13" => BF_13,
"BF-BAL" => BF_BAL,
"BF-BAM" => BF_BAM,
"BF-BAN" => BF_BAN,
"BF-BAZ" => BF_BAZ,
"BF-BGR" => BF_BGR,
"BF-BLG" => BF_BLG,
"BF-BLK" => BF_BLK,
"BF-COM" => BF_COM,
"BF-GAN" => BF_GAN,
"BF-GNA" => BF_GNA,
"BF-GOU" => BF_GOU,
"BF-HOU" => BF_HOU,
"BF-IOB" => BF_IOB,
"BF-KAD" => BF_KAD,
"BF-KEN" => BF_KEN,
"BF-KMD" => BF_KMD,
"BF-KMP" => BF_KMP,
"BF-KOP" => BF_KOP,
"BF-KOS" => BF_KOS,
"BF-KOT" => BF_KOT,
"BF-KOW" => BF_KOW,
"BF-LER" => BF_LER,
"BF-LOR" => BF_LOR,
"BF-MOU" => BF_MOU,
"BF-NAM" => BF_NAM,
"BF-NAO" => BF_NAO,
"BF-NAY" => BF_NAY,
"BF-NOU" => BF_NOU,
"BF-OUB" => BF_OUB,
"BF-OUD" => BF_OUD,
"BF-PAS" => BF_PAS,
"BF-PON" => BF_PON,
"BF-SEN" => BF_SEN,
"BF-SIS" => BF_SIS,
"BF-SMT" => BF_SMT,
"BF-SNG" => BF_SNG,
"BF-SOM" => BF_SOM,
"BF-SOR" => BF_SOR,
"BF-TAP" => BF_TAP,
"BF-TUI" => BF_TUI,
"BF-YAG" => BF_YAG,
"BF-YAT" => BF_YAT,
"BF-ZIR" => BF_ZIR,
"BF-ZON" => BF_ZON,
"BF-ZOU" => BF_ZOU,
"BI-BB" => BI_BB,
"BI-BL" => BI_BL,
"BI-BM" => BI_BM,
"BI-BR" => BI_BR,
"BI-CA" => BI_CA,
"BI-CI" => BI_CI,
"BI-GI" => BI_GI,
"BI-KI" => BI_KI,
"BI-KR" => BI_KR,
"BI-KY" => BI_KY,
"BI-MA" => BI_MA,
"BI-MU" => BI_MU,
"BI-MW" => BI_MW,
"BI-MY" => BI_MY,
"BI-NG" => BI_NG,
"BI-RT" => BI_RT,
"BI-RY" => BI_RY,
"KH-10" => KH_10,
"KH-11" => KH_11,
"KH-12" => KH_12,
"KH-13" => KH_13,
"KH-14" => KH_14,
"KH-15" => KH_15,
"KH-16" => KH_16,
"KH-17" => KH_17,
"KH-18" => KH_18,
"KH-19" => KH_19,
"KH-1" => KH_1,
"KH-20" => KH_20,
"KH-21" => KH_21,
"KH-22" => KH_22,
"KH-23" => KH_23,
"KH-24" => KH_24,
"KH-2" => KH_2,
"KH-3" => KH_3,
"KH-4" => KH_4,
"KH-5" => KH_5,
"KH-6" => KH_6,
"KH-7" => KH_7,
"KH-8" => KH_8,
"KH-9" => KH_9,
"CM-AD" => CM_AD,
"CM-CE" => CM_CE,
"CM-EN" => CM_EN,
"CM-ES" => CM_ES,
"CM-LT" => CM_LT,
"CM-NO" => CM_NO,
"CM-NW" => CM_NW,
"CM-OU" => CM_OU,
"CM-SU" => CM_SU,
"CM-SW" => CM_SW,
"CA-AB" => CA_AB,
"CA-BC" => CA_BC,
"CA-MB" => CA_MB,
"CA-NB" => CA_NB,
"CA-NL" => CA_NL,
"CA-NS" => CA_NS,
"CA-NT" => CA_NT,
"CA-NU" => CA_NU,
"CA-ON" => CA_ON,
"CA-PE" => CA_PE,
"CA-QC" => CA_QC,
"CA-SK" => CA_SK,
"CA-YT" => CA_YT,
"CV-B" => CV_B,
"CV-BR" => CV_BR,
"CV-BV" => CV_BV,
"CV-CA" => CV_CA,
"CV-CF" => CV_CF,
"CV-CR" => CV_CR,
"CV-MA" => CV_MA,
"CV-MO" => CV_MO,
"CV-PA" => CV_PA,
"CV-PN" => CV_PN,
"CV-PR" => CV_PR,
"CV-RB" => CV_RB,
"CV-RG" => CV_RG,
"CV-RS" => CV_RS,
"CV-SD" => CV_SD,
"CV-SF" => CV_SF,
"CV-S" => CV_S,
"CV-SL" => CV_SL,
"CV-SM" => CV_SM,
"CV-SO" => CV_SO,
"CV-SS" => CV_SS,
"CV-SV" => CV_SV,
"CV-TA" => CV_TA,
"CV-TS" => CV_TS,
"KY-KY" => KY_KY,
"XC-BA" => XC_BA,
"XC-GI" => XC_GI,
"XC-LL" => XC_LL,
"XC-TA" => XC_TA,
"CF-AC" => CF_AC,
"CF-BB" => CF_BB,
"CF-BGF" => CF_BGF,
"CF-BK" => CF_BK,
"CF-HK" => CF_HK,
"CF-HM" => CF_HM,
"CF-HS" => CF_HS,
"CF-KB" => CF_KB,
"CF-KG" => CF_KG,
"CF-LB" => CF_LB,
"CF-MB" => CF_MB,
"CF-MP" => CF_MP,
"CF-NM" => CF_NM,
"CF-OP" => CF_OP,
"CF-SE" => CF_SE,
"CF-UK" => CF_UK,
"CF-VK" => CF_VK,
"TD-BA" => TD_BA,
"TD-BG" => TD_BG,
"TD-BO" => TD_BO,
"TD-CB" => TD_CB,
"TD-EE" => TD_EE,
"TD-EO" => TD_EO,
"TD-GR" => TD_GR,
"TD-HL" => TD_HL,
"TD-KA" => TD_KA,
"TD-LC" => TD_LC,
"TD-LO" => TD_LO,
"TD-LR" => TD_LR,
"TD-MA" => TD_MA,
"TD-MC" => TD_MC,
"TD-ME" => TD_ME,
"TD-MO" => TD_MO,
"TD-ND" => TD_ND,
"TD-OD" => TD_OD,
"TD-SA" => TD_SA,
"TD-SI" => TD_SI,
"TD-TA" => TD_TA,
"TD-TI" => TD_TI,
"TD-WF" => TD_WF,
"CL-AI" => CL_AI,
"CL-AN" => CL_AN,
"CL-AP" => CL_AP,
"CL-AR" => CL_AR,
"CL-AT" => CL_AT,
"CL-BI" => CL_BI,
"CL-CO" => CL_CO,
"CL-LI" => CL_LI,
"CL-LL" => CL_LL,
"CL-LR" => CL_LR,
"CL-MA" => CL_MA,
"CL-ML" => CL_ML,
"CL-RM" => CL_RM,
"CL-TA" => CL_TA,
"CL-VS" => CL_VS,
"CN-11" => CN_11,
"CN-12" => CN_12,
"CN-13" => CN_13,
"CN-14" => CN_14,
"CN-15" => CN_15,
"CN-21" => CN_21,
"CN-22" => CN_22,
"CN-23" => CN_23,
"CN-31" => CN_31,
"CN-32" => CN_32,
"CN-33" => CN_33,
"CN-34" => CN_34,
"CN-35" => CN_35,
"CN-36" => CN_36,
"CN-37" => CN_37,
"CN-41" => CN_41,
"CN-42" => CN_42,
"CN-43" => CN_43,
"CN-44" => CN_44,
"CN-45" => CN_45,
"CN-46" => CN_46,
"CN-50" => CN_50,
"CN-51" => CN_51,
"CN-52" => CN_52,
"CN-53" => CN_53,
"CN-54" => CN_54,
"CN-61" => CN_61,
"CN-62" => CN_62,
"CN-63" => CN_63,
"CN-64" => CN_64,
"CN-65" => CN_65,
"CN-71" => CN_71,
"CN-91" => CN_91,
"CN-92" => CN_92,
"CX-CX" => CX_CX,
"CC-CC" => CC_CC,
"CO-AMA" => CO_AMA,
"CO-ANT" => CO_ANT,
"CO-ARA" => CO_ARA,
"CO-ATL" => CO_ATL,
"CO-BOL" => CO_BOL,
"CO-BOY" => CO_BOY,
"CO-CAL" => CO_CAL,
"CO-CAQ" => CO_CAQ,
"CO-CAS" => CO_CAS,
"CO-CAU" => CO_CAU,
"CO-CES" => CO_CES,
"CO-CHO" => CO_CHO,
"CO-COR" => CO_COR,
"CO-CUN" => CO_CUN,
"CO-DC" => CO_DC,
"CO-GUA" => CO_GUA,
"CO-GUV" => CO_GUV,
"CO-HUI" => CO_HUI,
"CO-LAG" => CO_LAG,
"CO-MAG" => CO_MAG,
"CO-MET" => CO_MET,
"CO-NAR" => CO_NAR,
"CO-NSA" => CO_NSA,
"CO-PUT" => CO_PUT,
"CO-QUI" => CO_QUI,
"CO-RIS" => CO_RIS,
"CO-SAN" => CO_SAN,
"CO-SAP" => CO_SAP,
"CO-SUC" => CO_SUC,
"CO-TOL" => CO_TOL,
"CO-VAC" => CO_VAC,
"CO-VAU" => CO_VAU,
"CO-VID" => CO_VID,
"KM-A" => KM_A,
"KM-G" => KM_G,
"KM-M" => KM_M,
"CG-11" => CG_11,
"CG-12" => CG_12,
"CG-13" => CG_13,
"CG-14" => CG_14,
"CG-15" => CG_15,
"CG-16" => CG_16,
"CG-2" => CG_2,
"CG-5" => CG_5,
"CG-7" => CG_7,
"CG-8" => CG_8,
"CG-9" => CG_9,
"CG-BZV" => CG_BZV,
"CD-BC" => CD_BC,
"CD-BN" => CD_BN,
"CD-EQ" => CD_EQ,
"CD-KA" => CD_KA,
"CD-KE" => CD_KE,
"CD-KN" => CD_KN,
"CD-KW" => CD_KW,
"CD-MA" => CD_MA,
"CD-NK" => CD_NK,
"CD-OR" => CD_OR,
"CD-SK" => CD_SK,
"CK-CK" => CK_CK,
"CR-A" => CR_A,
"CR-C" => CR_C,
"CR-G" => CR_G,
"CR-H" => CR_H,
"CR-L" => CR_L,
"CR-P" => CR_P,
"CR-SJ" => CR_SJ,
"CI-01" => CI_01,
"CI-02" => CI_02,
"CI-03" => CI_03,
"CI-04" => CI_04,
"CI-05" => CI_05,
"CI-06" => CI_06,
"CI-07" => CI_07,
"CI-08" => CI_08,
"CI-09" => CI_09,
"CI-10" => CI_10,
"CI-11" => CI_11,
"CI-12" => CI_12,
"CI-13" => CI_13,
"CI-14" => CI_14,
"CI-15" => CI_15,
"CI-16" => CI_16,
"CI-17" => CI_17,
"CI-18" => CI_18,
"CI-19" => CI_19,
"HR-01" => HR_01,
"HR-02" => HR_02,
"HR-03" => HR_03,
"HR-04" => HR_04,
"HR-05" => HR_05,
"HR-06" => HR_06,
"HR-07" => HR_07,
"HR-08" => HR_08,
"HR-09" => HR_09,
"HR-10" => HR_10,
"HR-11" => HR_11,
"HR-12" => HR_12,
"HR-13" => HR_13,
"HR-14" => HR_14,
"HR-15" => HR_15,
"HR-16" => HR_16,
"HR-17" => HR_17,
"HR-18" => HR_18,
"HR-19" => HR_19,
"HR-20" => HR_20,
"HR-21" => HR_21,
"CU-01" => CU_01,
"CU-03" => CU_03,
"CU-04" => CU_04,
"CU-05" => CU_05,
"CU-06" => CU_06,
"CU-07" => CU_07,
"CU-08" => CU_08,
"CU-09" => CU_09,
"CU-10" => CU_10,
"CU-11" => CU_11,
"CU-12" => CU_12,
"CU-13" => CU_13,
"CU-14" => CU_14,
"CU-15" => CU_15,
"CU-16" => CU_16,
"CU-99" => CU_99,
"CW-CW" => CW_CW,
"CY-01" => CY_01,
"CY-02" => CY_02,
"CY-03" => CY_03,
"CY-04" => CY_04,
"CY-05" => CY_05,
"CY-06" => CY_06,
"CZ-101" => CZ_101,
"CZ-102" => CZ_102,
"CZ-103" => CZ_103,
"CZ-104" => CZ_104,
"CZ-105" => CZ_105,
"CZ-106" => CZ_106,
"CZ-107" => CZ_107,
"CZ-108" => CZ_108,
"CZ-109" => CZ_109,
"CZ-10A" => CZ_10A,
"CZ-10B" => CZ_10B,
"CZ-10C" => CZ_10C,
"CZ-10D" => CZ_10D,
"CZ-10E" => CZ_10E,
"CZ-10F" => CZ_10F,
"CZ-201" => CZ_201,
"CZ-202" => CZ_202,
"CZ-203" => CZ_203,
"CZ-204" => CZ_204,
"CZ-205" => CZ_205,
"CZ-206" => CZ_206,
"CZ-207" => CZ_207,
"CZ-208" => CZ_208,
"CZ-209" => CZ_209,
"CZ-20A" => CZ_20A,
"CZ-20B" => CZ_20B,
"CZ-20C" => CZ_20C,
"CZ-311" => CZ_311,
"CZ-312" => CZ_312,
"CZ-313" => CZ_313,
"CZ-314" => CZ_314,
"CZ-315" => CZ_315,
"CZ-316" => CZ_316,
"CZ-317" => CZ_317,
"CZ-321" => CZ_321,
"CZ-322" => CZ_322,
"CZ-323" => CZ_323,
"CZ-324" => CZ_324,
"CZ-325" => CZ_325,
"CZ-326" => CZ_326,
"CZ-327" => CZ_327,
"CZ-411" => CZ_411,
"CZ-412" => CZ_412,
"CZ-413" => CZ_413,
"CZ-421" => CZ_421,
"CZ-422" => CZ_422,
"CZ-423" => CZ_423,
"CZ-424" => CZ_424,
"CZ-425" => CZ_425,
"CZ-426" => CZ_426,
"CZ-427" => CZ_427,
"CZ-511" => CZ_511,
"CZ-512" => CZ_512,
"CZ-513" => CZ_513,
"CZ-514" => CZ_514,
"CZ-521" => CZ_521,
"CZ-522" => CZ_522,
"CZ-523" => CZ_523,
"CZ-524" => CZ_524,
"CZ-525" => CZ_525,
"CZ-531" => CZ_531,
"CZ-532" => CZ_532,
"CZ-533" => CZ_533,
"CZ-534" => CZ_534,
"CZ-611" => CZ_611,
"CZ-612" => CZ_612,
"CZ-613" => CZ_613,
"CZ-614" => CZ_614,
"CZ-615" => CZ_615,
"CZ-621" => CZ_621,
"CZ-622" => CZ_622,
"CZ-623" => CZ_623,
"CZ-624" => CZ_624,
"CZ-625" => CZ_625,
"CZ-626" => CZ_626,
"CZ-627" => CZ_627,
"CZ-711" => CZ_711,
"CZ-712" => CZ_712,
"CZ-713" => CZ_713,
"CZ-714" => CZ_714,
"CZ-715" => CZ_715,
"CZ-721" => CZ_721,
"CZ-722" => CZ_722,
"CZ-723" => CZ_723,
"CZ-724" => CZ_724,
"CZ-801" => CZ_801,
"CZ-802" => CZ_802,
"CZ-803" => CZ_803,
"CZ-804" => CZ_804,
"CZ-805" => CZ_805,
"CZ-806" => CZ_806,
"CZ-JC" => CZ_JC,
"CZ-JM" => CZ_JM,
"CZ-KA" => CZ_KA,
"CZ-KR" => CZ_KR,
"CZ-LI" => CZ_LI,
"CZ-MO" => CZ_MO,
"CZ-OL" => CZ_OL,
"CZ-PA" => CZ_PA,
"CZ-PL" => CZ_PL,
"CZ-PR" => CZ_PR,
"CZ-ST" => CZ_ST,
"CZ-US" => CZ_US,
"CZ-VY" => CZ_VY,
"CZ-ZL" => CZ_ZL,
"DK-81" => DK_81,
"DK-82" => DK_82,
"DK-83" => DK_83,
"DK-84" => DK_84,
"DK-85" => DK_85,
"DJ-AR" => DJ_AR,
"DJ-AS" => DJ_AS,
"DJ-DI" => DJ_DI,
"DJ-DJ" => DJ_DJ,
"DJ-OB" => DJ_OB,
"DJ-TA" => DJ_TA,
"DM-02" => DM_02,
"DM-03" => DM_03,
"DM-04" => DM_04,
"DM-05" => DM_05,
"DM-06" => DM_06,
"DM-07" => DM_07,
"DM-08" => DM_08,
"DM-09" => DM_09,
"DM-10" => DM_10,
"DM-11" => DM_11,
"DO-01" => DO_01,
"DO-02" => DO_02,
"DO-03" => DO_03,
"DO-04" => DO_04,
"DO-05" => DO_05,
"DO-06" => DO_06,
"DO-07" => DO_07,
"DO-08" => DO_08,
"DO-09" => DO_09,
"DO-10" => DO_10,
"DO-11" => DO_11,
"DO-12" => DO_12,
"DO-13" => DO_13,
"DO-14" => DO_14,
"DO-15" => DO_15,
"DO-16" => DO_16,
"DO-17" => DO_17,
"DO-18" => DO_18,
"DO-19" => DO_19,
"DO-20" => DO_20,
"DO-21" => DO_21,
"DO-22" => DO_22,
"DO-23" => DO_23,
"DO-24" => DO_24,
"DO-25" => DO_25,
"DO-26" => DO_26,
"DO-27" => DO_27,
"DO-28" => DO_28,
"DO-29" => DO_29,
"DO-30" => DO_30,
"DO-31" => DO_31,
"DO-32" => DO_32,
"DO-33" => DO_33,
"DO-34" => DO_34,
"DO-35" => DO_35,
"DO-36" => DO_36,
"DO-37" => DO_37,
"DO-38" => DO_38,
"DO-39" => DO_39,
"DO-40" => DO_40,
"DO-41" => DO_41,
"DO-42" => DO_42,
"EC-A" => EC_A,
"EC-B" => EC_B,
"EC-C" => EC_C,
"EC-D" => EC_D,
"EC-E" => EC_E,
"EC-F" => EC_F,
"EC-G" => EC_G,
"EC-H" => EC_H,
"EC-I" => EC_I,
"EC-L" => EC_L,
"EC-M" => EC_M,
"EC-N" => EC_N,
"EC-O" => EC_O,
"EC-P" => EC_P,
"EC-R" => EC_R,
"EC-SD" => EC_SD,
"EC-SE" => EC_SE,
"EC-S" => EC_S,
"EC-T" => EC_T,
"EC-U" => EC_U,
"EC-W" => EC_W,
"EC-X" => EC_X,
"EC-Y" => EC_Y,
"EC-Z" => EC_Z,
"EG-ALX" => EG_ALX,
"EG-ASN" => EG_ASN,
"EG-AST" => EG_AST,
"EG-BA" => EG_BA,
"EG-BH" => EG_BH,
"EG-BNS" => EG_BNS,
"EG-C" => EG_C,
"EG-DK" => EG_DK,
"EG-DT" => EG_DT,
"EG-FYM" => EG_FYM,
"EG-GH" => EG_GH,
"EG-GZ" => EG_GZ,
"EG-IS" => EG_IS,
"EG-JS" => EG_JS,
"EG-KB" => EG_KB,
"EG-KFS" => EG_KFS,
"EG-KN" => EG_KN,
"EG-LX" => EG_LX,
"EG-MN" => EG_MN,
"EG-MNF" => EG_MNF,
"EG-MT" => EG_MT,
"EG-PTS" => EG_PTS,
"EG-SHG" => EG_SHG,
"EG-SHR" => EG_SHR,
"EG-SIN" => EG_SIN,
"EG-SUZ" => EG_SUZ,
"EG-WAD" => EG_WAD,
"SV-AH" => SV_AH,
"SV-CA" => SV_CA,
"SV-CH" => SV_CH,
"SV-CU" => SV_CU,
"SV-LI" => SV_LI,
"SV-MO" => SV_MO,
"SV-PA" => SV_PA,
"SV-SA" => SV_SA,
"SV-SM" => SV_SM,
"SV-SO" => SV_SO,
"SV-SS" => SV_SS,
"SV-SV" => SV_SV,
"SV-UN" => SV_UN,
"SV-US" => SV_US,
"GQ-AN" => GQ_AN,
"GQ-BN" => GQ_BN,
"GQ-BS" => GQ_BS,
"GQ-C" => GQ_C,
"GQ-CS" => GQ_CS,
"GQ-I" => GQ_I,
"GQ-KN" => GQ_KN,
"GQ-LI" => GQ_LI,
"GQ-WN" => GQ_WN,
"ER-AN" => ER_AN,
"ER-DK" => ER_DK,
"ER-DU" => ER_DU,
"ER-GB" => ER_GB,
"ER-MA" => ER_MA,
"ER-SK" => ER_SK,
"EE-37" => EE_37,
"EE-39" => EE_39,
"EE-44" => EE_44,
"EE-49" => EE_49,
"EE-51" => EE_51,
"EE-57" => EE_57,
"EE-59" => EE_59,
"EE-65" => EE_65,
"EE-67" => EE_67,
"EE-70" => EE_70,
"EE-74" => EE_74,
"EE-78" => EE_78,
"EE-82" => EE_82,
"EE-84" => EE_84,
"EE-86" => EE_86,
"ET-AA" => ET_AA,
"ET-AF" => ET_AF,
"ET-AM" => ET_AM,
"ET-BE" => ET_BE,
"ET-DD" => ET_DD,
"ET-GA" => ET_GA,
"ET-HA" => ET_HA,
"ET-OR" => ET_OR,
"ET-SN" => ET_SN,
"ET-SO" => ET_SO,
"ET-TI" => ET_TI,
"FK-FK" => FK_FK,
"FO-FO" => FO_FO,
"FJ-01" => FJ_01,
"FJ-02" => FJ_02,
"FJ-03" => FJ_03,
"FJ-04" => FJ_04,
"FJ-05" => FJ_05,
"FJ-06" => FJ_06,
"FJ-07" => FJ_07,
"FJ-08" => FJ_08,
"FJ-09" => FJ_09,
"FJ-10" => FJ_10,
"FJ-11" => FJ_11,
"FJ-12" => FJ_12,
"FJ-13" => FJ_13,
"FJ-14" => FJ_14,
"FJ-C" => FJ_C,
"FJ-E" => FJ_E,
"FJ-N" => FJ_N,
"FJ-R" => FJ_R,
"FJ-W" => FJ_W,
"FI-01" => FI_01,
"FI-02" => FI_02,
"FI-03" => FI_03,
"FI-04" => FI_04,
"FI-05" => FI_05,
"FI-06" => FI_06,
"FI-07" => FI_07,
"FI-08" => FI_08,
"FI-09" => FI_09,
"FI-10" => FI_10,
"FI-11" => FI_11,
"FI-12" => FI_12,
"FI-13" => FI_13,
"FI-14" => FI_14,
"FI-15" => FI_15,
"FI-16" => FI_16,
"FI-17" => FI_17,
"FI-18" => FI_18,
"FI-19" => FI_19,
"FR-01" => FR_01,
"FR-02" => FR_02,
"FR-03" => FR_03,
"FR-04" => FR_04,
"FR-05" => FR_05,
"FR-06" => FR_06,
"FR-07" => FR_07,
"FR-08" => FR_08,
"FR-09" => FR_09,
"FR-10" => FR_10,
"FR-11" => FR_11,
"FR-12" => FR_12,
"FR-13" => FR_13,
"FR-14" => FR_14,
"FR-15" => FR_15,
"FR-16" => FR_16,
"FR-17" => FR_17,
"FR-18" => FR_18,
"FR-19" => FR_19,
"FR-21" => FR_21,
"FR-22" => FR_22,
"FR-23" => FR_23,
"FR-24" => FR_24,
"FR-25" => FR_25,
"FR-26" => FR_26,
"FR-27" => FR_27,
"FR-28" => FR_28,
"FR-29" => FR_29,
"FR-2A" => FR_2A,
"FR-2B" => FR_2B,
"FR-30" => FR_30,
"FR-31" => FR_31,
"FR-32" => FR_32,
"FR-33" => FR_33,
"FR-34" => FR_34,
"FR-35" => FR_35,
"FR-36" => FR_36,
"FR-37" => FR_37,
"FR-38" => FR_38,
"FR-39" => FR_39,
"FR-40" => FR_40,
"FR-41" => FR_41,
"FR-42" => FR_42,
"FR-43" => FR_43,
"FR-44" => FR_44,
"FR-45" => FR_45,
"FR-46" => FR_46,
"FR-47" => FR_47,
"FR-48" => FR_48,
"FR-49" => FR_49,
"FR-50" => FR_50,
"FR-51" => FR_51,
"FR-52" => FR_52,
"FR-53" => FR_53,
"FR-54" => FR_54,
"FR-55" => FR_55,
"FR-56" => FR_56,
"FR-57" => FR_57,
"FR-58" => FR_58,
"FR-59" => FR_59,
"FR-60" => FR_60,
"FR-61" => FR_61,
"FR-62" => FR_62,
"FR-63" => FR_63,
"FR-64" => FR_64,
"FR-65" => FR_65,
"FR-66" => FR_66,
"FR-67" => FR_67,
"FR-68" => FR_68,
"FR-69" => FR_69,
"FR-70" => FR_70,
"FR-71" => FR_71,
"FR-72" => FR_72,
"FR-73" => FR_73,
"FR-74" => FR_74,
"FR-75" => FR_75,
"FR-76" => FR_76,
"FR-77" => FR_77,
"FR-78" => FR_78,
"FR-79" => FR_79,
"FR-80" => FR_80,
"FR-81" => FR_81,
"FR-82" => FR_82,
"FR-83" => FR_83,
"FR-84" => FR_84,
"FR-85" => FR_85,
"FR-86" => FR_86,
"FR-87" => FR_87,
"FR-88" => FR_88,
"FR-89" => FR_89,
"FR-90" => FR_90,
"FR-91" => FR_91,
"FR-92" => FR_92,
"FR-93" => FR_93,
"FR-94" => FR_94,
"FR-95" => FR_95,
"FR-A" => FR_A,
"FR-B" => FR_B,
"FR-BL" => FR_BL,
"FR-C" => FR_C,
"FR-CP" => FR_CP,
"FR-D" => FR_D,
"FR-E" => FR_E,
"FR-F" => FR_F,
"FR-G" => FR_G,
"FR-GF" => FR_GF,
"FR-GP" => FR_GP,
"FR-H" => FR_H,
"FR-I" => FR_I,
"FR-J" => FR_J,
"FR-K" => FR_K,
"FR-L" => FR_L,
"FR-MF" => FR_MF,
"FR-M" => FR_M,
"FR-MQ" => FR_MQ,
"FR-NC" => FR_NC,
"FR-N" => FR_N,
"FR-O" => FR_O,
"FR-P" => FR_P,
"FR-PF" => FR_PF,
"FR-PM" => FR_PM,
"FR-Q" => FR_Q,
"FR-RE" => FR_RE,
"FR-R" => FR_R,
"FR-S" => FR_S,
"FR-TF" => FR_TF,
"FR-T" => FR_T,
"FR-U" => FR_U,
"FR-V" => FR_V,
"FR-WF" => FR_WF,
"FR-YT" => FR_YT,
"FR-ARA" => FR_ARA,
"FR-BFC" => FR_BFC,
"FR-BRE" => FR_BRE,
"FR-CVL" => FR_CVL,
"FR-20R" => FR_20R,
"FR-GES" => FR_GES,
"FR-HDF" => FR_HDF,
"FR-IDF" => FR_IDF,
"FR-NOR" => FR_NOR,
"FR-NAQ" => FR_NAQ,
"FR-OCC" => FR_OCC,
"FR-PDL" => FR_PDL,
"FR-PAC" => FR_PAC,
"GF-GF" => GF_GF,
"PF-PF" => PF_PF,
"TF-TF" => TF_TF,
"GA-1" => GA_1,
"GA-2" => GA_2,
"GA-3" => GA_3,
"GA-4" => GA_4,
"GA-5" => GA_5,
"GA-6" => GA_6,
"GA-7" => GA_7,
"GA-8" => GA_8,
"GA-9" => GA_9,
"GM-B" => GM_B,
"GM-L" => GM_L,
"GM-M" => GM_M,
"GM-N" => GM_N,
"GM-U" => GM_U,
"GM-W" => GM_W,
"GE-AB" => GE_AB,
"GE-AJ" => GE_AJ,
"GE-GU" => GE_GU,
"GE-IM" => GE_IM,
"GE-KA" => GE_KA,
"GE-KK" => GE_KK,
"GE-MM" => GE_MM,
"GE-RL" => GE_RL,
"GE-SJ" => GE_SJ,
"GE-SK" => GE_SK,
"GE-SZ" => GE_SZ,
"GE-TB" => GE_TB,
"DE-BB" => DE_BB,
"DE-BE" => DE_BE,
"DE-BW" => DE_BW,
"DE-BY" => DE_BY,
"DE-HB" => DE_HB,
"DE-HE" => DE_HE,
"DE-HH" => DE_HH,
"DE-MV" => DE_MV,
"DE-NI" => DE_NI,
"DE-NW" => DE_NW,
"DE-RP" => DE_RP,
"DE-SH" => DE_SH,
"DE-SL" => DE_SL,
"DE-SN" => DE_SN,
"DE-ST" => DE_ST,
"DE-TH" => DE_TH,
"GH-AA" => GH_AA,
"GH-AH" => GH_AH,
"GH-BA" => GH_BA,
"GH-CP" => GH_CP,
"GH-EP" => GH_EP,
"GH-NP" => GH_NP,
"GH-TV" => GH_TV,
"GH-UE" => GH_UE,
"GH-UW" => GH_UW,
"GH-WP" => GH_WP,
"GI-GI" => GI_GI,
"GR-01" => GR_01,
"GR-03" => GR_03,
"GR-04" => GR_04,
"GR-05" => GR_05,
"GR-06" => GR_06,
"GR-07" => GR_07,
"GR-11" => GR_11,
"GR-12" => GR_12,
"GR-13" => GR_13,
"GR-14" => GR_14,
"GR-15" => GR_15,
"GR-16" => GR_16,
"GR-17" => GR_17,
"GR-21" => GR_21,
"GR-22" => GR_22,
"GR-23" => GR_23,
"GR-24" => GR_24,
"GR-31" => GR_31,
"GR-32" => GR_32,
"GR-33" => GR_33,
"GR-34" => GR_34,
"GR-41" => GR_41,
"GR-42" => GR_42,
"GR-43" => GR_43,
"GR-44" => GR_44,
"GR-51" => GR_51,
"GR-52" => GR_52,
"GR-53" => GR_53,
"GR-54" => GR_54,
"GR-55" => GR_55,
"GR-56" => GR_56,
"GR-57" => GR_57,
"GR-58" => GR_58,
"GR-59" => GR_59,
"GR-61" => GR_61,
"GR-62" => GR_62,
"GR-63" => GR_63,
"GR-64" => GR_64,
"GR-69" => GR_69,
"GR-71" => GR_71,
"GR-72" => GR_72,
"GR-73" => GR_73,
"GR-81" => GR_81,
"GR-82" => GR_82,
"GR-83" => GR_83,
"GR-84" => GR_84,
"GR-85" => GR_85,
"GR-91" => GR_91,
"GR-92" => GR_92,
"GR-93" => GR_93,
"GR-94" => GR_94,
"GR-A1" => GR_A1,
"GR-A" => GR_A,
"GR-B" => GR_B,
"GR-C" => GR_C,
"GR-D" => GR_D,
"GR-E" => GR_E,
"GR-F" => GR_F,
"GR-G" => GR_G,
"GR-H" => GR_H,
"GR-I" => GR_I,
"GR-J" => GR_J,
"GR-K" => GR_K,
"GR-L" => GR_L,
"GR-M" => GR_M,
"GL-KU" => GL_KU,
"GL-QA" => GL_QA,
"GL-QE" => GL_QE,
"GL-SM" => GL_SM,
"GD-01" => GD_01,
"GD-02" => GD_02,
"GD-03" => GD_03,
"GD-04" => GD_04,
"GD-05" => GD_05,
"GD-06" => GD_06,
"GD-10" => GD_10,
"GP-GP" => GP_GP,
"GU-GU" => GU_GU,
"GT-AV" => GT_AV,
"GT-BV" => GT_BV,
"GT-CM" => GT_CM,
"GT-CQ" => GT_CQ,
"GT-ES" => GT_ES,
"GT-GU" => GT_GU,
"GT-HU" => GT_HU,
"GT-IZ" => GT_IZ,
"GT-JA" => GT_JA,
"GT-JU" => GT_JU,
"GT-PE" => GT_PE,
"GT-PR" => GT_PR,
"GT-QC" => GT_QC,
"GT-QZ" => GT_QZ,
"GT-RE" => GT_RE,
"GT-SA" => GT_SA,
"GT-SM" => GT_SM,
"GT-SO" => GT_SO,
"GT-SR" => GT_SR,
"GT-SU" => GT_SU,
"GT-TO" => GT_TO,
"GT-ZA" => GT_ZA,
"GG-GG" => GG_GG,
"GW-BA" => GW_BA,
"GW-BL" => GW_BL,
"GW-BM" => GW_BM,
"GW-BS" => GW_BS,
"GW-CA" => GW_CA,
"GW-GA" => GW_GA,
"GW-L" => GW_L,
"GW-N" => GW_N,
"GW-OI" => GW_OI,
"GW-QU" => GW_QU,
"GW-S" => GW_S,
"GW-TO" => GW_TO,
"GN-B" => GN_B,
"GN-BE" => GN_BE,
"GN-BF" => GN_BF,
"GN-BK" => GN_BK,
"GN-C" => GN_C,
"GN-CO" => GN_CO,
"GN-DB" => GN_DB,
"GN-DI" => GN_DI,
"GN-D" => GN_D,
"GN-DL" => GN_DL,
"GN-DU" => GN_DU,
"GN-FA" => GN_FA,
"GN-F" => GN_F,
"GN-FO" => GN_FO,
"GN-FR" => GN_FR,
"GN-GA" => GN_GA,
"GN-GU" => GN_GU,
"GN-KA" => GN_KA,
"GN-KB" => GN_KB,
"GN-KD" => GN_KD,
"GN-KE" => GN_KE,
"GN-K" => GN_K,
"GN-KN" => GN_KN,
"GN-KO" => GN_KO,
"GN-KS" => GN_KS,
"GN-LA" => GN_LA,
"GN-LE" => GN_LE,
"GN-L" => GN_L,
"GN-LO" => GN_LO,
"GN-MC" => GN_MC,
"GN-MD" => GN_MD,
"GN-ML" => GN_ML,
"GN-M" => GN_M,
"GN-MM" => GN_MM,
"GN-N" => GN_N,
"GN-NZ" => GN_NZ,
"GN-PI" => GN_PI,
"GN-SI" => GN_SI,
"GN-TE" => GN_TE,
"GN-TO" => GN_TO,
"GN-YO" => GN_YO,
"GY-BA" => GY_BA,
"GY-CU" => GY_CU,
"GY-DE" => GY_DE,
"GY-EB" => GY_EB,
"GY-ES" => GY_ES,
"GY-MA" => GY_MA,
"GY-PM" => GY_PM,
"GY-PT" => GY_PT,
"GY-UD" => GY_UD,
"GY-UT" => GY_UT,
"HT-AR" => HT_AR,
"HT-CE" => HT_CE,
"HT-GA" => HT_GA,
"HT-ND" => HT_ND,
"HT-NE" => HT_NE,
"HT-NI" => HT_NI,
"HT-NO" => HT_NO,
"HT-OU" => HT_OU,
"HT-SD" => HT_SD,
"HT-SE" => HT_SE,
"HM-HM" => HM_HM,
"HN-AT" => HN_AT,
"HN-CH" => HN_CH,
"HN-CL" => HN_CL,
"HN-CM" => HN_CM,
"HN-CP" => HN_CP,
"HN-CR" => HN_CR,
"HN-EP" => HN_EP,
"HN-FM" => HN_FM,
"HN-GD" => HN_GD,
"HN-IB" => HN_IB,
"HN-IN" => HN_IN,
"HN-LE" => HN_LE,
"HN-LP" => HN_LP,
"HN-OC" => HN_OC,
"HN-OL" => HN_OL,
"HN-SB" => HN_SB,
"HN-VA" => HN_VA,
"HN-YO" => HN_YO,
"HK-HK" => HK_HK,
"HU-BA" => HU_BA,
"HU-BC" => HU_BC,
"HU-BE" => HU_BE,
"HU-BK" => HU_BK,
"HU-BU" => HU_BU,
"HU-BZ" => HU_BZ,
"HU-CS" => HU_CS,
"HU-DE" => HU_DE,
"HU-DU" => HU_DU,
"HU-EG" => HU_EG,
"HU-ER" => HU_ER,
"HU-FE" => HU_FE,
"HU-GS" => HU_GS,
"HU-GY" => HU_GY,
"HU-HB" => HU_HB,
"HU-HE" => HU_HE,
"HU-HV" => HU_HV,
"HU-JN" => HU_JN,
"HU-KE" => HU_KE,
"HU-KM" => HU_KM,
"HU-KV" => HU_KV,
"HU-MI" => HU_MI,
"HU-NK" => HU_NK,
"HU-NO" => HU_NO,
"HU-NY" => HU_NY,
"HU-PE" => HU_PE,
"HU-PS" => HU_PS,
"HU-SD" => HU_SD,
"HU-SF" => HU_SF,
"HU-SH" => HU_SH,
"HU-SK" => HU_SK,
"HU-SN" => HU_SN,
"HU-SO" => HU_SO,
"HU-SS" => HU_SS,
"HU-ST" => HU_ST,
"HU-SZ" => HU_SZ,
"HU-TB" => HU_TB,
"HU-TO" => HU_TO,
"HU-VA" => HU_VA,
"HU-VE" => HU_VE,
"HU-VM" => HU_VM,
"HU-ZA" => HU_ZA,
"HU-ZE" => HU_ZE,
"IS-0" => IS_0,
"IS-1" => IS_1,
"IS-2" => IS_2,
"IS-3" => IS_3,
"IS-4" => IS_4,
"IS-5" => IS_5,
"IS-6" => IS_6,
"IS-7" => IS_7,
"IS-8" => IS_8,
"IN-AN" => IN_AN,
"IN-AP" => IN_AP,
"IN-AR" => IN_AR,
"IN-AS" => IN_AS,
"IN-BR" => IN_BR,
"IN-CH" => IN_CH,
"IN-CT" => IN_CT,
"IN-DD" => IN_DD,
"IN-DL" => IN_DL,
"IN-DN" => IN_DN,
"IN-GA" => IN_GA,
"IN-GJ" => IN_GJ,
"IN-HP" => IN_HP,
"IN-HR" => IN_HR,
"IN-JH" => IN_JH,
"IN-JK" => IN_JK,
"IN-KA" => IN_KA,
"IN-KL" => IN_KL,
"IN-LD" => IN_LD,
"IN-MH" => IN_MH,
"IN-ML" => IN_ML,
"IN-MN" => IN_MN,
"IN-MP" => IN_MP,
"IN-MZ" => IN_MZ,
"IN-NL" => IN_NL,
"IN-OR" => IN_OR,
"IN-PB" => IN_PB,
"IN-PY" => IN_PY,
"IN-RJ" => IN_RJ,
"IN-SK" => IN_SK,
"IN-TG" => IN_TG,
"IN-TN" => IN_TN,
"IN-TR" => IN_TR,
"IN-UP" => IN_UP,
"IN-UT" => IN_UT,
"IN-WB" => IN_WB,
"ID-AC" => ID_AC,
"ID-BA" => ID_BA,
"ID-BB" => ID_BB,
"ID-BE" => ID_BE,
"ID-BT" => ID_BT,
"ID-GO" => ID_GO,
"ID-JA" => ID_JA,
"ID-JB" => ID_JB,
"ID-JI" => ID_JI,
"ID-JK" => ID_JK,
"ID-JT" => ID_JT,
"ID-JW" => ID_JW,
"ID-KA" => ID_KA,
"ID-KB" => ID_KB,
"ID-KI" => ID_KI,
"ID-KR" => ID_KR,
"ID-KS" => ID_KS,
"ID-KT" => ID_KT,
"ID-KU" => ID_KU,
"ID-LA" => ID_LA,
"ID-MA" => ID_MA,
"ID-ML" => ID_ML,
"ID-MU" => ID_MU,
"ID-NB" => ID_NB,
"ID-NT" => ID_NT,
"ID-NU" => ID_NU,
"ID-PA" => ID_PA,
"ID-PB" => ID_PB,
"ID-PP" => ID_PP,
"ID-RI" => ID_RI,
"ID-SA" => ID_SA,
"ID-SB" => ID_SB,
"ID-SG" => ID_SG,
"ID-SL" => ID_SL,
"ID-SM" => ID_SM,
"ID-SN" => ID_SN,
"ID-SR" => ID_SR,
"ID-SS" => ID_SS,
"ID-ST" => ID_ST,
"ID-SU" => ID_SU,
"ID-YO" => ID_YO,
"IR-01" => IR_01,
"IR-02" => IR_02,
"IR-03" => IR_03,
"IR-04" => IR_04,
"IR-05" => IR_05,
"IR-06" => IR_06,
"IR-07" => IR_07,
"IR-08" => IR_08,
"IR-10" => IR_10,
"IR-11" => IR_11,
"IR-12" => IR_12,
"IR-13" => IR_13,
"IR-14" => IR_14,
"IR-15" => IR_15,
"IR-16" => IR_16,
"IR-17" => IR_17,
"IR-18" => IR_18,
"IR-19" => IR_19,
"IR-20" => IR_20,
"IR-21" => IR_21,
"IR-22" => IR_22,
"IR-23" => IR_23,
"IR-24" => IR_24,
"IR-25" => IR_25,
"IR-26" => IR_26,
"IR-27" => IR_27,
"IR-28" => IR_28,
"IR-29" => IR_29,
"IR-30" => IR_30,
"IR-31" => IR_31,
"IR-32" => IR_32,
"IQ-AN" => IQ_AN,
"IQ-AR" => IQ_AR,
"IQ-BA" => IQ_BA,
"IQ-BB" => IQ_BB,
"IQ-BG" => IQ_BG,
"IQ-DA" => IQ_DA,
"IQ-DI" => IQ_DI,
"IQ-DQ" => IQ_DQ,
"IQ-KA" => IQ_KA,
"IQ-KI" => IQ_KI,
"IQ-MA" => IQ_MA,
"IQ-MU" => IQ_MU,
"IQ-NA" => IQ_NA,
"IQ-NI" => IQ_NI,
"IQ-QA" => IQ_QA,
"IQ-SD" => IQ_SD,
"IQ-SU" => IQ_SU,
"IQ-WA" => IQ_WA,
"IE-C" => IE_C,
"IE-CE" => IE_CE,
"IE-CN" => IE_CN,
"IE-CO" => IE_CO,
"IE-CW" => IE_CW,
"IE-D" => IE_D,
"IE-DL" => IE_DL,
"IE-G" => IE_G,
"IE-KE" => IE_KE,
"IE-KK" => IE_KK,
"IE-KY" => IE_KY,
"IE-LD" => IE_LD,
"IE-LH" => IE_LH,
"IE-LK" => IE_LK,
"IE-L" => IE_L,
"IE-LM" => IE_LM,
"IE-LS" => IE_LS,
"IE-M" => IE_M,
"IE-MH" => IE_MH,
"IE-MN" => IE_MN,
"IE-MO" => IE_MO,
"IE-OY" => IE_OY,
"IE-RN" => IE_RN,
"IE-SO" => IE_SO,
"IE-TA" => IE_TA,
"IE-U" => IE_U,
"IE-WD" => IE_WD,
"IE-WH" => IE_WH,
"IE-WW" => IE_WW,
"IE-WX" => IE_WX,
"IM-IM" => IM_IM,
"IL-D" => IL_D,
"IL-HA" => IL_HA,
"IL-JM" => IL_JM,
"IL-M" => IL_M,
"IL-TA" => IL_TA,
"IL-Z" => IL_Z,
"IT-21" => IT_21,
"IT-23" => IT_23,
"IT-25" => IT_25,
"IT-32" => IT_32,
"IT-34" => IT_34,
"IT-36" => IT_36,
"IT-42" => IT_42,
"IT-45" => IT_45,
"IT-52" => IT_52,
"IT-55" => IT_55,
"IT-57" => IT_57,
"IT-62" => IT_62,
"IT-65" => IT_65,
"IT-67" => IT_67,
"IT-72" => IT_72,
"IT-75" => IT_75,
"IT-77" => IT_77,
"IT-78" => IT_78,
"IT-82" => IT_82,
"IT-88" => IT_88,
"IT-AG" => IT_AG,
"IT-AL" => IT_AL,
"IT-AN" => IT_AN,
"IT-AO" => IT_AO,
"IT-AP" => IT_AP,
"IT-AQ" => IT_AQ,
"IT-AR" => IT_AR,
"IT-AT" => IT_AT,
"IT-AV" => IT_AV,
"IT-BA" => IT_BA,
"IT-BG" => IT_BG,
"IT-BI" => IT_BI,
"IT-BL" => IT_BL,
"IT-BN" => IT_BN,
"IT-BO" => IT_BO,
"IT-BR" => IT_BR,
"IT-BS" => IT_BS,
"IT-BT" => IT_BT,
"IT-BZ" => IT_BZ,
"IT-CA" => IT_CA,
"IT-CB" => IT_CB,
"IT-CE" => IT_CE,
"IT-CH" => IT_CH,
"IT-CI" => IT_CI,
"IT-CL" => IT_CL,
"IT-CN" => IT_CN,
"IT-CO" => IT_CO,
"IT-CR" => IT_CR,
"IT-CS" => IT_CS,
"IT-CT" => IT_CT,
"IT-CZ" => IT_CZ,
"IT-EN" => IT_EN,
"IT-FC" => IT_FC,
"IT-FE" => IT_FE,
"IT-FG" => IT_FG,
"IT-FI" => IT_FI,
"IT-FM" => IT_FM,
"IT-FR" => IT_FR,
"IT-GE" => IT_GE,
"IT-GO" => IT_GO,
"IT-GR" => IT_GR,
"IT-IM" => IT_IM,
"IT-IS" => IT_IS,
"IT-KR" => IT_KR,
"IT-LC" => IT_LC,
"IT-LE" => IT_LE,
"IT-LI" => IT_LI,
"IT-LO" => IT_LO,
"IT-LT" => IT_LT,
"IT-LU" => IT_LU,
"IT-MB" => IT_MB,
"IT-MC" => IT_MC,
"IT-ME" => IT_ME,
"IT-MI" => IT_MI,
"IT-MN" => IT_MN,
"IT-MO" => IT_MO,
"IT-MS" => IT_MS,
"IT-MT" => IT_MT,
"IT-NA" => IT_NA,
"IT-NO" => IT_NO,
"IT-NU" => IT_NU,
"IT-OG" => IT_OG,
"IT-OR" => IT_OR,
"IT-OT" => IT_OT,
"IT-PA" => IT_PA,
"IT-PC" => IT_PC,
"IT-PD" => IT_PD,
"IT-PE" => IT_PE,
"IT-PG" => IT_PG,
"IT-PI" => IT_PI,
"IT-PN" => IT_PN,
"IT-PO" => IT_PO,
"IT-PR" => IT_PR,
"IT-PT" => IT_PT,
"IT-PU" => IT_PU,
"IT-PV" => IT_PV,
"IT-PZ" => IT_PZ,
"IT-RA" => IT_RA,
"IT-RC" => IT_RC,
"IT-RE" => IT_RE,
"IT-RG" => IT_RG,
"IT-RI" => IT_RI,
"IT-RM" => IT_RM,
"IT-RN" => IT_RN,
"IT-RO" => IT_RO,
"IT-SA" => IT_SA,
"IT-SI" => IT_SI,
"IT-SO" => IT_SO,
"IT-SP" => IT_SP,
"IT-SR" => IT_SR,
"IT-SS" => IT_SS,
"IT-SV" => IT_SV,
"IT-TA" => IT_TA,
"IT-TE" => IT_TE,
"IT-TN" => IT_TN,
"IT-TO" => IT_TO,
"IT-TP" => IT_TP,
"IT-TR" => IT_TR,
"IT-TS" => IT_TS,
"IT-TV" => IT_TV,
"IT-UD" => IT_UD,
"IT-VA" => IT_VA,
"IT-VB" => IT_VB,
"IT-VC" => IT_VC,
"IT-VE" => IT_VE,
"IT-VI" => IT_VI,
"IT-VR" => IT_VR,
"IT-VS" => IT_VS,
"IT-VT" => IT_VT,
"IT-VV" => IT_VV,
"JM-01" => JM_01,
"JM-02" => JM_02,
"JM-03" => JM_03,
"JM-04" => JM_04,
"JM-05" => JM_05,
"JM-06" => JM_06,
"JM-07" => JM_07,
"JM-08" => JM_08,
"JM-09" => JM_09,
"JM-10" => JM_10,
"JM-11" => JM_11,
"JM-12" => JM_12,
"JM-13" => JM_13,
"JM-14" => JM_14,
"JP-01" => JP_01,
"JP-02" => JP_02,
"JP-03" => JP_03,
"JP-04" => JP_04,
"JP-05" => JP_05,
"JP-06" => JP_06,
"JP-07" => JP_07,
"JP-08" => JP_08,
"JP-09" => JP_09,
"JP-10" => JP_10,
"JP-11" => JP_11,
"JP-12" => JP_12,
"JP-13" => JP_13,
"JP-14" => JP_14,
"JP-15" => JP_15,
"JP-16" => JP_16,
"JP-17" => JP_17,
"JP-18" => JP_18,
"JP-19" => JP_19,
"JP-20" => JP_20,
"JP-21" => JP_21,
"JP-22" => JP_22,
"JP-23" => JP_23,
"JP-24" => JP_24,
"JP-25" => JP_25,
"JP-26" => JP_26,
"JP-27" => JP_27,
"JP-28" => JP_28,
"JP-29" => JP_29,
"JP-30" => JP_30,
"JP-31" => JP_31,
"JP-32" => JP_32,
"JP-33" => JP_33,
"JP-34" => JP_34,
"JP-35" => JP_35,
"JP-36" => JP_36,
"JP-37" => JP_37,
"JP-38" => JP_38,
"JP-39" => JP_39,
"JP-40" => JP_40,
"JP-41" => JP_41,
"JP-42" => JP_42,
"JP-43" => JP_43,
"JP-44" => JP_44,
"JP-45" => JP_45,
"JP-46" => JP_46,
"JP-47" => JP_47,
"JE-JE" => JE_JE,
"JO-AJ" => JO_AJ,
"JO-AM" => JO_AM,
"JO-AQ" => JO_AQ,
"JO-AT" => JO_AT,
"JO-AZ" => JO_AZ,
"JO-BA" => JO_BA,
"JO-IR" => JO_IR,
"JO-JA" => JO_JA,
"JO-KA" => JO_KA,
"JO-MA" => JO_MA,
"JO-MD" => JO_MD,
"JO-MN" => JO_MN,
"KZ-AKM" => KZ_AKM,
"KZ-AKT" => KZ_AKT,
"KZ-ALA" => KZ_ALA,
"KZ-ALM" => KZ_ALM,
"KZ-AST" => KZ_AST,
"KZ-ATY" => KZ_ATY,
"KZ-KAR" => KZ_KAR,
"KZ-KUS" => KZ_KUS,
"KZ-KZY" => KZ_KZY,
"KZ-MAN" => KZ_MAN,
"KZ-PAV" => KZ_PAV,
"KZ-SEV" => KZ_SEV,
"KZ-VOS" => KZ_VOS,
"KZ-YUZ" => KZ_YUZ,
"KZ-ZAP" => KZ_ZAP,
"KZ-ZHA" => KZ_ZHA,
"KE-01" => KE_01,
"KE-02" => KE_02,
"KE-03" => KE_03,
"KE-04" => KE_04,
"KE-05" => KE_05,
"KE-06" => KE_06,
"KE-07" => KE_07,
"KE-08" => KE_08,
"KE-09" => KE_09,
"KE-10" => KE_10,
"KE-11" => KE_11,
"KE-12" => KE_12,
"KE-13" => KE_13,
"KE-14" => KE_14,
"KE-15" => KE_15,
"KE-16" => KE_16,
"KE-17" => KE_17,
"KE-18" => KE_18,
"KE-19" => KE_19,
"KE-20" => KE_20,
"KE-21" => KE_21,
"KE-22" => KE_22,
"KE-23" => KE_23,
"KE-24" => KE_24,
"KE-25" => KE_25,
"KE-26" => KE_26,
"KE-27" => KE_27,
"KE-28" => KE_28,
"KE-29" => KE_29,
"KE-30" => KE_30,
"KE-31" => KE_31,
"KE-32" => KE_32,
"KE-33" => KE_33,
"KE-34" => KE_34,
"KE-35" => KE_35,
"KE-36" => KE_36,
"KE-37" => KE_37,
"KE-38" => KE_38,
"KE-39" => KE_39,
"KE-40" => KE_40,
"KE-41" => KE_41,
"KE-42" => KE_42,
"KE-43" => KE_43,
"KE-44" => KE_44,
"KE-45" => KE_45,
"KE-46" => KE_46,
"KE-47" => KE_47,
"KI-G" => KI_G,
"KI-L" => KI_L,
"KI-P" => KI_P,
"KP-01" => KP_01,
"KP-02" => KP_02,
"KP-03" => KP_03,
"KP-04" => KP_04,
"KP-05" => KP_05,
"KP-06" => KP_06,
"KP-07" => KP_07,
"KP-08" => KP_08,
"KP-09" => KP_09,
"KP-10" => KP_10,
"KP-13" => KP_13,
"KR-11" => KR_11,
"KR-26" => KR_26,
"KR-27" => KR_27,
"KR-28" => KR_28,
"KR-29" => KR_29,
"KR-30" => KR_30,
"KR-31" => KR_31,
"KR-41" => KR_41,
"KR-42" => KR_42,
"KR-43" => KR_43,
"KR-44" => KR_44,
"KR-45" => KR_45,
"KR-46" => KR_46,
"KR-47" => KR_47,
"KR-48" => KR_48,
"KR-49" => KR_49,
"KR-50" => KR_50,
"KW-AH" => KW_AH,
"KW-FA" => KW_FA,
"KW-HA" => KW_HA,
"KW-JA" => KW_JA,
"KW-KU" => KW_KU,
"KW-MU" => KW_MU,
"KG-B" => KG_B,
"KG-C" => KG_C,
"KG-GB" => KG_GB,
"KG-GO" => KG_GO,
"KG-J" => KG_J,
"KG-N" => KG_N,
"KG-O" => KG_O,
"KG-T" => KG_T,
"KG-Y" => KG_Y,
"LA-AT" => LA_AT,
"LA-BK" => LA_BK,
"LA-BL" => LA_BL,
"LA-CH" => LA_CH,
"LA-HO" => LA_HO,
"LA-KH" => LA_KH,
"LA-LM" => LA_LM,
"LA-LP" => LA_LP,
"LA-OU" => LA_OU,
"LA-PH" => LA_PH,
"LA-SL" => LA_SL,
"LA-SV" => LA_SV,
"LA-VI" => LA_VI,
"LA-VT" => LA_VT,
"LA-XA" => LA_XA,
"LA-XE" => LA_XE,
"LA-XI" => LA_XI,
"LA-XN" => LA_XN,
"LV-001" => LV_001,
"LV-002" => LV_002,
"LV-003" => LV_003,
"LV-004" => LV_004,
"LV-005" => LV_005,
"LV-006" => LV_006,
"LV-007" => LV_007,
"LV-008" => LV_008,
"LV-009" => LV_009,
"LV-010" => LV_010,
"LV-011" => LV_011,
"LV-012" => LV_012,
"LV-013" => LV_013,
"LV-014" => LV_014,
"LV-015" => LV_015,
"LV-016" => LV_016,
"LV-017" => LV_017,
"LV-018" => LV_018,
"LV-019" => LV_019,
"LV-020" => LV_020,
"LV-021" => LV_021,
"LV-022" => LV_022,
"LV-023" => LV_023,
"LV-024" => LV_024,
"LV-025" => LV_025,
"LV-026" => LV_026,
"LV-027" => LV_027,
"LV-028" => LV_028,
"LV-029" => LV_029,
"LV-030" => LV_030,
"LV-031" => LV_031,
"LV-032" => LV_032,
"LV-033" => LV_033,
"LV-034" => LV_034,
"LV-035" => LV_035,
"LV-036" => LV_036,
"LV-037" => LV_037,
"LV-038" => LV_038,
"LV-039" => LV_039,
"LV-040" => LV_040,
"LV-041" => LV_041,
"LV-042" => LV_042,
"LV-043" => LV_043,
"LV-044" => LV_044,
"LV-045" => LV_045,
"LV-046" => LV_046,
"LV-047" => LV_047,
"LV-048" => LV_048,
"LV-049" => LV_049,
"LV-050" => LV_050,
"LV-051" => LV_051,
"LV-052" => LV_052,
"LV-053" => LV_053,
"LV-054" => LV_054,
"LV-055" => LV_055,
"LV-056" => LV_056,
"LV-057" => LV_057,
"LV-058" => LV_058,
"LV-059" => LV_059,
"LV-060" => LV_060,
"LV-061" => LV_061,
"LV-062" => LV_062,
"LV-063" => LV_063,
"LV-064" => LV_064,
"LV-065" => LV_065,
"LV-066" => LV_066,
"LV-067" => LV_067,
"LV-068" => LV_068,
"LV-069" => LV_069,
"LV-070" => LV_070,
"LV-071" => LV_071,
"LV-072" => LV_072,
"LV-073" => LV_073,
"LV-074" => LV_074,
"LV-075" => LV_075,
"LV-076" => LV_076,
"LV-077" => LV_077,
"LV-078" => LV_078,
"LV-079" => LV_079,
"LV-080" => LV_080,
"LV-081" => LV_081,
"LV-082" => LV_082,
"LV-083" => LV_083,
"LV-084" => LV_084,
"LV-085" => LV_085,
"LV-086" => LV_086,
"LV-087" => LV_087,
"LV-088" => LV_088,
"LV-089" => LV_089,
"LV-090" => LV_090,
"LV-091" => LV_091,
"LV-092" => LV_092,
"LV-093" => LV_093,
"LV-094" => LV_094,
"LV-095" => LV_095,
"LV-096" => LV_096,
"LV-097" => LV_097,
"LV-098" => LV_098,
"LV-099" => LV_099,
"LV-100" => LV_100,
"LV-101" => LV_101,
"LV-102" => LV_102,
"LV-103" => LV_103,
"LV-104" => LV_104,
"LV-105" => LV_105,
"LV-106" => LV_106,
"LV-107" => LV_107,
"LV-108" => LV_108,
"LV-109" => LV_109,
"LV-110" => LV_110,
"LV-DGV" => LV_DGV,
"LV-JEL" => LV_JEL,
"LV-JKB" => LV_JKB,
"LV-JUR" => LV_JUR,
"LV-LPX" => LV_LPX,
"LV-REZ" => LV_REZ,
"LV-RIX" => LV_RIX,
"LV-VEN" => LV_VEN,
"LV-VMR" => LV_VMR,
"LB-AK" => LB_AK,
"LB-AS" => LB_AS,
"LB-BA" => LB_BA,
"LB-BH" => LB_BH,
"LB-BI" => LB_BI,
"LB-JA" => LB_JA,
"LB-JL" => LB_JL,
"LB-NA" => LB_NA,
"LS-A" => LS_A,
"LS-B" => LS_B,
"LS-C" => LS_C,
"LS-D" => LS_D,
"LS-E" => LS_E,
"LS-F" => LS_F,
"LS-G" => LS_G,
"LS-H" => LS_H,
"LS-J" => LS_J,
"LS-K" => LS_K,
"LR-BG" => LR_BG,
"LR-BM" => LR_BM,
"LR-CM" => LR_CM,
"LR-GB" => LR_GB,
"LR-GG" => LR_GG,
"LR-GK" => LR_GK,
"LR-GP" => LR_GP,
"LR-LO" => LR_LO,
"LR-MG" => LR_MG,
"LR-MO" => LR_MO,
"LR-MY" => LR_MY,
"LR-NI" => LR_NI,
"LR-RG" => LR_RG,
"LR-RI" => LR_RI,
"LR-SI" => LR_SI,
"LY-BA" => LY_BA,
"LY-BU" => LY_BU,
"LY-DR" => LY_DR,
"LY-GT" => LY_GT,
"LY-JA" => LY_JA,
"LY-JG" => LY_JG,
"LY-JI" => LY_JI,
"LY-JU" => LY_JU,
"LY-KF" => LY_KF,
"LY-MB" => LY_MB,
"LY-MI" => LY_MI,
"LY-MJ" => LY_MJ,
"LY-MQ" => LY_MQ,
"LY-NL" => LY_NL,
"LY-NQ" => LY_NQ,
"LY-SB" => LY_SB,
"LY-SR" => LY_SR,
"LY-TB" => LY_TB,
"LY-WA" => LY_WA,
"LY-WD" => LY_WD,
"LY-WS" => LY_WS,
"LY-ZA" => LY_ZA,
"LI-01" => LI_01,
"LI-02" => LI_02,
"LI-03" => LI_03,
"LI-04" => LI_04,
"LI-05" => LI_05,
"LI-06" => LI_06,
"LI-07" => LI_07,
"LI-08" => LI_08,
"LI-09" => LI_09,
"LI-10" => LI_10,
"LI-11" => LI_11,
"LT-01" => LT_01,
"LT-02" => LT_02,
"LT-03" => LT_03,
"LT-04" => LT_04,
"LT-05" => LT_05,
"LT-06" => LT_06,
"LT-07" => LT_07,
"LT-08" => LT_08,
"LT-09" => LT_09,
"LT-10" => LT_10,
"LT-11" => LT_11,
"LT-12" => LT_12,
"LT-13" => LT_13,
"LT-14" => LT_14,
"LT-15" => LT_15,
"LT-16" => LT_16,
"LT-17" => LT_17,
"LT-18" => LT_18,
"LT-19" => LT_19,
"LT-20" => LT_20,
"LT-21" => LT_21,
"LT-22" => LT_22,
"LT-23" => LT_23,
"LT-24" => LT_24,
"LT-25" => LT_25,
"LT-26" => LT_26,
"LT-27" => LT_27,
"LT-28" => LT_28,
"LT-29" => LT_29,
"LT-30" => LT_30,
"LT-31" => LT_31,
"LT-32" => LT_32,
"LT-33" => LT_33,
"LT-34" => LT_34,
"LT-35" => LT_35,
"LT-36" => LT_36,
"LT-37" => LT_37,
"LT-38" => LT_38,
"LT-39" => LT_39,
"LT-40" => LT_40,
"LT-41" => LT_41,
"LT-42" => LT_42,
"LT-43" => LT_43,
"LT-44" => LT_44,
"LT-45" => LT_45,
"LT-46" => LT_46,
"LT-47" => LT_47,
"LT-48" => LT_48,
"LT-49" => LT_49,
"LT-50" => LT_50,
"LT-51" => LT_51,
"LT-52" => LT_52,
"LT-53" => LT_53,
"LT-54" => LT_54,
"LT-55" => LT_55,
"LT-56" => LT_56,
"LT-57" => LT_57,
"LT-58" => LT_58,
"LT-59" => LT_59,
"LT-60" => LT_60,
"LT-AL" => LT_AL,
"LT-KL" => LT_KL,
"LT-KU" => LT_KU,
"LT-MR" => LT_MR,
"LT-PN" => LT_PN,
"LT-SA" => LT_SA,
"LT-TA" => LT_TA,
"LT-TE" => LT_TE,
"LT-UT" => LT_UT,
"LT-VL" => LT_VL,
"LU-D" => LU_D,
"LU-G" => LU_G,
"LU-L" => LU_L,
"MO-MO" => MO_MO,
"MK-01" => MK_01,
"MK-02" => MK_02,
"MK-03" => MK_03,
"MK-04" => MK_04,
"MK-05" => MK_05,
"MK-06" => MK_06,
"MK-07" => MK_07,
"MK-08" => MK_08,
"MK-09" => MK_09,
"MK-10" => MK_10,
"MK-11" => MK_11,
"MK-12" => MK_12,
"MK-13" => MK_13,
"MK-14" => MK_14,
"MK-15" => MK_15,
"MK-16" => MK_16,
"MK-17" => MK_17,
"MK-18" => MK_18,
"MK-19" => MK_19,
"MK-20" => MK_20,
"MK-21" => MK_21,
"MK-22" => MK_22,
"MK-23" => MK_23,
"MK-24" => MK_24,
"MK-25" => MK_25,
"MK-26" => MK_26,
"MK-27" => MK_27,
"MK-28" => MK_28,
"MK-29" => MK_29,
"MK-30" => MK_30,
"MK-31" => MK_31,
"MK-32" => MK_32,
"MK-33" => MK_33,
"MK-34" => MK_34,
"MK-35" => MK_35,
"MK-36" => MK_36,
"MK-37" => MK_37,
"MK-38" => MK_38,
"MK-39" => MK_39,
"MK-40" => MK_40,
"MK-41" => MK_41,
"MK-42" => MK_42,
"MK-43" => MK_43,
"MK-44" => MK_44,
"MK-45" => MK_45,
"MK-46" => MK_46,
"MK-47" => MK_47,
"MK-48" => MK_48,
"MK-49" => MK_49,
"MK-50" => MK_50,
"MK-51" => MK_51,
"MK-52" => MK_52,
"MK-53" => MK_53,
"MK-54" => MK_54,
"MK-55" => MK_55,
"MK-56" => MK_56,
"MK-57" => MK_57,
"MK-58" => MK_58,
"MK-59" => MK_59,
"MK-60" => MK_60,
"MK-61" => MK_61,
"MK-62" => MK_62,
"MK-63" => MK_63,
"MK-64" => MK_64,
"MK-65" => MK_65,
"MK-66" => MK_66,
"MK-67" => MK_67,
"MK-68" => MK_68,
"MK-69" => MK_69,
"MK-70" => MK_70,
"MK-71" => MK_71,
"MK-72" => MK_72,
"MK-73" => MK_73,
"MK-74" => MK_74,
"MK-75" => MK_75,
"MK-76" => MK_76,
"MK-77" => MK_77,
"MK-78" => MK_78,
"MK-79" => MK_79,
"MK-80" => MK_80,
"MK-81" => MK_81,
"MK-82" => MK_82,
"MK-83" => MK_83,
"MK-84" => MK_84,
"MG-A" => MG_A,
"MG-D" => MG_D,
"MG-F" => MG_F,
"MG-M" => MG_M,
"MG-T" => MG_T,
"MG-U" => MG_U,
"MW-BA" => MW_BA,
"MW-BL" => MW_BL,
"MW-C" => MW_C,
"MW-CK" => MW_CK,
"MW-CR" => MW_CR,
"MW-CT" => MW_CT,
"MW-DE" => MW_DE,
"MW-DO" => MW_DO,
"MW-KR" => MW_KR,
"MW-KS" => MW_KS,
"MW-LI" => MW_LI,
"MW-LK" => MW_LK,
"MW-MC" => MW_MC,
"MW-MG" => MW_MG,
"MW-MH" => MW_MH,
"MW-MU" => MW_MU,
"MW-MW" => MW_MW,
"MW-MZ" => MW_MZ,
"MW-NB" => MW_NB,
"MW-N" => MW_N,
"MW-NE" => MW_NE,
"MW-NI" => MW_NI,
"MW-NK" => MW_NK,
"MW-NS" => MW_NS,
"MW-NU" => MW_NU,
"MW-PH" => MW_PH,
"MW-RU" => MW_RU,
"MW-SA" => MW_SA,
"MW-S" => MW_S,
"MW-TH" => MW_TH,
"MW-ZO" => MW_ZO,
"MY-01" => MY_01,
"MY-02" => MY_02,
"MY-03" => MY_03,
"MY-04" => MY_04,
"MY-05" => MY_05,
"MY-06" => MY_06,
"MY-07" => MY_07,
"MY-08" => MY_08,
"MY-09" => MY_09,
"MY-10" => MY_10,
"MY-11" => MY_11,
"MY-12" => MY_12,
"MY-13" => MY_13,
"MY-14" => MY_14,
"MY-15" => MY_15,
"MY-16" => MY_16,
"MV-00" => MV_00,
"MV-01" => MV_01,
"MV-02" => MV_02,
"MV-03" => MV_03,
"MV-04" => MV_04,
"MV-05" => MV_05,
"MV-07" => MV_07,
"MV-08" => MV_08,
"MV-12" => MV_12,
"MV-13" => MV_13,
"MV-14" => MV_14,
"MV-17" => MV_17,
"MV-20" => MV_20,
"MV-23" => MV_23,
"MV-24" => MV_24,
"MV-25" => MV_25,
"MV-26" => MV_26,
"MV-27" => MV_27,
"MV-28" => MV_28,
"MV-29" => MV_29,
"MV-CE" => MV_CE,
"MV-MLE" => MV_MLE,
"MV-NC" => MV_NC,
"MV-NO" => MV_NO,
"MV-SC" => MV_SC,
"MV-SU" => MV_SU,
"MV-UN" => MV_UN,
"MV-US" => MV_US,
"ML-1" => ML_1,
"ML-2" => ML_2,
"ML-3" => ML_3,
"ML-4" => ML_4,
"ML-5" => ML_5,
"ML-6" => ML_6,
"ML-7" => ML_7,
"ML-8" => ML_8,
"ML-BKO" => ML_BKO,
"MT-01" => MT_01,
"MT-02" => MT_02,
"MT-03" => MT_03,
"MT-04" => MT_04,
"MT-05" => MT_05,
"MT-06" => MT_06,
"MT-07" => MT_07,
"MT-08" => MT_08,
"MT-09" => MT_09,
"MT-10" => MT_10,
"MT-11" => MT_11,
"MT-12" => MT_12,
"MT-13" => MT_13,
"MT-14" => MT_14,
"MT-15" => MT_15,
"MT-16" => MT_16,
"MT-17" => MT_17,
"MT-18" => MT_18,
"MT-19" => MT_19,
"MT-20" => MT_20,
"MT-21" => MT_21,
"MT-22" => MT_22,
"MT-23" => MT_23,
"MT-24" => MT_24,
"MT-25" => MT_25,
"MT-26" => MT_26,
"MT-27" => MT_27,
"MT-28" => MT_28,
"MT-29" => MT_29,
"MT-30" => MT_30,
"MT-31" => MT_31,
"MT-32" => MT_32,
"MT-33" => MT_33,
"MT-34" => MT_34,
"MT-35" => MT_35,
"MT-36" => MT_36,
"MT-37" => MT_37,
"MT-38" => MT_38,
"MT-39" => MT_39,
"MT-40" => MT_40,
"MT-41" => MT_41,
"MT-42" => MT_42,
"MT-43" => MT_43,
"MT-44" => MT_44,
"MT-45" => MT_45,
"MT-46" => MT_46,
"MT-47" => MT_47,
"MT-48" => MT_48,
"MT-49" => MT_49,
"MT-50" => MT_50,
"MT-51" => MT_51,
"MT-52" => MT_52,
"MT-53" => MT_53,
"MT-54" => MT_54,
"MT-55" => MT_55,
"MT-56" => MT_56,
"MT-57" => MT_57,
"MT-58" => MT_58,
"MT-59" => MT_59,
"MT-60" => MT_60,
"MT-61" => MT_61,
"MT-62" => MT_62,
"MT-63" => MT_63,
"MT-64" => MT_64,
"MT-65" => MT_65,
"MT-66" => MT_66,
"MT-67" => MT_67,
"MT-68" => MT_68,
"MH-ALK" => MH_ALK,
"MH-ALL" => MH_ALL,
"MH-ARN" => MH_ARN,
"MH-AUR" => MH_AUR,
"MH-EBO" => MH_EBO,
"MH-ENI" => MH_ENI,
"MH-JAB" => MH_JAB,
"MH-JAL" => MH_JAL,
"MH-KIL" => MH_KIL,
"MH-KWA" => MH_KWA,
"MH-LAE" => MH_LAE,
"MH-LIB" => MH_LIB,
"MH-LIK" => MH_LIK,
"MH-L" => MH_L,
"MH-MAJ" => MH_MAJ,
"MH-MAL" => MH_MAL,
"MH-MEJ" => MH_MEJ,
"MH-MIL" => MH_MIL,
"MH-NMK" => MH_NMK,
"MH-NMU" => MH_NMU,
"MH-RON" => MH_RON,
"MH-T" => MH_T,
"MH-UJA" => MH_UJA,
"MH-UTI" => MH_UTI,
"MH-WTH" => MH_WTH,
"MH-WTJ" => MH_WTJ,
"MQ-MQ" => MQ_MQ,
"MR-01" => MR_01,
"MR-02" => MR_02,
"MR-03" => MR_03,
"MR-04" => MR_04,
"MR-05" => MR_05,
"MR-06" => MR_06,
"MR-07" => MR_07,
"MR-08" => MR_08,
"MR-09" => MR_09,
"MR-10" => MR_10,
"MR-11" => MR_11,
"MR-12" => MR_12,
"MR-NKC" => MR_NKC,
"MU-AG" => MU_AG,
"MU-BL" => MU_BL,
"MU-BR" => MU_BR,
"MU-CC" => MU_CC,
"MU-CU" => MU_CU,
"MU-FL" => MU_FL,
"MU-GP" => MU_GP,
"MU-MO" => MU_MO,
"MU-PA" => MU_PA,
"MU-PL" => MU_PL,
"MU-PU" => MU_PU,
"MU-PW" => MU_PW,
"MU-QB" => MU_QB,
"MU-RO" => MU_RO,
"MU-RR" => MU_RR,
"MU-SA" => MU_SA,
"MU-VP" => MU_VP,
"YT-YT" => YT_YT,
"MX-AGU" => MX_AGU,
"MX-BCN" => MX_BCN,
"MX-BCS" => MX_BCS,
"MX-CAM" => MX_CAM,
"MX-CHH" => MX_CHH,
"MX-CHP" => MX_CHP,
"MX-COA" => MX_COA,
"MX-COL" => MX_COL,
"MX-CMX" => MX_CMX,
"MX-DUR" => MX_DUR,
"MX-GRO" => MX_GRO,
"MX-GUA" => MX_GUA,
"MX-HID" => MX_HID,
"MX-JAL" => MX_JAL,
"MX-MEX" => MX_MEX,
"MX-MIC" => MX_MIC,
"MX-MOR" => MX_MOR,
"MX-NAY" => MX_NAY,
"MX-NLE" => MX_NLE,
"MX-OAX" => MX_OAX,
"MX-PUE" => MX_PUE,
"MX-QUE" => MX_QUE,
"MX-ROO" => MX_ROO,
"MX-SIN" => MX_SIN,
"MX-SLP" => MX_SLP,
"MX-SON" => MX_SON,
"MX-TAB" => MX_TAB,
"MX-TAM" => MX_TAM,
"MX-TLA" => MX_TLA,
"MX-VER" => MX_VER,
"MX-YUC" => MX_YUC,
"MX-ZAC" => MX_ZAC,
"FM-KSA" => FM_KSA,
"FM-PNI" => FM_PNI,
"FM-TRK" => FM_TRK,
"FM-YAP" => FM_YAP,
"MD-AN" => MD_AN,
"MD-BA" => MD_BA,
"MD-BD" => MD_BD,
"MD-BR" => MD_BR,
"MD-BS" => MD_BS,
"MD-CA" => MD_CA,
"MD-CL" => MD_CL,
"MD-CM" => MD_CM,
"MD-CR" => MD_CR,
"MD-CS" => MD_CS,
"MD-CT" => MD_CT,
"MD-CU" => MD_CU,
"MD-DO" => MD_DO,
"MD-DR" => MD_DR,
"MD-DU" => MD_DU,
"MD-ED" => MD_ED,
"MD-FA" => MD_FA,
"MD-FL" => MD_FL,
"MD-GA" => MD_GA,
"MD-GL" => MD_GL,
"MD-HI" => MD_HI,
"MD-IA" => MD_IA,
"MD-LE" => MD_LE,
"MD-NI" => MD_NI,
"MD-OC" => MD_OC,
"MD-OR" => MD_OR,
"MD-RE" => MD_RE,
"MD-RI" => MD_RI,
"MD-SD" => MD_SD,
"MD-SI" => MD_SI,
"MD-SN" => MD_SN,
"MD-SO" => MD_SO,
"MD-ST" => MD_ST,
"MD-SV" => MD_SV,
"MD-TA" => MD_TA,
"MD-TE" => MD_TE,
"MD-UN" => MD_UN,
"MC-CL" => MC_CL,
"MC-CO" => MC_CO,
"MC-FO" => MC_FO,
"MC-GA" => MC_GA,
"MC-JE" => MC_JE,
"MC-LA" => MC_LA,
"MC-MA" => MC_MA,
"MC-MC" => MC_MC,
"MC-MG" => MC_MG,
"MC-MO" => MC_MO,
"MC-MU" => MC_MU,
"MC-PH" => MC_PH,
"MC-SD" => MC_SD,
"MC-SO" => MC_SO,
"MC-SP" => MC_SP,
"MC-SR" => MC_SR,
"MC-VR" => MC_VR,
"MN-035" => MN_035,
"MN-037" => MN_037,
"MN-039" => MN_039,
"MN-041" => MN_041,
"MN-043" => MN_043,
"MN-046" => MN_046,
"MN-047" => MN_047,
"MN-049" => MN_049,
"MN-051" => MN_051,
"MN-053" => MN_053,
"MN-055" => MN_055,
"MN-057" => MN_057,
"MN-059" => MN_059,
"MN-061" => MN_061,
"MN-063" => MN_063,
"MN-064" => MN_064,
"MN-065" => MN_065,
"MN-067" => MN_067,
"MN-069" => MN_069,
"MN-071" => MN_071,
"MN-073" => MN_073,
"MN-1" => MN_1,
"ME-01" => ME_01,
"ME-02" => ME_02,
"ME-03" => ME_03,
"ME-04" => ME_04,
"ME-05" => ME_05,
"ME-06" => ME_06,
"ME-07" => ME_07,
"ME-08" => ME_08,
"ME-09" => ME_09,
"ME-10" => ME_10,
"ME-11" => ME_11,
"ME-12" => ME_12,
"ME-13" => ME_13,
"ME-14" => ME_14,
"ME-15" => ME_15,
"ME-16" => ME_16,
"ME-17" => ME_17,
"ME-18" => ME_18,
"ME-19" => ME_19,
"ME-20" => ME_20,
"ME-21" => ME_21,
"ME-22" => ME_22,
"ME-23" => ME_23,
"MS-MS" => MS_MS,
"MA-01" => MA_01,
"MA-02" => MA_02,
"MA-03" => MA_03,
"MA-04" => MA_04,
"MA-05" => MA_05,
"MA-06" => MA_06,
"MA-07" => MA_07,
"MA-08" => MA_08,
"MA-09" => MA_09,
"MA-10" => MA_10,
"MA-11" => MA_11,
"MA-12" => MA_12,
"MA-13" => MA_13,
"MA-14" => MA_14,
"MA-15" => MA_15,
"MA-16" => MA_16,
"MA-AGD" => MA_AGD,
"MA-AOU" => MA_AOU,
"MA-ASZ" => MA_ASZ,
"MA-AZI" => MA_AZI,
"MA-BEM" => MA_BEM,
"MA-BER" => MA_BER,
"MA-BES" => MA_BES,
"MA-BOD" => MA_BOD,
"MA-BOM" => MA_BOM,
"MA-CAS" => MA_CAS,
"MA-CHE" => MA_CHE,
"MA-CHI" => MA_CHI,
"MA-CHT" => MA_CHT,
"MA-ERR" => MA_ERR,
"MA-ESI" => MA_ESI,
"MA-ESM" => MA_ESM,
"MA-FAH" => MA_FAH,
"MA-FES" => MA_FES,
"MA-FIG" => MA_FIG,
"MA-GUE" => MA_GUE,
"MA-HAJ" => MA_HAJ,
"MA-HAO" => MA_HAO,
"MA-HOC" => MA_HOC,
"MA-IFR" => MA_IFR,
"MA-INE" => MA_INE,
"MA-JDI" => MA_JDI,
"MA-JRA" => MA_JRA,
"MA-KEN" => MA_KEN,
"MA-KES" => MA_KES,
"MA-KHE" => MA_KHE,
"MA-KHN" => MA_KHN,
"MA-KHO" => MA_KHO,
"MA-LAA" => MA_LAA,
"MA-LAR" => MA_LAR,
"MA-MED" => MA_MED,
"MA-MEK" => MA_MEK,
"MA-MMD" => MA_MMD,
"MA-MMN" => MA_MMN,
"MA-MOH" => MA_MOH,
"MA-MOU" => MA_MOU,
"MA-NAD" => MA_NAD,
"MA-NOU" => MA_NOU,
"MA-OUA" => MA_OUA,
"MA-OUD" => MA_OUD,
"MA-OUJ" => MA_OUJ,
"MA-RAB" => MA_RAB,
"MA-SAF" => MA_SAF,
"MA-SAL" => MA_SAL,
"MA-SEF" => MA_SEF,
"MA-SET" => MA_SET,
"MA-SIK" => MA_SIK,
"MA-SKH" => MA_SKH,
"MA-SYB" => MA_SYB,
"MA-TAI" => MA_TAI,
"MA-TAO" => MA_TAO,
"MA-TAR" => MA_TAR,
"MA-TAT" => MA_TAT,
"MA-TAZ" => MA_TAZ,
"MA-TET" => MA_TET,
"MA-TIZ" => MA_TIZ,
"MA-TNG" => MA_TNG,
"MA-TNT" => MA_TNT,
"MA-ZAG" => MA_ZAG,
"MZ-A" => MZ_A,
"MZ-B" => MZ_B,
"MZ-G" => MZ_G,
"MZ-I" => MZ_I,
"MZ-L" => MZ_L,
"MZ-MPM" => MZ_MPM,
"MZ-N" => MZ_N,
"MZ-P" => MZ_P,
"MZ-Q" => MZ_Q,
"MZ-S" => MZ_S,
"MZ-T" => MZ_T,
"MM-01" => MM_01,
"MM-02" => MM_02,
"MM-03" => MM_03,
"MM-04" => MM_04,
"MM-05" => MM_05,
"MM-06" => MM_06,
"MM-07" => MM_07,
"MM-11" => MM_11,
"MM-12" => MM_12,
"MM-13" => MM_13,
"MM-14" => MM_14,
"MM-15" => MM_15,
"MM-16" => MM_16,
"MM-17" => MM_17,
"MM-18" => MM_18,
"NA-CA" => NA_CA,
"NA-ER" => NA_ER,
"NA-HA" => NA_HA,
"NA-KA" => NA_KA,
"NA-KE" => NA_KE,
"NA-KH" => NA_KH,
"NA-KU" => NA_KU,
"NA-KW" => NA_KW,
"NA-OD" => NA_OD,
"NA-OH" => NA_OH,
"NA-ON" => NA_ON,
"NA-OS" => NA_OS,
"NA-OT" => NA_OT,
"NA-OW" => NA_OW,
"NR-01" => NR_01,
"NR-02" => NR_02,
"NR-03" => NR_03,
"NR-04" => NR_04,
"NR-05" => NR_05,
"NR-06" => NR_06,
"NR-07" => NR_07,
"NR-08" => NR_08,
"NR-09" => NR_09,
"NR-10" => NR_10,
"NR-11" => NR_11,
"NR-12" => NR_12,
"NR-13" => NR_13,
"NR-14" => NR_14,
"NP-1" => NP_1,
"NP-2" => NP_2,
"NP-3" => NP_3,
"NP-4" => NP_4,
"NP-5" => NP_5,
"NP-BA" => NP_BA,
"NP-BH" => NP_BH,
"NP-DH" => NP_DH,
"NP-GA" => NP_GA,
"NP-JA" => NP_JA,
"NP-KA" => NP_KA,
"NP-KO" => NP_KO,
"NP-LU" => NP_LU,
"NP-MA" => NP_MA,
"NP-ME" => NP_ME,
"NP-NA" => NP_NA,
"NP-RA" => NP_RA,
"NP-SA" => NP_SA,
"NP-SE" => NP_SE,
"NL-AW" => NL_AW,
"NL-BQ1" => NL_BQ1,
"NL-BQ2" => NL_BQ2,
"NL-BQ3" => NL_BQ3,
"NL-CW" => NL_CW,
"NL-DR" => NL_DR,
"NL-FL" => NL_FL,
"NL-FR" => NL_FR,
"NL-GE" => NL_GE,
"NL-GR" => NL_GR,
"NL-LI" => NL_LI,
"NL-NB" => NL_NB,
"NL-NH" => NL_NH,
"NL-OV" => NL_OV,
"NL-SX" => NL_SX,
"NL-UT" => NL_UT,
"NL-ZE" => NL_ZE,
"NL-ZH" => NL_ZH,
"NC-NC" => NC_NC,
"NZ-AUK" => NZ_AUK,
"NZ-BOP" => NZ_BOP,
"NZ-CAN" => NZ_CAN,
"NZ-CIT" => NZ_CIT,
"NZ-GIS" => NZ_GIS,
"NZ-HKB" => NZ_HKB,
"NZ-MBH" => NZ_MBH,
"NZ-MWT" => NZ_MWT,
"NZ-N" => NZ_N,
"NZ-NSN" => NZ_NSN,
"NZ-NTL" => NZ_NTL,
"NZ-OTA" => NZ_OTA,
"NZ-S" => NZ_S,
"NZ-STL" => NZ_STL,
"NZ-TAS" => NZ_TAS,
"NZ-TKI" => NZ_TKI,
"NZ-WGN" => NZ_WGN,
"NZ-WKO" => NZ_WKO,
"NZ-WTC" => NZ_WTC,
"NI-AN" => NI_AN,
"NI-AS" => NI_AS,
"NI-BO" => NI_BO,
"NI-CA" => NI_CA,
"NI-CI" => NI_CI,
"NI-CO" => NI_CO,
"NI-ES" => NI_ES,
"NI-GR" => NI_GR,
"NI-JI" => NI_JI,
"NI-LE" => NI_LE,
"NI-MD" => NI_MD,
"NI-MN" => NI_MN,
"NI-MS" => NI_MS,
"NI-MT" => NI_MT,
"NI-NS" => NI_NS,
"NI-RI" => NI_RI,
"NI-SJ" => NI_SJ,
"NG-AB" => NG_AB,
"NG-AD" => NG_AD,
"NG-AK" => NG_AK,
"NG-AN" => NG_AN,
"NG-BA" => NG_BA,
"NG-BE" => NG_BE,
"NG-BO" => NG_BO,
"NG-BY" => NG_BY,
"NG-CR" => NG_CR,
"NG-DE" => NG_DE,
"NG-EB" => NG_EB,
"NG-ED" => NG_ED,
"NG-EK" => NG_EK,
"NG-EN" => NG_EN,
"NG-FC" => NG_FC,
"NG-GO" => NG_GO,
"NG-IM" => NG_IM,
"NG-JI" => NG_JI,
"NG-KD" => NG_KD,
"NG-KE" => NG_KE,
"NG-KN" => NG_KN,
"NG-KO" => NG_KO,
"NG-KT" => NG_KT,
"NG-KW" => NG_KW,
"NG-LA" => NG_LA,
"NG-NA" => NG_NA,
"NG-NI" => NG_NI,
"NG-OG" => NG_OG,
"NG-ON" => NG_ON,
"NG-OS" => NG_OS,
"NG-OY" => NG_OY,
"NG-PL" => NG_PL,
"NG-RI" => NG_RI,
"NG-SO" => NG_SO,
"NG-TA" => NG_TA,
"NG-YO" => NG_YO,
"NG-ZA" => NG_ZA,
"NE-1" => NE_1,
"NE-2" => NE_2,
"NE-3" => NE_3,
"NE-4" => NE_4,
"NE-5" => NE_5,
"NE-6" => NE_6,
"NE-7" => NE_7,
"NE-8" => NE_8,
"NU-NU" => NU_NU,
"NF-NF" => NF_NF,
"MP-MP" => MP_MP,
"NO-01" => NO_01,
"NO-02" => NO_02,
"NO-03" => NO_03,
"NO-04" => NO_04,
"NO-05" => NO_05,
"NO-06" => NO_06,
"NO-07" => NO_07,
"NO-08" => NO_08,
"NO-09" => NO_09,
"NO-10" => NO_10,
"NO-11" => NO_11,
"NO-12" => NO_12,
"NO-14" => NO_14,
"NO-15" => NO_15,
"NO-16" => NO_16,
"NO-17" => NO_17,
"NO-18" => NO_18,
"NO-19" => NO_19,
"NO-20" => NO_20,
"NO-21" => NO_21,
"NO-22" => NO_22,
"OM-BA" => OM_BA,
"OM-BU" => OM_BU,
"OM-DA" => OM_DA,
"OM-MA" => OM_MA,
"OM-MU" => OM_MU,
"OM-SH" => OM_SH,
"OM-WU" => OM_WU,
"OM-ZA" => OM_ZA,
"OM-ZU" => OM_ZU,
"PK-BA" => PK_BA,
"PK-GB" => PK_GB,
"PK-IS" => PK_IS,
"PK-JK" => PK_JK,
"PK-KP" => PK_KP,
"PK-PB" => PK_PB,
"PK-SD" => PK_SD,
"PK-TA" => PK_TA,
"PW-002" => PW_002,
"PW-004" => PW_004,
"PW-010" => PW_010,
"PW-050" => PW_050,
"PW-100" => PW_100,
"PW-150" => PW_150,
"PW-212" => PW_212,
"PW-214" => PW_214,
"PW-218" => PW_218,
"PW-222" => PW_222,
"PW-224" => PW_224,
"PW-226" => PW_226,
"PW-227" => PW_227,
"PW-228" => PW_228,
"PW-350" => PW_350,
"PW-370" => PW_370,
"PS-BTH" => PS_BTH,
"PS-DEB" => PS_DEB,
"PS-GZA" => PS_GZA,
"PS-HBN" => PS_HBN,
"PS-JEM" => PS_JEM,
"PS-JEN" => PS_JEN,
"PS-JRH" => PS_JRH,
"PS-KYS" => PS_KYS,
"PS-NBS" => PS_NBS,
"PS-NGZ" => PS_NGZ,
"PS-QQA" => PS_QQA,
"PS-RBH" => PS_RBH,
"PS-RFH" => PS_RFH,
"PS-SLT" => PS_SLT,
"PS-TBS" => PS_TBS,
"PS-TKM" => PS_TKM,
"PA-10" => PA_10,
"PA-1" => PA_1,
"PA-2" => PA_2,
"PA-3" => PA_3,
"PA-4" => PA_4,
"PA-5" => PA_5,
"PA-6" => PA_6,
"PA-7" => PA_7,
"PA-8" => PA_8,
"PA-9" => PA_9,
"PA-EM" => PA_EM,
"PA-KY" => PA_KY,
"PA-NB" => PA_NB,
"PG-CPK" => PG_CPK,
"PG-CPM" => PG_CPM,
"PG-EBR" => PG_EBR,
"PG-EHG" => PG_EHG,
"PG-EPW" => PG_EPW,
"PG-ESW" => PG_ESW,
"PG-GPK" => PG_GPK,
"PG-HLA" => PG_HLA,
"PG-JWK" => PG_JWK,
"PG-MBA" => PG_MBA,
"PG-MPL" => PG_MPL,
"PG-MPM" => PG_MPM,
"PG-MRL" => PG_MRL,
"PG-NCD" => PG_NCD,
"PG-NIK" => PG_NIK,
"PG-NPP" => PG_NPP,
"PG-NSB" => PG_NSB,
"PG-SAN" => PG_SAN,
"PG-SHM" => PG_SHM,
"PG-WBK" => PG_WBK,
"PG-WHM" => PG_WHM,
"PG-WPD" => PG_WPD,
"PY-10" => PY_10,
"PY-11" => PY_11,
"PY-12" => PY_12,
"PY-13" => PY_13,
"PY-14" => PY_14,
"PY-15" => PY_15,
"PY-16" => PY_16,
"PY-19" => PY_19,
"PY-1" => PY_1,
"PY-2" => PY_2,
"PY-3" => PY_3,
"PY-4" => PY_4,
"PY-5" => PY_5,
"PY-6" => PY_6,
"PY-7" => PY_7,
"PY-8" => PY_8,
"PY-9" => PY_9,
"PY-ASU" => PY_ASU,
"PE-AMA" => PE_AMA,
"PE-ANC" => PE_ANC,
"PE-APU" => PE_APU,
"PE-ARE" => PE_ARE,
"PE-AYA" => PE_AYA,
"PE-CAJ" => PE_CAJ,
"PE-CAL" => PE_CAL,
"PE-CUS" => PE_CUS,
"PE-HUC" => PE_HUC,
"PE-HUV" => PE_HUV,
"PE-ICA" => PE_ICA,
"PE-JUN" => PE_JUN,
"PE-LAL" => PE_LAL,
"PE-LAM" => PE_LAM,
"PE-LIM" => PE_LIM,
"PE-LMA" => PE_LMA,
"PE-LOR" => PE_LOR,
"PE-MDD" => PE_MDD,
"PE-MOQ" => PE_MOQ,
"PE-PAS" => PE_PAS,
"PE-PIU" => PE_PIU,
"PE-PUN" => PE_PUN,
"PE-SAM" => PE_SAM,
"PE-TAC" => PE_TAC,
"PE-TUM" => PE_TUM,
"PE-UCA" => PE_UCA,
"PH-00" => PH_00,
"PH-01" => PH_01,
"PH-02" => PH_02,
"PH-03" => PH_03,
"PH-05" => PH_05,
"PH-06" => PH_06,
"PH-07" => PH_07,
"PH-08" => PH_08,
"PH-09" => PH_09,
"PH-10" => PH_10,
"PH-11" => PH_11,
"PH-12" => PH_12,
"PH-13" => PH_13,
"PH-14" => PH_14,
"PH-15" => PH_15,
"PH-40" => PH_40,
"PH-41" => PH_41,
"PH-ABR" => PH_ABR,
"PH-AGN" => PH_AGN,
"PH-AGS" => PH_AGS,
"PH-AKL" => PH_AKL,
"PH-ALB" => PH_ALB,
"PH-ANT" => PH_ANT,
"PH-APA" => PH_APA,
"PH-AUR" => PH_AUR,
"PH-BAN" => PH_BAN,
"PH-BAS" => PH_BAS,
"PH-BEN" => PH_BEN,
"PH-BIL" => PH_BIL,
"PH-BOH" => PH_BOH,
"PH-BTG" => PH_BTG,
"PH-BTN" => PH_BTN,
"PH-BUK" => PH_BUK,
"PH-BUL" => PH_BUL,
"PH-CAG" => PH_CAG,
"PH-CAM" => PH_CAM,
"PH-CAN" => PH_CAN,
"PH-CAP" => PH_CAP,
"PH-CAS" => PH_CAS,
"PH-CAT" => PH_CAT,
"PH-CAV" => PH_CAV,
"PH-CEB" => PH_CEB,
"PH-COM" => PH_COM,
"PH-DAO" => PH_DAO,
"PH-DAS" => PH_DAS,
"PH-DAV" => PH_DAV,
"PH-DIN" => PH_DIN,
"PH-EAS" => PH_EAS,
"PH-GUI" => PH_GUI,
"PH-IFU" => PH_IFU,
"PH-ILI" => PH_ILI,
"PH-ILN" => PH_ILN,
"PH-ILS" => PH_ILS,
"PH-ISA" => PH_ISA,
"PH-KAL" => PH_KAL,
"PH-LAG" => PH_LAG,
"PH-LAN" => PH_LAN,
"PH-LAS" => PH_LAS,
"PH-LEY" => PH_LEY,
"PH-LUN" => PH_LUN,
"PH-MAD" => PH_MAD,
"PH-MAG" => PH_MAG,
"PH-MAS" => PH_MAS,
"PH-MDC" => PH_MDC,
"PH-MDR" => PH_MDR,
"PH-MOU" => PH_MOU,
"PH-MSC" => PH_MSC,
"PH-MSR" => PH_MSR,
"PH-NCO" => PH_NCO,
"PH-NEC" => PH_NEC,
"PH-NER" => PH_NER,
"PH-NSA" => PH_NSA,
"PH-NUE" => PH_NUE,
"PH-NUV" => PH_NUV,
"PH-PAM" => PH_PAM,
"PH-PAN" => PH_PAN,
"PH-PLW" => PH_PLW,
"PH-QUE" => PH_QUE,
"PH-QUI" => PH_QUI,
"PH-RIZ" => PH_RIZ,
"PH-ROM" => PH_ROM,
"PH-SAR" => PH_SAR,
"PH-SCO" => PH_SCO,
"PH-SIG" => PH_SIG,
"PH-SLE" => PH_SLE,
"PH-SLU" => PH_SLU,
"PH-SOR" => PH_SOR,
"PH-SUK" => PH_SUK,
"PH-SUN" => PH_SUN,
"PH-SUR" => PH_SUR,
"PH-TAR" => PH_TAR,
"PH-TAW" => PH_TAW,
"PH-WSA" => PH_WSA,
"PH-ZAN" => PH_ZAN,
"PH-ZAS" => PH_ZAS,
"PH-ZMB" => PH_ZMB,
"PH-ZSI" => PH_ZSI,
"PN-PN" => PN_PN,
"PL-DS" => PL_DS,
"PL-KP" => PL_KP,
"PL-LB" => PL_LB,
"PL-LD" => PL_LD,
"PL-LU" => PL_LU,
"PL-MA" => PL_MA,
"PL-MZ" => PL_MZ,
"PL-OP" => PL_OP,
"PL-PD" => PL_PD,
"PL-PK" => PL_PK,
"PL-PM" => PL_PM,
"PL-SK" => PL_SK,
"PL-SL" => PL_SL,
"PL-WN" => PL_WN,
"PL-WP" => PL_WP,
"PL-ZP" => PL_ZP,
"PT-01" => PT_01,
"PT-02" => PT_02,
"PT-03" => PT_03,
"PT-04" => PT_04,
"PT-05" => PT_05,
"PT-06" => PT_06,
"PT-07" => PT_07,
"PT-08" => PT_08,
"PT-09" => PT_09,
"PT-10" => PT_10,
"PT-11" => PT_11,
"PT-12" => PT_12,
"PT-13" => PT_13,
"PT-14" => PT_14,
"PT-15" => PT_15,
"PT-16" => PT_16,
"PT-17" => PT_17,
"PT-18" => PT_18,
"PT-20" => PT_20,
"PT-30" => PT_30,
"PR-PR" => PR_PR,
"QA-DA" => QA_DA,
"QA-KH" => QA_KH,
"QA-MS" => QA_MS,
"QA-RA" => QA_RA,
"QA-US" => QA_US,
"QA-WA" => QA_WA,
"QA-ZA" => QA_ZA,
"RE-RE" => RE_RE,
"RO-AB" => RO_AB,
"RO-AG" => RO_AG,
"RO-AR" => RO_AR,
"RO-B" => RO_B,
"RO-BC" => RO_BC,
"RO-BH" => RO_BH,
"RO-BN" => RO_BN,
"RO-BR" => RO_BR,
"RO-BT" => RO_BT,
"RO-BV" => RO_BV,
"RO-BZ" => RO_BZ,
"RO-CJ" => RO_CJ,
"RO-CL" => RO_CL,
"RO-CS" => RO_CS,
"RO-CT" => RO_CT,
"RO-CV" => RO_CV,
"RO-DB" => RO_DB,
"RO-DJ" => RO_DJ,
"RO-GJ" => RO_GJ,
"RO-GL" => RO_GL,
"RO-GR" => RO_GR,
"RO-HD" => RO_HD,
"RO-HR" => RO_HR,
"RO-IF" => RO_IF,
"RO-IL" => RO_IL,
"RO-IS" => RO_IS,
"RO-MH" => RO_MH,
"RO-MM" => RO_MM,
"RO-MS" => RO_MS,
"RO-NT" => RO_NT,
"RO-OT" => RO_OT,
"RO-PH" => RO_PH,
"RO-SB" => RO_SB,
"RO-SJ" => RO_SJ,
"RO-SM" => RO_SM,
"RO-SV" => RO_SV,
"RO-TL" => RO_TL,
"RO-TM" => RO_TM,
"RO-TR" => RO_TR,
"RO-VL" => RO_VL,
"RO-VN" => RO_VN,
"RO-VS" => RO_VS,
"RU-AD" => RU_AD,
"RU-AL" => RU_AL,
"RU-ALT" => RU_ALT,
"RU-AMU" => RU_AMU,
"RU-ARK" => RU_ARK,
"RU-AST" => RU_AST,
"RU-BA" => RU_BA,
"RU-BEL" => RU_BEL,
"RU-BRY" => RU_BRY,
"RU-BU" => RU_BU,
"RU-CE" => RU_CE,
"RU-CHE" => RU_CHE,
"RU-CHU" => RU_CHU,
"RU-CU" => RU_CU,
"RU-DA" => RU_DA,
"RU-IN" => RU_IN,
"RU-IRK" => RU_IRK,
"RU-IVA" => RU_IVA,
"RU-KAM" => RU_KAM,
"RU-KB" => RU_KB,
"RU-KC" => RU_KC,
"RU-KDA" => RU_KDA,
"RU-KEM" => RU_KEM,
"RU-KGD" => RU_KGD,
"RU-KGN" => RU_KGN,
"RU-KHA" => RU_KHA,
"RU-KHM" => RU_KHM,
"RU-KIR" => RU_KIR,
"RU-KK" => RU_KK,
"RU-KL" => RU_KL,
"RU-KLU" => RU_KLU,
"RU-KO" => RU_KO,
"RU-KOS" => RU_KOS,
"RU-KR" => RU_KR,
"RU-KRS" => RU_KRS,
"RU-KYA" => RU_KYA,
"RU-LEN" => RU_LEN,
"RU-LIP" => RU_LIP,
"RU-MAG" => RU_MAG,
"RU-ME" => RU_ME,
"RU-MO" => RU_MO,
"RU-MOS" => RU_MOS,
"RU-MOW" => RU_MOW,
"RU-MUR" => RU_MUR,
"RU-NEN" => RU_NEN,
"RU-NGR" => RU_NGR,
"RU-NIZ" => RU_NIZ,
"RU-NVS" => RU_NVS,
"RU-OMS" => RU_OMS,
"RU-ORE" => RU_ORE,
"RU-ORL" => RU_ORL,
"RU-PER" => RU_PER,
"RU-PNZ" => RU_PNZ,
"RU-PRI" => RU_PRI,
"RU-PSK" => RU_PSK,
"RU-ROS" => RU_ROS,
"RU-RYA" => RU_RYA,
"RU-SAK" => RU_SAK,
"RU-SAM" => RU_SAM,
"RU-SAR" => RU_SAR,
"RU-SA" => RU_SA,
"RU-SE" => RU_SE,
"RU-SMO" => RU_SMO,
"RU-SPE" => RU_SPE,
"RU-STA" => RU_STA,
"RU-SVE" => RU_SVE,
"RU-TAM" => RU_TAM,
"RU-TA" => RU_TA,
"RU-TOM" => RU_TOM,
"RU-TUL" => RU_TUL,
"RU-TVE" => RU_TVE,
"RU-TY" => RU_TY,
"RU-TYU" => RU_TYU,
"RU-UD" => RU_UD,
"RU-ULY" => RU_ULY,
"RU-VGG" => RU_VGG,
"RU-VLA" => RU_VLA,
"RU-VLG" => RU_VLG,
"RU-VOR" => RU_VOR,
"RU-YAN" => RU_YAN,
"RU-YAR" => RU_YAR,
"RU-YEV" => RU_YEV,
"RU-ZAB" => RU_ZAB,
"RW-01" => RW_01,
"RW-02" => RW_02,
"RW-03" => RW_03,
"RW-04" => RW_04,
"RW-05" => RW_05,
"SH-AC" => SH_AC,
"SH-HL" => SH_HL,
"SH-TA" => SH_TA,
"KN-01" => KN_01,
"KN-02" => KN_02,
"KN-03" => KN_03,
"KN-04" => KN_04,
"KN-05" => KN_05,
"KN-06" => KN_06,
"KN-07" => KN_07,
"KN-08" => KN_08,
"KN-09" => KN_09,
"KN-10" => KN_10,
"KN-11" => KN_11,
"KN-12" => KN_12,
"KN-13" => KN_13,
"KN-15" => KN_15,
"KN-K" => KN_K,
"KN-N" => KN_N,
"LC-01" => LC_01,
"LC-02" => LC_02,
"LC-03" => LC_03,
"LC-05" => LC_05,
"LC-06" => LC_06,
"LC-07" => LC_07,
"LC-08" => LC_08,
"LC-10" => LC_10,
"LC-11" => LC_11,
"LC-12" => LC_12,
"VC-01" => VC_01,
"VC-02" => VC_02,
"VC-03" => VC_03,
"VC-04" => VC_04,
"VC-05" => VC_05,
"VC-06" => VC_06,
"WS-AA" => WS_AA,
"WS-AL" => WS_AL,
"WS-AT" => WS_AT,
"WS-FA" => WS_FA,
"WS-GE" => WS_GE,
"WS-GI" => WS_GI,
"WS-PA" => WS_PA,
"WS-SA" => WS_SA,
"WS-TU" => WS_TU,
"WS-VF" => WS_VF,
"WS-VS" => WS_VS,
"SM-01" => SM_01,
"SM-02" => SM_02,
"SM-03" => SM_03,
"SM-04" => SM_04,
"SM-05" => SM_05,
"SM-06" => SM_06,
"SM-07" => SM_07,
"SM-08" => SM_08,
"SM-09" => SM_09,
"ST-P" => ST_P,
"ST-S" => ST_S,
"SA-01" => SA_01,
"SA-02" => SA_02,
"SA-03" => SA_03,
"SA-04" => SA_04,
"SA-05" => SA_05,
"SA-06" => SA_06,
"SA-07" => SA_07,
"SA-08" => SA_08,
"SA-09" => SA_09,
"SA-10" => SA_10,
"SA-11" => SA_11,
"SA-12" => SA_12,
"SA-14" => SA_14,
"SN-DB" => SN_DB,
"SN-DK" => SN_DK,
"SN-FK" => SN_FK,
"SN-KA" => SN_KA,
"SN-KD" => SN_KD,
"SN-KE" => SN_KE,
"SN-KL" => SN_KL,
"SN-LG" => SN_LG,
"SN-MT" => SN_MT,
"SN-SE" => SN_SE,
"SN-SL" => SN_SL,
"SN-TC" => SN_TC,
"SN-TH" => SN_TH,
"SN-ZG" => SN_ZG,
"RS-00" => RS_00,
"RS-01" => RS_01,
"RS-02" => RS_02,
"RS-03" => RS_03,
"RS-04" => RS_04,
"RS-05" => RS_05,
"RS-06" => RS_06,
"RS-07" => RS_07,
"RS-08" => RS_08,
"RS-09" => RS_09,
"RS-10" => RS_10,
"RS-11" => RS_11,
"RS-12" => RS_12,
"RS-13" => RS_13,
"RS-14" => RS_14,
"RS-15" => RS_15,
"RS-16" => RS_16,
"RS-17" => RS_17,
"RS-18" => RS_18,
"RS-19" => RS_19,
"RS-20" => RS_20,
"RS-21" => RS_21,
"RS-22" => RS_22,
"RS-23" => RS_23,
"RS-24" => RS_24,
"RS-25" => RS_25,
"RS-26" => RS_26,
"RS-27" => RS_27,
"RS-28" => RS_28,
"RS-29" => RS_29,
"RS-KM" => RS_KM,
"RS-VO" => RS_VO,
"SC-01" => SC_01,
"SC-02" => SC_02,
"SC-03" => SC_03,
"SC-04" => SC_04,
"SC-05" => SC_05,
"SC-06" => SC_06,
"SC-07" => SC_07,
"SC-08" => SC_08,
"SC-09" => SC_09,
"SC-10" => SC_10,
"SC-11" => SC_11,
"SC-12" => SC_12,
"SC-13" => SC_13,
"SC-14" => SC_14,
"SC-15" => SC_15,
"SC-16" => SC_16,
"SC-17" => SC_17,
"SC-18" => SC_18,
"SC-19" => SC_19,
"SC-20" => SC_20,
"SC-21" => SC_21,
"SC-22" => SC_22,
"SC-23" => SC_23,
"SC-24" => SC_24,
"SC-25" => SC_25,
"GS-GS" => GS_GS,
"SL-E" => SL_E,
"SL-N" => SL_N,
"SL-S" => SL_S,
"SL-W" => SL_W,
"SG-01" => SG_01,
"SG-02" => SG_02,
"SG-03" => SG_03,
"SG-04" => SG_04,
"SG-05" => SG_05,
"SK-BC" => SK_BC,
"SK-BL" => SK_BL,
"SK-KI" => SK_KI,
"SK-NI" => SK_NI,
"SK-PV" => SK_PV,
"SK-TA" => SK_TA,
"SK-TC" => SK_TC,
"SK-ZI" => SK_ZI,
"SI-001" => SI_001,
"SI-002" => SI_002,
"SI-003" => SI_003,
"SI-004" => SI_004,
"SI-005" => SI_005,
"SI-006" => SI_006,
"SI-007" => SI_007,
"SI-008" => SI_008,
"SI-009" => SI_009,
"SI-010" => SI_010,
"SI-011" => SI_011,
"SI-012" => SI_012,
"SI-013" => SI_013,
"SI-014" => SI_014,
"SI-015" => SI_015,
"SI-016" => SI_016,
"SI-017" => SI_017,
"SI-018" => SI_018,
"SI-019" => SI_019,
"SI-020" => SI_020,
"SI-021" => SI_021,
"SI-022" => SI_022,
"SI-023" => SI_023,
"SI-024" => SI_024,
"SI-025" => SI_025,
"SI-026" => SI_026,
"SI-027" => SI_027,
"SI-028" => SI_028,
"SI-029" => SI_029,
"SI-030" => SI_030,
"SI-031" => SI_031,
"SI-032" => SI_032,
"SI-033" => SI_033,
"SI-034" => SI_034,
"SI-035" => SI_035,
"SI-036" => SI_036,
"SI-037" => SI_037,
"SI-038" => SI_038,
"SI-039" => SI_039,
"SI-040" => SI_040,
"SI-041" => SI_041,
"SI-042" => SI_042,
"SI-043" => SI_043,
"SI-044" => SI_044,
"SI-045" => SI_045,
"SI-046" => SI_046,
"SI-047" => SI_047,
"SI-048" => SI_048,
"SI-049" => SI_049,
"SI-050" => SI_050,
"SI-051" => SI_051,
"SI-052" => SI_052,
"SI-053" => SI_053,
"SI-054" => SI_054,
"SI-055" => SI_055,
"SI-056" => SI_056,
"SI-057" => SI_057,
"SI-058" => SI_058,
"SI-059" => SI_059,
"SI-060" => SI_060,
"SI-061" => SI_061,
"SI-062" => SI_062,
"SI-063" => SI_063,
"SI-064" => SI_064,
"SI-065" => SI_065,
"SI-066" => SI_066,
"SI-067" => SI_067,
"SI-068" => SI_068,
"SI-069" => SI_069,
"SI-070" => SI_070,
"SI-071" => SI_071,
"SI-072" => SI_072,
"SI-073" => SI_073,
"SI-074" => SI_074,
"SI-075" => SI_075,
"SI-076" => SI_076,
"SI-077" => SI_077,
"SI-078" => SI_078,
"SI-079" => SI_079,
"SI-080" => SI_080,
"SI-081" => SI_081,
"SI-082" => SI_082,
"SI-083" => SI_083,
"SI-084" => SI_084,
"SI-085" => SI_085,
"SI-086" => SI_086,
"SI-087" => SI_087,
"SI-088" => SI_088,
"SI-089" => SI_089,
"SI-090" => SI_090,
"SI-091" => SI_091,
"SI-092" => SI_092,
"SI-093" => SI_093,
"SI-094" => SI_094,
"SI-095" => SI_095,
"SI-096" => SI_096,
"SI-097" => SI_097,
"SI-098" => SI_098,
"SI-099" => SI_099,
"SI-100" => SI_100,
"SI-101" => SI_101,
"SI-102" => SI_102,
"SI-103" => SI_103,
"SI-104" => SI_104,
"SI-105" => SI_105,
"SI-106" => SI_106,
"SI-107" => SI_107,
"SI-108" => SI_108,
"SI-109" => SI_109,
"SI-110" => SI_110,
"SI-111" => SI_111,
"SI-112" => SI_112,
"SI-113" => SI_113,
"SI-114" => SI_114,
"SI-115" => SI_115,
"SI-116" => SI_116,
"SI-117" => SI_117,
"SI-118" => SI_118,
"SI-119" => SI_119,
"SI-120" => SI_120,
"SI-121" => SI_121,
"SI-122" => SI_122,
"SI-123" => SI_123,
"SI-124" => SI_124,
"SI-125" => SI_125,
"SI-126" => SI_126,
"SI-127" => SI_127,
"SI-128" => SI_128,
"SI-129" => SI_129,
"SI-130" => SI_130,
"SI-131" => SI_131,
"SI-132" => SI_132,
"SI-133" => SI_133,
"SI-134" => SI_134,
"SI-135" => SI_135,
"SI-136" => SI_136,
"SI-137" => SI_137,
"SI-138" => SI_138,
"SI-139" => SI_139,
"SI-140" => SI_140,
"SI-141" => SI_141,
"SI-142" => SI_142,
"SI-143" => SI_143,
"SI-144" => SI_144,
"SI-146" => SI_146,
"SI-147" => SI_147,
"SI-148" => SI_148,
"SI-149" => SI_149,
"SI-150" => SI_150,
"SI-151" => SI_151,
"SI-152" => SI_152,
"SI-153" => SI_153,
"SI-154" => SI_154,
"SI-155" => SI_155,
"SI-156" => SI_156,
"SI-157" => SI_157,
"SI-158" => SI_158,
"SI-159" => SI_159,
"SI-160" => SI_160,
"SI-161" => SI_161,
"SI-162" => SI_162,
"SI-163" => SI_163,
"SI-164" => SI_164,
"SI-165" => SI_165,
"SI-166" => SI_166,
"SI-167" => SI_167,
"SI-168" => SI_168,
"SI-169" => SI_169,
"SI-170" => SI_170,
"SI-171" => SI_171,
"SI-172" => SI_172,
"SI-173" => SI_173,
"SI-174" => SI_174,
"SI-175" => SI_175,
"SI-176" => SI_176,
"SI-177" => SI_177,
"SI-178" => SI_178,
"SI-179" => SI_179,
"SI-180" => SI_180,
"SI-181" => SI_181,
"SI-182" => SI_182,
"SI-183" => SI_183,
"SI-184" => SI_184,
"SI-185" => SI_185,
"SI-186" => SI_186,
"SI-187" => SI_187,
"SI-188" => SI_188,
"SI-189" => SI_189,
"SI-190" => SI_190,
"SI-191" => SI_191,
"SI-192" => SI_192,
"SI-193" => SI_193,
"SI-194" => SI_194,
"SI-195" => SI_195,
"SI-196" => SI_196,
"SI-197" => SI_197,
"SI-198" => SI_198,
"SI-199" => SI_199,
"SI-200" => SI_200,
"SI-201" => SI_201,
"SI-202" => SI_202,
"SI-203" => SI_203,
"SI-204" => SI_204,
"SI-205" => SI_205,
"SI-206" => SI_206,
"SI-207" => SI_207,
"SI-208" => SI_208,
"SI-209" => SI_209,
"SI-210" => SI_210,
"SI-211" => SI_211,
"SI-212" => SI_212,
"SB-CE" => SB_CE,
"SB-CH" => SB_CH,
"SB-CT" => SB_CT,
"SB-GU" => SB_GU,
"SB-IS" => SB_IS,
"SB-MK" => SB_MK,
"SB-ML" => SB_ML,
"SB-RB" => SB_RB,
"SB-TE" => SB_TE,
"SB-WE" => SB_WE,
"SO-AW" => SO_AW,
"SO-BK" => SO_BK,
"SO-BN" => SO_BN,
"SO-BR" => SO_BR,
"SO-BY" => SO_BY,
"SO-GA" => SO_GA,
"SO-GE" => SO_GE,
"SO-HI" => SO_HI,
"SO-JD" => SO_JD,
"SO-JH" => SO_JH,
"SO-MU" => SO_MU,
"SO-NU" => SO_NU,
"SO-SA" => SO_SA,
"SO-SD" => SO_SD,
"SO-SH" => SO_SH,
"SO-SO" => SO_SO,
"SO-TO" => SO_TO,
"SO-WO" => SO_WO,
"ZA-EC" => ZA_EC,
"ZA-FS" => ZA_FS,
"ZA-GT" => ZA_GT,
"ZA-LP" => ZA_LP,
"ZA-MP" => ZA_MP,
"ZA-NC" => ZA_NC,
"ZA-NL" => ZA_NL,
"ZA-NW" => ZA_NW,
"ZA-WC" => ZA_WC,
"SS-BN" => SS_BN,
"SS-BW" => SS_BW,
"SS-EC" => SS_EC,
"SS-EE" => SS_EE,
"SS-EW" => SS_EW,
"SS-JG" => SS_JG,
"SS-LK" => SS_LK,
"SS-NU" => SS_NU,
"SS-UY" => SS_UY,
"SS-WR" => SS_WR,
"ES-A" => ES_A,
"ES-AB" => ES_AB,
"ES-AL" => ES_AL,
"ES-AN" => ES_AN,
"ES-AR" => ES_AR,
"ES-AS" => ES_AS,
"ES-AV" => ES_AV,
"ES-BA" => ES_BA,
"ES-B" => ES_B,
"ES-BI" => ES_BI,
"ES-BU" => ES_BU,
"ES-CA" => ES_CA,
"ES-C" => ES_C,
"ES-CB" => ES_CB,
"ES-CC" => ES_CC,
"ES-CE" => ES_CE,
"ES-CL" => ES_CL,
"ES-CM" => ES_CM,
"ES-CN" => ES_CN,
"ES-CO" => ES_CO,
"ES-CR" => ES_CR,
"ES-CS" => ES_CS,
"ES-CT" => ES_CT,
"ES-CU" => ES_CU,
"ES-EX" => ES_EX,
"ES-GA" => ES_GA,
"ES-GC" => ES_GC,
"ES-GI" => ES_GI,
"ES-GR" => ES_GR,
"ES-GU" => ES_GU,
"ES-H" => ES_H,
"ES-HU" => ES_HU,
"ES-IB" => ES_IB,
"ES-J" => ES_J,
"ES-LE" => ES_LE,
"ES-L" => ES_L,
"ES-LO" => ES_LO,
"ES-LU" => ES_LU,
"ES-MA" => ES_MA,
"ES-MC" => ES_MC,
"ES-MD" => ES_MD,
"ES-ML" => ES_ML,
"ES-M" => ES_M,
"ES-MU" => ES_MU,
"ES-NA" => ES_NA,
"ES-NC" => ES_NC,
"ES-O" => ES_O,
"ES-OR" => ES_OR,
"ES-PM" => ES_PM,
"ES-PO" => ES_PO,
"ES-P" => ES_P,
"ES-PV" => ES_PV,
"ES-RI" => ES_RI,
"ES-SA" => ES_SA,
"ES-S" => ES_S,
"ES-SE" => ES_SE,
"ES-SG" => ES_SG,
"ES-SO" => ES_SO,
"ES-SS" => ES_SS,
"ES-TE" => ES_TE,
"ES-TF" => ES_TF,
"ES-TO" => ES_TO,
"ES-T" => ES_T,
"ES-VA" => ES_VA,
"ES-VC" => ES_VC,
"ES-VI" => ES_VI,
"ES-V" => ES_V,
"ES-ZA" => ES_ZA,
"ES-Z" => ES_Z,
"LK-11" => LK_11,
"LK-12" => LK_12,
"LK-13" => LK_13,
"LK-1" => LK_1,
"LK-21" => LK_21,
"LK-22" => LK_22,
"LK-23" => LK_23,
"LK-2" => LK_2,
"LK-31" => LK_31,
"LK-32" => LK_32,
"LK-33" => LK_33,
"LK-3" => LK_3,
"LK-41" => LK_41,
"LK-42" => LK_42,
"LK-43" => LK_43,
"LK-44" => LK_44,
"LK-45" => LK_45,
"LK-4" => LK_4,
"LK-51" => LK_51,
"LK-52" => LK_52,
"LK-53" => LK_53,
"LK-5" => LK_5,
"LK-61" => LK_61,
"LK-62" => LK_62,
"LK-6" => LK_6,
"LK-71" => LK_71,
"LK-72" => LK_72,
"LK-7" => LK_7,
"LK-81" => LK_81,
"LK-82" => LK_82,
"LK-8" => LK_8,
"LK-91" => LK_91,
"LK-92" => LK_92,
"LK-9" => LK_9,
"BL-BL" => BL_BL,
"SX-SX" => SX_SX,
"MF-MF" => MF_MF,
"PM-PM" => PM_PM,
"SD-DC" => SD_DC,
"SD-DE" => SD_DE,
"SD-DN" => SD_DN,
"SD-DS" => SD_DS,
"SD-DW" => SD_DW,
"SD-GD" => SD_GD,
"SD-GZ" => SD_GZ,
"SD-KA" => SD_KA,
"SD-KH" => SD_KH,
"SD-KN" => SD_KN,
"SD-KS" => SD_KS,
"SD-NB" => SD_NB,
"SD-NO" => SD_NO,
"SD-NR" => SD_NR,
"SD-NW" => SD_NW,
"SD-RS" => SD_RS,
"SD-SI" => SD_SI,
"SR-BR" => SR_BR,
"SR-CM" => SR_CM,
"SR-CR" => SR_CR,
"SR-MA" => SR_MA,
"SR-NI" => SR_NI,
"SR-PM" => SR_PM,
"SR-PR" => SR_PR,
"SR-SA" => SR_SA,
"SR-SI" => SR_SI,
"SR-WA" => SR_WA,
"SJ-SJ" => SJ_SJ,
"SZ-HH" => SZ_HH,
"SZ-LU" => SZ_LU,
"SZ-MA" => SZ_MA,
"SZ-SH" => SZ_SH,
"SE-AB" => SE_AB,
"SE-AC" => SE_AC,
"SE-BD" => SE_BD,
"SE-C" => SE_C,
"SE-D" => SE_D,
"SE-E" => SE_E,
"SE-F" => SE_F,
"SE-G" => SE_G,
"SE-H" => SE_H,
"SE-I" => SE_I,
"SE-K" => SE_K,
"SE-M" => SE_M,
"SE-N" => SE_N,
"SE-O" => SE_O,
"SE-S" => SE_S,
"SE-T" => SE_T,
"SE-U" => SE_U,
"SE-W" => SE_W,
"SE-X" => SE_X,
"SE-Y" => SE_Y,
"SE-Z" => SE_Z,
"CH-AG" => CH_AG,
"CH-AI" => CH_AI,
"CH-AR" => CH_AR,
"CH-BE" => CH_BE,
"CH-BL" => CH_BL,
"CH-BS" => CH_BS,
"CH-FR" => CH_FR,
"CH-GE" => CH_GE,
"CH-GL" => CH_GL,
"CH-GR" => CH_GR,
"CH-JU" => CH_JU,
"CH-LU" => CH_LU,
"CH-NE" => CH_NE,
"CH-NW" => CH_NW,
"CH-OW" => CH_OW,
"CH-SG" => CH_SG,
"CH-SH" => CH_SH,
"CH-SO" => CH_SO,
"CH-SZ" => CH_SZ,
"CH-TG" => CH_TG,
"CH-TI" => CH_TI,
"CH-UR" => CH_UR,
"CH-VD" => CH_VD,
"CH-VS" => CH_VS,
"CH-ZG" => CH_ZG,
"CH-ZH" => CH_ZH,
"SY-DI" => SY_DI,
"SY-DR" => SY_DR,
"SY-DY" => SY_DY,
"SY-HA" => SY_HA,
"SY-HI" => SY_HI,
"SY-HL" => SY_HL,
"SY-HM" => SY_HM,
"SY-ID" => SY_ID,
"SY-LA" => SY_LA,
"SY-QU" => SY_QU,
"SY-RA" => SY_RA,
"SY-RD" => SY_RD,
"SY-SU" => SY_SU,
"SY-TA" => SY_TA,
"TW-CHA" => TW_CHA,
"TW-CYI" => TW_CYI,
"TW-CYQ" => TW_CYQ,
"TW-HSQ" => TW_HSQ,
"TW-HSZ" => TW_HSZ,
"TW-HUA" => TW_HUA,
"TW-ILA" => TW_ILA,
"TW-KEE" => TW_KEE,
"TW-KHH" => TW_KHH,
"TW-KHQ" => TW_KHQ,
"TW-MIA" => TW_MIA,
"TW-NAN" => TW_NAN,
"TW-PEN" => TW_PEN,
"TW-PIF" => TW_PIF,
"TW-TAO" => TW_TAO,
"TW-TNN" => TW_TNN,
"TW-TNQ" => TW_TNQ,
"TW-TPE" => TW_TPE,
"TW-TPQ" => TW_TPQ,
"TW-TTT" => TW_TTT,
"TW-TXG" => TW_TXG,
"TW-TXQ" => TW_TXQ,
"TW-YUN" => TW_YUN,
"TJ-DU" => TJ_DU,
"TJ-GB" => TJ_GB,
"TJ-KT" => TJ_KT,
"TJ-SU" => TJ_SU,
"TZ-01" => TZ_01,
"TZ-02" => TZ_02,
"TZ-03" => TZ_03,
"TZ-04" => TZ_04,
"TZ-05" => TZ_05,
"TZ-06" => TZ_06,
"TZ-07" => TZ_07,
"TZ-08" => TZ_08,
"TZ-09" => TZ_09,
"TZ-10" => TZ_10,
"TZ-11" => TZ_11,
"TZ-12" => TZ_12,
"TZ-13" => TZ_13,
"TZ-14" => TZ_14,
"TZ-15" => TZ_15,
"TZ-16" => TZ_16,
"TZ-17" => TZ_17,
"TZ-18" => TZ_18,
"TZ-19" => TZ_19,
"TZ-20" => TZ_20,
"TZ-21" => TZ_21,
"TZ-22" => TZ_22,
"TZ-23" => TZ_23,
"TZ-24" => TZ_24,
"TZ-25" => TZ_25,
"TZ-26" => TZ_26,
"TZ-27" => TZ_27,
"TZ-28" => TZ_28,
"TZ-29" => TZ_29,
"TZ-30" => TZ_30,
"TH-10" => TH_10,
"TH-11" => TH_11,
"TH-12" => TH_12,
"TH-13" => TH_13,
"TH-14" => TH_14,
"TH-15" => TH_15,
"TH-16" => TH_16,
"TH-17" => TH_17,
"TH-18" => TH_18,
"TH-19" => TH_19,
"TH-20" => TH_20,
"TH-21" => TH_21,
"TH-22" => TH_22,
"TH-23" => TH_23,
"TH-24" => TH_24,
"TH-25" => TH_25,
"TH-26" => TH_26,
"TH-27" => TH_27,
"TH-30" => TH_30,
"TH-31" => TH_31,
"TH-32" => TH_32,
"TH-33" => TH_33,
"TH-34" => TH_34,
"TH-35" => TH_35,
"TH-36" => TH_36,
"TH-37" => TH_37,
"TH-38" => TH_38,
"TH-39" => TH_39,
"TH-40" => TH_40,
"TH-41" => TH_41,
"TH-42" => TH_42,
"TH-43" => TH_43,
"TH-44" => TH_44,
"TH-45" => TH_45,
"TH-46" => TH_46,
"TH-47" => TH_47,
"TH-48" => TH_48,
"TH-49" => TH_49,
"TH-50" => TH_50,
"TH-51" => TH_51,
"TH-52" => TH_52,
"TH-53" => TH_53,
"TH-54" => TH_54,
"TH-55" => TH_55,
"TH-56" => TH_56,
"TH-57" => TH_57,
"TH-58" => TH_58,
"TH-60" => TH_60,
"TH-61" => TH_61,
"TH-62" => TH_62,
"TH-63" => TH_63,
"TH-64" => TH_64,
"TH-65" => TH_65,
"TH-66" => TH_66,
"TH-67" => TH_67,
"TH-70" => TH_70,
"TH-71" => TH_71,
"TH-72" => TH_72,
"TH-73" => TH_73,
"TH-74" => TH_74,
"TH-75" => TH_75,
"TH-76" => TH_76,
"TH-77" => TH_77,
"TH-80" => TH_80,
"TH-81" => TH_81,
"TH-82" => TH_82,
"TH-83" => TH_83,
"TH-84" => TH_84,
"TH-85" => TH_85,
"TH-86" => TH_86,
"TH-90" => TH_90,
"TH-91" => TH_91,
"TH-92" => TH_92,
"TH-93" => TH_93,
"TH-94" => TH_94,
"TH-95" => TH_95,
"TH-96" => TH_96,
"TH-S" => TH_S,
"TL-AL" => TL_AL,
"TL-AN" => TL_AN,
"TL-BA" => TL_BA,
"TL-BO" => TL_BO,
"TL-CO" => TL_CO,
"TL-DI" => TL_DI,
"TL-ER" => TL_ER,
"TL-LA" => TL_LA,
"TL-LI" => TL_LI,
"TL-MF" => TL_MF,
"TL-MT" => TL_MT,
"TL-OE" => TL_OE,
"TL-VI" => TL_VI,
"TG-C" => TG_C,
"TG-K" => TG_K,
"TG-M" => TG_M,
"TG-P" => TG_P,
"TG-S" => TG_S,
"TK-TK" => TK_TK,
"TO-01" => TO_01,
"TO-02" => TO_02,
"TO-03" => TO_03,
"TO-04" => TO_04,
"TO-05" => TO_05,
"TT-ARI" => TT_ARI,
"TT-CHA" => TT_CHA,
"TT-CTT" => TT_CTT,
"TT-DMN" => TT_DMN,
"TT-ETO" => TT_ETO,
"TT-PED" => TT_PED,
"TT-POS" => TT_POS,
"TT-PRT" => TT_PRT,
"TT-PTF" => TT_PTF,
"TT-RCM" => TT_RCM,
"TT-SFO" => TT_SFO,
"TT-SGE" => TT_SGE,
"TT-SIP" => TT_SIP,
"TT-SJL" => TT_SJL,
"TT-TUP" => TT_TUP,
"TT-WTO" => TT_WTO,
"TN-11" => TN_11,
"TN-12" => TN_12,
"TN-13" => TN_13,
"TN-14" => TN_14,
"TN-21" => TN_21,
"TN-22" => TN_22,
"TN-23" => TN_23,
"TN-31" => TN_31,
"TN-32" => TN_32,
"TN-33" => TN_33,
"TN-34" => TN_34,
"TN-41" => TN_41,
"TN-42" => TN_42,
"TN-43" => TN_43,
"TN-51" => TN_51,
"TN-52" => TN_52,
"TN-53" => TN_53,
"TN-61" => TN_61,
"TN-71" => TN_71,
"TN-72" => TN_72,
"TN-73" => TN_73,
"TN-81" => TN_81,
"TN-82" => TN_82,
"TN-83" => TN_83,
"TR-01" => TR_01,
"TR-02" => TR_02,
"TR-03" => TR_03,
"TR-04" => TR_04,
"TR-05" => TR_05,
"TR-06" => TR_06,
"TR-07" => TR_07,
"TR-08" => TR_08,
"TR-09" => TR_09,
"TR-10" => TR_10,
"TR-11" => TR_11,
"TR-12" => TR_12,
"TR-13" => TR_13,
"TR-14" => TR_14,
"TR-15" => TR_15,
"TR-16" => TR_16,
"TR-17" => TR_17,
"TR-18" => TR_18,
"TR-19" => TR_19,
"TR-20" => TR_20,
"TR-21" => TR_21,
"TR-22" => TR_22,
"TR-23" => TR_23,
"TR-24" => TR_24,
"TR-25" => TR_25,
"TR-26" => TR_26,
"TR-27" => TR_27,
"TR-28" => TR_28,
"TR-29" => TR_29,
"TR-30" => TR_30,
"TR-31" => TR_31,
"TR-32" => TR_32,
"TR-33" => TR_33,
"TR-34" => TR_34,
"TR-35" => TR_35,
"TR-36" => TR_36,
"TR-37" => TR_37,
"TR-38" => TR_38,
"TR-39" => TR_39,
"TR-40" => TR_40,
"TR-41" => TR_41,
"TR-42" => TR_42,
"TR-43" => TR_43,
"TR-44" => TR_44,
"TR-45" => TR_45,
"TR-46" => TR_46,
"TR-47" => TR_47,
"TR-48" => TR_48,
"TR-49" => TR_49,
"TR-50" => TR_50,
"TR-51" => TR_51,
"TR-52" => TR_52,
"TR-53" => TR_53,
"TR-54" => TR_54,
"TR-55" => TR_55,
"TR-56" => TR_56,
"TR-57" => TR_57,
"TR-58" => TR_58,
"TR-59" => TR_59,
"TR-60" => TR_60,
"TR-61" => TR_61,
"TR-62" => TR_62,
"TR-63" => TR_63,
"TR-64" => TR_64,
"TR-65" => TR_65,
"TR-66" => TR_66,
"TR-67" => TR_67,
"TR-68" => TR_68,
"TR-69" => TR_69,
"TR-70" => TR_70,
"TR-71" => TR_71,
"TR-72" => TR_72,
"TR-73" => TR_73,
"TR-74" => TR_74,
"TR-75" => TR_75,
"TR-76" => TR_76,
"TR-77" => TR_77,
"TR-78" => TR_78,
"TR-79" => TR_79,
"TR-80" => TR_80,
"TR-81" => TR_81,
"TM-A" => TM_A,
"TM-B" => TM_B,
"TM-D" => TM_D,
"TM-L" => TM_L,
"TM-M" => TM_M,
"TM-S" => TM_S,
"TC-TC" => TC_TC,
"TV-FUN" => TV_FUN,
"TV-NIT" => TV_NIT,
"TV-NKF" => TV_NKF,
"TV-NKL" => TV_NKL,
"TV-NMA" => TV_NMA,
"TV-NMG" => TV_NMG,
"TV-NUI" => TV_NUI,
"TV-VAI" => TV_VAI,
"UG-101" => UG_101,
"UG-102" => UG_102,
"UG-103" => UG_103,
"UG-104" => UG_104,
"UG-105" => UG_105,
"UG-106" => UG_106,
"UG-107" => UG_107,
"UG-108" => UG_108,
"UG-109" => UG_109,
"UG-110" => UG_110,
"UG-111" => UG_111,
"UG-112" => UG_112,
"UG-113" => UG_113,
"UG-114" => UG_114,
"UG-115" => UG_115,
"UG-116" => UG_116,
"UG-117" => UG_117,
"UG-118" => UG_118,
"UG-119" => UG_119,
"UG-120" => UG_120,
"UG-121" => UG_121,
"UG-122" => UG_122,
"UG-123" => UG_123,
"UG-124" => UG_124,
"UG-201" => UG_201,
"UG-202" => UG_202,
"UG-203" => UG_203,
"UG-204" => UG_204,
"UG-205" => UG_205,
"UG-206" => UG_206,
"UG-207" => UG_207,
"UG-208" => UG_208,
"UG-209" => UG_209,
"UG-210" => UG_210,
"UG-211" => UG_211,
"UG-212" => UG_212,
"UG-213" => UG_213,
"UG-214" => UG_214,
"UG-215" => UG_215,
"UG-216" => UG_216,
"UG-217" => UG_217,
"UG-218" => UG_218,
"UG-219" => UG_219,
"UG-220" => UG_220,
"UG-221" => UG_221,
"UG-222" => UG_222,
"UG-223" => UG_223,
"UG-224" => UG_224,
"UG-225" => UG_225,
"UG-226" => UG_226,
"UG-227" => UG_227,
"UG-228" => UG_228,
"UG-229" => UG_229,
"UG-230" => UG_230,
"UG-231" => UG_231,
"UG-232" => UG_232,
"UG-301" => UG_301,
"UG-302" => UG_302,
"UG-303" => UG_303,
"UG-304" => UG_304,
"UG-305" => UG_305,
"UG-306" => UG_306,
"UG-307" => UG_307,
"UG-308" => UG_308,
"UG-309" => UG_309,
"UG-310" => UG_310,
"UG-311" => UG_311,
"UG-312" => UG_312,
"UG-313" => UG_313,
"UG-314" => UG_314,
"UG-315" => UG_315,
"UG-316" => UG_316,
"UG-317" => UG_317,
"UG-318" => UG_318,
"UG-319" => UG_319,
"UG-320" => UG_320,
"UG-321" => UG_321,
"UG-322" => UG_322,
"UG-323" => UG_323,
"UG-324" => UG_324,
"UG-325" => UG_325,
"UG-326" => UG_326,
"UG-327" => UG_327,
"UG-328" => UG_328,
"UG-329" => UG_329,
"UG-330" => UG_330,
"UG-331" => UG_331,
"UG-401" => UG_401,
"UG-402" => UG_402,
"UG-403" => UG_403,
"UG-404" => UG_404,
"UG-405" => UG_405,
"UG-406" => UG_406,
"UG-407" => UG_407,
"UG-408" => UG_408,
"UG-409" => UG_409,
"UG-410" => UG_410,
"UG-411" => UG_411,
"UG-412" => UG_412,
"UG-413" => UG_413,
"UG-414" => UG_414,
"UG-415" => UG_415,
"UG-416" => UG_416,
"UG-417" => UG_417,
"UG-418" => UG_418,
"UG-419" => UG_419,
"UG-420" => UG_420,
"UG-421" => UG_421,
"UG-422" => UG_422,
"UG-423" => UG_423,
"UG-424" => UG_424,
"UG-425" => UG_425,
"UG-C" => UG_C,
"UG-E" => UG_E,
"UG-N" => UG_N,
"UG-W" => UG_W,
"UA-05" => UA_05,
"UA-07" => UA_07,
"UA-09" => UA_09,
"UA-12" => UA_12,
"UA-14" => UA_14,
"UA-18" => UA_18,
"UA-21" => UA_21,
"UA-23" => UA_23,
"UA-26" => UA_26,
"UA-30" => UA_30,
"UA-32" => UA_32,
"UA-35" => UA_35,
"UA-40" => UA_40,
"UA-43" => UA_43,
"UA-46" => UA_46,
"UA-48" => UA_48,
"UA-51" => UA_51,
"UA-53" => UA_53,
"UA-56" => UA_56,
"UA-59" => UA_59,
"UA-61" => UA_61,
"UA-63" => UA_63,
"UA-65" => UA_65,
"UA-68" => UA_68,
"UA-71" => UA_71,
"UA-74" => UA_74,
"UA-77" => UA_77,
"AE-AJ" => AE_AJ,
"AE-AZ" => AE_AZ,
"AE-DU" => AE_DU,
"AE-FU" => AE_FU,
"AE-RK" => AE_RK,
"AE-SH" => AE_SH,
"AE-UQ" => AE_UQ,
"GB-ABD" => GB_ABD,
"GB-ABE" => GB_ABE,
"GB-AGB" => GB_AGB,
"GB-AGY" => GB_AGY,
"GB-ANS" => GB_ANS,
"GB-ANT" => GB_ANT,
"GB-ARD" => GB_ARD,
"GB-ARM" => GB_ARM,
"GB-BAS" => GB_BAS,
"GB-BBD" => GB_BBD,
"GB-BDF" => GB_BDF,
"GB-BDG" => GB_BDG,
"GB-BEN" => GB_BEN,
"GB-BEX" => GB_BEX,
"GB-BFS" => GB_BFS,
"GB-BGE" => GB_BGE,
"GB-BGW" => GB_BGW,
"GB-BIR" => GB_BIR,
"GB-BKM" => GB_BKM,
"GB-BLA" => GB_BLA,
"GB-BLY" => GB_BLY,
"GB-BMH" => GB_BMH,
"GB-BNB" => GB_BNB,
"GB-BNE" => GB_BNE,
"GB-BNH" => GB_BNH,
"GB-BNS" => GB_BNS,
"GB-BOL" => GB_BOL,
"GB-BPL" => GB_BPL,
"GB-BRC" => GB_BRC,
"GB-BRD" => GB_BRD,
"GB-BRY" => GB_BRY,
"GB-BST" => GB_BST,
"GB-BUR" => GB_BUR,
"GB-CAM" => GB_CAM,
"GB-CAY" => GB_CAY,
"GB-CBF" => GB_CBF,
"GB-CGN" => GB_CGN,
"GB-CGV" => GB_CGV,
"GB-CHE" => GB_CHE,
"GB-CHW" => GB_CHW,
"GB-CKF" => GB_CKF,
"GB-CKT" => GB_CKT,
"GB-CLD" => GB_CLD,
"GB-CLK" => GB_CLK,
"GB-CLR" => GB_CLR,
"GB-CMA" => GB_CMA,
"GB-CMD" => GB_CMD,
"GB-CMN" => GB_CMN,
"GB-CON" => GB_CON,
"GB-COV" => GB_COV,
"GB-CRF" => GB_CRF,
"GB-CRY" => GB_CRY,
"GB-CSR" => GB_CSR,
"GB-CWY" => GB_CWY,
"GB-DAL" => GB_DAL,
"GB-DBY" => GB_DBY,
"GB-DEN" => GB_DEN,
"GB-DER" => GB_DER,
"GB-DEV" => GB_DEV,
"GB-DGN" => GB_DGN,
"GB-DGY" => GB_DGY,
"GB-DNC" => GB_DNC,
"GB-DND" => GB_DND,
"GB-DOR" => GB_DOR,
"GB-DOW" => GB_DOW,
"GB-DRY" => GB_DRY,
"GB-DUD" => GB_DUD,
"GB-DUR" => GB_DUR,
"GB-EAL" => GB_EAL,
"GB-EAW" => GB_EAW,
"GB-EAY" => GB_EAY,
"GB-EDH" => GB_EDH,
"GB-EDU" => GB_EDU,
"GB-ELN" => GB_ELN,
"GB-ELS" => GB_ELS,
"GB-ENF" => GB_ENF,
"GB-ENG" => GB_ENG,
"GB-ERW" => GB_ERW,
"GB-ERY" => GB_ERY,
"GB-ESS" => GB_ESS,
"GB-ESX" => GB_ESX,
"GB-FAL" => GB_FAL,
"GB-FER" => GB_FER,
"GB-FIF" => GB_FIF,
"GB-FLN" => GB_FLN,
"GB-GAT" => GB_GAT,
"GB-GBN" => GB_GBN,
"GB-GLG" => GB_GLG,
"GB-GLS" => GB_GLS,
"GB-GRE" => GB_GRE,
"GB-GWN" => GB_GWN,
"GB-HAL" => GB_HAL,
"GB-HAM" => GB_HAM,
"GB-HAV" => GB_HAV,
"GB-HCK" => GB_HCK,
"GB-HEF" => GB_HEF,
"GB-HIL" => GB_HIL,
"GB-HLD" => GB_HLD,
"GB-HMF" => GB_HMF,
"GB-HNS" => GB_HNS,
"GB-HPL" => GB_HPL,
"GB-HRT" => GB_HRT,
"GB-HRW" => GB_HRW,
"GB-HRY" => GB_HRY,
"GB-IOS" => GB_IOS,
"GB-IOW" => GB_IOW,
"GB-ISL" => GB_ISL,
"GB-IVC" => GB_IVC,
"GB-KEC" => GB_KEC,
"GB-KEN" => GB_KEN,
"GB-KHL" => GB_KHL,
"GB-KIR" => GB_KIR,
"GB-KTT" => GB_KTT,
"GB-KWL" => GB_KWL,
"GB-LAN" => GB_LAN,
"GB-LBH" => GB_LBH,
"GB-LCE" => GB_LCE,
"GB-LDS" => GB_LDS,
"GB-LEC" => GB_LEC,
"GB-LEW" => GB_LEW,
"GB-LIN" => GB_LIN,
"GB-LIV" => GB_LIV,
"GB-LMV" => GB_LMV,
"GB-LND" => GB_LND,
"GB-LRN" => GB_LRN,
"GB-LSB" => GB_LSB,
"GB-LUT" => GB_LUT,
"GB-MAN" => GB_MAN,
"GB-MDB" => GB_MDB,
"GB-MDW" => GB_MDW,
"GB-MFT" => GB_MFT,
"GB-MIK" => GB_MIK,
"GB-MLN" => GB_MLN,
"GB-MON" => GB_MON,
"GB-MRT" => GB_MRT,
"GB-MRY" => GB_MRY,
"GB-MTY" => GB_MTY,
"GB-MYL" => GB_MYL,
"GB-NAY" => GB_NAY,
"GB-NBL" => GB_NBL,
"GB-NDN" => GB_NDN,
"GB-NEL" => GB_NEL,
"GB-NET" => GB_NET,
"GB-NFK" => GB_NFK,
"GB-NGM" => GB_NGM,
"GB-NIR" => GB_NIR,
"GB-NLK" => GB_NLK,
"GB-NLN" => GB_NLN,
"GB-NSM" => GB_NSM,
"GB-NTA" => GB_NTA,
"GB-NTH" => GB_NTH,
"GB-NTL" => GB_NTL,
"GB-NTT" => GB_NTT,
"GB-NTY" => GB_NTY,
"GB-NWM" => GB_NWM,
"GB-NWP" => GB_NWP,
"GB-NYK" => GB_NYK,
"GB-NYM" => GB_NYM,
"GB-OLD" => GB_OLD,
"GB-OMH" => GB_OMH,
"GB-ORK" => GB_ORK,
"GB-OXF" => GB_OXF,
"GB-PEM" => GB_PEM,
"GB-PKN" => GB_PKN,
"GB-PLY" => GB_PLY,
"GB-POL" => GB_POL,
"GB-POR" => GB_POR,
"GB-POW" => GB_POW,
"GB-PTE" => GB_PTE,
"GB-RCC" => GB_RCC,
"GB-RCH" => GB_RCH,
"GB-RCT" => GB_RCT,
"GB-RDB" => GB_RDB,
"GB-RDG" => GB_RDG,
"GB-RFW" => GB_RFW,
"GB-RIC" => GB_RIC,
"GB-ROT" => GB_ROT,
"GB-RUT" => GB_RUT,
"GB-SAW" => GB_SAW,
"GB-SAY" => GB_SAY,
"GB-SCB" => GB_SCB,
"GB-SCT" => GB_SCT,
"GB-SFK" => GB_SFK,
"GB-SFT" => GB_SFT,
"GB-SGC" => GB_SGC,
"GB-SHF" => GB_SHF,
"GB-SHN" => GB_SHN,
"GB-SHR" => GB_SHR,
"GB-SKP" => GB_SKP,
"GB-SLF" => GB_SLF,
"GB-SLG" => GB_SLG,
"GB-SLK" => GB_SLK,
"GB-SND" => GB_SND,
"GB-SOL" => GB_SOL,
"GB-SOM" => GB_SOM,
"GB-SOS" => GB_SOS,
"GB-SRY" => GB_SRY,
"GB-STB" => GB_STB,
"GB-STE" => GB_STE,
"GB-STG" => GB_STG,
"GB-STH" => GB_STH,
"GB-STN" => GB_STN,
"GB-STS" => GB_STS,
"GB-STT" => GB_STT,
"GB-STY" => GB_STY,
"GB-SWA" => GB_SWA,
"GB-SWD" => GB_SWD,
"GB-SWK" => GB_SWK,
"GB-TAM" => GB_TAM,
"GB-TFW" => GB_TFW,
"GB-THR" => GB_THR,
"GB-TOB" => GB_TOB,
"GB-TOF" => GB_TOF,
"GB-TRF" => GB_TRF,
"GB-TWH" => GB_TWH,
"GB-UKM" => GB_UKM,
"GB-VGL" => GB_VGL,
"GB-WAR" => GB_WAR,
"GB-WBK" => GB_WBK,
"GB-WDU" => GB_WDU,
"GB-WFT" => GB_WFT,
"GB-WGN" => GB_WGN,
"GB-WIL" => GB_WIL,
"GB-WKF" => GB_WKF,
"GB-WLL" => GB_WLL,
"GB-WLN" => GB_WLN,
"GB-WLS" => GB_WLS,
"GB-WLV" => GB_WLV,
"GB-WND" => GB_WND,
"GB-WNM" => GB_WNM,
"GB-WOK" => GB_WOK,
"GB-WOR" => GB_WOR,
"GB-WRL" => GB_WRL,
"GB-WRT" => GB_WRT,
"GB-WRX" => GB_WRX,
"GB-WSM" => GB_WSM,
"GB-WSX" => GB_WSX,
"GB-YOR" => GB_YOR,
"GB-ZET" => GB_ZET,
"UM-67" => UM_67,
"UM-71" => UM_71,
"UM-76" => UM_76,
"UM-79" => UM_79,
"UM-81" => UM_81,
"UM-84" => UM_84,
"UM-86" => UM_86,
"UM-89" => UM_89,
"UM-95" => UM_95,
"US-AK" => US_AK,
"US-AL" => US_AL,
"US-AR" => US_AR,
"US-AS" => US_AS,
"US-AZ" => US_AZ,
"US-CA" => US_CA,
"US-CO" => US_CO,
"US-CT" => US_CT,
"US-DC" => US_DC,
"US-DE" => US_DE,
"US-FL" => US_FL,
"US-GA" => US_GA,
"US-GU" => US_GU,
"US-HI" => US_HI,
"US-IA" => US_IA,
"US-ID" => US_ID,
"US-IL" => US_IL,
"US-IN" => US_IN,
"US-KS" => US_KS,
"US-KY" => US_KY,
"US-LA" => US_LA,
"US-MA" => US_MA,
"US-MD" => US_MD,
"US-ME" => US_ME,
"US-MI" => US_MI,
"US-MN" => US_MN,
"US-MO" => US_MO,
"US-MP" => US_MP,
"US-MS" => US_MS,
"US-MT" => US_MT,
"US-NC" => US_NC,
"US-ND" => US_ND,
"US-NE" => US_NE,
"US-NH" => US_NH,
"US-NJ" => US_NJ,
"US-NM" => US_NM,
"US-NV" => US_NV,
"US-NY" => US_NY,
"US-OH" => US_OH,
"US-OK" => US_OK,
"US-OR" => US_OR,
"US-PA" => US_PA,
"US-PR" => US_PR,
"US-RI" => US_RI,
"US-SC" => US_SC,
"US-SD" => US_SD,
"US-TN" => US_TN,
"US-TX" => US_TX,
"US-UM" => US_UM,
"US-UT" => US_UT,
"US-VA" => US_VA,
"US-VI" => US_VI,
"US-VT" => US_VT,
"US-WA" => US_WA,
"US-WI" => US_WI,
"US-WV" => US_WV,
"US-WY" => US_WY,
"UY-AR" => UY_AR,
"UY-CA" => UY_CA,
"UY-CL" => UY_CL,
"UY-CO" => UY_CO,
"UY-DU" => UY_DU,
"UY-FD" => UY_FD,
"UY-FS" => UY_FS,
"UY-LA" => UY_LA,
"UY-MA" => UY_MA,
"UY-MO" => UY_MO,
"UY-PA" => UY_PA,
"UY-RN" => UY_RN,
"UY-RO" => UY_RO,
"UY-RV" => UY_RV,
"UY-SA" => UY_SA,
"UY-SJ" => UY_SJ,
"UY-SO" => UY_SO,
"UY-TA" => UY_TA,
"UY-TT" => UY_TT,
"UZ-AN" => UZ_AN,
"UZ-BU" => UZ_BU,
"UZ-FA" => UZ_FA,
"UZ-JI" => UZ_JI,
"UZ-NG" => UZ_NG,
"UZ-NW" => UZ_NW,
"UZ-QA" => UZ_QA,
"UZ-QR" => UZ_QR,
"UZ-SA" => UZ_SA,
"UZ-SI" => UZ_SI,
"UZ-SU" => UZ_SU,
"UZ-TK" => UZ_TK,
"UZ-TO" => UZ_TO,
"UZ-XO" => UZ_XO,
"VU-MAP" => VU_MAP,
"VU-PAM" => VU_PAM,
"VU-SAM" => VU_SAM,
"VU-SEE" => VU_SEE,
"VU-TAE" => VU_TAE,
"VU-TOB" => VU_TOB,
"VA-VA" => VA_VA,
"VE-A" => VE_A,
"VE-B" => VE_B,
"VE-C" => VE_C,
"VE-D" => VE_D,
"VE-E" => VE_E,
"VE-F" => VE_F,
"VE-G" => VE_G,
"VE-H" => VE_H,
"VE-I" => VE_I,
"VE-J" => VE_J,
"VE-K" => VE_K,
"VE-L" => VE_L,
"VE-M" => VE_M,
"VE-N" => VE_N,
"VE-O" => VE_O,
"VE-P" => VE_P,
"VE-R" => VE_R,
"VE-S" => VE_S,
"VE-T" => VE_T,
"VE-U" => VE_U,
"VE-V" => VE_V,
"VE-W" => VE_W,
"VE-X" => VE_X,
"VE-Y" => VE_Y,
"VE-Z" => VE_Z,
"VN-01" => VN_01,
"VN-02" => VN_02,
"VN-03" => VN_03,
"VN-04" => VN_04,
"VN-05" => VN_05,
"VN-06" => VN_06,
"VN-07" => VN_07,
"VN-09" => VN_09,
"VN-13" => VN_13,
"VN-14" => VN_14,
"VN-18" => VN_18,
"VN-20" => VN_20,
"VN-21" => VN_21,
"VN-22" => VN_22,
"VN-23" => VN_23,
"VN-24" => VN_24,
"VN-25" => VN_25,
"VN-26" => VN_26,
"VN-27" => VN_27,
"VN-28" => VN_28,
"VN-29" => VN_29,
"VN-30" => VN_30,
"VN-31" => VN_31,
"VN-32" => VN_32,
"VN-33" => VN_33,
"VN-34" => VN_34,
"VN-35" => VN_35,
"VN-36" => VN_36,
"VN-37" => VN_37,
"VN-39" => VN_39,
"VN-40" => VN_40,
"VN-41" => VN_41,
"VN-43" => VN_43,
"VN-44" => VN_44,
"VN-45" => VN_45,
"VN-46" => VN_46,
"VN-47" => VN_47,
"VN-49" => VN_49,
"VN-50" => VN_50,
"VN-51" => VN_51,
"VN-52" => VN_52,
"VN-53" => VN_53,
"VN-54" => VN_54,
"VN-55" => VN_55,
"VN-56" => VN_56,
"VN-57" => VN_57,
"VN-58" => VN_58,
"VN-59" => VN_59,
"VN-61" => VN_61,
"VN-63" => VN_63,
"VN-66" => VN_66,
"VN-67" => VN_67,
"VN-68" => VN_68,
"VN-69" => VN_69,
"VN-70" => VN_70,
"VN-71" => VN_71,
"VN-72" => VN_72,
"VN-73" => VN_73,
"VN-CT" => VN_CT,
"VN-DN" => VN_DN,
"VN-HN" => VN_HN,
"VN-HP" => VN_HP,
"VN-SG" => VN_SG,
"VG-VG" => VG_VG,
"VI-VI" => VI_VI,
"WF-WF" => WF_WF,
"EH-EH" => EH_EH,
"YE-AB" => YE_AB,
"YE-AD" => YE_AD,
"YE-AM" => YE_AM,
"YE-BA" => YE_BA,
"YE-DA" => YE_DA,
"YE-DH" => YE_DH,
"YE-HD" => YE_HD,
"YE-HJ" => YE_HJ,
"YE-HU" => YE_HU,
"YE-IB" => YE_IB,
"YE-JA" => YE_JA,
"YE-LA" => YE_LA,
"YE-MA" => YE_MA,
"YE-MR" => YE_MR,
"YE-MW" => YE_MW,
"YE-RA" => YE_RA,
"YE-SA" => YE_SA,
"YE-SD" => YE_SD,
"YE-SH" => YE_SH,
"YE-SN" => YE_SN,
"YE-TA" => YE_TA,
"ZM-01" => ZM_01,
"ZM-02" => ZM_02,
"ZM-03" => ZM_03,
"ZM-04" => ZM_04,
"ZM-05" => ZM_05,
"ZM-06" => ZM_06,
"ZM-07" => ZM_07,
"ZM-08" => ZM_08,
"ZM-09" => ZM_09,
"ZM-10" => ZM_10,
"ZW-BU" => ZW_BU,
"ZW-HA" => ZW_HA,
"ZW-MA" => ZW_MA,
"ZW-MC" => ZW_MC,
"ZW-ME" => ZW_ME,
"ZW-MI" => ZW_MI,
"ZW-MN" => ZW_MN,
"ZW-MS" => ZW_MS,
"ZW-MV" => ZW_MV,
"ZW-MW" => ZW_MW,

};


///Subdivision map with  Code key 
pub const SUBDIVISION_COUNTRY_MAP: Map<&str, &[Subdivision]> = phf_map! {

"AF" => &[
AF_BAL,
AF_BAM,
AF_BDG,
AF_BDS,
AF_BGL,
AF_DAY,
AF_FRA,
AF_FYB,
AF_GHA,
AF_GHO,
AF_HEL,
AF_HER,
AF_JOW,
AF_KAB,
AF_KAN,
AF_KAP,
AF_KDZ,
AF_KHO,
AF_KNR,
AF_LAG,
AF_LOG,
AF_NAN,
AF_NIM,
AF_NUR,
AF_PAN,
AF_PAR,
AF_PIA,
AF_PKA,
AF_SAM,
AF_SAR,
AF_TAK,
AF_URU,
AF_WAR,
AF_ZAB,
],
"AX" => &[
AX_AX,
],
"AL" => &[
AL_01,
AL_02,
AL_03,
AL_04,
AL_05,
AL_06,
AL_07,
AL_08,
AL_09,
AL_10,
AL_11,
AL_12,
AL_BR,
AL_BU,
AL_DI,
AL_DL,
AL_DR,
AL_DV,
AL_EL,
AL_ER,
AL_FR,
AL_GJ,
AL_GR,
AL_HA,
AL_KA,
AL_KB,
AL_KC,
AL_KO,
AL_KR,
AL_KU,
AL_LB,
AL_LE,
AL_LU,
AL_MK,
AL_MM,
AL_MR,
AL_MT,
AL_PG,
AL_PQ,
AL_PR,
AL_PU,
AL_SH,
AL_SK,
AL_SR,
AL_TE,
AL_TP,
AL_TR,
AL_VL,
],
"DZ" => &[
DZ_01,
DZ_02,
DZ_03,
DZ_04,
DZ_05,
DZ_06,
DZ_07,
DZ_08,
DZ_09,
DZ_10,
DZ_11,
DZ_12,
DZ_13,
DZ_14,
DZ_15,
DZ_16,
DZ_17,
DZ_18,
DZ_19,
DZ_20,
DZ_21,
DZ_22,
DZ_23,
DZ_24,
DZ_25,
DZ_26,
DZ_27,
DZ_28,
DZ_29,
DZ_30,
DZ_31,
DZ_32,
DZ_33,
DZ_34,
DZ_35,
DZ_36,
DZ_37,
DZ_38,
DZ_39,
DZ_40,
DZ_41,
DZ_42,
DZ_43,
DZ_44,
DZ_45,
DZ_46,
DZ_47,
DZ_48,
],
"AS" => &[
AS_AS,
],
"AD" => &[
AD_02,
AD_03,
AD_04,
AD_05,
AD_06,
AD_07,
AD_08,
],
"AO" => &[
AO_BGO,
AO_BGU,
AO_BIE,
AO_CAB,
AO_CCU,
AO_CNN,
AO_CNO,
AO_CUS,
AO_HUA,
AO_HUI,
AO_LNO,
AO_LSU,
AO_LUA,
AO_MAL,
AO_MOX,
AO_NAM,
AO_UIG,
AO_ZAI,
],
"AI" => &[
AI_AI,
],
"AQ" => &[
AQ_AQ,
],
"AG" => &[
AG_03,
AG_04,
AG_05,
AG_06,
AG_07,
AG_08,
AG_10,
AG_11,
],
"AR" => &[
AR_A,
AR_B,
AR_C,
AR_D,
AR_E,
AR_F,
AR_G,
AR_H,
AR_J,
AR_K,
AR_L,
AR_M,
AR_N,
AR_P,
AR_Q,
AR_R,
AR_S,
AR_T,
AR_U,
AR_V,
AR_W,
AR_X,
AR_Y,
AR_Z,
],
"AM" => &[
AM_AG,
AM_AR,
AM_AV,
AM_ER,
AM_GR,
AM_KT,
AM_LO,
AM_SH,
AM_SU,
AM_TV,
AM_VD,
],
"AW" => &[
AW_AW,
],
"AU" => &[
AU_ACT,
AU_NSW,
AU_NT,
AU_QLD,
AU_SA,
AU_TAS,
AU_VIC,
AU_WA,
],
"AT" => &[
AT_1,
AT_2,
AT_3,
AT_4,
AT_5,
AT_6,
AT_7,
AT_8,
AT_9,
],
"AZ" => &[
AZ_ABS,
AZ_AGA,
AZ_AGC,
AZ_AGM,
AZ_AGS,
AZ_AGU,
AZ_AST,
AZ_BA,
AZ_BAB,
AZ_BAL,
AZ_BAR,
AZ_BEY,
AZ_BIL,
AZ_CAB,
AZ_CAL,
AZ_CUL,
AZ_DAS,
AZ_FUZ,
AZ_GAD,
AZ_GA,
AZ_GOR,
AZ_GOY,
AZ_GYG,
AZ_HAC,
AZ_IMI,
AZ_ISM,
AZ_KAL,
AZ_KAN,
AZ_KUR,
AZ_LAC,
AZ_LA,
AZ_LAN,
AZ_LER,
AZ_MAS,
AZ_MI,
AZ_NA,
AZ_NEF,
AZ_NV,
AZ_NX,
AZ_OGU,
AZ_ORD,
AZ_QAB,
AZ_QAX,
AZ_QAZ,
AZ_QBA,
AZ_QBI,
AZ_QOB,
AZ_QUS,
AZ_SAB,
AZ_SAD,
AZ_SAH,
AZ_SAK,
AZ_SAL,
AZ_SAR,
AZ_SA,
AZ_SAT,
AZ_SBN,
AZ_SIY,
AZ_SKR,
AZ_SMI,
AZ_SM,
AZ_SMX,
AZ_SR,
AZ_SUS,
AZ_TAR,
AZ_TOV,
AZ_UCA,
AZ_XAC,
AZ_XA,
AZ_XCI,
AZ_XIZ,
AZ_XVD,
AZ_YAR,
AZ_YEV,
AZ_YE,
AZ_ZAN,
AZ_ZAQ,
AZ_ZAR,
],
"BS" => &[
BS_AK,
BS_BI,
BS_BP,
BS_BY,
BS_CE,
BS_CI,
BS_CK,
BS_CO,
BS_CS,
BS_EG,
BS_EX,
BS_FP,
BS_GC,
BS_HI,
BS_HT,
BS_IN,
BS_LI,
BS_MC,
BS_MG,
BS_MI,
BS_NE,
BS_NO,
BS_NS,
BS_RC,
BS_RI,
BS_SA,
BS_SE,
BS_SO,
BS_SS,
BS_SW,
BS_WG,
],
"BH" => &[
BH_13,
BH_14,
BH_15,
BH_16,
BH_17,
],
"BD" => &[
BD_01,
BD_02,
BD_03,
BD_04,
BD_05,
BD_06,
BD_07,
BD_08,
BD_09,
BD_10,
BD_11,
BD_12,
BD_13,
BD_14,
BD_15,
BD_16,
BD_17,
BD_18,
BD_19,
BD_20,
BD_21,
BD_22,
BD_23,
BD_24,
BD_25,
BD_26,
BD_27,
BD_28,
BD_29,
BD_30,
BD_31,
BD_32,
BD_33,
BD_34,
BD_35,
BD_36,
BD_37,
BD_38,
BD_39,
BD_40,
BD_41,
BD_42,
BD_43,
BD_44,
BD_45,
BD_46,
BD_47,
BD_48,
BD_49,
BD_50,
BD_51,
BD_52,
BD_53,
BD_54,
BD_55,
BD_56,
BD_57,
BD_58,
BD_59,
BD_60,
BD_61,
BD_62,
BD_63,
BD_64,
BD_A,
BD_B,
BD_C,
BD_D,
BD_E,
BD_F,
BD_G,
],
"BB" => &[
BB_01,
BB_02,
BB_03,
BB_04,
BB_05,
BB_06,
BB_07,
BB_08,
BB_09,
BB_10,
BB_11,
],
"BY" => &[
BY_BR,
BY_HM,
BY_HO,
BY_HR,
BY_MA,
BY_MI,
BY_VI,
],
"BE" => &[
BE_BRU,
BE_VAN,
BE_VBR,
BE_VLG,
BE_VLI,
BE_VOV,
BE_VWV,
BE_WAL,
BE_WBR,
BE_WHT,
BE_WLG,
BE_WLX,
BE_WNA,
],
"BZ" => &[
BZ_BZ,
BZ_CY,
BZ_CZL,
BZ_OW,
BZ_SC,
BZ_TOL,
],
"BJ" => &[
BJ_AK,
BJ_AL,
BJ_AQ,
BJ_BO,
BJ_CO,
BJ_DO,
BJ_KO,
BJ_LI,
BJ_MO,
BJ_OU,
BJ_PL,
BJ_ZO,
],
"BM" => &[
BM_BM,
],
"BT" => &[
BT_11,
BT_12,
BT_13,
BT_14,
BT_15,
BT_21,
BT_22,
BT_23,
BT_24,
BT_31,
BT_32,
BT_33,
BT_34,
BT_41,
BT_42,
BT_43,
BT_44,
BT_45,
BT_GA,
BT_TY,
],
"BO" => &[
BO_B,
BO_C,
BO_H,
BO_L,
BO_N,
BO_O,
BO_P,
BO_S,
BO_T,
],
"BQ" => &[
BQ_BO,
BQ_SA,
BQ_SE,
],
"BA" => &[
BA_01,
BA_02,
BA_03,
BA_04,
BA_05,
BA_06,
BA_07,
BA_08,
BA_09,
BA_10,
BA_BIH,
BA_BRC,
BA_SRP,
],
"BW" => &[
BW_CE,
BW_CH,
BW_FR,
BW_GA,
BW_GH,
BW_JW,
BW_KG,
BW_KL,
BW_KW,
BW_LO,
BW_NE,
BW_NW,
BW_SE,
BW_SO,
BW_SP,
BW_ST,
],
"BV" => &[
BV_BV,
],
"BR" => &[
BR_AC,
BR_AL,
BR_AM,
BR_AP,
BR_BA,
BR_CE,
BR_DF,
BR_ES,
BR_GO,
BR_MA,
BR_MG,
BR_MS,
BR_MT,
BR_PA,
BR_PB,
BR_PE,
BR_PI,
BR_PR,
BR_RJ,
BR_RN,
BR_RO,
BR_RR,
BR_RS,
BR_SC,
BR_SE,
BR_SP,
BR_TO,
],
"IO" => &[
IO_IO,
],
"BN" => &[
BN_BE,
BN_BM,
BN_TE,
BN_TU,
],
"BG" => &[
BG_01,
BG_02,
BG_03,
BG_04,
BG_05,
BG_06,
BG_07,
BG_08,
BG_09,
BG_10,
BG_11,
BG_12,
BG_13,
BG_14,
BG_15,
BG_16,
BG_17,
BG_18,
BG_19,
BG_20,
BG_21,
BG_22,
BG_23,
BG_24,
BG_25,
BG_26,
BG_27,
BG_28,
],
"BF" => &[
BF_01,
BF_02,
BF_03,
BF_04,
BF_05,
BF_06,
BF_07,
BF_08,
BF_09,
BF_10,
BF_11,
BF_12,
BF_13,
BF_BAL,
BF_BAM,
BF_BAN,
BF_BAZ,
BF_BGR,
BF_BLG,
BF_BLK,
BF_COM,
BF_GAN,
BF_GNA,
BF_GOU,
BF_HOU,
BF_IOB,
BF_KAD,
BF_KEN,
BF_KMD,
BF_KMP,
BF_KOP,
BF_KOS,
BF_KOT,
BF_KOW,
BF_LER,
BF_LOR,
BF_MOU,
BF_NAM,
BF_NAO,
BF_NAY,
BF_NOU,
BF_OUB,
BF_OUD,
BF_PAS,
BF_PON,
BF_SEN,
BF_SIS,
BF_SMT,
BF_SNG,
BF_SOM,
BF_SOR,
BF_TAP,
BF_TUI,
BF_YAG,
BF_YAT,
BF_ZIR,
BF_ZON,
BF_ZOU,
],
"BI" => &[
BI_BB,
BI_BL,
BI_BM,
BI_BR,
BI_CA,
BI_CI,
BI_GI,
BI_KI,
BI_KR,
BI_KY,
BI_MA,
BI_MU,
BI_MW,
BI_MY,
BI_NG,
BI_RT,
BI_RY,
],
"KH" => &[
KH_10,
KH_11,
KH_12,
KH_13,
KH_14,
KH_15,
KH_16,
KH_17,
KH_18,
KH_19,
KH_1,
KH_20,
KH_21,
KH_22,
KH_23,
KH_24,
KH_2,
KH_3,
KH_4,
KH_5,
KH_6,
KH_7,
KH_8,
KH_9,
],
"CM" => &[
CM_AD,
CM_CE,
CM_EN,
CM_ES,
CM_LT,
CM_NO,
CM_NW,
CM_OU,
CM_SU,
CM_SW,
],
"CA" => &[
CA_AB,
CA_BC,
CA_MB,
CA_NB,
CA_NL,
CA_NS,
CA_NT,
CA_NU,
CA_ON,
CA_PE,
CA_QC,
CA_SK,
CA_YT,
],
"CV" => &[
CV_B,
CV_BR,
CV_BV,
CV_CA,
CV_CF,
CV_CR,
CV_MA,
CV_MO,
CV_PA,
CV_PN,
CV_PR,
CV_RB,
CV_RG,
CV_RS,
CV_SD,
CV_SF,
CV_S,
CV_SL,
CV_SM,
CV_SO,
CV_SS,
CV_SV,
CV_TA,
CV_TS,
],
"KY" => &[
KY_KY,
],
"XC" => &[
XC_BA,
XC_GI,
XC_LL,
XC_TA,
],
"CF" => &[
CF_AC,
CF_BB,
CF_BGF,
CF_BK,
CF_HK,
CF_HM,
CF_HS,
CF_KB,
CF_KG,
CF_LB,
CF_MB,
CF_MP,
CF_NM,
CF_OP,
CF_SE,
CF_UK,
CF_VK,
],
"TD" => &[
TD_BA,
TD_BG,
TD_BO,
TD_CB,
TD_EE,
TD_EO,
TD_GR,
TD_HL,
TD_KA,
TD_LC,
TD_LO,
TD_LR,
TD_MA,
TD_MC,
TD_ME,
TD_MO,
TD_ND,
TD_OD,
TD_SA,
TD_SI,
TD_TA,
TD_TI,
TD_WF,
],
"CL" => &[
CL_AI,
CL_AN,
CL_AP,
CL_AR,
CL_AT,
CL_BI,
CL_CO,
CL_LI,
CL_LL,
CL_LR,
CL_MA,
CL_ML,
CL_RM,
CL_TA,
CL_VS,
],
"CN" => &[
CN_11,
CN_12,
CN_13,
CN_14,
CN_15,
CN_21,
CN_22,
CN_23,
CN_31,
CN_32,
CN_33,
CN_34,
CN_35,
CN_36,
CN_37,
CN_41,
CN_42,
CN_43,
CN_44,
CN_45,
CN_46,
CN_50,
CN_51,
CN_52,
CN_53,
CN_54,
CN_61,
CN_62,
CN_63,
CN_64,
CN_65,
CN_71,
CN_91,
CN_92,
],
"CX" => &[
CX_CX,
],
"CC" => &[
CC_CC,
],
"CO" => &[
CO_AMA,
CO_ANT,
CO_ARA,
CO_ATL,
CO_BOL,
CO_BOY,
CO_CAL,
CO_CAQ,
CO_CAS,
CO_CAU,
CO_CES,
CO_CHO,
CO_COR,
CO_CUN,
CO_DC,
CO_GUA,
CO_GUV,
CO_HUI,
CO_LAG,
CO_MAG,
CO_MET,
CO_NAR,
CO_NSA,
CO_PUT,
CO_QUI,
CO_RIS,
CO_SAN,
CO_SAP,
CO_SUC,
CO_TOL,
CO_VAC,
CO_VAU,
CO_VID,
],
"KM" => &[
KM_A,
KM_G,
KM_M,
],
"CG" => &[
CG_11,
CG_12,
CG_13,
CG_14,
CG_15,
CG_16,
CG_2,
CG_5,
CG_7,
CG_8,
CG_9,
CG_BZV,
],
"CD" => &[
CD_BC,
CD_BN,
CD_EQ,
CD_KA,
CD_KE,
CD_KN,
CD_KW,
CD_MA,
CD_NK,
CD_OR,
CD_SK,
],
"CK" => &[
CK_CK,
],
"CR" => &[
CR_A,
CR_C,
CR_G,
CR_H,
CR_L,
CR_P,
CR_SJ,
],
"CI" => &[
CI_01,
CI_02,
CI_03,
CI_04,
CI_05,
CI_06,
CI_07,
CI_08,
CI_09,
CI_10,
CI_11,
CI_12,
CI_13,
CI_14,
CI_15,
CI_16,
CI_17,
CI_18,
CI_19,
],
"HR" => &[
HR_01,
HR_02,
HR_03,
HR_04,
HR_05,
HR_06,
HR_07,
HR_08,
HR_09,
HR_10,
HR_11,
HR_12,
HR_13,
HR_14,
HR_15,
HR_16,
HR_17,
HR_18,
HR_19,
HR_20,
HR_21,
],
"CU" => &[
CU_01,
CU_03,
CU_04,
CU_05,
CU_06,
CU_07,
CU_08,
CU_09,
CU_10,
CU_11,
CU_12,
CU_13,
CU_14,
CU_15,
CU_16,
CU_99,
],
"CW" => &[
CW_CW,
],
"CY" => &[
CY_01,
CY_02,
CY_03,
CY_04,
CY_05,
CY_06,
],
"CZ" => &[
CZ_101,
CZ_102,
CZ_103,
CZ_104,
CZ_105,
CZ_106,
CZ_107,
CZ_108,
CZ_109,
CZ_10A,
CZ_10B,
CZ_10C,
CZ_10D,
CZ_10E,
CZ_10F,
CZ_201,
CZ_202,
CZ_203,
CZ_204,
CZ_205,
CZ_206,
CZ_207,
CZ_208,
CZ_209,
CZ_20A,
CZ_20B,
CZ_20C,
CZ_311,
CZ_312,
CZ_313,
CZ_314,
CZ_315,
CZ_316,
CZ_317,
CZ_321,
CZ_322,
CZ_323,
CZ_324,
CZ_325,
CZ_326,
CZ_327,
CZ_411,
CZ_412,
CZ_413,
CZ_421,
CZ_422,
CZ_423,
CZ_424,
CZ_425,
CZ_426,
CZ_427,
CZ_511,
CZ_512,
CZ_513,
CZ_514,
CZ_521,
CZ_522,
CZ_523,
CZ_524,
CZ_525,
CZ_531,
CZ_532,
CZ_533,
CZ_534,
CZ_611,
CZ_612,
CZ_613,
CZ_614,
CZ_615,
CZ_621,
CZ_622,
CZ_623,
CZ_624,
CZ_625,
CZ_626,
CZ_627,
CZ_711,
CZ_712,
CZ_713,
CZ_714,
CZ_715,
CZ_721,
CZ_722,
CZ_723,
CZ_724,
CZ_801,
CZ_802,
CZ_803,
CZ_804,
CZ_805,
CZ_806,
CZ_JC,
CZ_JM,
CZ_KA,
CZ_KR,
CZ_LI,
CZ_MO,
CZ_OL,
CZ_PA,
CZ_PL,
CZ_PR,
CZ_ST,
CZ_US,
CZ_VY,
CZ_ZL,
],
"DK" => &[
DK_81,
DK_82,
DK_83,
DK_84,
DK_85,
],
"DJ" => &[
DJ_AR,
DJ_AS,
DJ_DI,
DJ_DJ,
DJ_OB,
DJ_TA,
],
"DM" => &[
DM_02,
DM_03,
DM_04,
DM_05,
DM_06,
DM_07,
DM_08,
DM_09,
DM_10,
DM_11,
],
"DO" => &[
DO_01,
DO_02,
DO_03,
DO_04,
DO_05,
DO_06,
DO_07,
DO_08,
DO_09,
DO_10,
DO_11,
DO_12,
DO_13,
DO_14,
DO_15,
DO_16,
DO_17,
DO_18,
DO_19,
DO_20,
DO_21,
DO_22,
DO_23,
DO_24,
DO_25,
DO_26,
DO_27,
DO_28,
DO_29,
DO_30,
DO_31,
DO_32,
DO_33,
DO_34,
DO_35,
DO_36,
DO_37,
DO_38,
DO_39,
DO_40,
DO_41,
DO_42,
],
"EC" => &[
EC_A,
EC_B,
EC_C,
EC_D,
EC_E,
EC_F,
EC_G,
EC_H,
EC_I,
EC_L,
EC_M,
EC_N,
EC_O,
EC_P,
EC_R,
EC_SD,
EC_SE,
EC_S,
EC_T,
EC_U,
EC_W,
EC_X,
EC_Y,
EC_Z,
],
"EG" => &[
EG_ALX,
EG_ASN,
EG_AST,
EG_BA,
EG_BH,
EG_BNS,
EG_C,
EG_DK,
EG_DT,
EG_FYM,
EG_GH,
EG_GZ,
EG_IS,
EG_JS,
EG_KB,
EG_KFS,
EG_KN,
EG_LX,
EG_MN,
EG_MNF,
EG_MT,
EG_PTS,
EG_SHG,
EG_SHR,
EG_SIN,
EG_SUZ,
EG_WAD,
],
"SV" => &[
SV_AH,
SV_CA,
SV_CH,
SV_CU,
SV_LI,
SV_MO,
SV_PA,
SV_SA,
SV_SM,
SV_SO,
SV_SS,
SV_SV,
SV_UN,
SV_US,
],
"GQ" => &[
GQ_AN,
GQ_BN,
GQ_BS,
GQ_C,
GQ_CS,
GQ_I,
GQ_KN,
GQ_LI,
GQ_WN,
],
"ER" => &[
ER_AN,
ER_DK,
ER_DU,
ER_GB,
ER_MA,
ER_SK,
],
"EE" => &[
EE_37,
EE_39,
EE_44,
EE_49,
EE_51,
EE_57,
EE_59,
EE_65,
EE_67,
EE_70,
EE_74,
EE_78,
EE_82,
EE_84,
EE_86,
],
"ET" => &[
ET_AA,
ET_AF,
ET_AM,
ET_BE,
ET_DD,
ET_GA,
ET_HA,
ET_OR,
ET_SN,
ET_SO,
ET_TI,
],
"FK" => &[
FK_FK,
],
"FO" => &[
FO_FO,
],
"FJ" => &[
FJ_01,
FJ_02,
FJ_03,
FJ_04,
FJ_05,
FJ_06,
FJ_07,
FJ_08,
FJ_09,
FJ_10,
FJ_11,
FJ_12,
FJ_13,
FJ_14,
FJ_C,
FJ_E,
FJ_N,
FJ_R,
FJ_W,
],
"FI" => &[
FI_01,
FI_02,
FI_03,
FI_04,
FI_05,
FI_06,
FI_07,
FI_08,
FI_09,
FI_10,
FI_11,
FI_12,
FI_13,
FI_14,
FI_15,
FI_16,
FI_17,
FI_18,
FI_19,
],
"FR" => &[
FR_01,
FR_02,
FR_03,
FR_04,
FR_05,
FR_06,
FR_07,
FR_08,
FR_09,
FR_10,
FR_11,
FR_12,
FR_13,
FR_14,
FR_15,
FR_16,
FR_17,
FR_18,
FR_19,
FR_21,
FR_22,
FR_23,
FR_24,
FR_25,
FR_26,
FR_27,
FR_28,
FR_29,
FR_2A,
FR_2B,
FR_30,
FR_31,
FR_32,
FR_33,
FR_34,
FR_35,
FR_36,
FR_37,
FR_38,
FR_39,
FR_40,
FR_41,
FR_42,
FR_43,
FR_44,
FR_45,
FR_46,
FR_47,
FR_48,
FR_49,
FR_50,
FR_51,
FR_52,
FR_53,
FR_54,
FR_55,
FR_56,
FR_57,
FR_58,
FR_59,
FR_60,
FR_61,
FR_62,
FR_63,
FR_64,
FR_65,
FR_66,
FR_67,
FR_68,
FR_69,
FR_70,
FR_71,
FR_72,
FR_73,
FR_74,
FR_75,
FR_76,
FR_77,
FR_78,
FR_79,
FR_80,
FR_81,
FR_82,
FR_83,
FR_84,
FR_85,
FR_86,
FR_87,
FR_88,
FR_89,
FR_90,
FR_91,
FR_92,
FR_93,
FR_94,
FR_95,
FR_A,
FR_B,
FR_BL,
FR_C,
FR_CP,
FR_D,
FR_E,
FR_F,
FR_G,
FR_GF,
FR_GP,
FR_H,
FR_I,
FR_J,
FR_K,
FR_L,
FR_MF,
FR_M,
FR_MQ,
FR_NC,
FR_N,
FR_O,
FR_P,
FR_PF,
FR_PM,
FR_Q,
FR_RE,
FR_R,
FR_S,
FR_TF,
FR_T,
FR_U,
FR_V,
FR_WF,
FR_YT,
FR_ARA,
FR_BFC,
FR_BRE,
FR_CVL,
FR_20R,
FR_GES,
FR_HDF,
FR_IDF,
FR_NOR,
FR_NAQ,
FR_OCC,
FR_PDL,
FR_PAC,
],
"GF" => &[
GF_GF,
],
"PF" => &[
PF_PF,
],
"TF" => &[
TF_TF,
],
"GA" => &[
GA_1,
GA_2,
GA_3,
GA_4,
GA_5,
GA_6,
GA_7,
GA_8,
GA_9,
],
"GM" => &[
GM_B,
GM_L,
GM_M,
GM_N,
GM_U,
GM_W,
],
"GE" => &[
GE_AB,
GE_AJ,
GE_GU,
GE_IM,
GE_KA,
GE_KK,
GE_MM,
GE_RL,
GE_SJ,
GE_SK,
GE_SZ,
GE_TB,
],
"DE" => &[
DE_BB,
DE_BE,
DE_BW,
DE_BY,
DE_HB,
DE_HE,
DE_HH,
DE_MV,
DE_NI,
DE_NW,
DE_RP,
DE_SH,
DE_SL,
DE_SN,
DE_ST,
DE_TH,
],
"GH" => &[
GH_AA,
GH_AH,
GH_BA,
GH_CP,
GH_EP,
GH_NP,
GH_TV,
GH_UE,
GH_UW,
GH_WP,
],
"GI" => &[
GI_GI,
],
"GR" => &[
GR_01,
GR_03,
GR_04,
GR_05,
GR_06,
GR_07,
GR_11,
GR_12,
GR_13,
GR_14,
GR_15,
GR_16,
GR_17,
GR_21,
GR_22,
GR_23,
GR_24,
GR_31,
GR_32,
GR_33,
GR_34,
GR_41,
GR_42,
GR_43,
GR_44,
GR_51,
GR_52,
GR_53,
GR_54,
GR_55,
GR_56,
GR_57,
GR_58,
GR_59,
GR_61,
GR_62,
GR_63,
GR_64,
GR_69,
GR_71,
GR_72,
GR_73,
GR_81,
GR_82,
GR_83,
GR_84,
GR_85,
GR_91,
GR_92,
GR_93,
GR_94,
GR_A1,
GR_A,
GR_B,
GR_C,
GR_D,
GR_E,
GR_F,
GR_G,
GR_H,
GR_I,
GR_J,
GR_K,
GR_L,
GR_M,
],
"GL" => &[
GL_KU,
GL_QA,
GL_QE,
GL_SM,
],
"GD" => &[
GD_01,
GD_02,
GD_03,
GD_04,
GD_05,
GD_06,
GD_10,
],
"GP" => &[
GP_GP,
],
"GU" => &[
GU_GU,
],
"GT" => &[
GT_AV,
GT_BV,
GT_CM,
GT_CQ,
GT_ES,
GT_GU,
GT_HU,
GT_IZ,
GT_JA,
GT_JU,
GT_PE,
GT_PR,
GT_QC,
GT_QZ,
GT_RE,
GT_SA,
GT_SM,
GT_SO,
GT_SR,
GT_SU,
GT_TO,
GT_ZA,
],
"GG" => &[
GG_GG,
],
"GW" => &[
GW_BA,
GW_BL,
GW_BM,
GW_BS,
GW_CA,
GW_GA,
GW_L,
GW_N,
GW_OI,
GW_QU,
GW_S,
GW_TO,
],
"GN" => &[
GN_B,
GN_BE,
GN_BF,
GN_BK,
GN_C,
GN_CO,
GN_DB,
GN_DI,
GN_D,
GN_DL,
GN_DU,
GN_FA,
GN_F,
GN_FO,
GN_FR,
GN_GA,
GN_GU,
GN_KA,
GN_KB,
GN_KD,
GN_KE,
GN_K,
GN_KN,
GN_KO,
GN_KS,
GN_LA,
GN_LE,
GN_L,
GN_LO,
GN_MC,
GN_MD,
GN_ML,
GN_M,
GN_MM,
GN_N,
GN_NZ,
GN_PI,
GN_SI,
GN_TE,
GN_TO,
GN_YO,
],
"GY" => &[
GY_BA,
GY_CU,
GY_DE,
GY_EB,
GY_ES,
GY_MA,
GY_PM,
GY_PT,
GY_UD,
GY_UT,
],
"HT" => &[
HT_AR,
HT_CE,
HT_GA,
HT_ND,
HT_NE,
HT_NI,
HT_NO,
HT_OU,
HT_SD,
HT_SE,
],
"HM" => &[
HM_HM,
],
"HN" => &[
HN_AT,
HN_CH,
HN_CL,
HN_CM,
HN_CP,
HN_CR,
HN_EP,
HN_FM,
HN_GD,
HN_IB,
HN_IN,
HN_LE,
HN_LP,
HN_OC,
HN_OL,
HN_SB,
HN_VA,
HN_YO,
],
"HK" => &[
HK_HK,
],
"HU" => &[
HU_BA,
HU_BC,
HU_BE,
HU_BK,
HU_BU,
HU_BZ,
HU_CS,
HU_DE,
HU_DU,
HU_EG,
HU_ER,
HU_FE,
HU_GS,
HU_GY,
HU_HB,
HU_HE,
HU_HV,
HU_JN,
HU_KE,
HU_KM,
HU_KV,
HU_MI,
HU_NK,
HU_NO,
HU_NY,
HU_PE,
HU_PS,
HU_SD,
HU_SF,
HU_SH,
HU_SK,
HU_SN,
HU_SO,
HU_SS,
HU_ST,
HU_SZ,
HU_TB,
HU_TO,
HU_VA,
HU_VE,
HU_VM,
HU_ZA,
HU_ZE,
],
"IS" => &[
IS_0,
IS_1,
IS_2,
IS_3,
IS_4,
IS_5,
IS_6,
IS_7,
IS_8,
],
"IN" => &[
IN_AN,
IN_AP,
IN_AR,
IN_AS,
IN_BR,
IN_CH,
IN_CT,
IN_DD,
IN_DL,
IN_DN,
IN_GA,
IN_GJ,
IN_HP,
IN_HR,
IN_JH,
IN_JK,
IN_KA,
IN_KL,
IN_LD,
IN_MH,
IN_ML,
IN_MN,
IN_MP,
IN_MZ,
IN_NL,
IN_OR,
IN_PB,
IN_PY,
IN_RJ,
IN_SK,
IN_TG,
IN_TN,
IN_TR,
IN_UP,
IN_UT,
IN_WB,
],
"ID" => &[
ID_AC,
ID_BA,
ID_BB,
ID_BE,
ID_BT,
ID_GO,
ID_JA,
ID_JB,
ID_JI,
ID_JK,
ID_JT,
ID_JW,
ID_KA,
ID_KB,
ID_KI,
ID_KR,
ID_KS,
ID_KT,
ID_KU,
ID_LA,
ID_MA,
ID_ML,
ID_MU,
ID_NB,
ID_NT,
ID_NU,
ID_PA,
ID_PB,
ID_PP,
ID_RI,
ID_SA,
ID_SB,
ID_SG,
ID_SL,
ID_SM,
ID_SN,
ID_SR,
ID_SS,
ID_ST,
ID_SU,
ID_YO,
],
"IR" => &[
IR_01,
IR_02,
IR_03,
IR_04,
IR_05,
IR_06,
IR_07,
IR_08,
IR_10,
IR_11,
IR_12,
IR_13,
IR_14,
IR_15,
IR_16,
IR_17,
IR_18,
IR_19,
IR_20,
IR_21,
IR_22,
IR_23,
IR_24,
IR_25,
IR_26,
IR_27,
IR_28,
IR_29,
IR_30,
IR_31,
IR_32,
],
"IQ" => &[
IQ_AN,
IQ_AR,
IQ_BA,
IQ_BB,
IQ_BG,
IQ_DA,
IQ_DI,
IQ_DQ,
IQ_KA,
IQ_KI,
IQ_MA,
IQ_MU,
IQ_NA,
IQ_NI,
IQ_QA,
IQ_SD,
IQ_SU,
IQ_WA,
],
"IE" => &[
IE_C,
IE_CE,
IE_CN,
IE_CO,
IE_CW,
IE_D,
IE_DL,
IE_G,
IE_KE,
IE_KK,
IE_KY,
IE_LD,
IE_LH,
IE_LK,
IE_L,
IE_LM,
IE_LS,
IE_M,
IE_MH,
IE_MN,
IE_MO,
IE_OY,
IE_RN,
IE_SO,
IE_TA,
IE_U,
IE_WD,
IE_WH,
IE_WW,
IE_WX,
],
"IM" => &[
IM_IM,
],
"IL" => &[
IL_D,
IL_HA,
IL_JM,
IL_M,
IL_TA,
IL_Z,
],
"IT" => &[
IT_21,
IT_23,
IT_25,
IT_32,
IT_34,
IT_36,
IT_42,
IT_45,
IT_52,
IT_55,
IT_57,
IT_62,
IT_65,
IT_67,
IT_72,
IT_75,
IT_77,
IT_78,
IT_82,
IT_88,
IT_AG,
IT_AL,
IT_AN,
IT_AO,
IT_AP,
IT_AQ,
IT_AR,
IT_AT,
IT_AV,
IT_BA,
IT_BG,
IT_BI,
IT_BL,
IT_BN,
IT_BO,
IT_BR,
IT_BS,
IT_BT,
IT_BZ,
IT_CA,
IT_CB,
IT_CE,
IT_CH,
IT_CI,
IT_CL,
IT_CN,
IT_CO,
IT_CR,
IT_CS,
IT_CT,
IT_CZ,
IT_EN,
IT_FC,
IT_FE,
IT_FG,
IT_FI,
IT_FM,
IT_FR,
IT_GE,
IT_GO,
IT_GR,
IT_IM,
IT_IS,
IT_KR,
IT_LC,
IT_LE,
IT_LI,
IT_LO,
IT_LT,
IT_LU,
IT_MB,
IT_MC,
IT_ME,
IT_MI,
IT_MN,
IT_MO,
IT_MS,
IT_MT,
IT_NA,
IT_NO,
IT_NU,
IT_OG,
IT_OR,
IT_OT,
IT_PA,
IT_PC,
IT_PD,
IT_PE,
IT_PG,
IT_PI,
IT_PN,
IT_PO,
IT_PR,
IT_PT,
IT_PU,
IT_PV,
IT_PZ,
IT_RA,
IT_RC,
IT_RE,
IT_RG,
IT_RI,
IT_RM,
IT_RN,
IT_RO,
IT_SA,
IT_SI,
IT_SO,
IT_SP,
IT_SR,
IT_SS,
IT_SV,
IT_TA,
IT_TE,
IT_TN,
IT_TO,
IT_TP,
IT_TR,
IT_TS,
IT_TV,
IT_UD,
IT_VA,
IT_VB,
IT_VC,
IT_VE,
IT_VI,
IT_VR,
IT_VS,
IT_VT,
IT_VV,
],
"JM" => &[
JM_01,
JM_02,
JM_03,
JM_04,
JM_05,
JM_06,
JM_07,
JM_08,
JM_09,
JM_10,
JM_11,
JM_12,
JM_13,
JM_14,
],
"JP" => &[
JP_01,
JP_02,
JP_03,
JP_04,
JP_05,
JP_06,
JP_07,
JP_08,
JP_09,
JP_10,
JP_11,
JP_12,
JP_13,
JP_14,
JP_15,
JP_16,
JP_17,
JP_18,
JP_19,
JP_20,
JP_21,
JP_22,
JP_23,
JP_24,
JP_25,
JP_26,
JP_27,
JP_28,
JP_29,
JP_30,
JP_31,
JP_32,
JP_33,
JP_34,
JP_35,
JP_36,
JP_37,
JP_38,
JP_39,
JP_40,
JP_41,
JP_42,
JP_43,
JP_44,
JP_45,
JP_46,
JP_47,
],
"JE" => &[
JE_JE,
],
"JO" => &[
JO_AJ,
JO_AM,
JO_AQ,
JO_AT,
JO_AZ,
JO_BA,
JO_IR,
JO_JA,
JO_KA,
JO_MA,
JO_MD,
JO_MN,
],
"KZ" => &[
KZ_AKM,
KZ_AKT,
KZ_ALA,
KZ_ALM,
KZ_AST,
KZ_ATY,
KZ_KAR,
KZ_KUS,
KZ_KZY,
KZ_MAN,
KZ_PAV,
KZ_SEV,
KZ_VOS,
KZ_YUZ,
KZ_ZAP,
KZ_ZHA,
],
"KE" => &[
KE_01,
KE_02,
KE_03,
KE_04,
KE_05,
KE_06,
KE_07,
KE_08,
KE_09,
KE_10,
KE_11,
KE_12,
KE_13,
KE_14,
KE_15,
KE_16,
KE_17,
KE_18,
KE_19,
KE_20,
KE_21,
KE_22,
KE_23,
KE_24,
KE_25,
KE_26,
KE_27,
KE_28,
KE_29,
KE_30,
KE_31,
KE_32,
KE_33,
KE_34,
KE_35,
KE_36,
KE_37,
KE_38,
KE_39,
KE_40,
KE_41,
KE_42,
KE_43,
KE_44,
KE_45,
KE_46,
KE_47,
],
"KI" => &[
KI_G,
KI_L,
KI_P,
],
"KP" => &[
KP_01,
KP_02,
KP_03,
KP_04,
KP_05,
KP_06,
KP_07,
KP_08,
KP_09,
KP_10,
KP_13,
],
"KR" => &[
KR_11,
KR_26,
KR_27,
KR_28,
KR_29,
KR_30,
KR_31,
KR_41,
KR_42,
KR_43,
KR_44,
KR_45,
KR_46,
KR_47,
KR_48,
KR_49,
KR_50,
],
"KW" => &[
KW_AH,
KW_FA,
KW_HA,
KW_JA,
KW_KU,
KW_MU,
],
"KG" => &[
KG_B,
KG_C,
KG_GB,
KG_GO,
KG_J,
KG_N,
KG_O,
KG_T,
KG_Y,
],
"LA" => &[
LA_AT,
LA_BK,
LA_BL,
LA_CH,
LA_HO,
LA_KH,
LA_LM,
LA_LP,
LA_OU,
LA_PH,
LA_SL,
LA_SV,
LA_VI,
LA_VT,
LA_XA,
LA_XE,
LA_XI,
LA_XN,
],
"LV" => &[
LV_001,
LV_002,
LV_003,
LV_004,
LV_005,
LV_006,
LV_007,
LV_008,
LV_009,
LV_010,
LV_011,
LV_012,
LV_013,
LV_014,
LV_015,
LV_016,
LV_017,
LV_018,
LV_019,
LV_020,
LV_021,
LV_022,
LV_023,
LV_024,
LV_025,
LV_026,
LV_027,
LV_028,
LV_029,
LV_030,
LV_031,
LV_032,
LV_033,
LV_034,
LV_035,
LV_036,
LV_037,
LV_038,
LV_039,
LV_040,
LV_041,
LV_042,
LV_043,
LV_044,
LV_045,
LV_046,
LV_047,
LV_048,
LV_049,
LV_050,
LV_051,
LV_052,
LV_053,
LV_054,
LV_055,
LV_056,
LV_057,
LV_058,
LV_059,
LV_060,
LV_061,
LV_062,
LV_063,
LV_064,
LV_065,
LV_066,
LV_067,
LV_068,
LV_069,
LV_070,
LV_071,
LV_072,
LV_073,
LV_074,
LV_075,
LV_076,
LV_077,
LV_078,
LV_079,
LV_080,
LV_081,
LV_082,
LV_083,
LV_084,
LV_085,
LV_086,
LV_087,
LV_088,
LV_089,
LV_090,
LV_091,
LV_092,
LV_093,
LV_094,
LV_095,
LV_096,
LV_097,
LV_098,
LV_099,
LV_100,
LV_101,
LV_102,
LV_103,
LV_104,
LV_105,
LV_106,
LV_107,
LV_108,
LV_109,
LV_110,
LV_DGV,
LV_JEL,
LV_JKB,
LV_JUR,
LV_LPX,
LV_REZ,
LV_RIX,
LV_VEN,
LV_VMR,
],
"LB" => &[
LB_AK,
LB_AS,
LB_BA,
LB_BH,
LB_BI,
LB_JA,
LB_JL,
LB_NA,
],
"LS" => &[
LS_A,
LS_B,
LS_C,
LS_D,
LS_E,
LS_F,
LS_G,
LS_H,
LS_J,
LS_K,
],
"LR" => &[
LR_BG,
LR_BM,
LR_CM,
LR_GB,
LR_GG,
LR_GK,
LR_GP,
LR_LO,
LR_MG,
LR_MO,
LR_MY,
LR_NI,
LR_RG,
LR_RI,
LR_SI,
],
"LY" => &[
LY_BA,
LY_BU,
LY_DR,
LY_GT,
LY_JA,
LY_JG,
LY_JI,
LY_JU,
LY_KF,
LY_MB,
LY_MI,
LY_MJ,
LY_MQ,
LY_NL,
LY_NQ,
LY_SB,
LY_SR,
LY_TB,
LY_WA,
LY_WD,
LY_WS,
LY_ZA,
],
"LI" => &[
LI_01,
LI_02,
LI_03,
LI_04,
LI_05,
LI_06,
LI_07,
LI_08,
LI_09,
LI_10,
LI_11,
],
"LT" => &[
LT_01,
LT_02,
LT_03,
LT_04,
LT_05,
LT_06,
LT_07,
LT_08,
LT_09,
LT_10,
LT_11,
LT_12,
LT_13,
LT_14,
LT_15,
LT_16,
LT_17,
LT_18,
LT_19,
LT_20,
LT_21,
LT_22,
LT_23,
LT_24,
LT_25,
LT_26,
LT_27,
LT_28,
LT_29,
LT_30,
LT_31,
LT_32,
LT_33,
LT_34,
LT_35,
LT_36,
LT_37,
LT_38,
LT_39,
LT_40,
LT_41,
LT_42,
LT_43,
LT_44,
LT_45,
LT_46,
LT_47,
LT_48,
LT_49,
LT_50,
LT_51,
LT_52,
LT_53,
LT_54,
LT_55,
LT_56,
LT_57,
LT_58,
LT_59,
LT_60,
LT_AL,
LT_KL,
LT_KU,
LT_MR,
LT_PN,
LT_SA,
LT_TA,
LT_TE,
LT_UT,
LT_VL,
],
"LU" => &[
LU_D,
LU_G,
LU_L,
],
"MO" => &[
MO_MO,
],
"MK" => &[
MK_01,
MK_02,
MK_03,
MK_04,
MK_05,
MK_06,
MK_07,
MK_08,
MK_09,
MK_10,
MK_11,
MK_12,
MK_13,
MK_14,
MK_15,
MK_16,
MK_17,
MK_18,
MK_19,
MK_20,
MK_21,
MK_22,
MK_23,
MK_24,
MK_25,
MK_26,
MK_27,
MK_28,
MK_29,
MK_30,
MK_31,
MK_32,
MK_33,
MK_34,
MK_35,
MK_36,
MK_37,
MK_38,
MK_39,
MK_40,
MK_41,
MK_42,
MK_43,
MK_44,
MK_45,
MK_46,
MK_47,
MK_48,
MK_49,
MK_50,
MK_51,
MK_52,
MK_53,
MK_54,
MK_55,
MK_56,
MK_57,
MK_58,
MK_59,
MK_60,
MK_61,
MK_62,
MK_63,
MK_64,
MK_65,
MK_66,
MK_67,
MK_68,
MK_69,
MK_70,
MK_71,
MK_72,
MK_73,
MK_74,
MK_75,
MK_76,
MK_77,
MK_78,
MK_79,
MK_80,
MK_81,
MK_82,
MK_83,
MK_84,
],
"MG" => &[
MG_A,
MG_D,
MG_F,
MG_M,
MG_T,
MG_U,
],
"MW" => &[
MW_BA,
MW_BL,
MW_C,
MW_CK,
MW_CR,
MW_CT,
MW_DE,
MW_DO,
MW_KR,
MW_KS,
MW_LI,
MW_LK,
MW_MC,
MW_MG,
MW_MH,
MW_MU,
MW_MW,
MW_MZ,
MW_NB,
MW_N,
MW_NE,
MW_NI,
MW_NK,
MW_NS,
MW_NU,
MW_PH,
MW_RU,
MW_SA,
MW_S,
MW_TH,
MW_ZO,
],
"MY" => &[
MY_01,
MY_02,
MY_03,
MY_04,
MY_05,
MY_06,
MY_07,
MY_08,
MY_09,
MY_10,
MY_11,
MY_12,
MY_13,
MY_14,
MY_15,
MY_16,
],
"MV" => &[
MV_00,
MV_01,
MV_02,
MV_03,
MV_04,
MV_05,
MV_07,
MV_08,
MV_12,
MV_13,
MV_14,
MV_17,
MV_20,
MV_23,
MV_24,
MV_25,
MV_26,
MV_27,
MV_28,
MV_29,
MV_CE,
MV_MLE,
MV_NC,
MV_NO,
MV_SC,
MV_SU,
MV_UN,
MV_US,
],
"ML" => &[
ML_1,
ML_2,
ML_3,
ML_4,
ML_5,
ML_6,
ML_7,
ML_8,
ML_BKO,
],
"MT" => &[
MT_01,
MT_02,
MT_03,
MT_04,
MT_05,
MT_06,
MT_07,
MT_08,
MT_09,
MT_10,
MT_11,
MT_12,
MT_13,
MT_14,
MT_15,
MT_16,
MT_17,
MT_18,
MT_19,
MT_20,
MT_21,
MT_22,
MT_23,
MT_24,
MT_25,
MT_26,
MT_27,
MT_28,
MT_29,
MT_30,
MT_31,
MT_32,
MT_33,
MT_34,
MT_35,
MT_36,
MT_37,
MT_38,
MT_39,
MT_40,
MT_41,
MT_42,
MT_43,
MT_44,
MT_45,
MT_46,
MT_47,
MT_48,
MT_49,
MT_50,
MT_51,
MT_52,
MT_53,
MT_54,
MT_55,
MT_56,
MT_57,
MT_58,
MT_59,
MT_60,
MT_61,
MT_62,
MT_63,
MT_64,
MT_65,
MT_66,
MT_67,
MT_68,
],
"MH" => &[
MH_ALK,
MH_ALL,
MH_ARN,
MH_AUR,
MH_EBO,
MH_ENI,
MH_JAB,
MH_JAL,
MH_KIL,
MH_KWA,
MH_LAE,
MH_LIB,
MH_LIK,
MH_L,
MH_MAJ,
MH_MAL,
MH_MEJ,
MH_MIL,
MH_NMK,
MH_NMU,
MH_RON,
MH_T,
MH_UJA,
MH_UTI,
MH_WTH,
MH_WTJ,
],
"MQ" => &[
MQ_MQ,
],
"MR" => &[
MR_01,
MR_02,
MR_03,
MR_04,
MR_05,
MR_06,
MR_07,
MR_08,
MR_09,
MR_10,
MR_11,
MR_12,
MR_NKC,
],
"MU" => &[
MU_AG,
MU_BL,
MU_BR,
MU_CC,
MU_CU,
MU_FL,
MU_GP,
MU_MO,
MU_PA,
MU_PL,
MU_PU,
MU_PW,
MU_QB,
MU_RO,
MU_RR,
MU_SA,
MU_VP,
],
"YT" => &[
YT_YT,
],
"MX" => &[
MX_AGU,
MX_BCN,
MX_BCS,
MX_CAM,
MX_CHH,
MX_CHP,
MX_COA,
MX_COL,
MX_CMX,
MX_DUR,
MX_GRO,
MX_GUA,
MX_HID,
MX_JAL,
MX_MEX,
MX_MIC,
MX_MOR,
MX_NAY,
MX_NLE,
MX_OAX,
MX_PUE,
MX_QUE,
MX_ROO,
MX_SIN,
MX_SLP,
MX_SON,
MX_TAB,
MX_TAM,
MX_TLA,
MX_VER,
MX_YUC,
MX_ZAC,
],
"FM" => &[
FM_KSA,
FM_PNI,
FM_TRK,
FM_YAP,
],
"MD" => &[
MD_AN,
MD_BA,
MD_BD,
MD_BR,
MD_BS,
MD_CA,
MD_CL,
MD_CM,
MD_CR,
MD_CS,
MD_CT,
MD_CU,
MD_DO,
MD_DR,
MD_DU,
MD_ED,
MD_FA,
MD_FL,
MD_GA,
MD_GL,
MD_HI,
MD_IA,
MD_LE,
MD_NI,
MD_OC,
MD_OR,
MD_RE,
MD_RI,
MD_SD,
MD_SI,
MD_SN,
MD_SO,
MD_ST,
MD_SV,
MD_TA,
MD_TE,
MD_UN,
],
"MC" => &[
MC_CL,
MC_CO,
MC_FO,
MC_GA,
MC_JE,
MC_LA,
MC_MA,
MC_MC,
MC_MG,
MC_MO,
MC_MU,
MC_PH,
MC_SD,
MC_SO,
MC_SP,
MC_SR,
MC_VR,
],
"MN" => &[
MN_035,
MN_037,
MN_039,
MN_041,
MN_043,
MN_046,
MN_047,
MN_049,
MN_051,
MN_053,
MN_055,
MN_057,
MN_059,
MN_061,
MN_063,
MN_064,
MN_065,
MN_067,
MN_069,
MN_071,
MN_073,
MN_1,
],
"ME" => &[
ME_01,
ME_02,
ME_03,
ME_04,
ME_05,
ME_06,
ME_07,
ME_08,
ME_09,
ME_10,
ME_11,
ME_12,
ME_13,
ME_14,
ME_15,
ME_16,
ME_17,
ME_18,
ME_19,
ME_20,
ME_21,
ME_22,
ME_23,
],
"MS" => &[
MS_MS,
],
"MA" => &[
MA_01,
MA_02,
MA_03,
MA_04,
MA_05,
MA_06,
MA_07,
MA_08,
MA_09,
MA_10,
MA_11,
MA_12,
MA_13,
MA_14,
MA_15,
MA_16,
MA_AGD,
MA_AOU,
MA_ASZ,
MA_AZI,
MA_BEM,
MA_BER,
MA_BES,
MA_BOD,
MA_BOM,
MA_CAS,
MA_CHE,
MA_CHI,
MA_CHT,
MA_ERR,
MA_ESI,
MA_ESM,
MA_FAH,
MA_FES,
MA_FIG,
MA_GUE,
MA_HAJ,
MA_HAO,
MA_HOC,
MA_IFR,
MA_INE,
MA_JDI,
MA_JRA,
MA_KEN,
MA_KES,
MA_KHE,
MA_KHN,
MA_KHO,
MA_LAA,
MA_LAR,
MA_MED,
MA_MEK,
MA_MMD,
MA_MMN,
MA_MOH,
MA_MOU,
MA_NAD,
MA_NOU,
MA_OUA,
MA_OUD,
MA_OUJ,
MA_RAB,
MA_SAF,
MA_SAL,
MA_SEF,
MA_SET,
MA_SIK,
MA_SKH,
MA_SYB,
MA_TAI,
MA_TAO,
MA_TAR,
MA_TAT,
MA_TAZ,
MA_TET,
MA_TIZ,
MA_TNG,
MA_TNT,
MA_ZAG,
],
"MZ" => &[
MZ_A,
MZ_B,
MZ_G,
MZ_I,
MZ_L,
MZ_MPM,
MZ_N,
MZ_P,
MZ_Q,
MZ_S,
MZ_T,
],
"MM" => &[
MM_01,
MM_02,
MM_03,
MM_04,
MM_05,
MM_06,
MM_07,
MM_11,
MM_12,
MM_13,
MM_14,
MM_15,
MM_16,
MM_17,
MM_18,
],
"NA" => &[
NA_CA,
NA_ER,
NA_HA,
NA_KA,
NA_KE,
NA_KH,
NA_KU,
NA_KW,
NA_OD,
NA_OH,
NA_ON,
NA_OS,
NA_OT,
NA_OW,
],
"NR" => &[
NR_01,
NR_02,
NR_03,
NR_04,
NR_05,
NR_06,
NR_07,
NR_08,
NR_09,
NR_10,
NR_11,
NR_12,
NR_13,
NR_14,
],
"NP" => &[
NP_1,
NP_2,
NP_3,
NP_4,
NP_5,
NP_BA,
NP_BH,
NP_DH,
NP_GA,
NP_JA,
NP_KA,
NP_KO,
NP_LU,
NP_MA,
NP_ME,
NP_NA,
NP_RA,
NP_SA,
NP_SE,
],
"NL" => &[
NL_AW,
NL_BQ1,
NL_BQ2,
NL_BQ3,
NL_CW,
NL_DR,
NL_FL,
NL_FR,
NL_GE,
NL_GR,
NL_LI,
NL_NB,
NL_NH,
NL_OV,
NL_SX,
NL_UT,
NL_ZE,
NL_ZH,
],
"NC" => &[
NC_NC,
],
"NZ" => &[
NZ_AUK,
NZ_BOP,
NZ_CAN,
NZ_CIT,
NZ_GIS,
NZ_HKB,
NZ_MBH,
NZ_MWT,
NZ_N,
NZ_NSN,
NZ_NTL,
NZ_OTA,
NZ_S,
NZ_STL,
NZ_TAS,
NZ_TKI,
NZ_WGN,
NZ_WKO,
NZ_WTC,
],
"NI" => &[
NI_AN,
NI_AS,
NI_BO,
NI_CA,
NI_CI,
NI_CO,
NI_ES,
NI_GR,
NI_JI,
NI_LE,
NI_MD,
NI_MN,
NI_MS,
NI_MT,
NI_NS,
NI_RI,
NI_SJ,
],
"NG" => &[
NG_AB,
NG_AD,
NG_AK,
NG_AN,
NG_BA,
NG_BE,
NG_BO,
NG_BY,
NG_CR,
NG_DE,
NG_EB,
NG_ED,
NG_EK,
NG_EN,
NG_FC,
NG_GO,
NG_IM,
NG_JI,
NG_KD,
NG_KE,
NG_KN,
NG_KO,
NG_KT,
NG_KW,
NG_LA,
NG_NA,
NG_NI,
NG_OG,
NG_ON,
NG_OS,
NG_OY,
NG_PL,
NG_RI,
NG_SO,
NG_TA,
NG_YO,
NG_ZA,
],
"NE" => &[
NE_1,
NE_2,
NE_3,
NE_4,
NE_5,
NE_6,
NE_7,
NE_8,
],
"NU" => &[
NU_NU,
],
"NF" => &[
NF_NF,
],
"MP" => &[
MP_MP,
],
"NO" => &[
NO_01,
NO_02,
NO_03,
NO_04,
NO_05,
NO_06,
NO_07,
NO_08,
NO_09,
NO_10,
NO_11,
NO_12,
NO_14,
NO_15,
NO_16,
NO_17,
NO_18,
NO_19,
NO_20,
NO_21,
NO_22,
],
"OM" => &[
OM_BA,
OM_BU,
OM_DA,
OM_MA,
OM_MU,
OM_SH,
OM_WU,
OM_ZA,
OM_ZU,
],
"PK" => &[
PK_BA,
PK_GB,
PK_IS,
PK_JK,
PK_KP,
PK_PB,
PK_SD,
PK_TA,
],
"PW" => &[
PW_002,
PW_004,
PW_010,
PW_050,
PW_100,
PW_150,
PW_212,
PW_214,
PW_218,
PW_222,
PW_224,
PW_226,
PW_227,
PW_228,
PW_350,
PW_370,
],
"PS" => &[
PS_BTH,
PS_DEB,
PS_GZA,
PS_HBN,
PS_JEM,
PS_JEN,
PS_JRH,
PS_KYS,
PS_NBS,
PS_NGZ,
PS_QQA,
PS_RBH,
PS_RFH,
PS_SLT,
PS_TBS,
PS_TKM,
],
"PA" => &[
PA_10,
PA_1,
PA_2,
PA_3,
PA_4,
PA_5,
PA_6,
PA_7,
PA_8,
PA_9,
PA_EM,
PA_KY,
PA_NB,
],
"PG" => &[
PG_CPK,
PG_CPM,
PG_EBR,
PG_EHG,
PG_EPW,
PG_ESW,
PG_GPK,
PG_HLA,
PG_JWK,
PG_MBA,
PG_MPL,
PG_MPM,
PG_MRL,
PG_NCD,
PG_NIK,
PG_NPP,
PG_NSB,
PG_SAN,
PG_SHM,
PG_WBK,
PG_WHM,
PG_WPD,
],
"PY" => &[
PY_10,
PY_11,
PY_12,
PY_13,
PY_14,
PY_15,
PY_16,
PY_19,
PY_1,
PY_2,
PY_3,
PY_4,
PY_5,
PY_6,
PY_7,
PY_8,
PY_9,
PY_ASU,
],
"PE" => &[
PE_AMA,
PE_ANC,
PE_APU,
PE_ARE,
PE_AYA,
PE_CAJ,
PE_CAL,
PE_CUS,
PE_HUC,
PE_HUV,
PE_ICA,
PE_JUN,
PE_LAL,
PE_LAM,
PE_LIM,
PE_LMA,
PE_LOR,
PE_MDD,
PE_MOQ,
PE_PAS,
PE_PIU,
PE_PUN,
PE_SAM,
PE_TAC,
PE_TUM,
PE_UCA,
],
"PH" => &[
PH_00,
PH_01,
PH_02,
PH_03,
PH_05,
PH_06,
PH_07,
PH_08,
PH_09,
PH_10,
PH_11,
PH_12,
PH_13,
PH_14,
PH_15,
PH_40,
PH_41,
PH_ABR,
PH_AGN,
PH_AGS,
PH_AKL,
PH_ALB,
PH_ANT,
PH_APA,
PH_AUR,
PH_BAN,
PH_BAS,
PH_BEN,
PH_BIL,
PH_BOH,
PH_BTG,
PH_BTN,
PH_BUK,
PH_BUL,
PH_CAG,
PH_CAM,
PH_CAN,
PH_CAP,
PH_CAS,
PH_CAT,
PH_CAV,
PH_CEB,
PH_COM,
PH_DAO,
PH_DAS,
PH_DAV,
PH_DIN,
PH_EAS,
PH_GUI,
PH_IFU,
PH_ILI,
PH_ILN,
PH_ILS,
PH_ISA,
PH_KAL,
PH_LAG,
PH_LAN,
PH_LAS,
PH_LEY,
PH_LUN,
PH_MAD,
PH_MAG,
PH_MAS,
PH_MDC,
PH_MDR,
PH_MOU,
PH_MSC,
PH_MSR,
PH_NCO,
PH_NEC,
PH_NER,
PH_NSA,
PH_NUE,
PH_NUV,
PH_PAM,
PH_PAN,
PH_PLW,
PH_QUE,
PH_QUI,
PH_RIZ,
PH_ROM,
PH_SAR,
PH_SCO,
PH_SIG,
PH_SLE,
PH_SLU,
PH_SOR,
PH_SUK,
PH_SUN,
PH_SUR,
PH_TAR,
PH_TAW,
PH_WSA,
PH_ZAN,
PH_ZAS,
PH_ZMB,
PH_ZSI,
],
"PN" => &[
PN_PN,
],
"PL" => &[
PL_DS,
PL_KP,
PL_LB,
PL_LD,
PL_LU,
PL_MA,
PL_MZ,
PL_OP,
PL_PD,
PL_PK,
PL_PM,
PL_SK,
PL_SL,
PL_WN,
PL_WP,
PL_ZP,
],
"PT" => &[
PT_01,
PT_02,
PT_03,
PT_04,
PT_05,
PT_06,
PT_07,
PT_08,
PT_09,
PT_10,
PT_11,
PT_12,
PT_13,
PT_14,
PT_15,
PT_16,
PT_17,
PT_18,
PT_20,
PT_30,
],
"PR" => &[
PR_PR,
],
"QA" => &[
QA_DA,
QA_KH,
QA_MS,
QA_RA,
QA_US,
QA_WA,
QA_ZA,
],
"RE" => &[
RE_RE,
],
"RO" => &[
RO_AB,
RO_AG,
RO_AR,
RO_B,
RO_BC,
RO_BH,
RO_BN,
RO_BR,
RO_BT,
RO_BV,
RO_BZ,
RO_CJ,
RO_CL,
RO_CS,
RO_CT,
RO_CV,
RO_DB,
RO_DJ,
RO_GJ,
RO_GL,
RO_GR,
RO_HD,
RO_HR,
RO_IF,
RO_IL,
RO_IS,
RO_MH,
RO_MM,
RO_MS,
RO_NT,
RO_OT,
RO_PH,
RO_SB,
RO_SJ,
RO_SM,
RO_SV,
RO_TL,
RO_TM,
RO_TR,
RO_VL,
RO_VN,
RO_VS,
],
"RU" => &[
RU_AD,
RU_AL,
RU_ALT,
RU_AMU,
RU_ARK,
RU_AST,
RU_BA,
RU_BEL,
RU_BRY,
RU_BU,
RU_CE,
RU_CHE,
RU_CHU,
RU_CU,
RU_DA,
RU_IN,
RU_IRK,
RU_IVA,
RU_KAM,
RU_KB,
RU_KC,
RU_KDA,
RU_KEM,
RU_KGD,
RU_KGN,
RU_KHA,
RU_KHM,
RU_KIR,
RU_KK,
RU_KL,
RU_KLU,
RU_KO,
RU_KOS,
RU_KR,
RU_KRS,
RU_KYA,
RU_LEN,
RU_LIP,
RU_MAG,
RU_ME,
RU_MO,
RU_MOS,
RU_MOW,
RU_MUR,
RU_NEN,
RU_NGR,
RU_NIZ,
RU_NVS,
RU_OMS,
RU_ORE,
RU_ORL,
RU_PER,
RU_PNZ,
RU_PRI,
RU_PSK,
RU_ROS,
RU_RYA,
RU_SAK,
RU_SAM,
RU_SAR,
RU_SA,
RU_SE,
RU_SMO,
RU_SPE,
RU_STA,
RU_SVE,
RU_TAM,
RU_TA,
RU_TOM,
RU_TUL,
RU_TVE,
RU_TY,
RU_TYU,
RU_UD,
RU_ULY,
RU_VGG,
RU_VLA,
RU_VLG,
RU_VOR,
RU_YAN,
RU_YAR,
RU_YEV,
RU_ZAB,
],
"RW" => &[
RW_01,
RW_02,
RW_03,
RW_04,
RW_05,
],
"SH" => &[
SH_AC,
SH_HL,
SH_TA,
],
"KN" => &[
KN_01,
KN_02,
KN_03,
KN_04,
KN_05,
KN_06,
KN_07,
KN_08,
KN_09,
KN_10,
KN_11,
KN_12,
KN_13,
KN_15,
KN_K,
KN_N,
],
"LC" => &[
LC_01,
LC_02,
LC_03,
LC_05,
LC_06,
LC_07,
LC_08,
LC_10,
LC_11,
LC_12,
],
"VC" => &[
VC_01,
VC_02,
VC_03,
VC_04,
VC_05,
VC_06,
],
"WS" => &[
WS_AA,
WS_AL,
WS_AT,
WS_FA,
WS_GE,
WS_GI,
WS_PA,
WS_SA,
WS_TU,
WS_VF,
WS_VS,
],
"SM" => &[
SM_01,
SM_02,
SM_03,
SM_04,
SM_05,
SM_06,
SM_07,
SM_08,
SM_09,
],
"ST" => &[
ST_P,
ST_S,
],
"SA" => &[
SA_01,
SA_02,
SA_03,
SA_04,
SA_05,
SA_06,
SA_07,
SA_08,
SA_09,
SA_10,
SA_11,
SA_12,
SA_14,
],
"SN" => &[
SN_DB,
SN_DK,
SN_FK,
SN_KA,
SN_KD,
SN_KE,
SN_KL,
SN_LG,
SN_MT,
SN_SE,
SN_SL,
SN_TC,
SN_TH,
SN_ZG,
],
"RS" => &[
RS_00,
RS_01,
RS_02,
RS_03,
RS_04,
RS_05,
RS_06,
RS_07,
RS_08,
RS_09,
RS_10,
RS_11,
RS_12,
RS_13,
RS_14,
RS_15,
RS_16,
RS_17,
RS_18,
RS_19,
RS_20,
RS_21,
RS_22,
RS_23,
RS_24,
RS_25,
RS_26,
RS_27,
RS_28,
RS_29,
RS_KM,
RS_VO,
],
"SC" => &[
SC_01,
SC_02,
SC_03,
SC_04,
SC_05,
SC_06,
SC_07,
SC_08,
SC_09,
SC_10,
SC_11,
SC_12,
SC_13,
SC_14,
SC_15,
SC_16,
SC_17,
SC_18,
SC_19,
SC_20,
SC_21,
SC_22,
SC_23,
SC_24,
SC_25,
],
"GS" => &[
GS_GS,
],
"SL" => &[
SL_E,
SL_N,
SL_S,
SL_W,
],
"SG" => &[
SG_01,
SG_02,
SG_03,
SG_04,
SG_05,
],
"SK" => &[
SK_BC,
SK_BL,
SK_KI,
SK_NI,
SK_PV,
SK_TA,
SK_TC,
SK_ZI,
],
"SI" => &[
SI_001,
SI_002,
SI_003,
SI_004,
SI_005,
SI_006,
SI_007,
SI_008,
SI_009,
SI_010,
SI_011,
SI_012,
SI_013,
SI_014,
SI_015,
SI_016,
SI_017,
SI_018,
SI_019,
SI_020,
SI_021,
SI_022,
SI_023,
SI_024,
SI_025,
SI_026,
SI_027,
SI_028,
SI_029,
SI_030,
SI_031,
SI_032,
SI_033,
SI_034,
SI_035,
SI_036,
SI_037,
SI_038,
SI_039,
SI_040,
SI_041,
SI_042,
SI_043,
SI_044,
SI_045,
SI_046,
SI_047,
SI_048,
SI_049,
SI_050,
SI_051,
SI_052,
SI_053,
SI_054,
SI_055,
SI_056,
SI_057,
SI_058,
SI_059,
SI_060,
SI_061,
SI_062,
SI_063,
SI_064,
SI_065,
SI_066,
SI_067,
SI_068,
SI_069,
SI_070,
SI_071,
SI_072,
SI_073,
SI_074,
SI_075,
SI_076,
SI_077,
SI_078,
SI_079,
SI_080,
SI_081,
SI_082,
SI_083,
SI_084,
SI_085,
SI_086,
SI_087,
SI_088,
SI_089,
SI_090,
SI_091,
SI_092,
SI_093,
SI_094,
SI_095,
SI_096,
SI_097,
SI_098,
SI_099,
SI_100,
SI_101,
SI_102,
SI_103,
SI_104,
SI_105,
SI_106,
SI_107,
SI_108,
SI_109,
SI_110,
SI_111,
SI_112,
SI_113,
SI_114,
SI_115,
SI_116,
SI_117,
SI_118,
SI_119,
SI_120,
SI_121,
SI_122,
SI_123,
SI_124,
SI_125,
SI_126,
SI_127,
SI_128,
SI_129,
SI_130,
SI_131,
SI_132,
SI_133,
SI_134,
SI_135,
SI_136,
SI_137,
SI_138,
SI_139,
SI_140,
SI_141,
SI_142,
SI_143,
SI_144,
SI_146,
SI_147,
SI_148,
SI_149,
SI_150,
SI_151,
SI_152,
SI_153,
SI_154,
SI_155,
SI_156,
SI_157,
SI_158,
SI_159,
SI_160,
SI_161,
SI_162,
SI_163,
SI_164,
SI_165,
SI_166,
SI_167,
SI_168,
SI_169,
SI_170,
SI_171,
SI_172,
SI_173,
SI_174,
SI_175,
SI_176,
SI_177,
SI_178,
SI_179,
SI_180,
SI_181,
SI_182,
SI_183,
SI_184,
SI_185,
SI_186,
SI_187,
SI_188,
SI_189,
SI_190,
SI_191,
SI_192,
SI_193,
SI_194,
SI_195,
SI_196,
SI_197,
SI_198,
SI_199,
SI_200,
SI_201,
SI_202,
SI_203,
SI_204,
SI_205,
SI_206,
SI_207,
SI_208,
SI_209,
SI_210,
SI_211,
SI_212,
],
"SB" => &[
SB_CE,
SB_CH,
SB_CT,
SB_GU,
SB_IS,
SB_MK,
SB_ML,
SB_RB,
SB_TE,
SB_WE,
],
"SO" => &[
SO_AW,
SO_BK,
SO_BN,
SO_BR,
SO_BY,
SO_GA,
SO_GE,
SO_HI,
SO_JD,
SO_JH,
SO_MU,
SO_NU,
SO_SA,
SO_SD,
SO_SH,
SO_SO,
SO_TO,
SO_WO,
],
"ZA" => &[
ZA_EC,
ZA_FS,
ZA_GT,
ZA_LP,
ZA_MP,
ZA_NC,
ZA_NL,
ZA_NW,
ZA_WC,
],
"SS" => &[
SS_BN,
SS_BW,
SS_EC,
SS_EE,
SS_EW,
SS_JG,
SS_LK,
SS_NU,
SS_UY,
SS_WR,
],
"ES" => &[
ES_A,
ES_AB,
ES_AL,
ES_AN,
ES_AR,
ES_AS,
ES_AV,
ES_BA,
ES_B,
ES_BI,
ES_BU,
ES_CA,
ES_C,
ES_CB,
ES_CC,
ES_CE,
ES_CL,
ES_CM,
ES_CN,
ES_CO,
ES_CR,
ES_CS,
ES_CT,
ES_CU,
ES_EX,
ES_GA,
ES_GC,
ES_GI,
ES_GR,
ES_GU,
ES_H,
ES_HU,
ES_IB,
ES_J,
ES_LE,
ES_L,
ES_LO,
ES_LU,
ES_MA,
ES_MC,
ES_MD,
ES_ML,
ES_M,
ES_MU,
ES_NA,
ES_NC,
ES_O,
ES_OR,
ES_PM,
ES_PO,
ES_P,
ES_PV,
ES_RI,
ES_SA,
ES_S,
ES_SE,
ES_SG,
ES_SO,
ES_SS,
ES_TE,
ES_TF,
ES_TO,
ES_T,
ES_VA,
ES_VC,
ES_VI,
ES_V,
ES_ZA,
ES_Z,
],
"LK" => &[
LK_11,
LK_12,
LK_13,
LK_1,
LK_21,
LK_22,
LK_23,
LK_2,
LK_31,
LK_32,
LK_33,
LK_3,
LK_41,
LK_42,
LK_43,
LK_44,
LK_45,
LK_4,
LK_51,
LK_52,
LK_53,
LK_5,
LK_61,
LK_62,
LK_6,
LK_71,
LK_72,
LK_7,
LK_81,
LK_82,
LK_8,
LK_91,
LK_92,
LK_9,
],
"BL" => &[
BL_BL,
],
"SX" => &[
SX_SX,
],
"MF" => &[
MF_MF,
],
"PM" => &[
PM_PM,
],
"SD" => &[
SD_DC,
SD_DE,
SD_DN,
SD_DS,
SD_DW,
SD_GD,
SD_GZ,
SD_KA,
SD_KH,
SD_KN,
SD_KS,
SD_NB,
SD_NO,
SD_NR,
SD_NW,
SD_RS,
SD_SI,
],
"SR" => &[
SR_BR,
SR_CM,
SR_CR,
SR_MA,
SR_NI,
SR_PM,
SR_PR,
SR_SA,
SR_SI,
SR_WA,
],
"SJ" => &[
SJ_SJ,
],
"SZ" => &[
SZ_HH,
SZ_LU,
SZ_MA,
SZ_SH,
],
"SE" => &[
SE_AB,
SE_AC,
SE_BD,
SE_C,
SE_D,
SE_E,
SE_F,
SE_G,
SE_H,
SE_I,
SE_K,
SE_M,
SE_N,
SE_O,
SE_S,
SE_T,
SE_U,
SE_W,
SE_X,
SE_Y,
SE_Z,
],
"CH" => &[
CH_AG,
CH_AI,
CH_AR,
CH_BE,
CH_BL,
CH_BS,
CH_FR,
CH_GE,
CH_GL,
CH_GR,
CH_JU,
CH_LU,
CH_NE,
CH_NW,
CH_OW,
CH_SG,
CH_SH,
CH_SO,
CH_SZ,
CH_TG,
CH_TI,
CH_UR,
CH_VD,
CH_VS,
CH_ZG,
CH_ZH,
],
"SY" => &[
SY_DI,
SY_DR,
SY_DY,
SY_HA,
SY_HI,
SY_HL,
SY_HM,
SY_ID,
SY_LA,
SY_QU,
SY_RA,
SY_RD,
SY_SU,
SY_TA,
],
"TW" => &[
TW_CHA,
TW_CYI,
TW_CYQ,
TW_HSQ,
TW_HSZ,
TW_HUA,
TW_ILA,
TW_KEE,
TW_KHH,
TW_KHQ,
TW_MIA,
TW_NAN,
TW_PEN,
TW_PIF,
TW_TAO,
TW_TNN,
TW_TNQ,
TW_TPE,
TW_TPQ,
TW_TTT,
TW_TXG,
TW_TXQ,
TW_YUN,
],
"TJ" => &[
TJ_DU,
TJ_GB,
TJ_KT,
TJ_SU,
],
"TZ" => &[
TZ_01,
TZ_02,
TZ_03,
TZ_04,
TZ_05,
TZ_06,
TZ_07,
TZ_08,
TZ_09,
TZ_10,
TZ_11,
TZ_12,
TZ_13,
TZ_14,
TZ_15,
TZ_16,
TZ_17,
TZ_18,
TZ_19,
TZ_20,
TZ_21,
TZ_22,
TZ_23,
TZ_24,
TZ_25,
TZ_26,
TZ_27,
TZ_28,
TZ_29,
TZ_30,
],
"TH" => &[
TH_10,
TH_11,
TH_12,
TH_13,
TH_14,
TH_15,
TH_16,
TH_17,
TH_18,
TH_19,
TH_20,
TH_21,
TH_22,
TH_23,
TH_24,
TH_25,
TH_26,
TH_27,
TH_30,
TH_31,
TH_32,
TH_33,
TH_34,
TH_35,
TH_36,
TH_37,
TH_38,
TH_39,
TH_40,
TH_41,
TH_42,
TH_43,
TH_44,
TH_45,
TH_46,
TH_47,
TH_48,
TH_49,
TH_50,
TH_51,
TH_52,
TH_53,
TH_54,
TH_55,
TH_56,
TH_57,
TH_58,
TH_60,
TH_61,
TH_62,
TH_63,
TH_64,
TH_65,
TH_66,
TH_67,
TH_70,
TH_71,
TH_72,
TH_73,
TH_74,
TH_75,
TH_76,
TH_77,
TH_80,
TH_81,
TH_82,
TH_83,
TH_84,
TH_85,
TH_86,
TH_90,
TH_91,
TH_92,
TH_93,
TH_94,
TH_95,
TH_96,
TH_S,
],
"TL" => &[
TL_AL,
TL_AN,
TL_BA,
TL_BO,
TL_CO,
TL_DI,
TL_ER,
TL_LA,
TL_LI,
TL_MF,
TL_MT,
TL_OE,
TL_VI,
],
"TG" => &[
TG_C,
TG_K,
TG_M,
TG_P,
TG_S,
],
"TK" => &[
TK_TK,
],
"TO" => &[
TO_01,
TO_02,
TO_03,
TO_04,
TO_05,
],
"TT" => &[
TT_ARI,
TT_CHA,
TT_CTT,
TT_DMN,
TT_ETO,
TT_PED,
TT_POS,
TT_PRT,
TT_PTF,
TT_RCM,
TT_SFO,
TT_SGE,
TT_SIP,
TT_SJL,
TT_TUP,
TT_WTO,
],
"TN" => &[
TN_11,
TN_12,
TN_13,
TN_14,
TN_21,
TN_22,
TN_23,
TN_31,
TN_32,
TN_33,
TN_34,
TN_41,
TN_42,
TN_43,
TN_51,
TN_52,
TN_53,
TN_61,
TN_71,
TN_72,
TN_73,
TN_81,
TN_82,
TN_83,
],
"TR" => &[
TR_01,
TR_02,
TR_03,
TR_04,
TR_05,
TR_06,
TR_07,
TR_08,
TR_09,
TR_10,
TR_11,
TR_12,
TR_13,
TR_14,
TR_15,
TR_16,
TR_17,
TR_18,
TR_19,
TR_20,
TR_21,
TR_22,
TR_23,
TR_24,
TR_25,
TR_26,
TR_27,
TR_28,
TR_29,
TR_30,
TR_31,
TR_32,
TR_33,
TR_34,
TR_35,
TR_36,
TR_37,
TR_38,
TR_39,
TR_40,
TR_41,
TR_42,
TR_43,
TR_44,
TR_45,
TR_46,
TR_47,
TR_48,
TR_49,
TR_50,
TR_51,
TR_52,
TR_53,
TR_54,
TR_55,
TR_56,
TR_57,
TR_58,
TR_59,
TR_60,
TR_61,
TR_62,
TR_63,
TR_64,
TR_65,
TR_66,
TR_67,
TR_68,
TR_69,
TR_70,
TR_71,
TR_72,
TR_73,
TR_74,
TR_75,
TR_76,
TR_77,
TR_78,
TR_79,
TR_80,
TR_81,
],
"TM" => &[
TM_A,
TM_B,
TM_D,
TM_L,
TM_M,
TM_S,
],
"TC" => &[
TC_TC,
],
"TV" => &[
TV_FUN,
TV_NIT,
TV_NKF,
TV_NKL,
TV_NMA,
TV_NMG,
TV_NUI,
TV_VAI,
],
"UG" => &[
UG_101,
UG_102,
UG_103,
UG_104,
UG_105,
UG_106,
UG_107,
UG_108,
UG_109,
UG_110,
UG_111,
UG_112,
UG_113,
UG_114,
UG_115,
UG_116,
UG_117,
UG_118,
UG_119,
UG_120,
UG_121,
UG_122,
UG_123,
UG_124,
UG_201,
UG_202,
UG_203,
UG_204,
UG_205,
UG_206,
UG_207,
UG_208,
UG_209,
UG_210,
UG_211,
UG_212,
UG_213,
UG_214,
UG_215,
UG_216,
UG_217,
UG_218,
UG_219,
UG_220,
UG_221,
UG_222,
UG_223,
UG_224,
UG_225,
UG_226,
UG_227,
UG_228,
UG_229,
UG_230,
UG_231,
UG_232,
UG_301,
UG_302,
UG_303,
UG_304,
UG_305,
UG_306,
UG_307,
UG_308,
UG_309,
UG_310,
UG_311,
UG_312,
UG_313,
UG_314,
UG_315,
UG_316,
UG_317,
UG_318,
UG_319,
UG_320,
UG_321,
UG_322,
UG_323,
UG_324,
UG_325,
UG_326,
UG_327,
UG_328,
UG_329,
UG_330,
UG_331,
UG_401,
UG_402,
UG_403,
UG_404,
UG_405,
UG_406,
UG_407,
UG_408,
UG_409,
UG_410,
UG_411,
UG_412,
UG_413,
UG_414,
UG_415,
UG_416,
UG_417,
UG_418,
UG_419,
UG_420,
UG_421,
UG_422,
UG_423,
UG_424,
UG_425,
UG_C,
UG_E,
UG_N,
UG_W,
],
"UA" => &[
UA_05,
UA_07,
UA_09,
UA_12,
UA_14,
UA_18,
UA_21,
UA_23,
UA_26,
UA_30,
UA_32,
UA_35,
UA_40,
UA_43,
UA_46,
UA_48,
UA_51,
UA_53,
UA_56,
UA_59,
UA_61,
UA_63,
UA_65,
UA_68,
UA_71,
UA_74,
UA_77,
],
"AE" => &[
AE_AJ,
AE_AZ,
AE_DU,
AE_FU,
AE_RK,
AE_SH,
AE_UQ,
],
"GB" => &[
GB_ABD,
GB_ABE,
GB_AGB,
GB_AGY,
GB_ANS,
GB_ANT,
GB_ARD,
GB_ARM,
GB_BAS,
GB_BBD,
GB_BDF,
GB_BDG,
GB_BEN,
GB_BEX,
GB_BFS,
GB_BGE,
GB_BGW,
GB_BIR,
GB_BKM,
GB_BLA,
GB_BLY,
GB_BMH,
GB_BNB,
GB_BNE,
GB_BNH,
GB_BNS,
GB_BOL,
GB_BPL,
GB_BRC,
GB_BRD,
GB_BRY,
GB_BST,
GB_BUR,
GB_CAM,
GB_CAY,
GB_CBF,
GB_CGN,
GB_CGV,
GB_CHE,
GB_CHW,
GB_CKF,
GB_CKT,
GB_CLD,
GB_CLK,
GB_CLR,
GB_CMA,
GB_CMD,
GB_CMN,
GB_CON,
GB_COV,
GB_CRF,
GB_CRY,
GB_CSR,
GB_CWY,
GB_DAL,
GB_DBY,
GB_DEN,
GB_DER,
GB_DEV,
GB_DGN,
GB_DGY,
GB_DNC,
GB_DND,
GB_DOR,
GB_DOW,
GB_DRY,
GB_DUD,
GB_DUR,
GB_EAL,
GB_EAW,
GB_EAY,
GB_EDH,
GB_EDU,
GB_ELN,
GB_ELS,
GB_ENF,
GB_ENG,
GB_ERW,
GB_ERY,
GB_ESS,
GB_ESX,
GB_FAL,
GB_FER,
GB_FIF,
GB_FLN,
GB_GAT,
GB_GBN,
GB_GLG,
GB_GLS,
GB_GRE,
GB_GWN,
GB_HAL,
GB_HAM,
GB_HAV,
GB_HCK,
GB_HEF,
GB_HIL,
GB_HLD,
GB_HMF,
GB_HNS,
GB_HPL,
GB_HRT,
GB_HRW,
GB_HRY,
GB_IOS,
GB_IOW,
GB_ISL,
GB_IVC,
GB_KEC,
GB_KEN,
GB_KHL,
GB_KIR,
GB_KTT,
GB_KWL,
GB_LAN,
GB_LBH,
GB_LCE,
GB_LDS,
GB_LEC,
GB_LEW,
GB_LIN,
GB_LIV,
GB_LMV,
GB_LND,
GB_LRN,
GB_LSB,
GB_LUT,
GB_MAN,
GB_MDB,
GB_MDW,
GB_MFT,
GB_MIK,
GB_MLN,
GB_MON,
GB_MRT,
GB_MRY,
GB_MTY,
GB_MYL,
GB_NAY,
GB_NBL,
GB_NDN,
GB_NEL,
GB_NET,
GB_NFK,
GB_NGM,
GB_NIR,
GB_NLK,
GB_NLN,
GB_NSM,
GB_NTA,
GB_NTH,
GB_NTL,
GB_NTT,
GB_NTY,
GB_NWM,
GB_NWP,
GB_NYK,
GB_NYM,
GB_OLD,
GB_OMH,
GB_ORK,
GB_OXF,
GB_PEM,
GB_PKN,
GB_PLY,
GB_POL,
GB_POR,
GB_POW,
GB_PTE,
GB_RCC,
GB_RCH,
GB_RCT,
GB_RDB,
GB_RDG,
GB_RFW,
GB_RIC,
GB_ROT,
GB_RUT,
GB_SAW,
GB_SAY,
GB_SCB,
GB_SCT,
GB_SFK,
GB_SFT,
GB_SGC,
GB_SHF,
GB_SHN,
GB_SHR,
GB_SKP,
GB_SLF,
GB_SLG,
GB_SLK,
GB_SND,
GB_SOL,
GB_SOM,
GB_SOS,
GB_SRY,
GB_STB,
GB_STE,
GB_STG,
GB_STH,
GB_STN,
GB_STS,
GB_STT,
GB_STY,
GB_SWA,
GB_SWD,
GB_SWK,
GB_TAM,
GB_TFW,
GB_THR,
GB_TOB,
GB_TOF,
GB_TRF,
GB_TWH,
GB_UKM,
GB_VGL,
GB_WAR,
GB_WBK,
GB_WDU,
GB_WFT,
GB_WGN,
GB_WIL,
GB_WKF,
GB_WLL,
GB_WLN,
GB_WLS,
GB_WLV,
GB_WND,
GB_WNM,
GB_WOK,
GB_WOR,
GB_WRL,
GB_WRT,
GB_WRX,
GB_WSM,
GB_WSX,
GB_YOR,
GB_ZET,
],
"UM" => &[
UM_67,
UM_71,
UM_76,
UM_79,
UM_81,
UM_84,
UM_86,
UM_89,
UM_95,
],
"US" => &[
US_AK,
US_AL,
US_AR,
US_AS,
US_AZ,
US_CA,
US_CO,
US_CT,
US_DC,
US_DE,
US_FL,
US_GA,
US_GU,
US_HI,
US_IA,
US_ID,
US_IL,
US_IN,
US_KS,
US_KY,
US_LA,
US_MA,
US_MD,
US_ME,
US_MI,
US_MN,
US_MO,
US_MP,
US_MS,
US_MT,
US_NC,
US_ND,
US_NE,
US_NH,
US_NJ,
US_NM,
US_NV,
US_NY,
US_OH,
US_OK,
US_OR,
US_PA,
US_PR,
US_RI,
US_SC,
US_SD,
US_TN,
US_TX,
US_UM,
US_UT,
US_VA,
US_VI,
US_VT,
US_WA,
US_WI,
US_WV,
US_WY,
],
"UY" => &[
UY_AR,
UY_CA,
UY_CL,
UY_CO,
UY_DU,
UY_FD,
UY_FS,
UY_LA,
UY_MA,
UY_MO,
UY_PA,
UY_RN,
UY_RO,
UY_RV,
UY_SA,
UY_SJ,
UY_SO,
UY_TA,
UY_TT,
],
"UZ" => &[
UZ_AN,
UZ_BU,
UZ_FA,
UZ_JI,
UZ_NG,
UZ_NW,
UZ_QA,
UZ_QR,
UZ_SA,
UZ_SI,
UZ_SU,
UZ_TK,
UZ_TO,
UZ_XO,
],
"VU" => &[
VU_MAP,
VU_PAM,
VU_SAM,
VU_SEE,
VU_TAE,
VU_TOB,
],
"VA" => &[
VA_VA,
],
"VE" => &[
VE_A,
VE_B,
VE_C,
VE_D,
VE_E,
VE_F,
VE_G,
VE_H,
VE_I,
VE_J,
VE_K,
VE_L,
VE_M,
VE_N,
VE_O,
VE_P,
VE_R,
VE_S,
VE_T,
VE_U,
VE_V,
VE_W,
VE_X,
VE_Y,
VE_Z,
],
"VN" => &[
VN_01,
VN_02,
VN_03,
VN_04,
VN_05,
VN_06,
VN_07,
VN_09,
VN_13,
VN_14,
VN_18,
VN_20,
VN_21,
VN_22,
VN_23,
VN_24,
VN_25,
VN_26,
VN_27,
VN_28,
VN_29,
VN_30,
VN_31,
VN_32,
VN_33,
VN_34,
VN_35,
VN_36,
VN_37,
VN_39,
VN_40,
VN_41,
VN_43,
VN_44,
VN_45,
VN_46,
VN_47,
VN_49,
VN_50,
VN_51,
VN_52,
VN_53,
VN_54,
VN_55,
VN_56,
VN_57,
VN_58,
VN_59,
VN_61,
VN_63,
VN_66,
VN_67,
VN_68,
VN_69,
VN_70,
VN_71,
VN_72,
VN_73,
VN_CT,
VN_DN,
VN_HN,
VN_HP,
VN_SG,
],
"VG" => &[
VG_VG,
],
"VI" => &[
VI_VI,
],
"WF" => &[
WF_WF,
],
"EH" => &[
EH_EH,
],
"YE" => &[
YE_AB,
YE_AD,
YE_AM,
YE_BA,
YE_DA,
YE_DH,
YE_HD,
YE_HJ,
YE_HU,
YE_IB,
YE_JA,
YE_LA,
YE_MA,
YE_MR,
YE_MW,
YE_RA,
YE_SA,
YE_SD,
YE_SH,
YE_SN,
YE_TA,
],
"ZM" => &[
ZM_01,
ZM_02,
ZM_03,
ZM_04,
ZM_05,
ZM_06,
ZM_07,
ZM_08,
ZM_09,
ZM_10,
],
"ZW" => &[
ZW_BU,
ZW_HA,
ZW_MA,
ZW_MC,
ZW_ME,
ZW_MI,
ZW_MN,
ZW_MS,
ZW_MV,
ZW_MW,
],

};

